package com.urovo.wgmf.db.dao.bill;

import android.database.Cursor;

import com.urovo.wgmf.db.dao.entitys.DingHuoExamineDetailEntity;
import com.urovo.wgmf.db.dao.entitys.DingHuoExamineEntity;
import com.urovo.wugumofang.system.BaseAppManager;

import java.util.ArrayList;
import java.util.List;

/**
 * 门店订货管理 操作PDA数据库的类
 * 
 * @author Administrator
 *
 */
public class DalDingHuoExamine {

	/**
	 * 保存订货单主表到本地数据库
	 * 
	 * @param bll
	 * @return
	 */
	public static boolean SaveDingHuoExamine(DingHuoExamineEntity bll) {
		String sql = "insert into DHSH001(BILL,StoreID,Wdate,ItemCount,Amount,State,Oper_Sh,Sh_Wdate,IsUp) values(?,?,?,?,?,?,?,?,?)";
		try {
			BaseAppManager.database().execSQL(sql, new Object[]{ bll.getBill(), bll.getStoreId(), bll.getWDate(), bll.getItemCount(), bll.getAmount(), bll.getState(), bll.getOper_Sh(), bll.getSh_Wdate(), 0 });
		}
		catch (Exception e) {

			BaseAppManager.loger().info("保存订货审核单主表发生异常:" + e);
			return false;
		}
		return true;
	}

	/**
	 * 保存订货单明细数据到本地数据库
	 * 
	 * @param list
	 * @return
	 */
	public static boolean SaveDingHuoExamineDetail(List<DingHuoExamineDetailEntity> list) {
		String sql = "insert into DHSH002(BILL,OrderQty,OplQty,ExamineQty,ItemID) values(?,?,?,?,?)";
		try {
			BaseAppManager.database().beginTransaction();
			for (DingHuoExamineDetailEntity p : list) {
				BaseAppManager.database().execSQL(sql, new Object[]{ p.getBill(), p.getOrder_qty(), p.getOpl_qty(), p.getExamine_qty(), p.getItemid() });
			}
			BaseAppManager.database().setTransactionSuccessful();
		}
		catch (Exception e) {

			BaseAppManager.loger().info("保存订货审核单明细表发生异常:" + e);
			return false;
		}
		finally {
			BaseAppManager.database().endTransaction();
		}
		return true;
	}

	/**
	 * 获取未上传的门店订货单据主表
	 * 
	 * @return
	 */
	public static List<DingHuoExamineEntity> GetUnUpload() {
		Cursor cursor = null;
		List<DingHuoExamineEntity> list = new ArrayList<DingHuoExamineEntity>();
		String sql = "select Bill,StoreID,Wdate,ItemCount,Amount,State ,oper_sh " + " from DHSH001 where IsUp=0";
		try {
			cursor = BaseAppManager.database().rawQuery(sql, null);
			while (cursor.moveToNext()) {
				DingHuoExamineEntity ent = new DingHuoExamineEntity();
				ent.setBill(cursor.getString(0));
				ent.setStoreId(cursor.getString(1));
				ent.setWDate(cursor.getString(2));
				ent.setItemCount(cursor.getLong(3));
				ent.setAmount(cursor.getDouble(4));
				ent.setState(cursor.getString(5));
				ent.setOper_Sh(cursor.getString(6));
				list.add(ent);
			}
		}
		catch (Exception e) {

			BaseAppManager.loger().info("获取未上传门店订货审核单主表发生异常:" + e);
		}
		finally {
			if(cursor != null) {
				cursor.close();
			}
		}
		return list;
	}

	/**
	 * 根据门店订货单据号获取产品明细
	 * 
	 * @param bill
	 * @return
	 */
	public static List<DingHuoExamineDetailEntity> GetBillDetail(String bill) {
		List<DingHuoExamineDetailEntity> list = new ArrayList<DingHuoExamineDetailEntity>();
		Cursor cursor = null;
		String sql = "select Bill,OrderQty,OplQty,ExamineQty,ItemID from DHSH002 where bill='" + bill + "'";
		try {
			cursor = BaseAppManager.database().rawQuery(sql, null);
			while (cursor.moveToNext()) {
				DingHuoExamineDetailEntity ent = new DingHuoExamineDetailEntity();
				ent.setBill(cursor.getString(0));
				ent.setOrder_qty(cursor.getDouble(1));
				ent.setOpl_qty(cursor.getDouble(2));
				ent.setExamine_qty(cursor.getDouble(3));
				ent.setItemid(cursor.getString(4));
				list.add(ent);
			}
		}
		catch (Exception e) {

			BaseAppManager.loger().info("根据门店订货审核单号获取产品明细发生异常:" + e);
		}
		finally {
			if(cursor != null) {
				cursor.close();
			}
		}
		return list;
	}

	/**
	 * 单据上传成功后调用此函数，更新终端数据库上的单据状态IsUp 从0更新为1
	 * 
	 * @param bill
	 */
	public static void UpdateBillState(String bill) {
		String sql = "update DHSH001 set IsUp=1 where bill='" + bill + "'";
		try {
			BaseAppManager.database().execSQL(sql);
		}
		catch (Exception e) {

			BaseAppManager.loger().info("更新门店订货审核单据上传状态发生异常" + e);
		}
	}

	/**
	 * 获取未上传单数
	 * 
	 * @return
	 */
	public static int GetUnUploadCount() {
		int total = 0;
		Cursor cursor = null;
		String sql = "select count(bill) from DHSH001 where IsUp=0";
		try {
			cursor = BaseAppManager.database().rawQuery(sql, null);
			if(cursor.moveToNext()) {
				total = cursor.getInt(0);
			}
		}
		catch (Exception e) {

			BaseAppManager.loger().info("获取未上传订货审核单据总数发生异常:" + e);
		}
		finally {
			if(cursor != null) {
				cursor.close();
			}
		}
		return total;
	}

	/**
	 * 删除DHSH001所有数据
	 */
	public static Boolean deleteDHSH001(String sql) {
		Boolean result = false;
		String sqlStr = "Delete from DHSH001";
		if(sql != null) {
			sqlStr += sql;
		}
		try {
			BaseAppManager.database().execSQL(sqlStr);
			result = true;
		}
		catch (Exception e) {
			BaseAppManager.loger().info(e);
			result = false;
		}
		finally {
		}
		return result;
	}

	/**
	 * 删除DHSH002所有数据
	 */
	public static Boolean deleteDHSH002(String sql) {
		Boolean result = false;
		String sqlStr = "Delete from DHSH002";
		if(sql != null) {
			sqlStr += sql;
		}
		try {
			BaseAppManager.database().execSQL(sqlStr);
			result = true;
		}
		catch (Exception e) {
			BaseAppManager.loger().info(e);
			result = false;
		}
		finally {
		}
		return result;
	}
}
