package com.urovo.wgmf.db.dao.dao;

import android.content.Context;
import android.database.Cursor;
import android.text.TextUtils;

import com.urovo.wgmf.db.dao.DBTool;
import com.urovo.wgmf.db.dao.entitys.CZYDA;
import com.urovo.wgmf.db.dao.entitys.Czyda001Db;
import com.urovo.wgmf.db.dao.entitys.YuanGongEntity;
import com.urovo.wugumofang.data.StringUtils;
import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.system.DeviceHelper;
import com.urovo.wugumofang.system.SystemHelper;

import java.util.ArrayList;
import java.util.List;

/**
 * 对数据库的增，删，改，查操作
 *
 * @author Administrator
 */
public class Czyda001DbOperate {

    private static DBTool dbTool;

    public Czyda001DbOperate(Context context) {
        super();
        dbTool = DBTool.getInstance(context);
    }

    public static String getEnv(String storeId) {
        String resault = null;
        String sql = "select ENV from czyda001 where storeId=?";
        Cursor cursor = null;
        try {
            cursor = BaseAppManager.database().rawQuery(sql, new String[]{storeId});
            if (cursor.moveToNext()) {
                resault = cursor.getString(0);
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("根据该门店操作环境发生异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return resault;
    }

    public static String setEnv(String storeId, String env) {
        String resault = null;
        String sql = "update czyda001 set ENV=? where storeId=?";
        try {
            BaseAppManager.database().execSQL(sql, new String[]{env, storeId});
        } catch (Exception e) {
            BaseAppManager.loger().info("根据该门店操作环境发生异常:" + e);
        } finally {
        }
        return resault;
    }

    /**
     * 插入数据
     */
    public static Boolean Insert(Czyda001Db czyda001) {
        Cursor cursor = null;
        try {
            String sql = "insert into czyda001(operateID,opername,staffID,password,sex,optype,deptcsID,storeID,status) values(?,?,?,?,?,?,?,?,?)";
            BaseAppManager.database().execSQL(sql, new String[]{czyda001.getOperatorID(), czyda001.getOpername(), czyda001.getStaffID(), czyda001.getPassword(), czyda001.getSex(), czyda001.getOptype(), czyda001.getDeptcsID(), czyda001.getStoreID(), czyda001.getStatus()});
            return true;
        } catch (Exception e) {
            BaseAppManager.loger().info("[Czyda001DbOperate.insert]" + e);
            return false;
        } finally {
            // database.close();
            if (cursor != null)
                cursor.close();
        }
    }

    /**
     * 查询方法的基方法，代码的复用，万精油
     *
     * @param sql
     * @return
     */
    public static List<Czyda001Db> selectBy(String sql) {
        List<Czyda001Db> list = new ArrayList<Czyda001Db>();
        Cursor cursor = null;
        String sqlStr = "select * from czyda001 ";
        if (sql != null)
            sqlStr += sql;
        try {
            cursor = BaseAppManager.database().rawQuery(sqlStr, new String[]{});
            while (cursor.moveToNext()) {
                Czyda001Db model = new Czyda001Db();
                model.setOperatorID(cursor.getString(cursor.getColumnIndex("OPERATORID")));
                model.setOpername(cursor.getString(cursor.getColumnIndex("OPERNAME")));
                model.setStaffID(cursor.getString(cursor.getColumnIndex("STAFFID")));
                model.setPassword(cursor.getString(cursor.getColumnIndex("PASSWORD")));
                model.setSex(cursor.getString(cursor.getColumnIndex("SEX")));
                model.setOptype(cursor.getString(cursor.getColumnIndex("OPTYPE")));
                model.setDeptcsID(cursor.getString(cursor.getColumnIndex("DEPTCSID")));
                model.setStoreID(cursor.getString(cursor.getColumnIndex("STOREID")));
                model.setStatus(cursor.getString(cursor.getColumnIndex("STATUS")));
                if (model.getOperatorID() != null && model.getOperatorID().length() > 0) {
                    list.add(model);
                }
            }
        } catch (Exception e) {

            BaseAppManager.loger().info("[Czyda001DbOperate.selectBy]" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return list;
    }

    /**
     * 删除所有数据
     */
    public static Boolean deleteAllGoods(String sql) {
        Boolean result = false;
        String sqlStr = "Delete from czyda001 ";
        if (sql != null) {
            sqlStr += sql;
        }
        try {
            BaseAppManager.database().execSQL(sqlStr);
            result = true;
        } catch (Exception e) {
            BaseAppManager.loger().info("[Czyda001DbOperate.deleteAllGoods]" + e);
            result = false;
        } finally {
        }
        return result;
    }

    /**
     * 门店编号、用户编号、密码来验证是否允许进入系统
     *
     * @param storeid  门店编号
     * @param userid   用户编号
     * @param password 登陆密码:数据库里面的密码是MD5密码
     * @return
     */
    public static boolean LoginCheck(String storeid, String userid, String password) {

        boolean result = false;
        password = DeviceHelper.getMD5Str(password); // MD5加密
        Cursor cursor = null;
        String sql = "select operatorid,opername,staffid,password,sex,optype,deptcsid,storeid,status from czyda001  " + "  where storeid=? and operatorid=? and password=? and status='0' ";
        try {
            cursor = BaseAppManager.database().rawQuery(sql, new String[]{storeid, userid, password});
            if (cursor.moveToNext()) {
                String operatorid = cursor.getString(0);
                SystemHelper.LoginUserId = StringUtils.isEmpty(operatorid) ? operatorid : "";
                String opername = cursor.getString(1);
                SystemHelper.LoginUserName = StringUtils.isEmpty(opername) ? opername : "";
                result = true;
            }
        } catch (Exception ex) {
            result = false;
            BaseAppManager.loger().info("根据用户名、密码、门店编号验证登陆信息发生异常:" + ex);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return result;
    }



    /**
     * 初始化员工角色标记
     */
    public static boolean initUserRoleStatic(String storeid, String userid) {
        boolean result = false;
//        password = DeviceHelper.getMD5Str(password); // MD5加密
        Cursor cursor = null;
        String sql = "select wgmfrole from czyda001  " + "  where storeid=? and operatorid=?  and status='0' ";
        try {
            cursor = BaseAppManager.database().rawQuery(sql, new String[]{storeid, userid});
            if (cursor.moveToNext()) {
                String role = cursor.getString(cursor.getColumnIndex("WGMFROLE"));
                SystemHelper.IsPartTimeStaff = !TextUtils.isEmpty(role) && role.equals("兼职");//如果是兼职人员，做好标志
                SystemHelper.IsPartTimeSalesStaff = !TextUtils.isEmpty(role) && role.equals("兼职销售");//如果是兼职人员，做好标志
                SystemHelper.IsMachineRepairStaff = !TextUtils.isEmpty(role) && role.equals("机修工程师");//如果是机修工程师，做好标志
                result = true;
            }
        } catch (Exception ex) {
            result = false;
            BaseAppManager.loger().info("根据用户名、密码、门店编号验证登陆信息发生异常:" + ex);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return result;
    }

    /**
     * 督导模式下的登陆验证:只能是区域人员才可以登陆,optype='QY'
     *
     * @param deptcsid
     * @param userid
     * @param password
     * @return
     */
    public static boolean LoginCheckBanShiChi(String deptcsid, String userid, String password) {

        boolean result = false;
        // password = DeviceHelper.getMD5Str(password); // MD5加密
        Cursor cursor = null;
        String sql = "select operatorid,opername,staffid,password,sex,optype,deptcsid,storeid,status from czyda001  " + "  where operatorid=? and password=?  and optype='QY'";
        try {
            cursor = BaseAppManager.database().rawQuery(sql, new String[]{userid, password});
            if (cursor.moveToNext()) {
                // 登陆成功
                String operatorid = cursor.getString(0);
                SystemHelper.LoginUserId = StringUtils.isEmpty(operatorid) ? operatorid : "";
                String opername = cursor.getString(1);
                SystemHelper.LoginUserName = StringUtils.isEmpty(opername) ? opername : "";
                result = true;
            }
        } catch (Exception e) {
            result = false;
            BaseAppManager.loger().info("根据用户名、密码、门店编号验证登陆信息发生异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return result;
    }

    /**
     * 根据id获取操作员名称
     */
    public static String GetOperateName(String operId) {
        String name = "未知";
        String sql = "select opername from czyda001 where operatorid=?";
        Cursor cursor = null;
        try {
            cursor = BaseAppManager.database().rawQuery(sql, new String[]{operId});
            if (cursor.moveToNext()) {
                name = StringUtils.isEmpty(cursor.getString(0)) ? cursor.getString(0) : "未知";
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("根据id获取操作员名称发生异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return name;
    }

    /**
     * 根据id获取操作员可销售的产品品牌
     */
    public static String getUserBrandList(String operId) {
        String brandList = "";
        String sql = "select BRANDLIST from czyda001 where operatorid=?";
        Cursor cursor = null;
        try {
            cursor = BaseAppManager.database().rawQuery(sql, new String[]{operId});
            if (cursor.moveToNext()) {
                brandList = StringUtils.isEmpty(cursor.getString(0)) ? cursor.getString(0) : "";
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("根据id获取操作员名称发生异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return brandList;
    }

    /**
     * 根据id获取操作员联系方式
     */
    public static String GetOperateMobile(String operId) {
        String name = "";
        String sql = "select mobile from czyda001 where operatorid=?";
        Cursor cursor = null;
        try {
            cursor = BaseAppManager.database().rawQuery(sql, new String[]{operId});
            if (cursor.moveToNext()) {
                name = (TextUtils.isEmpty(cursor.getString(0)) || cursor.getString(0).equals("null")) ?  "" : cursor.getString(0);
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("根据id获取操作员联系方式发生异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return name;
    }


    /**
     * 获取机构下的所有营养顾问
     */
    public static List<CZYDA> GetYingYangGuWen(String storeid) {
        Cursor cursor = null;
        List<CZYDA> list = new ArrayList<CZYDA>();
        String sql;
        try {
            if (!StringUtils.isEmpty(storeid)) {
                sql = "select operatorid,opername from czyda001 where optype='CX'";
                cursor = BaseAppManager.database().rawQuery(sql, null);
            } else {
                sql = "select operatorid,opername from czyda001 where optype='CX' and storeid=?";
                cursor = BaseAppManager.database().rawQuery(sql, new String[]{storeid});
            }
            while (cursor.moveToNext()) {
                CZYDA ent = new CZYDA();
                String operatorid = cursor.getString(cursor.getColumnIndex("OPERATORID"));
                ent.setOperateId(operatorid == null ? "" : operatorid);
                String opername = cursor.getString(cursor.getColumnIndex("OPERNAME"));
                ent.setOperateName(opername == null ? "" : opername);
                list.add(ent);
            }
        } catch (Exception e) {

            BaseAppManager.loger().info("督导模式下获取营养顾问异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return list;
    }

    /**
     * 根据id和门店获取获取操作员名称
     */
    public static String GetOperateName(String operId, String storeId) {
        String name = null;
        String sql = "select opername from czyda001 where operatorid=? and storeId=?";
        Cursor cursor = null;
        try {
            cursor = BaseAppManager.database().rawQuery(sql, new String[]{operId, storeId});
            if (cursor.moveToNext()) {
                name = StringUtils.isEmpty(cursor.getString(0)) ? cursor.getString(0) : "未知";
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("根据id获取操作员名称发生异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return name;
    }

    /**
     * 督导模式下，获取区号：从员工资料里面随便找一个员工编号为8个长度，然后取其前三位就是区号
     */
    public static String GetAreaNumber() {
        String area = "";
        String sql = "select operatorid from czyda001";
        Cursor cursor = null;
        try {
            cursor = BaseAppManager.database().rawQuery(sql, null);
            while (cursor.moveToNext()) {
                area = cursor.getString(0) == null ? "" : cursor.getString(0);
                if (area.length() == 8) {
                    area = area.substring(0, 3);
                    break;
                }
            }
        } catch (Exception e) {
            BaseAppManager.loger().info("根据员工资料获取区号异常:" + e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return area;
    }

    /**
     * 保存在线登陆用户
     */
    public static void saveOnlineUser(String userName, String userSex, String userId, String passWord, String mobile, String role,String brandList) {
        String sql = "insert or replace into CZYDA001(OPERATORID,OPERNAME,PASSWORD,SEX,OPTYPE,STOREID,STATUS,MOBILE,WGMFROLE,BRANDLIST) values(?,?,?,?,?,?,?,?,?,?)";
        try {
            BaseAppManager.database().execSQL(sql, new Object[]{userId, userName, DeviceHelper.getMD5Str(passWord), userSex, "CX", SystemHelper.LoginOrganId, "0", mobile, role,brandList});
        } catch (Exception e) {
            BaseAppManager.loger().info("保存在线登陆用户异常:" + e);
        }
    }

    /**
     * 修改员工密码
     *
     * @param yg
     */
    public static boolean updatePassWord(YuanGongEntity yg) {
        String sql = "update CZYDA001 set PASSWORD='" + yg.PASSWORD + "' where OPERATORID ='" + yg.OPERATORID + "'";
        try {
            BaseAppManager.database().execSQL(sql);
        } catch (Exception e) {

            BaseAppManager.loger().info("修改员工密码异常:" + e);
            return false;
        }
        return true;
    }

    /**
     * 修改员工密码
     */
    public static boolean updatePassWord(String operatorid, String password){
        password = DeviceHelper.getMD5Str(password); // MD5加密
        String sql = "update CZYDA001 set PASSWORD='" + password + "' where OPERATORID ='" + operatorid + "'";
        try {
            BaseAppManager.database().execSQL(sql);
        } catch (Exception e) {
            BaseAppManager.loger().info("修改员工密码异常:" + e);
            return false;
        }
        return true;
    }
}