@file:JvmName("PrivilegeDbOperate")

package com.urovo.wgmf.db.dao.dao

import android.app.AlertDialog
import android.content.Context
import android.content.DialogInterface
import android.database.SQLException
import com.urovo.wgmf.db.dao.entitys.PrivilegeRes
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.system.BaseAppManager
import com.urovo.wugumofang.system.SystemHelper
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.uiThread
import java.lang.ref.WeakReference

/**
 * @author Sasuke on 2018/3/19.
 * 操作privilege和privilegeRel两个表
 */
class PrivilegeDbOperate {

    companion object {
        /**
         * 下载权限资料的返回结果处理（写入）
         */
        @JvmStatic
        fun dealPrivilegeDataResult(res: PrivilegeRes?): Boolean {
            synchronized(PrivilegeDbOperate::class.java) {
                if (res == null) return false

                BaseAppManager.database().beginTransaction()
                var writeSuccess = false//是否写入成功
                try {
                    //写入基础数据
                    res.takeIf { true }?.let {
                        res.resultInfo.forEach { BaseAppManager.database().execSQL(it) }//循环执行insert
                    }
                    BaseAppManager.database().setTransactionSuccessful()
                    writeSuccess = true
                } catch (e: SQLException) {
                    e.printStackTrace()
                } finally {
                    BaseAppManager.database().endTransaction()
                }
                return writeSuccess
            }
        }

        /**
         * 用默认权限进行初始化（只在下载的时候，作为下载失败情况下的一种默认处理）
         * （本方法在主线程跑，因为外面需要手动跟刷新UI放在同一个子线程（刷新UI时手动切回主线程），
         * 以保证跑完本方法再执行界面跳转）
         */
        @JvmStatic
        fun writeDefaultPrivilege(userId: String): Boolean {
            synchronized(PrivilegeDbOperate::class.java) {
                BaseAppManager.database().beginTransaction()
                var writeSuccess = false//是否写入成功
                try {
                    BaseAppManager.database().execSQL("delete from privilege")
                    //写入基础数据
                    defaultPrivilege.forEach {
                        BaseAppManager.database()
                            .execSQL("insert into privilege (userno,reskey) values ('" + userId + "','" + it + "')")
                    }
                    BaseAppManager.database().setTransactionSuccessful()
                    writeSuccess = true
                } catch (e: SQLException) {
                    e.printStackTrace()
                } finally {
                    BaseAppManager.database().endTransaction()
                }
                return writeSuccess
            }
        }

        /**
         * 初始化本地写死的权限映射表
         */
        @JvmStatic
        fun initPrivilegeMap(context: Context) {
            val ref = WeakReference<Context>(context)
            doAsync {
                synchronized(PrivilegeDbOperate::class.java) {
                    BaseAppManager.database().beginTransaction()
                    var writeSuccess = false//是否写入成功
                    try {
                        BaseAppManager.database().execSQL("delete from privilegeRel")
                        //写入基础数据
                        privilegeMap.forEach {
                            BaseAppManager.database().execSQL(
                                "insert into privilegeRel (reskey,linkClass,params,title) " +
                                        "values ('" + it.resKey + "','" + it.linkClass + "','" + it.params + "','" + it.title + "')"
                            )
                        }
                        BaseAppManager.database().setTransactionSuccessful()
                        writeSuccess = true
                    } catch (e: SQLException) {
                        e.printStackTrace()
                    } finally {
                        BaseAppManager.database().endTransaction()
                    }
                    if (!writeSuccess && ref != null && ref.get() != null) {
                        uiThread {
                            AlertDialog.Builder(ref.get()).setMessage("权限本地映射表初始化失败")
                                .setPositiveButton(
                                    "重试",
                                    DialogInterface.OnClickListener { dialog, which ->
                                        ref.get()?.let { initPrivilegeMap(ref.get() as Context) }
                                    }).create().show()
                        }
                    }
                }
            }
        }

        /**
         * 校验用户是否有权限（如果从权限表里找到的符合数量大于0，就是true，否则false）
         */
        @JvmStatic
        fun checkUserPrivilege(linkClass: String): Boolean {
            synchronized(PrivilegeDbOperate::class.java) {
                return 0 < BaseAppManager.database().rawQuery(
                    "select count(p.reskey) as count from privilege p inner " +
                            "join privilegeRel pr on p.reskey=pr.reskey " +
                            "where p.userno = '" + SystemHelper.LoginUserId + "' and pr.linkClass = '" + linkClass + "'",
                    null
                )?.let { with(it?.moveToFirst()) { it?.getInt(0) } } ?: 0
            }
        }

        /**
         * 检查权限下载是否成功
         * （true则是网络下载成功，false则是失败后加载默认权限）
         */
        @JvmStatic
        fun isPrivilegeDownSuccess(): Boolean {
            synchronized(PrivilegeDbOperate::class.java) {
                BaseAppManager.database()
                    .rawQuery("select reskey from privilege where reskey='DEFAULT'", null)
                    ?.let { return false } ?: return true
            }
        }


        //        data class PrivilegeMap(var resKey: String,
//                                var linkClass: String,
//                                var params: String,
//                                var title: String): PrivilegeBase{
//
//        }
        class PrivilegeMap(
            var resKey: String,
            var linkClass: String,
            var params: String,
            var title: String
        ) {
            constructor(resKey: String, linkClass: String, title: String) : this(
                resKey,
                linkClass,
                "0",
                title
            )
        }

        /**
         * 写死的本地权限映射表的数据源
         * "01": 权限标志，前后端协商定义好的，后台移动端都用这一套。
         * “AppConstants.Home.HOME”：一开始定义的是该功能实现的类名，发现h5页面链接太长或者是有些模块根本就没有类名，所以就自定义了,反正都是自己存到数据库的值，checkUserPrivilege方法也是查自己数据库有没有这个值，所以写啥都行。
         * "首页"： 方便查阅是哪个功能
         */
        private val privilegeMap: List<PrivilegeMap> = listOf(

            PrivilegeMap("01", AppConstants.Home.HOME, "首页"),

            PrivilegeMap("02", AppConstants.Member.MEMBER, "会员"),
            PrivilegeMap("0201", AppConstants.Member.E_DISCOVERY, "数据展示"),
            PrivilegeMap("020101", AppConstants.Member.MEMBERSHIP_STORE_DATA, "本月门店会员数据"),
            PrivilegeMap("020102", AppConstants.Member.MEMBERSHIP_STORE_MALL_DATA, "本月门店会员商城数据"),
            PrivilegeMap("0202", AppConstants.Member.MARKETING_POLICY, "营销策略"),
            PrivilegeMap("020201", AppConstants.Member.MEMBER_LIST, "会员构成"),
            PrivilegeMap("020202", AppConstants.Member.SAN_XIANG_PAO, "新客三响炮"),
            PrivilegeMap("020203", AppConstants.Member.INTEGRAL_CONVERTIBLE, "积分兑换"),
            PrivilegeMap("020204", AppConstants.Member.OLD_GUEST_AWAKENS, "老客唤醒"),
            PrivilegeMap("020205", AppConstants.Member.DEVELOP_NEW_MEMBER, "发展新会员"),
            //PrivilegeMap("020206", AppConstants.Member.ORDER_CENTER, "订单中心"),
            //PrivilegeMap("020207", AppConstants.Member.MEMBER_DAY_SUBSCRIBE, "会员日预约名单"),
            PrivilegeMap("020208", AppConstants.Member.CUSTOMER_CARE_LIST, "客户关怀"),
            PrivilegeMap("020209", AppConstants.Member.WX_FRIENDS_RES, "朋友圈素材"),
            PrivilegeMap("020210", AppConstants.Member.CUSTOMER_PRIVILEGE, "特权查询"),
            PrivilegeMap("020211", AppConstants.Member.VOUCHER_CARD_QUERY, "权益卡查询"),

            PrivilegeMap("0203", AppConstants.Member.DATA_QUERY, "数据查询"),
            PrivilegeMap("020309", AppConstants.Member.MEMBER_WECHAT_GROUP, "加微入群人数提报"),
            PrivilegeMap("020301", AppConstants.Member.NEW_GUEST_MEMBER_DATA, "新客进塘数据"),
            PrivilegeMap("020302", AppConstants.Member.NEW_CUSTOMER_DATA, "新客数据"),
            PrivilegeMap("020303", AppConstants.Member.NEW_GUEST_REPURCHASE_RATE, "新客90天复购率"),
            PrivilegeMap("020310", AppConstants.Member.RA_90_NO_RETURN_LIST, "新客90天未回头名单"),
            PrivilegeMap("020304", AppConstants.Member.VIP_DAY_DATA, "会员日数据"),
            PrivilegeMap("020311", AppConstants.Member.VOUCHER_CARD_DATA, "权益卡数据"),

            //PrivilegeMap("020305", AppConstants.Member.MEMBER_SALE, "会员商城销售查询"),
            //PrivilegeMap("020306", AppConstants.Member.MEMBER_GIFT, "会员商城体验装兑换名单"),
            PrivilegeMap("020307", AppConstants.Member.QUERY_MEMBER_SALES_COMPARED, "会员销售占比查询"),
            PrivilegeMap("020308", AppConstants.Member.QUERY_CREDIT_STATEMENT, "积分兑换流水查询"),
            PrivilegeMap("020312", AppConstants.Member.CUSTOMER_EVALUATION_QUERY, "顾客评价查询"),

            PrivilegeMap("03", AppConstants.SalesPerformance.SALES_PERFORMANCE, "业绩"),
            PrivilegeMap("0301", AppConstants.SalesPerformance.SALES_STORE_MONTH, "本月门店销量"),
            //PrivilegeMap("0302", AppConstants.SalesPerformance.SALES_FORECAST_MONTH, "本月薪资预估"),
            PrivilegeMap("0303", AppConstants.SalesPerformance.SALES_DATA_QUERY, "数据查询"),
            PrivilegeMap(
                "030301",
                AppConstants.SalesPerformance.QUERY_SALES_DETAIL_NATIVE,
                "本机销售记录"
            ),
            PrivilegeMap(
                "030302",
                AppConstants.SalesPerformance.QUERY_FINAL_PRODUCT_SALES,
                "产品销售查询"
            ),
            PrivilegeMap("030303", AppConstants.SalesPerformance.QUERY_SALES_DETAIL_NET, "销售明细查询"),
            PrivilegeMap("030304", AppConstants.SalesPerformance.MONTH_SALE_ACT, "月销量查询"),
            PrivilegeMap("030305", AppConstants.SalesPerformance.MONTH_SALE_PRODUCT_ACT, "产品月销量查询"),
            PrivilegeMap("030306", AppConstants.SalesPerformance.QUERY_GUEST_FOLIO, "客单查询"),

            PrivilegeMap("04", AppConstants.MsgCenter.MSG_CENTER, "消息"),

            PrivilegeMap("05", AppConstants.More.MORE, "更多"),
            PrivilegeMap("0501", AppConstants.More.PERSONAL_CENTER, "个人中心"),
            PrivilegeMap("050101", AppConstants.More.SIGN_IN_AND_OUT_ACTIVITY, "打卡"),
            PrivilegeMap("050102", AppConstants.More.VACATION_ACT, "请假"),

            PrivilegeMap("05010201", AppConstants.More.NATIVE_ASK_ABSENCE_ACT, "请假申请"),
            PrivilegeMap("05010202", AppConstants.More.NATIVE_QUERY_ABSENCE_ACT, "请假查询"),
            PrivilegeMap("05010203", AppConstants.More.NATIVE_ANNUAL_VACATION_ACT, "年假查询"),
            PrivilegeMap("05010204", AppConstants.More.NATIVE_ASK_ABSENCE_ACT2, "调休申请"),//和请假申请相同
            PrivilegeMap("05010205", AppConstants.More.NATIVE_QUERY_ABSENCE_ACT2, "调休查询"),//和请假查询相同

            PrivilegeMap("050103", AppConstants.More.ATTENDANCE_MANAGER_MENU, "考勤管理"),

            PrivilegeMap("05010301", AppConstants.More.URL_CREW_ARRANGMENT_CHECK, "排班设置"),
            PrivilegeMap("05010302", AppConstants.More.NATIVE_ATTEN_HISTORY_ACT, "考勤记录查询"),
            PrivilegeMap(
                "05010303",
                AppConstants.More.NATIVE_SHIFT_QUERY_AND_CONFIRM_ACT,
                "排班查询及确认"
            ),

            PrivilegeMap("050104", AppConstants.More.SALARY_QUERY, "薪资查询"),

            PrivilegeMap("05010401", AppConstants.More.NATIVE_OFFICIAL_SALARY_QUERY, "正式薪资查询"),
            PrivilegeMap("05010402", AppConstants.More.URL_REAL_TIME_SALARY_QUERY, "本月薪资预估"),
            PrivilegeMap("05010403", AppConstants.More.URL_SALARY_CALCULATORS, "薪资计算器"),
            PrivilegeMap(
                "05010404",
                AppConstants.More.NATIVE_FLOW_SALES_PERSON_COMPLETION,
                "流动顾问销售完成情况"
            ),
            PrivilegeMap("05010405", AppConstants.More.NATIVE_YEAR_AWARD, "年终奖查询"),
            PrivilegeMap("05010406", AppConstants.More.URL_STORE_PROFIT, "门店利润查询"),
            PrivilegeMap("05010407", AppConstants.More.URL_UNION_OPERATION, "联营门店利润报表"),

            PrivilegeMap("050105", AppConstants.More.PERSONAL_ACT, "信息确认"),
            PrivilegeMap("050106", AppConstants.More.QR_CODE, "专属二维码"),
            PrivilegeMap("050107", AppConstants.More.ACTIONS_QRCODE_ACT, "活动二维码"),
            PrivilegeMap("050108", AppConstants.More.WECHAT_QRCODE_ACTIVITY, "微信二维码"),
            PrivilegeMap("050109", AppConstants.More.QUERY_TICKET_ACT, "优惠券查询"),

            PrivilegeMap("0505", AppConstants.More.QUALITY_BACK_MODEL, "上报反馈"),
            PrivilegeMap("050501", AppConstants.More.QUALITY_BACK_ACTIVITY, "问题上报"),
            PrivilegeMap("050502", AppConstants.More.MACHINE_REPAIR_ACT, "呼叫机修"),

            PrivilegeMap("050503", AppConstants.More.SHISAN_REPORT_ACT, "门店柜型"),
            PrivilegeMap("050504", AppConstants.More.QBACK_INPUT_ACT_CHINRETSU, "CHINRETSU", "多点陈列"),
            PrivilegeMap("050505", AppConstants.More.COMPLAIN_ENTRY_ACT_COMPLAIN, "COMPLAIN", "我要找督察"),
            PrivilegeMap("050506", AppConstants.More.QBACK_INPUT_ACT_SUGGESTION, "SUGGESTION", "意见与建议"),
            PrivilegeMap("050507", AppConstants.More.QBACK_INPUT_ACT_LIGHT, "LIGHT", "亮灯情况上报"),
            PrivilegeMap("050508", AppConstants.More.INSPECTOR_DISPLAY_CHECK_ACTIVITY, "LIGHT", "督察陈列检查"),
            PrivilegeMap("050509", AppConstants.More.DISPLAY_REPORT, "LIGHT", "陈列上报"),

            PrivilegeMap("0502", AppConstants.More.GOODS_MANAGEMENT, "货品管理"),
            PrivilegeMap("050201", AppConstants.More.ORDER_GOODS_TAB_ACT, "门店订货"),
            PrivilegeMap("050202", AppConstants.More.RECEIVE_GOODS_TAB_MENU, "配送收货"),
            PrivilegeMap("050203", AppConstants.More.CHECK_GOODS_TAB_MENU, "存货盘点"),
            PrivilegeMap("050204", AppConstants.More.TRANSFER_QUERY_ACTIVITY, "调拨"),
            PrivilegeMap("050205", AppConstants.More.STOCK_SEARCH_ACTIVITY, "库存查询"),
            PrivilegeMap("050206", AppConstants.More.GOODS_IN_OUT_STOCK_ACT, "产品出入库"),
            PrivilegeMap("050207", AppConstants.More.REIMBURSE_GOODS_ACTIVITY, "报损"),
            PrivilegeMap("050208", AppConstants.More.GIFT_GOODS_ACTIVITY, "赠送"),
            PrivilegeMap("050209", AppConstants.More.LABEL_PRINTER_ACT_PROD, "PROD", "标签打印"),
            PrivilegeMap("050210", AppConstants.More.EAT_MANAGER2, "试吃"),
            PrivilegeMap("050211", AppConstants.More.OVER_DUE_QUERY_ACTIVITY, "临期查询"),
            PrivilegeMap("050212", AppConstants.More.LABEL_PRINTER_ACT_DDLINE, "DDLINE", "期限标签"),
            PrivilegeMap("0503", AppConstants.More.LEARNING_AND_TRAINING, "学习培训"),
            PrivilegeMap("050301", AppConstants.More.E_LEARNING, "e-learning"),
            PrivilegeMap("050302", AppConstants.More.LEARNING_CENTER_MENU_ACTIVITY_1, "1", "学习中心"),
            PrivilegeMap("050303", AppConstants.More.LEARNING_CENTER_MENU_ACTIVITY_2, "2", "养生文库"),
            PrivilegeMap("050304", AppConstants.More.CAREER_TRAINING_ACT_TRAINING, "TRAINING", "入职培训"),
            PrivilegeMap("050305", AppConstants.More.CAREER_EVALUATE_LIST_ACT, "入职评估"),

            PrivilegeMap("0504", AppConstants.More.NATIVE_MANAGEMENT, "本机管理"),
            PrivilegeMap("050401", AppConstants.More.UP_AND_DOWN_DATA_DOWN, "数据下载"),
            PrivilegeMap("050402", AppConstants.More.UP_AND_DOWN_DATA_UP, "数据上传"),
            PrivilegeMap("050403", AppConstants.More.UP_AND_DOWN_DATA_CLEAN, "数据清理"),
            PrivilegeMap("050404", AppConstants.More.NORMAL_QUESTION_ACT, "常见问题")
        )

        /**
         * 在网络下载权限失败时，加载的写死的默认用户权限
         */
        private val defaultPrivilege: List<String> = listOf(
            "DEFAULT",
            "01",
            "05",
            "0501",
            "050101",
            "050106",
            "0502",
            "050202",
            "050203",
            "050210",
            "0504",
            "050401",
            "050402",
            "050403"
        )
    }
}