package com.urovo.wugumofang.constants;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.media.ExifInterface;
import android.os.Environment;
import android.util.Log;

import com.google.gson.Gson;
import com.urovo.wugumofang.base.BaseResponse;
import com.urovo.wugumofang.data.DateUtil;
import com.urovo.wugumofang.ui.BitmapUtil;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Random;

import androidx.annotation.NonNull;

import static org.greenrobot.eventbus.EventBus.TAG;

public class PhotoUtils {

    public static void saveBitmap(Bitmap bm, String picName) {
        try {
            File f = new File(Files.getPhotoPath(), picName + ".jpg");
            FileOutputStream out = new FileOutputStream(f);
            bm.compress(Bitmap.CompressFormat.JPEG, 90, out);
            out.flush();
            out.close();
        } catch (IOException e) {
            Log.e("PhotoUtils", "saveBitmap: ", e);
        }
    }

    /**
     * 把压缩并加了水印的图片写入文件
     */
    public static String compress2Local(Bitmap mBitmap, String savePhotoPath) {
        String sdStatus = Environment.getExternalStorageState();
        // 检测sd是否可用
        if (!sdStatus.equals(Environment.MEDIA_MOUNTED)) {
            return "";
        }
        FileOutputStream b = null;
        File file = new File(Files.photoPath);
        if (!file.exists()) {
            // 创建文件夹
            file.mkdirs();
        }
        try {
            File photoFile = new File(savePhotoPath);
            //缓存的图片可以删了
            photoFile.delete();
            b = new FileOutputStream(photoFile);
            //压缩到100KB以下
            int option = BitmapUtil.getCompressOption(mBitmap, 90);
            // 把数据写入文件
            mBitmap.compress(Bitmap.CompressFormat.JPEG, option, b);

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } finally {
            try {
                // 关闭流
                if (null != b) {
                    b.flush();
                    b.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
        return savePhotoPath;
    }


    /**
     * 修改图片尺寸
     *
     * @param path 图片路径
     */
    public static Bitmap revitionImageSize(String path) {
        Bitmap bitmap = null;
        BufferedInputStream in = null;
        try {
            in = new BufferedInputStream(new FileInputStream(new File(path)));
            BitmapFactory.Options options = new BitmapFactory.Options();
            options.inJustDecodeBounds = true;
            BitmapFactory.decodeStream(in, null, options);
            in.close();
            int i = 0;
            while (true) {
                if ((options.outWidth >> i <= 1000)
                        && (options.outHeight >> i <= 1000)) {
                    in = new BufferedInputStream(new FileInputStream(new File(
                            path)));
                    options.inSampleSize = (int) Math.pow(2.0D, i);
                    options.inJustDecodeBounds = false;
                    bitmap = BitmapFactory.decodeStream(in, null, options);
                    break;
                }
                i += 1;
            }

        } catch (IOException e) {
            Log.e("PhotoUtils", "revitionImageSize: ", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    Log.e("PhotoUtils", "revitionImageSize: ", e);
                }
            }
        }
        return bitmap;
    }

    /**
     * 压缩图片
     *
     * @param imgPath 图片路径
     * @return 压缩后的图片
     */
    public static Bitmap copressImage(String imgPath) {
        Bitmap bitmap = null;
        float imagew = 480;
        float imageh = 800;
        File picture = new File(imgPath);
        BitmapFactory.Options bitmapFactoryOptions = new BitmapFactory.Options();
        //下面这个设置是将图片边界不可调节变为可调节
        bitmapFactoryOptions.inJustDecodeBounds = true;
        bitmapFactoryOptions.inSampleSize = calculateInSampleSize(bitmapFactoryOptions, imagew, imageh);

        int yRatio = (int) Math.ceil(bitmapFactoryOptions.outHeight / imageh);
        int xRatio = (int) Math.ceil(bitmapFactoryOptions.outWidth / imagew);
        if (yRatio > 1 || xRatio > 1) {
            if (yRatio > xRatio) {
                bitmapFactoryOptions.inSampleSize = yRatio;
            } else {
                bitmapFactoryOptions.inSampleSize = xRatio;
            }

        }
        //false --- allowing the caller to query the bitmap without having to allocate the memory for its pixels.
        bitmapFactoryOptions.inJustDecodeBounds = false;
        bitmap = BitmapFactory.decodeFile(picture.getAbsolutePath(), bitmapFactoryOptions);
        if (bitmap != null) {
            return bitmap;
        }
        return null;
    }

    private static int calculateInSampleSize(BitmapFactory.Options options,
                                             float reqWidth, float reqHeight) {
        // 源图片的高度和宽度
        final int height = options.outHeight;
        final int width = options.outWidth;
        int inSampleSize = 1;
        if (height > reqHeight || width > reqWidth) {
            // 计算出实际宽高和目标宽高的比率
            final int heightRatio = Math.round((float) height / reqHeight);
            final int widthRatio = Math.round((float) width / reqWidth);
            // 选择宽和高中最小的比率作为inSampleSize的值，这样可以保证最终图片的宽和高
            // 一定都会大于等于目标的宽和高。
            inSampleSize = heightRatio < widthRatio ? heightRatio : widthRatio;
        }
        return inSampleSize;
    }

    /**
     * 质量压缩方法
     *
     * @param image
     * @return
     */
    public static Bitmap compressImage(Bitmap image, int size) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        image.compress(Bitmap.CompressFormat.JPEG, 100, baos);// 质量压缩方法，这里100表示不压缩，把压缩后的数据存放到baos中
        int options = 90;
        while (baos.toByteArray().length / 1024 > size) { // 循环判断如果压缩后图片是否大于100kb,大于继续压缩
            baos.reset(); // 重置baos即清空baos
            options -= 10;// 每次都减少10
            image.compress(Bitmap.CompressFormat.JPEG, options, baos);// 这里压缩options%，把压缩后的数据存放到baos中
            if (options <= 10) {
                break;
            }
        }
        ByteArrayInputStream isBm = new ByteArrayInputStream(baos.toByteArray());// 把压缩后的数据baos存放到ByteArrayInputStream中
        Bitmap bitmap = BitmapFactory.decodeStream(isBm, null, null);// 把ByteArrayInputStream数据生成图片
        //添加水印
        bitmap = PhotoUtils.setDateBitmap(bitmap);
        return bitmap;
    }

    public static byte[] getImageBytes(String filePath, int size) {
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inJustDecodeBounds = true;
        //第一次加载，返回null，没有生成bitmap
        BitmapFactory.decodeFile(filePath, options);
        // 源图片的高度和宽度
        int height = options.outHeight;
        int width = options.outWidth;

        int inSampleSize = 1;
        if (height > 1008 || width > 756) {
            // 计算出实际宽高和目标宽高的比率
            final int heightRatio = Math.round((float) height / (float) 1008);
            final int widthRatio = Math.round((float) width / (float) 756);
            inSampleSize = Math.min(heightRatio, widthRatio);
        }
        options.inSampleSize = inSampleSize;
        options.inJustDecodeBounds = false;

        Bitmap mBitmap = BitmapFactory.decodeFile(filePath, options);
        int degree = getBitmapDegree(filePath);
        //图片被系统旋转,那我们就要把它转回来
        if (degree != 0) {
            mBitmap = rotateBitmapByDegree(mBitmap, degree);
        }
        byte[] bytes = null;
        if (mBitmap != null) {
            //压缩到100KB以下（不用保存到本地，直接把字节流传给七牛）
            bytes = PhotoUtils.compressBmp2Bytes(mBitmap, size);

            mBitmap.recycle();
        }
        return bytes;
    }
    /**
     * 质量压缩方法
     * @param image
     * @return
     */
    /**
     * 通过压缩质量来压缩Bitmap图片（这里返回的是压缩后得到的byte数组 ）　　Sasukeより
     *
     * @param image
     * @return
     */
    public static byte[] compressBmp2Bytes(Bitmap image, int size) {
        byte[] bytes;
        //缩放图片,缩放会牺牲清晰度
//        if (null != image && (image.getHeight() > 756 || image.getWidth() > 1008)) {
//            image = resizeTo(image, 756, 1008);
//        }

        //添加水印
        image = PhotoUtils.setDateBitmapMin(image);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        image.compress(Bitmap.CompressFormat.JPEG, 100, baos);//质量压缩方法，这里100表示不压缩，把压缩后的数据存放到baos中
        int quality = 100;
        while (baos.toByteArray().length / 1024 > size) { //循环判断如果压缩后图片是否大于60kb,大于继续压缩
            baos.reset();//重置baos即清空baos
            quality -= 10;//每次都减少10
            image.compress(Bitmap.CompressFormat.JPEG, quality, baos);//这里压缩options%，把压缩后的数据存放到baos中
            if (quality == 0)//如果图片的质量已降到最低则，不再进行压缩
                break;
        }
        bytes = baos.toByteArray();
        try {
            baos.close();
        } catch (IOException e) {
            Log.e(TAG, "IOException from BitmapUtil.class");
            e.printStackTrace();
        }
        return bytes;
    }

    /**
     * 重新设置Bitmap尺寸
     *
     * @param bmp
     * @param newWidth
     * @param newHeight
     * @return
     */
    public static Bitmap resizeTo(Bitmap bmp, int newWidth, int newHeight) {
        int width = bmp.getWidth();
        int height = bmp.getHeight();
        // 计算缩放比例
        float scaleHeight;
        float scaleWidth;
        if (newWidth > newHeight) {
            scaleWidth = ((float) newWidth) / Math.max(width, height);
            scaleHeight = ((float) newHeight) / Math.min(width, height);
        } else {
            scaleWidth = ((float) newWidth) / Math.min(width, height);
            scaleHeight = ((float) newHeight) / Math.max(width, height);
        }
        /*float scaleWidth = ((float) newWidth) / width;
        float scaleHeight = ((float) newHeight) / height;*/
        // 取得想要缩放的matrix参数
        Matrix matrix = new Matrix();
        matrix.postScale(scaleWidth, scaleHeight);
        // 得到新的图片
        return Bitmap.createBitmap(bmp, 0, 0, width, height, matrix,
                true);
    }

    /**
     * @return Bitmap 返回类型
     * 添加水印
     */
    public static Bitmap setDateBitmap(Bitmap bmp) {
        Bitmap temp = null;
        try {
            //Android默认的颜色模式为ARGB_8888，这个颜色模式色彩最细腻，显示质量最高。
            //但同样的，占用的内存//也最大。也就意味着一个像素点占用4个字节的内存。我们来做一个简单的计算题：3200*2400*4 bytes //=30M。如此惊人的数字！哪怕生命周期超不过10s，Android也不会答应的。
            temp = Bitmap.createBitmap(bmp.getWidth(), bmp.getHeight(), Bitmap.Config.RGB_565);
            Canvas canvas = new Canvas(temp);
            // 建立画笔
            Paint paint = new Paint();
            paint.setDither(true);
            paint.setFilterBitmap(true);
            Rect src = new Rect(0, 0, bmp.getWidth(), bmp.getHeight());
            Rect dst = new Rect(0, 0, bmp.getWidth(), bmp.getHeight());
            canvas.drawBitmap(bmp, src, dst, paint);

            Paint textPaint = new Paint(Paint.ANTI_ALIAS_FLAG
                    | Paint.DEV_KERN_TEXT_FLAG);
            textPaint.setTextSize(80.0f);
            // 采用默认的宽度
            textPaint.setTypeface(Typeface.DEFAULT_BOLD);
            textPaint.setColor(Color.RED);

            Date now = new Date();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd-HH:mm:ss");
            String str = sdf.format(now);
            String d = str.split("-")[0];
            String t = str.split("-")[1];
            Rect m = new Rect();
            Rect c = new Rect();
            textPaint.getTextBounds(d, 0, d.length() - 1, m);
            textPaint.getTextBounds(t, 0, t.length() - 1, c);
            canvas.drawText(d, bmp.getWidth() - m.width() - 100, bmp.getHeight() - 20, textPaint);
            canvas.drawText(t, bmp.getWidth() - c.width() - 100, bmp.getHeight() - 100, textPaint);
            canvas.save();
            canvas.restore();
        } catch (Exception e) {
            temp = null;
            Log.e("PhotoUtil", "setDateBitmap: ", e);
        }
        return temp;
    }

    /**
     * @return Bitmap 返回类型
     * 添加水印
     */
    public static Bitmap setDateBitmapMin(Bitmap bmp) {
        Bitmap temp = null;
        try {
            //Android默认的颜色模式为ARGB_8888，这个颜色模式色彩最细腻，显示质量最高。
            //但同样的，占用的内存//也最大。也就意味着一个像素点占用4个字节的内存。我们来做一个简单的计算题：3200*2400*4 bytes //=30M。如此惊人的数字！哪怕生命周期超不过10s，Android也不会答应的。
            temp = Bitmap.createBitmap(bmp.getWidth(), bmp.getHeight(), Bitmap.Config.RGB_565);
            Canvas canvas = new Canvas(temp);
            // 建立画笔
            Paint paint = new Paint();
            paint.setDither(true);
            paint.setFilterBitmap(true);
            Rect src = new Rect(0, 0, bmp.getWidth(), bmp.getHeight());
            Rect dst = new Rect(0, 0, bmp.getWidth(), bmp.getHeight());
            canvas.drawBitmap(bmp, src, dst, paint);

            Paint textPaint = new Paint(Paint.ANTI_ALIAS_FLAG
                    | Paint.DEV_KERN_TEXT_FLAG);
            textPaint.setTextSize(20.0f);
            // 采用默认的宽度
            textPaint.setTypeface(Typeface.DEFAULT_BOLD);
            textPaint.setColor(Color.RED);

            Date now = new Date();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd-HH:mm:ss");
            String str = sdf.format(now);
            String d = str.split("-")[0];
            String t = str.split("-")[1];
            Rect m = new Rect();
            Rect c = new Rect();
            textPaint.getTextBounds(d, 0, d.length() - 1, m);
            textPaint.getTextBounds(t, 0, t.length() - 1, c);
            canvas.drawText(d, bmp.getWidth() - m.width() - 30, bmp.getHeight() - 10, textPaint);
            canvas.drawText(t, bmp.getWidth() - c.width() - 30, bmp.getHeight() - 30, textPaint);
            canvas.save();
            canvas.restore();
        } catch (Exception e) {
            temp = null;
            Log.e("PhotoUtil", "setDateBitmap: ", e);
        }
        return temp;
    }

    /**
     * 获得七牛上传凭证uploadtoken
     */

    public static String getUploadToken() {
        HttpClient client = new DefaultHttpClient();
        StringBuilder builder = new StringBuilder();

        HttpGet myget = new HttpGet(NetworkParams.URL_PRODUCT_TOKEN);
        Log.i("url request =", NetworkParams.URL_PRODUCT_TOKEN);

        BufferedReader reader = null;
        try {
            myget.getParams().setParameter("http.socket.timeout", 60000);
            HttpResponse response = client.execute(myget);
            reader = new BufferedReader(new InputStreamReader(
                    response.getEntity().getContent()));
            for (String s = reader.readLine(); s != null; s = reader.readLine()) {
                builder.append(s);
            }
            String json = builder.toString();
            Log.i("url response json=", json);

            BaseResponse mTokenBean = parseTokenData(json);

            return mTokenBean != null ? mTokenBean.retInfo : null;
        } catch (Exception e) {
            Log.e("url response", "false", e);
            return null;
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    Log.e("url response", "false", e);
                }
            }
        }
    }

    public static BaseResponse parseTokenData(String json) {
        Gson gson = new Gson();
        return gson.fromJson(json, BaseResponse.class);
    }

    public static String getFileUrlUUID(String shopId, String userId) {
        return "sygj_android_" + conversionString(shopId) + conversionString(userId)
                + conversionString(DateUtil.getDateTime1(new Date()))
                + (new Random().nextInt(10000));
    }

    @NonNull
    private static String conversionString(String str) {
        if (null == str) {
            str = "";
        } else {
            str = str + "_";
        }
        return str;
    }

    public static String getRealUrl(String fileUrlUUID) {
        return NetworkParams.QINIU_BASE + fileUrlUUID;
    }

    public static byte[] Bitmap2Bytes(Bitmap bm) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        if (null != bm) {
            bm.compress(Bitmap.CompressFormat.JPEG, 100, baos);
        }
        return baos.toByteArray();
    }

    /**
     * 图片添加水印
     *
     * @param path 图片路径
     */
    public static void setImageFlag(String path) {
        //通过图片路径拿到图片bitmap
        File picture = new File(path);
        if (picture.exists()) {
            BitmapFactory.Options bitmapFactoryOptions = new BitmapFactory.Options();
            bitmapFactoryOptions.inJustDecodeBounds = false;
            Bitmap bitmap = BitmapFactory.decodeFile(picture.getAbsolutePath(), bitmapFactoryOptions);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            if (null != bitmap) {
                bitmap.compress(Bitmap.CompressFormat.JPEG, 10, baos);
            }
            try {
                bitmap = setDateBitmap(bitmap);
            } catch (OutOfMemoryError e) {
                Log.e("PhotoUtils", "UploadReportPhoto: OutOfMemoryError");
            }
            if (bitmap != null && !bitmap.isRecycled()) {
                bitmap.recycle();
            }
            System.gc();
        } else {
            Log.e("PhotoUtils", "UploadReportPhoto: picture is not exists");
        }
    }

    /**
     * 读取图片的旋转的角度　　Sasukeより
     *
     * @param path 图片绝对路径
     * @return 图片的旋转角度
     */
    public static int getBitmapDegree(String path) {
        int degree = 0;
        try {
            // 从指定路径下读取图片，并获取其EXIF信息
            ExifInterface exifInterface = new ExifInterface(path);
            // 获取图片的旋转信息
            int orientation = exifInterface.getAttributeInt(ExifInterface.TAG_ORIENTATION,
                    ExifInterface.ORIENTATION_NORMAL);
            switch (orientation) {
                case ExifInterface.ORIENTATION_ROTATE_90:
                    degree = 90;
                    break;
                case ExifInterface.ORIENTATION_ROTATE_180:
                    degree = 180;
                    break;
                case ExifInterface.ORIENTATION_ROTATE_270:
                    degree = 270;
                    break;
                default:
                    break;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return degree;
    }

    /**
     * 将图片按照某个角度进行旋转　　Sasukeより
     *
     * @param bm     需要旋转的图片
     * @param degree 旋转角度
     * @return 旋转后的图片
     */
    public static Bitmap rotateBitmapByDegree(Bitmap bm, int degree) {
        Bitmap returnBm = null;

        // 根据旋转角度，生成旋转矩阵
        Matrix matrix = new Matrix();
        matrix.postRotate(degree);
        try {
            // 将原始图片按照旋转矩阵进行旋转，并得到新的图片
            returnBm = Bitmap.createBitmap(bm, 0, 0, bm.getWidth(), bm.getHeight(), matrix, true);
        } catch (OutOfMemoryError e) {
        }
        if (returnBm == null) {
            returnBm = bm;
        }
        if (bm != returnBm) {
            bm.recycle();
        }
        return returnBm;
    }
}