package com.urovo.wugumofang.data;

import android.util.Log;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class DataMonthUtils {
    private static final String TAG = DataMonthUtils.class.getName();
    static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");

    /**
     * 获取日期年份
     *
     * @param date
     * @return
     * @throws ParseException
     */
    public static int getYear(String date) throws ParseException {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateFormat.parse(date));
        return calendar.get(Calendar.YEAR);
    }

    /**
     * 获取日期月份
     *
     * @param date
     * @return
     * @throws ParseException
     */
    public static int getMonth(String date) throws ParseException {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateFormat.parse(date));
        return (calendar.get(Calendar.MONTH) + 1);
    }

    /**
     * 获取日期号
     *
     * @param date
     * @return
     * @throws ParseException
     */
    public static int getDay(String date) throws ParseException {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateFormat.parse(date));
        return calendar.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * 获取月份起始日期
     *
     * @param date
     * @return
     * @throws ParseException
     */
    public static String getMinMonthDate(String date) throws ParseException {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateFormat.parse(date));
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
        return dateFormat.format(calendar.getTime());
    }

    /**
     * 获取月份最后日期
     *
     * @param date
     * @return
     * @throws ParseException
     */
    public static String getMaxMonthDate(String date) throws ParseException {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateFormat.parse(date));
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        return dateFormat.format(calendar.getTime());
    }

    /**
     * 获取日期差，返回相差天数。
     *
     * @param startDate 开始时间
     * @param endDate   结束时间
     * @return Long 相差天数
     */
    public static Long getCompareDate(String startDate, String endDate) {
        try {
            Date date1 = dateFormat.parse(startDate);
            Date date2 = dateFormat.parse(endDate);
            long l = date2.getTime() - date1.getTime();
            long d = l / (24 * 60 * 60 * 1000);
            return d;
        } catch (ParseException e) {
            Log.e(TAG, "获取时间天数差失败。原因：" + e.getMessage());
            return null;
        }
    }

    /**
     * @param @return
     * @param @throws ParseException 参数说明
     * @return long 返回类型
     * @throws
     * @Title: getCompareDate
     * @Description: 返回相差天数
     */
    public static long getCompareDate(Date date1, Date date2) throws ParseException {
        long l = date2.getTime() - date1.getTime();
        long d = l / (24 * 60 * 60 * 1000);
        return d;
    }


    /**
     * 用给出的format来比较两个日期大小
     *
     * @param date1
     * @param date2
     * @param dateFormat 自己给定的format
     * @return
     */
    public static Long compareDate(String date1, String date2, SimpleDateFormat dateFormat) {
        try {
            long time1 = dateFormat.parse(date1).getTime();
            long time2 = dateFormat.parse(date2).getTime();
            return time1 - time2;
        } catch (ParseException e) {
            Log.e(TAG, "日期时间比较失败：" + e.getMessage());
            return null;
        }
    }
}
