package com.urovo.wugumofang.data;

import android.os.Bundle;

import androidx.annotation.NonNull;

import com.urovo.wugumofang.constants.AppConstants;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.Locale;

public class MoneyUtils {
    private static DecimalFormat decimalFormat = new DecimalFormat("#,##0.00");// 格式化设置
    private static DecimalFormat fourlastDF = new DecimalFormat("#0.####");
    private static DecimalFormat twolastDF = new DecimalFormat("#0.00");
    private static DecimalFormat onelastDF = new DecimalFormat("#0.0");

    private static DecimalFormat nolastDF = new DecimalFormat("#0");
    private static DecimalFormat formatnolastDF = new DecimalFormat("#,##0");
    private static DecimalFormat dynamicLastDF = new DecimalFormat("###.########");//有就显示，没有就不显示

    private static DecimalFormat onemostDF = new DecimalFormat("#0.#");//最多一位小数，有就显示1位，没有就不显示。

    /**
     * 单独后面两位小数点
     */
    public static String format(double d) {
        return decimalFormat.format(d);
    }

    public static String anylastDF(double formatValue, int accuracy) {
        String format = String.format(Locale.CHINA, "%." + accuracy + "f", 0d);
        return new DecimalFormat("#" + format).format(formatValue);
    }

    public static String twolastDF(double d) {
        return twolastDF.format(d);
    }

    public static String fourlastDF(double d) {
        return fourlastDF.format(d);
    }

//    /**
//     * 不进行四舍五入的保留两位
//     * @return
//     */
//    public static String twolastDFNo45(double d){
//        DecimalFormat df = new DecimalFormat("#0.00");
//        df.setRoundingMode(RoundingMode.FLOOR);
//        return df.format(d);
//    }

    public static String onelastDF(String str) {
        return onelastDF(Double.parseDouble(str));
    }

    public static String onelastDF(double d) {
        return onelastDF.format(d);
    }

    public static String nolastDF(double d) {
        return nolastDF.format(d);
    }

    public static String formatNolastDF(double d) {
        return formatnolastDF.format(d);
    }

    public static String dynamicLastDF(double d) {
        return dynamicLastDF.format(d);
    }

    public static String onemostDF(double d) {
        return onemostDF.format(d);
    }

    /**
     * 两位小数点情况下；是否需要进行四舍五入 flase 需要 true 不需要
     */
    public static String twolastDF(double d, boolean isDown) {
        if (isDown) {
            twolastDF.setRoundingMode(RoundingMode.DOWN);
        } else {
            twolastDF.setRoundingMode(RoundingMode.HALF_UP);
        }
        return twolastDF.format(d);
    }

    /**
     * double 转换为String字符串并精确accuracy位数
     *
     * @param formatValue 需要转换格式的数
     * @param valueLength 数的长度
     * @param accuracy    需要精确的位数
     * @return 转换后的String 字符串
     */
    public static String format2Str(double formatValue, int valueLength, int accuracy) {
        return (valueLength != 0) ? String.format(Locale.CHINA, "%0" + valueLength + "d",
                (int) Arith.mul(Double.parseDouble(accuracyAdjustment(formatValue, accuracy)), Math.pow(10, accuracy))) : "";
    }

    /**
     * 常数类型字符串精确保留accuracy位数 ，如：82299.99999999997 --> 823.00
     *
     * @param value    需要精确到accuracy位的常数
     * @param accuracy 需要精确的小数位数
     * @return 返回精确accuracy位数后的常数字符串
     */
    public static String accuracyAdjustment(Double value, int accuracy) {
        return new BigDecimal(value).setScale(accuracy, BigDecimal.ROUND_HALF_DOWN).toString();
    }

    //  不四舍五入取整
    public static String noRoundAdjustment(Double value) {
        return new BigDecimal(value).setScale(0, BigDecimal.ROUND_DOWN).toString();
    }

    /**
     * 传参
     *
     * @param args 参数
     */
    private static Bundle bundle = new Bundle();

    public static Bundle displayError(@NonNull String... args) {
        bundle.putString(AppConstants.Printer.TITLE_TIPS, args[0]);//打印错误信息标题
        bundle.putString(AppConstants.Printer.ERROR_TIPS, args[1]);// 打印错误信息内容
        return bundle;
    }
}
