package com.wgmf.library.location

import android.annotation.SuppressLint
import android.content.Context
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.os.Bundle
import com.google.gson.Gson
import com.urovo.wugumofang.constants.LocationParams
import com.urovo.wugumofang.system.LocationUtil
import com.zsy.libcore.common.android.log.Log
import java.util.*

/**
 * 系统定位
 * @author LW
 */
class SystemLocationManager(
    private val context: Context?,
    private val callback: LocationUtil.OnCallback?
) {

    private val mLocationManager =
        context?.getSystemService(Context.LOCATION_SERVICE) as LocationManager?

    private val mLocationListener = object : LocationListener {
        // 位置改变时获取经纬度
        override fun onLocationChanged(location: Location?) {
            location?.let {
                getGeocoder(it)
            }
        }

        // 状态改变时
        override fun onStatusChanged(provider: String, status: Int, extras: Bundle) {}

        // 提供者可以使用时
        override fun onProviderEnabled(provider: String) {}

        // 提供者不可以使用时
        override fun onProviderDisabled(provider: String) {}
    }

    /*
     **系统内容提供器介绍
     **一共为四种方式"passive","network","fused","gps"
     ** "gps"：GPS_PROVIDER(GNSS) GNSS HAL 接口和芯片打交道
     ** "network"：NETWORK_PROVIDER 依赖设备厂商的具体实现，Android 默认实现依赖 Google 提供的 GMS **  实现国内无法使用；一般厂商会实现为通过基站 / WIFI / 蓝牙进行融合定位
     ** "fused"：调用 GMS（谷歌移动服务） 进行定位，国内用不了
     ** "passive"：使用 PassiveProvider 在系统位置更新时通知给应用，分享其他 PROVIDER 的定位结果
     */
    @SuppressLint("MissingPermission")
    private fun getLastKnownLocation(locationManager: LocationManager?): Location? {
        var bestLocation: Location? = null
        locationManager?.let {
            val providers = it.getProviders(true)
            //由于国内用不了fused 剔除他
            if (providers.contains("fused")) providers.remove("fused")
            //这里我们按照首字母排序，使内容提供者调用顺序为gps network passive
            Collections.sort(providers)
            Log.i("Amap", "providers" + Gson().toJson(providers))
            for (provider in providers) {
                val location = it.getLastKnownLocation(provider)
                if (location == null) {
                    Log.i("Amap", "定位服务不可用")
                    //执行下一个
                    continue
                } else {
                    bestLocation = location
                    Log.i("Amap", provider + "定位服务成功")
                    //定位成功返回
                    break
                }
            }
        }
        return bestLocation
    }


    @SuppressLint("MissingPermission")
    fun startLocation() {
        val location: Location? = getLastKnownLocation(mLocationManager)
        if (location != null) {
            LocationParams.Longitude = location.longitude.toString()
            LocationParams.Latitude = location.latitude.toString()
            Log.i(
                "Amap",
                "Location:" + "Longitude:" + location.longitude + "Latitude:" + location.latitude
            )
            getGeocoder(location)
        } else {
            Log.i("Amap", "Location为空")
        }
    }


    /**
     * 关闭定位
     */
    @SuppressLint("MissingPermission")
    fun closeLocation() {
        mLocationManager?.removeUpdates(
            mLocationListener
        ) // 停止所有的定位服务
    }

    private fun getGeocoder(location: Location) {
        Log.i("Amap", "系统定位")
        LocationUtil.getGeocoder(context, location, callback)
    }
}