package com.urovo.wugumofang.net;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;

import java.lang.ref.WeakReference;

/**
 * @author chenji
 */
public class NetHelper {
    private static NetHelper mInstance;
    private final WeakReference<Context> mContext;
    /**
     * 没有连接网络
     */
    private static final int NETWORK_NONE = -1;
    /**
     * 移动网络
     */
    private static final int NETWORK_MOBILE = 0;
    /**
     * 无线网络
     */
    private static final int NETWORK_WIFI = 1;

    private NetHelper(Context context) {
        this.mContext = new WeakReference<>(context);
    }

    public static NetHelper getInstance(Context context) {
        if (mInstance == null) {
            synchronized (NetHelper.class) {
                if (mInstance == null) {
                    mInstance = new NetHelper(context);
                }
            }
        }
        return mInstance;
    }

    public static int getNetWorkState(Context context) {
        // 得到连接管理器对象
        ConnectivityManager connectivityManager = (ConnectivityManager) context
                .getSystemService(Context.CONNECTIVITY_SERVICE);

        NetworkInfo activeNetworkInfo = connectivityManager != null ? connectivityManager
                .getActiveNetworkInfo() : null;
        if (activeNetworkInfo != null && activeNetworkInfo.isConnected()) {

            if (activeNetworkInfo.getType() == (ConnectivityManager.TYPE_WIFI)) {
                return NETWORK_WIFI;
            } else if (activeNetworkInfo.getType() == (ConnectivityManager.TYPE_MOBILE)) {
                return NETWORK_MOBILE;
            }
        } else {
            return NETWORK_NONE;
        }
        return NETWORK_NONE;
    }

    /**
     * 初始化时判断有没有网络跟 IsHaveInternet() 的方法差不多效果，要实时监听，去回调NetEvent的onNetChange()接口
     */
    public boolean inspectNet() {
        return isNetConnect(getNetWorkState(mContext.get()));
    }

    /**
     * 判断有无网络 。
     *
     * @return true 有网, false 没有网络.
     */
    public boolean isNetConnect(int mNetMobile) {
        if (mNetMobile == 1) {
            return true;
        } else if (mNetMobile == 0) {
            return true;
        } else if (mNetMobile == -1) {
            return false;
        }
        return false;
    }

    /**
     * 是否联网网络 使用前提是需要加上权限: <uses-permission
     * android:name="android.permission.ACCESS_NETWORK_STATE"/>
     */

    public static boolean IsHaveInternet(final Context context) {
        try {
            ConnectivityManager manger = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
            NetworkInfo info = null;
            if (manger != null) {
                info = manger.getActiveNetworkInfo();
            }
            return info != null && info.isConnected() && info.getState() == NetworkInfo.State.CONNECTED;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }
}