package com.urovo.wugumofang.net;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import android.widget.ImageView;

import com.franmontiel.persistentcookiejar.PersistentCookieJar;
import com.franmontiel.persistentcookiejar.cache.SetCookieCache;
import com.franmontiel.persistentcookiejar.persistence.SharedPrefsCookiePersistor;
import com.google.gson.Gson;
import com.google.gson.internal.$Gson$Types;
import com.urovo.wugumofang.system.BaseAppManager;

import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.net.FileNameMap;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import okhttp3.Authenticator;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.Credentials;
import okhttp3.FormBody;
import okhttp3.Headers;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import okhttp3.Route;

/**
 * @author Sasuke
 * @date 2036/7/28
 */

public class OkHttpDominator {
    private static volatile OkHttpDominator mInstance;
    private OkHttpClient mOkHttpClient;
    private Handler mDelivery;
    private Gson mGson;

    public static final String TAG = "OkHttpClientManager";
    public final static int CONNECT_TIMEOUT = 30;
    public final static int READ_TIMEOUT = 30;
    public final static int WRITE_TIMEOUT = 30;
    public static final String POST = "POST";
    public static final String GET = "GET";

    public static final MediaType JSON = MediaType.parse("application/json; charset=utf-8");

    private static HashMap<String, String> myHeaders = new HashMap<>();//自定义请求头
    private static List<OkHttpDominator> timeoutInsList = new ArrayList<>();//用于存储设置了不同超时时间的OkHttpDominator。
//    private static final HashMap<String, List<Cookie>> cookieStore = new HashMap<>();


    protected OkHttpDominator() {
        mOkHttpClient = getClientBuilder().build();
//                .cookieJar(new CookieJar() {
//                               @Override
//                               public void saveFromResponse(HttpUrl httpUrl, List<Cookie> list) {
//                                   cookieStore.put(httpUrl.host(), list);
//                               }
//
//                               @Override
//                               public List<Cookie> loadForRequest(HttpUrl httpUrl) {
//                                   List<Cookie> cookies = cookieStore.get(httpUrl.host());
//                                   return cookies != null ? cookies : new ArrayList<Cookie>();
//                               }
//                           }
//                        )

        //cookie enabled
//        mOkHttpClient.setCookieHandler(new CookieManager(null, CookiePolicy.ACCEPT_ORIGINAL_SERVER));
//        mOkHttpClient.setReadTimeout(READ_TIMEOUT, TimeUnit.SECONDS);//设置读取超时时间
//        mOkHttpClient.setWriteTimeout(WRITE_TIMEOUT, TimeUnit.SECONDS);//设置写的超时时间
//        mOkHttpClient.setConnectTimeout(CONNECT_TIMEOUT, TimeUnit.SECONDS);//设置连接超时时间

        mDelivery = new Handler(Looper.getMainLooper());
        mGson = new Gson();
    }

    private static OkHttpDominator getInstance() {
        if (mInstance == null) {
            synchronized (OkHttpDominator.class) {
                if (mInstance == null) {
                    mInstance = new OkHttpDominator();
                }
            }
        }
        return mInstance;
    }


    /**
     * 添加请求头
     */
    public static void setHeaders(HashMap<String, String> myHeaders) {
        OkHttpDominator.myHeaders = myHeaders;
    }


    /**
     * 天生自带Header初始化
     */
    private class BuilderWithHeader extends Request.Builder {
        public BuilderWithHeader() {
            super();
            if (!myHeaders.isEmpty()) {
                for (String key : myHeaders.keySet()) {
                    this.addHeader(key, myHeaders.get(key));
                }
            }
        }
    }


    private static OkHttpClient.Builder getClientBuilder() {
        return new OkHttpClient().newBuilder()
                .readTimeout(READ_TIMEOUT, TimeUnit.SECONDS)
                .writeTimeout(WRITE_TIMEOUT, TimeUnit.SECONDS)
                .connectTimeout(CONNECT_TIMEOUT, TimeUnit.SECONDS)
                .cookieJar(new PersistentCookieJar(new SetCookieCache(), new SharedPrefsCookiePersistor(BaseAppManager.appContext())));
    }

    /**
     * Sasuke异步的post请求
     *
     * @param url
     * @param params
     * @param callback
     */
    private void _postAsyn(String url, Map<String, String> params, final ResultCallback callback) {
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, POST, null, paramsArr);
        String paramStr = params == null ? "" : params.toString();
        Log.i("Log", "请求报文Dominator[异步POST]：" + request.url().toString() + "\n" + paramStr);
        deliveryResult(callback, request);
    }

    /**
     * Sasuke异步post+MediaType请求（自定义MediaType，如“application/json”之类）
     *
     * @param url
     * @param params
     * @param callback
     */
    private void _postAsynWithType(String url, MediaType type, Map<String, String> params, final ResultCallback callback) {
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, POST, type, paramsArr);
        Log.i("Log", "请求报文Dominator[异步POST " + (type == null ? "" : type.toString()) + "]：" + request.url().toString() + "\n" + paramsArr[0].value);
        deliveryResult(callback, request);
    }

    /**
     * Sasuke异步的get请求
     *
     * @param url
     * @param callback
     */
    private void _getAsyn(String url, Map<String, String> params, final ResultCallback callback) {
//        final Request request = new BuilderWithHeader()
//                .url(url)
//                .build();
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, GET, null, paramsArr);
        String paramStr = params == null ? "" : params.toString();
        Log.i("Log", "请求报文Dominator[异步GET]：" + request.url().toString() + "\n" + paramStr);
        deliveryResult(callback, request);
    }



    private void _getAsynAuth(String url, String authname, String pwd, Map<String, String> params, final ResultCallback callback) {
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, GET, JSON, paramsArr);
        String paramStr = params == null ? "" : params.toString();
        Log.i("Log", "请求报文Dominator[异步GETAuth]：" + request.url().toString() + "\n" + paramStr);
        deliveryResult(callback, request, authname, pwd);
    }

    private void _postAsynAuth(String url, String authname, String pwd, Map<String, String> params, final ResultCallback callback) {
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, POST, JSON, paramsArr);
        String paramStr = params == null ? "" : params.toString();
        Log.i("Log", "请求报文Dominator[异步POSTAuth]：" + request.url().toString() + "\n" + paramStr);
        deliveryResult(callback, request, authname, pwd);
    }

    /**
     * Sasuke异步的get请求（给你直接获取json字符串的专门写一个好不好，信了你的邪。。。）
     *
     * @param url
     * @param callback
     */
    private void _getAsynForString(String url, Map<String, String> params, final ResultCallback callback) {
//        final Request request = new BuilderWithHeader()
//                .url(url)
//                .build();
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, GET, null, paramsArr);
        String paramStr = params == null ? "" : params.toString();
        Log.i("Log", "请求报文Dominator[异步GET]：" + request.url().toString() + "\n" + paramStr);
        deliveryResult(callback, request);
    }


    /**
     * Sasuke同步的Get请求
     *
     * @param url
     * @return 字符串
     */
    private <T> Object _getSyn(String url, Class<T> clazz, Map<String, String> params) {
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, GET, null, paramsArr);
        Object o = null;
        try {
            String paramStr = params == null ? "" : params.toString();
            Log.i("Log", "请求报文Dominator[同步GET]：" + request.url().toString() + "\n" + paramStr);
            Response response = mOkHttpClient.newCall(request).execute();
            String string = response.body().string();
            Log.i("Log", "响应报文Dominator：" + string);
            o = mGson.fromJson(string, clazz);
        } catch (com.google.gson.JsonParseException e) {
            e.printStackTrace();
//            showErrorMsg("JSON本地解析错误for"+ clazz.getSimpleName());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return o;
    }


    /**
     * Sasuke同步的Post请求
     *
     * @param url
     * @param params post的参数
     * @return 字符串
     */
    private <T> Object _postSyn(String url, Class<T> clazz, Map<String, String> params) {
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, POST, null, paramsArr);
        Object o = null;
        try {
            String paramStr = params == null ? "" : params.toString();
            Log.i("Log", "请求报文Dominator[同步POST]：" + request.url().toString() + "\n" + paramStr);
            Response response = mOkHttpClient.newCall(request).execute();
            String string = response.body().string();
            Log.i("Log", "响应报文Dominator：" + string);
            o = mGson.fromJson(string, clazz);
        } catch (com.google.gson.JsonParseException e) {
            e.printStackTrace();
//            showErrorMsg("JSON本地解析错误for"+ clazz.getSimpleName());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return o;
    }


    /**
     * Sasuke同步的Post请求（带type的）
     *
     * @param url
     * @param type   MediaType.JSON
     * @param clazz
     * @param params
     * @return
     */
    private <T> Object _postSynWithType(String url, MediaType type, Class<T> clazz, Map<String, String> params) {
        Param[] paramsArr = map2Params(params);
        Request request = buildRequestBody(url, POST, type, paramsArr);
        Object o = null;
        try {
            String paramStr = params == null ? "" : params.toString();
            Log.i("Log", "请求报文Dominator[异步POST " + (type == null ? "" : type.toString()) + "]：" + request.url().toString() + "\n" + paramStr);
            Response response = mOkHttpClient.newCall(request).execute();
            String string = response.body().string();
            Log.i("Log", "响应报文Dominator：" + string);
            o = mGson.fromJson(string, clazz);
        } catch (com.google.gson.JsonParseException e) {
            e.printStackTrace();
//            showErrorMsg("JSON本地解析错误for"+ clazz.getSimpleName());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return o;
    }

//    /**
//     * 同步的Post请求
//     *
//     * @param url
//     * @param params post的参数
//     * @return
//     */
//    private Response _post(String url, Map<String, String> params) throws IOException {
//        Param[] paramsArr = map2Params(params);
//        Request request = buildRequestBody(url, POST, null, paramsArr);
//        Response response = mOkHttpClient.newCall(request).execute();
//        return response;
//    }


//    /**
//     * 异步的post请求
//     *
//     * @param url
//     * @param callback
//     * @param params
//     */
//    private void _postAsyn(String url, final ResultCallback callback, Map<String, String> params) {
//        Param[] paramsArr = map2Params(params);
//        Request request = buildRequestBody(url, POST, null, paramsArr);
//        deliveryResult(callback, request);
//    }

    /**
     * 同步基于post的文件上传
     *
     * @param params
     * @return
     */
    private Response _post(String url, File[] files, String[] fileKeys, Map<String, String> params) throws IOException {
        Param[] paramsArr = map2Params(params);
        Request request = buildMultipartFormRequest(url, files, fileKeys, paramsArr);
        return mOkHttpClient.newCall(request).execute();
    }

    private Response _post(String url, File file, String fileKey) throws IOException {
        Request request = buildMultipartFormRequest(url, new File[]{file}, new String[]{fileKey}, null);
        return mOkHttpClient.newCall(request).execute();
    }

    private Response _post(String url, File file, String fileKey, Map<String, String> params) throws IOException {
        Param[] paramsArr = map2Params(params);
        Request request = buildMultipartFormRequest(url, new File[]{file}, new String[]{fileKey}, paramsArr);
        return mOkHttpClient.newCall(request).execute();
    }

    /**
     * 异步基于post的文件上传
     *
     * @param url
     * @param callback
     * @param files
     * @param fileKeys
     * @throws IOException
     */
    private void _postAsyn(String url, ResultCallback callback, File[] files, String[] fileKeys, Map<String, String> params) throws IOException {
        Param[] paramsArr = map2Params(params);
        Request request = buildMultipartFormRequest(url, files, fileKeys, paramsArr);
        String paramStr = params == null ? "" : params.toString();
        Log.i("Log", "请求报文Dominator[异步POST文件UP]：" + request.url().toString() + "\n" + paramStr);
        deliveryResult(callback, request);
    }

    /**
     * 异步基于post的文件上传，单文件不带参数上传
     *
     * @param url
     * @param callback
     * @param file
     * @param fileKey
     * @throws IOException
     */
    private void _postAsyn(String url, ResultCallback callback, File file, String fileKey) throws IOException {
        Request request = buildMultipartFormRequest(url, new File[]{file}, new String[]{fileKey}, null);
        Log.i("Log", "请求报文Dominator[异步POST文件UP]：" + request.url().toString() + "\n" + file.toString());
        deliveryResult(callback, request);
    }

    /**
     * 异步基于post的文件上传，单文件且携带其他form参数上传
     *
     * @param url
     * @param callback
     * @param file
     * @param fileKey
     * @param params
     * @throws IOException
     */
    private void _postAsyn(String url, ResultCallback callback, File file, String fileKey, Map<String, String> params) throws IOException {
        Param[] paramsArr = map2Params(params);
        Request request = buildMultipartFormRequest(url, new File[]{file}, new String[]{fileKey}, paramsArr);
        String paramStr = params == null ? "" : params.toString();
        Log.i("Log", "请求报文Dominator[异步POST文件UP]：" + request.url().toString() + "\n" + paramStr);
        deliveryResult(callback, request);
    }

    /**
     * 异步下载文件
     *
     * @param url
     * @param destFileDir 本地文件存储的文件夹
     * @param callback
     */
    private void _downloadAsyn(final String url, final String destFileDir, final ResultCallback callback) {
        final Request request = new BuilderWithHeader()
                .url(url)
                .build();
        final Call call = mOkHttpClient.newCall(request);
        call.enqueue(new Callback() {

            @Override
            public void onFailure(Call call, IOException e) {
                sendFailedStringCallback(call.request(), e, callback);
            }

            @Override
            public void onResponse(Call call, Response response) {
                InputStream is = null;
                byte[] buf = new byte[2048];
                int len = 0;
                FileOutputStream fos = null;
                try {
                    is = response.body().byteStream();
//                    File file = new File(destFileDir, getFileName(url));
                    File file = new File(destFileDir);
                    fos = new FileOutputStream(file);
                    while ((len = is.read(buf)) != -1) {
                        fos.write(buf, 0, len);
                    }
                    fos.flush();
                    //如果下载文件成功，第一个参数为文件的绝对路径
                    sendSuccessResultCallback(file.getAbsolutePath(), callback);
                } catch (IOException e) {
                    sendFailedStringCallback(response.request(), e, callback);
                } finally {
                    try {
                        if (is != null) is.close();
                    } catch (IOException e) {
                    }
                    try {
                        if (fos != null) fos.close();
                    } catch (IOException e) {
                    }
                }

            }
        });
    }

    /**
     * 异步下载图片为Bitmap
     */
    public void _downloadImg(String imgUrl, final ResultCallback<Bitmap> callback) {
        final Request request = new BuilderWithHeader()
                .get()
                .url(imgUrl)
                .build();
        final Call call = mOkHttpClient.newCall(request);
        Log.i("Log", "请求报文Dominator[异步GET图片toBmp]：" + imgUrl);
        call.enqueue(new Callback() {


            @Override
            public void onFailure(Call call , IOException e) {
                sendFailedStringCallback(call.request(), e, callback);
            }

            @Override
            public void onResponse(Call call ,Response response)  {
                //拿到字节流
                InputStream is = null;
                try {
                    is = response.body().byteStream();
                    Bitmap bmp = BitmapFactory.decodeStream(is);
                    sendSuccessResultCallback(bmp, callback);
                    if (bmp != null) {
                        Log.i("Log", "响应报文Dominator[异步GET图片结果]：图片下载成功，大小为：" + bmp.getByteCount() / 1000 + "KB");
                    }
                } /*catch (IOException e) {
                    Log.i("Log", "响应报文Dominator[异步GET图片结果]：图片下载失败，异常IOException");
                    e.printStackTrace();
                } */finally {
                    if (is != null) {
                        try {
                            is.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }

            }
        });
    }

    private String getFileName(String path) {
        int separatorIndex = path.lastIndexOf("/");
        return (separatorIndex < 0) ? path : path.substring(separatorIndex + 1, path.length());
    }

//    /**
//     * 加载图片
//     *
//     * @param view
//     * @param url
//     * @throws IOException
//     */
//    private void _displayImage(final ImageView view, final String url, final int errorResId)
//    {
//        final Request request = new BuilderWithHeader()
//                .url(url)
//                .build();
//        Call call = mOkHttpClient.newCall(request);
//        call.enqueue(new Callback()
//        {
//            @Override
//            public void onFailure(Request request, IOException e)
//            {
//                setErrorResId(view, errorResId);
//            }
//
//            @Override
//            public void onResponse(Response response)
//            {
//                InputStream is = null;
//                try
//                {
//                    is = response.body().byteStream();
//                    ImageUtils.ImageSize actualImageSize = ImageUtils.getImageSize(is);
//                    ImageUtils.ImageSize imageViewSize = ImageUtils.getImageViewSize(view);
//                    int inSampleSize = ImageUtils.calculateInSampleSize(actualImageSize, imageViewSize);
//                    try
//                    {
//                        is.reset();
//                    } catch (IOException e)
//                    {
//                        response = _getAsyn(url);
//                        is = response.body().byteStream();
//                    }
//
//                    BitmapFactory.Options ops = new BitmapFactory.Options();
//                    ops.inJustDecodeBounds = false;
//                    ops.inSampleSize = inSampleSize;
//                    final Bitmap bm = BitmapFactory.decodeStream(is, null, ops);
//                    mDelivery.post(new Runnable()
//                    {
//                        @Override
//                        public void run()
//                        {
//                            view.setImageBitmap(bm);
//                        }
//                    });
//                } catch (Exception e)
//                {
//                    setErrorResId(view, errorResId);
//
//                } finally
//                {
//                    if (is != null) try
//                    {
//                        is.close();
//                    } catch (IOException e)
//                    {
//                        e.printStackTrace();
//                    }
//                }
//            }
//        });
//
//
//    }

    private void setErrorResId(final ImageView view, final int errorResId) {
        mDelivery.post(new Runnable() {
            @Override
            public void run() {
                view.setImageResource(errorResId);
            }
        });
    }


    //*************对外公布的方法************
    public static void downImageAsyn(String imgUrl, final ResultCallback callback) {
        getInstance()._downloadImg(imgUrl, callback);
    }

    public static void getAsyn(String url, Map<String, String> params, final ResultCallback callback) {
        getInstance()._getAsyn(url, params, callback);
    }

    public static void postAsyn(String url, Map<String, String> params, final ResultCallback callback) {
        getInstance()._postAsyn(url, params, callback);
    }

    public static void postAsynAuth(String url, String authname ,String pwd, Map<String, String> params, final ResultCallback callback) {
        getInstance()._postAsynAuth(url, authname, pwd,params, callback);
    }

    public static void getAsynAuth(String url, String authname ,String pwd, Map<String, String> params, final ResultCallback callback) {
        getInstance()._getAsynAuth(url, authname,pwd ,params, callback);
    }

    /**
     * 带自定义超时时间的POST请求
     */
    public static void postAsynTimeout(String url, Map<String, String> params, final ResultCallback callback, int timeout) {
        getInstanceTimeout(timeout)._postAsyn(url, params, callback);
    }

    /**
     * 带自定义超时时间的POST请求【同步】
     */
    public static <T> Object postSynTimeout(String url, Map<String, String> param, Class<T> clazz, int timeout) {
        return getInstanceTimeout(timeout)._postSyn(url, clazz, param);
    }

    /**
     * 创建一个新的自定义超时时间的实例
     * 注：因为原Okhttp是单例的，单个OkhttpClient只能持有一个固定的延时timeout，
     * 所以针对不同的接口定义不同的超时时间的需求下，
     * 就需要另外创建新的带修改timeout后的instance实例，
     * 然而以前默认的instance还是用着默认的timeout，所以默认延时的接口不受影响，继续使用老的就行。
     */
    private static OkHttpDominator getInstanceTimeout(int timeout) {
        //先遍历一遍，看有没有已经设过超时的。
        for (OkHttpDominator instance : timeoutInsList) {
            if (instance.mOkHttpClient.connectTimeoutMillis() == timeout) {
                return instance;
            }
        }
        //如果没找到，就新建一个。然后添加到列表里供以后重复利用。
        synchronized (OkHttpDominator.class) {
            for (OkHttpDominator instance : timeoutInsList) {
                if (instance.mOkHttpClient.connectTimeoutMillis() == timeout) {
                    return instance;
                }
            }
            OkHttpDominator  instance = new OkHttpDominator();
            instance.mOkHttpClient = getClientBuilder().build();//new OkHttpClient.Builder().connectTimeout(timeout, TimeUnit.SECONDS).build();
            timeoutInsList.add(instance);
            //用新建的来发送请求。
            return instance;
        }

    }

    /**
     * 提供可自定义传入MediaType的post方式【异步】
     */
    public static void postAsynByType(String url, MediaType type, Map<String, String> params, final ResultCallback callback) {
        getInstance()._postAsynWithType(url, type, params, callback);
    }

    /**
     * 提供可自定义传入MediaType的post方式【异步】
     */
    public static void postAsynByTypeTimeout(String url, MediaType type, Map<String, String> params, final ResultCallback callback, int timeout) {
        getInstanceTimeout(timeout)._postAsynWithType(url, type, params, callback);
    }

    /**
     * 提供可自定义传入MediaType的post方式【同步】
     */
    public static Object postSynByType(String url, MediaType type, Class clazz, Map<String, String> params) {
        return getInstance()._postSynWithType(url, type, clazz, params);
    }

    /**
     * 同步GET
     */
    public static Object getSyn(String url, Class clazz, Map<String, String> params) {
        return getInstance()._getSyn(url, clazz, params);

    }

    /**
     * 同步POST
     */
    public static Object postSyn(String url, Class clazz, Map<String, String> params) {
        return getInstance()._postSyn(url, clazz, params);
    }
//    public static Response post(String url, Map<String, String> params) throws IOException {
//        return getInstance()._post(url, params);
//    }


    public static Response postFile(String url, File[] files, String[] fileKeys, Map<String, String> params) throws IOException {
        return getInstance()._post(url, files, fileKeys, params);
    }

    public static Response postFile(String url, File file, String fileKey) throws IOException {
        return getInstance()._post(url, file, fileKey);
    }

    public static Response postFile(String url, File file, String fileKey, Map<String, String> params) throws IOException {
        return getInstance()._post(url, file, fileKey, params);
    }

    public static void postFilesAsyn(String url, ResultCallback callback, File[] files, String[] fileKeys, Map<String, String> params) throws IOException {
        getInstance()._postAsyn(url, callback, files, fileKeys, params);
    }


    public static void postFileAsyn(String url, ResultCallback callback, File file, String fileKey) throws IOException {
        getInstance()._postAsyn(url, callback, file, fileKey);
    }


    public static void postFileAsyn(String url, ResultCallback callback, File file, String fileKey, Map<String, String> params) throws IOException {
        getInstance()._postAsyn(url, callback, file, fileKey, params);
    }

//    public static void displayImage(final ImageView view, String url, int errorResId) throws IOException
//    {
//        getInstance()._displayImage(view, url, errorResId);
//    }
//
//
//    public static void displayImage(final ImageView view, String url)
//    {
//        getInstance()._displayImage(view, url, -1);
//    }

    public static void downloadAsyn(String url, String destDir, ResultCallback callback) {
        getInstance()._downloadAsyn(url, destDir, callback);
    }

    //****************************


    private Request buildMultipartFormRequest(String url, File[] files,
                                              String[] fileKeys, Param[] params) {
        params = validateParam(params);

        MultipartBody.Builder builder = new MultipartBody.Builder().setType(MultipartBody.FORM);

        for (Param param : params) {
            builder.addPart(Headers.of("Content-Disposition", "form-data; name=\"" + param.key + "\""),
                    RequestBody.create(null, param.value));
        }
        if (files != null) {
            RequestBody fileBody = null;
            for (int i = 0; i < files.length; i++) {
                File file = files[i];
                String fileName = file.getName();
                fileBody = RequestBody.create(MediaType.parse(guessMimeType(fileName)), file);
                // 根据文件名设置contentType
                builder.addPart(Headers.of("Content-Disposition",
                        "form-data; name=\"" + fileKeys[i] + "\"; filename=\"" + fileName + "\""),
                        fileBody);
            }
        }

        RequestBody requestBody = builder.build();
        return new BuilderWithHeader()
                .url(url)
                .post(requestBody)
                .build();
    }

    private String guessMimeType(String path) {
        FileNameMap fileNameMap = URLConnection.getFileNameMap();
        String contentTypeFor = fileNameMap.getContentTypeFor(path);
        if (contentTypeFor == null) {
            contentTypeFor = "application/octet-stream";
        }
        return contentTypeFor;
    }


    private Param[] validateParam(Param[] params) {
        if (params == null) {
            return new Param[0];
        } else {
            return params;
        }
    }

    private Param[] map2Params(Map<String, String> params) {
        if (params == null) {
            return new Param[]{new Param("", "")};
        }
        int size = params.size();
        Param[] res = new Param[size];
        Set<Map.Entry<String, String>> entries = params.entrySet();
        int i = 0;
        for (Map.Entry<String, String> entry : entries) {
            res[i++] = new Param(entry.getKey(), entry.getValue());
        }
        return res;
    }

    private static final String SESSION_KEY = "Set-Cookie";
    private static final String mSessionKey = "JSESSIONID";

    private Map<String, String> mSessions = new HashMap<String, String>();

    private void deliveryResult(final ResultCallback callback, Request request,String name,String pwd) {
        OkHttpClient client = TextUtils.isEmpty(name) || TextUtils.isEmpty(pwd) ? mOkHttpClient : getClientBuilder().authenticator(
                new Authenticator() {
                    @Override
                    public Request authenticate(@NotNull Route route, @NotNull Response response) throws IOException {
                        String credential = Credentials.basic(name, pwd, Charset.forName("utf-8"));
                        return response.request().newBuilder().header("Authorization", credential).build();
                    }
                }
        ).build();

        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(@NotNull Call call, @NotNull IOException e) {
                sendFailedStringCallback(call.request(), e, callback);
            }

            @Override
            public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
                try {
                    if(response.body()!=null) {
                        final String body = response.body().string();
                        Log.i("Log", "响应报文Dominator：" + body);
                        //虽然是异步回调为空，用于不需要确认返回的情况，直接发出去就发出去了，成不成功不管。
                        if (callback == null) {
                            return;
                        }
                        if (callback.mType == String.class) {
                            sendSuccessResultCallback(body, callback);
                        } else {
                            Object o = mGson.fromJson(body, callback.mType);
                            sendSuccessResultCallback(o, callback);
                        }
                    }
                } catch (Exception e) {
                    sendFailedStringCallback(response.request(), e, callback);
                }

            }
        });
    }

    private void deliveryResult(final ResultCallback callback, Request request) {
        deliveryResult(callback,request,"","");
    }

    private void sendFailedStringCallback(final Request request, final Exception e, final ResultCallback callback) {
        mDelivery.post(new Runnable() {
            @Override
            public void run() {
                if (callback != null) {
                    e.printStackTrace();
                    callback.onError(request,ApiException.handleException(e));
                }
            }
        });
    }

    private void sendSuccessResultCallback(final Object object, final ResultCallback callback) {
        mDelivery.post(new Runnable() {
            @Override
            public void run() {
                callback.onResponse(object);
            }
        });
    }

    /**
     * 可自定义mediaType类型，如传json时用
     */
    private Request buildRequestBody(String url, String requestType, MediaType mediaType, Param[] params) {
        if (params == null) {
            params = new Param[0];
        }
        //如果是POST请求===================================================
        if (requestType.equals(POST)) {
            RequestBody requestBody;
            //如果携带了Media类型。
            if (mediaType != null) {
                requestBody = RequestBody.create(mediaType, params[0].value);
            } else {//如果并没有携带类型。
                FormBody.Builder builder = new FormBody.Builder();
                for (Param param : params) {
                    builder.add(param.key, param.value);
                }
                requestBody = builder.build();
            }
            return new BuilderWithHeader()
                    .url(url)
                    .post(requestBody)
                    .build();
            //如果是GET请求===================================================
        } else if (requestType.equals(GET)) {
            StringBuffer sb = new StringBuffer(url);
            int length = params.length;
            for (int i = 0; i < length; i++) {
                sb.append(i == 0 ? "?" : "&");//第一个参数不用&号而是？号
                sb.append(params[i].key + "=" + params[i].value);
            }
            url = sb.toString();
            return new BuilderWithHeader()
                    .url(url)
                    .build();
        }
        Log.i("Log", "请求报文生成错误");
        return new BuilderWithHeader().build();//返回个空的请求
    }


    public static abstract class ResultCallback<T> {
        Type mType;

        public ResultCallback() {
            mType = getSuperclassTypeParameter(getClass());
        }

        static Type getSuperclassTypeParameter(Class<?> subclass) {
            Type superclass = subclass.getGenericSuperclass();
            if (superclass instanceof Class) {
                throw new RuntimeException("Missing type parameter.");
            }
            ParameterizedType parameterized = (ParameterizedType) superclass;
            return $Gson$Types.canonicalize(parameterized.getActualTypeArguments()[0]);
        }

        public abstract void onError(Request request, Exception e);

        public abstract void onResponse(T response);
    }

    public static class Param {
        public Param() {
        }

        public Param(String key, String value) {
            this.key = key;
            this.value = value;
        }

        String key;
        String value;
    }

    /**回到主线程发弹框提示*/
//    private void showErrorMsg(final String msg){
//        mDelivery.post(new Runnable() {
//            @Override
//            public void run() {
//                MyToast.show(msg);
//            }
//        });
//    }
}
