package com.urovo.wugumofang.system;

import android.app.Activity;
import android.app.Application;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;

import com.alibaba.android.arouter.launcher.ARouter;
import com.sibyl.screenshotlistener.FeedbackCardMaker;
import com.sibyl.screenshotlistener.ScreenShotListenManager;
import com.tencent.wework.api.IWWAPI;
import com.tencent.wework.api.WWAPIFactory;
import com.urovo.wgmf.db.dao.DBTool;
import com.urovo.wugumofang.constants.Files;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.data.ASimpleCache;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.manager.ThreadManager;
import com.urovo.wugumofang.utils.DataCleanManager;
import com.xsj.crasheye.Crasheye;

import org.apache.log4j.Logger;

import java.io.File;
import java.lang.ref.WeakReference;

/**
 * @author Sasuke
 * @date 2017/8/24
 * 用于在两种开发模式下，分别存储对应不同的applicationContext并做公用的初始化（database & logger）
 */

public class BaseAppManager {
    public static boolean isOpen2SophixFlag = true;//默认开启，放在下面去用 DebuggableCheck 初始化！！用BuildConfig.DEBUG判断有问题//= !BuildConfig.DEBUG;// 发布默认状态下是打开的，调试的时候调成false,防止打线上的补丁出错
    private volatile static BaseAppManager instance;
    public static String PACKAGE_NAME;
    public static int SCREEN_WIDTH;
    public static int SCREEN_HEIGHT;
    public static float SCREEN_DENSITY;

    public static BaseAppManager getInstance() {
        if (instance == null) {
            synchronized (BaseAppManager.class) {
                if (instance == null) {
                    instance = new BaseAppManager();
                }
            }
        }
        return instance;
    }

    private Logger loger;
    private SQLiteDatabase database;
    private ASimpleCache preferCache;//基于ASimpleCache的缓存
    private Application mContext;
    public final static int DEFAULT_TIME_OUT = 1000 * 15;

    private ScreenShotListenManager shotListenManager;//截图监听
    public static IWWAPI iwwapi;//企业微信
    /**
     * ===========Application初始化 的都在这里做2（其实就是从WgmfAppLogic的初始化跳过来的）=================
     */
    public static BaseAppManager init(Application context) {
        DebuggableCheck.syncIsDebug(context);//初始化这个工具类，用来替代不够准确的BuildConfig.DEBUG
// =======================================崩溃监听配置=============================================
        //开发阶段测试崩溃方案平台（同样拥有优秀的崩溃分析能力，但不具备运营数据的统计，线上阶段用bugly）
        Crasheye.init(context, "808359e0");
//          设置获取log日志 获取行数和获取的关键字
//          @param lines    获取的行数(最大收集1000行)
//          @param filter    获取的关键字 。例如：filter 是"Crasheye"就相当于LogCat过滤栏中输入“logcat -d Crasheye”
        Crasheye.setLogging(1000, "Crasheye");
        Crasheye.setUserIdentifier("sygj");
//        String crasheyeUUID = Crasheye.getCrasheyeUUID();
//        设置该版本是否为测试版本,  是否为测试版本
        Crasheye.setIsBetaVersion(DebuggableCheck.isDebug());
// =======================================崩溃监听配置=============================================
        isOpen2SophixFlag = !DebuggableCheck.isDebug();
        if (getInstance().mContext == null) {
            getInstance().mContext = context;
        }
        if (DebuggableCheck.isDebug()) {           // 这两行必须写在init之前，否则这些配置在init过程中将无效
            ARouter.openLog();     // 打印日志
            ARouter.openDebug();   // 开启调试模式(如果在InstantRun模式下运行，必须开启调试模式！线上版本需要关闭,否则有安全风险)
        }
        ARouter.init(context); // 尽可能早，推荐在Application中初始化
        PACKAGE_NAME = BasePackageUtil.getAppPackageName(context);
        SCREEN_WIDTH = context.getResources().getDisplayMetrics().widthPixels;
        SCREEN_HEIGHT = context.getResources().getDisplayMetrics().heightPixels;
        SCREEN_DENSITY = context.getResources().getDisplayMetrics().density;
        PreferHelper.init(context);//组件化，所以必须先在这里获取一次Application
//        SYLoger.openDebutLog(DebuggableCheck.isDebug());
        getInstance().initLogger();
        getInstance().initDatabase();
        getInstance().initPreferCache();
        //监听当前Activity是谁
        getInstance().registerActivityLifecycle();
        getInstance().initScreenShotListener();

        //分享企业微信 注册
        iwwapi = WWAPIFactory.createWWAPI(appContext());
        iwwapi.registerApp(NetParam.EW_SCHEMA);

        //给SystemHelper做假数据
//        SystemHelperMock.mock();
        return instance;
    }

    /**分享到企业微信 要用*/
    public static String getAppName() {
        ApplicationInfo appInfo = appContext().getApplicationInfo();
        int stringId = appInfo.labelRes;
        return stringId == 0 ? appInfo.nonLocalizedLabel.toString() : appContext().getString(stringId);
    }

    public static Application appContext() {
        if (getInstance().mContext == null) {
            throw (new NullPointerException("BaseAppManager必须在application中进行init初始化"));
        } else {
            return getInstance().mContext;
        }
    }

    public ScreenShotListenManager getShotListener() {
        if (shotListenManager == null) {
            shotListenManager = ScreenShotListenManager.newInstance(appContext());
        }
        return shotListenManager;
    }

    /**
     * 初始化Logger
     */
    public void initLogger() {
        loger = ConfigureLog4J.configure(Files.logfile, "WGMF");
    }

    /**
     * 初始化数据库
     */
    public void initDatabase() {
        database = DBTool.getInstance(getInstance().mContext).getWritableDatabase();
    }

    /**
     * 初始化那什么Cache
     */
    public void initPreferCache() {
        preferCache = ASimpleCache.get(getInstance().mContext);
    }

    /**
     * 初始化截图监听
     */
    public void initScreenShotListener() {
        File cardFile = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + File.separator + "0カード"+File.separator + "JsonDominator");
        if (cardFile.exists()) return; //如果是测试机，那就不需要截屏监听
        getShotListener().setListener(new ScreenShotListenManager.OnScreenShotListener() {
            @Override
            public void onShot(final String imagePath) {
                if (getCurrentActivity() != null) {
                    ThreadManager.getThreadPool().execute(new Runnable() {
                        @Override
                        public void run() {
                            try {
                                Thread.sleep(1500);//有些垃圾系统截图时写入磁盘比较慢，所以这边要等一下。
                                FeedbackCardMaker cardMaker = new FeedbackCardMaker(appContext());
                                PackageInfo pkgInfo = appContext().getPackageManager().getPackageInfo(appContext().getPackageName(), 0);
                                int versionCodeLength = String.valueOf(pkgInfo.versionCode).length();//后面显示versionCode只显示末位4位算了。。。够用了
                                //绘制底部卡片
                                Bitmap bottomCard = cardMaker.drawInfo2BottomCard("版本：v" + pkgInfo.versionName + "（"+ String.valueOf(pkgInfo.versionCode).substring(versionCodeLength - 3,versionCodeLength) +"）",
                                        "门店：" + SystemHelper.LoginOrganId + " 工号：" + SystemHelper.LoginUserId,
                                        "设备：Android " + android.os.Build.VERSION.RELEASE + " / " + android.os.Build.MODEL);
                                //拼接到截图
                                cardMaker.mergeScrShot2BottomCard(imagePath, bottomCard);
                            } catch (PackageManager.NameNotFoundException e) {
                                e.printStackTrace();
                            }catch (InterruptedException e){
                                e.printStackTrace();
                            }catch(Exception e){
                                e.printStackTrace();
                            }
                        }
                    });

                }
            }
        });
        getShotListener().startListen();
    }

    /**
     * 公开的获取loger实例的方法
     *
     * @return
     */
    public static Logger loger() {
        return getInstance().getLoger();
    }

    /**
     * 公开的获取database的方法
     *
     * @return
     */
    public static SQLiteDatabase database() {
        return getInstance().getDatabase();
    }


    /**
     * 基于Prefer的缓存
     *
     * @return
     */
    public static ASimpleCache preferCache() {
        return getInstance().getPreferCache();
    }

    private Logger getLoger() {
        return loger;
    }

    private SQLiteDatabase getDatabase() {
        return database;
    }

    private ASimpleCache getPreferCache() {
        return preferCache;
    }

    /**
     * 清除所有缓存，重启
     */
    public void clearApplicationData(){
        DataCleanManager.clearPrivate(appContext());
        DataCleanManager.clearPublic(appContext());
        DataCleanManager.restartApp(appContext());
    }


    /**
     * 给外部调用来得到当前Activity
     */
    public Activity getCurrentActivity() {
        if (currentActRef == null) {
            return null;
        }
        return currentActRef.get();
    }

    /**
     * 当前APP是否位于前台
     */
    public boolean isAppFrontRunning() {
        return activityCounter>0;
    }

    private WeakReference<Activity> currentActRef;
    private int activityCounter = 0;

    private void registerActivityLifecycle() {
        appContext().registerActivityLifecycleCallbacks(new Application.ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle savedInstanceState) {

            }

            @Override
            public void onActivityStarted(Activity activity) {
                activityCounter++;
            }

            @Override
            public void onActivityResumed(Activity activity) {
                currentActRef = new WeakReference(activity);
            }

            @Override
            public void onActivityPaused(Activity activity) {

            }

            @Override
            public void onActivityStopped(Activity activity) {
                activityCounter--;
            }

            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

            }

            @Override
            public void onActivityDestroyed(Activity activity) {

            }
        });
    }
}
