package com.urovo.wugumofang.system;

import android.annotation.SuppressLint;
import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;

import com.urovo.wugumofang.constants.LocationParams;
import com.urovo.wugumofang.manager.ThreadManager;
import com.urovo.wugumofang.model.LocationEntity;
import com.wgmf.library.location.HwLocationManager;
import com.wgmf.library.location.SystemLocationManager;
import com.zsy.libcore.common.android.log.Log;
import java.util.List;
import java.util.Locale;

@SuppressWarnings("all")
public class LocationUtil {

    public LocationUtil(Context cxt, OnCallback callback) {
        HwLocationManager hwLocationManager = new HwLocationManager(cxt, callback);
        hwLocationManager.startLocation();

//        SystemLocationManager systemLocationManager = new SystemLocationManager(cxt, callback);
//        systemLocationManager.startLocation();
    }

    public interface OnCallback {
        void onLocationListener(LocationEntity entity);
    }

    private static double EARTH_RADIUS = 6378.137;

    private static double rad(double d) {
        return d * Math.PI / 180.0;
    }

    /**
     * 通过经纬度获取距离(单位：米)
     */
    public static double getDistance(double lat1, double lng1, double lat2, double lng2) {
        double radLat1 = rad(lat1);
        double radLat2 = rad(lat2);
        double a = radLat1 - radLat2;
        double b = rad(lng1) - rad(lng2);
        double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2)
                + Math.cos(radLat1) * Math.cos(radLat2)
                * Math.pow(Math.sin(b / 2), 2)));
        s = s * EARTH_RADIUS;
        s = Math.round(s * 10000d) / 10000d;
        s = s * 1000;
        return s;
    }

    /**
     * @param distance 距离单位米
     * @return 大于1千米显示单位为千米，否则单位为米
     */
    public static String setDistanceUint(double distance) {
        String dis;
        if (distance >= 1000) {
            return dis = distance / 1000 + "km";
        } else {
            return dis = distance + "m";
        }
    }

    /**
     * 定位信息
     *
     * @param application
     * @param location
     * @param callback
     * @return
     */
    @SuppressLint("MissingPermission")
    public static void getGeocoder(Context context, Location location, LocationUtil.OnCallback callback) {
        ThreadManager.runOnThread(new Runnable() {
            @Override
            public void run() {
                String province;
                String city;
                String area;
                String featureName;
                try {
                    Geocoder geocoder = new Geocoder(context, Locale.getDefault());
                    List<Address> addresses = geocoder.getFromLocation(location.getLatitude(), location.getLongitude(), 1);
                    if (addresses != null && addresses.size() > 0) {
                        Address address = addresses.get(0);
                        if (address != null) {
                            // 获取省份（province）
                            province = address.getAdminArea();
                            // 获取城市（City）
                            city = address.getLocality();
                            // 获取区县（area）
                            area = address.getSubLocality();
                            //  获取详细地址
                            featureName = address.getFeatureName();
                            // 获取街道地址
                            String addressLine = address.getAddressLine(0);
                            Log.i("Amap", "latitude: " + location.getLatitude());
                            Log.i("Amap", "longitude: " + location.getLongitude());

                            // 打印详细地址信息
                            Log.i("Amap", "province: " + province);
                            Log.i("Amap", "City: " + city);
                            Log.i("Amap", "area: " + area);
                            Log.i("Amap", "FeatureName: " + featureName);
                            Log.i("Amap", "Address Line: " + addressLine);

                            LocationParams.Longitude = String.valueOf(location.getLongitude());
                            LocationParams.Latitude = String.valueOf(location.getLatitude());
                            LocationParams.Address = addressLine;
                            LocationParams.city = city;
                            ThreadManager.runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    if (callback != null) {
                                        LocationEntity entity = new LocationEntity(String.valueOf(location.getLatitude()),
                                                String.valueOf(location.getLongitude()), province, city, area, addressLine);
                                        callback.onLocationListener(entity);
                                    }

                                }
                            });
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        });
    }
}