package com.urovo.wugumofang.ui

import android.content.Context
import android.content.res.TypedArray
import android.text.InputFilter
import android.text.InputType
import android.text.Spanned
import android.util.AttributeSet
import android.widget.EditText
import com.urovo.wugumofang.libutils.R

/**
 * @author Sasuke on 2017/12/6.
 * 用于保留两位小数的EditText
 */
class EditTextDecimal : EditText{
    private val DEFAULT_DECIMAL_NUMBER: Int = 2
    /**
     * 保留小数点后多少位
     */
    private var mDecimalNumber: Int? = DEFAULT_DECIMAL_NUMBER

//    constructor(context: Context?) : super(context)
//    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs)
//    constructor(context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr)


    constructor(context: Context?) : this(context,null)
    constructor(context: Context?, attrs: AttributeSet?) : this(context, attrs,android.R.attr.editTextStyle)
    constructor(context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr){
        val typedArray: TypedArray?= context?.obtainStyledAttributes(attrs, R.styleable.EditTextDecimal)
        mDecimalNumber = typedArray?.getInt(R.styleable.EditTextDecimal_decimalNumber,DEFAULT_DECIMAL_NUMBER)
        typedArray?.recycle()
        init()
    }



    fun init(){
        filters = arrayOf<InputFilter>(object :InputFilter{
            override fun filter(source: CharSequence?, start: Int, end: Int, dest: Spanned?, dstart: Int, dend: Int): CharSequence? {
                // source:当前输入的字符
                // start:输入字符的开始位置
                // end:输入字符的结束位置
                // dest：当前已显示的内容
                // dstart:当前光标开始位置
                // dend:当前光标结束位置
                val textBefore = dest.toString()
                when(true){
                    mDecimalNumber == 0 && source == "." -> return ""//不允许输入小数位
                    textBefore.isEmpty() && source == "."-> return "0."
                    dstart == 0 && dend == textBefore.length -> source.toString()//如果选取长度为原字长，说明有全选，那就直接返回输入内容
                    textBefore.contains(".")//含有小数点
                            && (dend > textBefore.indexOf("."))//在小数点后面输入
                            //总长 - 小数点序号 >= 保留位数+1，说明已经输满小数位
                            && (textBefore.length - textBefore.indexOf(".") >= mDecimalNumber!! + 1) -> return ""
                }
                return null
            }
        })
    }

    /**
     * 从外部对其小数位进行设置
     */
    fun setDecimal(decimal: Int){
        this.mDecimalNumber = decimal
    }

    override fun setInputType(type: Int) {
        super.setInputType(InputType.TYPE_NUMBER_FLAG_DECIMAL)
    }

    override fun onTextChanged(text: CharSequence?, start: Int, lengthBefore: Int, lengthAfter: Int) {
        super.onTextChanged(text, start, lengthBefore, lengthAfter)

    }
}


