package com.urovo.wugumofang.ui;

import java.util.ArrayList;
import java.util.List;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

/**
 * Created by Sasuke on 2036-3-7.
 * Fragment切换器，专业切换Fragment 30年。
 */

public class FragDominator {
    private int container;
    //之所以采用List而不是HashMap是考虑到还会用到顺序显示的问题，那时直接按顺序遍历list即可
    private List<FraContainer> fraList = new ArrayList<>();
    private int index = 0;
    AppCompatActivity activity;
    private Fragment currentFrag;//保存当前正在被show的Fragment

    class FraContainer{//再封装一层，有时候可能会用到tag来找frag
        public FraContainer(Fragment fra,String tag) {
            this.tag = tag;
            this.fra = fra;
        }

        String tag;
        Fragment fra;
    }

    public FragDominator(AppCompatActivity activity, int container) {
        this.activity = activity;
        this.container = container;
    }

    /**
     * 添加Fragment附带tag（注意tag不要重复）
     * @param fra
     * @param tag
     * @return
     */
    public FragDominator addWithTag(Fragment fra,String tag){
        fraList.add(new FraContainer(fra,tag));
        return this;
    }

    public FragDominator add(Fragment fra){
        fraList.add(new FraContainer(fra,""));
        return this;
    }





//    public FragDominator addAt(int pos,Fragment fra){
//        fraList.add(pos,fra);
//        return this;
//    }

    /**
     * 顺序显示下一个Fragment
     * 返回false表示显示到头了，后面没有了，
     */
    public boolean showFraNext(){
        if(fraList.size() != 0 && index < fraList.size()){
            showFra(fraList.get(index++).fra);
            return true;
        }
        return false;
    }

    /**
     * 不带动画的显示Fragment
     */
    public void showFra(Fragment toFra) {
        FragmentTransaction ft = activity.getSupportFragmentManager().beginTransaction();
        show(ft,toFra);
    }

    /**
     * 根据tag来显示Fragment
     * @param tag
     * @return 如果存在，则显示并返回true，如果不存在，则返回false
     */
    public boolean showFraByTag(String tag){
        for(FraContainer container : fraList){
            if(container.tag.equals(tag)){
                showFra(container.fra);
                return true;
            }
        }
        return false;
    }
//    public boolean showFraAt(int pos) {
//        FragmentTransaction ft = activity.getFragmentManager().beginTransaction();
//        return showAt(ft,pos);
//    }

    /**
     * 根据tag返回Fragment
     * @param tag
     * @return
     */
    public Fragment getFraWithTag(String tag){
        for(FraContainer container : fraList){
            if(container.tag.equals(tag)){
                return container.fra;
            }
        }
        return null;
    }

    /**
     *带动画的切换Fragment
     */
    public void showFraWithAnim(Fragment toFra,int enterAnim,int exitAnim){
        FragmentTransaction ft = activity.getSupportFragmentManager().beginTransaction();
        ft.setCustomAnimations(enterAnim,exitAnim);
        show(ft,toFra);
    }

//    private boolean showAt(FragmentTransaction ft,int pos){
//        if(fraList.size() != 0 && fraList.get(pos) != null){
//            show(ft,fraList.get(pos));
//            return true;
//        }else{
//            return false;//该位置不存在此Fra
//        }
//    }

//    public Fragment getFraAt(int pos){
//        if(fraList.size() != 0){
//            return fraList.get(pos);
//        }else{
//            return null;
//        }
//    }

    /**
     * 返回当前可见的Fragment
     * @return
     */
    public Fragment getCurrentFra(){
//        if (fraList.size() != 0) {
//            for (FraContainer item : fraList) {
//                if(item.fra.isVisible()){
//                    return item.fra;
//                }
//            }
//        }
        return currentFrag;
    }



    /**
     * 这里是核心，上面几个花式show都会调用这个，显示的同时隐藏其它
     * @param ft
     * @param toFra
     */
    private void show(FragmentTransaction ft, Fragment toFra){
        //先把非目标Fra全部隐藏
        if (fraList.size() != 0) {
            for (FraContainer item : fraList) {
                if (item.fra != toFra) {
                    ft.hide(item.fra);
                }
            }
        }

        //再把要显示的Fra显示出来（如果没有添加，就先添加再显示）
        if (!toFra.isAdded()) {
            //用commit会偶发产生此报错 java.lang.IllegalStateException: Can not perform this action after onSaveInstanceState
            ft.add(container, toFra).show(toFra).commitAllowingStateLoss();
        } else {
            ft.show(toFra).commitAllowingStateLoss();
        }
        currentFrag = toFra;
    }
}
