package com.urovo.wugumofang.ui;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.provider.MediaStore;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;

import androidx.appcompat.app.AppCompatActivity;

import com.squareup.picasso.Picasso;
import com.urovo.wugumofang.constants.Files;
import com.urovo.wugumofang.data.FileCache;
import com.urovo.wugumofang.data.FileUtil;
import com.urovo.wugumofang.libutils.R;
import com.urovo.wugumofang.utils.FuckGoogleAdaptUtil;
import com.urovo.wugumofang.view.BottomDialog;
import com.urovo.wugumofang.view.CustomPopupWindow;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Sasuke on 2017/7/24.
 * 选取图片工具类（拍照+本地相册）
 */

public class PhotoPickDominator {
    final private static String DEFAULT_DIR = Files.photoPath;
    /**
     * 拍照
     */
    final private static int PHOTO_REQUEST_TAKEPHOTO = 7479;
    /**
     * 从相册选取
     */
    final private static int PHOTO_REQUEST_GALLERY = 7450;
    private BottomDialog mDialog;
    private AppCompatActivity mActivity;
    /**
     * 具体地址
     */
    private String mPhotoPath = "";
    /**
     * 存图的目录（自定义的时候一定要像DEFAULT_DIR一样结尾带斜杠啊！）
     */
    private String mPhotoDir = "";

    //图片预览popupWindow
    CustomPopupWindow photoPopup;

    /**
     * 照片池，映射到外部的集合对象，然后外部到时候可以直接取这个值来用了。
     */
    private List<String> pathPool = new ArrayList<>();

    public PhotoPickDominator(AppCompatActivity activity) {
        this(activity, DEFAULT_DIR);
    }

    public PhotoPickDominator(AppCompatActivity activity, List<String> pathPool) {
        this(activity, DEFAULT_DIR);
        if (pathPool != null && pathPool.size() == 0){
            this.pathPool = pathPool;
        }
    }

    public PhotoPickDominator(AppCompatActivity activity, String photoDir) {
        this.mActivity = activity;
        this.mDialog = BottomDialog.newInstance(R.layout.ll_bottom_dialog);
        this.mPhotoDir = photoDir;
        //获取界面根布局
        ViewGroup root = (ViewGroup) ((ViewGroup) mActivity.findViewById(android.R.id.content)).getChildAt(0);
        initPhotoPopup(root);
    }

    /**
     * 初始化点击预览图片大图时的弹窗
     *
     * @param root 最外层的根布局ViewGroup
     */
    private void initPhotoPopup(ViewGroup root) {
        photoPopup = new CustomPopupWindow.Builder(mActivity)
                .setRoot(root)
                .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
                .setheight(ViewGroup.LayoutParams.MATCH_PARENT)
                .setContentView(R.layout.photo_preview_pop_ll) //设置布局文件
                .builder();
        photoPopup.getItemView(R.id.popupRoot).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                photoPopup.dismiss();
            }
        });
    }

    public interface PreviewCallback {
        public void run();
    }

    /**
     * 预览图片
     *
     * @return
     */
    public CustomPopupWindow previewPhotoUrl(String url, final ImageView iv, final PreviewCallback callback) {
        int scrWidth = mActivity.getResources().getDisplayMetrics().widthPixels;//获取屏幕宽
        Picasso.with(mActivity).load(url)
                .resize(scrWidth, (iv.getMeasuredHeight() / iv.getMeasuredWidth()) * scrWidth)
                .placeholder(R.drawable.progress_animation_small)
                .into((ImageView) photoPopup.getItemView(R.id.photoPreviewIv));
        photoPopup.showAtLocation(iv, Gravity.LEFT, 0, 0);
        //设置删除键监听
        photoPopup.getItemView(R.id.popupFab).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                iv.setImageResource(0);
                callback.run();
                photoPopup.dismiss();
            }
        });
        return photoPopup;
    }


    /**
     * 选完图后显示小图到UI上
     *
     * @param cameraAdd  点击添加图片的那个点击加号按钮, 用来量尺寸，作为预览小图的尺寸
     * @param photoGroup 小图放在哪个ViewGroup容器里
     */
    public void addPhoto2GroupByCamBtnSize(String path, View cameraAdd, final ViewGroup photoGroup) {
                addPhoto2Group(path,cameraAdd.getMeasuredWidth(), cameraAdd.getMeasuredHeight(),photoGroup);
//        ImageView iv = new ImageView(mActivity);
//        iv.setTag(path);//图片path路径存给自己
//        iv.setPadding(5, 0, 5, 0);//随便间隔一下，差不多一个意思
//        Picasso.with(mActivity).load(new File(path)).resize(cameraAdd.getMeasuredWidth(), cameraAdd.getMeasuredHeight()).centerCrop().into(iv);
//        //图片点击监听（点击显示大图）
//        final int scrWidth = mActivity.getResources().getDisplayMetrics().widthPixels;
//        iv.setOnClickListener(new View.OnClickListener() {
//            @Override
//            public void onClick(View v) {
//                Picasso.with(mActivity).load(new File((String) v.getTag()))
////                        .resize(scrWidth, (v.getHeight() / v.getWidth()) * scrWidth)
//                        .fit().centerCrop()
//                        .placeholder(R.drawable.progress_animation)
//                        .into((ImageView) photoPopup.getItemView(R.id.photoPreviewIv));
//                photoPopup.showAtLocation(v, Gravity.LEFT, 0, 0);
//                photoPopup.setTag(photoGroup.indexOfChild(v));//把图片序号存给popupWindow，用于删除元素
//            }
//        });
//        //监听点击删除图片
//        photoPopup.getItemView(R.id.popupFab).setOnClickListener(new View.OnClickListener() {
//            @Override
//            public void onClick(View v) {
//                int position = (int) photoPopup.getTag();
//                photoGroup.removeViewAt(position);
//                pathPool.remove(position);
//                photoPopup.dismiss();
//            }
//        });
//        //添加图片到容器
//        photoGroup.addView(iv, 0);
//        pathPool.add(0,path);
    }


    /**
     * 打包各ImageView的tag里存的path
     *
     * @param viewGroup
     * @return
     */
    public ArrayList<String> packPhotoPaths(ViewGroup viewGroup) {
        ArrayList<String> paths = new ArrayList<>();
        int childCount = viewGroup.getChildCount();
        for (int i = 0; i < childCount; i++) {
            Object tag = viewGroup.getChildAt(i).getTag();
            //该子View有tag并且tag转String后不为空，那就是成功获取到图片本地地址了
            if (tag != null && !TextUtils.isEmpty(((String) tag))) {
                paths.add((String) tag);
            }
        }
        return paths;
    }

    /**
     * 弹出“拍照”or“从本地相册选取”选择框
     *
     * @return
     */
    @SuppressLint("NewApi")
    public BottomDialog showDialog() {
        if (mDialog.isAdded()) {
            return mDialog;
        }
        mDialog.show(mActivity.getSupportFragmentManager().beginTransaction(), "BottomDialog");
        mDialog.setItemClickListener(new BottomDialog.OnSelectListener() {
            @Override
            public void onCameraClick(View v) {
                mDialog.dismiss();
                selectByCamera(mActivity);
            }

            @Override
            public void onLocalClick(View v) {
                mDialog.dismiss();
                selectByLocal(mActivity);
            }
        });
        return mDialog;
    }

    /**
     * 单独拍照获取
     */
    public void selectByCamera(Activity activity) {
        Intent cameraintent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        // 指定调用相机拍照后照片的储存路径
        String picName = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + ".jpg";
        File file = new File(mPhotoDir, picName);
        //临时存储，待onActivityResult成功返回后，再拿出来添加到列表
        mPhotoPath = file.getPath();
//        cameraintent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(file));
        cameraintent.putExtra(MediaStore.EXTRA_OUTPUT, FuckGoogleAdaptUtil.android7AdaptUri((Context) activity, Files.fileProviderAuth, file));
        activity.startActivityForResult(cameraintent, PhotoPickDominator.PHOTO_REQUEST_TAKEPHOTO);
    }

    /**
     * 单独从相册获取
     */
    public void selectByLocal(Activity activity) {
        Intent intent = new Intent(Intent.ACTION_PICK, null);
        intent.setDataAndType(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, "image/*");
        activity.startActivityForResult(intent, PHOTO_REQUEST_GALLERY);
    }


    /**
     * 获取选取的图片本机地址（用在引用的Activity的onActivityResult()里面）
     *
     * @return
     */
    public String onPhotoPathResult(Context context, int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            //拍照
            case PHOTO_REQUEST_TAKEPHOTO:
                if (resultCode == Activity.RESULT_OK) {
                    return mPhotoPath;
                }
                break;
            //相册
            case PHOTO_REQUEST_GALLERY:
                if (resultCode == Activity.RESULT_OK) {
                    String imgPath;
                    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.N) {
                        Cursor cursor = context.getContentResolver().query(data.getData(), null, null, null, null);
                        if (cursor != null && cursor.moveToFirst()) {
                            //安卓7.0以上，4才是路径，但低版本安卓，1是路径。
                            imgPath = cursor.getString(4);
                            //如果不是路径，就再找找。
                            if (!imgPath.contains("/storage/")) {
                                //找个6次应该就差不多了，7、8.。。。后面一般都是null
                                for (int i = 0; i < 7; i++) {
                                    if (cursor.getString(i).contains("/storage/")) {
                                        imgPath = cursor.getString(i);
                                        break;
                                    }
                                }
                            }
                            cursor.close();
                        } else {
                            imgPath = data.getDataString().replace("file://", "");
                        }
                    }else{//如果是安卓7.0或以上
                        imgPath = FuckGoogleAdaptUtil.getRealFilePath(context,data.getData());
                    }

                    String picName = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + ".jpg";
                    FileCache.copyFile(imgPath, mPhotoDir + picName);
                    return mPhotoDir + picName;
                }
                break;
            default:
        }
        return "";
    }


    /**
     * 把临时在SD卡上创建的图片临时文件删除
     */
    public void deletePicCache() {
        FileUtil.deleteAll(new File(mPhotoDir));
    }

    /**预览大图弹窗*/
    private CustomPopupWindow previewPop;

    public CustomPopupWindow getPreviewPop(){
        if (previewPop == null){
            previewPop = new CustomPopupWindow.Builder(mActivity)
                    .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
                    .setheight(ViewGroup.LayoutParams.MATCH_PARENT)
                    .setContentView(R.layout.photo_preview_pop_ll) //设置布局文件
                    .builder();
            previewPop.getItemView(R.id.popupRoot).setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    previewPop.dismiss();
                }
            });
        }
        return previewPop;
    }

    /**
     * 把图片显示到Group布局里
     */
    public void addPhoto2Group(String photoPath,int showWidth,int showHeight,ViewGroup photoGroup){
        if (photoPath == "") return;//如果没有选择图片，返回回来的路径是“”
        //用Path把图片显示出来
        ImageView iv = new ImageView(mActivity);
        iv.setTag(photoPath);//图片path路径存给自己
        iv.setPadding(8, 0, 8, 0);//随便间隔一下，差不多一个意思
        Picasso.with(mActivity).load(new File(photoPath)).resize(showWidth, showHeight).into(iv);

        //再挂监听
        iv.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Picasso.with(mActivity).load(new File((String)v.getTag()))
                        .resize(getGoalSizes(mActivity,photoPath)[0], getGoalSizes(mActivity,photoPath)[1])
                        .placeholder(R.drawable.progress_animation_small)
                        .into((ImageView) getPreviewPop().getItemView(R.id.photoPreviewIv));
                getPreviewPop().showAtLocation(v,Gravity.LEFT,0,0);
                getPreviewPop().setTag(photoGroup.indexOfChild(v));//把图片序号存给popupWindow
            }
        });
        //删除图片监听
        getPreviewPop().getItemView(R.id.popupFab).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int position = (int)getPreviewPop().getTag();
                photoGroup.removeViewAt(position);
                pathPool.remove(position);
                getPreviewPop().dismiss();
            }
        });
        photoGroup.addView(iv, 0);
        pathPool.add(0,photoPath);
    }


    private int [] getGoalSizes(Context context,String photoPath){
        //先计算好应该显示的大图尺寸
        int scrWidth = context.getResources().getDisplayMetrics().widthPixels;
        int scrHeight = context.getResources().getDisplayMetrics().heightPixels;
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inJustDecodeBounds = true;
        BitmapFactory.decodeFile(photoPath, options);

        int goalWidth = 0;
        int goalHeight = 0;
        if (options.outWidth / (float)options.outHeight > scrWidth / (float)scrHeight){
            goalWidth = scrWidth;//图的宽太大了，调成屏幕同宽
            goalHeight = (int)(scrWidth / (float)options.outWidth * options.outHeight);
        }else{
            goalHeight = scrHeight;//图的高太大了，调成屏幕同宽
            goalWidth = (int)(scrHeight / (float)options.outHeight * options.outWidth);
        }
        return new int[]{goalWidth,goalHeight};
    }
}
