package com.urovo.wugumofang.ui

import android.annotation.SuppressLint
import android.app.Activity
import android.app.ProgressDialog
import android.content.Context
import android.graphics.BitmapFactory
import android.os.Handler
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageButton
import android.widget.LinearLayout
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.tencent.wework.api.model.WWAuthMessage
import com.tencent.wework.api.model.WWMediaImage
import com.tencent.wework.api.model.WWMediaLink
import com.umeng.socialize.Config
import com.umeng.socialize.ShareAction
import com.umeng.socialize.bean.SHARE_MEDIA
import com.umeng.socialize.media.UMImage
import com.urovo.wugumofang.constants.Files
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.libutils.R
import com.urovo.wugumofang.system.BaseAppManager
import com.urovo.wugumofang.system.BaseAppManager.iwwapi
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.BottomDialog
import com.zsy.libext.widgets.clipboard.ClipboardUtil
import org.jetbrains.anko.find
import java.util.*

/**
 * @author Sasuke on 2019/8/20.
 * 用法eg：
 *  ①shareDominator.showDialog().setShareBtns(
        ShareDominator.CIRCLE_WX, ShareDominator.FRIENDS_WX, ShareDominator.COPY_LINK
            ).isShowHeadLine(false).setWebTitle(webTitle);//不用显示“分享到”文字两边的线条


        ②shareDominator = new ShareDominator(this, presenter.getData().getQrUrl(),"1");
        //初始化UI
        shareDominator.inflateBtnLayoutUI(bottomShareLayout)
        .setShareBtns(ShareDominator.ENTERPRISE_WX, ShareDominator.CIRCLE_WX, ShareDominator.FRIENDS_WX)
        .isShowHeadLine(true)//显示“分享到”两边的横线
        .isShowCancelBtn(false);//不显示底下取消按钮
        //初始化分享内容
        shareDominator.setWebTitle(presenter.getData().getTitle());
        shareDominator.setContentText(presenter.getData().getContent());
        shareDominator.setImage(presenter.getData().getEntity().getUMImage(this));
        shareDominator.setImageUrl(presenter.getData().getImageUrl());
        //以下埋点（点击了就算数，用回调好像有问题。）
        shareDominator.setOnShared(buryPoint -> {
                presenter.buryPoint(buryPoint);
                return null;
        });


 *  activityType: 1是会员商品转发，2是养生文库（分享微信文章时，用空白页中转，其它情况不用）
 */
class ShareDominator(val mActivity: AppCompatActivity,val url: String,var activityType: String = "") {

    companion object {
        const val ENTERPRISE_WX = "ENTERPRISE_WX"//分享到企业微信
        const val CIRCLE_WX = "CIRCLE_WX"//分享到朋友圈
        const val FRIENDS_WX = "FRIENDS_WX"//分享给微信朋友
        const val COPY_LINK = "COPY_LINK"//复制链接
        const val SAVE_LOCAL = "SAVE_LOCAL"//本地保存
        const val JUMP_SHARE_POSTER = "JUMP_SHARE_POSTER"//分享海报
        const val ENTERPRISE_WX_IMG = "ENTERPRISE_WX_IMG"//分享到企业微信（单图）
        const val CIRCLE_WX_IMG = "CIRCLE_WX_IMG"//分享到朋友圈（单图）
        const val FRIENDS_WX_IMG = "FRIENDS_WX_IMG"//分享给微信朋友（单图）


        @JvmStatic
        /**分享到企业微信
         * activityType: 1是会员商品转发，2是养生文库（分享微信文章时，用空白页中转，其它情况不用）
         * */
        fun doEnterpriseShare(url: String,webTitle: String,contentText: String = "",imageUrl: String = "",activityType: String = ""){
            val weixinUrl3 = url + (if (url.contains("?")) "&" else "?") + "time=" + Calendar.getInstance().timeInMillis + "&userName=" + SystemHelper.LoginUserId + "&type=${activityType}"
            val link = WWMediaLink().apply {
                thumbUrl = imageUrl
                webpageUrl = if (activityType == "2") NetParam.shareWeixinByRedirect(weixinUrl3, SystemHelper.LoginOrganId, SystemHelper.LoginUserId, activityType)
                                                 else weixinUrl3
                title = webTitle
                description = if (contentText.isNullOrBlank()) webTitle else contentText
                appPkg = BaseAppManager.PACKAGE_NAME
                appName = BaseAppManager.getAppName()
                appId = NetParam.EW_APPID
                agentId = NetParam.EW_AGENTID
            }
            iwwapi.sendMessage(link) { resp ->//不知道为什么，这里不回调，打日志也打不出来。
                if (resp is WWAuthMessage.Resp){
                    when(resp.errCode){
                        WWAuthMessage.ERR_CANCEL -> { MyToast.show("分享取消") }
                        WWAuthMessage.ERR_FAIL -> { MyToast.show("分享失败") }
                        WWAuthMessage.ERR_OK -> { MyToast.show("分享成功") }
                    }
                }
            }
        }

        @JvmStatic
                /**分享到企业微信【图片】
                 * activityType: 1是会员商品转发，2是养生文库（分享微信文章时，用空白页中转，其它情况不用）
                 * */
        fun doEnterpriseShare(imageName: String, imagePath: String ,activityType: String = "") {
            val img = WWMediaImage().apply {
                fileName = imageName
                filePath = imagePath
                appPkg = BaseAppManager.PACKAGE_NAME
                appName = BaseAppManager.getAppName()
                appId = NetParam.EW_APPID
                agentId = NetParam.EW_AGENTID
            }
            iwwapi.sendMessage(img) { resp ->//不知道为什么，这里不回调，打日志也打不出来。
                if (resp is WWAuthMessage.Resp){
                    when(resp.errCode){
                        WWAuthMessage.ERR_CANCEL -> { MyToast.show("分享取消") }
                        WWAuthMessage.ERR_FAIL -> { MyToast.show("分享失败") }
                        WWAuthMessage.ERR_OK -> { MyToast.show("分享成功") }
                    }
                }
            }
        }

            @JvmStatic
        /** 分享到朋友圈
         *  activityType: 1是会员商品转发，2是养生文库（分享微信文章时，用空白页中转，其它情况不用）
         * */
        fun doCircleWxShare(activity: Activity,url: String,webTitle: String, contentText: String = "",image: UMImage? = null, activityType: String = ""){
            val weixinUrl2 = url + (if (url.contains("?")) "&" else "?") + "time=" + Calendar.getInstance().timeInMillis + "&userName=" + SystemHelper.LoginUserId + "&type=${activityType}"
            ShareAction(activity).setPlatform(SHARE_MEDIA.WEIXIN_CIRCLE)
                    .withMedia(if (image == null )  UMImage(activity, BitmapFactory.decodeResource(activity.resources, R.drawable.wgmf_logo)) else image)
                    .withText(if (contentText.isNullOrBlank()) webTitle else contentText)
                    .withTitle(webTitle)
                    //如果是养生文库，就用空白页重定向
                    .withTargetUrl(if (activityType == "2") NetParam.shareWeixinByRedirect(weixinUrl2, SystemHelper.LoginOrganId, SystemHelper.LoginUserId, activityType)
                                                else weixinUrl2)
                    //                        .withTargetUrl(weixinUrl2)
                    .share()
        }

        @JvmStatic
                /** 分享到朋友圈【图片】
                 *  activityType: 1是会员商品转发，2是养生文库（分享微信文章时，用空白页中转，其它情况不用）
                 * */
        fun doCircleWxShare(activity: Activity,image: UMImage? = null, activityType: String = ""){
//            val weixinUrl2 = url + (if (url.contains("?")) "&" else "?") + "time=" + Calendar.getInstance().timeInMillis + "&userName=" + SystemHelper.LoginUserId + "&type=${activityType}"
            ShareAction(activity).setPlatform(SHARE_MEDIA.WEIXIN_CIRCLE)
                    .withMedia(if (image == null )  UMImage(activity, BitmapFactory.decodeResource(activity.resources, R.drawable.wgmf_logo)) else image)
//                    .withText(if (contentText.isNullOrBlank()) webTitle else contentText)
//                    .withTitle(webTitle)
//                    //如果是养生文库，就用空白页重定向
//                    .withTargetUrl(if (activityType == "2") NetParam.shareWeixinByRedirect(weixinUrl2, SystemHelper.LoginOrganId, SystemHelper.LoginUserId, activityType)
//                    else weixinUrl2)
                    //                        .withTargetUrl(weixinUrl2)
                    .share()
        }

        @JvmStatic
        /**分享到微信好友
         *  activityType: 1是会员商品转发，2是养生文库（分享微信文章时，用空白页中转，其它情况不用）
         * */
        fun doFriendsShare(activity: Activity, url: String, webTitle: String, contentText: String = "", image: UMImage? = null, activityType: String = ""){
            val weixinUrl = url + (if (url.contains("?")) "&" else "?") + "time=" + Calendar.getInstance().timeInMillis + "&userName=" + SystemHelper.LoginUserId
            ShareAction(activity).setPlatform(SHARE_MEDIA.WEIXIN)/*.setCallback(umShareListener)*/
                    .withMedia(if (image == null )  UMImage(activity, BitmapFactory.decodeResource(activity.resources, R.drawable.wgmf_logo)) else image)
                    .withText(if (contentText.isNullOrBlank()) webTitle else contentText)
                    .withTitle(webTitle)//因样式问题，这个不用title
                    //如果是养生文库，就用空白页重定向
                    .withTargetUrl(if (activityType == "2") NetParam.shareWeixinByRedirect(weixinUrl, SystemHelper.LoginOrganId, SystemHelper.LoginUserId, activityType)
                                                else weixinUrl )
                    .share()
        }

        @JvmStatic
                /**分享到微信好友【图片】
                 *  activityType: 1是会员商品转发，2是养生文库（分享微信文章时，用空白页中转，其它情况不用）
                 * */
        fun doFriendsShare(activity: Activity,image: UMImage? = null, activityType: String = ""){
//            val weixinUrl = url + (if (url.contains("?")) "&" else "?") + "time=" + Calendar.getInstance().timeInMillis + "&userName=" + SystemHelper.LoginUserId
            ShareAction(activity).setPlatform(SHARE_MEDIA.WEIXIN)/*.setCallback(umShareListener)*/
                    .withMedia(if (image == null )  UMImage(activity, BitmapFactory.decodeResource(activity.resources, R.drawable.wgmf_logo)) else image)
//                    .withText(if (contentText.isNullOrBlank()) webTitle else contentText)
//                    .withTitle(webTitle)//因样式问题，这个不用title
//                    //如果是养生文库，就用空白页重定向
//                    .withTargetUrl(if (activityType == "2") NetParam.shareWeixinByRedirect(weixinUrl, SystemHelper.LoginOrganId, SystemHelper.LoginUserId, activityType)
//                    else weixinUrl )
                    .share()
        }

        @JvmStatic
        //复制链接
        fun doCopyLink(context: Context,url: String){
            ClipboardUtil.copyText(context,url )// 复制到系统的剪切板
            MyToast.show("复制成功")
        }

    }


    /**禁用或启用分享按钮点击*/
    var isEnable = true

    private val mDialog by lazy {
        BottomDialog.newInstance(R.layout.ll_bottom_share_dialog)
    }

    private var leftLine: View? = null//“分享到”左边的线条
    private var rightLine: View? = null//“分享到”右边的线条
    private var cancelBtnLayout: LinearLayout? = null//底部取消按钮
    private var btnsRv: RecyclerView? = null
    var webTitle: String = ""
    var contentText = ""
    var image: UMImage? = null//分享时的缩略图（非企业微信情况 ，毕竟用的是友盟）
    var viewForBitmap: View? = null//用于转换成Bitmap的View
    var imageTag = ""
    var imageUrl = ""//分享时的缩略图的url（企业微信情况，因为企业微信的sdk里需要imageUrl）
    /**分享之后要做的操作，留给外部调用时自定义*/
    var onShared:((buryPointType: Int) ->Unit)? = null//企业微信

    fun setImage(image: UMImage,tag: String){
        this.image = image
        this.imageTag = tag
    }

    /**把需要转换成bitmap的View保存起来，因为刚加载完图的时候，直接转换成bitmap可能会图还没加载完就转换完了，
     * 导致后面分享时转换的图是空白的，所以先保存View，在分享前，再把View转换成Bitmap*/
    fun saveViewForBitmap(view: View,tag: String){
        this.viewForBitmap = view
        this.imageTag = tag
    }

    private fun transView2UMImage(){
        viewForBitmap?.let {
            setImage(UMImage(mActivity, BitmapUtil.view2Bitmap(it)),imageTag)
        }
    }

    private var btnsData: List<ShareItem> = listOf()
    /**弹出底部弹窗*/
    @SuppressLint("NewApi")
    fun showDialog(): ShareDominator {
        if (mDialog.isAdded()) {
            return this
        }
        mDialog.show(mActivity.getSupportFragmentManager().beginTransaction(), "BottomDialog")
        Handler().post {
            inflateBtnLayoutUI(mDialog.view)
        }

        return this
    }

    fun inflateBtnLayoutUI(view: View?, spanCount: Int): ShareDominator {
        leftLine = view?.find(R.id.leftLine)
        rightLine = view?.find(R.id.rightLine)
        cancelBtnLayout = view?.find(R.id.cancelBtnLayout)
        btnsRv = view?.find<RecyclerView>(R.id.btnsRv)
        btnsRv?.run {
            layoutManager = object: GridLayoutManager(mActivity,spanCount){
                override fun canScrollVertically(): Boolean {
//                        return super.canScrollVertically()
                    return false
                }
            }
            adapter = ShareBtnGridAdapter(mActivity,btnsData,this@ShareDominator)
        }

        //取消
        (view?.find(R.id.cancelBtn) as Button).setOnClickListener {
            mDialog.dismiss()
        }
        return this
    }

        /**渲染按钮们的显示*/
    fun inflateBtnLayoutUI(view: View?): ShareDominator{
            return inflateBtnLayoutUI(view,3)
    }

    /**外部传入 指定需要展示哪几种分享按钮*/
    fun setShareBtns(vararg btnTypes: String): ShareDominator{
        var tempList = mutableListOf<ShareItem>()
        btnTypes.forEach {
            var iconRid: Int = 0
            var text: String = ""
            when(it){
                ENTERPRISE_WX ->{ iconRid = R.drawable.btn_share_enterprise_wechat ; text = "企业微信" }
                CIRCLE_WX -> { iconRid = R.drawable.btn_share_card ; text = "朋友圈" }
                FRIENDS_WX -> { iconRid = R.drawable.btn_share_wechat ; text = "微信朋友"}
                COPY_LINK -> { iconRid = R.drawable.btn_copylink ; text = "复制链接"}
                SAVE_LOCAL ->{ iconRid = R.mipmap.btn_save_local ; text = "本地保存"}
                JUMP_SHARE_POSTER -> {iconRid = R.mipmap.btn_share_poster ; text = "分享海报"}
                ENTERPRISE_WX_IMG ->{ iconRid = R.drawable.btn_share_enterprise_wechat ; text = "企业微信" }
                CIRCLE_WX_IMG -> { iconRid = R.drawable.btn_share_card ; text = "朋友圈" }
                FRIENDS_WX_IMG -> { iconRid = R.drawable.btn_share_wechat ; text = "微信朋友"}
            }
            tempList.add(ShareItem(iconRid, text, it))
        }
        btnsData = tempList
        btnsRv?.adapter = ShareBtnGridAdapter(mActivity,btnsData,this)
        return this
    }

    /**是否显示“分享到”的左右线条*/
    fun isShowHeadLine(isShow: Boolean): ShareDominator{
        Handler().post {
            leftLine?.visibility = if (isShow) View.VISIBLE else View.GONE
            rightLine?.visibility = if (isShow) View.VISIBLE else View.GONE
        }
        return this
    }

    /**是否显示取消按钮*/
    fun isShowCancelBtn(isShow: Boolean): ShareDominator{
        Handler().post {
            cancelBtnLayout?.visibility = if (isShow) View.VISIBLE else View.GONE
        }
        return this
    }

    //保存海报回调
//    var onSaveLocalCallback: (() ->Unit)? = null
    //跳转海报页回调
    var onJumpSharePosterCallback: (() ->Unit)? = null


    //以下 Class 们


    class ShareBtnGridAdapter(val context: AppCompatActivity,val dataList: List<ShareItem>,val dominator: ShareDominator): RecyclerView.Adapter<ShareVH>(){
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ShareVH {
            return ShareVH(LayoutInflater.from(context).inflate(R.layout.ll_bottom_share_grid_item,parent,false))
        }

        override fun getItemCount(): Int {
            return dataList.size
        }

        override fun onBindViewHolder(holder: ShareVH, pos: Int) {
            holder.apply {
                icon?.setImageResource(dataList[pos].iconRid)
                text?.text = dataList[pos].text
                //默认监听
                icon?.setOnClickListener {
                    if (!dominator.isEnable) return@setOnClickListener
                    val dialog = ProgressDialog(context)
                    dialog.setTitle("启动分享中")
                    dialog.setMessage("请稍等.....")
                    Config.dialog = dialog
                    dominator.mDialog.takeIf { it.isAdded }?.dismiss()
                    dominator.transView2UMImage()//如果是海报模式，需要把预先存进的View转成Bitmap
                    when (dataList[pos].type) {
                        ENTERPRISE_WX -> {//企业微信分享
                            dominator.run { doEnterpriseShare(url, webTitle, contentText, imageUrl) }
                            dominator.onShared?.let { it.invoke(2) }//外部埋点
                        }
                        CIRCLE_WX -> {//朋友圈分享
                            dominator.run { doCircleWxShare(context, url, webTitle, contentText, image, activityType) }
                            dominator.onShared?.let { it.invoke(0) };//外部埋点
                        }
                        FRIENDS_WX -> {//微信好友分享
                            dominator.run { doFriendsShare(context, url, webTitle, contentText, image, activityType) }
                            dominator.onShared?.let { it.invoke(1) };//外部埋点
                        }
                        COPY_LINK -> {
                            doCopyLink(context, dominator.url)
                            return@setOnClickListener
                        }
                        JUMP_SHARE_POSTER -> {//跳转分享海报页面
                            dominator.onJumpSharePosterCallback?.invoke()
                        }
                        //以下海报模式（需求是海报暂时不做埋点）===================================
                        ENTERPRISE_WX_IMG -> {//企业微信分享图片
                            //企业微信要先保存到本地，再分享，因为企业微信的分享用到图片的本地路径。
                            PictureUtil.saveBitmapFile( dominator.image?.asBitmap(), Files.tempPicture, dominator.imageTag +".jpg")
                            dominator.run { doEnterpriseShare(dominator.imageTag +".jpg", Files.tempPicture + dominator.imageTag +".jpg") }
//                            dominator.onShared?.let { it.invoke(2) };//外部埋点
                        }
                        CIRCLE_WX_IMG -> {//朋友圈分享图片
                            dominator.run { doCircleWxShare( context,image, activityType) }
//                            dominator.onShared?.let { it.invoke(0) };//外部埋点
                        }
                        FRIENDS_WX_IMG -> {//微信好友分享图片
                            dominator.run { doFriendsShare( context, image, activityType) }
//                            dominator.onShared?.let { it.invoke(1) };//外部埋点
                        }

                        SAVE_LOCAL -> {//保存本地
//                            dominator.onSaveLocalCallback?.invoke()
                            dominator.image?.let {
                                PictureUtil.saveBitmapFile( it.asBitmap(), Files.tempPicture, dominator.imageTag +".jpg")
                                MyToast.show("已保存")
                            }
                        }

                    }

                }
            }
        }
    }

    class ShareVH(view: View): RecyclerView.ViewHolder(view){
        var icon: ImageButton? = null
        var text: TextView? = null

        init {
            icon = view.find(R.id.shareIcon)
            text = view.find(R.id.shareTv)
        }
    }

    data class ShareItem(var iconRid: Int,var text: String,var type: String)

//    var onEnterpriseShared:(() ->Unit)? = null//企业微信
//    var onFriendsShared:(() ->Unit)? = null//微信朋友
//    var onCircleShared:(() ->Unit)? = null//朋友圈

}