package com.urovo.wugumofang.utils

import android.app.Activity
import android.graphics.Rect

object SoftKeyBoardUtil {

    /**
     * 软件盘隐藏||显示的监听
     */
    fun setOnSoftKeyBoardListener(activity: Activity, listener: OnSoftKeyBoardChangeListener) {
        //监听视图树中全局布局发生改变或者视图树中的某个视图的可视状态发生改变
        val rootView = activity.window.decorView
        rootView.viewTreeObserver.addOnGlobalLayoutListener {
            //获取当前根视图在屏幕上显示的大小
            val rect = Rect()
            rootView.getWindowVisibleDisplayFrame(rect)
            val screenHeight = rootView.rootView.height
            val keyboardHeight: Int = screenHeight - rect.bottom
            // 根据键盘的高度调整底部控件的位置
            if (keyboardHeight > 200) {
                // 键盘显示时的处理逻辑
                listener.keyBoardShow(keyboardHeight)
            } else {
                // 键盘隐藏时的处理逻辑
                listener.keyBoardHide(keyboardHeight)
            }
        }
    }

    interface OnSoftKeyBoardChangeListener {
        fun keyBoardShow(height: Int)

        fun keyBoardHide(height: Int)
    }
}