package com.urovo.wugumofang.view

import android.animation.ObjectAnimator
import android.app.AlertDialog
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.animation.AccelerateDecelerateInterpolator
import android.widget.ProgressBar
import com.urovo.wugumofang.libutils.R
import org.jetbrains.anko.find

/**
 * @author Sasuke on 2018/3/22.
 * 进度条弹窗封装
 */
class ProgressBarDialog(var builder: ProgressBarDialog.Builder) {
    var dialog: AlertDialog? = null

    init {
        dialog = AlertDialog.Builder(builder.context).setTitle(builder.title)
                .setView(builder.view)
                .setCancelable(false)
                .create()
        builder.view?.find<ProgressBar>(R.id.progressBar)?.apply {
            max = builder.max//设置最大刻度
//            interpolator = AccelerateDecelerateInterpolator()//动画效果
        }
    }

    fun show(): ProgressBarDialog {
        dialog?.show()
        return this
    }



    fun dismiss() {
        dialog?.dismiss()
    }

    /**
     * 注意要在UI线程调用
     */
    fun setProgress(progress: Int) {
        ObjectAnimator.ofInt(builder.view?.find<ProgressBar>(R.id.progressBar), "progress", progress).apply {
            setDuration(1000)
            interpolator = AccelerateDecelerateInterpolator()
        }.start()

        if (progress == builder.max) {
            dialog?.dismiss()
        }
    }

    /**
     * progress自增
     */
    fun setProgressPlus(){
        builder.view?.find<ProgressBar>(R.id.progressBar)?.progress?.let {
            setProgress(it + (builder.max / builder.unitCount))//总刻度除以总等分，就是每一个等分跑多少刻度。
        }
    }

    /**
     * 传百分比
     */
    fun setProgressPercent(progress: Double) {
        setProgress((progress * builder.max).toInt())
    }


    /**
     * max是总刻度。。。而业务上的总等分是unitCount
     */
    class Builder(var context: Context,var unitCount: Int = 10, var max: Int = 1000/*默认1000吧*/) {
        var view: View? = null
        var title: String? = null

        init {
            view = LayoutInflater.from(context).inflate(R.layout.ll_progressbar_dialog, null, false)
        }

        fun create(): ProgressBarDialog {
            return ProgressBarDialog(this)
        }
    }
}



