package com.urovo.wugumofang.view

import android.content.Context
import android.graphics.Color
import android.util.AttributeSet
import android.util.TypedValue
import android.view.Gravity
import android.view.animation.AnimationUtils
import android.widget.FrameLayout
import android.widget.TextSwitcher
import android.widget.TextView
import com.urovo.wugumofang.libutils.R
import org.jetbrains.anko.dip
import org.jetbrains.anko.sp


/**
 * @author Sasuke on 2019-03-26 0026.
 * 对TextSwitcher的封装
 */
class TextSwitcherDominator: TextSwitcher {
    private var mTvGravity: Int? = Gravity.CENTER
    private var mText: String? = ""
    private var mTextColor: Int?
    private var mTextSize: Float? = dip(16).toFloat()

    constructor(context: Context?) : this(context,null)
    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs){
        val attr = context?.obtainStyledAttributes(attrs, R.styleable.TextSwitcherDominator)
        mTvGravity = attr?.getInt(R.styleable.TextSwitcherDominator_gravity,0)
        mText = attr?.getString(R.styleable.TextSwitcherDominator_text)
        mTextColor = attr?.getColor(R.styleable.TextSwitcherDominator_textColor,Color.BLACK)
        mTextSize = attr?.getDimension(R.styleable.TextSwitcherDominator_textSize,sp(16).toFloat())
        attr?.recycle()

        init()
    }

    fun init(){
        setFactory {
            TextView(context).apply {
                text = mText
                setTextColor(mTextColor!!)
                setSingleLine()
                setTextSize(TypedValue.COMPLEX_UNIT_PX,mTextSize!!)
                gravity = when(mTvGravity){
                    0 -> Gravity.CENTER_HORIZONTAL
                    1-> Gravity.LEFT
                    else -> Gravity.RIGHT
                } and Gravity.CENTER_VERTICAL
                layoutParams = FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT)
            }
        }
        val inAnime = AnimationUtils.loadAnimation(context,R.anim.design_bottom_sheet_slide_in)
        val outAnime = AnimationUtils.loadAnimation(context,R.anim.design_bottom_sheet_slide_out)
        setInAnimation(inAnime)//设置文本出现动画
        setOutAnimation(outAnime)//设置文本消失动画
        setCurrentText(mText)
    }

    /**
     * 获取当前显示文字
     */
    fun getText(): CharSequence?{
        return (currentView as TextView)?.text
    }
}