package com.urovo.wugumofang.view

import android.graphics.Bitmap
import android.graphics.Canvas
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.style.ImageSpan
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.annotation.DrawableRes
import androidx.core.content.res.ResourcesCompat
import com.urovo.wugumofang.libutils.R
import com.urovo.wugumofang.ui.DensityUtil


fun TextView.addTag(
    content: String = " ",
    tag: String = " ",
    @DrawableRes tagBgId: Int = R.drawable.shape_rect_round_6_fill_ffffff,
    tagColor: Int = R.color.important_txt_color,
    tagTextSize: Float = 11F,
    bottomPadding: Float = 3F
) {

    val totalContent = content.plus(" ").plus(if (tag.isEmpty()) " " else tag)
    val layout = LinearLayout(context)
    val tagView = if (tag.isEmpty()) ImageView(context).apply {
        setImageResource(tagBgId)
        setPadding(0, 0, 0, DensityUtil.dip2px(context,bottomPadding))
    } else TextView(this.context).apply {
        text = tag
        background = ResourcesCompat.getDrawable(context.resources, tagBgId, null)
        textSize = tagTextSize
        setTextColor(ResourcesCompat.getColor(context.resources, tagColor, null))
        includeFontPadding = false
        setPadding(
            DensityUtil.dip2px(context,3f),
            DensityUtil.dip2px(context,3f),
            DensityUtil.dip2px(context,3f),
            DensityUtil.dip2px(context,bottomPadding)
        )
        height = DensityUtil.dip2px(context,18f)
        gravity = Gravity.CENTER
    }

    val layoutParams = LinearLayout.LayoutParams(
        ViewGroup.LayoutParams.WRAP_CONTENT,
        ViewGroup.LayoutParams.WRAP_CONTENT
    )
    layout.addView(tagView, layoutParams)
    layout.measure(
        View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED),
        View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED)
    )
    layout.layout(0, 0, tagView.measuredWidth, tagView.measuredHeight)
    layout.setPadding( DensityUtil.dip2px(context,20f), 0, 0, 0)

    val bitmap = Bitmap.createBitmap(tagView.width, tagView.height, Bitmap.Config.ARGB_8888)
    val canvas = Canvas(bitmap)
    tagView.draw(canvas)

    val imageSpan = ImageSpan(context, bitmap)
    val spannableStringBuilder = SpannableStringBuilder(totalContent)
    spannableStringBuilder.setSpan(
        imageSpan,
        content.length + 1,
        totalContent.length,
        Spanned.SPAN_EXCLUSIVE_INCLUSIVE
    )
    this.text = spannableStringBuilder
}
