package com.urovo.wugumofang.view.dialog

import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.view.*
import android.widget.*
import com.urovo.wugumofang.libutils.R
import com.urovo.wugumofang.ui.DensityUtil
import com.urovo.wugumofang.utils.notNullAndEmpty

/**
 * 容器数据弹窗
 * @author LW
 */
class PublicContainerDataDialog(context: Context, theme: Int) : Dialog(context, theme) {

    init {
        //这句代码设置这个对话框是否能被用户触摸取消掉
        setCanceledOnTouchOutside(false)
        //这句代码设置这个对话框是否能被用户按[返回键]而取消掉
        setCancelable(true)
    }

    companion object {

        class Builder(private val context: Context) {
            private var title: String? = ""
            private var message: String? = ""
            private var isDataEntry: Boolean = true
            private var dialogHeight: Int = -1
            private var dialogInitContainerDataListener: OnDialogInitContainerDataListener? = null
            private var positiveButtonText: String? = null
            private var negativeButtonText: String? = null
            private var positiveButtonClickListener: DialogInterface.OnClickListener? = null
            private var negativeButtonClickListener: DialogInterface.OnClickListener? = null

            fun setTitle(title: Int): Builder {
                this.title = context.getText(title) as String
                return this
            }

            fun setTitle(title: String?): Builder {
                this.title = title
                return this
            }

            fun setMessage(message: String?): Builder {
                this.message = message
                return this
            }

            fun setMessage(message: Int): Builder {
                this.message = context.getText(message) as String
                return this
            }

            fun isDataEntry(isDataEntry: Boolean): Builder {
                this.isDataEntry = isDataEntry
                return this
            }

            fun dialogHeight(height: Int): Builder {
                this.dialogHeight = height
                return this
            }

            fun setDialogInitContainerDataListener(
                listener: OnDialogInitContainerDataListener
            ): Builder {
                this.dialogInitContainerDataListener = listener
                return this
            }

            fun setPositiveButton(
                positiveButtonTextId: Int = -1,
                listener: DialogInterface.OnClickListener?
            ): Builder {
                if (positiveButtonTextId > 0)
                    this.positiveButtonText = context
                        .getText(positiveButtonTextId) as String
                positiveButtonClickListener = listener
                return this
            }

            fun setPositiveButton(
                positiveButtonText: String?,
                listener: DialogInterface.OnClickListener?
            ): Builder {
                this.positiveButtonText = positiveButtonText
                positiveButtonClickListener = listener
                return this
            }

            fun setNegativeButton(
                negativeButtonTextId: Int = -1,
                listener: DialogInterface.OnClickListener?
            ): Builder {
                if (negativeButtonTextId > 0)
                    this.negativeButtonText = context
                        .getText(negativeButtonTextId) as String
                negativeButtonClickListener = listener
                return this
            }

            fun setNegativeButton(
                negativeButtonText: String?,
                listener: DialogInterface.OnClickListener?
            ): Builder {
                this.negativeButtonText = negativeButtonText
                negativeButtonClickListener = listener
                return this
            }

            fun create(): PublicContainerDataDialog {
                val dialog = PublicContainerDataDialog(
                    context, R.style.textview_dialog
                ).apply {
                    window?.let {
                        //解决边距问题
                        it.decorView.apply {
                            background = null
                            setPadding(0, 0, 0, 0)
                        }

                        val params: WindowManager.LayoutParams = it.attributes
                        params.gravity = Gravity.CENTER
                        params.width = DensityUtil.getDialogW(context)
                        if (dialogHeight > 0) {
                            params.height = dialogHeight
                        } else {
                            params.height = WindowManager.LayoutParams.WRAP_CONTENT
                        }
                        it.attributes = params
                    }
                }

                val layout: View =
                    LayoutInflater.from(context)
                        .inflate(R.layout.view_public_container_data_dialog, null)

                dialog.addContentView(
                    layout, ViewGroup.LayoutParams(
                        ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT
                    )
                )
                layout.findViewById<TextView>(R.id.public_dialog_title_tv).text = title

                layout.findViewById<TextView>(R.id.public_dialog_content_tv).apply {
                    text = message
                    if (message.notNullAndEmpty()) {
                        visibility = View.VISIBLE
                    } else {
                        visibility = View.GONE
                    }
                }

                layout.findViewById<Button>(R.id.public_dialog_cancel_btn)?.apply {
                    if (positiveButtonText.notNullAndEmpty()) {
                        text = positiveButtonText
                    }
                    if (positiveButtonClickListener != null) {
                        setOnClickListener {
                            positiveButtonClickListener?.onClick(
                                dialog,
                                BUTTON_POSITIVE
                            )
                        }
                    } else {
                        visibility = View.GONE
                    }
                }

                layout.findViewById<Button>(R.id.public_dialog_confirm_btn)?.apply {
                    if (negativeButtonText.notNullAndEmpty()) {
                        text = negativeButtonText
                    }

                    if (negativeButtonClickListener != null) {
                        setOnClickListener {
                            negativeButtonClickListener?.onClick(
                                dialog,
                                BUTTON_NEGATIVE
                            )
                        }
                    } else {
                        visibility = View.GONE
                    }
                }
                layout.findViewById<ImageView>(R.id.public_dialog_close_iv)
                    .setOnClickListener {
                        dialog.dismiss()
                    }
                if (isDataEntry) {
                    layout.findViewById<LinearLayout>(R.id.public_container_data_dialog_emtry_ll).visibility =
                        View.VISIBLE
                } else {
                    dialogInitContainerDataListener?.onContainerData(
                        dialog,
                        layout.findViewById<FrameLayout>(R.id.public_container_data_dialog_fl)
                            .apply {
                                visibility = View.VISIBLE
                            })
                }
                dialog.setContentView(layout)
                return dialog
            }
        }
    }

    interface OnDialogInitContainerDataListener {
        fun onContainerData(dialog: PublicContainerDataDialog, container: FrameLayout)
    }
}