package com.urovo.wugumofang.view.dialog;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Build;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.urovo.wugumofang.libutils.R;
import com.urovo.wugumofang.ui.DensityUtil;

/**
 * 输入弹窗
 */
public class PublicEditDialog extends Dialog {

    private PublicEditDialog(Context context) {
        super(context, R.style.textview_dialog);
    }

    public static class Builder {
        private Context context;
        private String title;
        private String editHint;
        private String positiveButtonText;
        private String negativeButtonText;
        private OnClickListener positiveButtonClickListener;
        private OnClickListener negativeButtonClickListener;
        private EditText mInputEt;

        public Builder(Context context) {
            this.context = context;
        }

        public Builder setTitle(int title) {
            this.title = (String) context.getText(title);
            return this;
        }

        public Builder setTitle(String title) {
            this.title = title;
            return this;
        }

        public Builder setEditHint(int hint) {
            this.editHint = (String) context.getString(hint);
            return this;
        }

        public Builder setEditHint(String hint) {
            this.editHint = hint;
            return this;
        }

        public String getEditContent() {
            return mInputEt.getText().toString();
        }

        public Builder setPositiveButton(int positiveButtonText,
                                         OnClickListener listener) {
            this.positiveButtonText = (String) context
                    .getText(positiveButtonText);
            this.positiveButtonClickListener = listener;
            return this;
        }

        public Builder setPositiveButton(OnClickListener listener) {
            this.positiveButtonText = "取消";
            this.positiveButtonClickListener = listener;
            return this;
        }

        public Builder setNegativeButton(int negativeButtonText,
                                         OnClickListener listener) {
            this.negativeButtonText = (String) context
                    .getText(negativeButtonText);
            this.negativeButtonClickListener = listener;
            return this;
        }

        public Builder setNegativeButton(OnClickListener listener) {
            this.negativeButtonText = "确定";
            this.negativeButtonClickListener = listener;
            return this;
        }

        public PublicEditDialog create() {
            final PublicEditDialog dialog = new PublicEditDialog(context);
            View layout = LayoutInflater.from(context).inflate(R.layout.view_public_edit_dialog, null);

            Window window = dialog.getWindow();
            if (window != null) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                    window.getDecorView().setBackground(null);
                }
                WindowManager.LayoutParams layoutParams = window.getAttributes();
                layoutParams.width = DensityUtil.getDialogW(context);
                layoutParams.height = LayoutParams.WRAP_CONTENT;
                window.setAttributes(layoutParams);
            }

            ((TextView) layout.findViewById(R.id.public_dialog_title_tv))
                    .setText(title);

            mInputEt = ((EditText) layout
                    .findViewById(R.id.public_dialog_input_et));
            mInputEt.setHint(editHint);

            if (positiveButtonText != null) {
                ((Button) layout.findViewById(R.id.public_dialog_cancel_btn))
                        .setText(positiveButtonText);
                if (positiveButtonClickListener != null) {
                    layout.findViewById(R.id.public_dialog_cancel_btn)
                            .setOnClickListener(v -> positiveButtonClickListener.onClick(dialog,
                                    DialogInterface.BUTTON_POSITIVE));
                }
            } else {
                layout.findViewById(R.id.public_dialog_cancel_btn).setVisibility(
                        View.GONE);
            }
            if (negativeButtonText != null) {
                ((Button) layout.findViewById(R.id.public_dialog_confirm_btn))
                        .setText(negativeButtonText);
                if (negativeButtonClickListener != null) {
                    layout.findViewById(R.id.public_dialog_confirm_btn)
                            .setOnClickListener(v -> negativeButtonClickListener.onClick(dialog,
                                    DialogInterface.BUTTON_NEGATIVE));
                }
            } else {
                layout.findViewById(R.id.public_dialog_confirm_btn).setVisibility(
                        View.GONE);
            }
            dialog.setContentView(layout);
            return dialog;
        }
    }
}