package com.urovo.wugumofang.view.dialog

import android.app.Dialog
import android.content.Context
import android.view.*
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.urovo.wugumofang.libutils.R

/**
 * 底部列表选择弹窗
 */
class PublicSelectBottomDialog(context: Context, theme: Int) : Dialog(context, theme) {

    init {
        //这句代码设置这个对话框是否能被用户触摸取消掉
        setCanceledOnTouchOutside(false)
        //这句代码设置这个对话框是否能被用户按[返回键]而取消掉
        setCancelable(false)
    }

    companion object {

        class Builder(private val context: Context) {
            private var title: String? = ""
            private var isDataEntry: Boolean = true
            private var dialogHeight: Int = -1
            private var publicDialogTitleTv: TextView? = null
            private var dialogListLoadDataListener: OnDialogListLoadDataListener? = null
            private var confirmClickListener: OnConfirmClickListener? = null

            fun setTitle(title: Int): Builder {
                this.title = context.getText(title) as String
                return this
            }

            fun setTitle(title: String?): Builder {
                this.title = title
                return this
            }

            fun isDataEntry(isDataEntry: Boolean): Builder {
                this.isDataEntry = isDataEntry
                return this
            }

            fun dialogHeight(height: Int): Builder {
                this.dialogHeight = height
                return this
            }

            fun setTitleView(gravity: Int, color: Int): Builder {
                publicDialogTitleTv?.apply {
                    setGravity(gravity)
                    setTextColor(color)
                }
                return this
            }

            fun setDialogListLoadDataListener(
                listener: OnDialogListLoadDataListener
            ): Builder {
                this.dialogListLoadDataListener = listener
                return this
            }

            fun setConfirmListener(listener: OnConfirmClickListener): Builder {
                this.confirmClickListener = listener
                return this
            }

            fun create(): PublicSelectBottomDialog {
                val dialog = PublicSelectBottomDialog(
                    context, R.style.bottom_default_dialog
                ).apply {
                    window?.let {
                        //解决边距问题
                        it.decorView.apply {
                            background = null
                            setPadding(0, 0, 0, 0)
                        }

                        val params: WindowManager.LayoutParams = it.attributes
                        params.gravity = Gravity.BOTTOM
                        params.width = WindowManager.LayoutParams.MATCH_PARENT
                        if (dialogHeight > 0) {
                            // DensityUtil.getScreenH(context) - DensityUtil.dip2px(context, 120f)
                            params.height = dialogHeight
                        } else {
                            params.height = WindowManager.LayoutParams.WRAP_CONTENT
                        }
                        it.attributes = params
                    }
                }

                val layout: View =
                    LayoutInflater.from(context).inflate(R.layout.view_public_select_bottom_dialog, null)

                dialog.addContentView(
                    layout, ViewGroup.LayoutParams(
                        ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT
                    )
                )
                publicDialogTitleTv =
                    layout.findViewById<TextView>(R.id.public_dialog_title_tv).apply {
                        text = title
                    }

                layout.findViewById<ImageView>(R.id.public_dialog_close_iv)
                    .setOnClickListener {
                        dialog.dismiss()
                        confirmClickListener?.onClick(dialog)
                    }
                if (isDataEntry) {
                    layout.findViewById<LinearLayout>(R.id.public_select_bottom_dialog_emtry_ll).visibility =
                        View.VISIBLE
                } else {
                    val list = layout.findViewById<RecyclerView>(R.id.public_select_bottom_dialog_list_rv)
                    list.visibility = View.VISIBLE
                    dialogListLoadDataListener?.onListLoadData(dialog, list)
                }
                dialog.setContentView(layout)
                return dialog
            }
        }
    }

    interface OnDialogListLoadDataListener {
        fun onListLoadData(dialog: PublicSelectBottomDialog, list: RecyclerView)
    }

    interface OnConfirmClickListener {
        fun onClick(dialog: PublicSelectBottomDialog)
    }
}