package com.urovo.wugumofang.view.webview;

import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.net.http.SslError;
import android.webkit.SslErrorHandler;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import com.zsy.libext.widgets.webview.CustomWebView;

/**
 * Convenient extension of WebViewClient.
 * @author ...
 */
public class CustomWebViewClient extends WebViewClient {

    private BaseWebFra baseWebFra;
    /**
     * 当前加载的url
     */
    private String currentWebUrl = "";

    public CustomWebViewClient(BaseWebFra baseWebViewFra) {
        super();
        baseWebFra = baseWebViewFra;
    }

    @Override
    public void onPageFinished(WebView view, String url) {
        ((CustomWebView) view).notifyPageFinished();
        baseWebFra.onPageFinished(view, url);

        super.onPageFinished(view, url);
    }

    @Override
    public void onPageStarted(WebView view, String url, Bitmap favicon) {
        ((CustomWebView) view).notifyPageStarted();
        baseWebFra.onPageStarted(view, url);
        super.onPageStarted(view, url, favicon);
    }

    @Override
    public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
        super.onReceivedError(view, errorCode, description, failingUrl);
        view.stopLoading();
        view.clearView();
        baseWebFra.onReceivedError();
    }
// https://stackoverflow.com/questions/5882656/no-activity-found-to-handle-intent-android-intent-action-view --No.21
    @Override
    public boolean shouldOverrideUrlLoading(WebView view, String url) {
        if (url != null) {
            currentWebUrl = url.trim();
            ((CustomWebView) view).resetLoadedUrl();
            // 控制加载url
            if ((currentWebUrl.startsWith("http://") || currentWebUrl.startsWith("https://"))) {
                view.loadUrl(currentWebUrl);
                baseWebFra.shouldOverrideUrlLoading(view, currentWebUrl);
                return false;
            }
            // Otherwise allow the OS to handle things like tel, mailto, etc.
            try {
                Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(currentWebUrl));
                if (
                        baseWebFra != null
                                && baseWebFra.getActivity() != null
                                && intent.resolveActivity(baseWebFra.getActivity().getPackageManager()) != null) {
                    baseWebFra.getActivity().startActivity(intent);
                }
            } catch (ActivityNotFoundException e) {
                e.printStackTrace();
            }
        }
        return true;
    }

    @Override
    public void onReceivedSslError(WebView view, SslErrorHandler handler, SslError error) {
        if(error.getPrimaryError() == android.net.http.SslError.SSL_INVALID ){// 校验过程遇到了bug（安卓7.0可能会遇到WebView打不开的情况，所以需要允许这种情况继续执行）
            handler.proceed();
        }else{
            handler.cancel();
        }
//        handler.proceed();
    }

    public String getCurrentWebUrl() {
        return currentWebUrl;
    }
}
