package com.urovo.wugumofang.view.webview;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;

import com.zsy.libcore.ui.fra.FragmentArgs;
import com.zsy.libext.R;
import com.zsy.libext.ui.act.BaseAct;

import java.lang.reflect.Method;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

/**
 * Fragment容器Activity;
 * 现在目前还有点问题，是通过这种嵌套Fragment，
 * 现只能通过Serializable来实现，而不能通过Parcelable
 *
 * @author samy(hongfeiliuxing @ gmail.com)
 * @datetime 2015-09-06 23:35
 */
public class FraContainerAct extends BaseAct {
    public static final String FRAGMENT_TAG = "FRAGMENT_CONTAINER";
    public static final String IS_HIDE_TOOLBAR = "isHideToolbar";
    public static final String CLASS_NAME = "CLASS_NAME";
    public static final String ARGS = "ARGS";
    private int overrideTheme = -1;
    private boolean isHideToolbar = false;

    public static void launchWithFinish(Context context, Class<? extends Fragment> clazz, FragmentArgs args, boolean isHideToobar) {
        launch(context, clazz, args, isHideToobar);
        ((Activity) context).finish();
    }

    public static void launch(Context context, Class<? extends Fragment> clazz, FragmentArgs args, boolean isHideToobar) {
        Intent intent = new Intent(context, FraContainerAct.class);
        intent.putExtra(IS_HIDE_TOOLBAR, isHideToobar);
        intent.putExtra(CLASS_NAME, clazz.getName());
        if (args != null) {
            intent.putExtra(ARGS, args);
        }
        context.startActivity(intent);
    }

    public static void launchForResult(BaseAct from, Class<? extends Fragment> clazz, FragmentArgs args, int requestCode) {
        Intent intent = new Intent(from, FraContainerAct.class);
        intent.putExtra(CLASS_NAME, clazz.getName());
        if (args != null) {
            intent.putExtra(ARGS, args);
        }
        from.startActivityForResult(intent, requestCode);
    }

    @Override
    protected void getBundleExtras(Bundle extras) {
        super.getBundleExtras(extras);
        isHideToolbar = extras.getBoolean(IS_HIDE_TOOLBAR);
    }

//    @Override
//    public void onDestroy() {
//        super.onDestroy();
//        SYLoger.print("FraContainerAct::onDestory()");
//    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    @Override
    protected void onCreate(Bundle savedInstanceState) {
//        SYLoger.print("FraContainerAct::onCreate()");
        String className = getIntent().getStringExtra(CLASS_NAME);
        if (TextUtils.isEmpty(className)) {
            finish();
            return;
        }

        // int contentId = R.layout.common_activity_fragment_container;
        // 后期做成,可以控制是否要头部,跟BaseFraWithTitle处理配合
        // 还得考虑头部阴影效果
        // int contentId = android.R.id.content;

        FragmentArgs values = (FragmentArgs) getIntent().getSerializableExtra(ARGS);
        // FragmentArgs values = (FragmentArgs) getIntent().getParcelableExtra(ARGS);

        Fragment fragment = null;
        if (savedInstanceState == null) {
            try {
                Class clazz = Class.forName(className);
                fragment = (Fragment) clazz.newInstance();
                // 设置参数给Fragment
                if (values != null) {
                    try {
                        Method method = clazz.getMethod("setArguments", Bundle.class);
                        method.invoke(fragment, FragmentArgs.transToBundle(values));
                    } catch (Exception e) {
// ...
                    }
                }
                // 重写Activity的主题
//				try {
//					Method method = clazz.getMethod("setTheme");
//					if (method != null) overrideTheme = Integer.parseInt(method.invoke(fragment).toString());
//				}
//				catch (Exception e) {
//				}
                // 重写Activity的contentView
                // try {
                // Method method = clazz.getMethod("setActivityContentView");
                // if (method != null)
                // contentId = Integer.parseInt(method.invoke(fragment).toString());
                // } catch (Exception e) {

                // }
            } catch (Exception e) {
                e.printStackTrace();
                finish();
                return;
            }
        }
        super.onCreate(savedInstanceState);
        if (isHideToolbar) {
            commonToolbar.setVisibility(View.GONE);
        }
        // setContentView(contentId);
        if (fragment != null) {
            FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
            transaction.replace(R.id.container_fl, fragment, FRAGMENT_TAG)
                    .addToBackStack("MemberHomeFra")
                    .commit();
        }
        // if (getSupportActionBar() != null)
        // getSupportActionBar().setDisplayShowHomeEnabled(false);

    }

    @Override
    protected int getContentViewLayoutID() {
        return R.layout.act_fra_container;
    }

    @Override
    protected void initWidgetsAndEvents(Bundle savedInstanceState) {

    }

    @Override
    public void onBackPressed() {
        //在回退栈为0的时候，要finish掉这个窗口，否则按一次返回时，还会停留在一个空白窗口页。
        if(getFragmentManager().getBackStackEntryCount() == 0){
            finish();
        }
        super.onBackPressed();
    }
}
