package com.urovo.wugumofang.view.webview;

import android.content.DialogInterface;
import android.os.Bundle;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.Toolbar;

import android.view.KeyEvent;
import android.view.View;
import android.webkit.WebView;
import android.widget.Button;
import android.widget.TextView;

import com.urovo.wugumofang.ui.ShareDominator;
import com.zsy.libext.R;
import com.zsy.libext.ui.act.BaseAct;

/**
 * 处理复杂的web界面(H5内多个界面交互)
 *
 * @author samy(hongfeiliuxing @ gmail.com)
 * @date 2015-08-22 20:19
 */
public class WebContainerAct extends BaseAct {
    private String webUrl;
    private String webTitle;
    private boolean isShowBottom;
    private boolean isShowShareBtn;//是否显示顶栏右上角分享按钮

    BaseWebFra baseWebFra;
    WebView webView;
    TextView common_web_toolbar_title_tv;
    Button common_web_toolbar_share_btn;//分享按钮
    ShareDominator shareDominator;

    @Override
    protected void getBundleExtras(Bundle extras) {
        super.getBundleExtras(extras);

        webUrl = extras.getString("url");
        webTitle = extras.getString("title");
        isShowBottom = extras.getBoolean("isShowBottom");
        isShowShareBtn = extras.getBoolean("isShowShareBtn", false);

    }

    @Override
    protected void initWidgetsAndEvents(Bundle savedInstanceState) {
        setTitle("");

        shareDominator = new ShareDominator(this,webUrl,"");
        commonToolbar = (Toolbar) findViewById(R.id.common_web_toolbar);
        common_web_toolbar_title_tv = (TextView) findViewById(R.id.common_web_toolbar_title_tv);
        //标题栏右上角分享按钮
        common_web_toolbar_share_btn = (Button) findViewById(R.id.common_web_toolbar_share_btn);
        common_web_toolbar_share_btn.setVisibility(isShowShareBtn ? View.VISIBLE : View.GONE);
        common_web_toolbar_share_btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                String webTitle = common_web_toolbar_title_tv.getText().toString();
                //要先show出来了才能有实例来设置属性
                shareDominator.showDialog().setShareBtns(
                        ShareDominator.CIRCLE_WX, ShareDominator.FRIENDS_WX, ShareDominator.COPY_LINK
                ).isShowHeadLine(false).setWebTitle(webTitle);//不用显示“分享到”文字两边的线条
            }
        });

        updateWebTitle(webTitle);

        baseWebFra = new BaseWebFra();
        baseWebFra.setArguments(BaseWebFra.webContainerBundle(webUrl, webTitle, isShowBottom));
        changeFragment(R.id.container_fl, baseWebFra, true);

        commonToolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                backOperWeb();
            }
        });

        findViewById(R.id.common_web_toolbar_left_iv).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showTips("", "是否关闭当前界面?", true);
            }
        });
    }

    //注意这里的获取时间
    @Override
    protected void onStart() {
        super.onStart();
        //应要携带数据，没有用这种方式处理；
        //BaseWebFra fragment =
        //    (BaseWebFra) getSupportFragmentManager().findFragmentById(R.id.base_web_fra);
        webView = baseWebFra.getWebView();
    }

    public void updateWebTitle(String title) {
        if (common_web_toolbar_title_tv != null && title != null) {
            common_web_toolbar_title_tv.setText(title);
        }
    }

    @Override
    protected int getContentViewLayoutID() {
        return R.layout.act_web_container;
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (event.getAction() == KeyEvent.ACTION_DOWN) {
            switch (keyCode) {
                case KeyEvent.KEYCODE_BACK:
                    backOperWeb();
                    return true;
                default:
                    break;
            }
        }
        return super.onKeyDown(keyCode, event);
    }

    private void backOperWeb() {
        if (webView.canGoBack()) { //目前先这样固定式
            showTips("startTest.html", "是否关闭当前界面?", false);
        } else {
            showTips("", "是否关闭当前界面?", true);
        }
    }

    /**
     * 直接点击关闭结束或者关闭特殊界面的Url
     * 是否结束当前界面提示
     */
    private void showTips(String containCharPath, String tipsMsg, final boolean isFinish) {
        if ((!isFinish && baseWebFra.getWebUrl().contains(containCharPath)) || isFinish) {
            new AlertDialog.Builder(this).setMessage(tipsMsg)
                    .setNegativeButton("取消", null)
                    .setPositiveButton("确定", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            dialog.dismiss();
                            if (isFinish) {
                                finish();
                            } else {
                                webView.goBack();
                            }
                        }
                    })
                    .show();
        } else {
            webView.goBack();
        }
    }

    //public interface WebCallBack {
    //  /**webView控件*/
    //  void getWebView(WebView webView);
    //  /**当前界面webUrl*/
    //  String getCurrentWebUrl(String webUrl);
    //}
}
