package com.zsy.libcore.common.other;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import androidx.core.content.FileProvider;

import java.io.File;
import java.security.MessageDigest;

/**
 * @author samy
 * @Description:包工具类
 * @date 2015-8-4 下午2:34:29
 */
public class PackageUtil {

    public static PackageInfo getPackageInfo(Context context) {
        PackageInfo packageInfo = null;
        try {
            packageInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
        } catch (NameNotFoundException e) {
//			e.printStackTrace();
            throw new SYException(PackageUtil.class.getName() + "the application not found");
        }
        return packageInfo;

    }

    /**
     * 获取当前应用程序的包名
     */
    public static String getAppPackageName(Context context) {
        String packageName = "0";
        try {
            packageName = context.getPackageManager().getPackageInfo(context.getPackageName(), 0).packageName;
        } catch (NameNotFoundException e) {
            throw new SYException(PackageUtil.class.getName() + "the application not found");
        }
        return packageName;
    }

    /**
     * 获取当前应用程序的版本号
     */
    public static String getAppVersion(Context context) {
        String version = "0";
        try {
            version = context.getPackageManager().getPackageInfo(context.getPackageName(), 0).versionName;
        } catch (NameNotFoundException e) {
            throw new SYException(PackageUtil.class.getName() + "the application not found");
        }
        return version;
    }

    /**
     * 获取应用签名
     *
     * @param context
     * @param pkgName
     */
    public static String getSign(Context context, String pkgName) {
        try {
            PackageInfo pis = context.getPackageManager().getPackageInfo(pkgName, PackageManager.GET_SIGNATURES);
            return hexdigest(pis.signatures[0].toByteArray());
        } catch (NameNotFoundException e) {
            throw new SYException(PackageUtil.class.getName() + "the " + pkgName + "'s application not found");
        }
    }

    /**
     * 判断是否已经安装APK
     */
    public static boolean checkApkExist(Context context, String pkgName) {
        if (pkgName == null || "".equals(pkgName)) return false;
        try {
            ApplicationInfo info = context.getPackageManager().getApplicationInfo(pkgName, PackageManager.GET_UNINSTALLED_PACKAGES);
            return true;
        } catch (NameNotFoundException e) {
            throw new SYException(PackageUtil.class.getName() + "the " + pkgName + "'s application not found");
        }
    }

    /**
     * 安装apk
     *
     * @param context
     * @param file
     */
    public static void installApk(Context context, File file) {
        Intent intent = new Intent();
        intent.setAction("android.intent.action.VIEW");
        intent.addCategory("android.intent.category.DEFAULT");
        intent.setType("application/vnd.android.package-archive");
//        intent.setData(Uri.fromFile(file));
        intent.setData(FileProvider.getUriForFile(context, "com.urovo.wugumofang.fileprovider",file));
//        intent.setDataAndType(Uri.fromFile(file), "application/vnd.android.package-archive");
        intent.setDataAndType(FileProvider.getUriForFile(context, "com.urovo.wugumofang.fileprovider",file), "application/vnd.android.package-archive");
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(intent);
    }

    /**
     * 将签名字符串转换成需要的32位签名
     */
    private static String hexdigest(byte[] paramArrayOfByte) {
        final char[] hexDigits = {48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 97, 98, 99, 100, 101, 102};
        try {
            MessageDigest localMessageDigest = MessageDigest.getInstance("MD5");
            localMessageDigest.update(paramArrayOfByte);
            byte[] arrayOfByte = localMessageDigest.digest();
            char[] arrayOfChar = new char[32];
            for (int i = 0, j = 0; ; i++, j++) {
                if (i >= 16) {
                    return new String(arrayOfChar);
                }
                int k = arrayOfByte[i];
                arrayOfChar[j] = hexDigits[(0xF & k >>> 4)];
                arrayOfChar[++j] = hexDigits[(k & 0xF)];
            }
        } catch (Exception e) {
        }
        return "";
    }
}
