package com.zsy.libcore.presenter.impl;

import androidx.annotation.NonNull;
import android.view.View;
import com.zsy.libcore.presenter.BasePresenter;
import com.zsy.libcore.view.BaseView;

/**
 *
 *
 * @author samy
 * @date 16/4/13
 */
public abstract  class BasePresenterImpl implements BasePresenter {
  /**是否是第一次加载，一般会跟start对应上*/
  protected boolean mFirstLoad = true;

  private BaseView mMvpView;

  public BasePresenterImpl(@NonNull BaseView mMvpView) {
    if (mMvpView!=null){
      this.mMvpView = mMvpView;
      mMvpView.setPresenter(this);
    }
  }

  protected BaseView getMvpView(){
    if (mMvpView!=null){
      return mMvpView;
    }else {
      throw new BasePresenterImpl.MvpViewNotAttachedException();
    }
  }

  protected void detachView() {
    this.mMvpView = null;
  }

  protected void showPresenterError(String msgError, View.OnClickListener onClickListener) {
    this.checkViewAttached();
    this.mMvpView.hideLoading();
    this.mMvpView.showError(msgError, onClickListener);
  }

  /**
   * 如果在Fragment中可以用BaseAppCompatView isActive方法判断处理；其他情况下用isViewAttached处理；
   * @return ...
   */
  protected boolean isViewAttached() {
    return this.mMvpView != null;
  }

  protected void checkViewAttached() {
    if(!this.isViewAttached()) {
      throw new BasePresenterImpl.MvpViewNotAttachedException();
    }
  }

  protected static class MvpViewNotAttachedException extends RuntimeException {
    public MvpViewNotAttachedException() {
      super("Please call BasePresenter.attachView(MvpView) before requesting data to the Presenter");
    }
  }

}
