package com.zsy.libcore.ui.act;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.WindowManager;

import com.google.android.material.snackbar.Snackbar;
import com.umeng.analytics.MobclickAgent;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libcore.event.BaseCompatActEvent;

import org.greenrobot.eventbus.EventBus;

import androidx.appcompat.app.AppCompatActivity;

/**
 * 数据访问层（DAL）
 * 业务逻辑层（BLL）
 * 表现层（UI）
 * <p/>
 * Act第一层
 * 控制屏幕翻转，和Activity生命周期打印调试
 *
 * @author samy(hongfeiliuxing@gmail.com)
 * @datetime 2015-09-01 16:19
 */
public abstract class BaseAppCompatAct extends AppCompatActivity {
	protected static String TAG_LOG = null;
	/** Activity状态 */
	public ActivityState activityState = ActivityState.DESTROY;
	protected Context mContext = null;
	protected LayoutInflater mInflater;
	/** 屏幕方向 */
	private ScreenOrientation orientation = ScreenOrientation.VERTICAL;
	/** 是否允许全屏 */
	private boolean mAllowFullScreen = false;
	/** 是否隐藏ActionBar */
	private boolean mHiddenActionBar = false;

	/** 是否全屏显示本Activity，全屏后将隐藏状态栏，默认不全屏（若修改必须在构造方法中调用） */
	public void setAllowFullScreen(boolean allowFullScreen) {
		this.mAllowFullScreen = allowFullScreen;
	}

	/** 是否隐藏ActionBar，默认隐藏（若修改必须在构造方法中调用） */
	public void setHiddenActionBar(boolean hiddenActionBar) {
		this.mHiddenActionBar = hiddenActionBar;
	}

	/** 修改屏幕显示方向，默认竖屏锁定（若修改必须在构造方法中调用） */
	public void setScreenOrientation(ScreenOrientation orientation) {
		this.orientation = orientation;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		activityState = ActivityState.CREATE;
		TAG_LOG = this.getClass().getSimpleName();
		SYLoger.state(TAG_LOG, "=====================" + activityState.toString());
		SYActManager.getInstance().addActivity(this);
		mContext = this;
		mInflater = LayoutInflater.from(mContext);
		// 初始化进入样式
		initWindow(savedInstanceState);
		super.onCreate(savedInstanceState);
		Bundle extras = getIntent().getExtras();
		if (null != extras) {
			getBundleExtras(extras);
		}

		if (isBindEventBusHere()) {
			EventBus.getDefault().register(this);
		}

		if (getContentViewLayoutID() != 0) {
			setContentView(getContentViewLayoutID());
		}
		else {
			throw new IllegalArgumentException("You must return a right contentView layout resource Id");
		}
	}

	@Override
	protected void onRestart() {
		super.onRestart();
		activityState = ActivityState.RESTART;
		SYLoger.state(TAG_LOG, "=====================" + activityState);
	}

	@Override
	protected void onStart() {
		super.onStart();
		activityState = ActivityState.RESTART;
		SYLoger.state(TAG_LOG, "=====================" + activityState);
		MobclickAgent.updateOnlineConfig(this);
	}

	@Override
	protected void onResume() {
		super.onResume();
		activityState = ActivityState.RESUME;
		SYLoger.state(TAG_LOG, "=====================" + activityState);
		MobclickAgent.onResume(this);
	}

	@Override
	protected void onPause() {
		super.onPause();
		activityState = ActivityState.PAUSE;
		SYLoger.state(TAG_LOG, "=====================" + activityState);
		MobclickAgent.onPause(this);
	}

	@Override
	protected void onStop() {
		super.onStop();
		activityState = ActivityState.STOP;
		SYLoger.state(TAG_LOG, "=====================" + activityState);
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		activityState = ActivityState.DESTROY;
		SYLoger.state(TAG_LOG, "=====================" + activityState);
		if (isBindEventBusHere()) {
			EventBus.getDefault().unregister(this);
		}
		SYActManager.getInstance().finishActivity(this);
	}

	protected void initWindow(Bundle savedInstanceState) {
		switch (orientation) {
			case HORIZONTAL:
				setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
				break;
			case VERTICAL:
				setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
				break;
			case AUTO:
				setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_USER);
				break;
		}
		// 这里设置不要跟配置文件中的冲突,现在没有什么用（有Toolbar在这里）
		// if (mHiddenActionBar) {
		// requestWindowFeature(Window.FEATURE_NO_TITLE);
		// } else {
		// ActionBar a = getActionBar();
		// a.show();
		// }
		if (mAllowFullScreen) {
			getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
		}
	}

	protected void showToast(String msg) {
		if (!TextUtils.isEmpty(msg)) {
			Snackbar.make(getWindow().getDecorView(), msg, Snackbar.LENGTH_SHORT).show();
		}
	}

	public void onEventMainThread(BaseCompatActEvent baseCompatActEvent) {
		if (null != baseCompatActEvent) {
			onEventBaseCompatAct(baseCompatActEvent);
		}
	}

	// protected abstract boolean isBindEventBusHere();
	// protected abstract void onEventComming(BaseAppCompatEvent baseAppCompatEvent);
	protected boolean isBindEventBusHere() {
		return false;
	}

	protected void onEventBaseCompatAct(BaseCompatActEvent baseCompatActEvent) {
	}

	// protected abstract void getBundleExtras(Bundle extras);
	protected void getBundleExtras(Bundle extras) {
	}

	protected abstract int getContentViewLayoutID();

	/**
	 * 当前Activity状态
	 */
	public static enum ActivityState {
		CREATE, RESTART, START, RESUME, PAUSE, STOP, DESTROY
	}

	/**
	 * Activity显示方向
	 */
	public static enum ScreenOrientation {
		HORIZONTAL, VERTICAL, AUTO
	}
}