package com.zsy.libcore.ui.ada;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import java.util.ArrayList;
import java.util.List;

/**
 * @author samy
 * @description：普通适配器基类 如搜索过滤功能直接到继承这个类后实现Filterable，
 * 再次实现Filterable里面的方法；getFilter();
 * @date 2015-3-24 下午4:22:56
 */
// public abstract class BaseAda<T> extends BaseAdapter implements Filterable {
public abstract class BaseAda<E> extends BaseAdapter {
    public List<E> mDataSet;
    protected Context mContext;
    protected LayoutInflater mInflater;
    protected BaseAdaFilter<E> baseAdaFilter;
    private int mLayoutId;

    public BaseAda(Context context, List<E> dataSet, int layoutId) {
        super();
        this.mContext = context;
        this.mLayoutId = layoutId;
        this.mDataSet = dataSet;
        mInflater = LayoutInflater.from(mContext);
    }

    // @Override
    // public Filter getFilter() {
    // if (null == baseAdaFilter) {
    // baseAdaFilter = new BaseAdaFilter<T>(this);
    // }
    // return baseAdaFilter;
    // }

    // @Override
    // public View getView(int position, View convertView, ViewGroup parent) {
    // final ViewHolder viewHolder;
    // if (convertView == null) {
    // convertView = mInflater.inflate(getItemLayout(), parent, false);
    // viewHolder = new ViewHolder(convertView);
    // convertView.setTag(viewHolder);
    // }
    // else {
    // viewHolder = (ViewHolder) convertView.getTag();
    // }
    // refreshItemData(viewHolder, position);
    // return convertView;
    // }

    @Override
    public int getCount() {
        return (mDataSet == null) ? 0 : mDataSet.size();
    }

    @Override
    public E getItem(int position) {
        //防止外部调用时越界
        if (mDataSet == null || position < 0 || position > mDataSet.size()) {
            return null;
        }
        return mDataSet.get(position);
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        // final ViewHolder vh = ViewHolder.getViewHolder(this.mContext, convertView, parent, this.mLayoutId, position);
        final ViewHolder vh = getViewHodler(convertView, parent, position);
        bindDataToItemView(vh, getItem(position));
        return vh.getConvertView();
    }

    // 获取ViewHodler
    public ViewHolder getViewHodler(View convertView, ViewGroup parent, int position) {
        return ViewHolder.getViewHolder(this.mContext, convertView, parent, this.mLayoutId, position);
    }

    public abstract void bindDataToItemView(ViewHolder vh, E item);

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public boolean isEmpty() {
        return mDataSet == null || mDataSet.isEmpty();
    }

    public void setList(List<E> g) {
        if (mDataSet != null) {
            mDataSet.clear();
        }
        mDataSet.addAll(g);
        notifyDataSetChanged();
    }

    public void addItems(List<E> items) {
        if (items != null) {
            mDataSet.addAll(items);
            notifyDataSetChanged();
        }
    }

    public void addItem(E item) {
        mDataSet.add(item);
        notifyDataSetChanged();
    }

    public void addItemNoNotify(E item) {
        mDataSet.add(item);
    }

    public void clearList(boolean notify) {
        if (mDataSet != null) {
            mDataSet.clear();
            if (notify) {
                notifyDataSetChanged();
            }
        }
    }

    public void removeItem(int position) {
        if (mDataSet == null || position < 0 || position > mDataSet.size()) {
            return;
        }
        mDataSet.remove(position);
        notifyDataSetChanged();
        return;
    }

    public void checkListNull() {
        if (mDataSet == null) {
            mDataSet = new ArrayList<E>();
        }
    }
}
