package com.zsy.libcore.ui.fra;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.umeng.analytics.MobclickAgent;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libcore.event.BaseCompatFraEvent;

import org.greenrobot.eventbus.EventBus;

import java.lang.reflect.Field;

import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

/**
 * 数据访问层（DAL）
 * 业务逻辑层（BLL）
 * 表现层（UI）
 * <p/>
 * Fra第一层
 * 与Activity交互管理，和Fragment生命周期打印调试
 * <p>
 * <p/>
 * 懒加载Fragment; 后面有时间处理这个加载方式
 * 只有创建并显示的时候才会调用onCreateViewLazy方法<br>
 * <p/>
 * 懒加载的原理onCreateView的时候Fragment有可能没有显示出来。
 * 但是调用到setUserVisibleHint(boolean isVisibleToUser),isVisibleToUser = true的时候就说明有显示出来
 * 但是要考虑onCreateView和setUserVisibleHint的先后问题所以才有了下面的代码
 * <p/>
 * 注意：
 * 《1》原先的Fragment的回调方法名字后面要加个Lazy，比如Fragment的onCreateView方法， 就写成onCreateViewLazy
 * 《2》使用该LazyFragment会导致多一层布局深度
 *
 * @author: samy(hongfeiliuxing @ gmail.com)
 * @datetime: 2015-09-04 10:36
 */
public abstract class BaseAppCompatFra extends Fragment {
    protected static String TAG_LOG;
    /**
     * Activity状态
     */
    public FragmentState fragmentState = FragmentState.DETACH;
    protected Context mContext;
    protected View rootView;
    private boolean isPrepared;
    /**
     * 第一次onResume中的调用onUserVisible避免操作与onFirstUserVisible操作重复
     */
    private boolean isFirstResume = true;
    private boolean isFirstVisible = true;
    private boolean isFirstInvisible = true;

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        fragmentState = FragmentState.ATTACH;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
        mContext = activity;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
// 这里发现V4Fragment跟Fragement有很大的区别
// mContext = getActivity();
        fragmentState = FragmentState.CREATE;
        TAG_LOG = this.getClass().getSimpleName();
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);

        Bundle extras = getArguments();
        if (null != extras) {
            getBundleExtras(extras);
        }

        if (isBindEventBusHere()) {
            EventBus.getDefault().register(this);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        fragmentState = FragmentState.CREATE_VIEW;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
        if (getContentViewLayoutID() > 0) {
//            if (rootView != null) {
//                ViewGroup parent = (ViewGroup) rootView.getParent();
//                if (null != parent) {
//                    parent.removeView(rootView);
//                }
//            } else {
                rootView = inflater.inflate(getContentViewLayoutID(), container, false);
                rootView.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
//            }
            return rootView;
        }
        return super.onCreateView(inflater, container, savedInstanceState);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        fragmentState = FragmentState.VIEW_CREATE;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        fragmentState = FragmentState.ACTIVITY_CREATE;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
        initPrepare();
    }
    @Override
    public void onStart() {
        super.onStart();
        fragmentState = FragmentState.START;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
    }

    @Override
    public void onResume() {
        super.onResume();
        fragmentState = FragmentState.RESUME;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
        MobclickAgent.onPageStart(TAG_LOG);
        if (isFirstResume) {
            isFirstResume = false;
            return;
        }
        if (getUserVisibleHint()) {
            onUserVisible();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        fragmentState = FragmentState.PAUSE;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
        MobclickAgent.onPageEnd(TAG_LOG);
        if (getUserVisibleHint()) {
            onUserInvisible();
        }
    }

    @Override
    public void onStop() {
        super.onStop();
        fragmentState = FragmentState.STOP;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        fragmentState = FragmentState.DESTORY_VIEW;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        fragmentState = FragmentState.DESTROY;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
        if (isBindEventBusHere()) {
            EventBus.getDefault().unregister(this);
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        fragmentState = FragmentState.DETACH;
        SYLoger.state(TAG_LOG, "=====================" + fragmentState);
// for bug ---> java.lang.IllegalStateException: Activity has been destroyed
        try {
            Field childFragmentManager = Fragment.class.getDeclaredField("mChildFragmentManager");
            childFragmentManager.setAccessible(true);
            childFragmentManager.set(this, null);
        } catch (NoSuchFieldException e) {
            throw new RuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        if (isVisibleToUser) {
            if (isFirstVisible) {
                isFirstVisible = false;
                initPrepare();
            } else {
                onUserVisible();
            }
        } else {
            if (isFirstInvisible) {
                isFirstInvisible = false;
                onFirstUserInvisible();
            } else {
                onUserInvisible();
            }
        }
    }

    private synchronized void initPrepare() {
        if (isPrepared) {
            onFirstUserVisible();
        } else {
            isPrepared = true;
        }
    }

    public void onEventMainThread(BaseCompatFraEvent baseCompatFraEvent) {
        if (null != baseCompatFraEvent) {
            onEventBaseCompatFra(baseCompatFraEvent);
        }
    }

    protected boolean isBindEventBusHere() {
        return false;
    }

    protected void onEventBaseCompatFra(BaseCompatFraEvent baseCompatFraEvent) {
    }

    /**
     * 第一次fragment可见（进行初始化工作）
     * when fragment is visible for the first time, here we can do some initialized work or refresh
     * data only once
     */
    protected void onFirstUserVisible() {
    }

    /**
     * fragment可见（切换回来或者onResume）
     */
    protected void onUserVisible() {
    }

    /**
     * fragment不可见（切换掉或者onPause）
     */
    protected void onUserInvisible() {
    }

    /**
     * 第一次fragment不可见（不建议在此处理事件）
     */
    protected void onFirstUserInvisible() {
    }

    protected void getBundleExtras(Bundle extras) {
    }

    /**
     * fragment 内容布局
     *
     * @return ...
     */
    protected abstract int getContentViewLayoutID();

    /**
     * 当前Fragment状态
     */
    public static enum FragmentState {
        ATTACH, CREATE, CREATE_VIEW, VIEW_CREATE, ACTIVITY_CREATE, START, RESUME, PAUSE, STOP, DESTORY_VIEW, DESTROY, DETACH
    }
}
