package com.zsy.libext.ui.act;

import android.annotation.TargetApi;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;

import com.zsy.libcore.ui.act.BaseAppCompatAct;
import com.zsy.libcore.widgets.bar.SystemBarTintManager;
import com.zsy.libext.R;
import com.zsy.libext.model.net.status.NetChangeObserver;
import com.zsy.libext.model.net.status.NetStateReceiver;
import com.zsy.libext.model.net.status.NetUtils;
import com.zsy.libext.model.perfer.LibExtPrefer;
import com.zsy.libext.ui.fra.BaseFra;
import com.zsy.libext.utils.ToolbarUtils;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;

import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

/**
 * Act第二层
 * Actionbar、主题样式、进入跳转样式动画,Toolbar，遮盖物
 *
 * @author samy(hongfeiliuxing @ gmail.com)
 * @datetime 2015-08-22 20:19
 */
public abstract class BaseAct extends BaseAppCompatAct implements View.OnClickListener {
    protected BaseAct baseAct;
    // @InjectView(R.id.common_toolbar)
    public Toolbar commonToolbar;
    protected NetChangeObserver mNetChangeObserver = null;
    public FragmentManager mFragmentManager;
    protected int mFrgmContainer;
    // 当有Fragment Attach到这个Activity的时候，就会保存
    private Map<String, WeakReference<BaseAct>> fragmentRefs;

    public enum TransitionMode {
        NONE, LEFT, RIGHT, TOP, BOTTOM, SCALE, FADE
    }

//    Unbinder unbinder;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        overridePendingTransition();
        baseAct = this;
        // ButterKnife.inject(this);
//        unbinder = ButterKnife.bind(this);
//		initPresenter();
        // //这里默认的处理了，如果发现布局没有ToolBar情况，不会报错；
        commonToolbar = findViewById( R.id.common_toolbar);
        initToolbar(commonToolbar);
        initFragment();

        initWidgetsAndEvents(savedInstanceState);

        mNetChangeObserver = new NetChangeObserver() {
            @Override
            public void onNetConnected(NetUtils.NetType type) {
                super.onNetConnected(type);
                // 后期结合Toolbar设置提示设置网络,还有snackbar也可以
                onNetworkConnected(type);
            }

            @Override
            public void onNetDisConnect() {
                super.onNetDisConnect();
                onNetworkDisConnected();
            }
        };
        NetStateReceiver.registerObserver(mNetChangeObserver);
    }

    /**
     * 注意这里可以设置钩子
     */
    @Override
    protected void initWindow(Bundle savedInstanceState) {
        super.initWindow(savedInstanceState);
        initWindowBar();
    }

    private void initWindowBar() {
        // SmartBarUtils.hide(getWindow().getDecorView());
        // 状态栏透明 需要在创建SystemBarTintManager 之前调用。
        // 状态栏透明可能导致布局向上便宜，所以需要设置根布局的android:fitsSystemWindows="true"
        applyKitKatTranslucency(LibExtPrefer.getInstance().getBoolean(LibExtPrefer.USE_TRANSLUCENT_KEY, true));
    }

    protected void applyKitKatTranslucency(boolean on) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            setTranslucentStatus(on);
            setSystemBarTintDrawable(getStatusBarColor());
        }
    }

    @TargetApi(19)
    protected void setTranslucentStatus(boolean on) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            Window win = getWindow();
            WindowManager.LayoutParams winParams = win.getAttributes();
            // win.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
            // win.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION);
            final int bits = WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS;
            if (on) {
                winParams.flags |= bits;
            } else {
                winParams.flags &= ~bits;
            }
            win.setAttributes(winParams);
        }
    }

    protected void setSystemBarTintDrawable(int tintDrawable) {
        // protected void setSystemBarTintDrawable(Drawable tintDrawable) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            SystemBarTintManager mTintManager = new SystemBarTintManager(this);
            // if (tintDrawable != null) {
            if (tintDrawable != 0) {
                // 使StatusBarTintView 和 actionbar的颜色保持一致，风格统一。
                mTintManager.setStatusBarTintEnabled(true);
                mTintManager.setNavigationBarTintEnabled(true);
                // mTintManager.setTintDrawable(tintDrawable);
                mTintManager.setStatusBarTintColor(tintDrawable);
                mTintManager.setNavigationBarTintColor(tintDrawable);
                // 设置状态栏的文字颜色
                // mTintManager.setStatusBarDarkMode(true, this);
            } else {
                mTintManager.setStatusBarTintEnabled(false);
                mTintManager.setTintDrawable(null);
            }
        }
    }

    @Override
    public void finish() {
        super.finish();
        overridePendingTransition();
    }

    private void overridePendingTransition() {
        if (toggleOverridePendingTransition()) {
            switch (getOverridePendingTransitionMode()) {
                // 没有设置动画时，直接不处理,如不设置的，有默认的主题动画切换
                case NONE:
                    break;
                case LEFT:
                    // overridePendingTransition(R.anim.abc_fade_in, R.anim.abc_fade_out);
                    overridePendingTransition(R.anim.left_in, R.anim.left_out);
                    break;
                case RIGHT:
                    overridePendingTransition(R.anim.right_in, R.anim.right_out);
                    break;
                case TOP:
                    overridePendingTransition(R.anim.top_in, R.anim.top_out);
                    break;
                case BOTTOM:
                    overridePendingTransition(R.anim.bottom_in, R.anim.bottom_out);
                    break;
                case SCALE:
                    overridePendingTransition(R.anim.scale_in, R.anim.scale_out);
                    break;
                case FADE:
                    overridePendingTransition(R.anim.fade_in, R.anim.fade_out);
                    break;
                default:
                    break;
            }
        }
    }

    // 这里默认的处理了，如果发现布局没有ToolBar情况，不会报错；
    protected void initToolbar(Toolbar toolbar) {
        ToolbarUtils.initToolbar(toolbar, this);
    }

    public int getStatusBarColor() {
        return getColorPrimary();
    }

    public int getColorPrimary() {
        TypedValue typedValue = new TypedValue();
        getTheme().resolveAttribute(R.attr.colorPrimary, typedValue, true);
        return typedValue.data;
    }

    public int getDarkColorPrimary() {
        TypedValue typedValue = new TypedValue();
        getTheme().resolveAttribute(R.attr.colorPrimaryDark, typedValue, true);
        return typedValue.data;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // ButterKnife.reset(this);
//        ButterKnife.unbind(this);
//        unbinder.unbind();
        NetStateReceiver.removeRegisterObserver(mNetChangeObserver);
    }

    /**
     * 增加了默认的返回finish事件
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        switch (id) {
            case android.R.id.home:
                finish();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    public void widgetsClick(View v) {
    }

    @Override
    public void onClick(View v) {
        widgetsClick(v);
    }

    /**
     * 界面跳转(公共方法)
     */
    public void interfaceJump(Class<?> clazz, Bundle bundle, boolean isFinish) {
        Intent intent = new Intent(this, clazz);
        if (null != bundle) {
            intent.putExtras(bundle);
        }
        this.startActivity(intent);
        if (isFinish) {
            this.finish();
        }
    }

    protected boolean toggleOverridePendingTransition() {
        return true;
    }

    protected TransitionMode getOverridePendingTransitionMode() {
        return TransitionMode.NONE;
    }

    protected abstract void initWidgetsAndEvents(Bundle savedInstanceState);

    protected void onNetworkConnected(NetUtils.NetType type) {
    }

    protected void onNetworkDisConnected() {
    }

    private void initFragment() {
        mFragmentManager = getSupportFragmentManager();
        // mFragmentTransaction.setTransitionStyle(FragmentTransaction.TRANSIT_FRAGMENT_FADE);
        fragmentRefs = new HashMap<String, WeakReference<BaseAct>>();
    }

    protected void setFragmentContainer(int container) {
        mFrgmContainer = container;
    }

    protected void addFragment(BaseFra targetFragment) {
        mFragmentManager.beginTransaction().add(mFrgmContainer, targetFragment).commit();
    }

    protected void replaceFragment(BaseFra targetFragment) {
        mFragmentManager.beginTransaction().replace(mFrgmContainer, targetFragment).commit();
    }

    /**
     * 改变界面的fragment
     */
    public void changeFragment(int resView, BaseFra targetFragment, boolean isAdd) {
        FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
        if (isAdd) {
            transaction.add(resView, targetFragment, targetFragment.getClass().getName());
        } else {
            transaction.replace(resView, targetFragment, targetFragment.getClass().getName());
        }
        transaction.commit();
    }

    public void addFragment(String tag, BaseAct fragment) {
        fragmentRefs.put(tag, new WeakReference<BaseAct>(fragment));
    }

    public void removeFragment(String tag) {
        fragmentRefs.remove(tag);
    }

    /**
     * 先没有用上，做的是两个的综合考虑
     */
    protected boolean isApplyKitKatTranslucency() {
        return true;
    }

    // 一般用在全图片加载显示和顶部图片显示;这个时候会跟isApplyKitKatTranslucency设置样式相反;isApplyStatusBarTranslucency 为true isApplyKitKatTranslucency 为false
    // 其他情况他两配置一样为true
    protected boolean isApplyStatusBarTranslucency() {
        return true;
    }
}
