package com.zsy.libext.utils;

import android.content.Context;
import android.text.Editable;
import android.text.Selection;
import android.text.TextWatcher;
import android.widget.EditText;

/**
 * 用于判断输入框中是否为中文或者英文 length参数为输入框中允许输入的长度 中文占两个字节 英文占一个
 */
public class TextChinaOrEnglishLister implements TextWatcher {
  private EditText mFilter;
  private int length; // 总长度
  private Context mContext;
  private boolean onceLoad = true;// 防止有些手机在进行编辑信息时 空位在最前面
  private TextChangeListener textChangeListener;

  public TextChangeListener getTextChangeListener() {
    return textChangeListener;
  }

  public void setTextChangeListener(TextChangeListener textChangeListener) {
    this.textChangeListener = textChangeListener;
  }

  public TextChinaOrEnglishLister(EditText mFilter, int length) {
    super();
    this.mFilter = mFilter;
    this.length = length;
  }

  public TextChinaOrEnglishLister(EditText mFilter, int length, Context context) {
    super();
    this.mFilter = mFilter;
    this.length = length;
    mContext = context;
  }

  @Override public void onTextChanged(CharSequence s, int start, int before, int count) {
    int countSize = 0;
    Editable editable = mFilter.getText();
    String str = editable.toString();
    char[] array = str.toCharArray();
    int i = 0;
    if (onceLoad) {
      mFilter.setSelection(s.length());
      onceLoad = false;
    }
    for (i = 0; i < array.length; i++) {
      if ((char) (byte) array[i] != array[i])// 判断是否为中文
      {
        countSize += 2; // 如果为中文或者中文特殊符号则占两个字节
      } else {
        countSize += 1; // 英文则占一个字节
      }
      if (countSize > length) {// 最大值
        int selEndIndex = Selection.getSelectionEnd(editable);
        str = editable.toString();
        String newStr = str.substring(0, i);
        mFilter.setText(newStr);
        editable = mFilter.getText();
        int newLen = editable.length();
        if (selEndIndex > newLen) {
          selEndIndex = editable.length();
        }
        Selection.setSelection(editable, selEndIndex);
        break;
      }
    }
    if (textChangeListener != null) {
      textChangeListener.onTextChange(countSize, length);
    }
  }

  @Override public void beforeTextChanged(CharSequence s, int start, int count, int after) {
    String data = s.toString();
  }

  @Override public void afterTextChanged(Editable s) {
  }

  public static interface TextChangeListener {
    void onTextChange(int currentLen, int limitLen);
  }
}
