package com.zsy.libext.widgets.dialog.effectsdialog;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.text.InputFilter;
import android.text.TextUtils;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.TextView;

import com.zsy.libext.R;
import com.zsy.libext.widgets.dialog.effectsdialog.effects.BaseEffects;

/**
 * 通用弹出对话框
 */
public class EffectsAlertDialog extends Dialog {
  /** 默认标题文字颜色 */
  private final String defTextColor = "#333333";
  /**默认分隔线颜色*/
  // private final String defDividerColor = "#d0d0d0";
  /** 默认内容文字颜色 */
  private final String defMsgColor = "#3333333";
  /** 默认对话框背景色 */
  private final String defDialogColor = "#e3e3e3";
  /** 动画类型 */
  private Effectstype type = Effectstype.SlideBottom;
  private LinearLayout mLinearLayoutView;
  private LinearLayout mRelativeLayoutView;
  private LinearLayout mContentLayout;
  private LinearLayout mTitleLayout;
  private LinearLayout mBottomLayoutView;
  private ImageView scanBtn;//扫描按钮
  private FrameLayout mFrameLayoutCustomView;
  private View mDialogView;
  // private View mDivider;
  /** 对话框标题TextView */
  private TextView mTitleTv;
  /** 对话框提示内容TextView */
  private TextView mContentTv;
  /** 输入对话框输入框EditText */
  private EditText mContentEdt;
  /**标题左边带图片*/
  // private ImageView mIcon;
  /** 对话框左边按钮（默认取消） */
  private Button mCancelBtn;
  /** 对话框右边按钮（默认是操作功能） */
  private Button mSureBtn;
  private View lineView;

  /** 动画时间 */
  private int mDuration = 350;

  public EffectsAlertDialog(Context context) {
    super(context);
    init(context);
  }

  public EffectsAlertDialog(Context context, int theme) {
    super(context, theme);
    init(context);
  }

  @Override protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    WindowManager.LayoutParams params = getWindow().getAttributes();
    params.height = ViewGroup.LayoutParams.MATCH_PARENT;
    params.width = ViewGroup.LayoutParams.MATCH_PARENT;
    getWindow().setAttributes((WindowManager.LayoutParams) params);
  }

  /**
   * 这里不能使用单例，已经改过来，待改方法名称
   */
  public static EffectsAlertDialog getInstance(Context context) {
    return new EffectsAlertDialog(context, R.style.common_effects_dialog);
  }

  private void init(Context context) {
    mDialogView = View.inflate(context, R.layout.view_common_effects_dialog, null);
    mLinearLayoutView = (LinearLayout) mDialogView.findViewById(R.id.dialog_panel_ll);
    mRelativeLayoutView = (LinearLayout) mDialogView.findViewById(R.id.dialog_main_root_ll);
    mTitleLayout = (LinearLayout) mDialogView.findViewById(R.id.dialog_title_ll);
    mContentLayout = (LinearLayout) mDialogView.findViewById(R.id.dialog_content_ll);
    mBottomLayoutView = (LinearLayout) mDialogView.findViewById(R.id.dialog_bottom_ll);
    scanBtn = (ImageView) mDialogView.findViewById(R.id.scanBtn);
    mFrameLayoutCustomView = (FrameLayout) mDialogView.findViewById(R.id.dialog_custom_fl);
    mTitleTv = (TextView) mDialogView.findViewById(R.id.dialog_title_tv);
    mContentTv = (TextView) mDialogView.findViewById(R.id.dialog_content_tv);
    mContentEdt = (EditText) mDialogView.findViewById(R.id.dialog_content_et);
    lineView = mDialogView.findViewById(R.id.common_btn_line_view);
    // mIcon = (ImageView) mDialogView.findViewById(R.id.icon);
    // mDivider = mDialogView.findViewById(R.id.titleDivider);
    mCancelBtn = (Button) mDialogView.findViewById(R.id.cancel_btn);
    mSureBtn = (Button) mDialogView.findViewById(R.id.submit_btn);
    setContentView(mDialogView);

    this.setOnShowListener(new OnShowListener() {
      @Override public void onShow(DialogInterface dialogInterface) {

        mLinearLayoutView.setVisibility(View.VISIBLE);
        if (type == null) {
          type = Effectstype.Slidetop;
        }
        start(type);
      }
    });
    mCancelBtn.setOnClickListener(new View.OnClickListener() {

      @Override public void onClick(View v) {
        dismiss();
      }
    });
  }

  /** 设置对话框对应文字 */
  public void toDefault() {
    mTitleTv.setTextColor(Color.parseColor(defTextColor));
    // mDivider.setBackgroundColor(Color.parseColor(defDividerColor));
    mContentTv.setTextColor(Color.parseColor(defMsgColor));
    mLinearLayoutView.setBackgroundColor(Color.parseColor(defDialogColor));
  }

  /**设置分隔线颜色*/
  // public EffectsAlertDialog withDividerColor(String colorString) {
  // mDivider.setBackgroundColor(Color.parseColor(colorString));
  // return this;
  // }

  /** 设置对话框标题 */
  public EffectsAlertDialog withTitle(CharSequence title) {
    mCancelBtn.setVisibility(View.VISIBLE);
    lineView.setVisibility(View.VISIBLE);
    if (TextUtils.isEmpty(title)) {
      mTitleLayout.setVisibility(View.GONE);
    } else {
      mTitleLayout.setVisibility(View.VISIBLE);
      // toggleView(mTitleLayout, title);
      mTitleTv.setText(title);
    }
    return this;
  }

  /** 设置对话框标题 */
  public EffectsAlertDialog hideTitle() {
    mTitleLayout.setVisibility(View.GONE);
    mContentLayout.setVisibility(View.GONE);
    return this;
  }

  /** 设置对话框标题文字大小 */
  public EffectsAlertDialog withTitleTextSize(int size) {
    mTitleTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, size);
    return this;
  }

  /** 设置对话框标题颜色 */
  public EffectsAlertDialog withTitleColor(String colorString) {
    mTitleTv.setTextColor(Color.parseColor(colorString));
    return this;
  }

  /** 设置对话框标题颜色 */
  public EffectsAlertDialog withTitleColor(int color) {
    mTitleTv.setTextColor(color);
    return this;
  }

  /** 设置对话框标题上下间距 */
  public EffectsAlertDialog withTitleMargin(int left, int top, int right, int bottom) {
    LayoutParams params =
        new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT);
    params.setMargins(left, top, right, bottom);// 设置边距
    mTitleTv.setLayoutParams(params);
    return this;
  }

  /** 设置对话框内容上下间距 */
  public EffectsAlertDialog withContentMargin(int left, int top, int right, int bottom) {
    LayoutParams params =
        new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT);
    params.setMargins(left, top, right, bottom);// 设置边距
    mContentTv.setLayoutParams(params);
    return this;
  }

  /** 设置对话框中输入框下间距 */
  public EffectsAlertDialog withInputMargin(int left, int top, int right, int bottom) {
    LayoutParams params =
        new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT);
    params.setMargins(left, top, right, bottom);// 设置边距
    mContentTv.setLayoutParams(params);
    return this;
  }

  /** 设置对话框显示内容 */
  public EffectsAlertDialog withMessage(CharSequence msg) {
    return withMessage(msg, false);
  }

  /** 设置对话框显示内容 */
  public EffectsAlertDialog withMessage(CharSequence msg, boolean alignCenter) {
    mCancelBtn.setVisibility(View.VISIBLE);
    lineView.setVisibility(View.VISIBLE);
    if (!TextUtils.isEmpty(msg)) {
      mContentTv.setVisibility(View.VISIBLE);
      mContentEdt.setVisibility(View.GONE);
      if (alignCenter) {
        mContentTv.setGravity(Gravity.CENTER);
      }
      mContentTv.setText(msg);
    } else {
      mContentTv.setVisibility(View.GONE);
    }
    return this;
  }

  /** 设置对话框内容文字大小 */
  public EffectsAlertDialog withMessageTextSize(int size) {
    mContentTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, size);
    return this;
  }

  /**
   * 设置对话框为可以输入的对话框
   * 分别传入：提示文字 ，限制的长度，以及输入文字类型，如果没有限制，则字符传""，数字传0或负数
   */
  public EffectsAlertDialog withInputContent(CharSequence hintMsg, int maxLength, int inputType) {
    mContentTv.setVisibility(View.GONE);
    mContentEdt.setVisibility(View.VISIBLE);
    if (maxLength > 0) {
      mContentEdt.setFilters(new InputFilter[] { new InputFilter.LengthFilter(maxLength) });
    }
    if (inputType > 0) {
      mContentEdt.setInputType(inputType);
    }
    mContentEdt.setHint(hintMsg);
    return this;
  }

  /**
   * 设置输入文字颜色
   */
  public EffectsAlertDialog withInputContentColor(String colorStr) {
    mContentEdt.setTextColor(Color.parseColor(colorStr));
    return this;
  }

  /** 如果是输入提示框，获取输入的内容 */
  public String getInputStr() {
    return mContentEdt.getText().toString().trim();
  }

  /**给输入框填入内容*/
  public void setInputStr(String inputStr){
    mContentEdt.setText(inputStr);
  }

  /** 设置对话框内容颜色 */
  public EffectsAlertDialog withMessageColor(String colorString) {
    mContentTv.setTextColor(Color.parseColor(colorString));
    return this;
  }

  /** 设置对话框内容颜色 */
  public EffectsAlertDialog withMessageColor(int color) {
    mContentTv.setTextColor(color);
    return this;
  }

  /** 设置动画时间 */
  public EffectsAlertDialog withDuration(int duration) {
    this.mDuration = duration;
    return this;
  }

  /** 设置动画类型 */
  public EffectsAlertDialog withEffect(Effectstype type) {
    this.type = type;
    return this;
  }

  /** 设置按钮背景 */
  public EffectsAlertDialog withButtonDrawable(int resid) {
    mCancelBtn.setBackgroundResource(resid);
    mSureBtn.setBackgroundResource(resid);
    return this;
  }

  /** 设置左侧按钮文字 */
  public EffectsAlertDialog withLeftButtonText(CharSequence text) {
    if (TextUtils.isEmpty(text)) {
      mBottomLayoutView.setVisibility(View.GONE);
    } else {
      mBottomLayoutView.setVisibility(View.VISIBLE);
      mCancelBtn.setText(text);
    }
    return this;
  }

  /** 设置左侧按钮文字颜色 */
  public EffectsAlertDialog withLeftButtonTextColor(String color) {
    mCancelBtn.setTextColor(Color.parseColor(color));
    return this;
  }

  /** 设置右侧按钮文字 */
  public EffectsAlertDialog withRightButtonTextColor(String color) {
    mSureBtn.setTextColor(Color.parseColor(color));
    return this;
  }

  /** 设置右侧按钮文字颜色 */
  public EffectsAlertDialog withRightButtonText(CharSequence text) {
    if (TextUtils.isEmpty(text)) {
      mBottomLayoutView.setVisibility(View.GONE);
    } else {
      mBottomLayoutView.setVisibility(View.VISIBLE);
      mSureBtn.setText(text);
    }
    return this;
  }

  /** 设置只显示单个按钮的Dialog */
  public EffectsAlertDialog withSingleButton() {
    mCancelBtn.setVisibility(View.GONE);
    lineView.setVisibility(View.GONE);
    return this;
  }

  /** 设置左侧按钮点击操作 */
  public EffectsAlertDialog setRightButtonClick(View.OnClickListener click) {
    mBottomLayoutView.setVisibility(View.VISIBLE);
    mSureBtn.setOnClickListener(click);
    return this;
  }

  /**扫码按钮*/
  public EffectsAlertDialog setScanBtnClick(View.OnClickListener click) {
    scanBtn.setVisibility(View.VISIBLE);
    scanBtn.setOnClickListener(click);
    return this;
  }

  /** 设置右侧按钮点击操作 */
  public EffectsAlertDialog setLeftButtonClick(View.OnClickListener click) {
    mBottomLayoutView.setVisibility(View.VISIBLE);
    mCancelBtn.setOnClickListener(click);
    return this;
  }

  public void removeCustomView() {
    mFrameLayoutCustomView.setVisibility(View.GONE);
    if (mFrameLayoutCustomView.getChildCount() > 0) {
      mFrameLayoutCustomView.removeAllViews();
    }
  }

  /** 设置对话框自内容View */
  public EffectsAlertDialog setCustomView(int resId, Context context) {
    View customView = View.inflate(context, resId, null);
    mFrameLayoutCustomView.setVisibility(View.VISIBLE);
    if (mFrameLayoutCustomView.getChildCount() > 0) {
      mFrameLayoutCustomView.removeAllViews();
    }
    mFrameLayoutCustomView.addView(customView);
    return this;
  }

  /** 设置对话框自内容View */
  public EffectsAlertDialog setCustomView(View view, Context context) {
    mFrameLayoutCustomView.setVisibility(View.VISIBLE);
    if (mFrameLayoutCustomView.getChildCount() > 0) {
      mFrameLayoutCustomView.removeAllViews();
    }
    mFrameLayoutCustomView.addView(view);

    return this;
  }

  /** 设置对话框setCanceledOnTouchOutside */
  public EffectsAlertDialog isCancelableOnTouchOutside(boolean cancelable) {
    this.setCanceledOnTouchOutside(cancelable);
    return this;
  }

  /** 设置对话框setCancelable */
  public EffectsAlertDialog isCancelable(boolean cancelable) {
    this.setCancelable(cancelable);
    return this;
  }

  /** 执行动画 */
  private void start(Effectstype type) {
    BaseEffects animator = type.getAnimator();
    if (mDuration != -1) {
      animator.setDuration(Math.abs(mDuration));
    }
    animator.start(mRelativeLayoutView);
  }

  @Override public void dismiss() {
    super.dismiss();
  }


    /**
     * 公共Dialog的简单使用方法：
     EffectsAlertDialog dialogBuilder;
     dialogBuilder = EffectsAlertDialog.getInstance(this);
     dialogBuilder.withTitle("确定要执行操作？") // 设置对话框标题，无标题设置为""
     .withMessage("如果操作后数据可能就会丢失哦，请谨慎操作！") // 对话框显示的内容，如果无内容就设置为""
     //.withEffect(Effectstype.Shake) // 动画效果，确定后也进行统一设置
     .withRightButtonText("确定") // 右边操作按钮,如果不需要显示按钮，则设置按钮的文字为""
     .withSingleButton()//如果对话框只显示单个按钮，就调用这个
     .withInputContent("输入文字", 0, 0)// 参数分别指:输入框提示文字 ，限制的字符，输入的文字类型，后面参数无限制就输入0
     //					.setCustomView(R.layout.custom_view, v.getContext()) // 自己添加布局，布局中的相应处理要自己实现
     .setRightClick(new View.OnClickListener() {// 确定按钮点击事件
     @Override
     public void onClick(View v) {
     //执行对应的操作
     dialogBuilder.dismiss();//关闭对话框，这里还待优化
     }
     }.show();
     */
}
