package com.zsy.libext.widgets.viewpager.convenientbanner;

import android.content.Context;
import android.os.Handler;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;

import com.zsy.libext.R;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import androidx.viewpager.widget.ViewPager;

/**
 * 页面翻转控件
 *
 * @author Sai 支持自动翻页
 */
public class ConvenientBanner<T> extends LinearLayout {
	private CBViewHolderCreator holderCreator;
	private List<T> mDatas;
	private int[] page_indicatorId;
	private ArrayList<ImageView> mPointViews = new ArrayList<ImageView>();
	private CBPageChangeListener pageChangeListener;
	private CBPageAdapter pageAdapter;
	private CBLoopViewPager viewPager;
	private ViewGroup loPageTurningPoint;
	private long autoTurningTime;
	private boolean turning;
	private boolean canTurn = false;
	private Handler timeHandler = new Handler();
	private Runnable adSwitchTask = new Runnable() {
		@Override
		public void run() {
			if (viewPager != null && turning) {
				int page = viewPager.getCurrentItem() + 1;
				viewPager.setCurrentItem(page);
				timeHandler.postDelayed(adSwitchTask, autoTurningTime);
			}
		}
	};

	public ConvenientBanner(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context);
	}

	private void init(Context context) {
		View hView = LayoutInflater.from(context).inflate(R.layout.include_viewpager, this, true);
		viewPager = (CBLoopViewPager) hView.findViewById(R.id.cbLoopViewPager);
		loPageTurningPoint = (ViewGroup) hView.findViewById(R.id.loPageTurningPoint);
		initViewPagerScroll();
	}

	public ConvenientBanner setPages(CBViewHolderCreator holderCreator, List<T> datas) {
		this.mDatas = datas;
		this.holderCreator = holderCreator;
		pageAdapter = new CBPageAdapter(holderCreator, mDatas);
		viewPager.setAdapter(pageAdapter);
		viewPager.setBoundaryCaching(true);
		if (page_indicatorId != null) setPageIndicator(page_indicatorId);
		return this;
	}

	/**
	 * 设置底部指示器是否可见
	 */
	public ConvenientBanner setPointViewVisible(boolean visible) {
		loPageTurningPoint.setVisibility(visible ? View.VISIBLE : View.GONE);
		return this;
	}

	/**
	 * 底部指示器资源图片
	 */
	public ConvenientBanner setPageIndicator(int[] page_indicatorId) {
		loPageTurningPoint.removeAllViews();
		mPointViews.clear();
		this.page_indicatorId = page_indicatorId;
		for (int count = 0; count < mDatas.size(); count++) {
			// 翻页指示的点
			ImageView pointView = new ImageView(getContext());
			pointView.setPadding(5, 0, 5, 0);
			if (mPointViews.isEmpty()) {
				pointView.setImageResource(page_indicatorId[1]);
			}
			else {
				pointView.setImageResource(page_indicatorId[0]);
			}
			mPointViews.add(pointView);
			loPageTurningPoint.addView(pointView);
		}
		pageChangeListener = new CBPageChangeListener(mPointViews, page_indicatorId);
		viewPager.setOnPageChangeListener(pageChangeListener);

		return this;
	}

	public ConvenientBanner startTurning(long autoTurningTime) {
		canTurn = true;
		this.autoTurningTime = autoTurningTime;
		turning = true;
		timeHandler.postDelayed(adSwitchTask, autoTurningTime);
		return this;
	}

	public void stopTurning() {
		turning = false;
		timeHandler.removeCallbacks(adSwitchTask);
	}

	/**
	 * 自定义翻页动画效果
	 */
	public ConvenientBanner setPageTransformer(ViewPager.PageTransformer transformer) {
		viewPager.setPageTransformer(true, transformer);
		return this;
	}

	/**
	 * 自定义翻页动画效果，使用已存在的效果
	 */
	public ConvenientBanner setPageTransformer(Transformer transformer) {
		try {
			viewPager.setPageTransformer(true, (ViewPager.PageTransformer) Class.forName("com.huika.lib.core.ext.widget.convenientbanner.transforms." + transformer.getClassName()).newInstance());
		}
		catch (InstantiationException e) {
			e.printStackTrace();
		}
		catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return this;
	}

	/**
	 * 设置ViewPager的滑动速度
	 */
	private void initViewPagerScroll() {
		try {
			Field mScroller = null;
			mScroller = ViewPager.class.getDeclaredField("mScroller");
			mScroller.setAccessible(true);
			ViewPagerScroller scroller = new ViewPagerScroller(viewPager.getContext());
			// scroller.setScrollDuration(1500);
			mScroller.set(viewPager, scroller);
		}
		catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
		catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		catch (IllegalAccessException e) {
			e.printStackTrace();
		}
	}

	@Override
	public boolean dispatchTouchEvent(MotionEvent ev) {

		if (ev.getAction() == MotionEvent.ACTION_UP) {
			// 开始翻页
			if (canTurn) startTurning(autoTurningTime);
		}
		else if (ev.getAction() == MotionEvent.ACTION_DOWN) {
			// 停止翻页
			if (canTurn) stopTurning();
		}
		return super.dispatchTouchEvent(ev);
	}

	public enum Transformer {
		DefaultTransformer("DefaultTransformer"), AccordionTransformer("AccordionTransformer");

		private final String className;

		// 构造器默认也只能是private, 从而保证构造函数只能在内部使用
		Transformer(String className) {
			this.className = className;
		}

		public String getClassName() {
			return className;
		}
	}
}
