package com.urovo.wugumofang.membermodule.act;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import com.alibaba.android.arouter.launcher.ARouter;
import com.chad.library.adapter.base.BaseQuickAdapter;
import com.scwang.smartrefresh.layout.SmartRefreshLayout;
import com.scwang.smartrefresh.layout.api.RefreshLayout;
import com.scwang.smartrefresh.layout.listener.OnRefreshListener;
import com.urovo.wgmf.db.dao.dao.PrivilegeDbOperate;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.RouterParams;
import com.urovo.wugumofang.membermodule.R;
import com.urovo.wugumofang.membermodule.ada.MultipleItemQuickAdapter;
import com.urovo.wugumofang.membermodule.ent.MemberItem;
import com.urovo.wugumofang.membermodule.ent.MemberManager;
import com.urovo.wugumofang.membermodule.ent.MembershipStoreData;
import com.urovo.wugumofang.membermodule.ent.MultipleItem;
import com.urovo.wugumofang.membermodule.pre.MemberPre;
import com.urovo.wugumofang.membermodule.pre.MemberShowWeChatPre;
import com.urovo.wugumofang.net.CommonController;
import com.urovo.wugumofang.response.CommonEntity;
import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.utils.CommonTools;
import com.urovo.wugumofang.view.webview.BaseCookiesWebFra;
import com.zsy.libcore.common.android.log.SYLoger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

/**
 * 会员营销界面
 *
 * @author chenji
 * @date 18/01/03
 */
public class MemberAct extends BaseActivity
        implements CommonController.BaseView<MembershipStoreData>, CommonController.showAddFriOnWeChat<Boolean> {
    //    @BindView(R2.id.rv_member_list)
    RecyclerView mMemberRv;
    //    @BindView(R2.id.swipe_refresh_layout)
    SmartRefreshLayout mSwipeRefreshLayout;
    //    @BindView(R2.id.empty_view)
    TextView mEmptyViewTv;

    /**
     * 子模块集合
     */
    private List<MemberItem> mBodyList = new ArrayList<>();
    private List<MemberItem> mFootList = new ArrayList<>();

    /**
     * 页面构成所有的控件的集合
     */
    private List<MultipleItem> mTotalList = new ArrayList<>();

    /**
     * Item的子元素一行铺满的总列数(ps:一行最多能摆放几个子元素)
     */
    private final int ROW_ITEM_NUM = 3;

    //    Unbinder unbinder;
    private MultipleItemQuickAdapter multipleItemAdapter;
    private static final String LINK_TYPE_NATIVE = "native";
    private static final String LINK_TYPE_URL = "url";
    private MemberPre memberPre;
    private MemberShowWeChatPre memberShowWeChatPre;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.member_act_mem);
//        unbinder = ButterKnife.bind(this);
        //不知道为什么，ButterKnife的R2失效了，不能通过编译。
        mMemberRv = findViewById(R.id.rv_member_list);
        mSwipeRefreshLayout = findViewById(R.id.swipe_refresh_layout);
        mEmptyViewTv = findViewById(R.id.empty_view);
        // body模块因为是直接读本地数据,在这里初始化
        mBodyList.addAll(MemberManager.initBodyData());
        initRv();
        initRefreshLayout();
        refresh();
        initPrivilege2VisibilityOrGone();
    }

    /**
     * 整个页面没有权限时显示的布局
     */
    private void initPrivilege2VisibilityOrGone() {
        if (!PrivilegeDbOperate.checkUserPrivilege(AppConstants.Member.MEMBER)) {
            mSwipeRefreshLayout.setVisibility(View.GONE);
            mEmptyViewTv.setText("暂无使用权限!");
        }
    }

    /**
     * 刷新头监听器初始化
     */
    private void initRefreshLayout() {
        mSwipeRefreshLayout.setOnRefreshListener(new OnRefreshListener() {
            @Override
            public void onRefresh(final RefreshLayout refreshlayout) {
                refreshlayout.getLayout().postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        refresh();
                        refreshlayout.finishRefresh();
//                        refreshlayout.resetNoMoreData();
                        refreshlayout.setNoMoreData(false);
                        mSwipeRefreshLayout.autoRefresh();
                    }
                }, 2000);
            }
        });
    }

    private void refresh() {
        postDelayed(30000);
        //准备数据
        mFootList.clear();
        mFootList.addAll(MemberManager.initFootData());
        start2RequestTask();
//        showAddFriOnWeChatTask();
    }

    /**
     * 是否显示加微及入群人数提报页面
     */
    private void showAddFriOnWeChatTask() {
//        postDelayed(30000);
//        HashMap<String, String> param = new HashMap<>(1);
//        param.put("shopId", SystemHelper.LoginOrganId);
//        memberShowWeChatPre.onStart2requestData(param);
    }

    private void initRv() {
        memberShowWeChatPre = new MemberShowWeChatPre(this);
        memberPre = new MemberPre(this);
        //初始化没有数据,设置为null直接走离线数据
        mTotalList = getTotalList(null);
        multipleItemAdapter = new MultipleItemQuickAdapter(this, mTotalList);
        multipleItemAdapter.openLoadAnimation(BaseQuickAdapter.ALPHAIN);
        final GridLayoutManager manager = new GridLayoutManager(this, ROW_ITEM_NUM);
        mMemberRv.setLayoutManager(manager);
        multipleItemAdapter.setSpanSizeLookup(new BaseQuickAdapter.SpanSizeLookup() {
            @Override
            public int getSpanSize(GridLayoutManager gridLayoutManager, int position) {
                return mTotalList.get(position).getSpanSize();
            }
        });
        mMemberRv.setAdapter(multipleItemAdapter);

        multipleItemAdapter.setOnItemChildClickListener(new BaseQuickAdapter.OnItemChildClickListener() {
            @Override
            public void onItemChildClick(BaseQuickAdapter baseQuickAdapter, View view, int position) {
                final MultipleItem multipleItem = (MultipleItem) baseQuickAdapter.getData().get(position);
                if (multipleItem == null) {
                    return;
                }
                SYLoger.print("wgmf--会员页面--position：" + position);
                SYLoger.print("wgmf--会员页面--getLinkType：" + multipleItem.getLinkType());
                SYLoger.print("wgmf--会员页面--getLinkUrl：" + multipleItem.getLinkUrl());
                jump2Page(multipleItem.getLinkType(), multipleItem.getLinkUrl(), multipleItem.getItemPrivilegeId());
            }
        });
    }

    /**
     * 页面跳转
     *
     * @param linkType 是h5还是本地
     * @param linkUrl  跳转地址
     */
    private void jump2Page(String linkType, String linkUrl, String itemPrivilegeId) {
        if (LINK_TYPE_URL.equals(linkType)) {
//            if (multipleItem.getPosition() == 0) {
//                WebViewAct.launch(MemberAct.this, "", memberItem.pageUrl);
//            } else {
            //H5页面跳转,顾客评价H5多层级返回
            if (AppConstants.Member.CUSTOMER_EVALUATION_QUERY.equals(itemPrivilegeId)) {
                BaseCookiesWebFra.launch(MemberAct.this, new Intent(), linkUrl, "", true);
            } else {
                BaseCookiesWebFra.launch(MemberAct.this, linkUrl, "", true);
            }
//            }
        } else {
            //本地页面跳转
            ARouter.getInstance().build(linkUrl).navigation();
        }
    }

    @Override
    protected void onDestroy() {
//        unbinder.unbind();
        super.onDestroy();
    }

    /**
     * 网络请求后得到会员数据和会员商品数据的数组
     *
     * @param resultInfoBean ...
     * @return ...
     */
    private String[] getMemArray(MembershipStoreData.ResultInfoBean resultInfoBean) {
        return new String[]
                {
                        resultInfoBean.getTitle(),
                        resultInfoBean.getSubtitleOne(),
                        resultInfoBean.getSubtitleTwo(),
                        resultInfoBean.getSubtitleThree(),
                        resultInfoBean.getSubOneData(),
                        resultInfoBean.getSubTwoData(),
                        resultInfoBean.getSubThreeData()
                };
    }

    /**
     * 权限控制加载显示几部分的长度(目前分为head body foot三大部分)
     *
     * @param resultInfo 请求后返回的数据集合，这里是本月门店会员数据和会员商品的值
     * @return 返回MultipleItem对象的总的集合, ps:不能理解为head body foot 是个对象,只是作者为了好管理做的分类
     */
    private List<MultipleItem> getTotalList(List<MembershipStoreData.ResultInfoBean> resultInfo) {
        //清空上次的缓存数据
        clear(mTotalList);
        String[] memModulePrivilege = getResources().getStringArray(R.array.member_ship_module_privilege);
        //权限控制加载各个module
        for (int i = 0; i < memModulePrivilege.length; i++) {
            if (PrivilegeDbOperate.checkUserPrivilege(memModulePrivilege[i])) {
                loadModule(i, resultInfo);
            }
        }
        return mTotalList;
    }

    /**
     * 加载各个部分
     *
     * @param position   ...
     * @param resultInfo ...
     */
    private void loadModule(final int position, List<MembershipStoreData.ResultInfoBean> resultInfo) {
        switch (position) {
            case 0:
                // 头部
                headItem(resultInfo);
                break;
            case 1:
                //中部
                bodyItem();
                break;
            case 2:
                // 尾部
                footItem();
                break;
            default:
                break;
        }
    }

    /**
     * 头部--本月门店会员数据和本月门店会员商品
     *
     * @param resultInfo ...
     */
    private void headItem(List<MembershipStoreData.ResultInfoBean> resultInfo) {
        // 头部1 --本月门店会员数据,ps:设置为null,直接看adapter
        if (PrivilegeDbOperate.checkUserPrivilege(AppConstants.Member.MEMBERSHIP_STORE_DATA)) {
            mTotalList.add(new MultipleItem(MultipleItem.TEXT_0,
                    MultipleItem.TEXT_SPAN_SIZE,
                    resultInfo == null ? null : getMemArray(resultInfo.get(0))));
        }

        //头部2 --本月门店会员商品
        //添加更多按钮信息 跳转会员商品转发(MemberGoodsAct.class)
        if (PrivilegeDbOperate.checkUserPrivilege(AppConstants.Member.MEMBERSHIP_STORE_MALL_DATA)) {
            mTotalList.add(new MultipleItem(MultipleItem.TEXT_1,
                    MultipleItem.TEXT_SPAN_SIZE,
                    LINK_TYPE_NATIVE,
                    RouterParams.Member.MEMBER_GOODS,
                    resultInfo == null ? null : getMemArray(resultInfo.get(1))));
            //头部加高--补白色线条
            mTotalList.add(new MultipleItem(MultipleItem.HEAD_HORIZONTAL_LINE, MultipleItem.HORIZONTAL_LINE_SPAN_SIZE));
        }
    }

    /**
     * 中部 --会员构成 新客三响炮 老客唤醒 ...
     */
    private void bodyItem() {
        int count = 0;
        //间隔线
        mTotalList.add(new MultipleItem(MultipleItem.HORIZONTAL_LINE, MultipleItem.HORIZONTAL_LINE_SPAN_SIZE));

        //中部会员营销策略入口
        for (int i = 0; i < mBodyList.size(); i++) {
            if (PrivilegeDbOperate.checkUserPrivilege(mBodyList.get(i).itemPrivilegeId)) {
                mTotalList.add(new MultipleItem(MultipleItem.IMG_TEXT,
                        MultipleItem.IMG_TEXT_SPAN_SIZE,
                        mBodyList.get(i).iconId,
                        mBodyList.get(i).itemName,
                        mBodyList.get(i).linkType,
                        mBodyList.get(i).linkUrl,
                        mBodyList.get(i).itemPrivilegeId));
                count++;
            }
        }
        //集合总数为奇数时,需要填补空白item的处理
        final int emptyNum = emptyNumRule(count);
        for (int i = 0; i < emptyNum; i++) {
            mTotalList.add(new MultipleItem<>(MultipleItem.IMG_TEXT_EMPTY, MultipleItem.IMG_TEXT_SPAN_SIZE));
        }
        //中部body --会员构成部分缺少的白色线条
        mTotalList.add(new MultipleItem(MultipleItem.BODY_HORIZONTAL_LINE, MultipleItem.HORIZONTAL_LINE_SPAN_SIZE));
    }

    /**
     * 是否需要填补空白子元素的规则
     *
     * @param bodyListSize 显示列表的总数量
     * @return 填补空白子元素的数量
     */
    private int emptyNumRule(int bodyListSize) {
        int emptyNum;
        if (bodyListSize % ROW_ITEM_NUM > 0) {
            //奇数
            emptyNum = ROW_ITEM_NUM - bodyListSize % ROW_ITEM_NUM;
        } else {
            //偶数
            emptyNum = 0;
        }
        return emptyNum;
    }

    /**
     * 尾部 --新客微会员 新客数据 ...
     */
    private void footItem() {
        //间隔线
        mTotalList.add(new MultipleItem(MultipleItem.HORIZONTAL_LINE, MultipleItem.HORIZONTAL_LINE_SPAN_SIZE));

        //尾部foot --会员数据入口
        SYLoger.print("memPri-FootNum->" + mFootList.size());

        for (int j = 0; j < mFootList.size(); j++) {
            SYLoger.print("memPri-priID->" + mFootList.get(j).itemPrivilegeId);
            SYLoger.print("memPri-priIDBoolean->" + PrivilegeDbOperate.checkUserPrivilege(mFootList.get(j).itemPrivilegeId));
            if (PrivilegeDbOperate.checkUserPrivilege(mFootList.get(j).itemPrivilegeId)) {
                mTotalList.add(new MultipleItem(MultipleItem.IMG_TEXT_IMG,
                        MultipleItem.IMG_TEXT_IMG_SPAN_SIZE,
                        mFootList.get(j).iconId,
                        mFootList.get(j).itemName,
                        mFootList.get(j).linkType,
                        mFootList.get(j).linkUrl,
                        mFootList.get(j).itemPrivilegeId
                ));
            }
        }
        SYLoger.print("memPri-TotalNum->" + mTotalList.size());
        //添加尾部--本来自然,何需添加提示语
        mTotalList.add(new MultipleItem(MultipleItem.TEXT_FOOT_TIPS, MultipleItem.TEXT_SPAN_SIZE));
    }

    /**
     * 超时处理刷新状态
     *
     * @param delayTime ...
     */
    @SuppressWarnings("all")
    private void postDelayed(long delayTime) {
        CommonTools.INSTANCE.postDelayed(delayTime, new Runnable() {
            @Override
            public void run() {
                multipleItemAdapter.setEnableLoadMore(true);
                if (mSwipeRefreshLayout != null) {
                    mSwipeRefreshLayout.finishRefresh();
                }
            }
        });
    }

    /**
     * 开始请求
     */
    public void start2RequestTask() {
        postDelayed(30000);
        HashMap<String, String> param = new HashMap<>(1);
        param.put("shopId", SystemHelper.LoginOrganId);
        memberPre.onStart2requestData(param);
    }

    /**
     * 清空集合
     */
    private void clear(List mTotalList) {
        if (mTotalList != null && !mTotalList.isEmpty()) {
            mTotalList.clear();
        }
    }

    @Override
    public void onResponseSuccess(CommonEntity<MembershipStoreData> result) {
//      存储本月会员数据和会员商品数据的集合
        List<MembershipStoreData.ResultInfoBean> resultInfo = new ArrayList<>();
        final MembershipStoreData entity = result.entity;
        if (entity != null && entity.getResultInfo() != null) {
            // head 会员数据和会员商品的集合
            resultInfo.addAll(entity.getResultInfo());
            // head body foot 构成的控件的集合
            mTotalList = getTotalList(resultInfo);
            //更新集合数据
            if (this.isFinishing()) return;
            mSwipeRefreshLayout.finishRefresh();
            multipleItemAdapter.notifyDataSetChanged();
        }
    }

    @Override
    public void onFailure(String failInfo) {
        BaseAppManager.loger().error(failInfo);
        //断网等异常情况下  初始化数据,走离线
        mTotalList = getTotalList(null);

        //刷新和加载更多失败后的处理
        if (this.isFinishing()) return;
        mSwipeRefreshLayout.finishRefresh();
        multipleItemAdapter.notifyDataSetChanged();
    }

    @Override
    public void onShowAddFriOnWeChatSuccess(Boolean resultInfo) {
//        //准备数据
//        mFootList.clear();
//        mFootList.addAll(MemberManager.initFootData());
//        start2RequestTask();
//        SYLoger.print("wgmf-->是否显示加微及入群人数提报页面: " + resultInfo);
    }

    @Override
    public void onShowAddFriOnWeChatFailure(String failInfo) {
//        //准备数据
//        mFootList.clear();
//        mFootList.addAll(MemberManager.initFootData());
//        mFootList.remove(0);
//        start2RequestTask();
//        SYLoger.print("wgmf-->是否显示加微及入群人数提报页面: " + failInfo);
    }
}