package com.urovo.wugumofang.basewebview.ui;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.ZoomButtonsController;

import androidx.fragment.app.Fragment;

import com.bumptech.glide.Glide;
import com.urovo.wugumofang.basewebview.R;
import com.urovo.wugumofang.utils.MyToast;
import com.urovo.wugumofang.utils.NetworkUtil;

import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Field;

/**
 * 网页容器
 */
@SuppressWarnings("all")
public class HtmlFragment extends Fragment implements HtmlView {

    private HtmlPresenter htmlPresenter = new HtmlPresenter(this);

    private FrameLayout activityDataWebviewFl, activityVideoWebviewFl;
    private View mCustomView = null;

    WebView activityDataWebview;
    LinearLayout activityDataWebviewReloadLl;
    LinearLayout activityDataWebviewLl;
    TextView activityDataWebviewPbTv;
    View activityDataWebviewBottom;
    ImageView activityDataWebviewPbIv;
    ImageView activityDataWebviewReload;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContent();
    }

    private void setContent() {
        htmlPresenter.initData();
    }

    private void setWebClient() {
        //htmlPresenter.synCookies(getActivity());
        htmlPresenter.load();
    }

    @Override
    public View onCreateView(@NotNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_webview_content, container, false);
        activityDataWebviewFl = view.findViewById(R.id.activity_data_webview_fl);
        int bottomHeight = getArguments().getInt("bottomHeight");
        activityDataWebviewBottom = view.findViewById(R.id.activity_data_webview_bottom);
        if (bottomHeight > 0) {
            ViewGroup.LayoutParams layoutParams = activityDataWebviewBottom.getLayoutParams();
            layoutParams.height = bottomHeight;
            activityDataWebviewBottom.setLayoutParams(layoutParams);
            activityDataWebviewBottom.setVisibility(View.INVISIBLE);
        } else {
            activityDataWebviewBottom.setVisibility(View.GONE);
        }
        activityVideoWebviewFl = view.findViewById(R.id.activity_video_webview_fl);
        activityDataWebview = (WebView) view.findViewById(R.id.activity_data_webview);
        activityDataWebviewReloadLl = (LinearLayout) view.findViewById(R.id.activity_data_webview_reload_ll);
        activityDataWebviewLl = (LinearLayout) view.findViewById(R.id.activity_data_webview_ll);
        activityDataWebviewPbTv = (TextView) view.findViewById(R.id.activity_data_webview_pb_tv);
        activityDataWebviewPbIv = (ImageView) view.findViewById(R.id.activity_data_webview_pb_iv);
        Glide.with(getActivity()).load(R.drawable.load).asGif().into(activityDataWebviewPbIv);
        activityDataWebviewReload = (ImageView) view.findViewById(R.id.activity_data_webview_reload);
        activityDataWebviewReload.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                refresh();
            }
        });
        activityDataWebviewReloadLl.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                refresh();
            }
        });
        initWebClient();
        setWebClient();
        return view;
    }

    public void back() {
        if (activityDataWebview.canGoBack()) {
            // 返回前一个页面
            activityDataWebview.goBack();
        } else {
            getActivity().finish();
        }
    }

    @SuppressLint({"JavascriptInterface", "AddJavascriptInterface"})
    private void initWebClient() {
        activityDataWebview.getSettings().setJavaScriptEnabled(true);
        activityDataWebview.getSettings().setPluginState(WebSettings.PluginState.ON);
        activityDataWebview.getSettings().setJavaScriptCanOpenWindowsAutomatically(true);
        //提高渲染的优先级
        activityDataWebview.getSettings().setRenderPriority(WebSettings.RenderPriority.HIGH);
        if (NetworkUtil.isNetworkAvailable(getActivity().getApplicationContext())) {
            //根据cache-control决定是否从网络上取数据。
//            activityDataWebview.getSettings().setCacheMode(WebSettings.LOAD_DEFAULT);
            // 不使用缓存
            activityDataWebview.getSettings().setCacheMode(WebSettings.LOAD_NO_CACHE);
        } else {
            showError("您的网络未连接，请检查！");
            //没网，则从本地获取，即离线加载
            activityDataWebview.getSettings().setCacheMode(WebSettings.LOAD_CACHE_ELSE_NETWORK);
        }

        activityDataWebview.getSettings().setAllowFileAccess(true);

        activityDataWebview.getSettings().setLoadsImagesAutomatically(true);
        activityDataWebview.getSettings().setLoadWithOverviewMode(true);

        activityDataWebview.getSettings().setUseWideViewPort(true);
        activityDataWebview.getSettings().setAllowContentAccess(true);

        // 开启 DOM storage API 功能
        activityDataWebview.getSettings().setDomStorageEnabled(true);
        activityDataWebview.getSettings().setDatabaseEnabled(true);

        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {
            activityDataWebview.getSettings().setDatabasePath(activityDataWebview.getContext().getFilesDir() + activityDataWebview.getContext().getPackageName() + "/databases/");
        }
        // WebView 默认不允许一个安全的链接混合使用HTTPS +HTTP，需要设置webview支持HTTPS +HTTP
        activityDataWebview.getSettings().setBlockNetworkImage(false);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            activityDataWebview.getSettings().setMixedContentMode(android.webkit.WebSettings.MIXED_CONTENT_ALWAYS_ALLOW);
            activityDataWebview.getSettings().setAllowUniversalAccessFromFileURLs(true);
        }

        //去掉滚动条
        activityDataWebview.setVerticalScrollBarEnabled(true);
        activityDataWebview.setHorizontalScrollBarEnabled(false);

        // 设置可缩放
        activityDataWebview.getSettings().setSupportZoom(true);
        activityDataWebview.getSettings().setBuiltInZoomControls(true);
        // 根据版本号设置缩放按钮不显示
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.HONEYCOMB) {//用于判断是否为Android 3.0系统, 然后隐藏缩放控件
            activityDataWebview.getSettings().setDisplayZoomControls(false);
        } else {// Android 3.0(11) 以下使用以下方法
            setZoomControlGone(activityDataWebview);
        }

        activityDataWebview.addJavascriptInterface(new WebViewJsBridge(), "androidToJsBridge");

        WebChromeClient webChromeClient = new WebChromeClient() {

            private WebChromeClient.CustomViewCallback mCustomViewCallback = null;

            @Override
            public void onShowCustomView(View view, CustomViewCallback callback) {
                super.onShowCustomView(view, callback);
                try {
                    if (mCustomView != null) {
                        callback.onCustomViewHidden();
                        return;
                    }
                    mCustomView = view;
                    activityVideoWebviewFl.addView(mCustomView);
                    activityVideoWebviewFl.setVisibility(View.VISIBLE);
                    activityDataWebviewFl.setVisibility(View.GONE);
                    mCustomViewCallback = callback;
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onHideCustomView() {
                try {
                    activityDataWebviewFl.setVisibility(View.VISIBLE);
                    if (mCustomView == null) {
                        return;
                    }
                    activityVideoWebviewFl.setVisibility(View.GONE);
                    activityVideoWebviewFl.removeView(mCustomView);
                    if (mCustomViewCallback != null) {
                        mCustomViewCallback.onCustomViewHidden();
                    }
                    mCustomView = null;
                } catch (Exception e) {
                    e.printStackTrace();
                }
                super.onHideCustomView();
            }

            @Override
            public void onReceivedTitle(WebView view, String title) {
                super.onReceivedTitle(view, title);
                Log.d("onReceivedTitle", title);
                setTitleText(title);
            }

            @Override
            public void onProgressChanged(WebView view, int newProgress) {
                if (newProgress < 100) {
                    activityDataWebviewLl.setVisibility(View.VISIBLE);
                    activityDataWebview.setVisibility(View.GONE);
                } else {
                    activityDataWebviewLl.setVisibility(View.GONE);
                    activityDataWebview.setVisibility(View.VISIBLE);
                }
                String progress = newProgress + "%";
                activityDataWebviewPbTv.setText("网页已加载：" + progress);
            }
        };
        // 设置setWebChromeClient对象
        activityDataWebview.setWebChromeClient(webChromeClient);
        activityDataWebview.setWebViewClient(htmlPresenter.getWebViewClient(getActivity()));
        activityDataWebview.setScrollBarStyle(WebView.SCROLLBARS_OUTSIDE_OVERLAY);
    }

    /**
     * 实现放大缩小控件隐藏
     *
     * @param view
     */
    public void setZoomControlGone(View view) {
        Class classType;
        Field field;
        try {
            classType = WebView.class;
            field = classType.getDeclaredField("mZoomButtonsController");
            field.setAccessible(true);
            ZoomButtonsController mZoomButtonsController = new ZoomButtonsController(view);
            mZoomButtonsController.getZoomControls().setVisibility(View.GONE);
            try {
                field.set(view, mZoomButtonsController);
            } catch (IllegalArgumentException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        }
    }

    private void setTitleText(String title) {
        if (title.contains("error") || title.contains("404") || title.contains("网页无法打开")) {
            activityDataWebviewReloadLl.setVisibility(View.VISIBLE);
        } else {
            activityDataWebviewReloadLl.setVisibility(View.GONE);
        }
        if (!TextUtils.isEmpty(title) && !title.contains("index")) {
            onChangeTitleListener.changeTitle(title);
        }
    }

    /**
     * 页面Tab角标
     *
     * @param index
     */
    public void setPageTabParams(int index) {
        htmlPresenter.setPageTabParams(index);
        //回滚到跟页面
        rollback();
    }

    @Override
    public void refresh() {
        MyToast.show("重新加载网页数据...");
        activityDataWebview.reload();
        // 不使用缓存
        activityDataWebview.getSettings().setCacheMode(WebSettings.LOAD_NO_CACHE);
    }

    @Override
    public void loadData(String url) {
        activityDataWebview.loadData(url, "text/html; charset=UTF-8", null);
    }

    @Override
    public void rollback() {
        while (true) {
            if (activityDataWebview.canGoBack()) {
                activityDataWebview.goBack();
            } else {
                break;
            }
        }
    }

    @Override
    public void loadUrl(String url) {
        //每次刷新网页时，重新设置cookies
        //htmlPresenter.synCookies(getActivity());
        activityDataWebview.loadUrl(url);
    }


    @Override
    public void showLoadingDialog() {
//        showDialog();
    }

    @Override
    public void finishLoadingDialog() {
//        dismissDialog();
    }

    @Override
    public void showError(String error) {
        MyToast.show(error);
    }

    @Override
    public void finishView() {
        getActivity().finish();
    }

    OnChangeTitleListener onChangeTitleListener;

    public interface OnChangeTitleListener {
        public void changeTitle(String title);
    }

    @Override
    public void onAttach(Context activity) {
        super.onAttach(activity);
        try {
            onChangeTitleListener = (OnChangeTitleListener) getActivity();
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement OnHeadlineSelectedListener");
        }
    }
}