package com.urovo.wugumofang.printermodule;

import android.bluetooth.BluetoothSocket;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;

import com.snbc.sdk.LabelPrinter;
import com.urovo.wugumofang.constants.AppConstants;

import java.util.List;


public abstract class Printer {
    protected String mAddress;
    protected BluetoothSocket mBluetoothSocket;
    protected LabelPrinter mLabelPrinter = new LabelPrinter();
    protected static final Object mLock = new Object();
    protected boolean mIsPrinting = false;
    private PrinterUtil.PrintCallback mPrintCallback;

    /**
     * 文字单品 ，小类型(旧的)，一种商品打印一个条形码（小条码纸）
     */
    public abstract void printMsg(List<PrinterUtil.PrinterRule> list, double total);

    /**
     * 文字组合，长类型(新的)，多种商品打印一个条形码（大条码纸）
     */
    public abstract void printGroupMsg(List<PrinterUtil.PrinterRule> list, double total);

    /**
     * 图片(二维码)
     */
    public abstract void printBitmap(int x, int y, String path, String fileName, String titleText, String bottomText);

    /**
     * 打印标签
     *
     * @param list ...
     */
    public abstract void printLabel(List<PrinterUtil.MultipleItem> list);

    public abstract void stopPrint();

    public Printer(String address, PrinterUtil.PrintCallback callback) {
        mAddress = address;
        mPrintCallback = callback;
    }

    public boolean isPrint() {
        return mIsPrinting;
    }

    /**
     * 检查条形码只有数字
     */
    protected boolean checkPrintCellValid(String type, int index, PrinterUtil.PrinterRule cell) {
        boolean result = true;
        if (null == type || TextUtils.isEmpty(type)) {
            result = false;
        }
        if (!result) {
            Message msg = new Message();
            msg.what = AppConstants.Printer.TYPE_INVALID;
            msg.obj = type;
            mHandler.sendMessage(msg);
            return result;
        }
        if (null == cell.mCode || TextUtils.isEmpty(cell.mCode)) {
            result = false;
        } else {
            for (int i = 0; i < cell.mCode.length(); i++) {
                char c = cell.mCode.charAt(i);
                if (c < '0' || c > '9') { //检查条形码只有数字
                    result = false;
                    break;
                }
            }
        }
        if (!result) {
            Message msg = new Message();
            msg.what = AppConstants.Printer.CODE_INVALID;
            msg.arg1 = index;
            msg.obj = cell;
            mHandler.sendMessage(msg);
        }
        return result;
    }

    protected final Handler mHandler = new Handler(new Handler.Callback() {

        @Override
        public boolean handleMessage(Message msg) {
            if (null != mPrintCallback) {
                switch (msg.what) {
                    case (AppConstants.Printer.SEARCH): {
                        mPrintCallback.onSearchPrint();
                        break;
                    }
                    case (AppConstants.Printer.START): {
                        mPrintCallback.onStartPrint();
                        break;
                    }
                    case (AppConstants.Printer.ONE_SUCCESS): {
                        if (null == msg.obj) break;
                        int index = msg.arg1;
                        PrinterUtil.PrinterRule cell = (PrinterUtil.PrinterRule) msg.obj;
                        mPrintCallback.onPrintOneSuccessed(index, cell.mTitle, cell.mSale, cell.mCode);
                        break;
                    }
                    case (AppConstants.Printer.FINISH): {
                        mPrintCallback.onFinishPrint();
                        break;
                    }
                    case (AppConstants.Printer.STOP): {
                        mPrintCallback.onStopPrint();
                        break;
                    }
                    case (AppConstants.Printer.ADDRESS_INVALID): {
                        mPrintCallback.onAddressInvalidOrCanNotFound();
                        break;
                    }
                    case (AppConstants.Printer.TYPE_INVALID): {
                        if (null == msg.obj) break;
                        mPrintCallback.onBarCodeTypeInvalid((String) msg.obj);
                        break;
                    }
                    case (AppConstants.Printer.CODE_INVALID): {
                        if (null == msg.obj) break;
                        int index = msg.arg1;
                        PrinterUtil.PrinterRule cell = (PrinterUtil.PrinterRule) msg.obj;
                        mPrintCallback.onBarCodeIsInvalid(index, cell.mTitle, cell.mSale, cell.mCode);
                        break;
                    }
                    case (AppConstants.Printer.ERROR): {
                        if (null == msg.obj) break;
                        mPrintCallback.onPrinterError((String) msg.obj);
                        break;
                    }
                    case AppConstants.Printer.BARCODE_TO_THOUSAND://13情况下价格到达一千变成6位数；
                        if (null == msg.obj) break;
                        mPrintCallback.onFinishPrint();//这里算临时中断
                        mPrintCallback.onPrinterErrorBarcodeToThousand((String) msg.obj);
                        break;
                    default:
                        break;
                }
            }
            return true;
        }
    });
}
