package com.urovo.wugumofang.printermodule;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.widget.Toast;

import com.snbc.sdk.LabelPrinter;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.ConstantParams;
import com.urovo.wugumofang.data.Arith;
import com.urovo.wugumofang.data.MoneyUtils;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.manager.ThreadManager;
import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.system.SystemHelper;
import com.zsy.libcore.common.android.log.SYLoger;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.UUID;

/**
 * @author ...
 */
@SuppressWarnings("all")
public class PrinterManage extends Printer {

    static {
        try {
            LabelPrinter.LoadLibrary("LabelPrinterSDK");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private final boolean DEBUG = false;

    private boolean mNewPrinterType = false;

    private final UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    /**
     * 请求打印机字体
     */
    public final static String TSC_FONT_TYPE = "FONT001";
    public final static String SNBC_FONT_TYPE = "8";

    public final static String TSC_CODE_13 = "EAN13";
    public final static String TSC_CODE_128 = "128";
    public final static String SNBC_CODE_13 = "5";
    public final static String SNBC_CODE_128 = "1";

    private final int WIDTH_TSC = 60;// 40 60
    private final int HEIGHT_TSC = 28;// 25 50  150
    private final int ZUHE_WIDTH_TSC = 50;// 40 60
    private final int ZUHE_HEIGHT_TSC = 65;// 25 50 //350
    private final int WIDTH_SNBC = 480;
    private final int HEIGHT_SNBC = 240;
    private final int ZUHE_WIDTH_SNBC = 480;
    private final int ZUHE_HEIGHT_SNBC = 480;

    private final int SPEED = 10;
    private final int DENSITY = 15;
    private final int SENSOR = 0;
    private final int GAP = 2;
    private final int OFFSET = 0;

    private final int TEXT_X = 40;
    private int TEXT_Y;
    private final int TEXT_Y_STEP = 35;
    private final int BARCODE_Y = 55;
    private final int TEXT_X2 = 35;

    /**
     * RT_MART 特殊配置；
     */
    private final int RT_TEXT_X = 16;
    private int RT_TEXT_Y;
    private final int RT_TEXT_Y_STEP = 32;
    private final int RT_BARCODE_Y = RT_TEXT_Y_STEP * 2;

    private final int ONE_PAGE_COUNT = 9;
    private final int LABEL_START_X = 35;
    private final int LABEL_START_Y = 350;//底部
    private final String ENCODE = "gb2312";

    //根据官方文档整理后的状态提示/错误提示
    public static String READY = "待机";
    public static String HEAD_OPEN = "打印头开启";//就是盒盖未扣上
    public static String PAPER_JAM = "纸张卡纸";
    public static String PAPER_EMPTY = "缺纸";
    public static String HEAD_OPEN_AND_PAPER_JAM = "纸张卡纸并且打印头开启";
    public static String HEAD_OPEN_AND_PAPER_EMPTY = "缺纸并且打印头开启";
    public static String RIBBON_EMPTY = "无碳带";
    public static String HEAD_OPEN_AND_RIBBON_EMPTY = "无碳带并且打印头开启";
    public static String PAPER_JAM_AND_RIBBON_EMPTY = "无碳带并且卡纸";
    public static String HEAD_OPEN_AND_PAPER_JAM_AND_RIBBON_EMPTY = "无碳带、卡纸并且打印头开启";
    public static String PAPER_EMPTY_AND_RIBBON_EMPTY = "无碳带并且缺纸";
    public static String HEAD_OPEN_AND_PAPER_EMPTY_AND_RIBBON_EMPTY = "无碳带、缺纸并且打印头开启";
    public static String PAUSE = "暂停";
    public static String PRINTING = "打印中";
    public static String OTHER_PROBLEM = "其它错误";
    public static String HEAD_OVERHEAT = "打印头过热";
    public static String CUTTER_ERROR_STATE = "切刀处于错误状态";

    public static String NO_PAPER = "没纸了";
    public static String RIBBON_JAM = "卡碳带";
    public static String CUTTING = "Cutting";
    public static String WAIT_PRESS_KEY = "Waiting to Press Print Key";
    public static String WAIT_TAKE_LABEL = "Waiting to Take Label";

    /**
     * 条码标签条形码的高宽配置的参数
     */
    //条形码的宽窄
    public int mNarrow;
    //条形码的长短
    public int mWide;
    //目前设定为28位条码的高度
    private int barCodeHeight = 55;

    public PrinterManage(String address, PrinterUtil.PrintCallback callback) {
        super(address, callback);
    }

    /**
     * 一个标签打印一个商品 （小条码纸打印）
     */
    @Override
    public void printMsg(List<PrinterUtil.PrinterRule> list, double total) {
        ThreadManager.getThreadPool().execute(new MyRunnableListLabel(list, total));
    }

    /**
     * 一个标签打印多个商品 （大条码纸打印）
     */
    @Override
    public void printGroupMsg(List<PrinterUtil.PrinterRule> list, double total) {
        ThreadManager.getThreadPool().execute(new MyRunnableOneLabel(list, total));
    }

    /**
     * 图片(二维码)
     */
    @Override
    public void printBitmap(int x, int y, String path, String fileName, String titleText, String bottomText) {
        ThreadManager.getThreadPool().execute(new MyRunnablePrintBmp(x, y, path, fileName, titleText, bottomText));
    }

    /**
     * 纯打印(目前用做标签打印)
     */
    @Override
    public void printLabel(List<PrinterUtil.MultipleItem> list) {
        ThreadManager.getThreadPool().execute(new PrinterLabelRunnable(list));
    }

    @Override
    public void stopPrint() {
        mIsPrinting = false;
    }


    /**
     * @param address mac地址
     * @return
     */
    private boolean connectLogic(String address) {
        mNewPrinterType = PreferHelper.getInstance().getBoolean(
                AppConstants.PreKeys.NEW_PRINTER_TYPE, false
        );
        BarcodeUtilKt.setNewPrinterType(mNewPrinterType);
        if (mNewPrinterType) {
            return connectPrinter(address);
        } else {
            return openPort(address);
        }
    }

    /**
     * 链接打印器链接
     *
     * @param address 地址
     * @return 是否连接成功
     */
    private boolean connectPrinter(String address) {
        boolean result = false;
        //if (null != mLabelPrinter) {
        //    if (mLabelPrinter.Disconnect() == ConstantParams.CONSTANT_STARTS_0) {
        //        mLabelPrinter = null;
        //    } else {
        //        mHandler.sendEmptyMessage(AppConstants.Printer.ADDRESS_INVALID);
        //    }
        //}
        //mLabelPrinter = new LabelPrinter();
        int status = mLabelPrinter.ConnectPrinter(ConstantParams.CONSTANT_STARTS_7, address,
                ConstantParams.CONSTANT_STARTS_4);
        if (status == ConstantParams.CONSTANT_STARTS_0) {
            result = true;
            if (DEBUG) {
                Log.e("PrinterManage", "open ok");
            }
        } else {
            mHandler.sendEmptyMessage(AppConstants.Printer.ADDRESS_INVALID);
        }
        return result;
    }

    /**
     * 建立蓝牙Socket连接
     *
     * @param address 地址
     * @return 是否连接成功
     */
    private boolean openPort(String address) {
        boolean result = false;
        BluetoothDevice device = null;
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (null != mBluetoothSocket) {
            closePort();
            mBluetoothSocket = null;
        }
        if (null == bluetoothAdapter) {
            if (DEBUG) {
                Log.e("PrinterManage", "本设备不支持蓝牙功能");
            }
            return result;
        }
        try {
            // bluetoothAdapter.startDiscovery();
            device = bluetoothAdapter.getRemoteDevice(address);
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "蓝牙地址有误");
            }
            mHandler.sendEmptyMessage(AppConstants.Printer.ADDRESS_INVALID);
            return result;
        }
        try {
            mBluetoothSocket = device.createRfcommSocketToServiceRecord(MY_UUID);
            if (null != mBluetoothSocket) {
                bluetoothAdapter.cancelDiscovery();
                mBluetoothSocket.connect();
                result = true;
                if (DEBUG) {
                    Log.e("PrinterManage", "open ok");
                }
            }
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "openPort error:" + e.toString());
            }
            mHandler.sendEmptyMessage(AppConstants.Printer.ADDRESS_INVALID);
        }
        // bluetoothAdapter.cancelDiscovery();
        return result;
    }

    private boolean setup(int width, int height, int speed, int density, int sensor, int sensor_distance, int sensor_offset) {
        if (null == mBluetoothSocket) {
            return false;
        }
        boolean resault = false;
        String size = "SIZE " + width + " mm" + ", " + height + " mm";
        String speed_value = "SPEED " + speed;
        String density_value = "DENSITY " + density;

        String sensor_value = "";
        if (sensor == 0) {
            sensor_value = "GAP " + sensor_distance + " mm" + ", " + sensor_offset + " mm";
        } else if (sensor == 1) {
            sensor_value = "BLINE " + sensor_distance + " mm" + ", " + sensor_offset + " mm";
        }

        String message = "";
        message = size + "\n" + speed_value + "\n" + density_value + "\n" + sensor_value + "\n";

        try {
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
            resault = true;
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "setup error:" + e.toString());
            }
        }
        return resault;
    }

    private boolean clearBuffer() {
        if (null == mBluetoothSocket) {
            return false;
        }
        boolean resault = false;
        String message = "CLS\n";
        try {
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
            resault = true;
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "clearBuffer error:" + e.toString());
            }
        }
        return resault;
    }

    private String batch() {
        if (null == mBluetoothSocket) {
            return null;
        }
        int printvalue = 0;
        String printbatch = "";
        String stringbatch = "";
        String message = "~HS";
        byte[] batcharray = new byte[8];
        try {
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "batch error:" + e.toString());
            }
        }

        try {
            Thread.sleep(100L);
        } catch (InterruptedException e) {

        }
        byte[] readBuf = new byte[1024];
        try {
            InputStream is = mBluetoothSocket.getInputStream();
            while (is.available() > 0) {
                readBuf = new byte[1024];
                is.read(readBuf);
            }
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "batch read error:" + e.toString());
            }
        }

        if (readBuf[0] == 2) {
            System.arraycopy(readBuf, 55, batcharray, 0, 8);
            try {
                stringbatch = new String(batcharray, ENCODE);
            } catch (UnsupportedEncodingException e) {

            }
            printvalue = Integer.parseInt(stringbatch);
            printbatch = Integer.toString(printvalue);
        }
        return printbatch;
    }

    private void clearDocument() {
        String message = "KILL \"*\"\n";
        try {
            if (null == mBluetoothSocket) {
                return;
            }
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "clearDocument error:" + e.toString());
            }
        }
    }

    private void downloadBmpTtfPcx(String filepath, String filename) {
        try {
            if (mBluetoothSocket == null) {
                return;
            }
            FileInputStream fis = new FileInputStream(filepath + filename);

            byte[] data = new byte[fis.available()];
            String download = "DOWNLOAD F,\"" + filename + "\"," + data.length + ",";
            byte[] downloadHead = download.getBytes(ENCODE);

            while (fis.read(data) != -1) {
                OutputStream os = mBluetoothSocket.getOutputStream();
                os.write(downloadHead);
                os.write(data);
                fis.close();
            }
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "downloadBmpTtfPcx error:" + e.toString());
            }
        }
    }

    private void sendfile(String filepath, String filename) {
        try {
            FileInputStream fis = new FileInputStream(filepath + filename);
            byte[] data = new byte[fis.available()];
            int[] FF = new int[data.length];
            while (fis.read(data) != -1) {
                OutputStream os = mBluetoothSocket.getOutputStream();
                os.write(data);
                fis.close();
            }
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "sendfile error:" + e.toString());
            }
        }
    }

    private void formFeed() {
        String message = "FORMFEED\n";
        try {
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "formFeed error:" + e.toString());
            }
        }
    }

    private void noBackFeed() {
        String message = "SET TEAR OFF\n";
        try {
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "noBackFeed error:" + e.toString());
            }
        }
    }

    private void sendCommand(String message) {
        try {
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "sendCommand error:" + e.toString());
            }
        }
    }

    public String printerStatusSnbc() {
        String query = "";
        if (mLabelPrinter == null) {
            return query;
        }

        LabelPrinter.PrinterStatus printerStatus = mLabelPrinter.GetStatus();
        if (printerStatus != null) {
            if (printerStatus.is_ready_to_print == ConstantParams.CONSTANT_STARTS_1) {
                query = "00";
            } else if (printerStatus.is_paused == ConstantParams.CONSTANT_STARTS_1) {
                query = "10";
            } else if (printerStatus.is_paper_out == ConstantParams.CONSTANT_STARTS_1) {
                query = "04";
            } else if (printerStatus.is_head_too_hot == ConstantParams.CONSTANT_STARTS_1) {
                query = "30";
            } else if (printerStatus.is_head_opened == ConstantParams.CONSTANT_STARTS_1) {
                query = "01";
            } else if (printerStatus.is_ribbon_out == ConstantParams.CONSTANT_STARTS_1) {
                query = "08";
            } else if (printerStatus.is_cutter_error == ConstantParams.CONSTANT_STARTS_1) {
                query = "40";
            } else if (printerStatus.is_printer_busy == ConstantParams.CONSTANT_STARTS_1) {
                query = "20";
            }

            //通过从数据库映射出对应的错误说明
            query = StatusDbMap.statusDbMap(query);
        }
        return query;
    }

    /**
     * 从SDK源码拷过来的判断错误的方法加了点修改
     * sdk已引入，详情请参考tscsdk.jar这个包里的TSCActivity.class，
     * 这些方法其实在这个类里都有
     *
     * @return
     */
    public String printerStatusTsc() {
        String query = "";
        if (mBluetoothSocket == null) {
            return query;
        }

        byte[] message = new byte[]{27, 33, 63};
        byte[] readBuf = new byte[1024];
        InputStream is = null;
        try {
            OutputStream os = mBluetoothSocket.getOutputStream();
            is = mBluetoothSocket.getInputStream();
            os.write(message);
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "status message error:" + e.toString());
            }
        }

        try {
            Thread.sleep(1000L);
        } catch (InterruptedException var4) {
            var4.printStackTrace();
        }

        try {
            while (is.available() > 0) {
                is.read(readBuf);
            }
        } catch (Exception e) {
            BaseAppManager.loger().info(e);
        }

        if (readBuf[0] == 0) {
            query = "00";
        } else if (readBuf[0] == 1) {
            query = "01";
        } else if (readBuf[0] == 2) {
            query = "02";
        } else if (readBuf[0] == 3) {
            query = "03";
        } else if (readBuf[0] == 4) {
            query = "04";
        } else if (readBuf[0] == 5) {
            query = "05";
        } else if (readBuf[0] == 8) {
            query = "08";
        } else if (readBuf[0] == 9) {
            query = "09";
        } else if (readBuf[0] == 10) {
            query = "0A";
        } else if (readBuf[0] == 11) {
            query = "0B";
        } else if (readBuf[0] == 12) {
            query = "0C";
        } else if (readBuf[0] == 13) {
            query = "0D";
        } else if (readBuf[0] == 16) {
            query = "10";
        } else if (readBuf[0] == 32) {
            query = "20";
        } else if (readBuf[0] == 128) {
            query = "80";
        }
        //通过从数据库映射出对应的错误说明
        query = StatusDbMap.statusDbMap(query);

        return query;
    }

    private String status() {
        String printerstatus = "";
        if (mBluetoothSocket == null) {
            return printerstatus;
        }
        byte[] readBuf = new byte[1024];
        try {
            byte[] message = {27, 33, 83};
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(message);
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "status message error:" + e.toString());
            }
        }
        try {
            Thread.sleep(80L);
        } catch (InterruptedException e) {
        }

        try {
            InputStream is = mBluetoothSocket.getInputStream();
            if (is.available() > 0) {
                readBuf = new byte[1024];
                is.read(readBuf);
            }
        } catch (Exception e) {
            BaseAppManager.loger().info(e);
        }

        if ((readBuf[0] == 2) && (readBuf[5] == 3)) {
            for (int tim = 0; tim <= 7; tim++) {
                //readBuf{2, 64, 64 ,64 ,64 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 64) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 64) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = READY;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 69, 64 ,64 ,96 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 69) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 96) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = HEAD_OPEN;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 64, 64 ,64 ,96 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 64) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 96) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = HEAD_OPEN;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 69, 64 ,64 ,72 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 69) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 72) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = RIBBON_JAM;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 69, 64 ,64 ,68 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 69) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 68) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = RIBBON_EMPTY;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 69, 64 ,64 ,65 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 69) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 65) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = NO_PAPER;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 69, 64 ,64 ,66 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 69) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 66) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = PAPER_JAM;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 69, 64 ,64 ,65 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 69) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 65) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = PAPER_EMPTY;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 67, 64 ,64 ,64 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 67) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 64) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = CUTTING;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 75, 64 ,64 ,64 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 75) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 64) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = WAIT_PRESS_KEY;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 76, 64 ,64 ,64 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 76) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 64) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = WAIT_TAKE_LABEL;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 80, 64 ,64 ,64 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 80) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 64) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = PRINTING;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 96, 64 ,64 ,64 ,3}
                if ((readBuf[tim] == 2) && (readBuf[(tim + 1)] == 96) && (readBuf[(tim + 2)] == 64) && (
                        readBuf[(tim + 3)] == 64) && (readBuf[(tim + 4)] == 64) && (readBuf[(tim + 5)] == 3)) {
                    printerstatus = PAUSE;
                    readBuf = new byte[1024];
                    break;
                }
                //readBuf{2, 69, 64 ,64 ,64 ,3}
                if ((readBuf[tim] != 2) || (readBuf[(tim + 1)] != 69) || (readBuf[(tim + 2)] != 64) || (
                        readBuf[(tim + 3)] != 64) || (readBuf[(tim + 4)] != 64) || (readBuf[(tim + 5)] != 3)) {
                    continue;
                }
                printerstatus = PAUSE;
                readBuf = new byte[1024];
                break;
            }
        }
        if (DEBUG) {
            Log.e("PrinterManage", "status printerstatus :" + printerstatus);
        }
        return printerstatus;
    }

    private boolean barcode(int x, int y, String type, int height, int readable, int rotation, int narrow, int wide, String string) {
        boolean result = false;
        if (mNewPrinterType) {
            if (null == mLabelPrinter) {
                return false;
            }
            try {
                int status = mLabelPrinter.PrintBarcode1D(x, y, Integer.parseInt(type), rotation, string, height, readable, narrow, wide);
                if (status == ConstantParams.CONSTANT_STARTS_0) {
                    result = true;
                }
            } catch (Exception e) {
                if (DEBUG) {
                    Log.e("PrinterManage", "barcode error:" + e.toString());
                }
            }
        } else {
            if (null == mBluetoothSocket) {
                return false;
            }

            String message = "";
            String barcode = "BARCODE ";
            String position = x + "," + y;
            String mode = "\"" + type + "\"";
            String content = "\"" + string + "\"";

            message = barcode + position + " ," + mode + " ," + height + " ," + readable + " ," + rotation + " ," + narrow + " ," + wide + " ," + content + "\n";

            try {
                byte[] msgBuffer = message.getBytes(ENCODE);
                OutputStream os = mBluetoothSocket.getOutputStream();
                os.write(msgBuffer);
                os.flush();
                result = true;
            } catch (Exception e) {
                if (DEBUG) {
                    Log.e("PrinterManage", "barcode error:" + e.toString());
                }
            }
        }
        return result;
    }

    private boolean qrcode(int x, int y, String level, int width, String mode, int rotation,
                           String data) {
        if (null == mBluetoothSocket) {
            return false;
        }
        boolean resault = false;
        String message = "";
        String text = "QRCODE ";
        String position = x + "," + y;
        String level_value = "\"" + level + "\"";
        String mode_value = "\"" + mode + "\"";
        message =
                text + position + " ," + level + " ," + width + " ," + mode_value + " ," + rotation + " ,"
                        + " A, 0, M2, S7 ," + data + "\n";
        try {
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
            resault = true;
        } catch (Exception e) {
            if (DEBUG) {
                return resault;
            }
            Log.e("PrinterManage", "printerfont error:" + e.toString());
        }
        return resault;
    }

    private boolean putBitmap(int x, int y, String path, String fileName) {
        boolean result = false;
        if (mNewPrinterType) {
            if (null == mLabelPrinter) {
                return false;
            }
            try {
                File file = new File(path, fileName);
                if (!file.exists()) {
                    return false;
                }
                int status = mLabelPrinter.PrintImageFile(x, y, file.getPath());
                if (status == ConstantParams.CONSTANT_STARTS_0) {
                    result = true;
                }
            } catch (Exception e) {
                if (DEBUG) {
                    Log.e("PrinterManage", "printerbitmap error:" + e.toString());
                }
            }
        } else {
            if (null == mBluetoothSocket) {
                return false;
            }
            String message = "";
            String text = "PUTBMP ";
            String position = x + "," + y;
            message = text + position + ",\"" + fileName + "\"\n";
            try {
                byte[] msgBuffer = message.getBytes(ENCODE);
                OutputStream os = mBluetoothSocket.getOutputStream();
                os.write(msgBuffer);
                result = true;
            } catch (Exception e) {
                if (DEBUG) {
                    Log.e("PrinterManage", "printerbitmap error:" + e.toString());
                }
            }
        }
        return result;
    }

    private boolean bitmap(int x, int y, int widthBytes, int height, byte[] bitmap) {
        if (null != mBluetoothSocket) {
            boolean resault = false;
            String message = "";
            String text = "BITMAP ";
            String position = x + "," + y;
            message = text + position + "," + widthBytes + "," + height + ","
                    + "\"00000000000007FF03FF11FF18FF1C7F1E3F1F1F1F8F1FC71FE31FF71FFF1FFF\"" + "\n";
            try {
                bitmap = "00000000000007FF03FF11FF18FF1C7F1E3F1F1F1F8F1FC71FE31FF71FFF1FFF".getBytes(ENCODE);
                byte[] msgBuffer = message.getBytes(ENCODE);
                OutputStream os = mBluetoothSocket.getOutputStream();
                os.write(msgBuffer);
                resault = true;
            } catch (Exception e) {
                if (DEBUG) {
                    Log.e("PrinterManage", "printerfont error:" + e.toString());
                }
            }
            return resault;
        } else {
            return false;
        }
    }

    /**
     * 打印文字相关的:标题、￥价格、条形码字符串;请求打印一条数据
     * 打印多行字体
     *
     * @param x    横向间距
     * @param y    竖向间距
     * @param size 字体大小
     */
    private boolean printerfont(int x, int y, String size, int rotation, int x_multiplication,
                                int y_multiplication, String string) {
        boolean result = false;
        if (mNewPrinterType) {
            if (null == mLabelPrinter) {
                return false;
            }
            try {
                int status = mLabelPrinter.PrintText(x, y, size, string, 0, 0, 0, 0);
                if (status == ConstantParams.CONSTANT_STARTS_0) {
                    return true;
                }
            } catch (Exception e) {
                if (DEBUG) {
                    Log.e("PrinterManage", "printerfont error:" + e.toString());
                }
            }
        } else {
            if (null == mBluetoothSocket) {
                return false;
            }

            String message = "";
            String text = "TEXT ";
            String position = x + "," + y;
            String size_value = "\"" + size + "\"";
            String string_value = "\"" + string + "\"";
            message = text + position + " ," + size_value + " ," + rotation + " ," + x_multiplication + " ,"
                    + y_multiplication + " ," + string_value + "\n";

            try {
                byte[] msgBuffer = message.getBytes(ENCODE);
                OutputStream os = mBluetoothSocket.getOutputStream();
                os.write(msgBuffer);
                result = true;
            } catch (Exception e) {
                if (DEBUG) {
                    Log.e("PrinterManage", "printerfont error:" + e.toString());
                }
            }
        }
        return result;
    }

    private void printSnbc(int labelNum, int copyNum) {
        mLabelPrinter.PrintLabel(labelNum, copyNum);
    }

    /**
     * 请求打印数据,(请求打印在一张纸上),使用之前记得 先printerfont 再clearBuffer()
     */
    private boolean printTsc(int quantity, int copy) {
        if (null == mBluetoothSocket) {
            return false;
        }
        boolean resault = false;
        String message = "";
        message = "PRINT " + quantity + ", " + copy + "\n";
        try {
            byte[] msgBuffer = message.getBytes(ENCODE);
            OutputStream os = mBluetoothSocket.getOutputStream();
            os.write(msgBuffer);
            resault = true;
        } catch (Exception e) {
            if (DEBUG) {
                return resault;
            }
            Log.e("PrinterManage", "print error:" + e.toString());
        }
        return resault;
    }

    private boolean closeLogic() {
        if (mNewPrinterType) {
            return closePrinter();
        } else {
            return closePort();
        }
    }

    private boolean closePort() {
        if (null == mBluetoothSocket) {
            return false;
        }
        boolean result = false;
        try {
            mBluetoothSocket.close();
            mBluetoothSocket = null;
            result = true;
        } catch (Exception e) {
            if (DEBUG) {
                Log.e("PrinterManage", "closePort error:" + e.toString());
            }
        }
        return result;
    }

    private boolean closePrinter() {
        if (null == mLabelPrinter) {
            return false;
        }
        boolean result = false;
        if (mLabelPrinter.Disconnect() == ConstantParams.CONSTANT_STARTS_0) {
            result = true;
            mLabelPrinter = null;
        }
        return result;
    }


    private void setupPrinterSnbc(int width, int height) {
        mLabelPrinter.SetLabelSize(width, height);
    }

    private void setupPrinterTsc(int width, int height) {
        setup(width, height, SPEED, DENSITY, SENSOR, GAP, OFFSET);
    }

    /**
     * 根据条码长度来判断
     *
     * @return fonts[0] 是mNarrow
     * fonts[1] 是mWide
     */
    public int[] getCodeFont(PrinterUtil.PrinterRule rule) {
        int[] fonts = new int[2];
        if (mNewPrinterType) {
            fonts[0] = 1;
            fonts[1] = 2;
        } else {
            fonts[0] = rule.mCode.length() > 28 ? 1 : 2;
            fonts[1] = rule.mCode.length() > 28 ? 2 : 5;
        }
        return fonts;
    }

    /**
     * 一个标签对应一个产品的排版布局
     * 打印排版
     * 打印表单：分多部分处理;
     * 标题、￥价格、条形码字符串和条形码
     */
    private void doPrintCell(String type, int index, PrinterUtil.PrinterRule cell) {
//        boolean resault = true;
        int length = cell.mIngredient.length();
        int mSaveLength = cell.mSaveCondition.length();
        int mProSupplierLength = cell.getProSupplier().length();
        int mProAreaLength = cell.getProArea().length();
        String fontType = BarcodeUtilKt.fontType();
        //初始化下条形码的长短和宽窄
        mNarrow = getCodeFont(cell)[0];
        mWide = getCodeFont(cell)[1];
        clearBuffer();
        // 组合码大纸专用条码打印排版,大条码纸,一个标签对应一个商品
//        if (type.startsWith("ZH")) {
        switch (cell.mBarCodePaper) {

            //排版四：纸张类型就是排版类型
            case AppConstants.Printer.COMBINED_CODE_BIG_PAPER:
                RT_TEXT_Y = 32;
                printerfont(horizontalCenterWidth(cell.mTitle), 5, fontType, 0, 1, 1, cell.mTitle);//条码打印标题

                /*------------------------------------------配料---------------------------------------------*/
                if (length <= 12) {
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "配料:" + cell.mIngredient);//第一行
                } else if (length > 12 && length <= 26) {
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "配料:" + cell.mIngredient.substring(0, 12));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + 30;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "    " + cell.mIngredient.substring(12, length));//第二行
                } else if (length > 26) {
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "配料:" + cell.mIngredient.substring(0, 12));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + 30;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.mIngredient.substring(12, 26));//第二行
                    RT_TEXT_Y = RT_TEXT_Y + 30;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.mIngredient.substring(26, length));//第三行
                }

                /*------------------------------------------保存及食用方法--------------------------------*/
                if (mSaveLength <= 7) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "保存及食用方法:" + cell.mSaveCondition);//第一行
                } else if (mSaveLength > 7 && mSaveLength <= 22) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "保存及食用方法:" + cell.mSaveCondition.substring(0, 7));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.mSaveCondition.substring(7, mSaveLength));//第二行
                }

                /*-----------------------------------------暂时不用换行的----------------------------------*/
                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP, fontType, 0, 1, 1, "包装日期:见包装");//包装日期

                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP * 2, fontType, 0, 1, 1, "保质日期:见包装");//保质日期


                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP * 3, fontType, 0, 1, 1, cell.mCode.substring(3, 9));//货号

                printerfont(RT_TEXT_X + 125, RT_TEXT_Y + RT_TEXT_Y_STEP * 3, fontType, 0, 1, 1, cell.mCount + " 件");//数量

                printerfont(RT_TEXT_X + 250, RT_TEXT_Y + RT_TEXT_Y_STEP * 3, fontType, 0, 1, 1, "￥" + MoneyUtils.format(cell.mPrice * cell.mCount));//价格


                barcode(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP * 4, BarcodeUtilKt.barCodeType(cell.mCode), RT_BARCODE_Y, 1, 0, mNarrow, mWide, cell.mCode);//条码

//                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP * 6, fontType, 0, 1, 1, cell.mCode);//条码字符串


                /*--------------------------------------------产品供应商（公司名）----------------------------*/
                if (mProSupplierLength <= 14) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP * 7;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProSupplier());//第一行
                } else if (mProSupplierLength > 14 && mProSupplierLength <= 28) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP * 7;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProSupplier().substring(0, 14));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProSupplier().substring(14, mProSupplierLength));//第二行
                }

                /*------------------------------------------地址-----------------------------------------*/
                if (mProAreaLength <= 13) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProArea());//第一行
                } else if (mProAreaLength > 13 && mProAreaLength <= 26) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;//每次增加RT_TEXT_Y_STEP间隔
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProArea().substring(0, 13));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProArea().substring(13, mProAreaLength));//第二行
                }

                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP, fontType, 0, 1, 1, "电话: 400-6836-080");//电话

                break;

            // 大润发专用条码打印排版,大条码纸打印,一个标签对应一个商品
//        else if (PrinterUtil.ONE_BARCODE_13_RT_MART.equals(type)) {

            //排版三
            case AppConstants.Printer.RTMART_PAPER:
                RT_TEXT_Y = 32;
                printerfont(horizontalCenterWidth(cell.mTitle), 5, fontType, 0, 1, 1, cell.mTitle);//条码打印标题


                /*------------------------------------------配料---------------------------------------------*/
                if (length <= 12) {
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "配料:" + cell.mIngredient);//第一行
                } else if (length > 12 && length <= 26) {
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "配料:" + cell.mIngredient.substring(0, 12));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + 30;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "    " + cell.mIngredient.substring(12, length));//第二行
                } else if (length > 26) {
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "配料:" + cell.mIngredient.substring(0, 12));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + 30;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.mIngredient.substring(12, 26));//第二行
                    RT_TEXT_Y = RT_TEXT_Y + 30;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.mIngredient.substring(26, length));//第三行
                }

                /*------------------------------------------保存及食用方法--------------------------------*/
                if (mSaveLength <= 7) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "保存及食用方法:" + cell.mSaveCondition);//第一行
                } else if (mSaveLength > 7 && mSaveLength <= 22) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, "保存及食用方法:" + cell.mSaveCondition.substring(0, 7));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.mSaveCondition.substring(7, mSaveLength));//第二行
                }

                /*-----------------------------------------暂时不用换行的----------------------------------*/
                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP, fontType, 0, 1, 1, "包装日期:见包装");//包装日期

                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP * 2, BarcodeUtilKt.fontType(), 0, 1, 1, "保质日期:见包装");//保质日期

                barcode(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP * 3, BarcodeUtilKt.barCodeType(cell.mCode), RT_BARCODE_Y, 1, 0, BarcodeUtilKt.narrow(), BarcodeUtilKt.wide(), cell.mCode);//条码

//                printerfont(RT_TEXT_X + 250, RT_TEXT_Y + RT_TEXT_Y_STEP * 2, fontType, 0, 1, 1, cell.mCode.substring(1, 7));//商超码

                printerfont(RT_TEXT_X + 250, RT_TEXT_Y + RT_TEXT_Y_STEP * 3, fontType, 0, 1, 1, cell.mCount + " 件");//数量

                printerfont(RT_TEXT_X + 250, RT_TEXT_Y + RT_TEXT_Y_STEP * 4, fontType, 0, 1, 1, "￥" + MoneyUtils.format(cell.mPrice * cell.mCount));//价格

//                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP * 5, fontType, 0, 1, 1, cell.mCode);//条码字符串


                /*--------------------------------------------产品供应商（公司名）----------------------------*/
                if (mProSupplierLength <= 14) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP * 6;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProSupplier());//第一行
                } else if (mProSupplierLength > 14 && mProSupplierLength <= 28) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP * 6;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProSupplier().substring(0, 14));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProSupplier().substring(14, mProSupplierLength));//第二行
                }

                /*------------------------------------------地址-----------------------------------------*/
                if (mProAreaLength <= 13) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProArea());//第一行
                } else if (mProAreaLength > 13 && mProAreaLength <= 26) {
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;//每次增加RT_TEXT_Y_STEP间隔
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProArea().substring(0, 13));//第一行
                    RT_TEXT_Y = RT_TEXT_Y + RT_TEXT_Y_STEP;
                    printerfont(RT_TEXT_X, RT_TEXT_Y, fontType, 0, 1, 1, cell.getProArea().substring(13, mProAreaLength));//第二行
                }

                printerfont(RT_TEXT_X, RT_TEXT_Y + RT_TEXT_Y_STEP, fontType, 0, 1, 1, "电话: 400-6836-080");//电话
                break;

            // 小条码纸排版 一个标签对应一个商品
            default:
                SYLoger.print("条码打印：小条码纸排版总价：" + cell.mTotalAMT);
                //第一步：商品名称头部
                if (mNewPrinterType) {
                    TEXT_Y = 10;
                } else {
                    TEXT_Y = 40;
                }
                printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, cell.mTitle);

                //第二步：商品总价格、价格和数量
                TEXT_Y = TEXT_Y + TEXT_Y_STEP;
                printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, cell.mSale);
                //第三步：条形码字符串 下面的barcode 的 readable 设置为1后就会自带字符串
                //TEXT_Y = TEXT_Y + TEXT_Y_STEP;
                //printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, cell.mCode);

                //第四步：条形码
                if (PrinterUtil.ONE_BARCODE_DIY.equals(type)) {//60
                    TEXT_Y = TEXT_Y + TEXT_Y_STEP;
                    printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, "五谷磨房 （无条码）");
                    //因为barcode 的 readable 设置为1后直接屏蔽掉了之前的字符串打印，所以针对没有条码的特殊条码--自定码，需要沿用以前的字符串打印方法。
                    TEXT_Y = TEXT_Y + TEXT_Y_STEP;
                    printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, cell.mCode);
                } else {
                    TEXT_Y = TEXT_Y + TEXT_Y_STEP;
                    barcode(TEXT_X2, TEXT_Y, BarcodeUtilKt.barCodeType(cell.mCode), BARCODE_Y, 1, 0, mNarrow, mWide, cell.mCode);
                    //barcodeFinishPrint( StatusDbMap.statusDbMap("PRT103","该条码无法打印！" ).replace(StatusDbMap.PRO_NAME,""));
                }
                break;
        }

        if (mNewPrinterType) {
            printSnbc(1, 1);
        } else {
            printTsc(1, 1);
        }

        Message msg = new Message();
        msg.what = AppConstants.Printer.ONE_SUCCESS;
        msg.arg1 = index;
        msg.obj = cell;
        mHandler.sendMessage(msg);
    }

    private void doPrintTotal(String total) {
        String msg1 = "*********************";
        String msg2 = "#                   #";
        String msg3 = "--->合计:￥" + total;
        String msg4 = "#                   #";
        String msg5 = "*********************";
        clearBuffer();
        String fontType = BarcodeUtilKt.fontType();
        //第五步：合计总价格部分
        if (mNewPrinterType) {
            TEXT_Y = 20;
        } else {
            TEXT_Y = 50;
        }
        printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, msg1);
        TEXT_Y = TEXT_Y + 25;
        printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, msg2);
        TEXT_Y = TEXT_Y + 25;
        printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, msg3);
        TEXT_Y = TEXT_Y + 25;
        printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, msg4);
        TEXT_Y = TEXT_Y + 25;
        printerfont(TEXT_X, TEXT_Y, fontType, 0, 1, 1, msg5);
        if (mNewPrinterType) {
            printSnbc(1, 1);
        } else {
            printTsc(1, 1);
        }
    }

    private void doPrintORCode(int mX, int mY, String mPath, String mFileName) {
        String fontType = BarcodeUtilKt.fontType();
        // 第六步：二维码打印总价
        clearBuffer();
        clearDocument();
        downloadBmpTtfPcx(mPath, mFileName);
        putBitmap(mX, mY, mPath, mFileName);
        printerfont(mX + 30, mY + 320, fontType, 90, 1, 1, "企业微信专属二维码");
        printerfont(mX + 200, mY + 370, fontType, 90, 1, 1, SystemHelper.LoginUserName);
        printTsc(1, 1);
        waitStatusOk();
    }

    private void waitStatusOk() {
        String status;
        int counts = 0;
        do {
            if (mNewPrinterType) {
                status = printerStatusSnbc();
            } else {
                status = printerStatusTsc();
            }
            counts++;
        } while (counts < 100 && (TextUtils.isEmpty(status) || PRINTING.equals(status)));
        if (!WAIT_TAKE_LABEL.equals(status) && !WAIT_PRESS_KEY.equals(status) && !READY.equals(
                status) && !PAUSE.equals(status)) {
            mIsPrinting = false;
            Message msg = new Message();
            msg.what = AppConstants.Printer.ERROR;
            msg.obj = status;
            mHandler.sendMessage(msg);
        }
    }

    @SuppressWarnings("all")
    private void barcodeFinishPrint(String tempMsg) {
//        mHandler.obtainMessage(BARCODE_TO_THOUSAND, String.format("打印失败！只有总金额小于%s时才能打印条码", tempMsg)).sendToTarget();
        mHandler.obtainMessage(AppConstants.Printer.BARCODE_TO_THOUSAND, tempMsg).sendToTarget();
        closeLogic();
    }

    /**
     * 打印标签（只作打印）
     */
    private class PrinterLabelRunnable implements Runnable {
        List<PrinterUtil.MultipleItem> mList = new ArrayList<>();

        public PrinterLabelRunnable(List<PrinterUtil.MultipleItem> list) {
            mList.addAll(list);
        }

        @Override
        public void run() {
            if (null == mList || mList.size() == 0) {
                return;
            }
            SYLoger.print("打印标签--" + mList);
            String fontType = BarcodeUtilKt.fontType();
            mIsPrinting = true;
            synchronized (mLock) {
                if (mIsPrinting) {
                    mHandler.sendEmptyMessage(AppConstants.Printer.SEARCH);
                }
                if (connectLogic(mAddress)) {
                    if (mIsPrinting) {
                        mHandler.sendEmptyMessage(AppConstants.Printer.START);
                    }
                    if (mNewPrinterType) {
                        setupPrinterSnbc(ZUHE_WIDTH_SNBC, ZUHE_HEIGHT_SNBC);
                    } else {
                        //复用之前条码大标签的宽高
                        setupPrinterTsc(ZUHE_WIDTH_TSC, ZUHE_HEIGHT_TSC);
                        clearBuffer();
                        clearDocument();
                    }

                    if (mIsPrinting) {
                        //title 打印的标题
                        String title = mList.get(0).key + " " + mList.get(0).value;
                        RT_TEXT_Y = 5;
                        printerfont(horizontalCenterWidth(title), RT_TEXT_Y, fontType, 0, 1, 1, title);//条码打印标题
                        //content 打印的内容
                        RT_TEXT_Y = 30;
                        for (int i = 1; i < mList.size() - 1; i++) {
                            SYLoger.print("wgmf-->标签打印--" + mList.get(i).key);
                            String printContent = mList.get(i).key + "" + mList.get(i).value;

                            SYLoger.print("wgmf-->标签打印--printContent-->" + printContent);
                            printLabel(printContent);
                        }
                        //printerNum 打印几张（数量）
                        String value = mList.get(mList.size() - 1).value;
                        int printNum = Integer.parseInt(value.isEmpty() ? "0" : value);
                        if (mNewPrinterType) {
                            printSnbc(1, 1);
                        } else {
                            printTsc(1, printNum);
                        }
                    }

                    waitStatusOk();
                    if (mIsPrinting) {
                        mHandler.sendEmptyMessage(AppConstants.Printer.STOP);
                    }
                }
                if (mIsPrinting) {
                    mHandler.sendEmptyMessage(AppConstants.Printer.FINISH);
                }
                closeLogic();
                stopPrint();
            }
        }
    }

    /**
     * 1个标签对应1个商品，（小条码纸）组合码 （大条码纸）大润发码 条码纸张大小需要判断
     */
    private class MyRunnableListLabel implements Runnable {
        List<PrinterUtil.PrinterRule> mList = new ArrayList<>();
        double mTotal;

        private MyRunnableListLabel(List<PrinterUtil.PrinterRule> list, double total) {
            mTotal = total;
            mList.addAll(list);
        }

        @Override
        public void run() {
            if (null == mList || mList.size() == 0) {
                return;
            }
            mIsPrinting = true;
            String barCodePaper = mList.get(0).mBarCodePaper;//条码纸大小：1：小条码纸 2：大条码纸 3：大润发条码纸 4：新大润发22位条码
            PreferHelper.getInstance().setString(AppConstants.PreKeys.BAR_CODE_PAPER, barCodePaper);
            synchronized (mLock) {
                if (mIsPrinting) {
                    mHandler.sendEmptyMessage(AppConstants.Printer.SEARCH);
                }
                if (connectLogic(mAddress)) {
                    if (mIsPrinting) {
                        mHandler.sendEmptyMessage(AppConstants.Printer.START);
                    }
                    //单品码标签的宽高设置
                    if (!mNewPrinterType) {
                        setupPrinterTsc(WIDTH_TSC, ZUHE_HEIGHT_TSC);
                    }

                    SYLoger.print("条码打印" + ", 单品码标签的宽高设置");
                    double total = 0;
                    for (int i = 0; i < mList.size() && mIsPrinting; i++) {
                        PrinterUtil.PrinterRule cell = mList.get(i);
                        String type = cell.mType;
                        double tempSinglePrice = Arith.mul(cell.mPrice, cell.mCount);
                        total += tempSinglePrice;
                        if (checkPrintCellValid(type, i, cell)) {
                            int counts = 0;
                            String status = null;
                            do {
                                if (mNewPrinterType) {
                                    status = printerStatusSnbc();
                                } else {
                                    status = printerStatusTsc();
                                }
                                counts++;
                            }
                            while (counts < 100 && (TextUtils.isEmpty(status) || PRINTING.equals(status)));
                            if (!WAIT_TAKE_LABEL.equals(status) && !WAIT_PRESS_KEY.equals(status) && !READY.equals(status) && !PAUSE.equals(status)) {
                                mIsPrinting = false;
                                Message msg = new Message();
                                msg.what = AppConstants.Printer.ERROR;
                                msg.obj = status;
                                mHandler.sendMessage(msg);
                                break;
                            }
                            if (mNewPrinterType) {
                                if (AppConstants.Printer.SMALL_PAPER.equals(barCodePaper)) {
                                    setupPrinterSnbc(WIDTH_SNBC, HEIGHT_SNBC);
                                } else {
                                    setupPrinterSnbc(ZUHE_WIDTH_SNBC, ZUHE_HEIGHT_SNBC);
                                }
                            }
                            doPrintCell(type, i, cell);
                            try {
                                Thread.sleep(50L);
                            } catch (InterruptedException e) {
                            }
                        }
                    }
                    if (mList.size() > 0 && mIsPrinting) {//多个商品，多个条形码，一个合计总价格
                        if (mNewPrinterType) {
                            setupPrinterSnbc(WIDTH_SNBC, HEIGHT_SNBC);
                        }
                        doPrintTotal(String.format(Locale.CHINA, "%.2f", total));
                        try {
                            Thread.sleep(200L);
                        } catch (InterruptedException e) {
                        }
                    }
                    if (mIsPrinting) {
                        mHandler.sendEmptyMessage(AppConstants.Printer.STOP);
                    }
                }
                if (mIsPrinting) {
                    mHandler.sendEmptyMessage(AppConstants.Printer.FINISH);
                }
                closeLogic();
                stopPrint();
            }
        }
    }

    /**
     * 单个标签对应多个商品 组合码 大条码纸打印
     */
    private class MyRunnableOneLabel implements Runnable {
        List<PrinterUtil.PrinterRule> mList = new ArrayList<>();
        double mTotal;

        private MyRunnableOneLabel(List<PrinterUtil.PrinterRule> list, double total) {
            mTotal = total;
            mList.addAll(list);
        }

        @Override
        public void run() {
            if (null == mList || mList.size() == 0) {
                return;
            }
            mIsPrinting = true;
            String barCodePaper = mList.get(0).mBarCodePaper;//条码纸大小：1：小条码纸 2：大条码纸 3：大润发条码纸
            PreferHelper.getInstance().setString(AppConstants.PreKeys.BAR_CODE_PAPER, barCodePaper);
            synchronized (mLock) {
                if (mIsPrinting) {
                    mHandler.sendEmptyMessage(AppConstants.Printer.SEARCH);
                }
                if (connectLogic(mAddress)) {
                    if (mIsPrinting) {
                        mHandler.sendEmptyMessage(AppConstants.Printer.START);
                    }
                    if (!mNewPrinterType) {
                        setupPrinterTsc(ZUHE_WIDTH_TSC, ZUHE_HEIGHT_TSC);
                    }
                    SYLoger.print("条码打印" + ", 单个标签多商品的宽高设置，大条码纸排版");
                    int count = mList.size();
                    for (int j = 0; j < (count + ONE_PAGE_COUNT - 1) / ONE_PAGE_COUNT; j++) {//纸循环；
                        if (mNewPrinterType) {
                            setupPrinterSnbc(ZUHE_WIDTH_SNBC, ZUHE_HEIGHT_SNBC);
                        } else {
                            clearBuffer();
                        }
                        PrinterUtil.PrinterRule cell = mList.get(ONE_PAGE_COUNT * j);//以一张单的第一个商品的信息生成条形码
                        doPrintTypeTwo(count, j, cell);
                        try {
                            Thread.sleep(50L);
                        } catch (InterruptedException e) {
                        }
                    }
                    if (mIsPrinting) {
                        mHandler.sendEmptyMessage(AppConstants.Printer.STOP);
                    }
                    SYLoger.print("条码打印" + ", 单个标签多商品的.");
                }
                if (mIsPrinting) {
                    mHandler.sendEmptyMessage(AppConstants.Printer.FINISH);
                }
                closeLogic();
                stopPrint();
            }
        }

        /**
         * 一个标签对应多个产品的排版布局
         *
         * @param count ...
         * @param j     ...
         * @param cell  ...
         */
        private void doPrintTypeTwo(int count, int j, PrinterUtil.PrinterRule cell) {
            if (cell.mBarCodePaper.equals(AppConstants.Printer.BIG_PAPER)) {
                String fontType = BarcodeUtilKt.fontType();
                //第一步：头部
                printerfont(horizontalCenterWidth("五谷磨房"), 5, fontType, 0, 1, 1, "五谷磨房");

                //PrinterUtil.PrinterRule cell = mList.get(ONE_PAGE_COUNT * j);//以一张单的但一个商品的信息生成条形码
                String type = cell.mType;
                int lastPageDistance = 350;//默认是直接在底部；40+30*10 = 340;

                double total = 0;
                double totalCount = 0;
                int i = 0;
                for (; i < ONE_PAGE_COUNT && (i < count - ONE_PAGE_COUNT * j) && mIsPrinting; i++) {//当前纸中商品列表循环；
                    int index = ONE_PAGE_COUNT * j + i;
                    PrinterUtil.PrinterRule cellTemp = mList.get(index);

                    totalCount = cellTemp.mCount;
                    total += cellTemp.mPrice * cellTemp.mCount;

                    //第二步：中间商品名称 价格和数量
                    printerfont(LABEL_START_X, 40 + i * 30, fontType, 0, 1, 1, cellTemp.mTitle + " " + (cellTemp.mSale.split(" ")[1]));//按之前空格分组取出数组第二个(价格*数量)
                }
                lastPageDistance = 40 + i * 30 + 10;
                String msg1 = "合计标签:" + i;
                String msg2 = "   总价:￥" + new DecimalFormat(".00").format(total);
                //第三步：合计标签 和总价
                printerfont(LABEL_START_X, lastPageDistance, fontType, 0, 1, 1, msg1 + msg2);

                //未加校验码的barcode
                String barcode = PrinterUtil.makeCodeNotCheck(cell, total, i);
                barcode = BarcodeUtilKt.makeCheckCode(barcode, cell.checkCodeFlag);

                //第四步：条形码
                barcode(LABEL_START_X, lastPageDistance + 30, BarcodeUtilKt.barCodeType(cell.mCode), barCodeHeight, 1, 0, getCodeFont(cell)[0], getCodeFont(cell)[1], barcode);

                SYLoger.print("条码打印: 标签： " + barcode);
            }
            if (mNewPrinterType) {
                printSnbc(1, 1);
            } else {
                printTsc(1, 1);
            }
            waitStatusOk();
        }
    }

    /**
     * 专属二维码打印
     */
    private class MyRunnablePrintBmp implements Runnable {
        int mX, mY;
        String mPath;
        String mFileName;
        String titleText;
        String bottomText;

        private MyRunnablePrintBmp(int x, int y, String path, String fileName, String titleText, String bottomText) {
            mX = x;
            mY = y;
            mPath = path;
            mFileName = fileName;
            this.titleText = titleText;
            this.bottomText = bottomText;
        }

        @Override
        public void run() {
            mIsPrinting = true;
            synchronized (mLock) {
                if (mIsPrinting) {
                    mHandler.sendEmptyMessage(AppConstants.Printer.SEARCH);
                }
                if (connectLogic(mAddress)) {
                    if (mIsPrinting) {
                        mHandler.sendEmptyMessage(AppConstants.Printer.START);
                    }
                    if (mNewPrinterType) {
                        setupPrinterSnbc(ZUHE_WIDTH_SNBC, ZUHE_HEIGHT_SNBC);
                    } else {
                        setupPrinterTsc(ZUHE_WIDTH_TSC, ZUHE_HEIGHT_TSC);
                        clearBuffer();
                        clearDocument();
                        downloadBmpTtfPcx(mPath, mFileName);
                    }
                    putBitmap(mX, mY, mPath, mFileName);
                    String fontType = BarcodeUtilKt.fontType();
                    printerfont(mX + 30, mY + 320, fontType, 0, 1, 1, bottomText);
                    printerfont(mX + 200, mY + 370, fontType, 0, 1, 1, titleText);

                    if (mNewPrinterType) {
                        printSnbc(1, 1);
                    } else {
                        printTsc(1, 1);
                    }
                    waitStatusOk();
                    if (mIsPrinting) {
                        mHandler.sendEmptyMessage(AppConstants.Printer.STOP);
                    }
                }
                if (mIsPrinting) {
                    mHandler.sendEmptyMessage(AppConstants.Printer.FINISH);
                }
                closeLogic();
                stopPrint();
            }
        }
    }

    /**
     * 自动获取大条码纸水平居中的 x 坐标
     * 1行==16 字
     * 1行==16 字
     * 1字==23 （打印机的x距离单元）
     * 1字==3 字节
     *
     * @param str 打印的内容
     * @return 距离原始位的距离 x
     */
    private int horizontalCenterWidth(String str) {
        //1个字等于3字节，这里得到多少个字
        int strLength = str.getBytes().length / 3;
        if (mNewPrinterType) {
            return ((23 * 20) - (23 * strLength)) / 2;
        } else {
            return ((23 * 16) - (23 * strLength)) / 2;
        }
    }

    /**
     * 一行的总字数
     */
    @SuppressWarnings("all")
    private int textYStep = 0;
    private int first = 0;

    /**
     * 标签打印排版
     *
     * @param str ...
     */
    private void printLabel(String str) {
        //待打印的字符串长度
        int strLength = str.length();
        //待打印的字符串字节长度
        int bytesLength = str.getBytes().length;
        //只打印一行字符串最大长度
        int oneLineMaxStrLength = 18;
        //只打印一行字符串最大字节长度
        int oneLineMaxBytesLength = 52;
        int textX = RT_TEXT_X * 2;
        if (!mNewPrinterType) {
            textX = RT_TEXT_X - 16;
            oneLineMaxStrLength = 16;
            oneLineMaxBytesLength = 46;
        }
        String fontType = BarcodeUtilKt.fontType();
        try {
            //打印内容只有1行 ps:有特殊符号的时候需要判断下字节长度
            if (bytesLength < oneLineMaxBytesLength
                    || strLength <= oneLineMaxStrLength) {
                SYLoger.print("wgmf-->标签打印--》只有1行");
                // first (第一行打印内容是1行的时候)
                if (first == 0) {
                    RT_TEXT_Y += textYStep + 10;
                    textYStep = 20;
                    printerfont(textX, RT_TEXT_Y, fontType, 0, 1, 1, str);//第一行
                } else {
                    //second(第一行打印内容是2行的时候) or three(第一行打印内容是3行的时候)
                    RT_TEXT_Y += textYStep + 10;
                    textYStep = 20;
                }
                printerfont(textX, RT_TEXT_Y, fontType, 0, 1, 1, str);//第一行
            }
            //打印内容只有2行
            else if (bytesLength < 2 * oneLineMaxBytesLength
                    || strLength <= 2 * oneLineMaxStrLength) {
                first++;
                SYLoger.print("wgmf-->标签打印--》只有2行");
                RT_TEXT_Y += textYStep + 10;
                printerfont(textX, RT_TEXT_Y, fontType, 0, 1, 1, str.substring(0, oneLineMaxStrLength));//第一行
                RT_TEXT_Y += 30;
                printerfont(textX, RT_TEXT_Y, fontType, 0, 1, 1, str.substring(oneLineMaxStrLength, strLength));//第二行
                //textYStep = 30;
            }
            //打印内容只有3行
            else {
                first++;
                SYLoger.print("wgmf-->标签打印--》只有3行");
                RT_TEXT_Y += textYStep + 10;
                printerfont(textX, RT_TEXT_Y, fontType, 0, 1, 1, str.substring(0, 16));//第一行
                RT_TEXT_Y += 30;
                printerfont(textX, RT_TEXT_Y, fontType, 0, 1, 1, str.substring(16, 32));//第二行
                RT_TEXT_Y += 30;
                printerfont(textX, RT_TEXT_Y, fontType, 0, 1, 1, str.substring(32, strLength));//第三行
                //textYStep = 30;
            }
        } catch (Exception e) {
            Toast.makeText(BaseAppManager.appContext(), StatusDbMap.statusDbMap("PRT102", "当前排版不支持打印!"), Toast.LENGTH_SHORT).show();
            e.printStackTrace();
        }
    }

    /**
     * 打印的内容占几行
     *
     * @param value1 打印的内容的字数
     * @param value2 一行的总字数
     * @return 占用几行，如果占用2行就需要换行打印
     */
    @SuppressWarnings("unused")
    private int getRow(int value1, int value2) {
        BigDecimal decimal1 = new BigDecimal(value1);
        BigDecimal decimal2 = new BigDecimal(value2);
        BigDecimal decimal = decimal1.divideToIntegralValue(decimal2);
        return Integer.parseInt(decimal.toString()) + 1;
    }
}
