package com.urovo.wugumofang.printermodule;

import android.text.TextUtils;

import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.MoneyUtils;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.system.SystemHelper;
import com.zsy.libcore.common.android.log.Log;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libext.utils.DateTimeUtils;

import org.jetbrains.anko.ToastsKt;

import java.util.Calendar;
import java.util.List;
import java.util.Locale;

import androidx.annotation.NonNull;

@SuppressWarnings("all")
public class PrinterUtil {
    private static PrinterUtil mPrinterUtil = null;
    /**
     * 非TSC设备，不支持打印
     */
    public static final int NONE = 0;
    /**
     * TSC设备
     */
    public static final int TSC = 1;
    /**
     * SNBC 设备
     */
    public static final int SNBC = 2;
    /**
     * 类型	打印类型	总价码组成
     * 01	13位
     * 02	13位	7位编码	5位金额	1位序列号
     * 03	13位	8位条码	4位数量	1位序列号
     * -------7位条码	5位数量	1位序列号
     * 11	18位	7位条码	5位数量	5位金额	1位校验码
     * 12	18位	6位条码	3位数量	8位金额（数量*单价）	1位校验码
     * 13	18位	7位条码	5位金额	5位数量	1位校验码
     */
    /**
     * 以下为单品类型:
     */
    /**
     * 8位单品
     */
    public static final String ONE_BARCODE_8 = "8";
    /**
     * 9位单品
     */
    public static final String ONE_BARCODE_9 = "9";
    /**
     * 11位单品
     */
    public static final String ONE_BARCODE_11 = "11";
    /**
     * 12位单品
     */
    public static final String ONE_BARCODE_12 = "12";
    /**
     * 13位单品
     */
    public static final String ONE_BARCODE_13 = "13";
    /**
     * 13位单品，大润发特殊条码
     */
    public static final String ONE_BARCODE_13_RT_MART = "13_1";
    /**
     * 18位单品
     */
    public static final String ONE_BARCODE_18 = "18";
    /**
     * 20位单品
     */
    public static final String ONE_BARCODE_20 = "20";
    /**
     * 22位单品
     */
    public static final String ONE_BARCODE_22 = "22";
    /**
     * 28位单品
     */
    public static final String ONE_BARCODE_28 = "28";
    /**
     * ,特殊单品码 bartype为60，并不是60位
     */
    public static final String ONE_BARCODE_DIY = "60";
    /**
     * 以下为组合类型：
     */
    /**
     * 13位	7位编码	5位金额	1位序列号
     */
    public static final String ZUHE_13_751 = "ZH13751";
    /**
     * 18位	6位条码	3位数量	8位金额	1位校验码
     */
    public static final String ZUHE_18_6381 = "ZH186381";
    /**
     * 18位	7位条码	5位金额  5位数量	 1位校验码
     */
    public static final String ZUHE_18_7551_1 = "ZH1875511";
    /**
     * 18位	7位条码 	5位金额  5位数量(3位小数)	 1位校验码
     */
    public static final String ZUHE_18_7551_3 = "ZH1875513";
    /**
     * 18位	6位条码	8位金额  3位数量	 1位校验码
     */
    public static final String ZUHE_18_6831 = "ZH186831";
    /**
     * 18位	7位条码  5位数量	 5位金额	 1位校验码
     */
    public static final String ZUHE_18_7551_2 = "ZH1875512";

    /**
     * 20位 9位条码 5位数量  5位金额   1位校验码
     */
    public static final String ZUHE_20_9551 = "ZH209551";

    /**
     * 22位 6位条码 5位数量  7位金额   1位校验码
     */
    public static final String ZUHE_22_9571 = "ZH229571";

    /**
     * 24位 7位条码   5位金额  5位数量  6位保质期  1位校验码
     */
    public static final String ZUHE_24_7551 = "ZH247551";

    /**
     * 28位 6位条码 5位数量  7位金额  1位校验码
     */
    public static final String ZUHE_28_9661 = "ZH289661";
    /**
     * 18位	6位条码  5位数量 	6位金额	 1位校验码
     */
    public static final String ZUHE_18_6561 = "ZH186561";
    /**
     * (type为'24'的'20位额数8651组合码') 8位条码 6位金额	 5位数量 	1位校验码
     */
    public static final String ZUHE_20_8651 = "ZH208651";

    private Printer mPrinter;
    protected String mAddress;

    /**
     * 打印ITEM（基本包含所有所选的产品数据）
     */
    public static class PrinterRule implements Cloneable {
        /**
         * 条码类型
         */
        public String mType;
        /**
         * 产品名称
         */
        public String mTitle;
        /**
         * 产品单价
         */
        public double mPrice;
        /**
         * 产品数量
         */
        public double mCount;
        /**
         * 产品金额
         */
        public double mTotalAMT;

        /**
         * 合计：总价格;二维码上面部分
         */
        public String mSale;

        /**
         * 商品条形码barcode
         */
        public String mCode;
        /**
         * 新价格比例,默认比例为1
         */
        public double newPricePercentage = 1;

        /**
         * 成分
         */
        public String mIngredient;

        /**
         * 保存及食用方法
         */
        public String mSaveCondition;

        private String proArea = "";//产地
        private String proSupplier = "";//供应商

        /**
         * 纸张大小
         */
        public String mBarCodePaper;

        /**
         * 打印规则
         */
        public String mPrintRule;

        // 条码类型：单品码(A) 组合码(B) 大润发码(C)
        public String barCodeType;

        // 商超产品编码（barcode）长度
        public int barCodeLength;

        // 组合码规则1（金额：M或数量:N或无：“”）
        public String combinedCodeTypeOne;

        //规则1长度
        public int combinedCodeTypeLengthOne;

        //规则1小数点精确度
        public int combinedCodeTypeDotOne;

        // 组合码规则2（金额：M或数量:N或无：“”）
        public String combinedCodeTypeTwo;

        //规则2长度
        public int combinedCodeTypeLengthTwo;

        //规则2小数点精确度
        public int combinedCodeTypeDotTwo;

        //默认保质期是在规则二后，但是有可能后台配置，就跑到了规则一或者规则二的位置，
        // 所以下面定义的字段意思在某些时候可能是金额或者数量，不一定是保质期。
        //规则3（期限保质期--标志）/金额 数量
        public String shelfLifeTypeThree;
        //规则3（期限保质期--条码字符串长度）
        public int shelfLifeLengthThree;
        //规则3（期限保质期--保存天数）/精确度
        public int shelfLifeDaysThree;

        //校验码标志有或无（T:true/F:false）
        public boolean checkCodeFlag;

        //是否是特殊BOM产品，y:是
        public String BOMSPEC;

        public String getProArea() {
            return proArea;
        }

        public void setProArea(String proArea) {
            this.proArea = proArea;
        }

        public String getProSupplier() {
            return proSupplier;
        }

        public void setProSupplier(String proSupplier) {
            this.proSupplier = proSupplier;
        }

        @Override
        public PrinterRule clone() {
            PrinterRule o = null;
            try {
                o = (PrinterRule) super.clone();
            } catch (CloneNotSupportedException e) {
                e.printStackTrace();
            }
            return o;
        }

        @Override
        public String toString() {
            return "PrinterRule{" +
                    "mType='" + mType + '\'' +
                    ", mTitle='" + mTitle + '\'' +
                    ", mPrice=" + mPrice +
                    ", mCount=" + mCount +
                    ", mTotalAMT=" + mTotalAMT +
                    ", mSale='" + mSale + '\'' +
                    ", mCode='" + mCode + '\'' +
                    ", newPricePercentage=" + newPricePercentage +
                    ", mIngredient='" + mIngredient + '\'' +
                    ", mSaveCondition='" + mSaveCondition + '\'' +
                    ", proArea='" + proArea + '\'' +
                    ", proSupplier='" + proSupplier + '\'' +
                    ", mBarCodePaper='" + mBarCodePaper + '\'' +
                    ", mPrintRule='" + mPrintRule + '\'' +
                    ", barCodeType='" + barCodeType + '\'' +
                    ", barCodeLength=" + barCodeLength +
                    ", combinedCodeTypeOne='" + combinedCodeTypeOne + '\'' +
                    ", combinedCodeTypeLengthOne=" + combinedCodeTypeLengthOne +
                    ", combinedCodeTypeDotOne=" + combinedCodeTypeDotOne +
                    ", combinedCodeTypeTwo='" + combinedCodeTypeTwo + '\'' +
                    ", combinedCodeTypeLengthTwo=" + combinedCodeTypeLengthTwo +
                    ", combinedCodeTypeDotTwo=" + combinedCodeTypeDotTwo +
                    ", shelfLifeTypeThree='" + shelfLifeTypeThree + '\'' +
                    ", shelfLifeLengthThree=" + shelfLifeLengthThree +
                    ", shelfLifeDaysThree=" + shelfLifeDaysThree +
                    ", checkCodeFlag=" + checkCodeFlag +
                    '}';
        }
    }

    /**
     * 打印标签
     */
    public static class MultipleItem {
        public String key;
        public String value;

        public MultipleItem(String key, String value) {
            this.key = key;
            this.value = value;
        }

        @Override
        public String toString() {
            return "MultipleItem{" +
                    "key='" + key + '\'' +
                    ", value='" + value + '\'' +
                    '}';
        }
    }

    public interface PrintCallback {
        public void onSearchPrint();

        public void onFinishPrint();

        public void onStartPrint();

        public void onStopPrint();

        public void onAddressInvalidOrCanNotFound();

        public void onPrinterError(String error);

        public void onPrintOneSuccessed(int index, String title, String sale, String code);

        public void onBarCodeTypeInvalid(String type);

        public void onBarCodeIsInvalid(int index, String title, String sale, String code);

        /**
         * 打印失败！只有总金额小于999.99时才能打印条码
         */
        void onPrinterErrorBarcodeToThousand(String error);
    }

    public static PrinterUtil getInstance() {
        if (null == mPrinterUtil) {
            mPrinterUtil = new PrinterUtil();
        }
        return mPrinterUtil;
    }

    private boolean setMachine(String addr, int machine, PrintCallback callback) {
        mAddress = null;
        if (checkAddress(addr)) {
            mAddress = addr;
        } else {
            return false;
        }
        stopPrint();
        if (machine == TSC || machine == SNBC) {
            mPrinter = new PrinterManage(mAddress, callback);
        } else {
            return false;
        }
        return true;
    }

    private boolean checkAddress(String addr) {
        if (null == addr || TextUtils.isEmpty(addr)) {
            return false;
        }
        return true;
    }

    /**
     * 小条码纸打印
     *
     * @param addr     ...
     * @param machine  ...
     * @param callback ...
     * @param list     ...
     * @param total    ...
     */
    public void printMsg(String addr, int machine, PrintCallback callback, List<PrinterRule> list,
                         double total) {
        if (!setMachine(addr, machine, callback)) {
            throw new IllegalArgumentException("PrinterUtil need a collect addr and machine! ");
        }
        if (null != mPrinter) {
            mPrinter.printMsg(list, total);
        }
    }

    /**
     * 大条码纸打印
     *
     * @param addr     ...
     * @param machine  ...
     * @param callback ...
     * @param list     ...
     * @param total    ...
     */
    public void printGroupMsg(String addr, int machine, PrintCallback callback,
                              List<PrinterRule> list, double total) {
        if (!setMachine(addr, machine, callback)) {
            throw new IllegalArgumentException("PrinterUtil need a collect addr and machine! ");
        }
        if (null != mPrinter) {
            mPrinter.printGroupMsg(list, total);
        }
    }

    public void printBitmap(String addr, int machine, PrintCallback callback, int x, int y,
                            String path, String fileName, String titleText, String bottomText) {
        if (!setMachine(addr, machine, callback)) {
            throw new IllegalArgumentException("PrinterUtil need a collect addr and machine! ");
        }
        if (null != mPrinter) {
            mPrinter.printBitmap(x, y, path, fileName, titleText, bottomText);
        }
    }

    /**
     * 打印标签
     *
     * @param addr
     * @param machine
     * @param callback
     * @param list
     */
    public void printLabel(String addr, int machine, PrintCallback callback, List<MultipleItem> list) {
        if (!setMachine(addr, machine, callback)) {
            throw new IllegalArgumentException("PrinterUtil need a collect addr and machine! ");
        }
        if (null != mPrinter) {
            mPrinter.printLabel(list);
        }
    }

    public void stopPrint() {
        if (null != mPrinter) {
            mPrinter.stopPrint();
        }
    }

    /**
     *
     * 13位 计算校验位过程【13位的校验码＝10－（偶数位和*3+奇数位和）取个位数】
     * 1.先取偶数位相加的结果乘以3，得到S1
     * 2.取奇数位相加的结果，得到S2
     * 3.S1和S2相加的结果S3，取S3的个位数S4
     * 4.10减去S4的结果S5
     * 5.校验位为S5%10
     * 示例：
     * 213787375520 ----> 2137873755206
     * s1=(1+7+7+7+5+0)*3=27*3=81
     * s2=2+3+8+3+5+2=23
     * s3=s1+s2=81+23=104
     * s4=s3%10=104%10=4
     * s5=10-s4=10-4=6
     * x = 6%10=6
     *
     * @param barcode 数据库查询的商品条形码
     * @param isCheck 后台返回规则判断是否需要校验
     * @return 最终校验生成的商品条形码
     */

    /**
     * 18位 计算校验位过程【18位的校验码＝10－（偶数位和+奇数位和*3）取个位数】,跟上面算法一样，区别在于位数
     * 1.先取偶数位相加的结果，得到S1
     * 2.取奇数位相加的结果乘以3，得到S2
     * 3.S1和S2相加的结果S3，取S3的个位数S4
     * 4.10减去S4的结果S5
     * 5.校验位为S5%10
     * 示例：
     * 23876610000122800 ----> 238766100001228000
     * s1=3+7+6+0+0+1+2+0=19
     * s2=(2+8+6+1+0+0+2+8+0)*3=27*3=81
     * s3=s1+s2=19+81=100
     * s4=s3%10=100%10=0
     * s5=10-s4=10-0=10
     * x = 10%10=0
     *
     * @param barcode 数据库查询的商品条形码
     * @return 最终校验生成的商品条形码
     */
    public String barcode20Check(String barcode, boolean isCheck) {
        return barcodeCheckByType(barcode, BarCodeType.TYPE_20, isCheck);
    }

    /**
     * 【新：下面的注释不用看了，我已经提取出来一个barcodeCommonCheck()通用方法了，13、22、28等等的计算规则都是一样的】
     * <p>
     * 22位 计算校验位过程【22位的校验码＝10－（偶数位和*3+奇数位和）取个位数】,跟上面13位数算法一样，区别在于位数
     * 1.先取偶数位相加的结果，得到S1
     * 2.取奇数位相加的结果乘以3，得到S2
     * 3.S1和S2相加的结果S3，取S3的个位数S4
     * 4.10减去S4的结果S5
     * 5.校验位为S5%10
     *
     * @param barcode 数据库查询的商品条形码
     * @param isCheck 后台返回规则判断是否需要校验
     * @return 最终校验生成的商品条形码
     */
    public String barcode22Check(String barcode, boolean isCheck) {
        return barcodeCheckByType(barcode, BarCodeType.TYPE_22, isCheck);
    }

    /**
     * 固定1位0+4位随机码组成
     *
     * @param barcode 数据库查询的商品条形码
     * @return 最终校验生成的商品条形码
     */
    public String barcode26Check(String barcode) {
        return barcode + "0" + generateRandom(4);
    }

    /**
     * 生成固定位数随机数
     *
     * @param digitNumber 随机数位数
     * @return
     */
    private static String generateRandom(int digitNumber) {
        if (digitNumber == 0) {
            return "";
        }
        String randomStr = String.valueOf((int) ((Math.random() * 9 + 1) * (Math.pow(10, digitNumber - 1))));
        return randomStr;
    }

    /**
     * 18位 计算校验位过程【18位的校验码＝10－（偶数位和+奇数位和*3）取个位数】,跟上面算法一样，区别在于位数
     * 1.先取偶数位相加的结果，得到S1
     * 2.取奇数位相加的结果乘以3，得到S2
     * 3.S1和S2相加的结果S3，取S3的个位数S4
     * 4.10减去S4的结果S5
     * 5.校验位为S5%10
     * <p>
     * 22位 计算校验位过程【22位的校验码＝10－（偶数位和*3+奇数位和）取个位数】,跟上面13位数算法一样，区别在于位数
     * 1.先取偶数位相加的结果，得到S1
     * 2.取奇数位相加的结果乘以3，得到S2
     * 3.S1和S2相加的结果S3，取S3的个位数S4
     * 4.10减去S4的结果S5
     * 5.校验位为S5%10
     * <p>
     * 1234 --12342
     * s1=1+3=4
     * s2=2+4=6
     * s2=4*3=12
     * s3=18
     * s4=18%10=8
     * s5=10-8=2
     * s=2%10=8
     */
    public String barcodeCheckByType(String barCode, BarCodeType barCodeType, boolean isCheck) {
        if (isCheck) {
            int evenNumberSum = 0;//s1
            int oddNumberSum = 0;//s2
            int s3 = 0;

            barCode = barCode.replace(" ", "");

            for (int index = 0; index < barCode.length(); index++) {
                if (index % 2 != 0) {// 偶数位求和
                    evenNumberSum = evenNumberSum + Integer.parseInt(barCode.charAt(index) + "");
                } else {
                    oddNumberSum = oddNumberSum + Integer.parseInt(barCode.charAt(index) + "");
                }
            }

            switch (barCodeType) {
                case TYPE_20://奇数位之和
                    oddNumberSum = oddNumberSum * 3;
                    break;
                case TYPE_22://偶数位之和
                    evenNumberSum = evenNumberSum * 3;
                    break;
                default:
                    break;
            }
            s3 = oddNumberSum + evenNumberSum;
            int s4 = s3 % 10;
            int x = (10 - s4) % 10;
            return barCode + x;
        }
        return barCode;
    }

    /**
     * 目前有两种类型（TYPE_20代表奇数位求和* 3，TYPE_22代表偶数位求和* 3）
     */
    public enum BarCodeType {
        TYPE_20, TYPE_22
    }

    /**
     * 校验末尾是否需要添加校验码
     *
     * @param c       ...
     * @param barcode ...
     */
    public void checkLastCode(PrinterUtil.PrinterRule c,
                              String barcode,
                              List<PrinterUtil.PrinterRule> printList,
                              @NonNull CodePrinter.PrinterCallBack printerCallBack) {
        if ("111".equals(c.mType) || "112".equals(c.mType)) {
            // 26位总价码检验位：固定1位0+4位验证码组成）
            c.mCode = PrinterUtil.getInstance().barcode26Check(barcode);
        } else {
            if (!c.checkCodeFlag && c.barCodeLength == 0) {
                // 自定义码，不需要处理，直接打印
                c.mType = PrinterUtil.ONE_BARCODE_DIY;
            } else if (barcode.length() == c.barCodeLength + c.combinedCodeTypeLengthOne + c.combinedCodeTypeLengthTwo + c.shelfLifeLengthThree) {
                //【B7变B9情况】以前默认是7，后面出了个28位的条码类型，barCodeLength就变成9了，可能还有其他类型，故设置个临时变量。如果涉及到这个问题，后面排版对应的条码code长度也要去看看。
                c.mCode = BarcodeUtilKt.makeCheckCode(barcode, c.checkCodeFlag);
            } else {
                printList.clear();
            }
        }
        if (TextUtils.isEmpty(c.mCode)) {
            printerCallBack.displayError(MoneyUtils.displayError(c.mTitle, "条码校验失败"));
            String msg = StatusDbMap.statusDbMap("PRT108", "打印条码标题为：" + c.mTitle + " 的条码校验失败，该条码无法打印！")
                    .replace(StatusDbMap.PRO_NAME, c.mTitle);
            printerCallBack.showToast(msg);
        }
    }

    /**
     * 打印排版
     *
     * @param rule      规则
     * @param barcode   ...
     * @param printList 打印的列表（所选产品数据都以rule对象存在其中）
     */
    public void printCode(PrinterUtil.PrinterRule rule, String barcode, List<PrinterUtil.PrinterRule> printList) {
        switch (rule.mBarCodePaper) {
            case AppConstants.Printer.SMALL_PAPER:
                rule.mType = PrinterUtil.ONE_BARCODE_13;
                SYLoger.print("条码打印: 小条码纸");
                break;
            case AppConstants.Printer.BIG_PAPER:
                rule.mType = PrinterUtil.ZUHE_13_751;
                SYLoger.print("条码打印: 大条码纸");
                break;
            case AppConstants.Printer.RTMART_PAPER:
            case AppConstants.Printer.COMBINED_CODE_BIG_PAPER:
                rule.mType = PrinterUtil.ONE_BARCODE_13_RT_MART;
                SYLoger.print("条码打印: 大润发条码纸");
                break;
        }
        //添加到打印集合中（列表）狗日，原来PrinterTool那边的集合是这里添加进来的
        printList.add(rule);
        SYLoger.print("条码打印： " + "LoginOrganId=" + SystemHelper.LoginOrganId + rule);
    }

    /**
     * 金额上限限制
     *
     * @param rule ...
     */
    public void amountLimitation(PrinterUtil.PrinterRule rule,
                                 int amountCodeLength, int accuracy,
                                 List<PrinterUtil.PrinterRule> printList,
                                 @NonNull CodePrinter.PrinterCallBack printerCallBack) {
        boolean mIsShowToast = PreferHelper.getInstance().getBoolean(AppConstants.Printer.IS_SHOW_TOAST, true);
        double totalAMT = (Math.pow(10, amountCodeLength) - 1) / (Math.pow(10, accuracy));//总金额 如：10的4次方-1=9999，保留2位再除以100，以此类推
        //如果是M金额码的时候，amountCodeLength accuracy 都为0，不在处理限制
        if (totalAMT == 0) {
            return;
        }
        if (rule.mTotalAMT > totalAMT && mIsShowToast) {//总金额
            printList.clear();
            printerCallBack.displayError(MoneyUtils.displayError(rule.mTitle, "金额超过" + totalAMT));
            String msg = StatusDbMap.statusDbMap("PRT106", "打印条码标题为：" + rule.mTitle + " 的金额超过" + totalAMT + "，该条码无法打印！")
                    .replace(StatusDbMap.PRO_NAME, rule.mTitle)
                    .replace(StatusDbMap.PRO_AMOUNT, totalAMT + "");
            printerCallBack.showToast(msg);
            // 处理组合条纸码混合打印，某张纸商品总价超出金额上限问题
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
        }
    }

    /**
     * 数量上限限制
     *
     * @param rule          ...
     * @param numCodeLength ...
     * @param accuracy      ...
     */
    public void numLimitation(PrinterUtil.PrinterRule rule,
                              int numCodeLength, int accuracy,
                              List<PrinterUtil.PrinterRule> printList,
                              @NonNull CodePrinter.PrinterCallBack printerCallBack) {
        double totalNum = (Math.pow(10, numCodeLength) - 1) / (Math.pow(10, accuracy));//总数量 如：10的4次方-1=9999，保留2位再除以100，以此类推
        //如果是N数量码的时候，amountCodeLength accuracy 都为0，不在处理限制
        if (totalNum == 0) {
            return;
        }
        if (rule.mCount > totalNum) {//总数量
            printList.clear();
            SYLoger.print("条码打印：数量超过：" + MoneyUtils.noRoundAdjustment(totalNum));
            printerCallBack.displayError(MoneyUtils.displayError(rule.mTitle, "数量超过" + MoneyUtils.noRoundAdjustment(totalNum)));
            String msg = StatusDbMap.statusDbMap("PRT105", "打印条码标题为：" + rule.mTitle + " 的数量超过" + MoneyUtils.noRoundAdjustment(totalNum) + "，该条码无法打印！")
                    .replace(StatusDbMap.PRO_NAME, rule.mTitle)
                    .replace(StatusDbMap.PRO_NUM, MoneyUtils.noRoundAdjustment(totalNum));
            printerCallBack.showToast(msg);
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
        }
    }

    /**
     * 组装条码（还没加上末尾校验码）
     *
     * @param rule   ...
     * @param amount ...金额
     * @param count  ...数量（或重量）
     * @return ...
     */
    @NonNull
    static public String makeCodeNotCheck(PrinterUtil.PrinterRule rule, double amount, double count) {
        String stepOne = "";
        String stepTwo = "";
        String stepThree = "";
        String barcode = "";

        //如果是欧尚和大润发超市使用的”一品一码”即26位总价码
        //具体需求如下：
        //1）需开发的条码类型为：26位总价码（由货号9位+重量6位+价格6位+固定1位0+4位验证码组成）
        //2）需打印出以下两款货号，如下（红豆薏米粉单价为111->84元/斤，核桃芝麻粉单价为112->99元/斤）
        //3）重量=价格/单价*500,重量四舍五入取整
        //4）金额保留两位小数
        if ("111".equals(rule.mType) || "112".equals(rule.mType)) {
            if ("111".equals(rule.mType)) {
                stepOne = MoneyUtils.format2Str(amount / 84 * 500, rule.combinedCodeTypeLengthOne, rule.combinedCodeTypeDotOne);
            }
            if ("112".equals(rule.mType)) {
                stepOne = MoneyUtils.format2Str(amount / 99 * 500, rule.combinedCodeTypeLengthOne, rule.combinedCodeTypeDotOne);
            }
            stepTwo = MoneyUtils.format2Str(amount, rule.combinedCodeTypeLengthTwo, rule.combinedCodeTypeDotTwo);
            stepThree = "";
        } else {
            if ("M".equals(rule.combinedCodeTypeOne)) {
                //amount
                stepOne = MoneyUtils.format2Str(amount, rule.combinedCodeTypeLengthOne, rule.combinedCodeTypeDotOne);
            } else if ("N".equals(rule.combinedCodeTypeOne)) {
                //num
                stepOne = MoneyUtils.format2Str(count, rule.combinedCodeTypeLengthOne, rule.combinedCodeTypeDotOne);
            } else if ("ET".equals(rule.combinedCodeTypeOne)) {
                //shelfLifeDays
                stepOne = getShelfLifeDays(rule.combinedCodeTypeLengthOne, rule.combinedCodeTypeDotOne);
            } else {
                stepOne = "";
            }

            if ("M".equals(rule.combinedCodeTypeTwo)) {
                //amount
                stepTwo = MoneyUtils.format2Str(amount, rule.combinedCodeTypeLengthTwo, rule.combinedCodeTypeDotTwo);
            } else if ("N".equals(rule.combinedCodeTypeTwo)) {
                //num
                stepTwo = MoneyUtils.format2Str(count, rule.combinedCodeTypeLengthTwo, rule.combinedCodeTypeDotTwo);
            } else if ("ET".equals(rule.combinedCodeTypeTwo)) {
                //shelfLifeDays
                stepTwo = getShelfLifeDays(rule.combinedCodeTypeLengthTwo, rule.combinedCodeTypeDotTwo);
            } else {
                stepTwo = "";
            }

            if ("M".equals(rule.shelfLifeTypeThree)) {
                //amount
                stepThree = MoneyUtils.format2Str(amount, rule.shelfLifeLengthThree, rule.shelfLifeDaysThree);
            } else if ("N".equals(rule.shelfLifeTypeThree)) {
                //num
                stepThree = MoneyUtils.format2Str(count, rule.shelfLifeLengthThree, rule.shelfLifeDaysThree);
            } else if ("ET".equals(rule.shelfLifeTypeThree)) {
                //shelfLifeDays
                stepThree = getShelfLifeDays(rule.shelfLifeLengthThree, rule.shelfLifeDaysThree);
            } else {
                stepThree = "";
            }
        }
        //这里还是要subString截断的，因为后面也要调用，传进来的mCode不一定只能是产品码，也可能是无校验码的完整码
        if (rule.mCode.length() >= rule.barCodeLength) {//如果后台配置错误，比如mCode为空，则这里不能让它崩溃，要校验一下。
            barcode = rule.mCode.substring(0, rule.barCodeLength) + stepOne + stepTwo + stepThree;
        }
        return barcode;
    }

    /**
     * 根据保质期的长度选择对应的
     */
    static private String getShelfLifeDays(int shelfLifeLength, int shelfLifeDays) {
        String shelfLifeDaysStr = "";
        if (shelfLifeLength == 6) {
            shelfLifeDaysStr = DateTimeUtils.getDay(Calendar.getInstance(), "yyMMdd", shelfLifeDays);
            SYLoger.print("shelfLifeDays --> " + shelfLifeDays);
        } else if (shelfLifeLength == 8) {
            shelfLifeDaysStr = DateTimeUtils.getDay(Calendar.getInstance(), "yyyyMMdd", shelfLifeDays);

        } else if (shelfLifeLength == 0) {
            //后台没有配置或者配错了导致这个字段没有赋值
        } else {
            Log.e("PrinterUtil", "保质期长度不对！！！！！！！！！！！！！！");
            ToastsKt.toast(BaseAppManager.appContext(), StatusDbMap.statusDbMap("PRT107", "保质期长度不对！"));
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
        }
        return shelfLifeDaysStr;
    }

    /**
     * 初始化判断(组合码使用)
     *
     * @param rule 规则
     */
    public void initJudgement(PrinterUtil.PrinterRule rule, @NonNull CodePrinter.PrinterCallBack printerCallBack) {
        String mTotalAMT = String.format(Locale.CHINA, "%.2f", rule.mTotalAMT);
        String mPrice = String.format(Locale.CHINA, "%.2f", rule.mPrice);
        rule.mSale = String.format(Locale.CHINA, "￥%s (%s*%d)", mTotalAMT, mPrice, (int) rule.mCount);//合计:总价格

        if (TextUtils.isEmpty(rule.mCode)) {
            printerCallBack.displayError(MoneyUtils.displayError(rule.mTitle, "该商品无条码"));
            String msg = StatusDbMap.statusDbMap("PRT103", "打印条码标题为：" + rule.mTitle + " 的该商品无条码，该条码无法打印！").replace(StatusDbMap.PRO_NAME, rule.mTitle);
            printerCallBack.showToast(msg);
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
            SYLoger.print("question: 无条码，这里应该终止其他产品打印的流程。");
            return;
        }
        if (rule.mCode.length() != rule.barCodeLength) {
            SYLoger.print("条码打印B: 位数： rule.mCode:" + rule.mCode + ", rule.mCode length: " + rule.mCode.length() + ", barCodeLength:" + rule.barCodeLength);
            printerCallBack.displayError(MoneyUtils.displayError(rule.mTitle, "条码不是" + rule.barCodeLength + "位"));
            String msg = StatusDbMap.statusDbMap("PRT104", "打印条码标题为：" + rule.mTitle + " 的条码不是" + rule.barCodeLength + "位，该条码无法打印！")
                    .replace(StatusDbMap.PRO_NAME, rule.mTitle);
            printerCallBack.showToast(msg);
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
            SYLoger.print("question: 条码位数，这里应该终止其他产品打印的流程。");
            return;
        }
        double totalAMT;
        // 前面遍历算出所选产品总价，在条码制作中，是大条码纸组合码就必须判断总金额上限
        if (AppConstants.Printer.mCombineCode.equals(rule.barCodeType)
                && AppConstants.Printer.BIG_PAPER.equals(rule.mBarCodePaper)) {//规则：B2
            if (!TextUtils.isEmpty(rule.combinedCodeTypeOne)
                    && rule.combinedCodeTypeOne.equals(AppConstants.Printer.AMOUNT_CODE)) {
                totalAMT = (Math.pow(10, rule.combinedCodeTypeLengthOne) - 1) / (Math.pow(10, rule.combinedCodeTypeDotOne));//总金额
            } else {
                totalAMT = (Math.pow(10, rule.combinedCodeTypeLengthTwo) - 1) / (Math.pow(10, rule.combinedCodeTypeDotTwo));//总金额
            }
            boolean isOpen = PreferHelper.getInstance().getBoolean(AppConstants.Printer.IS_OPEN, false);// 在循环中只执行一次的开关
            List<Double> totalAMTs = PreferHelper.getInstance().getList(AppConstants.Printer.TOTAL_AMTS);// 遍历所选产品得到的总价（每9款产品一张纸算）
            SYLoger.print("totalPrices: " + "isOpen--" + isOpen + ", totalAMTs:" + totalAMTs.toString());
            //因为在前面PrinterTool P383行方法中已经遍历过，不再多次执行,并且单次商品金额不超过上限
            if (isOpen) {
                for (Double total : totalAMTs) {//大条码纸组合码多张纸金额上限处理
                    if (total > totalAMT) {//总金额
                        String msg = StatusDbMap.statusDbMap("PRT106", "商品总金额超过" + totalAMT + "，该条码无法打印！")
                                .replace(StatusDbMap.PRO_NAME, "")
                                .replace(StatusDbMap.PRO_AMOUNT, totalAMT + "");
                        printerCallBack.showToast(msg);
                        PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_SHOW_TOAST, false);//设置吐司开关为关闭，屏蔽上面吐司以外的吐司
                        //标记一下有问题的纸张，按照单品组合码的判断方式打个标记，在遍历完后清空集合
                        PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_END_PRINTER, true);
                    }
                }
                // 开关
                PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_OPEN, false);
            }
            SYLoger.print("totalPrices -- " + totalAMT + "----mTotalAMT: " + rule.mTotalAMT);
        } else {
            PreferHelper.getInstance().setBoolean(AppConstants.Printer.IS_SHOW_TOAST, true);// 单品组合码的时候需要把标志状态恢复回来
        }
    }
}
