package com.urovo.wugumofang.printermodule

import com.urovo.wgmf.db.dao.dao.PrinterRetcodeDbOperate
import com.urovo.wugumofang.utils.notNullAndEmpty

/**
 * @author Sasuke on 2018/8/7.
 * 把打印机返回的错误码/状态码，映射成后台给的对应的提示信息
 *
 * PRT100	请选择产品再进行打印操作！
PRT101	门店未配置条码信息，找销管部易晨！
PRT102	条码规则解析出错，找销管部易晨维护！
PRT103	proName产品条码配置错误，找销管部易晨维护！
PRT104	proName产品维护的条码长度有误，找销管部易晨！
PRT105	proName产品的数量超过了最大值proNum，请减少数量再打印！
PRT106	proName产品金额超过最大值proAmount，请分多单打印！
PRT107	保质期配置有误，请在平台反馈群进行反馈！
PRT108	proName产品校验码验证失败，请在平台反馈群进行反馈！
PRT109	proName产品条码长度判断错误，请在平台反馈群进行反馈
PRT110	全部条码打印完成
PRT111	条码标签获取失败，请在平台反馈群进行反馈！
PRT112	蓝牙设备不具备打印功能，重启打印机再试或更改打印机
PRT113	地址有误或者不在附近，请重启打印机再返回重新打印或者配对蓝牙
PRT114	打印机连接超时
PRT115	您已经取消了打印
PRT116	在打开蓝牙，请过会重新点击打印！
00	待机
01	打印头开启
02	纸张卡纸
03	纸张卡纸并且打印头开启
04	缺纸
05	缺纸并且打印头开启
08	无碳带
09	无碳带并且打印头开启
OA	无碳带并且卡纸
OB	无碳带、卡纸并且打印头开启
OC	无碳带并且缺纸
OD	无碳带、缺纸并且打印头开启
10	暂停
20	打印中
80	其它错误
 *
 */

object StatusDbMap {
    const val PRO_NAME = "proName"
    const val PRO_NUM = "proNum"
    const val PRO_AMOUNT = "proAmount"

    @JvmStatic
    fun statusDbMap(status: String): String {
        val dbStatus = PrinterRetcodeDbOperate.getInfoByStatusCode(status)
        return when (status) {
            "00" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.READY = it }; PrinterManage.READY
            }
            "01" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.HEAD_OPEN = it }; PrinterManage.HEAD_OPEN
            }
            "02" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.PAPER_JAM = it }; PrinterManage.PAPER_JAM
            }
            "03" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.HEAD_OPEN_AND_PAPER_JAM = it }; PrinterManage.HEAD_OPEN_AND_PAPER_JAM
            }
            "04" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.PAPER_EMPTY = it }; PrinterManage.PAPER_EMPTY
            }
            "05" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.HEAD_OPEN_AND_PAPER_EMPTY = it }; PrinterManage.HEAD_OPEN_AND_PAPER_EMPTY
            }
            "08" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.RIBBON_EMPTY = it }; PrinterManage.RIBBON_EMPTY
            }
            "09" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.HEAD_OPEN_AND_RIBBON_EMPTY = it }; PrinterManage.HEAD_OPEN_AND_RIBBON_EMPTY
            }
            "0A" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.PAPER_JAM_AND_RIBBON_EMPTY = it }; PrinterManage.PAPER_JAM_AND_RIBBON_EMPTY
            }
            "0B" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.HEAD_OPEN_AND_PAPER_JAM_AND_RIBBON_EMPTY = it }; PrinterManage.HEAD_OPEN_AND_PAPER_JAM_AND_RIBBON_EMPTY
            }
            "0C" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.PAPER_EMPTY_AND_RIBBON_EMPTY = it }; PrinterManage.PAPER_EMPTY_AND_RIBBON_EMPTY
            }
            "0D" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.HEAD_OPEN_AND_PAPER_EMPTY_AND_RIBBON_EMPTY = it }; PrinterManage.HEAD_OPEN_AND_PAPER_EMPTY_AND_RIBBON_EMPTY
            }
            "10" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.PAUSE = it }; PrinterManage.PAUSE
            }
            "20" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.PRINTING = it }; PrinterManage.PRINTING
            }
            "30" ->{
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.HEAD_OVERHEAT = it }; PrinterManage.HEAD_OVERHEAT
            }
            "40" ->{
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.CUTTER_ERROR_STATE = it }; PrinterManage.CUTTER_ERROR_STATE
            }
            "80" -> {
                dbStatus.takeIf { it.notNullAndEmpty() }?.let { PrinterManage.OTHER_PROBLEM = it }; PrinterManage.OTHER_PROBLEM
            }

            else -> dbStatus
        }
    }

    /**
     * 传入一个defaultStatus，如果调上面的statusDbMap返回为空，就把这个默认的status给返回去
     */
    @JvmStatic
    fun statusDbMap(status: String, defaultStatus: String): String {
        return statusDbMap(status).takeIf { it.notNullAndEmpty() }?.run { this } ?: defaultStatus
    }
}