package com.urovo.activity.MemberProfile.view

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.Base3Response
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.MyToast
import kotlinx.android.synthetic.main.guanhuai_feedback_act.*
import okhttp3.Request

/**
 * @author HUANGSHI-PC on 2020-03-03 0003.
 */
class GuanhuaiFeedbackAct : BaseActivity() {
    val mobile by lazy { intent.getStringExtra("mobile") }
    val taskId by lazy { intent.getStringExtra("taskId") }
    val staffId by lazy { SystemHelper.LoginUserId }

    companion object {
        const val FEEDBACK_SUCCESS = 1828

        @JvmStatic
        fun launchGuanhuaiFeedbackAct(context: Activity, mobile: String, taskId: String) {
            context.startActivityForResult(Intent(context, GuanhuaiFeedbackAct::class.java).apply {
                putExtra("mobile", mobile)
                putExtra("taskId", taskId)
            }, 0)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.guanhuai_feedback_act)
        initTitle()
        init()
    }


    fun init() {
        titleTextView.text = "回访结果反馈"
        feedbackSubmitBtn.isEnabled = hasBuy.isChecked || hasNotBuy.isChecked
        hasBuyGroup.setOnCheckedChangeListener { group, checkedId -> feedbackSubmitBtn.isEnabled = true }

        feedbackSubmitBtn.setOnClickListener {
            val feedback = if (hasBuy.isChecked) "1" else "2"
            ViewUtil.showProgress(this@GuanhuaiFeedbackAct, "请稍等", "正在提交...")
            OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.GUANHUAI_FEED_BACK,
                    mapOf("mobile" to mobile,
                            "taskId" to taskId,
                            "staffId" to staffId,
                            "feedback" to feedback), object : OkHttpDominator.ResultCallback<Base3Response>() {
                override fun onError(request: Request?, e: Exception?) {
                    ViewUtil.dismissProgress()
                    MyToast.show("网络异常: ${e?.message}")
                }

                override fun onResponse(res: Base3Response?) {
                    ViewUtil.dismissProgress()
                    res?.let {
                        MyToast.show(res.msgInfo)
                        if (res.msgCode == 0) {
                            setResult(FEEDBACK_SUCCESS, null)
                            finish()
                        }
                    }
                }
            })
        }

    }
}