package com.urovo.activity.addsellactivity.presenter

import com.google.gson.Gson
import com.sibyl.addsellactivity.presenter.AddSellContract
import com.urovo.activity.addsellactivity.module.MainData
import com.urovo.activity.addsellactivity.net.ProStockRes
import com.urovo.wgmf.db.dao.dao.Item001DbOperate
import com.urovo.wgmf.db.dao.entitys.ProductEntity
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.Arith
import com.urovo.wugumofang.data.PreferHelper
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.SystemHelper
import okhttp3.Request

/**
 * @author Sasuke on 2018/1/3.
 */
class AddSellMainPre(val mainView: AddSellContract.mainView) : AddSellContract.mainPre {
    val data: MainData by lazy {
        MainData()
    }

    init {
        mainView.setPresenter(this)
    }

    override fun start() {
        mainView.init()
    }


    override fun initAllList(
        storeId: String,
        intentSelected: List<ProductEntity>?,
        actListIds: List<String>?,
        phoneNum: String?,
        isMemGoods: Boolean
    ) {
        //先从数据库里把所有可销售的都加载出来
        if (isMemGoods) {//不是会员商品情况
            loadAllMemGoods(storeId, intentSelected)
        } else {//是会员商品情况
            loadAllListData(storeId, intentSelected)
        }
        saveActIdList(actListIds)//再把其中与之前传进来的活动id相同的去掉
        removeNewBazhen(phoneNum)//好像是 如果该号码买过八珍，才显示环保装，否则不显示
        removeOfflineRightsCard()//移除线下支付权益卡
    }

    /**
     * 从数据库加载所有可销售产品列表
     */
    fun loadAllListData(storeId: String, intentSelected: List<ProductEntity>?) {
        val selectedList = intentSelected?.map { it.itemId }
            ?: listOf() //？.  ?:是为了保证该list不为空，方便后面做item in list的判断
        intentSelected?.forEach {
            if (SystemHelper.isWgmf(it.brand)) it.amount = Arith.mul(it.tqty, it.price)
        }//如果是五谷磨房，就重新计算一下总价
        //这里的逻辑是，先把上个页面传来的产品按id删除总表的同款产品，再把传来的产品添加进总表，
        //达到更新总表已选产品信息目的
        data.originAllList = Item001DbOperate.GetCanXiaoShouProducts(storeId, "")//查（先不用品牌过滤，先查所有！）
        data.allList = data.originAllList//过滤
            .filter { it.brandId.toString() in SystemHelper.LoginUserBrandList }//按品牌筛选
            .filterNot { it.itemId in selectedList }//过滤同款
            .filterNot { it.itemType.equals("1") }//过滤公共产品为1,不显示
            .toMutableList().apply {
                if (intentSelected != null) addAll(0, intentSelected)//添加传进来的
                sortBy { it.sortNo }//排序，其实并没什么鬼用，好像大家都是0
            }
        //初始化两个品牌的分类标签
        data.wgmfLabels = data.allList.filter { it.brand == SystemHelper.WgmfBrand }
            .distinctBy { it.classid }.filterNot { it.classid.isNullOrEmpty() }//有些classId是“”,要去掉
            .sortedBy { it.classidorder }
            .flatMap { listOf(it.classid) }.toMutableList().apply { add("全部") }

        data.xdyLabels =
            data.allList.filter { it.brand != SystemHelper.WgmfBrand && it.brand != SystemHelper.MemGoodsBrand }//这里不能直接==鲜稻研，因为可能还会有散装
                .distinctBy { it.classid }
                .filterNot { it.classid.isNullOrEmpty() }//有些classId是“”,要去掉
                .sortedBy { it.classidorder }
                .flatMap { listOf(it.classid) }.toMutableList().apply { add("全部") }
    }


    /**
     * 从数据库加载所有可销售产品列表【会员商品版】
     */
    fun loadAllMemGoods(storeId: String, intentSelected: List<ProductEntity>?) {
        val selectedList = intentSelected?.map { it.itemId }
            ?: listOf() //？.  ?:是为了保证该list不为空，方便后面做item in list的判断
        intentSelected?.forEach { it.amount = Arith.mul(it.tqty, it.price) }//如果是五谷磨房，就重新计算一下总价
        //这里的逻辑是，先把上个页面传来的产品按id删除总表的同款产品，再把传来的产品添加进总表，
        //达到更新总表已选产品信息目的
        data.originAllList = Item001DbOperate.GetCanXiaoShouProducts(storeId, "")//查（先不用品牌过滤，先查所有！）
        data.allList = data.originAllList//过滤
            .filter { it.brand == SystemHelper.MemGoodsBrand }//按品牌筛选
            .filterNot { it.itemId in selectedList }//过滤同款
            .toMutableList().apply {
                if (intentSelected != null) addAll(0, intentSelected)//添加传进来的
                sortBy { it.sortNo }//排序，其实并没什么鬼用，好像大家都是0
                forEach { it.tqty = 0.0 }//会员商品因为会有库存为0的情况，所以默认数量为0
            }
        //初始化两个品牌的分类标签（现在新加了会员商品品牌）
        data.wgmfLabels = data.allList.filter { it.brand == SystemHelper.MemGoodsBrand }
            .distinctBy { it.classid }.filterNot { it.classid.isNullOrEmpty() }//有些classId是“”,要去掉
            .sortedBy { it.classidorder }
            .flatMap { listOf(it.classid) }.toMutableList().apply { add("全部") }

        data.xdyLabels = mutableListOf("全部")

    }

    /**请求库存*/
    fun queryStock() {

        OkHttpDominator.postAsynByType(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_PRO_STOCK,
            OkHttpDominator.JSON,
            mapOf("productIds" to Gson().toJson(data.allList.map { it.itemId })),
            object : OkHttpDominator.ResultCallback<ProStockRes>() {
                override fun onError(request: Request?, e: Exception?) {
                    mainView.refreshStockFailed()
                    mainView.loadWaitError("获取产品库存失败")
                }

                override fun onResponse(res: ProStockRes?) {
                    data.allList.forEach { pro ->
                        var stock = res?.resultInfo?.find { it.productId == pro.itemId }?.stock
                        pro.stock_Qty = stock.toString().run { if (this == "null") "0" else this }
                    }
                    mainView.refreshStockSuccess()
                    mainView.loadWaitDismiss()
                }
            })
    }

    /**
     * 全部产品 过滤掉活动产品（避免重复添加
     */
    fun allListNoAct(): List<ProductEntity> {
        data.actListIds?.let {
            return data.allList.filterNot { it.itemId in data.actListIds!! }
        }
        return data.allList
    }

    /**
     * 根据给的品牌和过滤标签获得相应的产品
     */
    fun getProByBrandLabel(brand: String, label: String): List<ProductEntity> {
        return if (label == "全部") allListNoAct().filter { it.brand == brand }
        else allListNoAct().filter { it.brand == brand && it.classid == label }
    }

    /**
     * 计算总数（eg.一个产品选了5个也算5，并不是产品类数）
     */
    fun countProNum(): Double {
        return data.allList?.sumByDouble { if (it.isChecked) it.tqty else 0.0 }
    }

    /**
     * 计算总价（这里需要用BigDecimal进行精确计算）
     */
    fun countProAmount(): Double {
        return data.allList.fold(0.00) { amount, productEntity ->
            Arith.add(amount, if (productEntity.isChecked) productEntity.amount else 0.00)
        }
    }

    /**
     * 过滤活动产品用的
     */
    fun saveActIdList(actListIds: List<String>?) {
        data.actListIds = actListIds
    }

    /**
     * 八珍环保装过滤
     */
    private fun removeNewBazhen(phoneNum: String?) {
        var hasBuyBazhen = 0
        var hasBuyNewBazhen = 0
        phoneNum?.let {
            hasBuyBazhen =
                if (PreferHelper.getInstance().getString(AppConstants.ExtraKeys.HAS_BUY_BAZHEN)
                        .contains(phoneNum)
                ) 1 else 0
            hasBuyNewBazhen =
                if (PreferHelper.getInstance().getString(AppConstants.ExtraKeys.HAS_BUY_NEW_BAZHEN)
                        .contains(phoneNum)
                ) 1 else 0
        }
        data.allList =
            data.allList.filterNot { (it.itemId == "2.40172" && hasBuyBazhen == 0) || (it.itemId == "2.40173" && hasBuyNewBazhen == 0) }
    }

    /**
     * 移除线下支付权益卡
     */
    private fun removeOfflineRightsCard() {
        data.allList = data.allList.filterNot { it.itemId == "2.70224" }
    }


    /**
     * 刷新底栏价格显示
     */
    override fun refreshAmount() {
        mainView.refreshShow()
    }

    /**
     * 打包所有已选数据
     */
    override fun packAll(): List<ProductEntity> {
        return data.allList.filterNot {
            !it.isChecked || when (true) {//如果是五谷磨房，就过滤掉数量为0的；如果是鲜稻研，就过滤掉总价为0的
                SystemHelper.isWgmf(it.brand) || it.brand == SystemHelper.MemGoodsBrand -> it.tqty == 0.00
                else -> it.amount == 0.00
            }
        }
    }
}