package com.urovo.activity.addsellactivity.ui

import android.util.TypedValue
import android.widget.LinearLayout
import android.widget.RadioButton
import android.widget.RadioGroup
import org.jetbrains.anko.dip
import org.jetbrains.anko.firstChild
import org.jetbrains.anko.forEachChild
import org.jetbrains.anko.forEachChildWithIndex

/**
 * @author Sasuke on 2018/1/3.
 * 没时间解释了，现在只做一个横向的动态添加RadioButton,并实现单选点击，回调响应
 */
class AutoRadioGroup(val radioGroup: LinearLayout) {
    init {
        removeAllChild()
    }

    /**
     * 直接添加默认的宽wrap  高match的View
     */
    fun addView(view: RadioButton, text: String, margin: Array<Int>? = arrayOf(4, 0, 4, 0), paramWith: Int = RadioGroup.LayoutParams.WRAP_CONTENT,
                paramheight: Int = RadioGroup.LayoutParams.MATCH_PARENT, listener: ((pos: Int) -> Unit)?) {
        addViewWithLayoutParams(view, text, margin, paramWith, paramheight, listener)
    }


    fun addViewWithLayoutParams(view: RadioButton, text: String?, paramWith: Int, paramheight: Int, listener: ((pos: Int) -> Unit)?) {
        addViewWithLayoutParams(view, text, null, paramWith, paramheight, true, listener)//默认weight 为1F
    }


    /**
     * 通过自己自定义宽高的LayoutParams来添加View（默认weight 为1F）
     */
    fun addViewWithLayoutParams(view: RadioButton, text: String?, margin: Array<Int>? = null, paramWith: Int, paramheight: Int, listener: ((pos: Int) -> Unit)?) {
        addViewWithLayoutParams(view, text, margin, paramWith, paramheight, true, listener)//默认weight 为1F
    }


    fun addViewWithLayoutParams(view: RadioButton, text: String?, margin: Array<Int>?, paramWith: Int, paramheight: Int, isExpand: Boolean, listener: ((pos: Int) -> Unit)?) {
        radioGroup.addView(view.apply {
            layoutParams = RadioGroup.LayoutParams(paramWith, paramheight, if (isExpand) 1F else 0F).apply {
                margin?.let {
                    if (margin.isNotEmpty() && margin.size == 4) {
                        setMargins(dip(margin.get(0)), dip(margin.get(1)), dip(margin.get(2)), dip(margin.get(3)))
                    }
                }
                setTextSize(TypedValue.COMPLEX_UNIT_SP, 15f)
                setText(text)
                setOnCheckedChangeListener { btn, isChecked ->
                    if (isChecked) {
                        radioGroup.forEachChild { if (btn != it) (it as RadioButton).isChecked = false }//点击一个，就把其余的全都置为false
                        listener?.invoke(radioGroup.indexOfChild(btn))
                    }
                }
            }
        })
    }


    fun setChildChecked(pos: Int) {
        radioGroup.apply {
            //先把所有子按钮全都置为“未选”,再把目标子按钮置为“已选”
            //forEachChild { (it as RadioButton).isChecked = false }
            val view = getChildAt(pos)
            if (view != null) {
                (view as RadioButton).isChecked = true
            }
        }
    }

    /**
     * 返回被选中的孩子的显示文字
     */
    fun getCheckedText(): String {
        return (radioGroup.firstChild { (it as RadioButton).isChecked } as RadioButton?)?.text.toString()
    }

    /**
     *返回被选中的孩子的序号
     */
    fun getCheckedPos(): Int {
        radioGroup.forEachChildWithIndex { pos, view ->
            if ((view as RadioButton).isChecked) {
                return pos
            }
        }
        return -1
    }

    /**
     * 卖掉所有孩子
     */
    fun removeAllChild() {
        radioGroup.removeAllViews()
    }
}