package com.urovo.activity.attendancemanagement.attendancemanager.net;

import com.urovo.activity.attendancemanagement.attendancemanager.ent.ShiftConfirmResponseEntity;
import com.urovo.activity.attendancemanagement.attendancemanager.ent.ShiftQueryEntity;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.CommonController;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.net.RetrofitHelper;
import com.urovo.wugumofang.response.CommonEntity;
import com.urovo.wugumofang.ui.ViewUtil;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libcore.model.bean.RequestResult;
import com.zsy.libcore.model.bean.ResultBaseCallback;
import com.zsy.libcore.model.bean.ResultCallback;

import java.util.Map;

import okhttp3.Request;
import retrofit2.Call;
import retrofit2.Response;

/**
 * 考勤管理界面功能接口
 *
 * @author chenji last on 2018/1/2 19:51
 */

public class AttendanceManagerNet {
    private IAttendanceManagerNet mIAttendanceManagerNet;

    /**
     * retrofit的配置
     */
    public AttendanceManagerNet() {
        mIAttendanceManagerNet = RetrofitHelper.createApi(IAttendanceManagerNet.class);
    }

    /**
     * 排班查询
     *
     * @param presenter ...
     */
    public void start2ShiftQueryRequest(final CommonController.BasePre<ShiftQueryEntity.ResultBean> presenter, Map<String, String> params) {
        final Call<ShiftQueryEntity> call = mIAttendanceManagerNet.getShiftQueryList(params);
        call.enqueue(new ResultBaseCallback<ShiftQueryEntity>() {
            @Override
            public void onResponse(Call<ShiftQueryEntity> call, Response<ShiftQueryEntity> response) {
                super.onResponse(call, response);
                if (response.isSuccessful()) {
                    final ShiftQueryEntity responseBody = response.body();
                    if (responseBody.getMsgCode() != 0) {
                        presenter.onFailure("排班查询异常！");
                    } else {//正常情况下
                        presenter.onResponseSuccess(new CommonEntity<>(response.body().getResult()));
                    }
                } else {
                    this.onFailure(call, new Throwable(response.errorBody().toString()));
                }
            }

            @Override
            public void onFailure(Call<ShiftQueryEntity> call, Throwable t) {
                if (presenter != null) {
                    presenter.onFailure("排班查询异常：" + t);
                }
            }
        });
    }

    /**
     * 排班确认
     */
    public void start2ShiftConfirmRequest(final CommonController.ShiftBasePre<ShiftQueryEntity.ResultBean> presenter, Map<String, String> params) {
        OkHttpDominator.postAsynByType(NetParam.YINXIAOBU_SERVER + ApiUrls.YXGJ.SHIFT_CONFIRM,
                OkHttpDominator.JSON,
                params,
                new OkHttpDominator.ResultCallback<ShiftConfirmResponseEntity>() {
                    @Override
                    public void onError(Request request, Exception e) {
                        ViewUtil.dismissProgress();
//                        MyToast.show(context, "排班确认异常：" + e, Toast.LENGTH_SHORT);
                        presenter.onFailure("排班确认异常：" + e);
                    }

                    @Override
                    public void onResponse(ShiftConfirmResponseEntity res) {
                        ViewUtil.dismissProgress();
                        if (res == null) {
//                            MyToast.show("后台返回为空");
                            presenter.onFailure("后台返回为空：");
                            return;
                        }
                        //上传成功
                        if (res.getMsgCode() == 0) {
//                            MyToast.show(context, res.getMsgContent(), Toast.LENGTH_SHORT);
                            presenter.onConfirmResponseSuccess(res.getMsgContent());
                            //手动把确认状态恢复，下次遍历排班列表的时候，如果存在未确认的，就设置为true
//                            PreferHelper.getInstance().setBoolean(AppConstants.ShiftQueryAndConfirm.ISUNCONFIRMED, false);
                        }
                    }
                });

//        final Call<ShiftConfirmResponseEntity> call = mIAttendanceManagerNet.getShiftConfirmList(params);
//        call.enqueue(new ResultBaseCallback<ShiftConfirmResponseEntity>() {
//            @Override
//            public void onResponse(Call<ShiftConfirmResponseEntity> call, Response<ShiftConfirmResponseEntity> response) {
//                super.onResponse(call, response);
//                if (response.isSuccessful()) {
//                    final ShiftConfirmResponseEntity responseBody = response.body();
//                    if (responseBody.getMsgCode() != 0) {
//                        MyToast.show(context, "排班确认异常！", Toast.LENGTH_SHORT);
//                    } else {//正常情况下
//                        SYLoger.print("chenji-->" + response.body().getMsgContent());
//                        MyToast.show(context, response.body().getMsgContent(), Toast.LENGTH_SHORT);
//                    }
//                } else {
//                    this.onFailure(call, new Throwable(response.errorBody().toString()));
//                }
//            }
//
//            @Override
//            public void onFailure(Call<ShiftConfirmResponseEntity> call, Throwable t) {
//                MyToast.show(context, "排班确认异常：" + t, Toast.LENGTH_SHORT);
//            }
//        });
    }

    /**
     * 是否显示排班设置
     *
     * @param presenter ...
     * @param params    ...
     */
    public void start2RequestShiftSetting(final CommonController.BasePre<Boolean> presenter, Map<String, String> params) {
        final Call<RequestResult<Boolean>> call = mIAttendanceManagerNet.getShiftSettingFlag(params);
        call.enqueue(new ResultCallback<Boolean>() {
            @Override
            protected void onResponseSuccess(RequestResult<Boolean> responseBody) {
                super.onResponseSuccess(responseBody);
                if (responseBody.msgCode == 0) {
                    presenter.onResponseSuccess(new CommonEntity<>(responseBody.resultInfo));
                    SYLoger.print("lemmonade --" + responseBody.resultInfo.toString());
                } else {
                    presenter.onFailure("排班设置接口数据异常！");
                }
            }

            @Override
            public void onFailure(Call<RequestResult<Boolean>> call, Throwable t) {
                presenter.onFailure("排班设置接口数据异常:" + t);
            }
        });
    }

}
