package com.urovo.activity.checkGoodsTabAct.presenter

import com.urovo.activity.checkGoodsTabAct.service.CheckGoodsDepository
import com.urovo.wgmf.db.dao.dao.Cpbill002DbOperate
import com.urovo.wgmf.db.dao.entitys.CanCheckRes3
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.JsonDominator
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isNullOrEmpty
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.model.bean.RequestResult
import okhttp3.Request
import retrofit2.Call
import retrofit2.Response

/**
 * 盘点 Presenter
 * @author LW
 */
class CheckGoodsSubmitListPresenter(
    private var view: CheckGoodsSubmitListContract.IView?,
    private var depository: CheckGoodsDepository = CheckGoodsDepository()
) : AbsPresenter(),
    CheckGoodsSubmitListContract.IPresenter {

    private var resultDatas: List<CanCheckRes3.ResultInfo>? = null

    /**
     * 获取盘点数据
     */
    override fun requestCurrentTransferCheck() {
        view?.showTipDialog("正在获取本期盘点数据...")
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.CURRENT_TRANSFER_CHECK,
            mapOf("storeNo" to SystemHelper.LoginOrganId, "userNo" to SystemHelper.LoginUserId),
            object : OkHttpDominator.ResultCallback<CanCheckRes3>() {
                override fun onError(request: Request, e: Exception) {
                    view?.showError(e.message)
                    view?.dismissDialog()
                }

                override fun onResponse(response: CanCheckRes3?) {
                    view?.dismissDialog()
                    var res = response
                    JsonDominator.loadFromSD("CanCheckRes3", CanCheckRes3::class.java)?.let {
                        res = it
                    }

                    if (res?.msgCode != AppConstants.Http.RESPONSE_SUCCESS) {
                        res?.let { view?.showError(it.msgInfo) }
                        return
                    }

                    if (res == null || res?.resultInfo == null || res!!.resultInfo.all { it.detailList.isEmpty() }) {
                        view?.showState(LoadWaitDominator.NO_DATA)
                        return
                    }
                    //处理一下返回结果（品牌分类，排序）
                    res!!.resultInfo.forEach { resultInfo: CanCheckRes3.ResultInfo ->
                        resultInfo.pdUserName = SystemHelper.LoginUserName
                        resultInfo.detailList.forEach {
                            it.pdUserNo = SystemHelper.LoginUserId
                            it.isDecimal = Cpbill002DbOperate.isDecimalById(it.proCode)
                        }
                    }
//                        res!!.resultInfo.brandList.forEach { brandList ->
//                            brandList.detailList = res!!.resultInfo.detailList
//                                    .filter {
//                                        it.pdUserNo = SystemHelper.LoginUserId
//                                        it.isDecimal = Cpbill002DbOperate.isDecimalById(it.proCode)
//                                        it.brandId == brandList.brandId
//                                    }
//                        }

                    resultDatas = res?.resultInfo
                    view?.currentTransferCheckToView(resultDatas)
                }
            })
    }

    override fun requestTransferCheck() {
        val storeNo = SystemHelper.LoginOrganId
        if (storeNo.isNullOrEmpty()) {
            view?.showError(R.string.common_load_error)
            return
        }
        depository.requestTransferCheck(storeNo).enqueue(object :
            RetrofitCallback<RequestResult<Any>>() {
            override fun onResponse(
                call: Call<RequestResult<Any>>,
                res: Response<RequestResult<Any>>
            ) {
                val body = res.body()
                body?.let {
                    if (it.msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                        view?.transferCheckStatus(true, "")
                    } else {
                        view?.transferCheckStatus(false, it.msgInfo)
                    }
                }
            }

            override fun onFailure(exception: ApiException) {
                view?.showError(exception.message)
            }
        })
    }
}