package com.urovo.activity.checkGoodsTabAct.presenter

import com.google.gson.Gson
import com.urovo.activity.qualityBackAct.model.QBackInput
import com.urovo.wgmf.adapter.PanDianDiffAdapter
import com.urovo.wgmf.db.dao.dao.Djpd001DbOperate
import com.urovo.wgmf.db.dao.entitys.CanCheckRes3
import com.urovo.wgmf.db.dao.entitys.PanDianEntity
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.Arith
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.zsy.libcore.model.bean.RequestResult
import okhttp3.Request
import java.util.*
import kotlin.math.abs

/**
 * @author LW on 2022/8/18.
 */
class CheckGoodsSubmitPresenter(private var view: CheckGoodsSubmitContract.IView?) : AbsPresenter(),
    CheckGoodsSubmitContract.IPresenter {

    var mBackInput: QBackInput = QBackInput()
    var mResultInfo: CanCheckRes3.ResultInfo? = null

    var mRemarkText = ""
    var mUrlImage = ""

    var mAllDiffNum = 0.0
    var mAllDiffAmount = 0.0

    override fun submit() {
        val param = HashMap<String, String>()
        var url = NetParam.YINXIAOBU_SERVER + ApiUrls.APP.SUBMIT_TRANSFER_CHECK
        if (mRemarkText.notNullAndEmpty()) {
            url += "?remark=" + mRemarkText
        }
        if (mUrlImage.notNullAndEmpty()) {
            url += "?url=" + mUrlImage
        }

        mResultInfo?.detailList?.forEach {
            if (it.pdAlterNum != null) {
                it.pdQty = it.pdAlterNum!!
                it.pdAmount = Arith.mul(it.pdQty ?: 0.0, it.proPrice ?: 0.0)
                it.diffQty = abs(Arith.sub(it.pdQty ?: 0.0, it.stockQty ?: 0.0))
                it.diffAmount =
                    Arith.mul(Arith.sub(it.pdQty ?: 0.0, it.stockQty ?: 0.0), (it.proPrice ?: 0.0))
            }
        }

        param["detailList"] = Gson().toJson(mResultInfo?.detailList)
        OkHttpDominator.postAsynByType(url, OkHttpDominator.JSON,
            param, object : OkHttpDominator.ResultCallback<RequestResult<Any>>() {
                override fun onError(request: Request, e: Exception) {
                    view?.showToast(e.message)
                    view?.dismissDialog()
                }

                override fun onResponse(response: RequestResult<Any>) {
                    view?.dismissDialog()
                    response.let {
                        if (it.msgCode != AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.showToast(it.msgInfo)
                            return
                        }
                        Djpd001DbOperate.UpdateBillState(mPdBill)// 如果上传成功，更新单据上传状态字段
                        view?.showToast(response.msgInfo)
                        //弹出差异弹窗
                        view?.showPanDianDiffDialog(getDiffList())
                        view?.resetDataToView()
                    }
                }
            })
    }

    /**
     * 整理出差异数据
     */
    private fun getDiffList(): List<PanDianDiffAdapter.Diff> {
        val diffList = ArrayList<PanDianDiffAdapter.Diff>()

        mResultInfo?.detailList?.filter { it.pdQty != it.stockQty }?.forEach { pro ->
            diffList.add(PanDianDiffAdapter.Diff().apply {
                entity = pro
                currQty = pro.stockQty.toString()//当前账面库存
            })
            mAllDiffNum += pro.diffQty ?: 0.0
            mAllDiffAmount += pro.diffAmount ?: 0.0
        }
        return diffList
    }

    /**
     * 校验盘点列表合法性
     */
    override fun checkListCorrectAndSubmit(): Boolean {
        if (mResultInfo == null || mResultInfo?.detailList?.size == 0) {
            return false
        }
        // 保存盘点数据到本地：盘点主表和明细表
        var zeroCount = 0
        mResultInfo?.detailList?.forEach { pro ->
            if (mResultInfo?.detailList?.any { !it.isPanDian } == true) {
                view?.showToast("还有产品未盘点！")
                return false
            } else {
                if (!matchConditionsNotEdit(pro)) {
                    if (pro.pdAlterNum == 0.0) {
                        zeroCount += 1
                    } else {
                        zeroCount += 0
                    }
                }
            }
        }

        var info = "提交后将不能再次盘点，请确认是否提交！"
        if (zeroCount != 0) {
            info = "本月盘点共有 ${zeroCount} 种产品数量为0，${info}"
        }
        var zero = false
        if (zeroCount == mResultInfo?.detailList?.size) {
            zero = true
        }
        view?.showAlertDialog(zero, info) {
            savePanDianDataToDb()
        }// 保存盘点数据到本地：盘点主表和明细表
        return true
    }

    fun includeSpecialProduct(): Boolean {
        mResultInfo?.let {
            it.detailList?.forEach {
                if (matchConditionsNotEdit(it)) {
                    return true
                }
            }
        }
        return false
    }

    /**
     * 符合下列条件，不能编辑
     */
    fun matchConditionsNotEdit(item: CanCheckRes3.ResultInfo.DetailList?): Boolean {
        item?.let {
            if ("2.20211" == it.proCode || "2.20212" == it.proCode || "2.20214" == it.proCode ||
                "2.20215" == it.proCode || "2.20216" == it.proCode || "2.20217" == it.proCode
            ) {
                return true
            }
        }
        return false
    }

    override fun saveAndSubmit() {
        view?.savePanDianDataToDbAndSubmit {
            savePanDianDataToDb()
        }
    }

    private val mPdBill by lazy {
        SystemHelper.LoginOrganId + DateUtil.format(mPdDay, "yyyyMMdd")
    }

    private val mPdDay by lazy {
        Calendar.getInstance().time//反正现在都由后端来判断了，前端就不用判断逻辑了，直接返回日期标记就行
    }

    /**
     * 把盘点数据存到db数据库
     */
    private fun savePanDianDataToDb(): Boolean {
        //以下是从老的submit()中移过来的，把盘点数据保存到本地数据库（虽然我也不清楚这有什么卵用）
        val pan = PanDianEntity().apply {
            storeId = SystemHelper.LoginOrganId
            oper_Lr = SystemHelper.LoginUserId
            lr_Name = SystemHelper.LoginUserName
            wDate = DateUtil.getDate(Date())
            pdDate = DateUtil.getDate(mPdDay)
            remark = mRemarkText
            url = mUrlImage
            bill = mPdBill
        }
        Djpd001DbOperate.DeletePanDian(pan.bill)
        return true
    }

}