package com.urovo.activity.common;

import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.widget.ExpandableListView;
import android.widget.TextView;

import com.google.gson.Gson;
import com.urovo.activity.qualityBackAct.model.DispatchDetail;
import com.urovo.activity.qualityBackAct.model.DispatchDetailCollect;
import com.urovo.activity.qualityBackAct.model.DispatchDetailCollectData;
import com.urovo.activity.qualityBackAct.ui.OrderGoodsChooseMonthAdapter;
import com.urovo.wgmf.db.dao.entitys.ProductEntity;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.data.StringUtils;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.ViewUtil;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import okhttp3.Request;

/**
 * 根据最近30天的已收货单来选择产品
 * <p>
 * Created by xiep on 2020/6/3.
 */
public class OrderGoodsChoseMonthActivity extends BaseActivity implements OrderGoodsChooseMonthAdapter.OnItemViewListener {
    public static final int activityResultCode = 20;// 选择成功后返回的code
    public static final String activityResultData = "activityResultData";// 选择成功后返回的code
    public static final int type_Oder = 1;// 订货状态
    public static final int type_quality = 4;//上报反馈
    public static final String Search_type = "Search_type";// 需查询的类别
    private TextView texttitle;
    private ExpandableListView exList;
    private int type = type_Oder;// 查询状态默认为订货
    private String storeid;
    private OrderGoodsChooseMonthAdapter exAdapter;
    private List<DispatchDetailCollect> dispatchDetailCollects = new ArrayList<>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.goods_chose_month);
        initTitle();
        init();
        getData();
    }

    private void getData() {
        if (!TextUtils.isEmpty(SystemHelper.LoginOrganId) && !TextUtils.isEmpty(SystemHelper.LoginUserId)) {
            HashMap<String, String> param = new HashMap<>();
            param.put("storeNo", SystemHelper.LoginOrganId);
            param.put("userNo", SystemHelper.LoginUserId);
            ViewUtil.showProgress(OrderGoodsChoseMonthActivity.this, "", "请稍等.....");
            OkHttpDominator.postAsyn(NetParam.URL_DISPATCH_PRODUCT, param,
                    new OkHttpDominator.ResultCallback<String>() {
                        @Override
                        public void onError(Request request, Exception e) {
                            ViewUtil.dismissProgress();
                            showMsg("获取30天之内已收货的配送收货单的产品明细" + getString(R.string.request_data_fail));
                        }

                        @Override
                        public void onResponse(String res) {
                            ViewUtil.dismissProgress();
                            if (res == null || TextUtils.isEmpty(res)) {
                                showMsg("没有返回数据");
                            } else {
                                DispatchDetailCollectData dispatchDetailCollectData = new Gson().fromJson(res, DispatchDetailCollectData.class);
                                if (null != dispatchDetailCollectData) {
                                    if (dispatchDetailCollectData.msgCode == 0) {
                                        if (null != dispatchDetailCollectData.getResultInfo() && !dispatchDetailCollectData.getResultInfo().isEmpty()) {
                                            dispatchDetailCollects = dispatchDetailCollectData.getResultInfo();


                                            setListener();
                                        }
                                    } else {
                                        showMsg(dispatchDetailCollectData.msgInfo);
                                    }
                                }
                            }
                        }
                    });
        }
    }

    private void init() {
        Intent intent = getIntent();
        storeid = intent.getStringExtra("storeid");
        type = intent.getIntExtra(Search_type, type_Oder);// 取传过来的查询种类
        texttitle = (TextView) findViewById(R.id.text_title);
        exList = (ExpandableListView) findViewById(R.id.qback_main_menu);
        texttitle.setText("选择产品");
    }

    private void setListener() {
        exAdapter = new OrderGoodsChooseMonthAdapter(this, dispatchDetailCollects, this);
        exList.setGroupIndicator(null);
        exList.setDividerHeight(1);
        exList.setAdapter(exAdapter);
    }

    @Override
    public void onViewListener(DispatchDetail mDispatchDetail) {
        ProductEntity productEntity = new ProductEntity();
        if (null != mDispatchDetail) {
            productEntity.setAmount(StringUtils.toDouble(mDispatchDetail.getProSumAmount(), 0));
            productEntity.setItemId(mDispatchDetail.getProCode());
            productEntity.setItemName(mDispatchDetail.getProName());
            productEntity.setPrice(StringUtils.toDouble(mDispatchDetail.getProPrice(), 0));
        }
        Intent intent = new Intent();
        intent.putExtra(activityResultData, (Serializable) productEntity);
        setResult(activityResultCode, intent);
        OrderGoodsChoseMonthActivity.this.finish();
    }
}