package com.urovo.activity.customercare.view

import android.os.Bundle
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.customercare.net.CustomerCareDetailRes
import com.urovo.activity.customercare.presenter.CustomerCareDetailCon
import com.urovo.activity.customercare.presenter.CustomerCareDetailPre
import com.urovo.activity.customercare.ui.CustomerCareDetailAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.view.LoadWaitDominator
import com.urovo.wugumofang.view.RecycleViewDivider
import kotlinx.android.synthetic.main.ll_ra90_detail_act.*

/**
 *  @author XP on 2019/8/23.
 *  客户关怀-回访名单
 */
class CustomerCareListDetailAct : BaseActivity(), CustomerCareDetailCon.view {
    companion object {
        const val INTENT_ID = "INTENT_ID"//类型ID

        const val NO_RETURN = 0//未回访
        const val RETURNED = 1//未回访
    }

    val taskId by lazy { intent.getStringExtra(INTENT_ID) }


    var loadWait: LoadWaitDominator? = null
    var mPre: CustomerCareDetailPre? = null
    var month: String? = null
    val dataList: MutableList<CustomerCareDetailRes.ResultInfo.DetailItem> = mutableListOf()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.ll_ra90_detail_act)
        initTitle()
        CustomerCareDetailPre(this).start()
    }

    /**
     * 根据intent传过来的type类型，来显示界面标题
     */
    fun setTitleText() {
        val title = "回访名单"
        titleTextView.text = title
        noReturnBtn.text = "未回访（0）"
        returnBtn.text = "已回访（0）"
    }

    override fun init() {
        setTitleText()
        radioGroup.setOnCheckedChangeListener { group, checkedId ->
            taskId?.let {
                mPre?.queryData(when (true) {
                    noReturnBtn.isChecked -> NO_RETURN
                    returnBtn.isChecked -> RETURNED
                    else -> -1
                }, it)
            }
        }
        loadWait = LoadWaitDominator(this, dataContainer)
        detailRv.apply {
            layoutManager = LinearLayoutManager(this@CustomerCareListDetailAct)
            adapter = taskId?.let { CustomerCareDetailAdapter(this@CustomerCareListDetailAct, dataList, it, returnBtn.isChecked) }
            addItemDecoration(RecycleViewDivider(this@CustomerCareListDetailAct, RecyclerView.HORIZONTAL, 1, resources.getColor(R.color.divider_line)))
        }
    }

    override fun onResume() {
        super.onResume()
        taskId?.let {
            mPre?.queryData(when (true) {
                noReturnBtn.isChecked -> NO_RETURN
                returnBtn.isChecked -> RETURNED
                else -> -1
            }, it)
        }
    }

    override fun showData(labelType: Int) {
        if (this.isFinishing || labelType == -1) {
            return
        }
        mPre?.resSave?.let {
            //返回的不为空
            noReturnBtn.text = "未回访（${it.resultInfo.pandingCareList?.size ?: 0}）"
            returnBtn.text = "已回访（${it.resultInfo.careList?.size ?: 0}）"
            dataList.apply {
                clear()//清除原来的
                when (labelType) {
                    NO_RETURN -> it.resultInfo.pandingCareList
                    RETURNED -> it.resultInfo.careList
                    else -> null
                }?.let {
                    addAll(it)
                }//添加新的
            }
            detailRv.adapter?.notifyDataSetChanged()//刷新
            (detailRv.adapter as CustomerCareDetailAdapter?)?.hasCared = returnBtn.isChecked
        }
    }

    override fun showLoading() {
        loadWait?.let { it.show(LoadWaitDominator.LOADING) }
    }

    override fun showNetError() {
        loadWait?.let { it.show(LoadWaitDominator.NET_ERROR) }
    }

    override fun showNoData() {
        loadWait?.let { it.show(LoadWaitDominator.NO_DATA) }
    }

    override fun dismissLoadWait() {
        loadWait?.let { it.dismissAll() }
    }

    override fun setPresenter(presenter: CustomerCareDetailCon.pre?) {
        this.mPre = presenter as CustomerCareDetailPre
    }
}