package com.urovo.activity.customerlabel

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.urovo.activity.customerlabel.model.CustomerLabelEntity
import com.urovo.activity.customerlabel.model.CustomerLabelSubmitEntity
import com.urovo.activity.customerlabel.presenter.CustomerLabelContract
import com.urovo.activity.customerlabel.presenter.CustomerLabelPresenter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.urovo.wugumofang.view.LoadWaitDominator
import com.urovo.wugumofang.view.flowlayout.FlowLayout
import com.urovo.wugumofang.view.flowlayout.TagAdapter
import com.urovo.wugumofang.view.flowlayout.TagFlowLayout
import com.urovo.wugumofang.view.flowlayout.TagView
import kotlinx.android.synthetic.main.act_customer_label.*

/**
 * 顾客标签页
 * @author LW
 */
class CustomerLabelActivity : BaseActivity(), CustomerLabelContract.IView {

    private var mCustomerLabelPresenter: CustomerLabelPresenter? = null

    override fun addRootLayoutId(): Int = R.layout.act_customer_label

    override fun rootViewGroup(): ViewGroup = root_container_fl

    private val mPageEditStatus: Boolean by lazy {
        intent.getBooleanExtra(AppConstants.ExtraKeys.PAGE_EDIT_STATUS, false)
    }

    private val mMobile: String by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.PHONE_NUM)
    }

    private var mCustomerLabelDetailLists: List<CustomerLabelEntity>? = listOf()

    override fun labelListData(data: List<CustomerLabelEntity>?) {
        dismissAllState()
        if (data.isNotNullOrSize()) {
            mCustomerLabelDetailLists = data
            if (mPageEditStatus) {
                labelListDataToView()
            } else {
                labelListDetailDataToView()
            }
        } else {
            showState(LoadWaitDominator.NO_DATA)
        }
    }


    override fun initTitle() {
        super.initTitle()
        if (mPageEditStatus) {
            titleTextView.text = getString(R.string.customer_lable_add_title)
            complete.apply {
                text = getString(R.string.save_btn_txt)
                setBackgroundResource(R.drawable.selector_add_pro_sell)
                visibility = View.VISIBLE
                setOnClickListener {
                    //提交标签数据
                    submitLabelData()
                }
            }
        } else {
            titleTextView.text = getString(R.string.customer_label_title)
        }
    }

    override fun initViewNew() {
        initTitle()

        val customerLabel = intent.getStringExtra(AppConstants.ExtraKeys.CUSTOMER_LABEL)
        if (customerLabel.notNullAndEmpty()) {
            mCustomerLabelDetailLists = Gson().fromJson(
                customerLabel,
                object : TypeToken<List<CustomerLabelEntity>?>() {}.getType()
            )
        } else {
            mCustomerLabelPresenter?.requestLabelList(mMobile)
        }
    }

    override fun initDataNew() {
        if (mCustomerLabelDetailLists.isNotNullOrSize()) {
            if (mPageEditStatus) {
                labelListDataToView()
            } else {
                labelListDetailDataToView()
            }
        } else {
            showState(LoadWaitDominator.NO_DATA)
        }
    }

    /**
     * 提交标签数据
     */
    private fun submitLabelData() {
        val customerLabelSubmitList = mutableListOf<CustomerLabelSubmitEntity>()
        mCustomerLabelDetailLists?.forEach {
            if (it.labels.isNotNullOrSize()) {
                for (bean in it.labels!!) {
                    if (bean.isSelected) {
                        customerLabelSubmitList.add(composeSubmitData(bean))
                    }
                }
            }

        }

        if (customerLabelSubmitList.isNotNullOrSize()) {
            //提交标签数据
            mCustomerLabelPresenter?.requestSaveLabelDetail(customerLabelSubmitList)
        } else {
            showToast(R.string.customer_label_entry)
        }
    }

    /**
     * 组合提交数据
     */
    private fun composeSubmitData(bean: CustomerLabelEntity.LabelBean): CustomerLabelSubmitEntity {
        return CustomerLabelSubmitEntity(
            bean.labelId,
            bean.labelName,
            mMobile
        )
    }

    /**
     * 标签信息详情
     */
    private fun labelListDetailDataToView() {
        val inflater = LayoutInflater.from(mContext)
        mCustomerLabelDetailLists?.forEach {
            val itemView =
                inflater.inflate(R.layout.item_customer_label, customer_label_container_ll, false)
            itemView.findViewById<TextView>(R.id.customer_label_item_title_tv).text =
                it.labelGroupName

            val labelBeanLists = mutableListOf<CustomerLabelEntity.LabelBean>()
            if (it.labels.isNotNullOrSize()) {
                it.labels?.forEach { label ->
                    if (label.isSelected) {
                        labelBeanLists.add(label)
                    }
                }
            }

            val customerLabelItemTagTf =
                itemView.findViewById<TagFlowLayout>(R.id.customer_label_item_tag_tf)
            customerLabelItemTagTf.adapter =
                object :
                    TagAdapter<CustomerLabelEntity.LabelBean>(labelBeanLists) {
                    override fun getView(
                        parent: FlowLayout,
                        position: Int,
                        bean: CustomerLabelEntity.LabelBean
                    ): View {
                        val tagView = inflater.inflate(
                            R.layout.item_customer_label_view,
                            customerLabelItemTagTf, false
                        ) as TextView
                        tagView.text = bean.labelName
                        tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                        tagView.setTextColor(
                            ContextCompat.getColor(
                                mContext,
                                R.color.common_three_txt_color
                            )
                        )
                        return tagView
                    }
                }
            customer_label_container_ll.addView(itemView)
        }
    }

    /**
     * 标签信息
     */
    private fun labelListDataToView() {
        val inflater = LayoutInflater.from(mContext)
        mCustomerLabelDetailLists?.forEachIndexed { index, entity ->
            val itemView =
                inflater.inflate(R.layout.item_customer_label, customer_label_container_ll, false)
            itemView.findViewById<TextView>(R.id.customer_label_item_title_tv).text =
                entity.labelGroupName

            if (entity.labels.isNotNullOrSize()) {
                val customerLabelItemTagTf =
                    itemView.findViewById<TagFlowLayout>(R.id.customer_label_item_tag_tf)
                customerLabelItemTagTf.adapter =
                    object :
                        TagAdapter<CustomerLabelEntity.LabelBean>(entity.labels) {
                        override fun getView(
                            parent: FlowLayout,
                            position: Int,
                            bean: CustomerLabelEntity.LabelBean
                        ): View {
                            val tagView = inflater.inflate(
                                R.layout.item_customer_label_view,
                                customerLabelItemTagTf, false
                            ) as TextView
                            tagView.text = bean.labelName
                            if (bean.isSelected) {
                                tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                                tagView.setTextColor(
                                    ContextCompat.getColor(
                                        mContext,
                                        R.color.common_three_txt_color
                                    )
                                )
                            } else {
                                tagView.setBackgroundResource(R.drawable.shape_white_btn_state)
                                tagView.setTextColor(
                                    ContextCompat.getColor(
                                        mContext,
                                        R.color.gray_txt_color
                                    )
                                )
                            }
                            return tagView
                        }
                    }
                customerLabelItemTagTf.setOnTagClickListener { view, position, _ ->
                    val customerLabelEntity = mCustomerLabelDetailLists?.get(index)
                    customerLabelEntity?.apply {
                        val label = labels?.get(position)
                        label?.let {
                            if (selectType == ConstantParams.CONSTANT_STARTS_2) {
                                labels?.forEachIndexed { index, labelBean ->
                                    if (index != position) {
                                        labelBean.isSelected = false
                                    }
                                }
                                it.isSelected = !it.isSelected
                                customerLabelItemTagTf.adapter.notifyDataChanged()
                            } else {
                                val tagView = (view as TagView).tagView as TextView
                                if (it.isSelected) {
                                    tagView.setBackgroundResource(R.drawable.shape_white_btn_state)
                                    tagView.setTextColor(
                                        ContextCompat.getColor(
                                            mContext,
                                            R.color.gray_txt_color
                                        )
                                    )
                                } else {
                                    tagView.setBackgroundResource(R.drawable.shape_theme_btn_state)
                                    tagView.setTextColor(
                                        ContextCompat.getColor(
                                            mContext,
                                            R.color.common_three_txt_color
                                        )
                                    )
                                }
                                it.isSelected = !it.isSelected
                            }
                        }
                    }
                    true
                }
            }
            customer_label_container_ll.addView(itemView)
        }
    }

    override fun saveLabelDetailStatus(status: Boolean) {
        super.saveLabelDetailStatus(status)
        if (status) {
            this.finish()
        } else {
            showToast(R.string.customer_label_submit_error)
        }
    }

    override fun addPresenter(): BasePresenter {
        mCustomerLabelPresenter = CustomerLabelPresenter(this)
        return mCustomerLabelPresenter!!
    }
}