package com.urovo.activity.customerprivilege.ui;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.google.gson.Gson;
import com.urovo.activity.customerprivilege.model.CustomerPrivilegeResult;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.constants.RouterParams;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.ui.ViewUtil;

import java.util.HashMap;

import okhttp3.Request;

/**
 * 特权查询
 */
@Route(path = RouterParams.Member.CUSTOMER_PRIVILEGE)
public class CustomerPrivilegeActivity extends BaseActivity {
    private Button btSearch;
    private EditText etSearchCustomer;
    private LinearLayout lvSearchCustomerPrivilegeHeader;
    private TextView listName;
    private TextView careStaffName;
    private TextView lastCareDate;
    private ListView lvSearchCustomerPrivilege;

    private CustomerPrivilegeAdapter customerPrivilegeAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.customer_privilege_activity);
        initTitle();
        initView();
    }

    @Override
    public void initTitle() {
        super.initTitle();
        titleTextView.setText("特权查询");
    }

    private void initView() {
        btSearch = (Button) findViewById(R.id.bt_search);
        etSearchCustomer = (EditText) findViewById(R.id.et_search_customer);
        lvSearchCustomerPrivilegeHeader = (LinearLayout) findViewById(R.id.lv_search_customer_privilege_header);
        listName = (TextView) findViewById(R.id.list_name);
        careStaffName = (TextView) findViewById(R.id.care_staff_name);
        lastCareDate = (TextView) findViewById(R.id.last_care_date);
        lvSearchCustomerPrivilege = (ListView) findViewById(R.id.lv_search_customer_privilege);
        btSearch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String customer = etSearchCustomer.getText().toString();
                if (TextUtils.isEmpty(customer)) {
                    showMsg("请输入会员手机号查询");
                    return;
                }
                getData(customer);
            }
        });
    }

    private void getData(String mobile) {
        HashMap<String, String> param = new HashMap<>();
        param.put("mobile", mobile);
        ViewUtil.showProgress(CustomerPrivilegeActivity.this, "", "请稍等.....");
        OkHttpDominator.getAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.SYGJ_EWX.CUSTOMER_PRIVILEGE, param,
                new OkHttpDominator.ResultCallback<String>() {
                    @Override
                    public void onError(Request request, Exception e) {
                        ViewUtil.dismissProgress();
                        showMsg("获取特权信息" + getString(R.string.request_data_fail));
                    }

                    @Override
                    public void onResponse(String json) {
                        ViewUtil.dismissProgress();
                        if (json == null || TextUtils.isEmpty(json)) {
                            showMsg("没有返回数据");
                            return;
                        }
                        CustomerPrivilegeResult mCustomerPrivilegeResult = new Gson().fromJson(json, CustomerPrivilegeResult.class);
                        if (null != mCustomerPrivilegeResult && mCustomerPrivilegeResult.msgCode == 0) {
                            if (null != mCustomerPrivilegeResult.getResultInfo()) {
                                setViewData(mCustomerPrivilegeResult);
                            } else {
                                lvSearchCustomerPrivilegeHeader.setVisibility(View.GONE);
                                lvSearchCustomerPrivilege.setVisibility(View.GONE);
                                showMsg("没有查询到相关数据");
                            }
                        } else {
                            showMsg("没有返回数据");
                        }
                    }
                });
    }

    private void setViewData(CustomerPrivilegeResult mCustomerPrivilegeResult) {
        lvSearchCustomerPrivilegeHeader.setVisibility(View.VISIBLE);
        lvSearchCustomerPrivilege.setVisibility(View.VISIBLE);
        listName.setText(mCustomerPrivilegeResult.getResultInfo().getListName());
        careStaffName.setText(mCustomerPrivilegeResult.getResultInfo().getCareStaffName() == null ? "--" : mCustomerPrivilegeResult.getResultInfo().getCareStaffName());
        lastCareDate.setText(mCustomerPrivilegeResult.getResultInfo().getLastCareDate());
        customerPrivilegeAdapter = new CustomerPrivilegeAdapter(mCustomerPrivilegeResult.getResultInfo().getActs(), CustomerPrivilegeActivity.this);
        lvSearchCustomerPrivilege.setAdapter(customerPrivilegeAdapter);
    }
}