package com.urovo.activity.display

import android.annotation.SuppressLint
import android.annotation.TargetApi
import android.content.Context
import android.content.Intent
import android.os.*
import android.text.TextUtils
import android.util.Log
import android.view.*
import android.widget.Toast
import androidx.core.content.ContextCompat
import com.qiniu.android.common.FixedZone
import com.qiniu.android.http.ResponseInfo
import com.qiniu.android.storage.*
import com.qiniu.android.storage.persistent.FileRecorder
import com.qiniu.android.utils.UrlSafeBase64
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseResponse
import com.urovo.wugumofang.base.PermissionsActivity
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.base.mvp.AbsView
import com.urovo.wugumofang.constants.Files
import com.urovo.wugumofang.constants.PhotoUtils
import com.urovo.wugumofang.data.PrefTool
import com.urovo.wugumofang.system.DeviceHelper
import com.urovo.wugumofang.system.PdaMsgConfig
import com.urovo.wugumofang.system.Prefs
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.DialogHelper
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.*
import com.urovo.wugumofang.view.LoadWaitDominator
import com.xp.pro.imagepickerlib.ImagePickerBaseActivity
import com.xp.pro.imagepickerlib.bean.ImageItem
import com.xp.pro.imagepickerlib.utils.PathConfig
import com.xp.pro.imagepickerlib.widgets.TipDialog
import com.zsy.libcore.ui.act.SYActManager
import com.zsy.libcore.widgets.bar.SystemBarTintManager
import kotlinx.android.synthetic.main.public_container_view.*
import kotlinx.android.synthetic.main.public_user_title.*
import org.json.JSONObject
import java.io.*
import java.util.*
import kotlin.collections.isNullOrEmpty

/**
 * @author LW
 * 封装图片上传逻辑 ReportWithPhotoBaseActivity
 */
abstract class ReportWithPhotoBaseActivity : ImagePickerBaseActivity(), AbsView,
    DialogHelper.DialogResult {

    /**
     * 请求码
     */
    private val REQUEST_CODE = 10000

    /**
     * 上报类型
     */
    protected var mReportType = "UNKOWN"

    /**
     * 上报图片状态
     */
    private var mImgStatus = ""

    /**
     * 上报图片方式
     */
    private var mImgType = true

    /**
     * 上传个数
     */
    private var mUploadCount = 0

    /**
     * 上下文
     */
    protected lateinit var mContext: Context

    /**
     * 图片URL集合(已上传)
     */
    protected var mUrlLists: MutableList<String> = mutableListOf()

    /**
     * 图片七牛名称集合（未上传）
     */
    protected var mQiniuUrlLists: MutableList<String> = mutableListOf()

    /**
     * 七牛云工具类
     */
    private var mUploadManager: UploadManager? = null

    /**
     * 加载状态类
     */
    private var mLoadWaitDominator: LoadWaitDominator? = null

    /**
     * 基类Presenter
     */
    protected var mPresenter: AbsPresenter? = null

    /**
     * 七牛上传凭证uploadToken
     */
    private var mToken: String? = ""

    /**
     * 权限控制
     */
    private val mPermissionsChecker: PermissionsCheckerUtil by lazy {
        PermissionsCheckerUtil(this)
    }

    /**
     * 提示信息
     */
    private var mTip: String = ""

    /**
     * 用户 Id
     */
    private var mUserId: String = ""

    /**
     * 门店 Id
     */
    private var mShopId: String? = ""

    init {
        //断点上传
        var dirPath = Files.getDownfile()
        var recorder: Recorder? = null
        try {
            val file = File.createTempFile("qiniu_xxxx", ".tmp")
            dirPath = file.parent
            recorder = FileRecorder(dirPath)
        } catch (e: Exception) {
            e.printStackTrace()
        }

        val dirPath1 = dirPath
        //默认使用 key 的url_safe_base64编码字符串作为断点记录文件的文件名。
        //避免记录文件冲突（特别是key指定为null时），也可自定义文件名(下方为默认实现)：
        val keyGen =
            KeyGenerator { key, file -> // 不必使用url_safe_base64转换，uploadManager内部会处理
                // 该返回值可替换为基于key、文件内容、上下文的其它信息生成的文件名
                val path = key + "_._" + StringBuffer(file.absolutePath).reverse()
                val f = File(dirPath1, UrlSafeBase64.encodeToString(path))
                val reader: BufferedReader?
                try {
                    reader = BufferedReader(FileReader(f))
                    var tempString: String?
                    var line = 1
                    try {
                        while (reader.readLine().also { tempString = it } != null) {
                            Log.d("qiniu", "line $line: $tempString")
                            line++
                        }
                    } catch (e: IOException) {
                        e.printStackTrace()
                    } finally {
                        try {
                            reader.close()
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                } catch (e: FileNotFoundException) {
                    e.printStackTrace()
                }
                path
            }

        val config = Configuration.Builder()
            .chunkSize(512 * 1024) // 分片上传时，每片的大小。 默认256K
            .putThreshhold(1024 * 1024) // 启用分片上传阀值。默认512K
            .connectTimeout(10) // 链接超时。默认10秒
            .useHttps(true) // 是否使用https上传域名
            .responseTimeout(120) // 服务器响应超时。默认60秒
            .recorder(recorder) // recorder分片上传时，已上传片记录器。默认null
            .recorder(recorder, keyGen) // keyGen 分片上传时，生成标识符，用于片记录器区分是那个文件的上传记录
            .zone(FixedZone.zone0) // 设置区域，指定不同区域的上传域名、备用域名、备用IP。
            .build()
        // 重用uploadManager。一般地，只需要创建一个uploadManager对象
        mUploadManager = UploadManager(config)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        initWindowView()

        super.onCreate(savedInstanceState)
        setContentView(R.layout.public_container_view)

        SYActManager.getInstance().addActivity(this)

        val childLayoutId = addChildLayoutId()
        if (childLayoutId > 0) {
            val childView = View.inflate(this, childLayoutId, null)
            mPublicContainer.addView(
                childView,
                ViewGroup.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.MATCH_PARENT
                )
            )
        }
        mContext = this
        mLoadWaitDominator = LoadWaitDominator(this, mPublicContainer)

        initPdaConfig()

        mTip =
            "\n版本：${DeviceHelper.getVersion(this)} \n设备：Android" +
                    " ${Build.VERSION.RELEASE} / ${Build.MODEL}"

        initTitle()
        initView()
        initData()
    }


    /**
     * 通知栏一体化
     */
    protected fun initWindowView() {
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            setTranslucentStatus()
        }
        val tintManager = SystemBarTintManager(this)
        tintManager.isStatusBarTintEnabled = true
        // 通知栏所需颜色
        tintManager.setStatusBarTintResource(R.color.normal_color)
    }

    /**
     * 设置状态栏透明状态
     */
    @TargetApi(19)
    protected fun setTranslucentStatus() {
        val win = window
        val winParams = win.attributes
        val bits = WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS
        winParams.flags = winParams.flags or bits
        win.attributes = winParams
    }

    /**
     * 初始化 initTitle
     */
    protected fun initTitle() {
        mPublicTitleTextTv.apply {
            val tempEnv = PrefTool.getIntPreferences(
                this@ReportWithPhotoBaseActivity,
                Prefs.PRE_ENVIRONMENT_STATUS,
                0
            )
            if (tempEnv != 0) {
                setTextColor(
                    ContextCompat.getColor(
                        this@ReportWithPhotoBaseActivity,
                        R.color.important_txt_color
                    )
                )
                setBackgroundResource(R.drawable.test_env_icon)
            } else {
                setTextColor(
                    ContextCompat.getColor(
                        this@ReportWithPhotoBaseActivity,
                        R.color.white
                    )
                )
                setBackgroundResource(R.color.normal_color)
            }

            val titleResId = fetchToolBarTitle()
            if (titleResId > 0) {
                setText(titleResId)
            }
        }

        mPublicTitleBackLl.visibility = View.VISIBLE
        mPublicTitleMorell.visibility = View.INVISIBLE
        mPublicTitleMorell.setOnClickListener { more() }
        mPublicTitleBackLl.setOnClickListener { back() }
    }

    /**
     * 标题
     */
    protected open fun fetchToolBarTitle(): Int = -1

    /**
     * 标题栏左边事件
     */
    protected fun back() {
        finish()
    }

    /**
     * 标题栏右边事件
     */
    protected fun more() {}

    /**
     * 孩子布局Id
     */
    protected abstract fun addChildLayoutId(): Int

    /**
     * 初始化 initData
     */
    protected abstract fun initData()

    /**
     * 初始化 initView
     */
    protected abstract fun initView()

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        // 拒绝时, 关闭页面, 缺少主要权限, 无法运行
        if (requestCode == REQUEST_CODE && resultCode == PermissionsActivity.PERMISSIONS_DENIED) {
            finish()
        }
    }

    override fun onResume() {
        // 缺少权限时, 进入权限配置页面
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (Build.VERSION.SDK_INT > 32) {
                if (mPermissionsChecker.lacksPermissions(*PermissionsCheckerUtil.NEWPERMISSIONS)) {
                    PermissionsActivity.startActivityForResult(
                        this,
                        REQUEST_CODE,
                        *PermissionsCheckerUtil.NEWPERMISSIONS
                    )
                }
            } else {
                if (mPermissionsChecker.lacksPermissions(*PermissionsCheckerUtil.PERMISSIONS)) {
                    PermissionsActivity.startActivityForResult(
                        this,
                        REQUEST_CODE,
                        *PermissionsCheckerUtil.PERMISSIONS
                    )
                }
            }
        }
        super.onResume()
        mPresenter?.onResume()
    }

    override fun onPause() {
        super.onPause()
        mPresenter?.onPause()
    }

    private fun initPdaConfig() {
        SystemHelper.LoginOrganId = PdaMsgConfig.getLoginOrgID(mContext)
        SystemHelper.IsAdmin = PdaMsgConfig.getIsAdmin(mContext)
        SystemHelper.LoginOrganName = PdaMsgConfig.getLoginOrganName(mContext)
        SystemHelper.LoginUserId = PdaMsgConfig.getLogUserId(mContext)
        SystemHelper.LoginUserName = PdaMsgConfig.getLoginUserName(mContext)
    }

    override fun onDialogResult(tag: Int) {}

    protected var mCount = 0
    private var mHandler: Handler? = Handler()
    private var mRunnable: Runnable? = object : Runnable {
        override fun run() {
            update()
            // 间隔100ms
            if (mCount == 0) {
                mHandler?.postDelayed(this, 100)
            }
        }

        fun update() {
            if (mUrlLists.isNotEmpty() && mUrlLists.size == mQiniuUrlLists.size) {
                if (mReportType == "PTPD" || mReportType == "PTJC") {
                    if (mUrlLists.size == mImageselectList.size) {
                        //如果所有图片上上传完成，则开始上传数据
                        mImgStatus = "成功。$mTip"
                        uploadData()
                        mCount = 1
                        mHandler?.removeCallbacks(this)
                    }
                } else {
                    //如果所有图片上上传完成，则开始上传数据
                    mImgStatus = "成功。$mTip"
                    uploadData()
                    mCount = 1
                    mHandler?.removeCallbacks(this)
                }
            }
        }
    }

    /**
     * 1 点击底部确定按钮执行第一步：设置上传数据
     */
    protected open fun setUploadData() {
        mCount = 0

        setUploadImage()
    }

    /**
     * 上传图片数据
     */
    private fun setUploadImage() {
        showTipDialog("上传图片中~")

        //存放图片路径的ArrayList
        mImageselectList = idImagePickerContainer.getmImageselectList()
        if (mImageselectList.isNullOrEmpty()) {
            //没有图片，直接提交数据
            mImgStatus = "没有资源。$mTip"
            uploadData()
        } else if (checkAreadyUplaod()) {
            //没有图片，直接提交数据
            uploadData()
        } else {
            //清空数据
            mQiniuUrlLists.clear()
            mUrlLists.clear()

            if (mImgType) {
                uploadByFilePathBefore()
            } else {
                for (i in mImageselectList.indices) {
                    val imageItem = mImageselectList[i]
                    if (1 != imageItem.getType()) {
                        mUploadCount = 1
                        uploadByFilePath(imageItem.getImagePath())
                    }
                }
            }
            if (null != mHandler && null != mRunnable) {
                mHandler?.postDelayed(mRunnable, 100)
            }
        }
    }

    /**
     * 上传图片数据
     */
    protected fun setUploadImage(list: ArrayList<ImageItem>) {
        showTipDialog("上传图片中~")

        mImageselectList = list
        if (mImageselectList.isNullOrEmpty()) {
            //没有图片，直接提交数据
            mImgStatus = "没有资源。$mTip"
            uploadData()
        } else if (checkAreadyUplaod()) {
            //没有图片，直接提交数据
            uploadData()
        } else {
            //清空数据
            mQiniuUrlLists.clear()
            mUrlLists.clear()

            if (mImgType) {
                uploadByFilePathBefore()
            } else {
                for (i in mImageselectList.indices) {
                    val imageItem = mImageselectList[i]
                    if (1 != imageItem.getType()) {
                        mUploadCount = 1
                        uploadByFilePath(imageItem.getImagePath())
                    }
                }
            }
            if (null != mHandler && null != mRunnable) {
                mHandler?.postDelayed(mRunnable, 100)
            }
        }
    }

    /**
     * 判断图片是否上传完成
     * @return true 是 false 否
     */
    private fun checkAreadyUplaod(): Boolean {
        if (mImageselectList.isNotNullOrSize()) {
            val filterList = mImageselectList.filter { item ->
                item.getType() != 1
            }
            return filterList.isEmpty()
        }
        return true
    }

    /**
     * 点击底部确定按钮执行第二步：提交数据
     */
    protected open fun uploadData() {
        dismissDialog()
    }

    /**
     * 上传文件到七牛
     *
     * @param filePath 文件路径
     */
    protected fun uploadByFilePathBefore() {
        Thread {
            for (i in mImageselectList.indices) {
                val imageItem = mImageselectList[i]
                if (1 != imageItem.getType()) {
                    mUploadCount = 1

                    val key = getQiNiuImageKey()
                    //将图片地址加入集合，图片上传成功则能找到该图片，图片上传失败则找不到图片，服务端使用默认图片显示
                    mQiniuUrlLists.add(PhotoUtils.getRealUrl(key))

                    //图片旋转，压缩处理
                    val imageBytes = PhotoUtils.getImageBytes(imageItem.getImagePath(), 200)

                    // 如果是第一次获取，获得七牛上传凭证uploadToken
                    if (mToken == null || "" == mToken) {
                        mToken = PhotoUtils.getUploadToken()
                    }
                    if (mToken != null) {
                        val uploadManager = UploadManager()
                        uploadManager.put(
                            imageBytes, key, mToken,
                            { arg0, info, response ->
                                //首先检测上传是否成功
                                if (!info.isOK) {
                                    uploadFail("\n图片上传请求超时，请稍后再试。\n")
                                }
                                if (null != response) {
                                    val json = response.toString()
                                    val mTokenBean: BaseResponse? = PhotoUtils.parseTokenData(json)

                                    // 解析出返回的token
                                    if (null != mTokenBean && "0" == mTokenBean.retCode) {
                                        val url = PhotoUtils.getRealUrl(mTokenBean.retInfo)
                                        mUrlLists.add(url)
                                        //TODO 原始设置无法定位
                                        imageItem.setImgneturl(url)
                                        imageItem.setType(1)
                                    } else {
                                        uploadFail("\n七牛返回错误信息:$info。\n")
                                    }
                                } else {
                                    uploadFail("\n未获取到七牛返回信息。\n")
                                }
                            }, null
                        )
                    } else {
                        uploadFail("\n当前网络访问超时。\n")
                    }
                }
            }

        }.start()
    }

    protected fun getQiNiuImageKey(): String {
        if (SystemHelper.LoginUserId != null) {
            mUserId = SystemHelper.LoginUserId
        }

        if (SystemHelper.LoginOrganId != null) {
            mShopId = SystemHelper.LoginOrganId
        }
        return PhotoUtils.getFileUrlUUID(mShopId, mUserId)
    }

    /**
     * 上传文件到七牛
     *
     * @param filePath 文件路径
     */
    protected fun uploadByFilePath(filePath: String?) {
        //异步上传图片
        UploadImageAsyncTask().execute(filePath)
    }

    @SuppressLint("StaticFieldLeak")
    private inner class UploadImageAsyncTask : AsyncTask<String?, Void?, ByteArray?>() {

        override fun onPreExecute() {
            super.onPreExecute()
            showTipDialog("上传图片中~")
        }

        override fun doInBackground(vararg params: String?): ByteArray? {
            val filePath = params[0]
            // 如果是第一次获取，获得七牛上传凭证uploadToken
            if (TextUtils.isEmpty(mToken)) {
                mToken = PhotoUtils.getUploadToken()
            }
            return PhotoUtils.getImageBytes(filePath, 200)
        }

        //onPostExecute用于UI的更新.此方法的参数为doInBackground方法返回的值.
        override fun onPostExecute(imageBytes: ByteArray?) {
            super.onPostExecute(imageBytes)
            if (imageBytes == null) {
                uploadFail("\n图片有问题，请重新选择！\n")
                return
            }
            val key: String = getQiNiuImageKey()
            //将图片地址加入集合，图片上传成功则能找到该图片，图片上传失败则找不到图片，服务端使用默认图片显示
            mQiniuUrlLists.add(PhotoUtils.getRealUrl(key))
            if (mToken != null) {
                val map = HashMap<String, String>()
                map["x:phone"] = "12345678"
                mIsCancelled = false
                mUploadManager?.put(imageBytes, key, mToken,
                    { arg0, info, response ->
                        getQiNiuResponse(
                            info,
                            response
                        )
                    }, UploadOptions(map, null, false,
                        { _, percent ->
                            //progressbar.setVisibility(View.VISIBLE);
                            val progress = (percent * 1000).toInt()
                            //progressbar.setProgress(progress);
                            //if (progress == 1000) {
                            //progressbar.setVisibility(View.GONE);
                            //}
                        }, { mIsCancelled })
                )
            } else {
                uploadFail("\nToken获取失败,网络或服务器异常！\n")
            }
        }
    }

    /**
     * 七牛图片上传完成后回调
     *
     * @param info     七牛返回信息
     * @param response 返回结果
     */
    private fun getQiNiuResponse(info: ResponseInfo, response: JSONObject?) {
        //首先检测上传是否成功
        if (!info.isOK) {
            uploadFail("\n图片上传请求超时，请稍后再试。\n")
        }
        if (null != response) {
            val json: String = response.toString()
            val mTokenBean: BaseResponse? = PhotoUtils.parseTokenData(json)

            // 解析出返回的token
            if (null != mTokenBean && "0" == mTokenBean.retCode) {
                val url = PhotoUtils.getRealUrl(mTokenBean.retInfo)
                mUrlLists.add(url)
            } else {
                uploadFail("\n七牛返回错误信息:$info。\n")
            }
        } else {
            uploadFail("\n图片上传请求超时，请稍后再试。\n")
        }
    }

    /**
     * 图片上传失败处理
     */
    private fun uploadFail(msg: String): String {
        cancell()
        mUrlLists.clear()
        mQiniuUrlLists.clear()
        runOnUiThread {
            if (mUploadCount == 1) {
                if (mReportType === "PTPD" || mReportType === "PTJC") {
                    //如果是兼职督察功能，图片上传失败需直接上传数据
                    mImgStatus = "失败，$msg$mTip"
                    uploadData()
                } else {
                    dismissDialog()
                    showTipDialog("提示", "图片上传失败，请重新上传!\n$msg$mTip")
                }
                mUploadCount = 0
            }
        }
        return msg
    }


    /**
     * 初始化、执行上传
     */
    @Volatile
    private var mIsCancelled = false

    /**
     * 点击取消按钮，让UpCancellationSignal##isCancelled()方法返回true，以停止上传
     */
    private fun cancell() {
        mIsCancelled = true
    }

    override fun showToast(msg: String?) {
        if (!isFinishing) {
            runOnUiThread {
                Toast.makeText(
                    this@ReportWithPhotoBaseActivity, msg, Toast.LENGTH_SHORT
                ).show()
            }
        }
    }

    override fun showToast(resId: Int) {
        showToast(getString(resId))
    }

    override fun showTipDialog(msg: String?) {
        if (msg.notNullAndEmpty()) {
            ViewUtil.showProgress(this, "请稍等", msg)
        }
    }

    override fun showTipDialog(resId: Int) {
        if (resId > 0) {
            ViewUtil.showProgress(this, "请稍等", getString(resId))
        }
    }

    override fun dismissDialog() {
        ViewUtil.dismissProgress()
    }

    override fun showIsLoading() {
        mLoadWaitDominator?.show(LoadWaitDominator.LOADING)
    }

    override fun showError(msg: String?) {
        mLoadWaitDominator?.show(msg ?: "netError")
    }

    override fun showError(resId: Int) {
        mLoadWaitDominator?.show(getString(resId) ?: "netError")
    }

    override fun showState(state: Int) {
        mLoadWaitDominator?.show(state)
    }

    override fun dismissLoading() {
        mLoadWaitDominator?.dismissLoading()
    }

    override fun dismissAllState() {
        mLoadWaitDominator?.dismissAll()
    }

    private var mTipDialog: TipDialog? = null

    /**
     * 显示提示框
     */
    override fun showTipDialog(title: String?, content: String?) {
        if (mTipDialog == null) {
            mTipDialog = TipDialog.createDialog(mContext, title)
        }
        mTipDialog?.let {
            it.setTitle(title)
            it.setContent(content)
            it.setOnDismissListener { mTipDialog = null }
            if (!it.isShowing) {
                it.show()
            }
        }
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        if (event.action == 0) {
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                finish()
                return true
            }
        }
        return super.onKeyDown(keyCode, event)
    }

    override fun onDestroy() {
        SYActManager.getInstance().finishActivity(this)
        ViewUtil.dismissProgress()
        super.onDestroy()
        //清除自定义文件夹下的文件
        DataCleanManager.cleanCustomCache(Files.getPhotoPath())
        DataCleanManager.cleanCustomCache(Files.getLogfile())
        DataCleanManager.cleanCustomCache(PathConfig.getImagePath())

        mPresenter?.onDestroy()
    }

}