package com.urovo.activity.display.presenter

import com.urovo.activity.display.model.DisplayReportDetailEntity
import com.urovo.activity.display.model.DisplayReportEntity
import com.urovo.activity.display.model.DisplayReportSubmitEntity
import com.urovo.activity.display.service.DisplayReportDepository
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isEmptyParameter
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.utils.isNullOrEmpty
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Response

/**
 * 陈列上报 Presenter
 * @author LW
 */
class DisplayReportPresenter(
    private var view: DisplayReportContract.IView?,
    private var depository: DisplayReportDepository = DisplayReportDepository()
) : AbsPresenter(), DisplayReportContract.IPresenter {


    override fun requestMissionByStaffId() {
        val staffId = SystemHelper.LoginUserId
        if (staffId.isNullOrEmpty()) {
            view?.showError(R.string.common_load_error)
            return
        }
        view?.showIsLoading()
        depository.requestMissionByStaffId(staffId).enqueue(object :
            RetrofitCallback<RequestResult<List<DisplayReportEntity>>>() {
            override fun onResponse(
                call: Call<RequestResult<List<DisplayReportEntity>>>,
                res: Response<RequestResult<List<DisplayReportEntity>>>
            ) {
                val body = res.body()
                body.takeIf { it != null }?.run {
                    if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                        if (resultInfo.isNotNullOrSize()) {
                            view?.missionByStaffIdData(resultInfo)
                        } else {
                            view?.showState(LoadWaitDominator.NO_DATA)
                        }
                    } else {
                        view?.showError(msgInfo)
                    }
                }?: view?.showState(LoadWaitDominator.NET_ERROR)

                view?.dismissLoading()
            }

            override fun onFailure(exception: ApiException) {
                view?.dismissLoading()
                view?.showError(exception.message)
            }
        })
    }

    override fun requestMissionDetail(taskId: String?, shopId: String?) {
        if (isEmptyParameter(taskId, shopId)) {
            view?.showError(R.string.common_load_error)
            return
        }

        view?.showTipDialog(R.string.inspector_display_add_data_tip)
        depository.requestMissionDetail(taskId, shopId)
            .enqueue(object : RetrofitCallback<RequestResult<DisplayReportDetailEntity>>() {
                override fun onResponse(
                    call: Call<RequestResult<DisplayReportDetailEntity>>,
                    res: Response<RequestResult<DisplayReportDetailEntity>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.missionDetailData(resultInfo)
                        } else {
                            view?.showError(msgInfo)
                        }
                    } ?: view?.showState(LoadWaitDominator.NET_ERROR)

                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.showError(exception.message)
                }
            })
    }

    override fun requestSubmitMissionDetail(
        reportId: String?,
        auditId: String?,
        taskId: String?,
        shopId: String?,
        stockOut: String?,
        stockOutInfo: String?,
        excepSpec: String?,
        exportImages: List<String>?,
        wholeImages: List<String>?,
        answers: List<DisplayReportSubmitEntity.AnswersBean?>
    ) {
        val userNo = SystemHelper.LoginUserId
        val userName = SystemHelper.LoginUserName

        val valueStatus: Boolean
        if (stockOut == ConstantParams.CONSTANT_PARAMS_1) {
            valueStatus = isEmptyParameter(
                taskId,
                shopId,
                userNo,
                stockOutInfo,
                stockOut
            )
        } else {
            valueStatus = isEmptyParameter(taskId, shopId, userNo, stockOut)
        }

        if (valueStatus || exportImages.isNullOrEmpty() || wholeImages.isNullOrEmpty()) {
            view?.showToast(R.string.common_load_error)
            return
        }

        view?.showTipDialog(R.string.inspector_display_add_data_tip)

        val displayReportSubmitEntity = DisplayReportSubmitEntity(
            reportId,
            auditId,
            userName,
            userNo,
            taskId,
            shopId,
            stockOut,
            stockOutInfo,
            excepSpec,
            exportImages,
            wholeImages,
            answers
        )

        depository.requestSubmitMissionDetail(displayReportSubmitEntity)
            .enqueue(object : RetrofitCallback<RequestResult<Any>>() {
                override fun onResponse(
                    call: Call<RequestResult<Any>>,
                    res: Response<RequestResult<Any>>
                ) {
                    val body = res.body()
                    body?.let {
                        if (it.msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.submitionStatus(true)
                        } else {
                            view?.submitionStatus(false)
                            view?.showToast(it.msgInfo)
                        }
                    }
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.submitionStatus(false)
                    view?.showToast(exception.message)
                }
            })
    }
}