package com.urovo.activity.display.service

import com.urovo.activity.display.model.DisplayReportDetailEntity
import com.urovo.activity.display.model.DisplayReportEntity
import com.urovo.activity.display.model.DisplayReportSubmitEntity
import com.urovo.wugumofang.net.RetrofitHelper
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call

/**
 * 陈列上报仓库
 * @author LW
 */
class DisplayReportDepository {

    private val mApiService by lazy {
        RetrofitHelper.createApi(ApiService::class.java)
    }

    /**
     * 查询货架陈列上报任务
     * @param staffId String
     */
    fun requestMissionByStaffId(staffId: String?): Call<RequestResult<List<DisplayReportEntity>>> {
        return mApiService.queryMissionByStaffId(staffId)
    }

    /**
     * 查询货架陈列上报任务明细
     * @param taskId taskId
     * @param shopId 门店号
     */
    fun requestMissionDetail(
        taskId: String?,
        shopId: String?
    ): Call<RequestResult<DisplayReportDetailEntity>> {
        return mApiService.queryMissionDetail(taskId, shopId)
    }

    /**
     * 提交货架陈列上报任务数据
     * @param reportId 提交编号(有就填)
     * @param auditId 审核编号(有就填)
     * @param taskId  taskId
     * @param shopId shopId
     * @param userNo  提交人工号
     * @param userName  提交人名字
     * @param stockOut 缺货（0：不缺货，1：缺货）
     * @param stockOutInfo 缺货信息
     * @param excepSpec  异常说明
     * @param submitUrl 图片url,多张图片用
     */
    fun requestSubmitMissionDetail(
       entity: DisplayReportSubmitEntity?
    ): Call<RequestResult<Any>> {
        return mApiService.submitMissionDetail(entity)
    }
}