package com.urovo.activity.display.view

import android.content.Intent
import android.net.Uri
import android.view.LayoutInflater
import android.view.View
import android.widget.*
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.display.ReportWithPhotoBaseActivity
import com.urovo.activity.display.model.DisplayReportDetailEntity
import com.urovo.activity.display.model.DisplayReportSubmitEntity
import com.urovo.activity.display.model.TasksChild
import com.urovo.activity.display.presenter.DisplayReportContract
import com.urovo.activity.display.presenter.DisplayReportPresenter
import com.urovo.activity.display.view.adapter.DisplayReportEditAdapter
import com.urovo.activity.display.view.widget.MultipleChooseWidget
import com.urovo.activity.display.view.widget.SingleChooseWidget
import com.urovo.activity.display.view.widget.SingleLineInputWidget
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.ui.InputMethodManagerTool
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.wgmf.trade.utils.ImageLoader
import com.xp.pro.imagepickerlib.TakePictureActivity
import com.xp.pro.imagepickerlib.bean.ImageItem
import com.xp.pro.imagepickerlib.global.Params
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout.ImagePicker
import com.zsy.libcore.ui.ada.helper.DividerItemDecoration
import kotlinx.android.synthetic.main.ll_qback_report_error_show.*
import kotlinx.android.synthetic.main.public_confirm_btn_view.*
import kotlinx.android.synthetic.main.public_list_content_view.*
import org.jetbrains.anko.childrenSequence
import org.jetbrains.anko.find
import java.io.File
import java.util.*
import kotlin.collections.ArrayList

/**
 * 编辑陈列上报
 * @author LW
 */
class DisplayReportEditActivity : ReportWithPhotoBaseActivity(), DisplayReportContract.IView,
    ImagePicker, DisplayReportEditAdapter.OnPictureClickListener,
    SingleChooseWidget.OnSingleChooseClickListener ,
    MultipleChooseWidget.OnMultipleChooseClickListener {

    private var mDisplayReportPresenter: DisplayReportPresenter? = null

    override fun fetchToolBarTitle(): Int = R.string.display_report_title

    override fun addChildLayoutId(): Int = R.layout.public_list_content_view

    private var mDisplayReportEditAdapter: DisplayReportEditAdapter? = null

    private var mDisplayReportDetailEntity: DisplayReportDetailEntity? = null

    private var mRecyclerFootView: View? = null

    private var mRecyclerOneHeaderView: View? = null
    private var mRecyclerTwoHeaderView: View? = null
    private var mItemDisplayReportCaseView: View? = null

    /**
     * 整体图片
     */
    private var mWholeImageList: ArrayList<ImageItem> = arrayListOf()

    /**
     * 导出图片
     */
    private var mExportImageList: ArrayList<ImageItem> = arrayListOf()

    private val mTasksChild: TasksChild? by lazy {
        intent.getSerializableExtra("tasksChild") as TasksChild
    }

    /**
     *缺货状态
     */
    private var mStockout: String = ""

    override fun initView() {
        mDisplayReportPresenter = DisplayReportPresenter(this).apply {
            mPresenter = this
        }

        mRecyclerFootView = LayoutInflater.from(mContext).inflate(
            R.layout.display_report_add_footer_view,
            null, false
        );
        mDisplayReportEditAdapter = DisplayReportEditAdapter(listOf(), this)

        mPublicListViewLv.apply {
            layoutManager = LinearLayoutManager(mContext, RecyclerView.VERTICAL, false)
            addItemDecoration(
                DividerItemDecoration(
                    mContext,
                    DividerItemDecoration.VERTICAL_LIST,
                    1,
                    ContextCompat.getColor(mContext, R.color.pop_query_devider)
                )
            )
            adapter = mDisplayReportEditAdapter
        }

        mRecyclerOneHeaderView = View.inflate(mContext, R.layout.public_title_item, null)

        mRecyclerTwoHeaderView = View.inflate(mContext, R.layout.public_title_item, null)

        mPublicContentConfirmLl.visibility = View.VISIBLE
        if (mTasksChild?.audit.toString() == ConstantParams.CONSTANT_PARAMS_0) {
            mPublicContentConfirmBtn.text = getString(R.string.submit_btn_txt)
        } else {
            mPublicContentConfirmBtn.text = getString(R.string.modify_btn_txt)
        }
        mPublicContentConfirmBtn.setOnClickListener {
            setUploadData()
        }
        initFootView()
        initHeaderView()
    }

    private fun initHeaderView() {
        mRecyclerOneHeaderView?.let {
            it.find<TextView>(R.id.public_list_item_title).text =
                getString(R.string.display_report_last_time_submit_details)
        }
        mRecyclerTwoHeaderView?.let {
            it.find<TextView>(R.id.public_list_item_title).text =
                getString(R.string.display_report_current_submit)
        }
    }

    private var mDisplayReportCaseContainerLl: LinearLayout? = null
    private var mDisplayReportChooseContainerLl: LinearLayout? = null

    private var mDisplayWholeImagePickerContainer: ImagePickerLayout? = null
    private var mDisplayExportImagePickerContainer: ImagePickerLayout? = null

    private var mReportOutOfStockNoRb: RadioButton? = null
    private var mReportOutOfStockYesRb: RadioButton? = null

    private var mReportStoreInfoTv: TextView? = null
    private var mReportOutOfStockInfoEt: EditText? = null
    private var mReportReasonInputEt: EditText? = null


    private fun initFootView() {

        mRecyclerFootView?.let {
            mDisplayReportCaseContainerLl = it.find(R.id.display_report_case_container_ll)

            it.findViewById<RadioGroup>(R.id.display_report_out_of_stock_rg)
                .setOnCheckedChangeListener { group, checkedId ->

                    when (checkedId) {
                        R.id.display_report_out_of_stock_no_rb -> {
                            mStockout = ConstantParams.CONSTANT_PARAMS_0
                        }
                        R.id.display_report_out_of_stock_yes_rb -> {
                            mStockout = ConstantParams.CONSTANT_PARAMS_1
                        }
                    }
                }

            mReportOutOfStockNoRb = it.find(R.id.display_report_out_of_stock_no_rb)
            mReportOutOfStockYesRb = it.find(R.id.display_report_out_of_stock_yes_rb)

            mReportStoreInfoTv = it.find(R.id.display_report_store_info_tv)
            mReportOutOfStockInfoEt = it.find(R.id.display_report_out_of_stock_info_et)

            mReportReasonInputEt = it.find(R.id.display_report_reason_input_et)

            mDisplayReportChooseContainerLl = it.find(R.id.display_report_choose_container_ll)

            mDisplayWholeImagePickerContainer =
                it.find<ImagePickerLayout>(R.id.display_whole_image_picker_container).apply {
                    title = "整体照片:"
                    tip = "(最多可添加15张)"
                    imagePicker = this@DisplayReportEditActivity
                    sizePhotoNum = 3
                    maxPhotoNum = 15
                    setmImageselectList(mWholeImageList)
                    imagePickerView = this
                    isOlnyViewMode = false
                    tag = ConstantParams.CONSTANT_PARAMS_1
                }

            mDisplayExportImagePickerContainer =
                it.find<ImagePickerLayout>(R.id.display_export_image_picker_container).apply {
                    title = "导出照片:"
                    tip = "(最多可添加15张)"
                    imagePicker = this@DisplayReportEditActivity
                    sizePhotoNum = 3
                    maxPhotoNum = 15
                    setmImageselectList(mExportImageList)
                    imagePickerView = this
                    isOlnyViewMode = false
                    tag = ConstantParams.CONSTANT_PARAMS_2
                }
        }
    }

    override fun initData() {
        mReportType = "DR"

        mTasksChild?.let {
            mDisplayReportPresenter?.requestMissionDetail(it.taskId, it.shopId)
        }
    }

    override fun missionDetailData(data: DisplayReportDetailEntity) {
        super.missionDetailData(data)
        mDisplayReportDetailEntity = data
        setData(data)
    }

    /**
     * 设置数据
     * @param displayReportDetailEntity DisplayReportDetailEntity
     */
    private fun setData(displayReportDetailEntity: DisplayReportDetailEntity) {
        mDisplayReportEditAdapter?.removeAllHeaderView()
        mDisplayReportEditAdapter?.removeAllFooterView()

        displayReportDetailEntity.apply {
            if (mTasksChild?.audit.toString() == ConstantParams.CONSTANT_PARAMS_1) {
                if (ema != null) {
                    mDisplayReportEditAdapter?.setHeaderView(mRecyclerOneHeaderView)
                    mDisplayReportEditAdapter?.setFooterView(mRecyclerTwoHeaderView, 0)

                    mDisplayReportEditAdapter?.setNewData(mutableListOf(ema))
                }
            }
            //1已审核  0：不可整改
            if (mTasksChild?.audit.toString() == ConstantParams.CONSTANT_PARAMS_1 &&
                mTasksChild?.type.toString() == ConstantParams.CONSTANT_PARAMS_0
            ) {
                mDisplayReportEditAdapter?.removeAllHeaderView()
                mDisplayReportEditAdapter?.removeAllFooterView()
                mPublicContentConfirmLl.visibility = View.GONE
            } else {
                mDisplayReportEditAdapter?.setFooterView(mRecyclerFootView, 1)
                mPublicContentConfirmLl.visibility = View.VISIBLE
            }

            mDisplayReportCaseContainerLl?.removeAllViews()

            val list = ems?.list
            list?.let {
                it.forEach { data ->

                    mItemDisplayReportCaseView =
                        View.inflate(mContext, R.layout.list_item_display_report_case, null)

                    mItemDisplayReportCaseView?.let {
                        it.find<TextView>(R.id.display_report_case_title_tv).text = data.projectName
                        it.find<TextView>(R.id.display_report_case_content_tv).text =
                            data.emcDescription

                        ImageLoader.loadImg(
                            mContext,
                            data.photoUrl,
                            it.find(R.id.display_report_case_picture_iv),
                            R.drawable.pop_hone_qgwsr_default,
                            R.drawable.pop_hone_qgwsr_default
                        )

                        (it.find(R.id.display_report_case_picture_iv) as ImageView).setOnClickListener {
                            showPreviewPhoto(data.photoUrl)
                        }
                    }

                    mDisplayReportCaseContainerLl?.addView(
                        mItemDisplayReportCaseView,
                        LinearLayout.LayoutParams(
                            LinearLayout.LayoutParams.MATCH_PARENT,
                            LinearLayout.LayoutParams.WRAP_CONTENT
                        )
                    )
                }
            }

            if (mDisplayReportChooseContainerLl?.childCount!! > 0) {
                mDisplayReportChooseContainerLl?.removeAllViews()
            }
            ehqs?.let {
                it.forEach { data ->
                    if (data.type == ConstantParams.CONSTANT_PARAMS_3) {
                        //单选
                        val block: SingleChooseWidget.() -> View = {
                            val widget = this
                            init(mContext, data, this@DisplayReportEditActivity).apply {
                                tag = widget
                            }
                        }
                        mDisplayReportChooseContainerLl?.addView(SingleChooseWidget().run(block))
                    } else if (data.type == ConstantParams.CONSTANT_PARAMS_4) {
                        //多选
                        val block: MultipleChooseWidget.() -> View = {
                            val widget = this
                            init(mContext, data,this@DisplayReportEditActivity).apply {
                                tag = widget
                            }
                        }
                        mDisplayReportChooseContainerLl?.addView(MultipleChooseWidget().run(block))
                    } else {
                        //单行填空
                        val block: SingleLineInputWidget.() -> View = {
                            val widget = this
                            init(mContext, data).apply {
                                tag = widget
                            }
                        }
                        mDisplayReportChooseContainerLl?.addView(SingleLineInputWidget().run(block))
                    }
                }
            }

            mStockout = ems?.stockout.toString()
            mReportStoreInfoTv?.text = ems?.productInfo
            mReportOutOfStockInfoEt?.setText(ems?.stockoutInfo)
            mReportReasonInputEt?.setText(ems?.excepSpec)
            if (ems?.stockout == 0) {
                mReportOutOfStockNoRb?.isChecked = true
                mReportOutOfStockYesRb?.isChecked = false
            } else if (ems?.stockout == 1) {
                mReportOutOfStockYesRb?.isChecked = true
                mReportOutOfStockNoRb?.isChecked = false
            }

            val wholeImage: List<String>? = ems?.wholeImages
            if (wholeImage.isNotNullOrSize()) {
                for (i in wholeImage!!.indices) {
                    val item1 = ImageItem()
                    item1.setImgneturl(wholeImage[i])
                    item1.setImagePath(wholeImage[i])
                    item1.setImageId(i.toString() + "")
                    item1.setUri(Uri.fromFile(File(wholeImage[i])))
                    item1.setType(1)
                    mWholeImageList.add(item1)
                }
            }
            mDisplayWholeImagePickerContainer?.refreshPhotoContentView(mWholeImageList)

            val exportImage: List<String>? = ems?.exportImages
            if (exportImage.isNotNullOrSize()) {
                for (i in exportImage!!.indices) {
                    val item1 = ImageItem()
                    item1.setImgneturl(exportImage[i])
                    item1.setImagePath(exportImage[i])
                    item1.setImageId(i.toString() + "")
                    item1.setUri(Uri.fromFile(File(exportImage[i])))
                    item1.setType(1)
                    mExportImageList.add(item1)
                }
            }
            mDisplayExportImagePickerContainer?.refreshPhotoContentView(mExportImageList)
        }
    }

    /**
     * 显示预览图片
     * @param url:String
     */
    private fun showPreviewPhoto(url: String) {
        toPhotoPreviewFragment(0, listOf(ImageItem().apply {
            imgneturl = url
            imagePath = url
            imageId = "0"
            uri = Uri.fromFile(File(url))
            type = 1
        }))
        clearAllEditFocus()
    }

    override fun submitionStatus(isSuccess: Boolean) {
        super.submitionStatus(isSuccess)

        if (isSuccess) {
            finish()
        } else {
            mUrlLists.clear()
            mQiniuUrlLists.clear()
        }
    }

    override fun setUploadData() {
        if (mWholeImageList.isNullOrEmpty() || mExportImageList.isNullOrEmpty() ||
            (mStockout == ConstantParams.CONSTANT_PARAMS_1 && mReportOutOfStockInfoEt?.text.toString()
                .isEmpty())
        ) {
            dismissDialog()
            showToast(R.string.diaplay_report_picture_out_of_stock_not_empty_tip)
            return
        }

        mCount = 0

        val list: ArrayList<ImageItem> = arrayListOf()
        list.addAll(mWholeImageList)
        list.addAll(mExportImageList)

        setUploadImage(list)
    }


    override fun uploadData() {
        super.uploadData()

        var wholeImages = listOf<String>()
        var exportImages = listOf<String>()

        if (mImageselectList.isNotNullOrSize()) {
            val totalSize = mWholeImageList.size + mExportImageList.size
            if (totalSize == mImageselectList.size) {
                //先添加已上传的
                mWholeImageList = ArrayList(mImageselectList.subList(0, mWholeImageList.size))
                mExportImageList =
                    ArrayList(mImageselectList.subList(mWholeImageList.size, mImageselectList.size))

                wholeImages = mWholeImageList.filter { item -> item.getType() == 1 }
                    .map { item -> item.imgneturl }.toList()
                exportImages = mExportImageList.filter { item -> item.getType() == 1 }
                    .map { item -> item.imgneturl }.toList()
            }
        }

        var reportId = ""
        var auditId = ""
        //已提交状态
        if (mDisplayReportDetailEntity?.status == ConstantParams.CONSTANT_STARTS_1) {
            reportId = mDisplayReportDetailEntity?.ems?.reportId ?: ""
        }
        //已审核状态
        if (mDisplayReportDetailEntity?.audit == ConstantParams.CONSTANT_STARTS_1) {
            auditId = mDisplayReportDetailEntity?.ema?.auditId ?: ""
        }

        val answers = mutableListOf<DisplayReportSubmitEntity.AnswersBean?>()
        mDisplayReportChooseContainerLl?.childrenSequence()?.forEach { view ->
            view.tag?.let {
                when (it) {
                    is SingleChooseWidget -> {
                        val answersBean = it.getSingleChooseData()
                        if(answersBean!=null){
                            answers += answersBean
                        }
                    }
                    is MultipleChooseWidget -> {
                        val answersBean = it.getMultipleChooseData()
                        if(answersBean!=null) {
                            answers += answersBean
                        }
                    }
                    is SingleLineInputWidget -> {
                        val answersBean = it.getSingleLineInputData()
                        if(answersBean!=null) {
                            answers += answersBean
                        }
                    }
                    else -> {
                    }
                }
            }
        }

        mDisplayReportPresenter?.requestSubmitMissionDetail(
            reportId, auditId,
            mTasksChild?.taskId,
            mTasksChild?.shopId,
            mStockout,
            mReportOutOfStockInfoEt?.text.toString(),
            mReportReasonInputEt?.text.toString(),
            exportImages, wholeImages, answers
        )
    }

    override fun setSelectDialog(selectType: Int, view: View?) {
        idImagePickerContainer = view as ImagePickerLayout
        clearAllEditFocus()
        if (idImagePickerContainer != null) {
            setSelectDialogListeners(View.OnClickListener { //拍照处理
                takePhoto(idImagePickerContainer)
                hideSelectDialog()

            }, View.OnClickListener { //相片处理
                getPictrue(idImagePickerContainer)
                hideSelectDialog()
            }, View.OnClickListener { hideSelectDialog() })

            showSelectDialog("选择图片", null, "相机拍照", "本地相册", "取消")
        }
    }

    override fun toPhotoPreview(i: Int, arrayList: ArrayList<ImageItem>?) {
        toPhotoPreviewFragment(i, arrayList) //大图显示
        clearAllEditFocus()
    }

    /**
     * 清除所有焦点
     */
    private fun clearAllEditFocus() {
        mReportOutOfStockInfoEt?.let {
            it.clearFocus()
            InputMethodManagerTool.getInstance(mContext).hideInputMethod(it)
        }

        mReportReasonInputEt?.let {
            it.clearFocus()
            InputMethodManagerTool.getInstance(mContext).hideInputMethod(it)
        }

        mDisplayReportChooseContainerLl?.childrenSequence()?.forEach { view ->
            view.tag?.let {
                when (it) {
                    is SingleLineInputWidget -> {
                        it.clearEditFocus(mContext)
                    }
                    else -> {
                    }
                }
            }
        }
    }

    /**
     * 调用系统拍照
     */
    protected fun takePhoto(idImagePickerContainer: ImagePickerLayout?) {
        val intent = Intent(this, TakePictureActivity::class.java)
        intent.putExtra("authority", "com.urovo.wugumofang.fileprovider")
        startActivityForResultByAnimation(intent, Params.TAKE_PICTURE)
    }

    override fun refreshAfterGetPicture(
        data: Intent?,
        mImageselectList: ArrayList<ImageItem>?,
        idImagePickerContainer: ImagePickerLayout?
    ) {
        super.refreshAfterGetPicture(data, mImageselectList, idImagePickerContainer)
        val imageSelectList = idImagePickerContainer?.getmImageselectList()
        val pos = idImagePickerContainer?.tag as String
        if (pos == ConstantParams.CONSTANT_PARAMS_1) {
            mWholeImageList = imageSelectList ?: arrayListOf()
        } else if (pos == ConstantParams.CONSTANT_PARAMS_2) {
            mExportImageList = imageSelectList ?: arrayListOf()
        }
    }

    override fun refreshAfterTakePicture(
        data: Intent?,
        mImageselectList: ArrayList<ImageItem>?,
        idImagePickerContainer: ImagePickerLayout?
    ) {
        super.refreshAfterTakePicture(data, mImageselectList, idImagePickerContainer)
        val pos = idImagePickerContainer?.tag as String
        if (pos == ConstantParams.CONSTANT_PARAMS_1) {
            mWholeImageList = mImageselectList ?: arrayListOf()
        } else if (pos == ConstantParams.CONSTANT_PARAMS_2) {
            mExportImageList = mImageselectList ?: arrayListOf()
        }
    }

    override fun setItemSelectDialog(selectType: Int, view: View?) {
        showToast(R.string.query_status_not_update_picture_tip)
    }

    override fun toItemPhotoPreview(i: Int, arrayList: ArrayList<ImageItem>?) {
        toPhotoPreviewFragment(i, arrayList) //大图显示
        clearAllEditFocus()
    }

    override fun onSingleChooseClick(option: DisplayReportDetailEntity.Ehqs.Option?) {
        clearAllEditFocus()
    }

    override fun onMultipleChooseClick(option: DisplayReportDetailEntity.Ehqs.Option?) {
        clearAllEditFocus()
    }

}