package com.urovo.activity.display.view

import android.content.Intent
import android.view.View
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.display.model.InspectorDisplayEntity
import com.urovo.activity.display.presenter.InspectorDisplayContract
import com.urovo.activity.display.presenter.InspectorDisplayPresenter
import com.urovo.activity.display.view.adapter.InspectorDisplayAdapter
import com.urovo.activity.display.view.adapter.InspectorDisplayAdapter.OnPictureClickListener
import com.urovo.wugumofang.R
import com.urovo.activity.display.ReportWithPhotoBaseActivity
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.system.SystemHelper
import com.xp.pro.imagepickerlib.bean.ImageItem
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout
import com.zsy.libcore.ui.ada.helper.DividerItemDecoration
import kotlinx.android.synthetic.main.public_confirm_btn_view.*
import kotlinx.android.synthetic.main.public_list_content_view.*
import kotlinx.android.synthetic.main.public_title_more_ll.*

/**
 * 督察陈列检查
 * @author LW
 */
class InspectorDisplayCheckActivity : ReportWithPhotoBaseActivity(), InspectorDisplayContract.IView,
    OnPictureClickListener, View.OnClickListener {

    private var mInspectorDisplayPresenter: InspectorDisplayPresenter? = null

    private var mInspectorDisplayAdapter: InspectorDisplayAdapter? = null

    private var mInspectorDisplayEntity: InspectorDisplayEntity? = null

    private var mShopStatus: String? = ""

    override fun fetchToolBarTitle(): Int = R.string.inspector_display_check_title

    override fun addChildLayoutId(): Int = R.layout.public_list_content_view


    override fun initView() {
        mInspectorDisplayPresenter = InspectorDisplayPresenter(this).apply {
            mPresenter = this
        }

        mInspectorDisplayAdapter = InspectorDisplayAdapter(listOf(), this)
        mPublicListViewLv.apply {
            layoutManager = LinearLayoutManager(mContext, RecyclerView.VERTICAL, false)
            addItemDecoration(
                DividerItemDecoration(
                    mContext,
                    DividerItemDecoration.VERTICAL_LIST,
                    1,
                    ContextCompat.getColor(mContext, R.color.pop_query_devider)
                )
            )
            adapter = mInspectorDisplayAdapter
        }
    }
    override fun initData() {

    }

    override fun onResume() {
        super.onResume()
        mInspectorDisplayPresenter?.requestMissionByStoreNo()
    }


    /**
     * 门店状态, 1待提交、2待审核可修改、3已审核可整改、4可再次整改、5免提交、6已截止
     * 1 待提交 清除数据 高亮显示提交按钮
     * 2 待审核可修改 取最新一条数据 修改按钮高亮，新增按钮置灰
     * 3 已审核可整改 取最新一条数据 修改按钮置灰，新增按钮高亮
     * 4 可再次整改 取最新一条数据 修改按钮置灰，新增按钮置灰
     * 5 免提交 无按钮
     * 6 已截止 无按钮
     */
    override fun missionByStoreNoData(data: InspectorDisplayEntity) {
        super.missionByStoreNoData(data)
        mInspectorDisplayEntity = data
        if (null == data.submitList || data.submitList?.isEmpty() == true) {
            // 新增一条空数据
            val submitList: MutableList<InspectorDisplayEntity.SubmitBean> = ArrayList()
            val submitBean = InspectorDisplayEntity.SubmitBean(
                "",
                "",
                "",
                "",
                "",
                data.endTime,
                "",
                SystemHelper.LoginUserId,
                SystemHelper.LoginUserName,
                data.missionName,
                "1",
                SystemHelper.LoginOrganId,
                SystemHelper.LoginOrganName,
                null
            )
            submitList.add(submitBean)
            data.storeNo = SystemHelper.LoginOrganId
            data.storeName = SystemHelper.LoginOrganName
            data.submitList = submitList
        }
        setData(data)
    }

    /**
     * 门店状态, 1待提交、2待审核可修改、3已审核可整改、4可再次整改、5免提交、6已截止
     * 1 待提交 清除数据 高亮显示提交按钮
     * 2 待审核可修改 取最新一条数据 修改按钮高亮，新增按钮置灰
     * 3 已审核可整改 取最新一条数据 修改按钮置灰，新增按钮高亮
     * 4 可再次整改 取最新一条数据 修改按钮置灰，新增按钮置灰
     * 5 免提交 无按钮
     * 6 已截止 无按钮
     */
    private fun setData(data: InspectorDisplayEntity) {
        data.apply {
            mPublicSubTitleLl.visibility = View.VISIBLE
            mPublicSubTitleTv.text = storeName

            mShopStatus = shopStatus
            mInspectorDisplayAdapter?.setNewData(submitList)
            when (mShopStatus) {
                ConstantParams.CONSTANT_PARAMS_1 -> {
                    //如果是待提交直接进入提交页面
                    jumpInspectorDisplayEditActivity(0)
                    finish()
                }
                ConstantParams.CONSTANT_PARAMS_2, ConstantParams.CONSTANT_PARAMS_4 -> {
                    mPublicContentConfirmLl.visibility = View.VISIBLE
                    mPublicContentModifyBtn.visibility = View.VISIBLE
                    mPublicContentConfirmBtn.visibility = View.VISIBLE
                    mPublicContentModifyBtn.setOnClickListener(null)
                    mPublicContentConfirmBtn.setOnClickListener(this@InspectorDisplayCheckActivity)
                    mPublicContentModifyBtn.setText(R.string.add_btn_txt)
                    mPublicContentModifyBtn.setTextColor(
                        ContextCompat.getColor(
                            mContext,
                            R.color.white
                        )
                    )
                    mPublicContentModifyBtn.setBackgroundResource(
                        R.drawable.btn_gray_round_fill_nomal
                    )

                    mPublicContentConfirmBtn.setText(R.string.modify_btn_txt)
                    mPublicContentConfirmBtn.setTextColor(
                        ContextCompat.getColor(
                            mContext,
                            R.color.white
                        )
                    )
                    mPublicContentConfirmBtn.setBackgroundResource(
                        R.drawable.btn_orange_round_fill_nomal
                    )
                }
                ConstantParams.CONSTANT_PARAMS_3 -> {
                    mPublicContentConfirmLl.visibility = View.VISIBLE
                    mPublicContentModifyBtn.visibility = View.VISIBLE
                    mPublicContentConfirmBtn.visibility = View.VISIBLE

                    mPublicContentModifyBtn.setOnClickListener(this@InspectorDisplayCheckActivity)
                    mPublicContentConfirmBtn.setOnClickListener(null)
                    mPublicContentModifyBtn.setText(R.string.add_btn_txt)

                    mPublicContentModifyBtn.setTextColor(
                        ContextCompat.getColor(
                            mContext,
                            R.color.white
                        )
                    )
                    mPublicContentModifyBtn.setBackgroundResource(
                        R.drawable.btn_orange_round_fill_nomal
                    )
                    mPublicContentConfirmBtn.setText(R.string.modify_btn_txt)
                    mPublicContentConfirmBtn.setTextColor(
                        ContextCompat.getColor(
                            mContext,
                            R.color.white
                        )
                    )
                    mPublicContentConfirmBtn.setBackgroundResource(
                        R.drawable.btn_gray_round_fill_nomal
                    )
                }
                ConstantParams.CONSTANT_PARAMS_5 -> {
                    showError(R.string.inspector_display_no_submission)
                    showTipDialog(getString(R.string.hint), getString(R.string.inspector_display_task_not_submit))

                }
                ConstantParams.CONSTANT_PARAMS_6 -> {
                    mPublicContentConfirmLl.setVisibility(View.GONE)
                    showTipDialog(getString(R.string.hint), getString(R.string.inspector_display_task_has_expired))
                }
                ConstantParams.CONSTANT_PARAMS_7 -> {
                    showError(R.string.inspector_display_no_task)
                    mPublicContentConfirmLl.setVisibility(View.GONE)
                    showTipDialog(getString(R.string.hint), getString(R.string.inspector_display_not_task))
                }
                else -> {
                }
            }
        }
    }

    override fun setSelectDialog(selectType: Int, view: View?) {
        showToast(R.string.query_status_not_update_picture_tip)
    }

    override fun toPhotoPreview(i: Int, arrayList: ArrayList<ImageItem>?) {
        toPhotoPreviewFragment(i, mImageselectList) //大图显示
    }

    override fun onClick(v: View) {
        if (v.id == R.id.mPublicContentModifyBtn) { //新增
            jumpInspectorDisplayEditActivity(0)
        } else if (v.id == R.id.mPublicContentConfirmBtn) { //修改
            jumpInspectorDisplayEditActivity(1)
        }
    }

    override fun refreshAfterTakePicture(
        data: Intent?,
        mImageselectList: java.util.ArrayList<ImageItem>?,
        idImagePickerContainer: ImagePickerLayout?
    ) {
        super.refreshAfterTakePicture(data, mImageselectList, idImagePickerContainer)
        if (mImageselectList != null && idImagePickerContainer != null) {
            mInspectorDisplayAdapter?.refreshAfterTakePicture(
                mImageselectList,
                idImagePickerContainer
            )
        }
    }

    /**
     * 跳转到 InspectorDisplayEditActivity
     */
    private fun jumpInspectorDisplayEditActivity(type: Int) {
        val intent = Intent(this, InspectorDisplayEditActivity::class.java)
        intent.putExtra("shopStatus", mShopStatus)
        intent.putExtra("type", type.toString() + "")
        startActivity(intent)
    }
}