package com.urovo.activity.display.view

import android.content.Intent
import android.net.Uri
import android.text.TextUtils
import android.view.View
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.display.model.InspectorDisplayEntity
import com.urovo.activity.display.presenter.InspectorDisplayContract
import com.urovo.activity.display.presenter.InspectorDisplayPresenter
import com.urovo.activity.display.view.adapter.InspectorDisplayEditAdapter
import com.urovo.wugumofang.R
import com.urovo.activity.display.ReportWithPhotoBaseActivity
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.system.SystemHelper
import com.xp.pro.imagepickerlib.TakePictureActivity
import com.xp.pro.imagepickerlib.bean.ImageItem
import com.xp.pro.imagepickerlib.global.Params
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout.ImagePicker
import com.zsy.libcore.ui.ada.helper.DividerItemDecoration
import kotlinx.android.synthetic.main.public_confirm_btn_view.*
import kotlinx.android.synthetic.main.public_list_content_view.*
import org.jetbrains.anko.find
import java.io.File
import java.util.*
import kotlin.collections.ArrayList

/**
 * 编辑督察陈列检查
 * @author LW
 */
class InspectorDisplayEditActivity : ReportWithPhotoBaseActivity(), InspectorDisplayContract.IView,
    ImagePicker {

    private var mInspectorDisplayPresenter: InspectorDisplayPresenter? = null

    override fun fetchToolBarTitle(): Int = R.string.inspector_display_check_title

    override fun addChildLayoutId(): Int = R.layout.public_list_content_view

    private var mShopStatus: String? = ""

    private var mInspectorDisplayEditAdapter: InspectorDisplayEditAdapter? = null

    private var mInspectorDisplayEntity: InspectorDisplayEntity? = null

    private var mSubmitBean: InspectorDisplayEntity.SubmitBean? = null

    private var mRecyclerFootView: View? = null

    private val mType: String by lazy {
        intent.getStringExtra("type")
    }

    override fun initView() {
        mInspectorDisplayPresenter = InspectorDisplayPresenter(this).apply {
            mPresenter = this
        }

        mRecyclerFootView = layoutInflater.inflate(R.layout.inspector_display_add_footer_view, null)
        mInspectorDisplayEditAdapter = InspectorDisplayEditAdapter(listOf()).apply {
            setFooterView(mRecyclerFootView)
        }

        mPublicListViewLv.apply {
            layoutManager = LinearLayoutManager(mContext, RecyclerView.VERTICAL, false)
            addItemDecoration(
                DividerItemDecoration(
                    mContext,
                    DividerItemDecoration.VERTICAL_LIST,
                    1,
                    ContextCompat.getColor(mContext, R.color.pop_query_devider)
                )
            )
            adapter = mInspectorDisplayEditAdapter
        }

        mPublicContentConfirmLl.visibility=View.VISIBLE
        mPublicContentConfirmBtn.setOnClickListener {
            setUploadData()
        }
        initFootView()
    }

    private var mReportNameTv: TextView? = null
    private var mReportTimeTv: TextView? = null
    private var mReportStaffNameTv: TextView? = null
    private var mReportStaffIdTv: TextView? = null
    private var mReportExplainEt: EditText? = null


    private fun initFootView() {
        mRecyclerFootView?.let {
            mReportNameTv = it.find(R.id.report_name)
            mReportTimeTv = it.find(R.id.report_time)
            mReportStaffNameTv = it.find(R.id.report_staff_name)
            mReportStaffIdTv = it.find(R.id.report_staff_id)
            mReportExplainEt = it.find(R.id.report_explain_edt)

            it.find<LinearLayout>(R.id.report_submit_time_ll).visibility = View.GONE
            it.find<LinearLayout>(R.id.report_status_ll).visibility = View.GONE

            idImagePickerContainer = it.find<ImagePickerLayout>(R.id.id_image_picker_container).apply {
                title = "陈列照片:"
                tip = "(最多可添加8张)"
                imagePicker = this@InspectorDisplayEditActivity
                sizePhotoNum = 3
                maxPhotoNum = 8
                setmImageselectList(mImageselectList)
                imagePickerView = this
                isOlnyViewMode = false
            }
        }


    }

    override fun initData() {
        mReportType = "TD"

        mInspectorDisplayPresenter?.requestMissionByStoreNo()
    }


    override fun missionByStoreNoData(data: InspectorDisplayEntity) {
        super.missionByStoreNoData(data)
        mInspectorDisplayEntity = data
        if (null == data.submitList || data.submitList?.isEmpty() == true) {
            // 新增一条空数据
            val submitList: MutableList<InspectorDisplayEntity.SubmitBean> = ArrayList()
            val submitBean = InspectorDisplayEntity.SubmitBean(
                "",
                "",
                "",
                "",
                "",
                data.endTime,
                "",
                SystemHelper.LoginUserId,
                SystemHelper.LoginUserName,
                data.missionName,
                "1",
                SystemHelper.LoginOrganId,
                SystemHelper.LoginOrganName,
                null
            )
            submitList.add(submitBean)
            data.storeNo = SystemHelper.LoginOrganId
            data.storeName = SystemHelper.LoginOrganName
            data.submitList = submitList
        }
        setData(data)
    }

    private fun setData(inspectorDisplayBean: InspectorDisplayEntity) {
        val submitList = inspectorDisplayBean.submitList

        if (null != submitList && submitList.isNotEmpty()) {
            if (submitList.size > 1) {
                mSubmitBean = selectLastOne(submitList)
            } else {
                mSubmitBean = submitList.get(0)
            }
            mSubmitBean?.detailId = inspectorDisplayBean.detailId
            mShopStatus = mSubmitBean?.shopStatus
            mReportNameTv?.text = mSubmitBean?.missionName
            mReportTimeTv?.text =
                if (null == inspectorDisplayBean.missionRemark) inspectorDisplayBean.endTime else inspectorDisplayBean.missionName
            mReportStaffNameTv?.text = mSubmitBean?.userName
            mReportStaffIdTv?.text = mSubmitBean?.userNo
            if (!TextUtils.isEmpty(mType) && mType == "1") { //修改需填充数据，新增不需要填充数据
                mReportExplainEt?.setText(mSubmitBean?.remark)
                val imgs: List<String>? = mSubmitBean?.urls
                if (imgs!=null&&imgs.isNotEmpty()) {
                    for (i in imgs.indices) {
                        val item1 = ImageItem()
                        item1.setImgneturl(imgs[i])
                        item1.setImagePath(imgs[i])
                        item1.setImageId(i.toString() + "")
                        item1.setUri(Uri.fromFile(File(imgs[i])))
                        item1.setType(1)
                        mImageselectList.add(item1)
                    }
                }
                mSubmitBean?.storeImageSelectList = mImageselectList
                idImagePickerContainer.refreshPhotoContentView(mImageselectList)
            }
        }
    }

    fun selectLastOne(list: List<InspectorDisplayEntity.SubmitBean>): InspectorDisplayEntity.SubmitBean {
        var submitBean = InspectorDisplayEntity.SubmitBean()
        val dates = arrayOfNulls<Long>(list.size)
        for (i in list.indices) {
            // 把date类型的时间对象转换为long类型，时间越往后，long的值就越大，
            // 所以就依靠这个原理来判断距离现在最近的时间
            val date: Date = DateUtil.String2Date(list[i].updateTime, "yyyy-MM-dd HH:mm:ss")
            dates[i] = date.time
        }
        var maxIndex = dates[0] // 定义最大值为该数组的第一个数
        for (j in dates.indices) {
            if (maxIndex!! < dates[j]!!) {
                maxIndex = dates[j]
                // 找到了这个j
                submitBean = list[j]
            }
        }
        return submitBean
    }

    override fun submitionStatus(isSuccess: Boolean) {
        super.submitionStatus(isSuccess)

        if (isSuccess) {
            finish()
        } else {
            mUrlLists.clear()
            mQiniuUrlLists.clear()
        }
    }


    override fun setUploadData() {
        if (TextUtils.isEmpty(
                mReportExplainEt?.text.toString()
            ) && idImagePickerContainer.getmImageselectList().isEmpty()
        ) {
            dismissDialog()
            showToast(R.string.inspector_display_picture_remark_not_empty_tip)
            return
        }
        super.setUploadData()
    }


    override fun uploadData() {
        super.uploadData()
        if (mImageselectList != null && mImageselectList.isNotEmpty()) {
            //先添加已上传的
            for (i in mImageselectList.indices) {
                //如果图片时上传过后再寻得到则存在id
                if (1 == mImageselectList[i].getType()) {
                    //将已经在上传过的图片添加到url中
                    mUrlLists.add(mImageselectList[i].getImgneturl())
                }
            }
        }
        var submitUrl = ""
        if (mUrlLists.isNotEmpty()) {
            for (i in 0 until mUrlLists.size) {
                if (i < mUrlLists.size - 1) {
                    submitUrl += mUrlLists.get(i) + ","
                } else {
                    submitUrl += mUrlLists.get(i)
                }
            }
        }
        val remark: String = mReportExplainEt?.text.toString()
        // shopStatus (integer, optional):门店状态, 1待提交、2待审核可修改、3已审核可整改、4可再次整改、5免提交、6已截止 ,
        if (ConstantParams.CONSTANT_PARAMS_2 == mShopStatus || ConstantParams.CONSTANT_PARAMS_4 == mShopStatus) {
            mInspectorDisplayPresenter?.requestUpdateSubmition(
                mSubmitBean?.submitId ?: "",
                remark,
                submitUrl
            )
        } else {
            mInspectorDisplayPresenter?.requestAddSubmition(
                mSubmitBean?.detailId ?: "",
                remark,
                submitUrl
            )

        }
    }


    override fun setSelectDialog(selectType: Int, view: View?) {
        idImagePickerContainer = view as ImagePickerLayout
        if (idImagePickerContainer != null) {
            setSelectDialogListeners(View.OnClickListener { //拍照处理
                takePhoto(idImagePickerContainer)
                hideSelectDialog()
            }, View.OnClickListener { //相片处理
                getPictrue(idImagePickerContainer)
                hideSelectDialog()
            }, View.OnClickListener { hideSelectDialog() })
            showSelectDialog("选择图片", null, "相机拍照", "本地相册", "取消")
        }
    }

    override fun toPhotoPreview(i: Int, arrayList: ArrayList<ImageItem>?) {
        toPhotoPreviewFragment(i, mImageselectList) //大图显示
    }

    /**
     * 调用系统拍照
     */
    protected fun takePhoto(idImagePickerContainer: ImagePickerLayout?) {
        val intent = Intent(this, TakePictureActivity::class.java)
        intent.putExtra("authority", "com.urovo.wugumofang.fileprovider")
        startActivityForResultByAnimation(intent, Params.TAKE_PICTURE)
    }

}