package com.urovo.activity.display.view.adapter

import android.view.View
import com.chad.library.adapter.base.BaseMultiItemQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.chad.library.adapter.base.entity.MultiItemEntity
import com.urovo.activity.display.model.DisplayReportGroup
import com.urovo.activity.display.model.TasksChild
import com.urovo.wugumofang.R

/**
 * 陈列上报适配器
 * @author LW
 */
class DisplayReportAdapter(
    entity: List<MultiItemEntity>,
    private val onClickListener: OnClickListener?
) :
    BaseMultiItemQuickAdapter<MultiItemEntity?, BaseViewHolder?>(entity) {

    companion object {
        const val TYPE_LEVEL_0 = 0
        const val TYPE_LEVEL_1 = 1
    }

    init {
        addItemType(TYPE_LEVEL_0, R.layout.ll_report_exlist_group)
        addItemType(TYPE_LEVEL_1, R.layout.ll_report_exlist_child)
    }

    override fun convert(holder: BaseViewHolder?, item: MultiItemEntity?) {
        holder?.apply {
            item?.let {
                when (itemViewType) {
                    TYPE_LEVEL_0 -> {
                        val displayReportGroup: DisplayReportGroup = it as DisplayReportGroup
                        setText(R.id.qback_group_type, displayReportGroup.shopName)
                        setImageResource(
                            R.id.list_group_arrow,
                            if (displayReportGroup.isExpanded) R.drawable.list_group_open else R.drawable.list_group_close
                        )
                        itemView.setOnClickListener {
                            val pos = adapterPosition
                            val expanded = displayReportGroup.isExpanded
                            if (displayReportGroup.isExpanded) {
                                collapse(pos)
                            } else {
                                expand(pos)
                            }
                            onClickListener?.onGroupClick(itemView, pos, displayReportGroup,expanded)
                        }
                    }
                    TYPE_LEVEL_1 -> {
                        val tasksChild: TasksChild = it as TasksChild
                        setText(
                            R.id.report_child_title, tasksChild.taskName
                        ).setText(R.id.report_child_subtitle, tasksChild.remark)
                        itemView.setOnClickListener {
                            val pos = adapterPosition
                            onClickListener?.onChildClick(itemView, pos, tasksChild)
                        }
                    }
                    else -> {
                    }
                }
            }
        }
    }

    interface OnClickListener {

        fun onGroupClick(view: View?, pos: Int, bean: DisplayReportGroup?,expanded:Boolean)

        fun onChildClick(view: View?, pos: Int, bean: TasksChild?)
    }
}