package com.urovo.activity.display.view.adapter

import android.net.Uri
import android.text.TextUtils
import android.view.View
import android.widget.LinearLayout
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.urovo.activity.display.model.InspectorDisplayEntity.SubmitBean
import com.urovo.activity.display.service.DataTransform
import com.urovo.wugumofang.R
import com.xp.pro.imagepickerlib.bean.ImageItem
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout
import com.xp.pro.imagepickerlib.widgets.ImagePickerLayout.ImagePicker
import java.io.File

/**
 * 督察陈列检查适配器
 * @author LW
 */
class InspectorDisplayAdapter(
    private val submitBean: List<SubmitBean>,
    private val onPictureClickListener: OnPictureClickListener?
) : BaseQuickAdapter<SubmitBean, BaseViewHolder>(
    R.layout.list_item_inspector_display_view,
    submitBean
), ImagePicker {

    override fun convert(helper: BaseViewHolder?, item: SubmitBean?) {
        helper?.apply{
            item?.let {
                setText(R.id.report_name,it.missionName)
                setText(R.id.report_submit_time,it.updateTime)
                setText(R.id.report_time,  if (null == it.missionRemark) it.endTime else it.missionRemark)
                setText(R.id.report_staff_name,it.userName)
                setText(R.id.report_staff_id,it.userNo)
                setText(R.id.report_status,DataTransform.getShopStatus(it.shopStatus))
                setText(R.id.report_rate_item_tip,it.remark)

                if (TextUtils.isEmpty(it.updateTime)) {
                    getView<LinearLayout>(R.id.report_submit_time_ll).visibility=View.GONE
                }

                val imageselectList = mutableListOf<ImageItem>()
                val imgs = it.urls
                if (null != imgs && !imgs.isEmpty()) {
                    for (i in imgs.indices) {
                        val item1 = ImageItem()
                        item1.setImgneturl(imgs[i])
                        item1.setImagePath(imgs[i])
                        item1.setImageId(i.toString() + "")
                        item1.setUri(Uri.fromFile(File(imgs[i])))
                        item1.setType(1)
                        imageselectList.add(item1)
                    }
                }
                it.storeImageSelectList = imageselectList as ArrayList<ImageItem>
                getView<ImagePickerLayout>(R.id.id_image_picker_container_img).apply {
                    setmImageselectList(it.storeImageSelectList)
                    setTitleVisibility(View.GONE)
                    setTipVisibility(View.GONE)
                    isOlnyViewMode = true
                    imagePicker = this@InspectorDisplayAdapter
                    maxPhotoNum = 8
                    sizePhotoNum = 3
                    imagePickerView = this
                    refreshPhotoContentView(it.storeImageSelectList)
                }
            }
        }
    }

    override fun setSelectDialog(selectType: Int, view: View) {
        onPictureClickListener?.setSelectDialog(selectType, view)
    }

    override fun toPhotoPreview(i: Int, arrayList: ArrayList<ImageItem>) {
        onPictureClickListener?.toPhotoPreview(i, arrayList)
    }

    interface OnPictureClickListener {
        fun setSelectDialog(selectType: Int, view: View?)
        fun toPhotoPreview(i: Int, arrayList: ArrayList<ImageItem>?)
    }

    fun refreshAfterTakePicture(
        mImageselectList: ArrayList<ImageItem>,
        idImagePickerContainer: ImagePickerLayout
    ) {
        val position = idImagePickerContainer.tag as Int
        //在对应位置图片集合里加入拍照返回图片
        submitBean[position].storeImageSelectList = mImageselectList
    }
}