package com.urovo.activity.display.view.widget

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.urovo.activity.display.model.DisplayReportDetailEntity
import com.urovo.activity.display.model.DisplayReportSubmitEntity
import com.urovo.activity.display.view.holder.MultipleChooseHolder
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.utils.isNotNullOrSize
import org.jetbrains.anko.find

/**
 * 多选选择View
 * @author LW
 */
class MultipleChooseWidget {

    private var mEhqs: DisplayReportDetailEntity.Ehqs? = null

    /**
     * 获取多选数据
     * @return DisplayReportSubmitEntity.AnswersBean
     */
    fun getMultipleChooseData(): DisplayReportSubmitEntity.AnswersBean? {
        mEhqs?.let {
            val selected = mutableListOf<String>()
            it.values?.forEach { option ->
                if (option.checked == ConstantParams.CONSTANT_PARAMS_1) {
                    selected.add(option.optId)
                }
            }
            if (selected.isNotNullOrSize()) {
                return DisplayReportSubmitEntity.AnswersBean(
                    it.content,
                    it.modelId,
                    selected,
                    it.type
                )
            }
        }
        return null
    }

    fun init(
        context: Context,
        data: DisplayReportDetailEntity.Ehqs,
        listener: OnMultipleChooseClickListener?
    ): View {
        this.mEhqs = data

        val multipleChooseParent: View = LayoutInflater.from(context)
            .inflate(R.layout.list_item_display_multiple_choose, null, false)

        multipleChooseParent.find<TextView>(R.id.item_display_title_tv)
            .apply {
                text = data.title
                tag = data.modelId
            }
        val containerLl =
            multipleChooseParent.find<LinearLayout>(R.id.item_display_container_Ll)
        if (containerLl.childCount > 0) {
            containerLl.removeAllViews()
        }
        data.values?.forEach { value ->

            val multipleChooseChild = LayoutInflater.from(context)
                .inflate(
                    R.layout.list_item_display_multiple_choose_child,
                    null,
                    false
                )

            multipleChooseChild.tag = MultipleChooseHolder().apply {
                chooseCbIv =
                    multipleChooseChild.find<ImageView>(R.id.item_display_choose_cb_iv)
                        .apply {
                            if (value.checked == ConstantParams.CONSTANT_PARAMS_1) {
                                setBackgroundResource(R.drawable.square_orange)
                            } else {
                                setBackgroundResource(R.drawable.square_gray)
                            }
                            tag = value
                        }
                chooseCbTv =
                    multipleChooseChild.find<TextView>(R.id.item_display_choose_cb_tv)
                        .apply {
                            text = value.value
                        }
                multipleChooseChild.setOnClickListener { view ->
                    view.tag?.let {
                        (it as MultipleChooseHolder).apply {
                            val option = chooseCbIv?.tag as DisplayReportDetailEntity.Ehqs.Option
                            listener?.onMultipleChooseClick(option)
                            if (option.checked == ConstantParams.CONSTANT_PARAMS_1) {
                                option.checked = ConstantParams.CONSTANT_PARAMS_0
                                chooseCbIv?.setBackgroundResource(R.drawable.square_gray)
                            } else {
                                option.checked = ConstantParams.CONSTANT_PARAMS_1
                                chooseCbIv?.setBackgroundResource(R.drawable.square_orange)
                            }
                        }
                    }
                }
            }
            containerLl.addView(multipleChooseChild)
        }
        return multipleChooseParent
    }

    interface OnMultipleChooseClickListener {
        fun onMultipleChooseClick(option: DisplayReportDetailEntity.Ehqs.Option?)
    }
}