package com.urovo.activity.display.view.widget

import android.content.Context
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.widget.EditText
import android.widget.TextView
import com.urovo.activity.display.model.DisplayReportDetailEntity
import com.urovo.activity.display.model.DisplayReportSubmitEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.ui.InputMethodManagerTool
import com.urovo.wugumofang.utils.notNullAndEmpty
import org.jetbrains.anko.find

/**
 * 单行输入View
 * @author LW
 */

class SingleLineInputWidget {

    private var mEhqs: DisplayReportDetailEntity.Ehqs? = null

    /**
     * 获取输入框数据
     * @return DisplayReportSubmitEntity.AnswersBean
     */
    fun getSingleLineInputData(): DisplayReportSubmitEntity.AnswersBean? {
        mEhqs?.let {
            if (it.content.notNullAndEmpty()) {
                return DisplayReportSubmitEntity.AnswersBean(it.content, it.modelId, null, it.type)
            }
        }
        return null
    }

    /**
     * 失去焦点
     */
    fun clearEditFocus(context:Context) {
        mInputEt?.let {
            it.clearFocus()
            InputMethodManagerTool.getInstance(context).hideInputMethod(it)
        }
    }

    private var mInputEt: EditText? = null


    fun init(context: Context, data: DisplayReportDetailEntity.Ehqs): View {
        this.mEhqs = data

        val singleLineInputParent: View = LayoutInflater.from(context)
            .inflate(R.layout.list_item_display_single_line_input, null, false)

        singleLineInputParent.find<TextView>(R.id.item_display_title_tv)
            .apply {
                text = data.title
            }

        mInputEt = singleLineInputParent.find(R.id.item_display_input_et)
        mInputEt?.apply {
            val content = data.content
            if (content.notNullAndEmpty()) {
                setText(content)
            }
            tag = data

            addTextChangedListener(object : TextWatcher {
                override fun beforeTextChanged(
                    s: CharSequence?,
                    start: Int,
                    count: Int,
                    after: Int
                ) {
                }

                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                }

                override fun afterTextChanged(s: Editable?) {
                    mEhqs?.content = s.toString()
                }
            })
        }
        return singleLineInputParent
    }
}