package com.urovo.activity.goodsmanager

import android.view.View
import android.view.ViewGroup
import com.urovo.activity.goodsmanager.adapter.order.OrderDeliveryDifferenceQueryAdapter
import com.urovo.activity.goodsmanager.model.OrderCommonEntity
import com.urovo.wgmf.db.dao.entitys.DingHuoOplQtyEntity
import com.urovo.activity.goodsmanager.model.OrderDeliveryDifferenceQueryEntity
import com.urovo.activity.goodsmanager.model.OrderGoodsQueryEntity
import com.urovo.wgmf.tool.TimeHelper
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.urovo.wugumofang.view.LoadWaitDominator
import com.wgmf.trade.presenter.OrderGoodPre
import com.wgmf.trade.view.OrderGoodCon
import kotlinx.android.synthetic.main.activity_order_delivery_difference_query.*
import kotlinx.android.synthetic.main.popup_order_goods_query.*
import xp.timepicker.com.widget.bean.DateType
import java.util.*

/**
 * @author LW on 2023/11/30.
 * 订货与发货差异查询
 */
class OrderDeliveryDifferenceQueryActivity : BaseActivity(), OrderGoodCon.View {
    private var mOrderGoodPre: OrderGoodPre? = null

    private var mOrderDeliveryDifferenceQueryAdapter: OrderDeliveryDifferenceQueryAdapter? = null

    private val mBeginDate: String by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.ORDER_DELIVERY_DIFFERENCE_QUERY_BEGIN_DATE)
    }

    private val mEndDate: String by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.ORDER_DELIVERY_DIFFERENCE_QUERY_END_DATE)
    }

    override fun addRootLayoutId(): Int = R.layout.activity_order_delivery_difference_query

    override fun rootViewGroup(): ViewGroup = order_delivery_differencequery_container_rl

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.order_delivery_difference_query)
    }

    override fun initViewNew() {
        initTitle()
        initPopView()

        mOrderGoodPre = OrderGoodPre(this)

        mOrderDeliveryDifferenceQueryAdapter =
            OrderDeliveryDifferenceQueryAdapter(
                mContext, mutableListOf()
            )
        order_delivery_difference_query_lv.setAdapter(mOrderDeliveryDifferenceQueryAdapter)

        order_delivery_difference_query_title_tv.apply {
            text =
                StringUtils.formatString(
                    mContext.getString(R.string.query_stores),
                    SystemHelper.LoginOrganName,
                    order_goods_query_start_date_tv.text.toString(),
                    order_goods_query_end_date_tv.text.toString()
                )
            setOnClickListener { showPopWindow(true) }
        }
        order_goods_query_btn.setOnClickListener {
            queryOrderGoodsDiff()
        }
        order_goods_query_btn.performClick()
    }

    private fun queryOrderGoodsDiff() {
        mOrderGoodPre?.getOrderGoodsDiff(
            SystemHelper.LoginOrganId,
            order_goods_query_start_date_tv.text.toString(),
            order_goods_query_end_date_tv.text.toString()
        )
    }

    /**
     * 显隐弹窗
     */
    private fun showPopWindow(isVisible: Boolean) {
        order_delivery_difference_query_pop.run {
            visibility = if (isVisible) View.VISIBLE else View.GONE
        }
    }

    private fun initPopView() {
        order_delivery_difference_query_pop.apply {
            setOnClickListener { showPopWindow(false) }
            order_goods_query_store_tv.text = SystemHelper.LoginOrganName
            order_goods_query_start_date_tv?.apply {
                if (mBeginDate.notNullAndEmpty()) {
                    text = mBeginDate
                    tag = DateUtil.String2Date(text.toString(), DateUtil.TYPE_YMD)
                } else {
                    text = DateUtil.getDayBefore(Date(),6)
                    tag = DateUtil.String2Date(text.toString(), DateUtil.TYPE_YMD)

                }
                setOnClickListener {
                    TimeHelper.showDatePickDialog(
                        this@OrderDeliveryDifferenceQueryActivity,
                        DateType.TYPE_YMD
                    ).apply {
                        val date = tag
                        if (date != null && date is Date) {
                            setSelectDate(date)
                        }
                        setOnSureLisener {
                            if (it != null) {
                                tag = it
                                text = DateUtil.getDate(it)
                                refreshDataToView()
                            }
                        }
                    }.show()
                }
            }

            //结束日期
            order_goods_query_end_date_tv?.apply {
                if (mEndDate.notNullAndEmpty()) {
                    text = mEndDate
                    tag = DateUtil.String2Date(mEndDate, DateUtil.TYPE_YMD)
                } else {
                    text = DateUtil.getDate(Date())
                }
                setOnClickListener {
                    TimeHelper.showDatePickDialog(
                        this@OrderDeliveryDifferenceQueryActivity,
                        DateType.TYPE_YMD
                    ).apply {
                        val date = tag
                        if (date != null && date is Date) {
                            setSelectDate(date)
                        }
                        setOnSureLisener {
                            if (it != null) {
                                tag = it
                                text = DateUtil.getDate(it)
                                refreshDataToView()
                            }
                        }
                    }.show()
                }
            }
        }
    }

    private fun refreshDataToView() {
        order_delivery_difference_query_title_tv.text = StringUtils.formatString(
            mContext.getString(R.string.query_stores),
            SystemHelper.LoginOrganName,
            order_goods_query_start_date_tv.text.toString(),
            order_goods_query_end_date_tv.text.toString()
        )
    }

    override fun showOrderGoodsDetail(list: MutableList<OrderGoodsQueryEntity>?) {

    }

    override fun showOrderCommon(list: MutableList<OrderCommonEntity>?) {
    }

    /**
     * 订货与差异
     */
    override fun showOrderGoodsDiff(list: MutableList<OrderDeliveryDifferenceQueryEntity>?) {
        showPopWindow(false)
        dismissAllState()
        list?.let {
            val filterProductCode = order_goods_query_filter_product_et?.text.toString()
            //过滤
            it.forEach {
                if (filterProductCode.isNotBlank()) {
                    it.diffDetails =
                        it.diffDetails?.filter { it.proNo == filterProductCode }
                }
            }
            val resultInfo =
                it.filter { it.diffDetails.isNotNullOrSize() }
            if (resultInfo.isEmpty()) {
                mOrderDeliveryDifferenceQueryAdapter?.setNewData(mutableListOf())
                showState(LoadWaitDominator.NO_DATA)
                return
            }

            mOrderDeliveryDifferenceQueryAdapter?.setNewData(resultInfo)
            order_delivery_difference_query_lv.showListHeaderView()
            mOrderDeliveryDifferenceQueryAdapter?.setRecyclerHeaderView(
                order_delivery_difference_query_lv.getRecyclerHeaderView()
            )
        } ?: apply {
            mOrderDeliveryDifferenceQueryAdapter?.setNewData(mutableListOf())
            showState(LoadWaitDominator.NO_DATA)
        }
    }

    override fun showCanOrdering(msgCode: Int, msgInfo: String?, isSubmitOrder: Boolean) {
    }

    override fun showOrderList(dingHuoOplQtyEntityList: DingHuoOplQtyEntity.ResultInfo?) {
    }

    override fun submitSuccess(msg: String?) {
    }

    override fun submitFailed(msg: String?) {
    }

    override fun showDismissAll() {
    }
}