package com.urovo.activity.goodsmanager

import android.view.View
import android.view.ViewGroup
import com.urovo.activity.goodsmanager.adapter.order.OrderGoodsQueryAdapter
import com.urovo.activity.goodsmanager.model.OrderCommonEntity
import com.urovo.wgmf.db.dao.entitys.DingHuoOplQtyEntity
import com.urovo.activity.goodsmanager.model.OrderDeliveryDifferenceQueryEntity
import com.urovo.activity.goodsmanager.model.OrderGoodsQueryEntity
import com.urovo.wgmf.tool.TimeHelper
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.view.LoadWaitDominator
import com.wgmf.trade.presenter.OrderGoodPre
import com.wgmf.trade.view.OrderGoodCon
import kotlinx.android.synthetic.main.activity_order_goods_query.*
import kotlinx.android.synthetic.main.include_check_goods_submit_product_list.*
import kotlinx.android.synthetic.main.popup_order_goods_query.*
import xp.timepicker.com.widget.bean.DateType
import java.util.*

/**
 * @author Sasuke on 2019/4/5.
 * 订货查询
 */
class OrderGoodsQueryActivity : BaseActivity(), OrderGoodCon.View {
    private var mOrderGoodPre: OrderGoodPre? = null

    private var mOrderGoodsQueryAdapter: OrderGoodsQueryAdapter? = null

    override fun addRootLayoutId(): Int = R.layout.activity_order_goods_query

    override fun rootViewGroup(): ViewGroup = order_goods_query_container_rl

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.order_goods_query)
    }

    override fun initViewNew() {
        initTitle()
        initPopView()

        mOrderGoodPre = OrderGoodPre(this)
        mOrderGoodsQueryAdapter =
            OrderGoodsQueryAdapter(
                mContext,
                listOf()
            )
        order_goods_query_lv.setAdapter(mOrderGoodsQueryAdapter)

        order_goods_query_title_tv.apply {
            text =
                StringUtils.formatString(
                    mContext.getString(R.string.query_stores),
                    SystemHelper.LoginOrganName,
                    order_goods_query_start_date_tv.text.toString(),
                    order_goods_query_end_date_tv.text.toString()
                )
            setOnClickListener { showPopWindow(true) }
        }
        order_goods_query_btn.setOnClickListener {
            queryOrderGoodsDetail()
        }
        order_goods_query_btn.performClick()
    }

    private fun queryOrderGoodsDetail() {
        mOrderGoodPre?.getOrderGoodsDetail(
            SystemHelper.LoginOrganId,
            order_goods_query_start_date_tv.text.toString(),
            order_goods_query_end_date_tv.text.toString()
        )
    }

    /**
     * 显隐弹窗
     */
    private fun showPopWindow(isVisible: Boolean) {
        order_goods_query_pop.run { visibility = if (isVisible) View.VISIBLE else View.GONE }
    }

    private fun initPopView() {
        order_goods_query_pop.apply {
            order_goods_query_out_view.setOnClickListener { showPopWindow(false) }
            order_goods_query_store_tv.text = SystemHelper.LoginOrganName
            order_goods_query_start_date_tv?.apply {
                text = DateUtil.getDate(Date())
                setOnClickListener {
                    TimeHelper.showDatePickDialog(
                        this@OrderGoodsQueryActivity,
                        DateType.TYPE_YMD
                    ).apply {
                        val date = tag
                        if (date != null && date is Date) {
                            setSelectDate(date)
                        }
                        setOnSureLisener {
                            if (it != null) {
                                tag = it
                                text = DateUtil.getDate(it)
                                refreshDataToView()
                            }
                        }
                    }.show()
                }
            }

            //结束日期
            order_goods_query_end_date_tv?.apply {
                text = DateUtil.getDate(Date())
                setOnClickListener {
                    TimeHelper.showDatePickDialog(
                        this@OrderGoodsQueryActivity,
                        DateType.TYPE_YMD
                    ).apply {
                        val date = tag
                        if (date != null && date is Date) {
                            setSelectDate(date)
                        }
                        setOnSureLisener {
                            if (it != null) {
                                tag = it
                                text = DateUtil.getDate(it)
                                refreshDataToView()
                            }
                        }
                    }.show()
                }
            }
        }
    }

    private fun refreshDataToView() {
        order_goods_query_title_tv.text = StringUtils.formatString(
            mContext.getString(R.string.query_stores),
            SystemHelper.LoginOrganName,
            order_goods_query_start_date_tv.text.toString(),
            order_goods_query_end_date_tv.text.toString()
        )
    }

    /**
     * 查询到的订货结果
     */
    override fun showOrderGoodsDetail(list: MutableList<OrderGoodsQueryEntity>?) {
        dismissAllState()
        showPopWindow(false)
        list?.let {
            val filterProductCode = order_goods_query_filter_product_et?.text.toString()
            //过滤
            it.forEach {
                if (filterProductCode.isNotBlank()) {
                    it.goodDetailList =
                        it.goodDetailList?.filter { it.goodCode == filterProductCode }
                }
            }
            val resultInfo =
                it.filter { it.goodDetailList.isNotNullOrSize() }
            if (resultInfo.isEmpty()) {
                mOrderGoodsQueryAdapter?.setNewData(listOf())
                showState(LoadWaitDominator.NO_DATA)
                return
            }

            mOrderGoodsQueryAdapter?.setNewData(resultInfo)
            order_goods_query_lv.showListHeaderView()
        } ?: apply {
            mOrderGoodsQueryAdapter?.setNewData(listOf())
            showState(LoadWaitDominator.NO_DATA)
        }
    }

    override fun showOrderCommon(list: MutableList<OrderCommonEntity>?) {
    }

    override fun showOrderGoodsDiff(list: MutableList<OrderDeliveryDifferenceQueryEntity>?) {
    }

    override fun showCanOrdering(msgCode: Int, msgInfo: String?, isSubmitOrder: Boolean) {
    }

    override fun showOrderList(dingHuoOplQtyEntityList: DingHuoOplQtyEntity.ResultInfo?) {
    }

    override fun submitSuccess(msg: String?) {
    }

    override fun submitFailed(msg: String?) {
    }
    override fun showDismissAll() {
    }
}