package com.urovo.activity.goodsmanager

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import com.urovo.activity.goodsmanager.service.OrderGoodsRepository
import com.urovo.activity.goodsmanager.viewmodel.OrderGoodsFactory
import com.urovo.activity.goodsmanager.viewmodel.OrderGoodsViewModel
import com.urovo.wugumofang.base.BaseActivity
import androidx.lifecycle.ViewModelProviders
import com.urovo.wugumofang.R
import java.util.*
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.notNullAndEmpty
import kotlinx.android.synthetic.main.ordergoods_list_act.*
import com.urovo.wugumofang.view.DividerItemDecoration
import com.wgmf.trade.ui.fra.index.OrderGoodsAdapter

/**
 * @author LW
 * 店订货/查询
 */
class OrderGoodsTabAct : BaseActivity() {

    private var mOrderTitleList: List<Int> =
        arrayListOf(
            R.string.order_goods_submit,
            R.string.order_goods_query,
            R.string.order_delivery_difference_query
        )

    private var mOrderGoodsAdapter: OrderGoodsAdapter? = null

    private val mOrderGoodsViewModel by lazy {
        ViewModelProviders.of(this, OrderGoodsFactory(OrderGoodsRepository(), application))
            .get(OrderGoodsViewModel::class.java)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bindViewModel()
        setContentView(R.layout.ordergoods_list_act)
        initTitle()
        initView()

        mOrderGoodsViewModel.run {
            //门店订货周期变更提醒
            queryOrderGoodsTip(mContext)
            //查询近一周内是否有发货差异
            queryOrderGoodsDiffByWeek()
        }
    }

    private fun initView() {
        titleTextView.text = mContext.getString(R.string.store_order)
        mOrderGoodsAdapter = OrderGoodsAdapter(mOrderTitleList)
        orderGoodsEntryRv.run {
            layoutManager = LinearLayoutManager(this@OrderGoodsTabAct)
            addItemDecoration(
                DividerItemDecoration(
                    this@OrderGoodsTabAct, DividerItemDecoration.HORIZONTAL_LIST,
                    1, Color.parseColor("#E3E7EE")
                )
            )
            adapter = mOrderGoodsAdapter
        }
        mOrderGoodsAdapter?.setOnItemClickListener { adapter, view, position ->
            val item = adapter.getItem(position)
            item?.let {
                when (item) {
                    mOrderTitleList[0] -> {
                        startActivity(
                            Intent(
                                this@OrderGoodsTabAct,
                                OrderGoodsSubmitActivity::class.java
                            )
                        )
                    }
                    mOrderTitleList[1] -> {
                        startActivity(
                            Intent(
                                this@OrderGoodsTabAct,
                                OrderGoodsQueryActivity::class.java
                            )
                        )
                    }
                    else -> {
                        startActivity(
                            Intent(
                                this@OrderGoodsTabAct,
                                OrderDeliveryDifferenceQueryActivity::class.java
                            )
                        )
                    }
                }
            }
        }
    }

    private fun bindViewModel() {
        mOrderGoodsViewModel.progressBarText.observe(this, Observer {
            if (it.isNullOrBlank()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this@OrderGoodsTabAct, "请稍等", it)
            }
        })

        mOrderGoodsViewModel.diffByWeekText.observe(this, Observer {
            if (it.notNullAndEmpty()) {
                mOrderGoodsAdapter?.setTipToPos(it, 2)
            }
        })
    }
}
