package com.urovo.activity.goodsmanager

import android.content.Intent
import android.graphics.drawable.BitmapDrawable
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.*
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.goodsmanager.adapter.transfer.TransferQueryAdapter
import com.urovo.activity.goodsmanager.model.StoreAllotEntity
import com.urovo.activity.goodsmanager.presenter.TransferContract
import com.urovo.activity.goodsmanager.presenter.TransferPresenter
import com.urovo.wgmf.tool.TimeHelper
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isEmptyParameter
import com.urovo.wugumofang.utils.notNullAndEmpty
import kotlinx.android.synthetic.main.activity_transfer_query.*
import kotlinx.android.synthetic.main.activity_voucher_card_query.*
import xp.timepicker.com.widget.bean.DateType
import java.util.*

/**
 * 调拨查询
 * @author LW
 */
class TransferQueryActivity : BaseActivity(), TransferContract.IView {

    private var mTransferPresenter: TransferPresenter? = null

    private var mTransferQueryAdapter: TransferQueryAdapter? = null

    override fun addRootLayoutId(): Int = R.layout.activity_transfer_query

    override fun rootViewGroup(): ViewGroup = root_container_fl

    private var mPopupWindow: PopupWindow? = null

    private var mStoreQuire: TextView? = null
    private var mAllotDateFrom: TextView? = null
    private var mAllotDateTo: TextView? = null
    private var mAllotTo: RadioButton? = null
    private var mAllotReceive: RadioButton? = null
    private var mOutView: View? = null

    private var mStoreId: String? = null
    private var mBeginDate: String? = null
    private var mEndDate: String? = null

    private var mStatus: Int = PRO_OUT

    private var mHasFirstLoad: Boolean = true

    companion object {
        const val PRO_OUT = 1 //出库

        const val PRO_IN = 2 //收货
    }

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.transfer_query_title)
    }

    override fun initViewNew() {
        initTitle()
        mStoreId = SystemHelper.LoginOrganId

        mTransferQueryAdapter = TransferQueryAdapter(mutableListOf())
        mTransferQueryAdapter?.setOnItemClickListener { adapter, view, position ->
            val item = adapter.getItem(position)
            if (item != null) {
                jumpTransferDetailActivity(item as StoreAllotEntity)
            }
        }

        transfer_query_list_rv.apply {
            layoutManager =
                LinearLayoutManager(mContext, RecyclerView.VERTICAL, false)
            adapter = mTransferQueryAdapter
        }

        transfer_query_header_ll.setOnClickListener {
            showPopWindow()
        }
    }

    private fun showPopWindow() {
        if (mPopupWindow == null) {
            val view = layoutInflater.inflate(R.layout.allotgoods_quire, null)
            initPopWindow(view)
            mPopupWindow = PopupWindow(
                view,
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.WRAP_CONTENT,
                true
            ).apply {
                isFocusable = false
                setBackgroundDrawable(BitmapDrawable())
                isOutsideTouchable = false
                mOutView?.setOnClickListener {
                    if (mHasFirstLoad) {
                        return@setOnClickListener
                    }
                    if (mPopupWindow != null && mPopupWindow?.isShowing == true) {
                        mPopupWindow?.dismiss()
                    }
                }
            }
        } else {
            mOutView?.setBackgroundColor(
                mContext.resources.getColor(R.color.result_view)
            )
        }
        titleTextView.post {
            mPopupWindow?.showAsDropDown(titleTextView)
        }
    }

    private fun initPopWindow(view: View) {
        mStoreQuire = view.findViewById<TextView>(R.id.ordergoods_quire).apply {
            text = "查询门店:" + SystemHelper.LoginOrganName
        }

        val endDate = DateUtil.getDate(Date())
        val beginDate = DateUtil.getDate(DateUtil.getFirstDayOfMonth())

        mAllotDateFrom = view.findViewById<TextView>(R.id.allot_date_from).apply {
            mBeginDate = beginDate
            text = mBeginDate
        }
        mAllotDateTo = view.findViewById<TextView>(R.id.allot_date_to).apply {
            mEndDate = endDate
            text = mEndDate
        }

        mAllotTo = view.findViewById<RadioButton>(R.id.allot_to).apply {
            isChecked = true
            mStatus = PRO_OUT
            setOnCheckedChangeListener { _, isChecked ->
                if (isChecked) {
                    mStatus = PRO_OUT
                }
            }
        }

        mAllotReceive = view.findViewById<RadioButton>(R.id.allot_receive).apply {
            setOnCheckedChangeListener { _, isChecked ->
                if (isChecked) {
                    mStatus = PRO_IN
                }
            }
        }
        mOutView = view.findViewById(R.id.out_view)
    }

    override fun initDataNew() {
        val startDate = intent.getStringExtra(AppConstants.ExtraKeys.TRANSFER_QUERY_START_DATE)
        if (startDate.notNullAndEmpty()) {
            this.mBeginDate = startDate
        }
        val endDate = intent.getStringExtra(AppConstants.ExtraKeys.TRANSFER_QUERY_END_DATE)
        if (endDate.notNullAndEmpty()) {
            this.mEndDate = endDate
        }
        val status = intent.getIntExtra(AppConstants.ExtraKeys.TRANSFER_QUERY_STATUS, -1)
        if (status >= 0) {
            this.mStatus = status
        }
        if (isEmptyParameter(mBeginDate, mEndDate) || mStatus < 0) {
            showPopWindow()
        } else {
            mTransferPresenter?.requestStoreAllotNew(mStoreId, mBeginDate, mEndDate, mStatus)
        }
    }

    fun allotQuire(view: View?) {
        mTransferPresenter?.requestStoreAllotNew(mStoreId, mBeginDate, mEndDate, mStatus)
    }

    override fun onResume() {
        super.onResume()
        if (!mHasFirstLoad) {
            mTransferPresenter?.requestStoreAllotNew(mStoreId, mBeginDate, mEndDate, mStatus)
        }
    }

    fun orderDatePickFrom(view: View?) {
        TimeHelper.showDatePickDialog(mContext, DateType.TYPE_YMD).apply {
            setSelectDate(DateUtil.parseDate(mBeginDate))
            setOnSureLisener {
                if (it != null) {
                    val date = DateUtil.format(it, DateType.TYPE_YMD.format)
                    mAllotDateFrom?.text = date
                    mBeginDate = date
                }
            }
        }.show()
    }

    fun orderDatePickTo(view: View?) {
        TimeHelper.showDatePickDialog(mContext, DateType.TYPE_YMD).apply {
            setSelectDate(DateUtil.parseDate(mEndDate))
            setOnSureLisener {
                if (it != null) {
                    val date = DateUtil.format(it, DateType.TYPE_YMD.format)
                    mAllotDateTo?.text = date
                    mEndDate = date
                }
            }
        }.show()
    }

    override fun storeAllotNew(data: List<StoreAllotEntity>?) {
        super.storeAllotNew(data)
        mHasFirstLoad = false
        if (mPopupWindow?.isShowing == true) {
            mPopupWindow?.dismiss()
        }
        transfer_query_store_tv.text = SystemHelper.LoginOrganName
        transfer_query_time_tv.text = mBeginDate + "至" + mEndDate

        mTransferQueryAdapter?.setNewData(data)
    }

    override fun addPresenter(): BasePresenter {
        mTransferPresenter = TransferPresenter(this)
        return mTransferPresenter!!
    }

    /**
     * 跳转到TransferDetailActivity
     */
    private fun jumpTransferDetailActivity(item: StoreAllotEntity) {
        startActivity(
            Intent(
                mContext,
                TransferDetailActivity::class.java
            ).apply {
                putExtra(AppConstants.ExtraKeys.STORE_ALLOT_ENTITY, item)
                putExtra(AppConstants.ExtraKeys.TRANSFER_QUERY_STATUS, mStatus)
            }
        )
    }
}