package com.urovo.activity.goodsmanager.adapter.order

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseExpandableListAdapter
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.goodsmanager.model.OrderDeliveryDifferenceQueryEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.view.DividerItemDecoration
import com.urovo.wugumofang.view.scrollablePanel.ScrollablePanelManager

/**
 * @author LW on 2023-11-30 0030.
 * 订货与差异适配器
 */
class OrderDeliveryDifferenceQueryAdapter(
    private val mContext: Context,
    private val mDataLists: MutableList<OrderDeliveryDifferenceQueryEntity>
) : BaseExpandableListAdapter() {

    private var mOrderDeliveryDifferenceTableAdapter: OrderDeliveryDifferenceTableAdapter? = null

    private var mRecyclerHeaderView: RecyclerView? = null

    fun setNewData(list: List<OrderDeliveryDifferenceQueryEntity>?) {
        list?.let {
            mDataLists.clear()
            mDataLists.addAll(it)
            notifyDataSetChanged()
        }
    }

    override fun getGroupCount(): Int = mDataLists.size

    override fun getChildrenCount(groupPosition: Int): Int {
        mDataLists.get(groupPosition).diffDetails?.let {
            return 1
        }
        return 0
    }

    override fun getGroup(groupPosition: Int): OrderDeliveryDifferenceQueryEntity? {
        return if (groupPosition >= 0 && groupPosition < mDataLists.size) {
            mDataLists.get(groupPosition)
        } else null
    }

    override fun getChild(
        groupPosition: Int,
        childPosition: Int
    ): List<OrderDeliveryDifferenceQueryEntity.DiffDetailsBean>? {
        return if (groupPosition >= 0 && groupPosition < mDataLists.size) {
            mDataLists.get(groupPosition).diffDetails
        } else null
    }

    override fun getGroupId(groupPosition: Int): Long = groupPosition.toLong()

    override fun getChildId(groupPosition: Int, childPosition: Int): Long = childPosition.toLong()

    override fun hasStableIds(): Boolean = false

    override fun getGroupView(
        groupPosition: Int,
        isExpanded: Boolean,
        convertView: View?,
        parent: ViewGroup?
    ): View {
        var groupView = convertView
        if (groupView == null) {
            groupView =
                LayoutInflater.from(mContext)
                    .inflate(R.layout.item_order_delivery_difference_query_list_group, null)
        }
        val orderItemContentCl: ConstraintLayout? =
            groupView?.findViewById(R.id.order_delivery_difference_item_content_cl)
        val orderArrowIv =
            groupView?.findViewById<ImageView>(R.id.order_delivery_difference_arrow_iv)

        getGroup(groupPosition)?.let {
            groupView?.findViewById<TextView>(R.id.order_delivery_difference_order_tv)?.apply {
                text = String.format(
                    mContext.getString(R.string.order_delivery_difference_delivery_order),
                    it.psbillCode
                )
            }

            groupView?.findViewById<TextView>(R.id.order_delivery_difference_date_tv)?.apply {
                text = String.format(
                    mContext.getString(R.string.order_delivery_difference_delivery_date),
                    it.psDate
                )
            }

            groupView?.findViewById<TextView>(R.id.order_delivery_difference_brand_tv)?.apply {
                text = String.format(
                    mContext.getString(R.string.order_delivery_difference_brand),
                    it.brandName
                )
            }

        }

        if (isExpanded) {
            orderArrowIv?.setImageResource(R.mipmap.ic_arrow_up)
            orderItemContentCl?.setBackgroundResource(R.drawable.shape_white_round_top_6)
        } else {
            orderArrowIv?.setImageResource(R.mipmap.ic_arrow_down)
            orderItemContentCl?.setBackgroundResource(R.drawable.shape_white_round_6)
        }
        return groupView!!
    }

    override fun getChildView(
        groupPosition: Int,
        childPosition: Int,
        isLastChild: Boolean,
        convertView: View?,
        parent: ViewGroup?
    ): View {
        var childView = convertView
        if (childView == null) {
            childView =
                LayoutInflater.from(mContext)
                    .inflate(R.layout.item_order_delivery_difference_query_list_child, null)
            val recyclerHeaderListView = mRecyclerHeaderView;
            childView?.tag = recyclerHeaderListView
        }

        val orderDeliveryDifferenceFirstItem =
            childView?.findViewById<FrameLayout>(R.id.order_delivery_difference_first_item)
        val orderDeliveryDifferenceRecyclerHeaderList =
            childView?.findViewById<RecyclerView>(R.id.order_delivery_difference_recycler_header_list)
        val otherRecyclerView = childView?.tag
        if (otherRecyclerView != null && otherRecyclerView is RecyclerView) {
            val orderDeliveryDifferenceRecyclerContentList =
                childView?.findViewById<RecyclerView>(R.id.order_delivery_difference_recycler_content_list)
                    ?.apply {
                        setHasFixedSize(true)
                        isNestedScrollingEnabled = false
                    }

            val list: List<OrderDeliveryDifferenceQueryEntity.DiffDetailsBean>? =
                getChild(groupPosition, childPosition)
            if (mOrderDeliveryDifferenceTableAdapter == null) {
                mOrderDeliveryDifferenceTableAdapter =
                    OrderDeliveryDifferenceTableAdapter(mContext)

                orderDeliveryDifferenceRecyclerContentList?.addItemDecoration(
                    DividerItemDecoration(
                        mContext,
                        DividerItemDecoration.HORIZONTAL_LIST,
                        1,
                        ContextCompat.getColor(mContext, R.color.color_E8E8E8)
                    )
                )
            }
            mOrderDeliveryDifferenceTableAdapter?.setTableTitleList(
                mContext.resources.getStringArray(R.array.order_delivery_difference_list_title)
                    .asList()
            )
            mOrderDeliveryDifferenceTableAdapter?.setDiffDetailsList(list as MutableList<OrderDeliveryDifferenceQueryEntity.DiffDetailsBean>)
            val scrollablePanelManager = ScrollablePanelManager()
            scrollablePanelManager.setPanelView(
                mContext,
                orderDeliveryDifferenceRecyclerContentList,
                orderDeliveryDifferenceRecyclerHeaderList,
                orderDeliveryDifferenceFirstItem,
                false
            )
            scrollablePanelManager.setOtherRecyclerView(
                otherRecyclerView
            )

            scrollablePanelManager.setPanelAdapter(
                mOrderDeliveryDifferenceTableAdapter,
                null
            )
        }
        return childView!!
    }

    override fun isChildSelectable(groupPosition: Int, childPosition: Int): Boolean = true

    fun setRecyclerHeaderView(view: RecyclerView?) {
        this.mRecyclerHeaderView = view;
    }

}