package com.urovo.activity.goodsmanager.adapter.order;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.constraintlayout.widget.ConstraintLayout;

import com.urovo.activity.goodsmanager.model.OrderGoodsQueryEntity;
import com.urovo.wugumofang.R;

import java.util.ArrayList;
import java.util.List;

/**
 * 订货查询适配器
 */
public class OrderGoodsQueryAdapter extends BaseExpandableListAdapter {

    private final Context context;
    private final ArrayList<OrderGoodsQueryEntity> dataList = new ArrayList<>();

    public OrderGoodsQueryAdapter(Context context, List<OrderGoodsQueryEntity> list) {
        this.context = context;
        if (list != null && list.size() > 0) {
            dataList.clear();
            dataList.addAll(list);
        }
    }

    public void setNewData(List<OrderGoodsQueryEntity> list) {
        dataList.clear();
        dataList.addAll(list);
        notifyDataSetChanged();
    }

    @Override
    public int getGroupCount() {
        return dataList.size();
    }

    @Override
    public int getChildrenCount(int groupPosition) {
        return dataList.get(groupPosition).getGoodDetailList() == null ? 0 : dataList.get(groupPosition).getGoodDetailList().size();
    }


    @Override
    public OrderGoodsQueryEntity getGroup(int groupPosition) {
        if (groupPosition >= 0 && groupPosition < dataList.size()) {
            return dataList.get(groupPosition);
        }
        return null;
    }

    @Override
    public OrderGoodsQueryEntity.OrderGoodsDetailBean getChild(int groupPosition, int childPosition) {
        List<OrderGoodsQueryEntity.OrderGoodsDetailBean> list = dataList.get(groupPosition).getGoodDetailList();
        if (list != null &&
                childPosition >= 0 && childPosition < list.size()) {
            return list.get(childPosition);
        }
        return null;
    }

    @Override
    public long getGroupId(int groupPosition) {
        return groupPosition;
    }

    @Override
    public long getChildId(int groupPosition, int childPosition) {
        return childPosition;
    }

    @Override
    public boolean hasStableIds() {
        return false;
    }

    @Override
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView, ViewGroup parent) {
        if (convertView == null) {
            convertView = LayoutInflater.from(context).inflate(R.layout.item_order_goods_query_list_group, null);
        }

        OrderGoodsQueryEntity dinghuoEntity = getGroup(groupPosition);
        ConstraintLayout clItemContent = convertView.findViewById(R.id.clItemContent);
        TextView tvOrderId = convertView.findViewById(R.id.tvOrderId);
        TextView tvDate = convertView.findViewById(R.id.tvDate);
        TextView tvBrand = convertView.findViewById(R.id.tvBrand);
        TextView tvTotalCount = convertView.findViewById(R.id.tvTotalCount);
        TextView tvMakingPeople = convertView.findViewById(R.id.tvMakingPeople);
        TextView tvTotalAmount = convertView.findViewById(R.id.tvTotalAmount);
        ImageView ivArrow = convertView.findViewById(R.id.ivArrow);

        if (dinghuoEntity != null) {
            tvOrderId.setText(String.format(context.getString(R.string.input_order_id),
                    dinghuoEntity.getOrderGoodBill()));
            tvDate.setText(String.format(context.getString(R.string.input_time),
                    dinghuoEntity.getOrderDate()));
            tvMakingPeople.setText(String.format(context.getString(R.string.input_making_people),
                    dinghuoEntity.getOrdererName()));
            tvBrand.setText(String.format(context.getString(R.string.input_brand),
                    dinghuoEntity.getBrandName()));
            tvTotalCount.setText(String.format(context.getString(R.string.input_total_count),
                    String.valueOf(dinghuoEntity.getTotalQty())));
            tvTotalAmount.setText(String.format(context.getString(R.string.input_total_amount),
                    String.valueOf(dinghuoEntity.getTotalAmount())));
        }
        if (isExpanded) {
            ivArrow.setImageResource(R.mipmap.ic_arrow_up);
            clItemContent.setBackgroundResource(R.drawable.shape_white_round_top_6);
        } else {
            ivArrow.setImageResource(R.mipmap.ic_arrow_down);
            clItemContent.setBackgroundResource(R.drawable.shape_white_round_6);
        }
        return convertView;
    }

    @Override
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild, View convertView, ViewGroup parent) {
        ViewHolder viewHolder;
        if (convertView == null) {
            convertView = LayoutInflater.from(context).inflate(R.layout.item_order_goods_query_list_child, null);
            viewHolder = new ViewHolder();
            viewHolder.llItemTitle = convertView.findViewById(R.id.llItemTitle);
            viewHolder.llItemContent = convertView.findViewById(R.id.llItemContent);
            viewHolder.tvProductName = convertView.findViewById(R.id.tvProductName);
            viewHolder.tvProductCode = convertView.findViewById(R.id.tvProductCode);
            viewHolder.tvPrice = convertView.findViewById(R.id.tvPrice);
            viewHolder.tvCount = convertView.findViewById(R.id.tvCount);
            viewHolder.tvTotalPrice = convertView.findViewById(R.id.tvTotalPrice);
            viewHolder.vLine = convertView.findViewById(R.id.vLine);
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        OrderGoodsQueryEntity.OrderGoodsDetailBean ent = getChild(groupPosition, childPosition);
        viewHolder.tvProductName.setText(ent.getGoodName());
        viewHolder.tvProductCode.setText(String.format(context.getString(R.string.parentheses_number), ent.getGoodCode()));
        viewHolder.tvPrice.setText(String.valueOf(ent.getGoodPrice()));
        viewHolder.tvCount.setText(String.valueOf(ent.getGoodQty()));
        viewHolder.tvTotalPrice.setText(String.valueOf(ent.getGoodAmount()));
        if (childPosition == 0) {
            viewHolder.llItemTitle.setVisibility(View.VISIBLE);
        } else {
            viewHolder.llItemTitle.setVisibility(View.GONE);
        }

        if (isLastChild) {
            viewHolder.llItemContent.setBackgroundResource(R.drawable.shape_f8f6f3_round_bottom_6);
            viewHolder.vLine.setVisibility(View.VISIBLE);
        } else {
            viewHolder.llItemContent.setBackgroundResource(R.drawable.shape_f8f6f3_solid);
            viewHolder.vLine.setVisibility(View.GONE);
        }
        return convertView;
    }

    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return true;
    }

    class ViewHolder {
        LinearLayout llItemTitle;
        LinearLayout llItemContent;
        TextView tvProductName;
        TextView tvProductCode;
        TextView tvPrice;
        TextView tvCount;
        TextView tvTotalPrice;
        View vLine;

    }
}
