package com.urovo.activity.goodsmanager.adapter.transfer

import android.content.Context
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.goodsmanager.model.StoreAllotEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.data.Arith
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.ui.EditTextDecimal
import com.urovo.wugumofang.utils.StringUtil
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.view.scrollablePanel.PanelAdapter
import com.urovo.wugumofang.view.scrollablePanel.ScrollablePanelManager

/**
 * @author LW
 * 调拨详情
 */
class TransferDetailTableAdapter(val mContext: Context) : PanelAdapter {

    private var mProductList: MutableList<StoreAllotEntity.ProductBean> = mutableListOf()

    /**
     * 表单每列标题
     */
    private var mTableTitleList: MutableList<String> = mutableListOf()

    private var mIsEditStatus: Boolean = false

    private var mScrollablePanelManager: ScrollablePanelManager? = null

    companion object {
        /**
         * 列标题类型
         */
        const val COLUMNS_TITLE_ITEM_TYPE: Int = 1

        /**
         * 行标题类型
         */
        const val ROW_TITLE_ITEM_TYPE: Int = 2

        /**
         * 内容标题类型
         */
        const val CONTENT_ITEM_TYPE: Int = 0
    }

    override fun getRowCount(): Int = mProductList.size

    override fun getColumnCount(): Int = mTableTitleList.size

    override fun getItemViewType(row: Int, column: Int): Int {
        if (row == 0) {
            return ROW_TITLE_ITEM_TYPE
        }
        if (column == 0) {
            return COLUMNS_TITLE_ITEM_TYPE
        }
        return CONTENT_ITEM_TYPE
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder?, row: Int, column: Int) {
        when (getItemViewType(row, column)) {
            ROW_TITLE_ITEM_TYPE -> {
                bindRowTitleItemData(holder as RowTitleViewHolder, column)
            }
            COLUMNS_TITLE_ITEM_TYPE -> {
                bindColumnTitleItemData(holder as ColumnsTitleViewHolder, row)
            }
            else -> {
                bindContentItemData(holder as ContentViewHolder, row, column)
            }
        }
    }

    private fun bindContentItemData(
        contentViewHolder: ContentViewHolder,
        row: Int,
        column: Int
    ) {
        if (mProductList.isNotNullOrSize()) {
            val product = mProductList[row - 1]
            product.apply {
                contentViewHolder.valueTv.text = when (column) {
                    1 -> proDate
                    2 -> "$proPrice/$proUnit"
                    3 -> (editProQty ?: proQty).toString()
                    4 -> StringUtil.formatDouble(
                        Arith.mul(
                            editProQty ?: proQty,
                            proPrice
                        ), "0.00"
                    )
                    else -> "--"
                }
                updateItemBgColor(row, contentViewHolder.itemRootLl)
            }
            if (column == 3 && mIsEditStatus) {  //编辑状态
                contentViewHolder.valueTv.visibility = View.GONE
                contentViewHolder.editValueEt.visibility = View.VISIBLE
                contentViewHolder.editValueEt.setSelectAllOnFocus(true)
                contentViewHolder.editValueEt.tag = row - 1
                contentViewHolder.editValueEt.setText(
                    (mProductList[row - 1].editProQty ?: mProductList[row - 1].proQty).toString()
                )
                contentViewHolder.editValueEt.setDecimal(if (product.proIsDecimal == "否") 0 else 2)
                contentViewHolder.editValueEt.addTextChangedListener(object : TextWatcher {
                    override fun beforeTextChanged(
                        s: CharSequence,
                        start: Int,
                        count: Int,
                        after: Int
                    ) {
                    }

                    override fun onTextChanged(
                        s: CharSequence,
                        start: Int,
                        before: Int,
                        count: Int
                    ) {
                    }

                    override fun afterTextChanged(s: Editable) {
                        val pos = contentViewHolder.editValueEt.tag as Int
                        if (pos >= 0 && pos < mProductList.size) {
                            mProductList[pos].let {
                                val input = s.toString()
                                val d: Double? = try {
                                    input.toDouble()
                                } catch (e: NumberFormatException) {
                                    null
                                }
                                val contentRecyclerItemView =
                                    mScrollablePanelManager?.getContentRecyclerItemView(pos, 3)

                                if (d != product.editProQty) {
                                    it.editProQty = d
                                    contentRecyclerItemView?.findViewById<TextView>(R.id.transfer_query_table_value_tv)?.text =
                                        StringUtil.formatDouble(
                                            Arith.mul(
                                                it.editProQty ?: it.proQty,
                                                it.proPrice
                                            ), "0.00"
                                        )

                                }
                            }
                        }
                    }
                })
            } else {
                contentViewHolder.valueTv.visibility = View.VISIBLE
                contentViewHolder.editValueEt.visibility = View.GONE
            }
        }
    }

    private fun bindColumnTitleItemData(
        titleViewHolder: ColumnsTitleViewHolder,
        row: Int
    ) {
        if (mProductList.isNotNullOrSize()) {
            val product = mProductList[row - 1]
            titleViewHolder.columnsTitleTv.text = product.proName
            titleViewHolder.columnsInfoTv.text = StringUtils
                .formatString(
                    mContext.getString(R.string.transfer_query_product_code),
                    product.proCode
                )
            updateItemBgColor(row, titleViewHolder.itemRootLl)
        }
    }

    private fun updateItemBgColor(position: Int, item: View) {
        if (position % 2 == 0) {
            item.setBackgroundColor(
                ContextCompat.getColor(
                    mContext,
                    R.color.color_FFFFFF
                )
            )
        } else {
            item.setBackgroundColor(
                ContextCompat.getColor(
                    mContext,
                    R.color.color_F4F4F4
                )
            )
        }
    }

    private fun bindRowTitleItemData(
        titleViewHolder: RowTitleViewHolder,
        column: Int
    ) {
        if (mTableTitleList.isNotNullOrSize()) {
            titleViewHolder.rowTitleTv.text = mTableTitleList[column]
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup?, viewType: Int): RecyclerView.ViewHolder {
        when (viewType) {
            ROW_TITLE_ITEM_TYPE -> {
                return RowTitleViewHolder(
                    LayoutInflater.from(
                        parent?.context
                    ).inflate(R.layout.item_transfer_query_table_row_title, parent, false)
                )
            }
            COLUMNS_TITLE_ITEM_TYPE -> {
                return ColumnsTitleViewHolder(
                    LayoutInflater.from(
                        parent?.context
                    ).inflate(R.layout.item_transfer_query_table_columns_title, parent, false)
                )
            }
            else -> {
                return ContentViewHolder(
                    LayoutInflater.from(
                        parent?.context
                    ).inflate(R.layout.item_transfer_query_table_item, parent, false)
                )
            }
        }
    }

    private class RowTitleViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val itemRootRl: RelativeLayout =
            view.findViewById(R.id.transfer_query_table_item_root_rl)
        val rowTitleTv: TextView = view.findViewById(R.id.transfer_query_table_row_title_tv)
    }

    private class ColumnsTitleViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val itemRootLl: LinearLayout =
            view.findViewById(R.id.transfer_query_table_item_root_ll)
        val columnsTitleTv: TextView = view.findViewById(R.id.transfer_query_table_columns_title_tv)
        val columnsInfoTv: TextView = view.findViewById(R.id.transfer_query_table_columns_info_tv)
    }

    private class ContentViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val itemRootLl: LinearLayout =
            view.findViewById(R.id.transfer_query_table_item_root_ll)
        val valueTv: TextView = view.findViewById(R.id.transfer_query_table_value_tv)
        val editValueEt: EditTextDecimal =
            view.findViewById(R.id.transfer_query_table_edit_value_et)
    }

    fun setTableTitleList(tableTitleList: MutableList<String>) {
        this.mTableTitleList = tableTitleList
    }

    fun setProductList(productList: MutableList<StoreAllotEntity.ProductBean>) {
        this.mProductList = productList
    }

    fun getProductList(): MutableList<StoreAllotEntity.ProductBean> {
        return mProductList
    }

    fun setEditStatus(status: Boolean) {
        this.mIsEditStatus = status
    }

    fun setScrollablePanelManager(scrollablePanelManager: ScrollablePanelManager?) {
        this.mScrollablePanelManager = scrollablePanelManager
    }
}