package com.urovo.activity.goodsmanager.adapter.transfer

import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.urovo.activity.goodsmanager.model.StoreAllotEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.utils.convertDefaultValue
import com.urovo.wugumofang.utils.isNotNullOrSize

/**
 * 调拨适配器
 * @author LW
 */
class TransferQueryAdapter(val items: MutableList<StoreAllotEntity>?) :
    BaseQuickAdapter<StoreAllotEntity,
            BaseViewHolder>(R.layout.item_transfer_query, items) {
    override fun convert(helper: BaseViewHolder?, item: StoreAllotEntity?) {
        helper?.apply {
            item?.let {
                if (it.isSh == ConstantParams.CONSTANT_STARTS_1||it.isSh == ConstantParams.CONSTANT_STARTS_2) {  //是否收货
                    getView<TextView>(R.id.transfer_query_item_title_tv).apply {
                        setBackgroundResource(R.drawable.shape_e8e8e8_round_11)
                        setTextColor(ContextCompat.getColor(mContext, R.color.common_txt_color))
                    }

                    if(it.isSh ==ConstantParams.CONSTANT_STARTS_1) {
                        getView<TextView>(R.id.transfer_query_item_order_status_tv).apply {
                            text = mContext.getString(R.string.transfer_query_received)
                            setTextColor(ContextCompat.getColor(mContext, R.color.common_txt_color))
                        }
                    }else{
                        getView<TextView>(R.id.transfer_query_item_order_status_tv).apply {
                            text = mContext.getString(R.string.transfer_query_refusal)
                            setTextColor(ContextCompat.getColor(mContext, R.color.common_txt_color))
                        }
                    }
                    getView<TextView>(R.id.transfer_query_item_out_store_tv).apply {
                        setTextColor(ContextCompat.getColor(mContext, R.color.color_common_gray_one))
                    }

                    getView<TextView>(R.id.transfer_query_item_in_store_tv).apply {
                        setTextColor(ContextCompat.getColor(mContext, R.color.color_common_gray_one))
                    }

                    getView<ImageView>(R.id.transfer_query_item_store_iv).setBackgroundResource(R.mipmap.icon_transfer_store_arrow_default)
                } else {
                    getView<TextView>(R.id.transfer_query_item_title_tv).apply {
                        setBackgroundResource(R.drawable.shape_fff2e2_round_11)
                        setTextColor(
                            ContextCompat.getColor(
                                mContext,
                                R.color.color_common_theme_one
                            )
                        )
                    }
                    getView<TextView>(R.id.transfer_query_item_order_status_tv).apply {
                        text = mContext.getString(R.string.transfer_query_await_receipt)
                        setTextColor(
                            ContextCompat.getColor(
                                mContext,
                                R.color.color_common_theme_one
                            )
                        )
                    }

                    getView<TextView>(R.id.transfer_query_item_out_store_tv).apply {
                        setTextColor(ContextCompat.getColor(mContext, R.color.common_two_txt_color))
                    }
                    getView<TextView>(R.id.transfer_query_item_in_store_tv).apply {
                        setTextColor(ContextCompat.getColor(mContext, R.color.common_two_txt_color))
                    }
                    getView<ImageView>(R.id.transfer_query_item_store_iv).setBackgroundResource(R.mipmap.icon_transfer_store_arrow)
                }
                setText(R.id.transfer_query_item_title_tv, it.allotDate)

                setText(
                    R.id.transfer_query_item_order_number_tv,
                    StringUtils.formatString(
                        mContext.getString(R.string.transfer_query_order_number),
                        it.allotmentId
                    )
                )

                setText(R.id.transfer_query_distance_tv, StringUtils.formatString(
                    mContext.getString(R.string.transfer_query_distance) ,it.distance.toString()
                ))
                setText(R.id.transfer_query_item_out_store_tv, it.allotOutStore)
                setText(R.id.transfer_query_item_in_store_tv, it.allotInStore)

                getView<LinearLayout>(R.id.transfer_query_item_content_ll).apply {
                    if (childCount > 0) {
                        removeAllViews()
                    }
                    if (it.products.isNotNullOrSize()) {
                        if (it.isSh == ConstantParams.CONSTANT_STARTS_1||it.isSh == ConstantParams.CONSTANT_STARTS_2) {
                            setBackgroundResource(R.drawable.shape_f4f4f4_round_5)
                        } else {
                            setBackgroundResource(R.drawable.shape_f8f6f3_round_5)
                        }
                        visibility = View.VISIBLE
                        it.products?.forEach { product ->
                            val view = LayoutInflater.from(mContext)
                                .inflate(R.layout.item_transfer_query_content_text, null)
                            view.findViewById<TextView>(R.id.transfer_query_item_product_name_tv)
                                .apply {
                                    if (it.isSh == ConstantParams.CONSTANT_STARTS_1||it.isSh == ConstantParams.CONSTANT_STARTS_2) {
                                        setTextColor(
                                            ContextCompat.getColor(
                                                mContext,
                                                R.color.color_common_gray_one
                                            )
                                        )
                                    } else {
                                        setTextColor(
                                            ContextCompat.getColor(
                                                mContext,
                                                R.color.color_AC6710
                                            )
                                        )
                                    }
                                    text = StringUtils.formatString(
                                        mContext.getString(R.string.transfer_query_product_name),
                                        product.proName,
                                        product.proCode
                                    )
                                }
                            view.findViewById<TextView>(R.id.transfer_query_item_product_number_tv)
                                .apply {
                                    if (it.isSh == ConstantParams.CONSTANT_STARTS_1||it.isSh == ConstantParams.CONSTANT_STARTS_2) {
                                        setTextColor(
                                            ContextCompat.getColor(
                                                mContext,
                                                R.color.color_common_gray_one
                                            )
                                        )
                                    } else {
                                        setTextColor(
                                            ContextCompat.getColor(
                                                mContext,
                                                R.color.color_AC6710
                                            )
                                        )
                                    }
                                    text =
                                        StringUtils.formatString(
                                            mContext.getString(R.string.transfer_query_product_number),
                                            product.proPrice.toString(),
                                            product.proQty.toString()
                                        )
                                }
                            addView(
                                view,
                                LinearLayout.LayoutParams(
                                    LinearLayout.LayoutParams.MATCH_PARENT,
                                    LinearLayout.LayoutParams.WRAP_CONTENT
                                )
                            )
                        }
                    } else {
                        visibility = View.GONE
                    }
                }
                setText(
                    R.id.transfer_query_item_prepared_by_tv,
                    StringUtils.formatString(
                        mContext.getString(R.string.transfer_query_prepared_by),
                        it.allotUserName.convertDefaultValue()
                    )
                )

                setText(
                    R.id.transfer_query_item_total_price_tv,
                    StringUtils.formatString(
                        mContext.getString(R.string.transfer_query_total_price_number),
                        it.amount.toString().convertDefaultValue()
                    )
                )
                if (it.isSh == ConstantParams.CONSTANT_STARTS_1||it.isSh == ConstantParams.CONSTANT_STARTS_2) {
                    setTextColor(
                        R.id.transfer_query_item_total_price_tv,
                        ContextCompat.getColor(mContext, R.color.color_common_gray_one)
                    )
                } else {
                    setTextColor(
                        R.id.transfer_query_item_total_price_tv,
                        ContextCompat.getColor(mContext, R.color.color_common_theme_one)
                    )
                }
            }
        }
    }
}