package com.urovo.activity.goodsmanager.presenter

import com.urovo.activity.goodsmanager.model.AllotmentNotificationEntity
import com.urovo.activity.goodsmanager.model.StoreAllotEntity
import com.urovo.activity.goodsmanager.model.StoreAllotUpdateEntity
import com.urovo.activity.goodsmanager.service.TransferRepository
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.utils.isEmptyParameter
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Response

/**
 * 顾客标签 Presenter
 * @author LW
 */
class TransferPresenter(
    private var view: TransferContract.IView?,
    private var depository: TransferRepository = TransferRepository()
) : AbsPresenter(), TransferContract.IPresenter {


    override fun requestStoreAllotNew(
        storeNo: String?,
        beginDate: String?,
        endDate: String?,
        status: Int
    ) {
        if (isEmptyParameter(storeNo, beginDate, endDate)) {
            //view?.showError(R.string.common_load_error)
            return
        }

        view?.showTipDialog(R.string.transfer_query_data_tip)
        depository.requestStoreAllotNew(storeNo, beginDate, endDate, status)
            .enqueue(object : RetrofitCallback<RequestResult<List<StoreAllotEntity>>>() {
                override fun onResponse(
                    call: Call<RequestResult<List<StoreAllotEntity>>>,
                    res: Response<RequestResult<List<StoreAllotEntity>>>
                ) {
                    view?.dismissDialog()
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            if (resultInfo.isNotNullOrSize()) {
                                view?.dismissAllState()
                                view?.storeAllotNew(resultInfo)
                            } else {
                                view?.showToast(R.string.empty_text)
                                view?.showState(LoadWaitDominator.NO_DATA)
                            }
                        } else {
                            view?.showError(msgInfo)
                            view?.showToast(msgInfo)
                        }
                    } ?: run {
                        view?.showError(R.string.common_http_error)
                        view?.showToast(R.string.common_http_error)
                    }
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.showToast(exception.message)
                    view?.showError(exception.message)
                }
            })
    }


    override fun requestNotificationStoreAllot(checkUserNo: String?, storeNo: String?) {
        if (isEmptyParameter(checkUserNo, storeNo)) {
            view?.showError(R.string.common_load_error)
            return
        }

        view?.showIsLoading()
        depository.requestNotificationStoreAllot(checkUserNo, storeNo)
            .enqueue(object : RetrofitCallback<RequestResult<AllotmentNotificationEntity>>() {
                override fun onResponse(
                    call: Call<RequestResult<AllotmentNotificationEntity>>,
                    res: Response<RequestResult<AllotmentNotificationEntity>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            if (resultInfo != null) {
                                view?.dismissAllState()
                                view?.notificationStoreAllot(resultInfo)
                            } else {
                                view?.showState(LoadWaitDominator.NO_DATA)
                            }
                        } else {
                            view?.showError(msgInfo)
                        }
                    } ?: view?.showState(LoadWaitDominator.NET_ERROR)

                    view?.dismissLoading()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissLoading()
                    view?.showError(exception.message)
                }
            })
    }

    override fun requestUpdateStoreAllot(dbDetails: List<StoreAllotUpdateEntity>) {
        view?.showTipDialog(R.string.transfer_query_update_transfer_reward_tip)
        depository.requestUpdateStoreAllot(dbDetails)
            .enqueue(object : RetrofitCallback<RequestResult<Any>>() {
                override fun onResponse(
                    call: Call<RequestResult<Any>>,
                    res: Response<RequestResult<Any>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.updateStoreAllotStatus(true)
                        } else {
                            view?.updateStoreAllotStatus(false)
                        }
                    }
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.updateStoreAllotStatus(false)
                }
            })
    }

    override fun requestConfirmStoreAllot(
        dbBillCode: String?,
        storeNo: String?,
        userNo: String?,
        reason: String?,
        type: Int
    ) {
        if (isEmptyParameter(dbBillCode, storeNo, userNo)) {
            view?.showError(R.string.common_load_error)
            return
        }

        if (type == ConstantParams.CONSTANT_STARTS_2 && reason.isNullOrEmpty()) {  //type类型(1收货，2拒收，3取回，4作废)
            view?.showError(R.string.transfer_query_reason_entry)
            return
        }

        view?.showTipDialog(R.string.transfer_query_store_confirm_reward_tip)
        depository.requestConfirmStoreAllot(dbBillCode, storeNo, userNo, reason, type)
            .enqueue(object : RetrofitCallback<RequestResult<Any>>() {
                override fun onResponse(
                    call: Call<RequestResult<Any>>,
                    res: Response<RequestResult<Any>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.confirmStoreAllotStatus(true)
                        } else {
                            view?.confirmStoreAllotStatus(false)
                        }
                        view?.showToast(msgInfo)
                    }
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.confirmStoreAllotStatus(false)
                }
            })
    }

    override fun requestCheckStoreAllot(dbBillCode: String?, storeNo: String?) {
        if (isEmptyParameter(dbBillCode, storeNo)) {
            view?.showError(R.string.common_load_error)
            return
        }
        view?.showTipDialog(R.string.transfer_query_store_confirm_ship_tip)
        depository.requestCheckStoreAllot(dbBillCode, storeNo)
            .enqueue(object : RetrofitCallback<RequestResult<Any>>() {
                override fun onResponse(
                    call: Call<RequestResult<Any>>,
                    res: Response<RequestResult<Any>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.checkStoreAllotStatus(true)
                        } else {
                            view?.checkStoreAllotStatus(false)
                        }
                        view?.showToast(msgInfo)
                    }
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.checkStoreAllotStatus(false)
                }
            })
    }
}