package com.urovo.activity.goodsmanager.service

import android.app.AlertDialog
import android.content.Context
import com.urovo.wgmf.entity.response.BaoSunQueryResponse
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.SystemHelper
import com.zsy.libcore.model.bean.RequestResult
import okhttp3.Request
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException
import kotlin.coroutines.suspendCoroutine

class OrderGoodsRepository {

    /**
     * 门店订货周期变更提醒
     */
    suspend fun queryOrderGoodsTip(context: Context) =
        suspendCoroutine<List<BaoSunQueryResponse.Bean>> { conti ->
            val param = HashMap<String, String>()
            param["shopId"] = SystemHelper.LoginOrganId

            OkHttpDominator.getAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_ORDER_GOODS_TIP,
                param,
                object : OkHttpDominator.ResultCallback<BaoSunQueryResponse?>() {
                    override fun onError(request: Request, e: Exception) {
                        conti.resumeWithException(Exception("网络异常"))
                    }

                    override fun onResponse(res: BaoSunQueryResponse?) {
                        if (res == null) {
                            conti.resumeWithException(Exception("暂无数据"))
                            return
                        }
                        if (res.msgCode != 0) {
                            AlertDialog.Builder(context)
                                .setTitle("订货周期调整提醒")
                                .setMessage(res.msgInfo)
                                .setNegativeButton("知道了") { dialog, which -> dialog.dismiss() }
                                .show()
                        }
                        conti.resume(res.list)
                    }
                })
        }


    /**
     * 查询近一周内是否有发货差异
     */
    suspend fun queryOrderGoodsDiffByWeek() =
        suspendCoroutine<String> { conti ->
            val param = HashMap<String, String>()
            param["storeNo"] = SystemHelper.LoginOrganId

            OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_ORDER_GOODS_DIFF_BY_WEEK,
                param,
                object : OkHttpDominator.ResultCallback<RequestResult<String>?>() {
                    override fun onError(request: Request, e: Exception) {
                        conti.resumeWithException(Exception("网络异常"))
                    }

                    override fun onResponse(res: RequestResult<String>?) {
                        if (res?.msgCode == 0) {
                            conti.resume(res.resultInfo)
                            return
                        }
                        conti.resumeWithException(Exception(res?.msgInfo))
                    }
                })
        }
}