package com.urovo.activity.goodsmanager.stocksearch;

import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.HorizontalScrollView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.urovo.activity.addsellactivity.ui.AutoRadioGroup;
import com.urovo.activity.goodsmanager.presenter.StockGoodsPresenter;
import com.urovo.wgmf.CommonController;
import com.urovo.wgmf.adapter.StockGoodsAdapter;
import com.urovo.wgmf.db.dao.entitys.StockGoodsDetailEntity;
import com.urovo.wgmf.net.CommonNet;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.net.NetHelper;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.DensityUtil;
import com.urovo.wugumofang.ui.ViewUtil;
import com.urovo.wugumofang.utils.CommonTools;
import com.urovo.wugumofang.utils.MyToast;
import com.wgmf.trade.model.bean.CommonEntity;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 库存查询  结果列表
 *
 * @author Administrator
 */
public class StockGoodsActivity extends BaseActivity implements CommonController.StockSearchView<StockGoodsDetailEntity> {
    private TextView storequire;
    private TextView texttitle;
    private ListView listview;
    private StockGoodsAdapter adapter;
    StockGoodsPresenter mPresenter;
    private HorizontalScrollView hScrollView;
    Map<String, String> mParams = new HashMap<>();
//    StockGoodsDetailEntity mStockList;

    RelativeLayout titleLayout;
    RelativeLayout tapLayout;
    AutoRadioGroup radioGroup;
    LinearLayout tapRadio;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.stocktgoods_quire);
        initTitle();
        init();
    }

    private void init() {
        storequire = (TextView) findViewById(R.id.ordergoods_quire);
        hScrollView = findViewById(R.id.hScrollView);
        if (DensityUtil.getScreenW(this) > DensityUtil.dip2px(this, 500)) {
            findViewById(R.id.widthLayout).setLayoutParams(new LinearLayout.LayoutParams(DensityUtil.getScreenW(this), LinearLayout.LayoutParams.WRAP_CONTENT));
        }
        hScrollView.setSmoothScrollingEnabled(true);
        storequire.setText(MessageFormat.format("查询门店:{0}", SystemHelper.LoginOrganName));
        texttitle = (TextView) findViewById(R.id.text_title);
        titleLayout = (RelativeLayout) findViewById(R.id.titleLayout);
        tapLayout = (RelativeLayout) findViewById(R.id.tapLayout);
        tapLayout.findViewById(R.id.back_ll).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                back();
            }
        });
        tapRadio = (LinearLayout) tapLayout.findViewById(R.id.title_container_ll);
        radioGroup = new AutoRadioGroup(tapRadio);

        assert texttitle != null;
        texttitle.setText("库存查询");
        listview = (ListView) findViewById(R.id.listview_stockgoods);
        mPresenter = new StockGoodsPresenter(this, new CommonNet());
        //先初始化了再说，后面请求回来了再重新setList()
        adapter = new StockGoodsAdapter(StockGoodsActivity.this, new ArrayList());
        listview.setAdapter(adapter);

        query();
    }

    /**
     * 库存明细查询请求
     */
    private void query() {
        if (NetHelper.IsHaveInternet(this)) {
//            clear();
            mParams.clear();
            CommonTools.INSTANCE.postDelayed(10000);
            ViewUtil.showProgress(this, "正在玩命加载...", "请稍等...");
            mParams.put("storeNo", SystemHelper.LoginOrganId);
            mPresenter.onStart2requestData(mParams);
        } else {
            MyToast.show("请检查网络是否已连接");
        }
    }

    /**
     * 把返回结果按库存天数排序，
     * 如果天数一样，就按产品 id 进行升序排序
     */
    private void sortList(List<StockGoodsDetailEntity.DetailList> list) {
        Collections.sort(list, new Comparator<StockGoodsDetailEntity.DetailList>() {
            @Override
            public int compare(StockGoodsDetailEntity.DetailList lhs, StockGoodsDetailEntity.DetailList rhs) {
                if (lhs.getStockDays() == rhs.getStockDays()) {
                    return lhs.getProductNo().compareTo(rhs.getProductNo());
                } else {
                    return (int) (rhs.getStockDays() - lhs.getStockDays());
                }
            }
        });
    }

    @Override
    public void onResponseSuccess(CommonEntity<StockGoodsDetailEntity> resultInfo) {//不要用这个resultInfo，没什么裸用
        if (isFinishing()) return;

        if (!mPresenter.checkHasData()) {
            showMsg("无门店库存明细列表数据");
        } else {
            sortList(mPresenter.getDetailList());
            //初始化顶栏显示
            int brandSum = mPresenter.getBrandList().size();
            tapLayout.setVisibility(brandSum > 1 ? View.VISIBLE : View.GONE);
            titleLayout.setVisibility(brandSum > 1 ? View.GONE : View.VISIBLE);

            //初始化顶栏标签内容样式
            for (final StockGoodsDetailEntity.BrandList brand : mPresenter.getBrandList()) {
                RadioButton radio = (RadioButton) getLayoutInflater().inflate(R.layout.tap_title_radio_btn, tapRadio, false);
                radioGroup.addViewWithLayoutParams(radio, brand.getBrandName(), 0, RadioGroup.LayoutParams.MATCH_PARENT, integer -> {
                    refreshListByRadio(brand.getBrandId());
                    return null;
                });

            }
            //默认选中第一个
            radioGroup.setChildChecked(0);
            //按 库存天数 和 产品编码 排序。
            ViewUtil.dismissProgress();//等所有数据全部显示完成了再取消转圈圈。

            //延时2秒来自动滑动横向scrollView，让用户知道右边还有东西
            new Handler().postDelayed(new Runnable() {
                public void run() {
                    hScrollView.fullScroll(HorizontalScrollView.FOCUS_RIGHT);
                }
            }, 500L);
            new Handler().postDelayed(new Runnable() {
                public void run() {
                    hScrollView.fullScroll(HorizontalScrollView.FOCUS_LEFT);
                }
            }, 1_000L);
        }
    }


    /**
     * 通过点击顶栏分类标签来刷新数据
     */
    private void refreshListByRadio(int brandId) {
        adapter.setList(mPresenter.filterByBrandId(brandId));
        adapter.notifyDataSetInvalidated();
    }


    @Override
    public void onFailure(String failInfo) {
        ViewUtil.dismissProgress();
        MyToast.show(failInfo);
    }

    /**
     * 清空所有缓存数据
     */
    private void clear() {
//        if (mStockList != null && !mStockList.isEmpty()) {
//            mStockList.clear();
//            adapter.notifyDataSetChanged();
//        }
    }
}
