package com.urovo.activity.goodsmanager.weight

import android.content.Context
import android.graphics.Canvas
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.goodsmanager.adapter.order.OrderDeliveryDifferenceTitleAdapter
import com.urovo.activity.goodsmanager.model.OrderDeliveryDifferenceQueryEntity
import com.urovo.wugumofang.R

/**
 * 订货与发货差异查询固定头部二级列表
 */
class OrderDeliveryDifferenceQueryListView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : ExpandableListView(context, attrs, defStyleAttr) {

    private var mHeaderView: ViewGroup? = null
    private var mOrderDeliveryDifferenceArrowIv: ImageView? = null
    private var mOrderDeliveryDifferenceItemContentCl: ConstraintLayout? = null
    private var mOrderDeliveryDifferenceRecyclerHeaderLl: LinearLayout? = null
    private var mOrderDeliveryDifferenceRecyclerHeaderList: RecyclerView? = null
    private var mOrderDeliveryDifferenceTitleTv: TextView? = null

    private var mHeaderViewWidth: Int = 0
    private var mHeaderViewHeight: Int = 0

    /**
     * 当前列表组角标
     */
    private var mCurrentGroupPosition: Int = 0

    /**
     * 列表展开收起状态值
     */
    private var mExpandedMap = mutableMapOf<Int, Boolean>()

    private var mPreviousFirstVisibleItem: Int = 0
    private var mPreviousEventTime: Long = 0
    private var mSpeed: Double = 0.0

    private var mIsGroupClick = false

    init {
        setOnGroupClickListener { parent, v, groupPosition, id ->
            mIsGroupClick = true
            val groupExpanded = isGroupExpanded(groupPosition)
            setExpanded(!groupExpanded)
            mExpandedMap[groupPosition] = !groupExpanded
            if (!groupExpanded) {
                parent.expandGroup(groupPosition, false);
                parent.setSelectedGroup(groupPosition);//点击分组置顶
            } else {
                parent.collapseGroup(groupPosition)
            }
            true
        }

        setOnScrollListener(object : OnScrollListener {
            override fun onScrollStateChanged(view: AbsListView?, scrollState: Int) {

            }

            override fun onScroll(
                view: AbsListView?,
                firstVisibleItem: Int,
                visibleItemCount: Int,
                totalItemCount: Int
            ) {
                if (mPreviousFirstVisibleItem != firstVisibleItem) {
                    val currTime: Long = System.currentTimeMillis()
                    val timeToScrollOneElement = currTime - mPreviousEventTime
                    mSpeed = (1.0 / timeToScrollOneElement) * 1000
                    mPreviousFirstVisibleItem = firstVisibleItem
                    mPreviousEventTime = currTime
                }
//                if (mSpeed > 15) {
//                    mHeaderView?.layout(0, -mHeaderViewHeight, mHeaderViewWidth, 0)
//                    return
//                }
                mHeaderView?.let { headerView ->
                    val firstGroup: Int =
                        getPackedPositionGroup(getExpandableListPosition(firstVisibleItem))
                    //val firstGroupChildCount: Int =
                    //    expandableListAdapter.getChildrenCount(firstGroup)
                    val nextPosition: Int = firstVisibleItem + 1
                    val nextGroup: Int =
                        getPackedPositionGroup(getExpandableListPosition(nextPosition))

                    if (firstGroup > mCurrentGroupPosition || firstGroup < mCurrentGroupPosition) {
                        bindListHeaderViewData(firstGroup)
                        setExpanded(mExpandedMap[firstGroup] ?: false)
                        setVisibilityTitle(mExpandedMap[firstGroup] ?: false)
                        mCurrentGroupPosition = firstGroup
                    }
                    if (mExpandedMap[firstGroup] == true) {
                        getChildAt(1)?.let { childView ->
                            val top = childView.top
                            if (firstGroup + 1 == nextGroup) {
                                val previousChildViewHeight = getChildAt(0).height
                                //if (mHeaderViewHeight == previousChildViewHeight) {
                                //默认情况，是根据mHeaderViewHeight高度来计算，View滑动
                                val delta =
                                    mHeaderViewHeight - (mHeaderViewHeight * (top / previousChildViewHeight))
                                if (top < mHeaderViewHeight) {
                                    headerView.layout(
                                        0,
                                        -delta,
                                        mHeaderViewWidth,
                                        mHeaderViewHeight - delta
                                    )
                                } else {
                                    headerView.layout(0, 0, mHeaderViewWidth, mHeaderViewHeight)
                                }
                                /*
                                } else {
                                    //由于previousChildViewHeight、mHeaderViewHeight两者的高度不一致，导致top滑动数值有偏差，所以增加倍数值。
                                    //其实也可以把两者进行比值处理，也是可以实现，但是体验不是很好。
                                    val delta = previousChildViewHeight - top
                                    if (top < previousChildViewHeight) {
                                        headerView.layout(
                                            0,
                                            -(delta * 4.0).roundToInt(),
                                            mHeaderViewWidth,
                                            mHeaderViewHeight - (delta * 4.0).roundToInt()
                                        )
                                    } else {
                                        headerView.layout(0, 0, mHeaderViewWidth, mHeaderViewHeight)
                                    }
                                }
                                */
                            } else {
                                headerView.layout(0, 0, mHeaderViewWidth, mHeaderViewHeight)
                            }
                        }
                    } else {
                        headerView.layout(0, -mHeaderViewHeight, mHeaderViewWidth, 0)
                    }
                }
            }
        })
    }

    fun showListHeaderView() {
        mHeaderView =
            LayoutInflater.from(context)
                .inflate(
                    R.layout.item_order_delivery_difference_query_list,
                    null
                ) as ViewGroup?

        mHeaderView?.layoutParams = RelativeLayout.LayoutParams(
            RelativeLayout.LayoutParams.MATCH_PARENT,
            RelativeLayout.LayoutParams.WRAP_CONTENT
        )

        mHeaderView?.let {
            mOrderDeliveryDifferenceArrowIv =
                it.findViewById(R.id.order_delivery_difference_arrow_iv)
            mOrderDeliveryDifferenceItemContentCl =
                it.findViewById(R.id.order_delivery_difference_item_content_cl)
            mOrderDeliveryDifferenceRecyclerHeaderLl =
                it.findViewById(R.id.order_delivery_difference_recycler_header_ll)
            mOrderDeliveryDifferenceTitleTv =
                it.findViewById(R.id.order_delivery_difference_title_tv)
            mOrderDeliveryDifferenceRecyclerHeaderList =
                it.findViewById(R.id.order_delivery_difference_recycler_header_list)

        }
        val list = context.resources.getStringArray(R.array.order_delivery_difference_list_title)
            .toMutableList()
        mOrderDeliveryDifferenceTitleTv?.text = list[0]

        mOrderDeliveryDifferenceRecyclerHeaderList?.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )
        mOrderDeliveryDifferenceRecyclerHeaderList?.setHasFixedSize(true)
        mOrderDeliveryDifferenceRecyclerHeaderList?.adapter =
            OrderDeliveryDifferenceTitleAdapter(
                list.subList(1, list.size)
            )

        mExpandedMap.clear()
        for (pos in 0..expandableListAdapter.groupCount) {
            mExpandedMap[pos] = false
        }

        //获取角标为0的实体类数据，并进行数据绑定
        bindListHeaderViewData(0)

        setExpanded(false)
        setVisibilityTitle(false)

    }

    fun getRecyclerHeaderView(): RecyclerView? = mOrderDeliveryDifferenceRecyclerHeaderList

    /**
     * 绑定Header View 数据
     */
    private fun bindListHeaderViewData(groupPosition: Int) {
        (expandableListAdapter.getGroup(groupPosition) as OrderDeliveryDifferenceQueryEntity?)?.let {
            mHeaderView?.findViewById<TextView>(R.id.order_delivery_difference_order_tv)?.apply {
                text = String.format(
                    context.getString(R.string.order_delivery_difference_delivery_order),
                    it.psbillCode
                )
            }

            mHeaderView?.findViewById<TextView>(R.id.order_delivery_difference_date_tv)?.apply {
                text = String.format(
                    context.getString(R.string.order_delivery_difference_delivery_date),
                    it.psDate
                )
            }

            mHeaderView?.findViewById<TextView>(R.id.order_delivery_difference_brand_tv)?.apply {
                text = String.format(
                    context.getString(R.string.order_delivery_difference_brand),
                    it.brandName
                )
            }
        }
    }

    /**
     * 展开，收起状态切换
     * @param isExpanded Boolean true 展开; false 收起
     */
    private fun setExpanded(isExpanded: Boolean) {
        if (isExpanded) {
            mOrderDeliveryDifferenceArrowIv?.setImageResource(R.mipmap.ic_arrow_up)
            mOrderDeliveryDifferenceItemContentCl?.setBackgroundResource(R.drawable.shape_white_round_top_6)
        } else {
            mOrderDeliveryDifferenceArrowIv?.setImageResource(R.mipmap.ic_arrow_down)
            mOrderDeliveryDifferenceItemContentCl?.setBackgroundResource(R.drawable.shape_white_round_6)
        }
    }


    /**
     * 展开，收起标题切换
     * @param isExpanded Boolean true 展开; false 收起
     */
    private fun setVisibilityTitle(isExpanded: Boolean) {
        if (isExpanded) {
            mOrderDeliveryDifferenceRecyclerHeaderLl?.visibility = View.VISIBLE
        } else {
            mOrderDeliveryDifferenceRecyclerHeaderLl?.visibility = View.GONE
        }

        measureChild(mHeaderView, 0, 0)
        requestLayout()
        postInvalidate()
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        mHeaderView?.let {
            measureChild(it, widthMeasureSpec, heightMeasureSpec)
            mHeaderViewWidth = it.measuredWidth
            mHeaderViewHeight = it.measuredHeight
        }
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        super.onLayout(changed, l, t, r, b)
        if (mIsGroupClick) {
            mIsGroupClick = false
            return
        }
//        mHeaderView?.layout(0, -mHeaderViewHeight, mHeaderViewWidth, 0)
        mHeaderView?.layout(0, 0, mHeaderViewWidth, mHeaderViewHeight)
    }

    override fun dispatchDraw(canvas: Canvas?) {
        super.dispatchDraw(canvas)
        mHeaderView?.let {
            drawChild(canvas, it, drawingTime)
        }
    }

    private var downX = 0
    private var downY = 0

    override fun dispatchTouchEvent(ev: MotionEvent?): Boolean {
        mOrderDeliveryDifferenceItemContentCl?.let {
            when (ev?.action) {
                MotionEvent.ACTION_DOWN -> {
                    downX = ev.x.toInt()
                    downY = ev.y.toInt()
                }
                MotionEvent.ACTION_UP -> {
                    if (downX >= it.top && downY <= it.bottom) {
                        val position = pointToPosition(downX, downY)
                        val groupPosition =
                            getPackedPositionGroup(getExpandableListPosition(position))
                        val groupExpanded = isGroupExpanded(groupPosition)
                        if (groupPosition != INVALID_POSITION) {
                            if (groupExpanded) {
                                collapseGroup(groupPosition)
                            } else {
                                expandGroup(groupPosition, false)
                            }
                            setExpanded(!groupExpanded)
                            setVisibilityTitle(!groupExpanded)
                            mExpandedMap[groupPosition] = !groupExpanded
                        }
                        return true
                    }
                }
            }
        }
        return super.dispatchTouchEvent(ev)
    }
}