package com.urovo.activity.learning

import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.learning.model.ExamArchiveEntity
import com.urovo.activity.learning.presenter.ArchiveDetailContract
import com.urovo.activity.learning.presenter.LearningCenterPresenter
import com.urovo.activity.learning.view.adapter.ExamArchiveListAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.utils.convertDefaultValue
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.activity_exam_archive_list.*

/**
 * 考试档案
 * @author LW
 */
class ExamArchiveListActivity : BaseActivity(), ArchiveDetailContract.IView {

    private var mLearningCenterPresenter: LearningCenterPresenter? = null

    private var mExamArchiveListAdapter: ExamArchiveListAdapter? = null

    override fun addRootLayoutId(): Int = R.layout.activity_exam_archive_list

    override fun rootViewGroup(): ViewGroup = exam_archive_container_fl

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.learning_center_exam_archive)
    }

    override fun initViewNew() {
        initTitle()

        mExamArchiveListAdapter = ExamArchiveListAdapter(arrayListOf())
        exam_archive_list_view_rv.apply {
            layoutManager = LinearLayoutManager(mContext)
            adapter = mExamArchiveListAdapter
        }

        mLearningCenterPresenter?.requestExamArchiveInfo()
    }

    override fun addPresenter(): BasePresenter {
        mLearningCenterPresenter = LearningCenterPresenter(this)
        return mLearningCenterPresenter!!
    }

    override fun examArchiveInfoData(data: ExamArchiveEntity) {
        super.examArchiveInfoData(data)
        val archiveDetailList = data.archiveDetailList
        if (archiveDetailList.isNotNullOrSize()) {
            mExamArchiveListAdapter?.setNewData(archiveDetailList)
        } else {
            showState(LoadWaitDominator.NO_DATA)
        }

        exam_archive_header_container_rl.visibility = View.VISIBLE

        exam_archive_name_tv.text = StringUtils.formatString(
            mContext.getString(R.string.learning_center_exam_archive_name_code),
            data.staffName.convertDefaultValue(), data.staffId.convertDefaultValue()
        )

        exam_archive_entry_time_tv.text = StringUtils.formatString(
            mContext.getString(R.string.learning_center_exam_archive_entry_time),
            data.wgmfWorkDate.convertDefaultValue()
        )
        exam_archive_status_tv.text = StringUtils.formatString(
            mContext.getString(R.string.learning_center_exam_archive_status),
            data.isOnService.convertDefaultValue()
        )
    }
}