package com.urovo.activity.learning.presenter

import com.urovo.activity.learning.model.ExamArchiveEntity
import com.urovo.activity.learning.service.LearningCenterDepository
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.utils.isNullOrEmpty
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Response

/**
 * 学习中心 Presenter
 * @author LW
 */
class LearningCenterPresenter(
    private var view: ArchiveDetailContract.IView?,
    private var depository: LearningCenterDepository = LearningCenterDepository()
) : AbsPresenter(), ArchiveDetailContract.IPresenter {

    override fun requestExamArchiveInfo() {
        val staffId = SystemHelper.LoginUserId
        if (staffId.isNullOrEmpty()) {
            view?.showError(R.string.common_load_error)
            return
        }
        view?.showIsLoading()
        depository.requestExamArchiveInfo(staffId).enqueue(object :
            RetrofitCallback<RequestResult<ExamArchiveEntity>>() {
            override fun onResponse(
                call: Call<RequestResult<ExamArchiveEntity>>,
                res: Response<RequestResult<ExamArchiveEntity>>
            ) {
                val body = res.body()
                body.takeIf { it != null }?.run {
                    if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                        if (payload != null) {
                            view?.examArchiveInfoData(payload)
                        } else {
                            view?.showState(LoadWaitDominator.NO_DATA)
                        }
                    } else {
                        view?.showError(msgInfo)
                    }
                } ?: view?.showState(LoadWaitDominator.NET_ERROR)
                view?.dismissLoading()
            }

            override fun onFailure(exception: ApiException) {
                view?.dismissLoading()
                view?.showError(exception.message)
            }
        })
    }


}