package com.urovo.activity.learning.view.adapter

import androidx.core.content.ContextCompat
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.urovo.activity.learning.model.ExamArchiveEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.utils.TextColorDominator
import com.urovo.wugumofang.utils.convertDefaultValue
import com.urovo.wugumofang.utils.notNullAndEmpty

/**
 * 考试档案适配器
 * @author LW
 */
class ExamArchiveListAdapter(
    archiveDetailBean: List<ExamArchiveEntity.ArchiveDetailBean>
) : BaseQuickAdapter<ExamArchiveEntity.ArchiveDetailBean, BaseViewHolder>(
    R.layout.item_exam_archive_list,
    archiveDetailBean
) {
    override fun convert(helper: BaseViewHolder?, item: ExamArchiveEntity.ArchiveDetailBean?) {
        helper?.apply {
            item?.let {
                setText(
                    R.id.exam_archive_item_suject_tv,
                    it.examCourseName
                )

                setText(
                    R.id.exam_archive_item_title_tv,
                    it.examName.convertDefaultValue()
                )

                //考试得分
                val content: String = StringUtils.formatString(
                    mContext.getString(R.string.learning_center_exam_archive_test_score),
                    it.score.toString().convertDefaultValue()
                )

                //状态
                if (it.isClearance == ConstantParams.CONSTANT_STARTS_1) {
                    setText(
                        R.id.exam_archive_item_status_tv,
                        R.string.learning_center_exam_archive_cleared
                    )
                    setTextColor(
                        R.id.exam_archive_item_status_tv,
                        ContextCompat.getColor(mContext, R.color.common_two_txt_color)
                    )
                    setText(
                        R.id.exam_archive_item_test_score_tv,
                        TextColorDominator.styleTextColor(
                            content, ContextCompat.getColor(
                                mContext,
                                R.color.common_two_txt_color
                            )
                        )
                    )
                } else {
                    setText(
                        R.id.exam_archive_item_status_tv,
                        R.string.learning_center_exam_archive_not_cleared
                    )
                    setTextColor(
                        R.id.exam_archive_item_status_tv,
                        ContextCompat.getColor(mContext, R.color.color_common_red_one)
                    )

                    setText(
                        R.id.exam_archive_item_test_score_tv,
                        TextColorDominator.styleTextNumberColor(
                            content, ContextCompat.getColor(
                                mContext,
                                R.color.color_common_red_one
                            )
                        )
                    )
                }

                //通关日期
                if (it.clearanceDate.notNullAndEmpty() && it.isClearance == ConstantParams.CONSTANT_STARTS_1) {
                    setText(
                        R.id.exam_archive_item_clearance_date_tv,
                        StringUtils.formatString(
                            mContext.getString(R.string.learning_center_exam_archive_clearance_date),
                            it.clearanceDate.convertDefaultValue()
                        )
                    )
                    setGone(R.id.exam_archive_item_clearance_date_tv, true)
                } else {
                    setGone(R.id.exam_archive_item_clearance_date_tv, false)
                }
            }
        }
    }
}