package com.urovo.activity.learningsearch.view;

import android.content.Context;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.View;
import android.view.animation.AnimationUtils;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.RelativeLayout;

import com.urovo.activity.learningsearch.presenter.LearnSearchPresenter;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.system.CountDownDominator;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.constants.AppConstants;

import java.util.List;
import java.util.Timer;


/**
 * Created by Sasuke on 2017/3/6.
 * 学习中心 养生文库 关键词搜索
 */

public class LearnSearchAct extends BaseActivity implements CountDownDominator.CountDownCallback {
    LearnSearchPresenter searchPre;
    ImageButton back;
    LearnResultFra resultFra;//结果页
    EditText searchInput;//输入框
    RelativeLayout searchAnimView;//要进行动画的组件

    private final long SEARCH_PERIOD = 1 * 1000;//输入完成后过多少秒自动执行搜索

    Timer autoTimer;//用来在输入框发生改变时，自动倒数进行刷新的装置。
//    final int AUTO_SEARCH_DELAY = 2;//自动执行搜索的延时。（秒）
//    int goal;//目标时间。
//    int t = 0;//当前秒数。

    CountDownDominator counter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.learn_search_ll);

        resultFra = new LearnResultFra();

        searchPre = new LearnSearchPresenter(resultFra);
        searchPre.start();
        init();
        setListeners();
    }


    public void init() {
        int type = getIntent().getIntExtra(AppConstants.ExtraKeys.ARTICLE_SEARCH_TYPE, 1);
        searchPre.getData().setType(type);
        searchPre.getData().setAlreadyStores(
                (List<String>) getIntent().getSerializableExtra(AppConstants.ExtraKeys.ARTICLE_SEARCH_ALREADY_STORES));
//        if (type != 1 ) {
//            showMsg("Intent传值错误(LearningCenterSearch.class)");
//            finish();
//        }
        searchAnimView = (RelativeLayout) findViewById(R.id.search_anim_view);
        searchInput = (EditText) findViewById(R.id.search_input);
        back = (ImageButton) findViewById(R.id.search_back);

        getSupportFragmentManager().beginTransaction().add(R.id.result_container, resultFra).commit();
        //搜索框从右至左滑动的动画
        searchAnimView.startAnimation(AnimationUtils.loadAnimation(this, R.anim.searchbar_right_in));
        //构造自动搜索
        autoTimer = new Timer();
        counter = new CountDownDominator(this, this);
        counter.build(autoTimer);
    }

    @Override
    public void doWorkWhenTimeUp(int tag) {
        searchPre.searchArticle(searchPre.getData().getKeyWord());
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    private void setListeners() {
        //普通的左上角返回退出
        back.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        //输入框文字改变监听
        searchInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                searchPre.getData().setKeyWord(s.toString());
                if (autoTimer == null) {
                    autoTimer = new Timer();
                }
                counter.start(SEARCH_PERIOD);
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        //键盘回车搜索
        searchInput.setOnKeyListener(new View.OnKeyListener() {
            @Override
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                //因为DOWN和UP事件都是KEYCODE_ENTER事件，所以会触发两次，所以必须加一道判断ACTION_UP
                if (keyCode == event.KEYCODE_ENTER && event.getAction() == KeyEvent.ACTION_UP) {
                    hideKeyboard();
                    searchPre.searchArticle(searchInput.getText().toString());
                    counter.cancel();//取消计时
                }
                return false;
            }
        });
    }

    /***
     * 隐藏键盘
     */
    public void hideKeyboard() {
        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.showSoftInput(searchInput, InputMethodManager.SHOW_FORCED);
        imm.hideSoftInputFromWindow(searchInput.getWindowToken(), 0); //强制隐藏键盘
    }

    @Override
    protected void onPause() {
//        刚进来这个Activity时，这个 Fragment应该是空的，实例化是以后的事，
//        所以可以依靠这一点来判断，这个onPase()执行的时机，是刚进来？还是在跳转新界面。
        if (resultFra == null) {//只有刚进Activity时才不需要过渡动画，让别人以为这个搜索是在同一个界面进行的。。。神不知鬼不觉。
            overridePendingTransition(0, 0);//切换界面时不要动画
        }
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (autoTimer != null) {
            autoTimer.cancel();
        }
    }
}
