package com.urovo.activity.login.presenter;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.database.SQLException;
import android.text.TextUtils;
import android.widget.ImageView;

import com.google.gson.Gson;
import com.squareup.picasso.Picasso;
import com.urovo.activity.login.model.LoginData;
import com.urovo.activity.login.net.AlertDeclarationRes;
import com.urovo.activity.login.net.CheckStoreValidRes;
import com.urovo.activity.login.net.DeclarationRes;
import com.urovo.activity.login.net.LoginImageRes;
import com.urovo.activity.login.net.LoginOnlineRes;
import com.urovo.activity.login.net.WelcomeRes;
import com.urovo.activity.login.view.PartTimeSalesStaffAct;
import com.urovo.activity.newGuideAct.view.NewGuideAct;
import com.urovo.activity.tabMenu.util.JPush.JPushCase;
import com.urovo.wgmf.AsyncTask.BaseAppManageServerTask;
import com.urovo.wgmf.AsyncTask.GetRetCodeInfoTask;
import com.urovo.wgmf.AsyncTask.GetServerVersionTask;
import com.urovo.wgmf.db.dao.dao.Czyda001DbOperate;
import com.urovo.wgmf.db.dao.dao.PdaparamOperate;
import com.urovo.wgmf.db.dao.dao.Store001DbOprate;
import com.urovo.wgmf.db.dao.entitys.Pdaparam;
import com.urovo.wgmf.db.dao.entitys.Store001Db;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.Files;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.data.DateUtil;
import com.urovo.wugumofang.data.JsonDominator;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.response.BaseOldResponse;
import com.urovo.wugumofang.system.BaseAppManager;
import com.urovo.wugumofang.system.DeviceHelper;
import com.urovo.wugumofang.system.PdaMsgConfig;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.utils.FuckGoogleAdaptUtil;
import com.urovo.wugumofang.utils.MyToast;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libcore.ui.act.SYActManager;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import okhttp3.Request;

/**
 * Created by Sasuke on 2016/10/25.
 * ...
 */

public class LoginPresenter implements LoginContract.presenter {
    LoginContract.view mView;
    LoginData data;
    Context context;

    public LoginPresenter(LoginContract.view mView) {
        this.mView = mView;
        mView.setPresenter(this);
        data = new LoginData();
        this.context = (Context) mView;
    }

    public LoginData getData() {
        if (data == null) {
            data = new LoginData();
        }
        return data;
    }

    /**
     * 获取状态码文案信息
     */
    public void getRetCodeInfo() {
        new GetRetCodeInfoTask((BaseActivity) context, "", false).execute("");
    }

    public void getServerVersion(String userNo) {
        getData().getParams().put("userNo", userNo);
        new GetServerVersionTask((BaseActivity) context, "检测应用是否有更新...", false, getData().getParams()).execute("");
    }

    /**
     * 显示免责声明，并跳转页面
     * 注：会在本地和后台都记录一份“已弹记录”，即点击“确定使用”后记录
     */
    public void startActivityWithDeclaration(final String staffId) {
        //先看本地有没有该工号的记录，如果已经弹过一次免责声明弹窗了，那就不需要再弹
        if (PreferHelper.getInstance().getString(AppConstants.PreKeys.DECLARATION_USERS).contains(staffId)) {
            mView.startActivityAndFinish(PartTimeSalesStaffAct.class, null);//直接跳转，不做后面的弹窗
            return;
        }
        final HashMap<String, String> params = new HashMap<>();
        params.put("staffId", staffId);
        mView.showProgress("请稍等", "验证成功，正在进入兼职销售模式...");
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.SHOW_DECLARATION, params,
                new OkHttpDominator.ResultCallback<AlertDeclarationRes>() {
                    @Override
                    public void onError(Request request, Exception e) {
                        mView.dismissProgress();
                        mView.showMsg(e.toString());
                        mView.startActivityAndFinish(PartTimeSalesStaffAct.class, null);
                    }

                    @Override
                    public void onResponse(AlertDeclarationRes res) {
                        mView.dismissProgress();
                        if (res == null) {
                            mView.showMsg("网络连接超时，请检查您的网络状态，稍后重试！");
                            return;
                        }
                        //如果已阅读过声明
                        if (res.msgCode == 1) {
                            mView.startActivityAndFinish(PartTimeSalesStaffAct.class, null);
                            return;
                        }
                        new AlertDialog.Builder(context).setTitle(res.getResultInfo().getTitle())
                                .setMessage(res.getResultInfo().getInfo())
                                .setCancelable(false)
                                .setNegativeButton("取消使用", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        SYActManager.getInstance().AppExit();//不同意免责声明就退出程序
                                    }
                                })
                                .setPositiveButton("确定使用", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        //① 只有同意之后，才记录工号，使下次不再弹窗
                                        String newLog = PreferHelper.getInstance().getString(AppConstants.PreKeys.DECLARATION_USERS) + staffId + ",";
                                        PreferHelper.getInstance().setString(AppConstants.PreKeys.DECLARATION_USERS, newLog);
                                        //② 后台也要记录一份
                                        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.AGREE_DECLARATION, params, null);
                                        //③ 记录完成，跳转页面
                                        mView.startActivityAndFinish(PartTimeSalesStaffAct.class, null);
                                    }
                                }).create().show();
                    }
                });
    }

    @Override
    public void start() {
        checkIsAppInitSuccess();//有时会出现log is not exist 的问题，其实是database等初始化失败的原因，
        //以上保险措施验证完了再执行下面的初始化。
        mView.init();
        mView.initUI();
//        initOneApm();
        JPushCase.createNotificationChannel(BaseAppManager.appContext());//安卓8.0+的系统上，需要创建通知渠道
    }

    /**
     * 检查是否database  Prefs  loger是否初始化成功，
     * 如果失败，就再次初始化，以防万一
     */
    public void checkIsAppInitSuccess() {
        String warnText = "";
        if (BaseAppManager.appContext() == null) {
            warnText += "BaseAppManager.appContext() is null\n";
        }
        if (BaseAppManager.loger() == null) {
            warnText += "BaseAppManager.loger() is null\n";
        }
        if (BaseAppManager.database() == null) {
            warnText += "BaseAppManager.database() is null\n";
        }
        if (BaseAppManager.preferCache() == null) {
            warnText += "BaseAppManager.preferCache() is null\n";
        }

        if (!TextUtils.isEmpty(warnText)) {//说明初始化有异常
            MyToast.show(warnText);
            BaseAppManager.getInstance().initLogger();
            BaseAppManager.getInstance().initDatabase();
            BaseAppManager.getInstance().initPreferCache();
        }
    }

//    public void initOneApm(){
    //启动Agent
//        OneApmAgent.init(context.getApplicationContext()).setToken("19D90B21481E6DF50CE645BF87C9A17B90").start();
    //功能开关（可选）
//        PerformanceConfiguration.getInstance().setEnableFps(true);
    //用户信息配置（可选）
//        HashMap<String,String> extraData = new HashMap<String, String>();
//        String storeId = SystemHelper.LoginOrganId;
//        extraData.put("storeId", storeId);
//        extraData.put("userId", SystemHelper.LoginUserId);
//        ContextConfig config = new ContextConfig();
//        config.setSearchValue(storeId+"_"+SystemHelper.LoginUserId); // 设置一个搜索值
//        config.setExtra(extraData);
//        OneApmAgent.setContextConfig(config);
//    }

//    public boolean updateData(){
//        String time = PdaMsgConfig.getDataUpdateTime(context);
//        if(!TextUtils.isEmpty(time))
//        {
//            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//            try {
//                Date update = dateFormat.parse(time);
//                Date now = new Date();
//                Calendar c = Calendar.getInstance();
//                c.set(Calendar.YEAR, update.getYear() + 1900);
//                c.set(Calendar.MONTH, update.getMonth());
//                c.set(Calendar.DATE, update.getDate());
//                c.set(Calendar.HOUR_OF_DAY, update.getHours());
//                c.set(Calendar.MINUTE, update.getMinutes());
//                c.set(Calendar.SECOND, update.getSeconds());
//                c.add(Calendar.DATE, 2);
//                String t1 = dateFormat.format(update);
//                update = c.getTime();
//                String t2 = dateFormat.format(update);
//                String t3 = dateFormat.format(now);
//                if(now.compareTo(update)<0)
//                {
//                    return false;
//                }
//            }
//            catch (Exception e) {
//                Log.e("login",e.toString());
//            }
//
//        }
//        List<String> UpdateTables = new ArrayList<String>();
//        UpdateTables.add("ITEM001");
//        UpdateTables.add("DEPTCSDA");
//        UpdateTables.add("STORE001");
//        UpdateTables.add("STORECS_GRP002");
//        UpdateTables.add("CZYDA001");
//        UpdateTables.add("T_DHLPITEM");
//        UpdateTables.add("USERDTR");
//        UpdateTables.add("ITEM005");
//        UpdateTables.add("CPBILL002");
//        UpdateTables.add("DJCXITEM");
//        UpdateTables.add("CPBARCODE");
//        UpdateTables.add("XBZPF");
//        new GetUpdateSqlFileNameTask((BaseActivity) context, "获取需更新的数据", true, UpdateTables).execute("");
//        return true;
//    }

    /**
     * 登录记录请求（一进登陆界面就请求）
     */
    public void loginInfoRecord(String userId) {
        if (!TextUtils.isEmpty(userId)) {//只有在工号存在的时候，才做请求。
            Map<String, String> p = new HashMap<String, String>();
            p.put("MAC", SystemHelper.Mac);
            p.put("IMEI", SystemHelper.Imei);
            p.put("storeId", SystemHelper.LoginOrganId);
            p.put("staffNo", userId);
            p.put("orgNo", SystemHelper.LoginOrganId);
            p.put("current_version", SystemHelper.Version);
            p.put("android_version", SystemHelper.AndroidVersion);
            new BaseAppManageServerTask("login_info", BaseOldResponse.class, (BaseActivity) context,
                    "正在登录记录...", false, "/app/logininfo", p).execute("");
        }
    }

    /**
     * 验证门店
     *
     * @param isLogin 是否只验证门店？（还是验证门店之后一并验证用户进行登陆
     */
    public void checkStoreValid(final String orgId, final boolean isLogin) {
        //new CheckStoreValidTask((BaseActivity)context, "正在验证门店合法性！", true).execute("");
        HashMap<String, String> params = new HashMap<>();
        params.put("storeNo", orgId);
//        new BaseAppManageServerTask(TextUtils.isEmpty(tag)?"CheckStoreValid":tag, CheckStoreValidRes.class,(BaseActivity) context,"正在验证门店...",true,
//                                                                "/information/checkStore",params).execute("");
        mView.showProgress("请稍等", "正在验证门店...");
        OkHttpDominator.postAsynTimeout(NetParam.YINXIAOBU_SERVER + ApiUrls.APPMANAGE.CHECK_STORE,
                params,
                new OkHttpDominator.ResultCallback<CheckStoreValidRes>() {
                    @Override
                    public void onError(Request request, Exception e) {
                        e.printStackTrace();
                        mView.dismissProgress();
                        //如果没到5天强制登陆时间 并且 如果有缓存，那就当作请求成功处理，把缓存看作后台返回。
                        if (!isStrongCheckStore(orgId)) {
                            //把返回的数据写入数据库
                            saveStoreInfo(JsonDominator.loadFromDir(Files.getCheckStoreCache(orgId), CheckStoreValidRes.class, true));
                            mView.refreshUI();
                            if (isLogin) {//验证完门店紧接着验证用户，切换门店后一键登陆时使用，
                                mView.login();
                            }
                            return;
                        }
                        //在成功登陆超过5天后，强制验证门店时，如果验证门店失败，才弹窗提示检查网络。
//                        if (isStrongCheckStore(orgId)) {
                        mView.storeCheckError();
//                        }
//                        mView.showMsg("网络异常: "+e.toString());
                    }

                    @Override
                    public void onResponse(CheckStoreValidRes res) {
                        mView.dismissProgress();
                        //===========test==========
                        CheckStoreValidRes resTemp = JsonDominator.loadFromSD("CheckStoreValid", CheckStoreValidRes.class);
                        if (resTemp != null) {
                            //......do_anything
                            res = resTemp;
                        }
                        //===========test==========
                        if (res != null && "0".equals(res.msgCode)) {//验证门店成功
//                            if (isStrongCheckStore(orgId)) {
                            PreferHelper.getInstance().setString(
                                    AppConstants.PreKeys.LAST_CHECK_STORE_VERSION,
                                    SystemHelper.Version);
//                            }
                            //把返回的数据写入数据库
                            saveStoreInfo(res);
                            //把返回的Json做个Gzip压缩再写入缓存文件（SystemHelper.LoginOrganId是在saveStoreInfo()方法里被赋值的，所以一定要放在它后面执行）
                            JsonDominator.fire2Dir(new Gson().toJson(res), Files.getCheckStoreCache(SystemHelper.LoginOrganId), true);
                            mView.refreshUI();
                            if (isLogin) {//验证完门店紧接着验证用户，切换门店后一键登陆时使用，
                                mView.login();
                            }
                            //mPresenter.getInitlizeSqlFileName();
                        } else {
                            mView.showMsg(res.msgInfo);
                            mView.resetOrgIdVibrate();//恢复门店号并抖动
//                orgNameEdittext.setText(res.msgInfo);
//                            AnimeUtil.setTextAlpha(orgIdEdittext, SystemHelper.LoginOrganId);//验证失败了，把门店号改回原来
                        }
                    }
                }, 10);//自定义10秒超时
    }

    /**
     * 是否强制验证门店（判断距上次验证门店是否超过5天，超过5天就强制）
     *
     * @return
     */
    public boolean isStrongCheckStore(String storeId) {
        //如果不存在对应门店的缓存文件，说明没有验证成功过，所以一定要强制验证
        if (!Files.getCheckStoreCache(storeId).exists()) {
            //清空密码，使他不能通过离线绕过登陆
            PdaMsgConfig.setPassWord("", context);
            return true;
        }
        //如果间隔大于等于5天，就强制验证（用当前long时间减去缓存文件的最后修改时间） 或者 上次验证门店成功时的版本号与本次APP版本不同
        if (DateUtil.diffDate(Calendar.getInstance().getTimeInMillis(), Files.getCheckStoreCache(storeId).lastModified()) >= 5 ||
                !PreferHelper.getInstance().getString(AppConstants.PreKeys.LAST_CHECK_STORE_VERSION).equals(SystemHelper.Version)) {
            //清空密码，使他不能通过离线绕过登陆
            PdaMsgConfig.setPassWord("", context);
            Files.getCheckStoreCache(storeId).delete();
            return true;
        }
        return false;

    }

    /**
     * 门店验证通过后，把门店相关信息写入数据库
     */
    public void saveStoreInfo(CheckStoreValidRes res) {
        Store001DbOprate.deleteBy(null);
        Store001Db db = new Store001Db();
        db.setCpbill(res.getResultInfo().cpBill);
        db.setDeptcsID(res.getResultInfo().deptcsId);
        db.setGcID(res.getResultInfo().gcId);
        db.setStatus(res.getResultInfo().status);
        db.setStoreID(res.getResultInfo().storeId);
        db.setStorename(res.getResultInfo().storeName);
        db.setYymode(res.getResultInfo().yyMode);
        db.setZgName(res.getResultInfo().zgName);//主管Name
        db.setZgNo(res.getResultInfo().zgNo);//主管No

        try {
            BaseAppManager.database().beginTransaction();
        } catch (NullPointerException e) {
            MyToast.show("LoginPresenter.saveStoreInfo(): ..database()初始化失败");
            e.printStackTrace();
        }
        try {
            Store001DbOprate.Insert(db);
            for (String sql : res.getSqls()) {
                BaseAppManager.database().execSQL(sql);//循环执行insert
            }
            BaseAppManager.database().setTransactionSuccessful();
            SystemHelper.LoginZgId = res.getResultInfo().zgNo;//如果门店数据库写入成功，就趁机把主管id和主管 Name也存到静态里。
            SystemHelper.LoginZgName = res.getResultInfo().zgName;
            //门店名&&门店号
            SystemHelper.LoginOrganName = res.getResultInfo().storeName;
            PdaMsgConfig.setLoginOrganName(res.getResultInfo().storeName, context);
            SystemHelper.LoginOrganId = res.getResultInfo().storeId;
            PdaMsgConfig.setLoginOrgID(res.getResultInfo().storeId, context);
        } catch (SQLException e) {
            e.printStackTrace();
            BaseAppManager.loger().info("LoginPresenter_saveStoreInfo()_db_write_exception");
            //这里可以改为弹窗提示，并丢到Act里处理重新下载*/
        } finally {
            //结束事务
            BaseAppManager.database().endTransaction();
        }
    }

    /**
     * 在线验证登陆
     *
     * @param userId   工号
     * @param password 密码
     * @param orgId    门店号
     */
    public void loginOnline(final String orgId, final String userId, final String password) {
        SYLoger.print("兼职：在线（online）验证登录");
        HashMap<String, String> params = new HashMap<>(3);
        params.put("storeNo", orgId);
        params.put("userNo", userId);
        params.put("passWord", DeviceHelper.getMD5Str(password));
//        new BaseAppManageServerTask("LoginOnline", LoginOnlineRes.class,(BaseActivity) context,"正在在线验证登陆...",true,
//                "/information/loginOnLine",params).execute("");
//        //new LoginOnlineTask((BaseActivity)context, "正在进行远程登陆验证", true, info).execute("");
        mView.showProgress("请稍等", "正在在线验证登陆...");
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPMANAGE.LOGIN_ONLINE,
                params,
                new OkHttpDominator.ResultCallback<LoginOnlineRes>() {
                    @Override
                    public void onError(Request request, Exception e) {
                        mView.dismissProgress();
                        mView.showMsg("在线验证网络异常");
                        //===========test==========
                        LoginOnlineRes res0 = JsonDominator.loadFromSD("LoginOnlineRes", LoginOnlineRes.class);
                        if (res0 != null) {
                            //......do_anything
//                            saveUserDb(userId, password, res0.getUserName(), res0.getSex(), res0.getMobile(), res0.getWgmfRole());
                            saveUserDb(res0.getUserName(), res0.getSex(), userId, password, res0.getMobile(), res0.getWgmfRole(), res0.getBrandList());//存数据库，供别的地方调
//                            PdaMsgConfig.setLoginUserRole(context, res0.getWgmfRole());// 存到本地下次做离线用
                            initAfterLogin(orgId, userId, password/*, res0.getWgmfRole()*/, true);
                            mView.loginSuccessJump(userId, password);
                            return;
                        }
                        //===========test==========
                    }

                    @Override
                    public void onResponse(LoginOnlineRes res) {
                        mView.dismissProgress();
                        //===========test==========
                        LoginOnlineRes res0 = JsonDominator.loadFromSD("LoginOnlineRes", LoginOnlineRes.class);
                        if (res0 != null) {
                            //......do_anything
//                            saveUserDb(userId, password, res0.getUserName(), res0.getSex(), res0.getMobile(), res0.getWgmfRole());
                            saveUserDb(res0.getUserName(), res0.getSex(), userId, password, res0.getMobile(), res0.getWgmfRole(), res0.getBrandList());//存数据库，供别的地方调
//                            PdaMsgConfig.setLoginUserRole(context, res0.getWgmfRole());// 存到本地下次做离线用
                            initAfterLogin(orgId, userId, password/*, res0.getWgmfRole()*/, true);
                            mView.loginSuccessJump(userId, password);
                            return;
                        }
                        //===========test==========
                        if (res != null) {
                            if ("0".equals(res.msgCode)) { //在线登录人员验证
                                saveUserDb(res.getUserName(), res.getSex(), userId, password, res.getMobile(), res.getWgmfRole(), res.getBrandList());//存数据库，供别的地方调
                                //这里直接改"兼职销售"做测试, 第二次启动就会走离线跳转到指定页面.
//                                PdaMsgConfig.setLoginUserRole(context, res.getWgmfRole());// 存到本地下次做离线用
                                initAfterLogin(orgId, userId, password/*, res.getWgmfRole()*/, true);
                                mView.loginSuccessJump(userId, password);
                            } else if ("1002".equals(res.msgCode)) {//密码已过期，请更换密码（6个月过期
                                initAfterLogin(orgId, userId, password/*, res.getWgmfRole()*/, true);
                                mView.popDialog2SettingPassword(res.msgInfo);
                            } else {
                                mView.showMsg(res.msgInfo);
                            }
                        }
                    }
                });
    }

    /**
     * 把用户信息写入数据库
     *
     * @param userId   不需要解释了，一看就明白
     * @param passWord ...
     * @param userName ...
     * @param userSex  ...
     * @param mobile   ...
     * @param role     登录人员角色身份
     */
    public void saveUserDb(String userName, String userSex, String userId, String passWord, String mobile, String role, String brandList) {
        Czyda001DbOperate.saveOnlineUser(userName, userSex, userId, passWord, mobile, role, brandList);
    }

//    /**
//     * 【远程验证登陆成功】后，把用户信息存到PreferCache（30天过期）
//     * @param userId 这里的UserId是要作为cache的key的
//     * @param info 点进UserInfo类自己看就知道了
//     */
//    public void putUser2Prefer(String userId,UserInfo info){
//        BaseAppManager.preferCache().put(AppConstants.PreKeys.LOGIN_LAST_ID,userId);//存最后一次登陆的工号
//        BaseAppManager.preferCache().put(AppConstants.PreKeys.LOGIN_USER_PREFIX + userId,info,30 * 24 * 60 * 60);//存Prefer缓存，供离线登陆
//    }

    /**
     * 离线登陆验证
     *
     * @param userId
     * @return UserInfo的实体
     */
    public boolean checkOffline(String orgId, String userId, String password) {
        //与门店数据库匹配  或  与Pref缓存匹配  都算离线验证通过（前提是缓存的密码不能为空，否则别人可以直接空着密码登陆了）
        return Czyda001DbOperate.LoginCheck(orgId, userId, password) /*||
                (userId.equals(PdaMsgConfig.getLogUserId(context))
                        && password.equals(PdaMsgConfig.getPassWord(context)))*/;
//        UserInfo info = (UserInfo)BaseAppManager.preferCache().getAsObject(AppConstants.PreKeys.LOGIN_USER_PREFIX + userId);
//        if(info != null){
//            return info.getPassword().equals(password);
//        }else{
//            return false;
//        }
    }

    /**
     * 离线验证成功后的处理
     *
     * @param userId        ...
     * @param password      ...
     * @param orgId         ...
     *                      （营养顾问，流动营养顾问，店长，督导，兼职）
     * @param isOnlineCheck 是否是远程在线验证成功后？false是离线验证后
     */
    public void initAfterLogin(String orgId, String userId, String password/*, String role*/, boolean isOnlineCheck) {
        if (!isOnlineCheck) {//虽然不知道这是干嘛的，但反正是在离线验证后要做的
            SystemHelper.LoginBC = "Z";
            Pdaparam param = new Pdaparam();
            param.setParamId("2");
            param.setParamName("当前登陆员编号");
            param.setParamValue(userId);
            param.setParamDesc("当前登陆员编号");
            PdaparamOperate.UpdatePDAParam(param);
        }
        saveOrg(orgId.trim());//登陆成功，保存到下拉历史记录
        PdaMsgConfig.setLogUserId(userId.trim(), context);
        PdaMsgConfig.setPassWord(password, context);
//        SystemHelper.IsPartTimeStaff = !TextUtils.isEmpty(role) && role.equals("兼职");//如果是兼职人员，做好标志
//        SystemHelper.IsPartTimeSalesStaff = !TextUtils.isEmpty(role) && role.equals("兼职销售");//如果是兼职人员，做好标志
//        SystemHelper.IsMachineRepairStaff = !TextUtils.isEmpty(role) && role.equals("机修工程师");//如果是机修工程师，做好标志
        Czyda001DbOperate.initUserRoleStatic(orgId, userId);
        SystemHelper.LoginUserId = userId;
        SystemHelper.LoginUserName = Czyda001DbOperate.GetOperateName(SystemHelper.LoginUserId);
        SystemHelper.LoginUserBrandList = Czyda001DbOperate.getUserBrandList(SystemHelper.LoginUserId);
//        if (SystemHelper.LoginUserName.equals("未知")) {
//            //真是信了你的邪。。。一验证门店就把登陆过的User信息给刷没了！所以这里离线情况下如果验证成功，并且无信息，就再写进去！
//            SystemHelper.LoginUserName = PdaMsgConfig.getLoginUserName(context);//获取名字
//            saveUserDb(PdaMsgConfig.getLoginUserName(context),
//                    "", PdaMsgConfig.getLogUserId(context),
//                    PdaMsgConfig.getPassWord(context), "", role);
//        }
        SystemHelper.LoginUserMobile = Czyda001DbOperate.GetOperateMobile(SystemHelper.LoginUserId);//获取电话
        PdaMsgConfig.setLoginUserName(SystemHelper.LoginUserName, context);
//        SystemHelper.LoginOrganName = Store001DbOprate.GetStoreName(SystemHelper.LoginOrganId);
//        PdaMsgConfig.setLoginOrganName(SystemHelper.LoginOrganName, context);
        //真是信了你的邪。。。一验证门店就把登陆过的User信息给刷没了！所以这里离线情况下如果验证成功，并且无信息，就再写进去！
//        if (Czyda001DbOperate.GetOperateName(userId) == "未知") {
////                Czyda001DbOperate.saveOnlineUser(PdaMsgConfig.getLoginUserName(this),
////                        "", PdaMsgConfig.getLogUserId(this),
////                        PdaMsgConfig.getPassWord(this), "")
////            UserInfo info = (UserInfo)BaseAppManager.preferCache().getAsObject(AppConstants.PreKeys.LOGIN_USER_PREFIX + userId);
//            Czyda001DbOperate.saveOnlineUser(info.getUserName(),info.getSex(),userId,password,info.getMobile());
//        }
    }


    /**
     * 在线验证登陆时是否打开新功能介绍（欢迎界面）
     */
    public void checkIsShowWelcome() {
        String lastOpen = PreferHelper.getInstance().getString(AppConstants.PreKeys.NEW_GUIDE_LAST_OPEN_TIME);
        if (!lastOpen.equals(
                new SimpleDateFormat("yyyy-MM-dd", Locale.CHINA).format(new Date()))) {//如果现在日期与上一次打开欢迎界面日期不相同。（一天只开一次）
            OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.WELCOME_IS_SHOW, null,
                    new OkHttpDominator.ResultCallback<WelcomeRes>() {
                        @Override
                        public void onError(Request request, Exception e) {
                            //这种东西验证失败了也无所谓。默认会给个false
                        }

                        @Override
                        public void onResponse(WelcomeRes response) {
                            if (response != null) {
                                getData().putGuideSignal(NewGuideAct.KEY_WELCOME, response.getResult());
                            }
                        }
                    });
        }
    }

    /**
     * 是否显示广告页（职业宣言）
     */
    public void checkIsShowAdPage() {
        String lastOpen = PreferHelper.getInstance().getString(AppConstants.PreKeys.DECLARATION_PAGE_LAST_OPEN_TIME);
        if (!lastOpen.equals(
                new SimpleDateFormat("yyyy-MM-dd", Locale.CHINA).format(new Date()))) {//如果现在日期与上一次打开欢迎界面日期不相同。（一天只开一次）
            OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.DECLARATION_PAGE, null,
                    new OkHttpDominator.ResultCallback<DeclarationRes>() {
                        @Override
                        public void onError(Request request, Exception e) {
                            //这种东西验证失败了也无所谓。默认会给个false
                        }

                        @Override
                        public void onResponse(DeclarationRes response) {
                            if (response != null && response.getResultInfo() != null && response.getResultInfo().getShowFlag()) {//如果要显示宣言
                                getData().putGuideSignal(NewGuideAct.KEY_AD, response.getResultInfo().getShowFlag());
                                getData().addDeclarationLine(response.getResultInfo().getTitle());
                                for (String s : response.getResultInfo().getContent()) {
                                    getData().addDeclarationLine(s);
                                }
                            }
                        }
                    });
        }
    }

    /**
     * 返回已保存在Prefs里的已登录过的门店号集合
     */
    public String[] getOrgsArray() {
        String orgs = PreferHelper.getInstance().getString(AppConstants.PreKeys.SAVED_ORG_CODE);
        String[] array = orgs.split(",");
        return array;
    }

    /**
     * 把验证成功的门店号保存起来
     */
    public void saveOrg(String org) {
        String orgs = PreferHelper.getInstance().getString(AppConstants.PreKeys.SAVED_ORG_CODE);
        if (orgs.contains(org)) {//如果已经保存过该门店号，那就不用执行后面了。
            return;
        }
        String[] array = orgs.split(",");
        StringBuffer sb = new StringBuffer(org);
        int num = array.length < LoginData.MAX_STORE_ORG_NUM ? array.length : LoginData.MAX_STORE_ORG_NUM - 1;
        for (int i = 0; i < num; i++) {//只添加array前4个
            sb.append("," + array[i]);
        }
        PreferHelper.getInstance().setString(AppConstants.PreKeys.SAVED_ORG_CODE, sb.toString());
    }

    /**
     * 获取登陆界面的背景图
     */
    public void downloadBackground(final ImageView iv) {
        final String userId = PdaMsgConfig.getLogUserId(context);
        //先加载上次保存的
        String urlDefault = PreferHelper.getInstance().getString(AppConstants.PreKeys.LAST_LOGIN_BACKGROUND_IMG_URL);
        if (!TextUtils.isEmpty(urlDefault)) {
            Picasso.with(context).load(urlDefault)
                    .placeholder(R.drawable.login_background_default)
                    .error(R.drawable.login_background_default)
                    .into(iv);
        }

        //再通过接口返回，来加载新的
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_LOGIN_IMAGE_URL, null,
                new OkHttpDominator.ResultCallback<LoginImageRes>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(LoginImageRes res) {
                        //后门
                        LoginImageRes resMock = JsonDominator.loadFromSD("LoginImageRes", LoginImageRes.class);
                        if (resMock != null) {
                            res = resMock;
                        }

                        if (res == null || res.getResultInfo() == null) {
                            return;
                        }
                        //按屏幕尺寸来加载不同的图片（以1080为分界）
                        String url = mView.isBigSceen() ? res.getResultInfo().getIMAGE2() : res.getResultInfo().getIMAGE1();
//                        url = "http://www.xsjjys.com/data/out/181/WHDQ-513199257.jpg";
                        if (TextUtils.isEmpty(url)) {
                            return;
                        }
                        Picasso.with(context).load(url)
                                .placeholder(R.drawable.login_background_default)
                                .error(R.drawable.login_background_default)
                                .into(iv);
                        PreferHelper.getInstance().setString(AppConstants.PreKeys.LAST_LOGIN_BACKGROUND_IMG_URL, url);

                    }
                });
    }

    /**
     * 安卓6.0动态权限适配
     */
    public void requestPermissions() {
//        if (Build.VERSION.SDK_INT  == Build.VERSION_CODES.M){
//            FuckGoogleAdaptUtil.android6WriteSettings(context);
//        }
        FuckGoogleAdaptUtil.grantPermissions((Activity) mView);
    }
}
