package com.urovo.activity.machinerepairact.model

import android.view.View
import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.urovo.activity.machinerepairact.repository.ManageCallRepo
import com.urovo.wugumofang.model.LocationEntity
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import kotlinx.coroutines.launch

/**
 * @author Xiep on 2020-6-4.
 */
class EngineerManageModel(val repo: ManageCallRepo) : ViewModel() {

    val MAX_PHOTO_NUM = 5

    /**当前位置*/
    var currLocation = ObservableField<LocationEntity>()

    var showProgressText = MutableLiveData<String>()

    var isFinish = MutableLiveData<Boolean>()
    /**
     * 照片路径缓存
     */
    var photoPool = mutableListOf<String>()

    /**弹窗文字*/
    var alertDialogState = MutableLiveData<String>()

    /**提交*/
    fun onClickSubmit(view: View) {
        viewModelScope.launch {
            showProgressText.value = "正在校验地理位置..."
            var locationCorrect = "n"
            try {
                val checkRes = repo.checkLocation(SystemHelper.LoginUserId, SystemHelper.LoginOrganId, currLocation.get()!!)
                locationCorrect = when (checkRes.msgCode) {
                    "0" -> "y"
                    else -> "n"
                }
            } catch (e: Exception) {

            }
            showProgressText.value = ""
            alertDialogState.value = locationCorrect
        }
    }

    /**真正提交 */
    fun realSubmit(lampId: String,locationCorrect: String) = viewModelScope.launch {
        //校验通过，开始上传
        try {
            var picUrl = ""
            if (!photoPool.isEmpty()) {
                showProgressText.value = "正在上传图片..."
                picUrl = repo.submitPhotos2Qiniu(photoPool)
            }
//            Log.i("SasukeLog", "图片上传完成")
            if(picUrl.isNotEmpty()) {
                showProgressText.value = "正在提交灯亮故障数据..."
                val result = repo.submitLogoLampRepair(
                        lampId,
                        picUrl,
                        locationCorrect)//位置是否正常,y正确 ,n异常
                if (result.msgCode == 0) {
                    MyToast.show("提交成功")
                    isFinish.value = true
                    showProgressText.value = ""
                }
            }else{
                MyToast.show("提交失败，请上传处理后的现场照片！")
            }
        } catch (e: Exception) {
            e.printStackTrace()
            MyToast.show(e.message)
            showProgressText.value = ""
        }
    }
}