package com.urovo.activity.machinerepairact.model

import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.urovo.activity.machinerepairact.net.EngineerMyStoresRes
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.coroutines.launch

/**
 * @author Sasuke on 2019-7-31 0031.
 */
class EngineerMyStoreModel(val repo: MachineCallRepo) : ViewModel() {
    /**我的门店数*/
    var storeCount = ObservableField<Int>(0)

    /**我的门店中的有销量的门店数*/
    var storeHasSaleCount = ObservableField<Int>(0)

    /**我的门店缓存*/
    var myStoresCache = MutableLiveData<List<EngineerMyStoresRes.ResultInfo>>()

    /**我的门店缓存--用于显示*/
    var showListCache: List<EngineerMyStoresRes.ResultInfo> = listOf()
        get() {
            return myStoresCache.value?.filter { searchWord.isNullOrBlank() || searchWord in it.storeNo || searchWord in it.storeName} ?: listOf()
        }

    /**loadwait指示*/
    var loadWaitMsg = MutableLiveData<Int>().apply { value = -1 }

    /**搜索关键词*/
    var searchWord = ""

    /**
     * 获取我的门店
     */
    fun queryMyStores(storeNameFilter: String?) = viewModelScope.launch {
        loadWaitMsg.value = LoadWaitDominator.LOADING
        try {
            myStoresCache.value = repo.queryMyStores(SystemHelper.LoginUserId)
            storeCount.set(myStoresCache.value?.size)
            storeHasSaleCount.set(myStoresCache.value?.count { it.isSaledLastMonth == 1 })
            //把数量算完了，再执行名称过滤
            storeNameFilter?.let {
                myStoresCache.value = myStoresCache.value?.filter { it.storeName == storeNameFilter }
            }
            //无数据
            if (storeCount.get() == 0) {
                loadWaitMsg.value = LoadWaitDominator.NO_DATA
            }else{
                loadWaitMsg.value = -1
            }
        } catch (e: Exception) {
//            MyToast.show(e.message)
            loadWaitMsg.value = LoadWaitDominator.NET_ERROR
        }
    }
}