package com.urovo.activity.machinerepairact.model

import android.app.Application
import android.view.View
import android.widget.CompoundButton
import android.widget.RadioGroup
import android.widget.TextView
import androidx.databinding.ObservableArrayList
import androidx.databinding.ObservableField
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.urovo.activity.machinerepairact.net.MachinRepairHistoryRes
import com.urovo.activity.machinerepairact.net.MachinRepairerRes
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.wugumofang.R
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.wgmf.trade.utils.IntentUtil
import kotlinx.coroutines.launch
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.HashSet

/**
 * @author Sasuke on 2019-04-26 0026.
 */
class MachineCallModel(val repo: MachineCallRepo, val app: Application) : AndroidViewModel(app) {
    val MAX_PHOTO_NUM = 5
    val DEALED_LABEL = 0//已处理标签
    val UN_DEALED_LABEL = -1//未处理标签

    var isFinish = MutableLiveData<Boolean>().apply { value = false }

    var showProgressText = MutableLiveData<String>().apply { value = "" }

    /**机修工单号  jx + 门店号 + yyMMddHHmmss,例jx0110074190422150633*/
    val mrAppNo by lazy { "jx${SystemHelper.LoginOrganId}${SimpleDateFormat("yyMMddHHmmss").format(Date())}" }

    var storeName = SystemHelper.LoginOrganName

    var staffName = SystemHelper.LoginUserName

    /**申请人联系电话*/
    var staffPhone: ObservableField<String> = ObservableField("")

    var tabTexts = listOf("呼叫", "记录")

    /**机修人员信息*/
    var repairerRes = MutableLiveData<MachinRepairerRes?>()

    /**申请事由*/
    var applyReason =
            MutableLiveData<HashSet<String>>().apply { value = HashSet() }

    /**其它申请事由*/
//    var applyReasonOther = ""

    /**是否勾选其它申请事由*/
    var isReasonOtherCheck = false

    /**故障描述*/
    var descri = ""

    /**
     * 照片路径缓存
     */
    var photoPool = mutableListOf<String>()

    /**
     * 机修记录（eg.2019年03月）
     */
    var historyDate = MutableLiveData<String>().apply { value = SimpleDateFormat("yyyy年MM月").format(Date()) }

    /**
     * 工单状态,0待处理,1待评价,2已评价
     */
    var historyLabel = MutableLiveData<Int>()

    /**
     * 请求到的机修记录数据
     */
    var historyCache: ObservableArrayList<MachinRepairHistoryRes.ResultInfo> = ObservableArrayList()

    var historyShow: ObservableArrayList<MachinRepairHistoryRes.ResultInfo> = ObservableArrayList()

    /**
     * 已处理，未处理标签变化监听
     */
    fun onStatusCheckChanged(group: RadioGroup, rId: Int) {
        when (rId) {
            R.id.dealed -> historyLabel.value = DEALED_LABEL
            else -> historyLabel.value = UN_DEALED_LABEL
        }
    }

    /**
     * 已选查询日期之后的处理
     */
    fun historyDateSelected(year: Int, month: Int) {
        val today = SimpleDateFormat("yyyy年MM月").format(Date())
        val selectDay = "${year}年${String.format("%02d", month + 1)}月"
        if (today.compareTo(selectDay) < 0) {
            MyToast.show("不能超过当月")
            return
        }
        historyDate.value = selectDay
    }

    /**
     * 校验数据（图片为选填，不需要校验）
     */
    fun checkData(): Boolean {
        if (staffPhone.get().isNullOrBlank() || staffPhone.get()?.length != 11){
            MyToast.show("请输入正确的联系电话！")
            return false
        }
        if (repairerRes.value?.resultInfo?.userName.isNullOrBlank() ||
                repairerRes.value?.resultInfo?.phone.isNullOrBlank() ||
                repairerRes.value?.resultInfo?.userNo.isNullOrBlank()) {
            MyToast.show("缺少机修人员信息")
            return false
        }
        if (applyReason.value.isNullOrEmpty()) {
            MyToast.show("申请事由为必填项")
            return false
        }
//        if (isReasonOtherCheck && applyReasonOther.isNullOrBlank()){
//            MyToast.show("请输入“其它”申请事由")
//            return false
//        }
        if (descri.isNullOrEmpty()) {
            MyToast.show("故障描述为必填项")
            return false
        }
        return true
    }

    /**
     * 获取修复人员信息
     */
    fun getRepairerInfo(storeNo: String) = viewModelScope.launch {
        try {
            repairerRes?.value = repo.getRepairerInfo(storeNo)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 查询申请人的电话号码
     */
    fun getStaffPhone(userNo: String) = viewModelScope.launch {
        try {
            staffPhone.set(repo.queryStaffPhoneMr(userNo))
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     *  点击打电话
     */
    fun onPhoneClick(view: View) {
        var phoneNum = (view as TextView).text.toString()
        if (phoneNum.isEmpty()) return
        IntentUtil.callPhone(phoneNum, getApplication())
    }

    /**
     * CheckBox监听
     */
    fun onCheckedChanged(btnView: CompoundButton, isChecked: Boolean) {
        if (isChecked) {
            applyReason.value?.add(btnView.text.toString())
        } else {
            applyReason.value?.remove(btnView.text.toString())
        }
    }

    /**
     * 提交机修工单
     */
    fun submitMRepairBill() = viewModelScope.launch {
        //申请事由
        var reasonChecked = ""
        applyReason.value?.forEach { reasonChecked += "${it}," }
        //附加上其它事由
//        if (isReasonOtherCheck) reasonChecked += applyReasonOther
        //去掉末尾逗号
        if (reasonChecked.endsWith(",")) {
            reasonChecked = with(reasonChecked) {
                substring(0, length - 1)
            }
        }


        //传参整理
        val params = hashMapOf("mrAppNo" to mrAppNo,
                "storeNo" to SystemHelper.LoginOrganId,
                "storeName" to SystemHelper.LoginOrganName,
                "userNo" to SystemHelper.LoginUserId,
                "userPhone" to staffPhone.get()?.trim(),
                "userName" to SystemHelper.LoginUserName,
                "mrOperatiorNo" to repairerRes.value?.resultInfo?.userNo,
                "mrOperatiorName" to repairerRes.value?.resultInfo?.userName,
                "mrOperatiorPhone" to repairerRes.value?.resultInfo?.phone,
                "applyReason" to reasonChecked,
                "problemDescription" to descri)
        //开始提交
        try {
            showProgressText.value = "正在上传图片"
            val qiniuUrls = repo.submitPhotos2Qiniu(photoPool)
            params.put("problemPicUrl", qiniuUrls)//已经经过转换变成逗号分隔的纯字符串了。
            //正式上传数据
            showProgressText.value = "正在提交工单"
            val res = repo.submitMRepairBill(params)
            MyToast.show(res?.msgInfo)
            if (res?.msgCode == 0) {
                isFinish.value = true//关闭当前Activity
            }
            showProgressText.value = ""
        } catch (e: Exception) {
            MyToast.show(e.message)
            showProgressText.value = ""
        }
    }


    /**
     * 查询机修记录
     */
    fun getRepairHistory(byNet: Boolean) = viewModelScope.launch {
        try {
            //如果要从网络查，而不是从已接收过的缓存查
            if (byNet) {
                historyCache.run { clear();addAll(repo.queryHistory(SystemHelper.LoginUserId, historyDate.value!!)) }
            }
            //缓存好了全部数据，再用标签来区分
            historyShow.run {
                clear()
                addAll(historyCache.filter {
                    when (historyLabel.value) {
                        UN_DEALED_LABEL -> it?.status == 0//待处理
                        else -> it?.status != 0//已处理
                    }
                })
            }
        } catch (e: Exception) {
            historyCache.clear()
            MyToast.show(e.message)
        }


    }


}

