package com.urovo.activity.machinerepairact.model

import android.view.View
import android.widget.RadioGroup
import android.widget.TextView
import androidx.databinding.ObservableArrayList
import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.taobao.weex.WXEnvironment.getApplication
import com.urovo.activity.machinerepairact.net.MachinRepairHistoryRes
import com.urovo.activity.machinerepairact.net.MachineDetailRes
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.view.MachineRepairDetailAct
import com.urovo.wugumofang.R
import com.urovo.wugumofang.net.NetHelper
import com.urovo.wugumofang.utils.MyToast
import com.wgmf.trade.utils.IntentUtil
import kotlinx.coroutines.launch

/**
 * @author Sasuke on 2019/5/2.
 */
class MachineDetailModel(val repo: MachineCallRepo) : ViewModel() {
    /**从上个页面传过来的网络返回RES*/
    var intentRes = MutableLiveData<MachinRepairHistoryRes.ResultInfo>()

    /**查询工单的返回*/
    var detailResultInfo = MutableLiveData<MachineDetailRes.ResultInfo>()

    /**营养顾问的图片*/
    var beforePicListShow = ObservableArrayList<String>()

    /**施工后，即机修人员图片*/
    var afterPicListShow = ObservableArrayList<String>()

    /**客户评价 未评价0 ,差1,中2,好3 ,这个主要用来记录更改后的*/
    var ratingTemp: ObservableField<Int> = ObservableField<Int>()

    /**建议*/
    var suggestionContent: String = ""

    var finishBy = MutableLiveData<Int>().apply { value = MachineRepairDetailAct.NOT_CHANGED }

    var showProgressText = MutableLiveData<String>().apply { value = "" }

    fun getTitleText(): String {
        return when (intentRes.value?.status) {
            0 -> "待处理工单"
            1 -> "待评价工单"
            2 -> "已评价工单"
            else -> "null"
        }
    }

    /**
     *  请求工单
     */
    fun queryDetail() = viewModelScope.launch {
        try {
            detailResultInfo.value = repo?.queryMachineDetail(intentRes.value!!.mroId)?.resultInfo
        } catch (e: Exception) {
            MyToast.show(e.message)
        }
    }


    /**
     *  点击打电话
     */
    fun onPhoneClick(view: View) {
        var phoneNum = (view as TextView).text.toString()
        if (phoneNum.isEmpty()) return
        IntentUtil.callPhone(phoneNum, getApplication())
    }

    /**
     * 评价的变换监听
     */
    fun OnCheckedChangeListener(radioGroup: RadioGroup, rId: Int) {
        ratingTemp.set(when (rId) {
            R.id.goodComment -> 3
            R.id.normalComment -> 2
            R.id.badComment -> 1
            else -> 0
        })
    }

    /**
     * 提交评价
     */
    fun submitRating(view: View) {
        if (!NetHelper.IsHaveInternet(view.context)){
            MyToast.show("网络不好,请稍后再试!")
            return
        }
        if (ratingTemp.get() == 0) {
            MyToast.show("请选择服务评价")
            return
        }
        if (ratingTemp.get() != 3 && suggestionContent.isBlank()) {
            MyToast.show("我们需要你宝贵的意见和建议")
            return
        }
        viewModelScope.launch {
            showProgressText.value = "正在提交评价..."
            try{
                var res = repo.submitRepairDetailRating(detailResultInfo.value?.mroId,ratingTemp.get(),suggestionContent)
                showProgressText.value = ""
                MyToast.show(res?.msgInfo)
                if (res?.msgCode == 0){
                    finishBy.value = MachineRepairDetailAct.HAS_CHANGED
                }
            }catch (e: Exception){
                MyToast.show(e.message)
            }
        }

    }
}