package com.urovo.activity.machinerepairact.repository

import androidx.databinding.ObservableArrayList
import androidx.lifecycle.MutableLiveData
import com.google.gson.Gson
import com.urovo.activity.machinerepairact.net.*
import com.urovo.activity.machinetrip.net.PatrolReportRes
import com.urovo.wgmf.tool.netUtil.QiNiuDominator
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.JsonDominator
import com.urovo.wugumofang.model.LocationEntity
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.Base3Response
import com.urovo.wugumofang.response.BaseOldResponse
import com.urovo.wugumofang.response.BaseSimpleResponse
import com.wgmf.trade.model.bean.UserHeadResBean
import okhttp3.Request
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException
import kotlin.coroutines.suspendCoroutine

/**
 * @author Sasuke on 2019/4/27.
 */
class ManageCallRepo {
    /**
     * 把图片传到七牛
     */
    suspend fun submitPhotos2Qiniu(photoPaths: MutableList<String>) = suspendCoroutine<String> { conti ->
        if (photoPaths.size == 0) {
            conti.resume("")
            return@suspendCoroutine
        }
        QiNiuDominator.uploadImages(photoPaths,
                { qiniuUrl ->
                    //CompleteCallback
                    //照片七牛url组装
                    var packUrls = ""
                    qiniuUrl.forEach { packUrls += "${it}," }
                    if (packUrls.isNotEmpty()) packUrls = packUrls.substring(0, packUrls.length - 1)
                    conti.resume(packUrls)
                },
                { str ->
                    //FailedCallback
                    conti.resumeWithException(Exception(str))
                }, false)
    }

    /**
     * 验证地理位置
     */
    suspend fun checkLocation(userId: String, storeId: String, location: LocationEntity?) = suspendCoroutine<BaseOldResponse> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.SIGN_IN_OR_OUT_LOCATION_CHECK,
                mapOf("userId" to userId,
                        "storeId" to storeId,
                        "requestType" to "1",
                        "lng" to location?.longitude.toString(),
                        "lat" to location?.latitude.toString(),
                        "userType" to "2",
                        "address" to location?.address,
                        "type" to "1",
                        "machine" to "1",
                        "signTime" to ""), object : OkHttpDominator.ResultCallback<BaseOldResponse>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: BaseOldResponse?) {
                res?.let {
                    conti.resume(res)
                }
            }
        })
    }

    /**
     *
    post /machineRepair/submitLogoLampRepair 提交灯箱维修状态
     */
    suspend fun submitLogoLampRepair(lampId: String, picUrl: String, locationCorrect: String) = suspendCoroutine<Base3Response> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.SUBMIT_SUBMIT_LOGO_LAMP_REPAIR,
                mapOf("lampId" to lampId,
                        "handleUrl" to picUrl,
                        "locationCorrect" to locationCorrect), object : OkHttpDominator.ResultCallback<Base3Response>() {
            override fun onError(request: Request?, e: java.lang.Exception) {
                e.printStackTrace()
                conti.resumeWithException(e)
            }

            override fun onResponse(res: Base3Response) {
                res?.let {
                    if (res.msgCode != 0) {
                        conti.resumeWithException(Exception(res.msgInfo))
                        return
                    }
                    conti.resume(res)
                }
            }
        })
    }
}