package com.urovo.activity.machinerepairact.view

import android.os.Bundle
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.GridLayoutManager
import com.urovo.activity.machinerepairact.model.EngineerDetailFactory
import com.urovo.activity.machinerepairact.model.EngineerDetailModel
import com.urovo.activity.machinerepairact.net.UndealRepairRes
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.ui.MachinePhotoGridAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.EngineerBillDetailActBinding
import com.urovo.wugumofang.view.LoadWaitDominator

/**
 * @author Sasuke on 2019-05-07 0007.
 */
class EngineerBillDetailAct : BaseActivity() {
    companion object {
        //resultCode
        const val HAS_CHANGED = -1080 //改变了数据，要求返回后刷新。
        const val NOT_CHANGED = -1081//没改变数据，返回不需要刷新。
        const val REQUEST_CODE = 1080//没改变数据，返回不需要刷新。

        const val INTENT_RES = "INTENT_RES"
    }

    val detailModel by lazy { ViewModelProviders.of(this, EngineerDetailFactory(MachineCallRepo())).get(EngineerDetailModel::class.java) }
    val detailBinding by lazy { DataBindingUtil.setContentView<EngineerBillDetailActBinding>(this, R.layout.engineer_bill_detail_act) }

    var loadWait: LoadWaitDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
        detailModel.getBillDetail(detailModel.intentCache.value?.mroId)
    }

    fun bind() {
        detailModel.intentCache.value = intent.getSerializableExtra(INTENT_RES) as UndealRepairRes.ResultPack.ResultInfo
        detailBinding.hasDealed = detailModel.intentCache.value?.status != 0
        detailBinding.titleText = if (detailModel.intentCache.value?.status == 0) "待处理工单" else "已处理工单"
        detailBinding.model = detailModel

    }

    fun init() {
        loadWait = LoadWaitDominator(this, detailBinding.loadWaitContainer)

        detailModel.loadWaitMessage.observe(this, Observer {
            when (it) {
                -1 -> loadWait?.dismissAll()
                LoadWaitDominator.LOADING -> loadWait?.show(LoadWaitDominator.LOADING)
                LoadWaitDominator.NET_ERROR -> loadWait?.show(LoadWaitDominator.NET_ERROR)
                LoadWaitDominator.NO_DATA -> loadWait?.show(LoadWaitDominator.NO_DATA)
            }
        })

        //请求返回缓存
        detailModel.resCache.observe(this, Observer { result ->
            detailBinding.data = detailModel.resCache.value
            result.picBeforeOperation?.let { detailModel.beforePicListShow.run { clear();addAll(it) } }//刷新图片数据
            result.picAfterOperation?.let { detailModel.afterPicListShow.run { clear();addAll(it) } }//刷新图片数据
            //刷新
            detailBinding.problemPhotoRv.adapter?.notifyDataSetChanged()
            detailBinding.repairPhotoRv.adapter?.notifyDataSetChanged()
        })

        //图片
        arrayOf(detailBinding.problemPhotoRv, detailBinding.repairPhotoRv).forEach {
            it.layoutManager = object : GridLayoutManager(this@EngineerBillDetailAct, 3) {
                override fun canScrollVertically(): Boolean {//不允许上下滑动
                    return false
                }
            }
        }
        detailBinding.problemPhotoRv.adapter = MachinePhotoGridAdapter(detailModel.beforePicListShow)
        detailBinding.repairPhotoRv.adapter = MachinePhotoGridAdapter(detailModel.afterPicListShow)
    }

}