package com.urovo.activity.machinerepairact.view

import android.app.AlertDialog
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.provider.Settings
import android.view.View
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.urovo.activity.machinerepairact.model.EngineerManageFactory
import com.urovo.activity.machinerepairact.model.EngineerManageModel
import com.urovo.activity.machinerepairact.repository.ManageCallRepo
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.EngineerManageActBinding
import com.urovo.wugumofang.system.LocationUtil
import com.urovo.wugumofang.ui.DensityUtil
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.notNullAndEmpty

/**
 * @author Xiep on 2020-6-4.
 */
class EngineerManageAct : BaseActivity() {

    private var mLocationTool:LocationUtil? = null

    companion object {
        @JvmStatic
        fun launch(
            context: Context,
            storeNoFilter: String,
            lampIdFilter: String,
            addressFilter: String,
            storeNameFilter: String
        ) {
            context.startActivity(Intent(context, EngineerManageAct::class.java).apply {
                putExtra("storeNameFilter", storeNameFilter)
                putExtra("storeNoFilter", storeNoFilter)
                putExtra("lampIdFilter", lampIdFilter)
                putExtra("addressFilter", addressFilter)
            })
        }
    }

    val binding by lazy {
        DataBindingUtil.setContentView<EngineerManageActBinding>(
            this,
            R.layout.engineer_manage_act
        )
    }

    val patrolModel by lazy {
        ViewModelProviders.of(this, EngineerManageFactory(ManageCallRepo()))
            .get(EngineerManageModel::class.java)
    }

    var picker: PhotoPickDominator? = null
    var isShowLocateToast = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()

    }

    fun bind() {
        binding.titleText = "灯亮故障处理"
        binding.model = patrolModel
        binding.storeName = intent.getStringExtra("storeNameFilter")
        binding.storeAddress = intent.getStringExtra("addressFilter")
        binding.titleLayout.confirm.apply {
            visibility = View.VISIBLE
            //重新定位按钮样式
            setText("重新定位")
            val drawable = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                getResources().getDrawable(R.drawable.ic_location_on_white_24dp, null)
            } else {
                getResources().getDrawable(R.drawable.ic_location_on_white_24dp)
            }
            drawable.setBounds(
                0,
                0,
                DensityUtil.dip2px(this@EngineerManageAct, 15f),
                DensityUtil.dip2px(this@EngineerManageAct, 22f)
            )
            setCompoundDrawables(drawable, null, null, null)
            //点击监听
            setOnClickListener {
                patrolModel.currLocation.set(null)
                startLocation()
            }
        }

        patrolModel.showProgressText.observe(this, Observer {
            if (it.isBlank()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this, "提示", it, false)
            }
        })

        patrolModel.isFinish.observe(this, Observer {
            if (it) {
                finish()
            }
        })
        //提交前的提醒
        patrolModel.alertDialogState.observe(this, Observer { locationCorrect ->
            AlertDialog.Builder(this).setTitle("${if (locationCorrect == "n") "异常" else ""}提醒")
                .setMessage(if (locationCorrect == "n") "当前位置在门店范围外,将会标记为位置异常,确定要提交吗?" else "确定要提交灯亮故障？")
                .setPositiveButton(
                    "确定",
                    DialogInterface.OnClickListener { dialogInterface, i ->
                        intent.getStringExtra("lampIdFilter")
                            ?.let { patrolModel.realSubmit(it, locationCorrect) }
                    })
                .setNegativeButton("取消", null)
                .show()
        })
    }

    fun init() {
        picker = PhotoPickDominator(this, patrolModel.photoPool)
        //添加图片
        binding.cameraAdd.setOnClickListener {
            when (binding.photoGroup.childCount) {
                patrolModel.MAX_PHOTO_NUM + 1 -> showMsg("最多添加 ${patrolModel.MAX_PHOTO_NUM} 张图片")//加1是里面还有一个相机添加按钮
                else -> picker?.showDialog()
            }
        }
        startLocation()
    }

    private fun startLocation() {
        //上传地理位置
        mLocationTool = LocationUtil(
            this
        ) { entity ->
            if (entity?.address.notNullAndEmpty()) {
                if (isShowLocateToast) showMsg("成功获取地理信息， 位置在: \n ${entity?.address}")
                patrolModel.currLocation.set(entity)
                isShowLocateToast = true
            }else{
                if (isShowLocateToast) {
                    AlertDialog.Builder(this@EngineerManageAct)
                        .setMessage("获取地理信息失败，请检查设置定位权限")
                        .setNegativeButton("算了", null)
                        .setPositiveButton(
                            "去开启权限",
                            { dialogInterface, i -> startActivity(Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)) })//跳转系统定位设置
                        .show()
                }
                isShowLocateToast = true
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val photoPath = picker?.let { it.onPhotoPathResult(this, requestCode, resultCode, data) };
        picker?.addPhoto2Group(
            photoPath,
            DensityUtil.dip2px(this, 60f),
            DensityUtil.dip2px(this, 60f),
            binding.photoGroup
        )
    }

}