package com.urovo.activity.machinerepairact.view

import android.app.AlertDialog
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.provider.Settings
import android.view.View
import android.widget.AdapterView
import android.widget.ArrayAdapter
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.urovo.activity.machinerepairact.model.EngineerSubmitFactory
import com.urovo.activity.machinerepairact.model.EngineerSubmitModel
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.EngineerSubmitActBinding
import com.urovo.wugumofang.system.LocationUtil
import com.urovo.wugumofang.ui.DensityUtil
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.notNullAndEmpty


/**
 * @author Sasuke on 2019-05-08 0008.
 */
class EngineerSubmitAct : BaseActivity() {
    companion object {
        const val INTENT_STORE_NAME = "INTENT_STORE_NAME"
        const val INTENT_STORE_LOCATION = "INTENT_STORE_LOCATION"
        const val INTENT_MROID = "INTENT_MROID"
    }

    private var mLocationTool: LocationUtil? = null

    val binding by lazy { DataBindingUtil.setContentView<EngineerSubmitActBinding>(this, R.layout.engineer_submit_act) }
    val model by lazy { ViewModelProviders.of(this, EngineerSubmitFactory(MachineCallRepo())).get(EngineerSubmitModel::class.java) }
    var isShowLocateToast = false
    var picker: PhotoPickDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        setListeners()
        startLocation()

    }

    fun setListeners() {
        binding.waySpinner.onItemSelectedListener = SpinnerSelectedListener()
    }


    fun bind() {
        picker = PhotoPickDominator(this, listOf())
        model.storeName.set(intent.getStringExtra(INTENT_STORE_NAME))
        model.storeLocation.set(intent.getStringExtra(INTENT_STORE_LOCATION))
        model.mroId.value = intent.getIntExtra(INTENT_MROID, -1)
        binding.waySpinner.adapter = createArrayAdapter(model.spinnerList)

        model.handleWay.observe(this, Observer {
            model.isShowPhotoSelect.set(it == "上门解决")
            model.isShowDoneTime.set(it != "其他")
        })

        binding.model = model
        binding.titleText = "工单处理"
        binding.titleLayout.confirm.apply {
            visibility = View.VISIBLE
            //重新定位按钮样式
            setText("重新定位")
            val drawable = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                getResources().getDrawable(R.drawable.ic_location_on_white_24dp, null)
            } else {
                getResources().getDrawable(R.drawable.ic_location_on_white_24dp)
            }
            drawable.setBounds(0, 0, DensityUtil.dip2px(this@EngineerSubmitAct, 15f), DensityUtil.dip2px(this@EngineerSubmitAct, 22f))
            setCompoundDrawables(drawable, null, null, null)
            //点击监听
            setOnClickListener {
                model.currLocation.set(null)
                startLocation()
            }
        }
        /**添加照片点击监听 */
        arrayOf(binding.addGenbaPhoto,
                binding.addSenroPhoto,
                binding.addSeibiPhoto,
                binding.addMisePhoto).forEach {
            it.setOnClickListener {
                model?.currAdd?.value = it.id
                picker?.showDialog()
            }
        }

        model.successFinish.observe(this, Observer {
            if (it) {
                startActivity(Intent(this@EngineerSubmitAct, EngineerBillsAct::class.java).apply {
                    setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                    putExtra("TO_TAB_POS", 1)//跳转回去之后，让它默认显示为”已处理“标签。
                })
//                EventBus.getDefault().post(EngineerBillsAct.EventChangeToDeal())
            }
        })

        model.showProgressText.observe(this, Observer {
            if (it.isBlank()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this, "提示", it, false)
            }
        })
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val photoPath = picker?.onPhotoPathResult(this, requestCode, resultCode, data)
        photoPath?.takeIf { it.isNotBlank() }?.let {
            when (model.currAdd.value) {
                R.id.addGenbaPhoto -> model.photoGenba
                R.id.addSenroPhoto -> model.photoSenro
                R.id.addSeibiPhoto -> model.photoSeibi
                R.id.addMisePhoto -> model.photoMise
                else -> mutableListOf<String>()
            }.add(0, photoPath)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        picker?.deletePicCache()
    }

    fun createArrayAdapter(array: Array<String>): ArrayAdapter<String> {
        return ArrayAdapter(this@EngineerSubmitAct,
                R.layout.gray_spinner_dropdown_item,
                array)
    }

    private fun startLocation() {
        //上传地理位置
        mLocationTool = LocationUtil(
            this
        ) { entity ->
            if (entity?.address.notNullAndEmpty()) {
                if (isShowLocateToast) showMsg("成功获取地理信息， 位置在: \n ${entity?.address}")
                model.currLocation.set(entity)
                isShowLocateToast = true
            }else{
                if (isShowLocateToast) {
                    AlertDialog.Builder(this@EngineerSubmitAct)
                        .setMessage("获取地理信息失败，请检查设置定位权限")
                        .setNegativeButton("算了", null)
                        .setPositiveButton(
                            "去开启权限",
                            { dialogInterface, i -> startActivity(Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)) })//跳转系统定位设置
                        .show()
                }
                isShowLocateToast = true
            }
        }
    }

    inner class SpinnerSelectedListener : AdapterView.OnItemSelectedListener {
        override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
            model.handleWay.value = model.spinnerList[position]
        }

        // 何も選択されなかった時の動作
        override fun onNothingSelected(parent: AdapterView<*>) {}
    }
}