package com.urovo.activity.machinerepairact.view

import android.content.Context
import android.content.Intent
import android.os.Bundle
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.machinerepairact.model.MachineEngineerFactory
import com.urovo.activity.machinerepairact.model.MachineEngineerModel
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.ui.LightNoProblemAdapter
import com.urovo.activity.machinerepairact.ui.LightProblemAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.LightProblemStoresActBinding
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.light_problem_stores_act.*
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author Sasuke on 2019-8-8 0008.
 * 门店灯亮故障
 *
 */
class LightProblemStoresAct : BaseActivity() {
    companion object {
        @JvmStatic
        fun launch(context: Context) {
            context.startActivity(Intent(context, LightProblemStoresAct::class.java))
        }
    }

    val binding by lazy {
        DataBindingUtil.setContentView<LightProblemStoresActBinding>(this, R.layout.light_problem_stores_act)
    }

    val lightModel by lazy {
        ViewModelProviders.of(this, MachineEngineerFactory(MachineCallRepo())).get(MachineEngineerModel::class.java)
    }

    var loadWait: LoadWaitDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
    }

    override fun onResume() {
        super.onResume()
        lightModel.queryLightProblem()
    }
    fun bind() {
        binding.titleText = "门店灯亮故障"
        val halfYear = if (Calendar.getInstance().get(Calendar.MONTH) >= 6) "下半年" else "上半年"
        binding.yearMonth = "${SimpleDateFormat("yyyy年", Locale.CHINA).format(Date())}${halfYear}"

        lightModel.lightProblemCache.observe(this, androidx.lifecycle.Observer {
            binding.arrangeNum = it.arrangeNum.toString()
            binding.reportNum = it.reportNum.toString()
            binding.notlightNum = it.notlightNum.toString()
            binding.lightNum = it.lightNum.toString()
            binding.reportRate = it.reportRate.toString() + "%"
            binding.lightRate = it.lightRate.toString() + "%"
            storesRv.adapter = LightProblemAdapter(this,it.detailList)
            storesRv2.adapter = LightNoProblemAdapter(this,it.notSubmitList)
        })
        //显示提示信息
        lightModel.loadWaitMsg.observe(this, androidx.lifecycle.Observer {
            loadWait?.show(it)
        })
    }

    fun init() {
        loadWait = LoadWaitDominator(this, containerLayout)
        storesRv.layoutManager = LinearLayoutManager(this)
        storesRv2.layoutManager = LinearLayoutManager(this)
    }
}