package com.urovo.activity.machinerepairact.view

import android.content.Context
import android.content.Intent
import android.os.Bundle
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.urovo.activity.machinerepairact.model.PatrolListFactory
import com.urovo.activity.machinerepairact.model.PatrolListModel
import com.urovo.activity.machinerepairact.repository.MachineCallRepo
import com.urovo.activity.machinerepairact.ui.PatrolListItemsAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.PatrolActListBinding
import com.urovo.wugumofang.view.LoadWaitDominator

/**
 * 巡店检查计划表
 */
class PatrolActList : BaseActivity() {
    companion object {
        @JvmStatic
        fun launch(context: Context) {
            context.startActivity(Intent(context, PatrolActList::class.java))
        }
    }

    val binding by lazy { DataBindingUtil.setContentView<PatrolActListBinding>(this, R.layout.patrol_act_list) }

    val patrolModel by lazy { ViewModelProviders.of(this, PatrolListFactory(MachineCallRepo())).get(PatrolListModel::class.java) }

    var loadWait: LoadWaitDominator? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        initTitle()
        init()
        patrolModel.queryItems()
    }

    fun bind() {
        binding.titleText = "巡店检查"
        binding.model = patrolModel

        //刷新列表数据
        patrolModel.itemsData.observe(this, Observer {
            if (it.isNullOrEmpty()) {
                loadWait?.show(LoadWaitDominator.NO_DATA)
                return@Observer
            }
            binding.exListView.setAdapter(PatrolListItemsAdapter(this@PatrolActList, it))
        })
    }

    fun init() {

    }
}