package com.urovo.activity.machinetrip.model

import android.app.Application
import android.view.View
import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.google.gson.GsonBuilder
import com.urovo.activity.common.DatePickerYearMonthDialog
import com.urovo.activity.machinetrip.net.TripRecordNewEntity
import com.urovo.activity.machinetrip.net.TripRecordRes
import com.urovo.activity.machinetrip.repo.MachineTripRepo
import com.urovo.activity.machinetrip.view.MachineTripAct
import com.urovo.wugumofang.data.ListDominator
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.coroutines.launch
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

/**
 * @author Sasuke on 2019-10-23 0023.
 */
class MachineTripModel(val app: Application, val repo: MachineTripRepo) : ViewModel() {
    /**默认可以编辑*/
    var editMode = ObservableField<Boolean>(true)

    /**页面类型（新建？记录查询+修改？）*/
    var activityType = ""

    /**出差记录入口列表*/
    var tripRecordsCache = MutableLiveData<List<TripRecordRes.ResultInfo>>()

    /**修改记录模式下，不显示加减号按钮*/
    var isAddBtnVisible = true

    /**由日期范围类型获取日期范围*/
    fun getDateRange(month: Int) = when (month) {
        1 -> "01月20日 至 02月20日"
        2 -> "02月20日 至 03月20日"
        3 -> "03月20日 至 04月20日"
        4 -> "04月20日 至 05月20日"
        5 -> "05月20日 至 06月20日"
        6 -> "06月20日 至 07月20日"
        7 -> "07月20日 至 08月20日"
        8 -> "08月20日 至 09月20日"
        9 -> "09月20日 至 10月20日"
        10 -> "10月20日 至 11月20日"
        11, -1 -> "11月20日 至 12月20日"
        12, 0 -> "12月20日 至 01月20日"
        else -> ""
    }

    /**由日期范围类型得到对应月份*/
    fun getMonthByRangeType(type: Int, yearNow: String = SimpleDateFormat("yyyy").format(Date())) = when (type) {
        -1 -> "${yearNow}-11"
        0 -> "${yearNow}-12"
        1 -> "${yearNow}-01"
        2 -> "${yearNow}-02"
        3 -> "${yearNow}-03"
        4 -> "${yearNow}-04"
        5 -> "${yearNow}-05"
        6 -> "${yearNow}-06"
        7 -> "${yearNow}-07"
        8 -> "${yearNow}-08"
        9 -> "${yearNow}-09"
        10 -> "${yearNow}-10"
        11 -> "${yearNow}-11"
        12 -> "${yearNow}-12"
        else -> ""
    }

    /**loadwait指示*/
    var loadWaitMsg = MutableLiveData<Int>().apply { value = -1 }

    /**用于提交的数据
     * 默认先填充一个数据
     * */
    val submitDatas: MutableList<TripRecordNewEntity> = mutableListOf<TripRecordNewEntity>()
            .apply { add(TripRecordNewEntity()) }

    //submitDatas的备份，用于在取消修改的时候，把  submitDatas 的 details 恢复原样。
    var oldListCache: List<TripRecordNewEntity> = listOf()

    //总预算
    val tripBudgetTotal: ObservableField<Double> = ObservableField()
    //可用预算
    val tripBudgetUsable: ObservableField<Double> = ObservableField()
    /**
     * 时间范围,1 1月20~2月19，2 2月20~3月19 3 3月20~4月19 ..... 12 12月20~1月19 ,
     */
    val selectDateRange: MutableLiveData<Int> = MutableLiveData()

    /**控制刷新Rv*/
    var isNotifyRv: MutableLiveData<Boolean> = MutableLiveData()

    /**关闭页面*/
    var isFinish: MutableLiveData<Boolean> = MutableLiveData()

    /**
     * 请求所使用的年月
     */
    var queryYm = ObservableField<String>(SimpleDateFormat("yyyy-MM").format(Date()))

    var alertDialogText = MutableLiveData<String>()

    /**转圈圈文字*/
    var progressText = MutableLiveData<String>()

    /**获取Spinner的数据源*/
    fun getSpinnerEntries(nowMonth: Int): List<String> {
        return mutableListOf<String>().apply {
            add(getDateRange(nowMonth - 1))
            add(getDateRange(nowMonth - 2))
        }.toList()
    }

    /**恢复原数据*/
    fun resumeSubmitData() {
        submitDatas.run {
            clear()
            //再用深拷贝还原回来
            addAll(ListDominator.clone(ArrayList(oldListCache)))
            submitDatas.forEachIndexed { index, item ->
                item.details = ListDominator.clone(ArrayList(oldListCache[index].details))
            }
            isNotifyRv.value = true
        }
    }

    /**判断是否可以插入当天记录*/
    fun checkCanInsertTravelDay(userNo: String, travelDay: String, failedCallback: () -> Unit) = viewModelScope.launch {
        try {
            repo.checkCanInsertTravelDay(userNo, travelDay)
        } catch (e: Exception) {
            MyToast.show(e.message)
            failedCallback.invoke()
        }
    }


    /**查询差旅预算*/
    fun queryTripBudget(userNo: String, ym: String) = viewModelScope.launch {
        try {
            repo.queryTripBudget(userNo, ym).run {
                tripBudgetTotal.set(totalBudget)
                tripBudgetUsable.set(useableBudget)
            }
        } catch (e: Exception) {
            MyToast.show(e.message)
        }
    }


    /**日期范围选择监听*/
    fun onItemSelectedNumber(parent: View, view: View, pos: Int, id: Long) = viewModelScope.launch {
        selectDateRange.value = Calendar.getInstance().get(Calendar.MONTH) /*+ 1 -1*/ - pos
//        repo.queryTripRecord(SystemHelper.LoginUserId, getMonthByRangeType(selectDateRange.value!!))?.takeIf { it.isNotEmpty() }?.let {
//            alertDialogText.value = "当前日期已经提交过出差记录，可前往记录查看和修改，如需继续上报请选择其他日期！"
//        }
    }

    /***
     * 查询出差记录
     */
    fun queryTripRecord(userNo: String, ym: String) = viewModelScope.launch {
        loadWaitMsg.value = LoadWaitDominator.LOADING
        try {
            tripRecordsCache.value = repo.queryTripRecord(userNo, ym)
            if (tripRecordsCache.value.isNullOrEmpty()) {
                loadWaitMsg.value = LoadWaitDominator.NO_DATA
            } else {
                loadWaitMsg.value = LoadWaitDominator.DISMISS
            }
        } catch (e: Exception) {
            MyToast.show(e.message)
            loadWaitMsg.value = LoadWaitDominator.NET_ERROR
        }
    }


    /**
     * 查询出差记录详情
     */
    fun queryTripRecordDetail(tid: Int) = viewModelScope.launch {
        loadWaitMsg.value = LoadWaitDominator.LOADING
        try {
            val entity = repo.queryTripRecordDetail(tid)
            submitDatas.run { clear(); add(entity.resultInfo!!) }
            //深拷贝备份
            oldListCache = ListDominator.clone(ArrayList(submitDatas))
            oldListCache.forEachIndexed { index, cache ->
                cache.details = ListDominator.clone(ArrayList(submitDatas[index].details))
            }
            isNotifyRv.value = true
            loadWaitMsg.value = LoadWaitDominator.DISMISS
        } catch (e: Exception) {
            loadWaitMsg.value = LoadWaitDominator.NO_DATA
            MyToast.show(e.message)
        }
    }

    /**点击提交按钮*/
    fun onSubmitClick(view: View) {
        viewModelScope.launch {
            //校验====================
//        progressText.value = "正在校验日期..."
//        repo.queryTripRecord(SystemHelper.LoginUserId, getMonthByRangeType(selectDateRange.value!!))?.takeIf { it.isNotEmpty() }?.let {
//            alertDialogText.value = "当前日期已经提交过出差记录，可前往记录查看和修改，如需继续上报请选择其他日期！"
//            progressText.value = ""
//            return@launch
//        }
            if (submitDatas.any { it.travelDay.isBlank() }) {
                MyToast.show("请选择日期")
                progressText.value = ""
                return@launch
            }
            if (submitDatas.any { it.startLocation.isBlank() }) {
                MyToast.show("请输入始发站")
                progressText.value = ""
                return@launch
            }
            //循环，检查+赋值
            submitDatas.forEach {
                //如果是负的，要转回正的，详见 getDateRange(month: Int) 方法
                it.timeRange = selectDateRange.value!!.run { if (this < 1) 12 + this else this }
                it.userNo = SystemHelper.LoginUserId

                if (it.details.any { it.location.isBlank() }) {
                    MyToast.show("请输入城市名")
                    progressText.value = ""
                    return@launch
                }
                it.details.find { it.reasonType in arrayOf(1,2) && it.workOrderNo.isNullOrBlank() }?.let {
                    MyToast.show(if (1 == it.reasonType) "请选择工单" else "请选择巡店记录")
                    progressText.value = ""
                    return@launch
                }
                //市内交通费&&车费&&住宿费 不填则默认为0，把UI的序号转换成数值
                it.details.forEach {
                    if (it.cityTrafficCost.isBlank()) it.cityTrafficCost = "0.00"
                    if (it.trafficCost.isBlank()) it.trafficCost = "0.00"
                    if (it.hotelCost.isBlank()) it.hotelCost = "0.00"
//                    it.syncReasonType(null)
                }
            }
            //开始真正上传====================
            progressText.value = "正在上传数据..."
            try {
                var result = ""
                if (activityType == MachineTripAct.TYPE_SUBMIT) {
                    result = repo.submitTripRecord(GsonBuilder().excludeFieldsWithoutExposeAnnotation().create().toJson(submitDatas))//新建模式
                } else {
                    result = repo.changeTripRecord(GsonBuilder().excludeFieldsWithoutExposeAnnotation().create().toJson(submitDatas[0]))//修改模式
                }
                MyToast.show(result)
                progressText.value = ""
                isFinish.value = true
            } catch (e: Exception) {
                MyToast.show(e.message)
                progressText.value = ""
            }
        }

    }


    /**日期选择点击监听*/
    fun dateSelectOnClick(view: View) {
        DatePickerYearMonthDialog(view.context, { year, monthOfYear ->
            dateSelected(year, monthOfYear)
        }).show()
    }

    /**
     * 已选查询日期之后的处理
     */
    fun dateSelected(year: Int, month: Int) {
        val today = SimpleDateFormat("yyyy-MM").format(Date())
        val selectDay = "${year}-${String.format("%02d", month + 1)}"
        if (today.compareTo(selectDay) < 0) {
            MyToast.show("不能超过当月")
            return
        }
        queryYm.set(selectDay)
        queryTripRecord(SystemHelper.LoginUserId, queryYm.get()!!)
    }

}