package com.urovo.activity.machinetrip.ui

import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.machinetrip.model.MachineTripModel
import com.urovo.activity.machinetrip.net.TripRecordNewEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.databinding.TripInputItemBinding
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.wgmf.trade.ui.ada.ViewHolderX
import java.util.*

/**
 * @author Sasuke on 2019-10-25 0025.
 */
class TripInputAdapter(val context: Context, val tripModel: MachineTripModel, val dataList: MutableList<TripRecordNewEntity>, val scrollToCallBack: ((Int) -> Unit)?) : RecyclerView.Adapter<ViewHolderX>() {
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolderX {
        val binding = DataBindingUtil.inflate<TripInputItemBinding>(
                LayoutInflater.from(context),
                R.layout.trip_input_item,
                parent,
                false)
        return ViewHolderX(binding.root)
    }

    override fun getItemCount(): Int {
        return dataList.size
    }

    override fun onBindViewHolder(holder: ViewHolderX, pos: Int) {
        val binding = DataBindingUtil.getBinding<TripInputItemBinding>(holder.view)
        val dataItem = dataList[pos]
        binding?.run {
            entity = dataItem
            model = tripModel
            //遮罩层，禁止编辑
            binding.coverView.setOnTouchListener { view, motionEvent -> true }

            //            itemPos = holder.adapterPosition
            childRv.run {
                layoutManager = object : LinearLayoutManager(holder.view.context) {
                    override fun canScrollVertically(): Boolean {//不允许滚动
                        return false
                    }
                }
                adapter = TripInputChildAdapter(holder.view.context,tripModel, dataItem.details)
            }
            //继续添加下一站
            addNextStationBtn.setOnClickListener {
                dataItem.details.add(TripRecordNewEntity.Details())
                childRv.adapter?.notifyItemInserted(dataItem.details.size - 1)
            }
            //选择日期（不能早于上一条的日期，不能晚于今日）
            selectDateTv.setOnClickListener {
                //起始限制（从0往后数最后一个日期不为空的）===========
//                var startLimit: String?
//                try {
//                    val lastNotEmptyDate = dataList.take(holder.adapterPosition).last { it.travelDay.isNotBlank() }.travelDay
//                    startLimit = if (holder.adapterPosition == 0) null
//                    else DateUtil.getDayAfter(DateUtil.String2Date(lastNotEmptyDate, "yyyy-MM-dd"))
//                } catch (e: NoSuchElementException) {
//                    startLimit = null
//                }
//
//                //终止限制（从自己往后数第一个日期不为空的）===========
//                var endLimit: String?
//                try {
//                    if (holder.adapterPosition == dataList.size - 1) {
//                        endLimit = DateUtil.getDate(Date())
//                    }else{
//                        val firstNotEmptyDate =  dataList.subList(holder.adapterPosition + 1, dataList.size ).first { it.travelDay.isNotBlank() }.travelDay
//                        endLimit = DateUtil.getDate(
//                                DateUtil.getDateBefore(
//                                        DateUtil.String2Date(firstNotEmptyDate, "yyyy-MM-dd"), 1))
//                    }
//                } catch (e: NoSuchElementException) {
//                    endLimit = DateUtil.getDate(Date())
//                }
                //弹窗选择日期
                selectDateTv.tag = selectDateTv.text.toString()//先保存一份，如果不合法，就恢复原数据
                DateUtil.pickDialog(context, selectDateTv,
                        "yyyy-MM-dd",
                        null,
                        Date()/*DateUtil.String2Date(endLimit, "yyyy-MM-dd")*/) {
                    if (dataList.any { it != dataItem && it.travelDay == selectDateTv.text.toString() }){
                        MyToast.show("日期重复，请重新选择")
                        selectDateTv.text = selectDateTv.tag as String?//恢复数据
                        return@pickDialog
                    }
                    dataItem.travelDay = selectDateTv.text.toString()
                    tripModel.checkCanInsertTravelDay(SystemHelper.LoginUserId, dataItem.travelDay) {
                        //后台判断不合法的情况
                        selectDateTv.text = selectDateTv.tag as String?//恢复数据
                        dataItem.travelDay = selectDateTv.tag as String? ?:""//恢复数据
                    }
                }.show()
            }
            //加号按钮
            addItemBtn.setOnClickListener {
                if (dataList.any { it.travelDay.isBlank() }) {
                    MyToast.show("请先填写完整再添加")
                    return@setOnClickListener
                }
                dataList.add(TripRecordNewEntity())
                notifyItemInserted(dataList.size - 1)
                if (dataList.size == 2) {
                    notifyItemChanged(0)//减号要有效化
                }
                scrollToCallBack?.invoke(dataList.size - 1)

            }
            //减号按钮
            removeItemBtn.run {
                isEnabled = !(pos == 0 && dataList.size == 1)
                setOnClickListener {
                    dataList.remove(dataItem)
                    notifyItemRemoved(holder.adapterPosition)
                    if (dataList.size == 1) {
                        notifyItemChanged(0)//减号要无效化
                    }
                }
            }
            executePendingBindings()//防闪屏
        }//binding?.run


    }
}