package com.urovo.activity.memberGoodsShare.presenter;

import android.content.Context;
import android.graphics.Bitmap;
import android.text.TextUtils;

import com.urovo.activity.memberGoodsShare.model.MemberGoodsData;
import com.urovo.activity.memberGoodsShare.model.MemberQrEntity;
import com.urovo.activity.memberGoodsShare.net.MemSharePosterRes;
import com.urovo.wgmf.zxing.ZxingCodeTool;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.response.BaseSimpleResponse;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.utils.MyToast;
import com.zsy.libcore.common.other.DensityUtil;

import java.util.HashMap;

import okhttp3.Request;

/**
 * Created by Sasuke on 2016/12/21.
 */

public class MemberGoodsQrPresenter implements MemberGoodsQrContract.presenter {
    MemberGoodsQrContract.view mView;
    MemberGoodsData data;
    Context context;
    public MemberGoodsQrPresenter(Context context, MemberGoodsQrContract.view mView){
        this.mView = mView;
        mView.setPresenter(this);
        data = new MemberGoodsData();
        this.context =context;
    }

    @Override
    public void start() {
        mView.init();
    }

    public MemberGoodsData getData(){
        if(data == null){
            data = new MemberGoodsData();
        }
        return data;
    }

    public void setEntity(MemberQrEntity entity){
        getData().setEntity(entity);
        new Thread(new Runnable() {
            @Override
            public void run() {
                getData().setErweima(transUrlTo2DCode(getData().getQrUrl()));
                mView.refreshUI(getData().getErweima());//先下载了分享时的缩略图再更新，确保分享时图片不为空。
            }
        }).start();

        if(!getData().getEntity().hasImage()){//前面一个页面产品图片没有下载成功时，就再下载一次；如果直接传过来了就直接刷新UI
            downloadUMImage(getData().getImageUrl());
        }
    }


    /**把Url生成二维码*/
    public Bitmap transUrlTo2DCode(String url){
        if(url != null){
            return ZxingCodeTool.create2DCode(url, DensityUtil.dip2px(context,350),DensityUtil.dip2px(context,350));
        }
        return null;
    }

    /**下载分享时的图片*/
    public void downloadUMImage(String imageUrl){//图片大小一定要转小一点，具体多少不知道，反正以前120x120是可以的
        OkHttpDominator.downImageAsyn(imageUrl, new OkHttpDominator.ResultCallback<Bitmap>() {
            @Override
            public void onError(Request request, Exception e) {
                mView.showMsg("产品图片下载失败，将使用默认图片");
            }

            @Override
            public void onResponse(Bitmap response) {
                if(response != null ){
                    getData().getEntity().setImage(response);
                    mView.refreshUI(getData().getErweima());//先下载了分享时的缩略图再更新，确保分享时图片不为空。
                }
            }
        });
    }

    /**请求海报相关图*/
    public void requestPosterInfo(String productTemplateId){
        HashMap<String,String> param = new HashMap<>(1);
        param.put("productTemplateId",productTemplateId);
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_POSTER_INFO,
                param, new OkHttpDominator.ResultCallback<MemSharePosterRes>() {
                    @Override
                    public void onError(Request request, Exception e) {
                        e.printStackTrace();
                        MyToast.show("网络异常：" + e.getMessage().toString());
                    }

                    @Override
                    public void onResponse(MemSharePosterRes res) {
                        if (res == null) return;
                        if (res.msgCode != 0){
                            MyToast.show(res.msgInfo);
                            return;
                        }
                        //好怕后台传个空图片过来 或者传个不是链接的字符串过来
                        if (TextUtils.isEmpty(res.getResultInfo().getImgUrl()) || !res.getResultInfo().getImgUrl().startsWith("http://")){
                            res.getResultInfo().setImgUrl("");
                        }
                        mView.refreshPoster(res.getResultInfo());
                    }
                });
    }

//    /**分享给朋友*/
//    public void share2Friends(Activity activity, UMShareListener umShareListener){
//        if(!TextUtils.isEmpty(getData().getQrUrl() )) {
//            new ShareAction(activity).setPlatform(SHARE_MEDIA.WEIXIN).setCallback(umShareListener)
//                    .withMedia(getData().getEntity().getUMImage(activity))
//                    .withText(getData().getContent())
//                    .withTitle(getData().getTitle())
//                    .withTargetUrl(getData().getQrUrl() + (getData().getQrUrl().contains("?") ? "&" : "?") + "time=" + Calendar.getInstance().getTimeInMillis() + "&userName=" + SystemHelper.LoginUserId + "&type=1")
//                    .share();
//        }
//    }
//
//    /**分享给朋友圈*/
//    public void share2Group(Activity activity, UMShareListener umShareListener){
//        if(!TextUtils.isEmpty(getData().getQrUrl() )){
//            new ShareAction(activity).setPlatform(SHARE_MEDIA.WEIXIN_CIRCLE).setCallback(umShareListener)
//                    .withMedia(getData().getEntity().getUMImage(activity))
//                    .withText(getData().getContent())
//                    .withTitle(getData().getTitle())
//                    .withTargetUrl(getData().getQrUrl() + (getData().getQrUrl().contains("?")?"&":"?")+ "time="+ Calendar.getInstance().getTimeInMillis()+"&userName="+SystemHelper.LoginUserId + "&type=1")
//                    .share();
//        }
//    }

    /**
     * 分享成功 埋点
     * type  0:分享到朋友圈,1:分享给朋友 2: 分享到企业微信
     */
    public void buryPoint(int type){
        HashMap<String,String> param = new HashMap<>();
        param.put("staffId",SystemHelper.LoginUserId);
        param.put("shopId",SystemHelper.LoginOrganId);
        param.put("type",type +"");
        param.put("qrcodeId",getData().getEntity().getQrCodeId() + "");//线上产品二维码id
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.VIP_BURY_POINT, param,
                new OkHttpDominator.ResultCallback<BaseSimpleResponse>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(BaseSimpleResponse response) {

                    }
                });
    }
}
