@file:JvmName("MemGoodsListData")

package com.urovo.activity.membererweima.model

import com.chad.library.adapter.base.entity.AbstractExpandableItem
import com.chad.library.adapter.base.entity.MultiItemEntity
import com.urovo.activity.membererweima.net.MemGoodsExRes

/**
 * Created by Sasuke on 2017/3/17.
 */

class MemGoodsListData {
    /**
     * 数据表
     */
    var productList: MutableList<GroupEntity> = mutableListOf()

    /**
     * 跟上面的数据表是一样的东西，就是结构不同（这是后台返回的未经处理的数据）
     * ，只是为了方便搜索，上面那个结构复杂不好搜
     */
    var searchPool: List<MemGoodsExRes.Data.PurchaseData.QrCodeInfo>? = listOf()


    //===================以下集中爆破==========================================
    /**
     * 一级目录元素
     */

    companion object {
        const val PURCHASE = "yes"
        const val NORMAL_SELL = "no"

        const val LIST_LEVEL_0 = 0
        const val LIST_LEVEL_1 = 1
        const val LIST_LEVEL_DATA = 2
    }

    data class FirstLevel(var title1: String, var count: Int = 0) : AbstractExpandableItem<SecondLevel>(), MultiItemEntity {
        override fun getLevel(): Int {
            return LIST_LEVEL_0
        }

        override fun getItemType(): Int {
            return LIST_LEVEL_0
        }
        //这里注意title2必须为Int，后台返回的是int，而这里二级目录也需要按照这个id来进行逆序排序，
        // 如果把int转成string来排序，那就会出现 “3” > “21” > “178” 这种情况，与需求不符
    }

    /**
     * 二级目录元素
     */
    data class SecondLevel(var title2: String, var count: Int = 0) : AbstractExpandableItem<MemGoodsExRes.Data.PurchaseData.QrCodeInfo>(), MultiItemEntity {
        override fun getLevel(): Int {
            return LIST_LEVEL_1
        }

        override fun getItemType(): Int {
            return LIST_LEVEL_1
        }
    }
    //====================以上集中爆破=========================================


    /**
     * 改版的元素模型
     */
    data class GroupEntity(var title: String?,//标题
                           var count: Int = 0,//包含data的个数
                           var isExpanded: Boolean = false,//是否是展开的
                           var children: List<GroupEntity?>?,//下一级的child（如果child为null，说明下级是data数据层；如果data数据层为null，说明下一级是child目录层级，两个必有一个为null）
                           var datas: List<MemGoodsExRes.Data.PurchaseData.QrCodeInfo?>?)//既可能包含child List，也可能只包含数据List


    /**
     * 将后台返回的乱七八糟的数据转成我们列表使用的整齐的数据
     */
    fun transRes2Data(data: MemGoodsExRes.Data) {
        productList.clear()
        //缓存原生数据方便搜索
        searchPool = data.purchaseNoResultList.flatMap { it.qrCodeInfo } + data.purchaseYesResultList.flatMap { it.qrCodeInfo }

        //重新组装数据
        productList.apply {
            add(GroupEntity(
                    "常规转发",
                    0,
                    false,
                    data.purchaseNoResultList?.flatMap { listOf(GroupEntity(it?.productGroupName, 0, false,null, it?.qrCodeInfo)) },
                    null)
            )
            add(GroupEntity(
                    "代下单转发",
                    0,
                    false,
                    data.purchaseYesResultList?.flatMap { listOf(GroupEntity(it?.productGroupName, 0,false,null, it?.qrCodeInfo)) },
                    null)
            )
            //计算数量
            this.forEach { groupEntity ->
                groupEntity.run { count = children?.sumBy { (it?.datas?.size ?:0).apply { it?.count = this } } ?:0 }
            }
        }

        //===========================以下为第二版的数据处理方式===========================
//        //①建立一级目录
//        productList.apply {
//            add(FirstLevel("常规转发", 0))
//            add(FirstLevel("代下单转发", 0))
//        }
//        //②建立二级目录
//        arrayOf(data.purchaseNoResultList, data.purchaseYesResultList).forEachIndexed { index, list ->
//            list?.forEach {
//                val second = SecondLevel(it.productGroupName, 0)
//                (productList[index] as FirstLevel).addSubItem(second)
//                it.qrCodeInfo?.forEach {
//                    second.addSubItem(it)
//                }
//            }
//        }
//        //③计算各级数量
//        productList.forEach {
//            (it as FirstLevel).subItems.forEach {
//                // 遍历二级目录
//                it.count = it.subItems.size//先把最底级的数量给算出来
//            }
//            //遍历一级目录
//            it.count = it.subItems.sumBy { it.count }
//        }
        //===========================以上为第二版的数据处理方式===========================


        //======================以下为第一版时的数据处理方式======================
        /*//        resList.forEach { data ->
//            // ① 如果没有一级目录，就创建
//            if (productList.none { (it as FirstLevel?)?.title1 == data.purchaseFlag }) {
//                productList.add(FirstLevel(data.purchaseFlag, 0))
//            }
//            // ② 如果没有二级目录，就创建
//            val firstLevel = productList.find { (it as FirstLevel).title1 == data.purchaseFlag } as FirstLevel
//            var index = firstLevel!!.subItems?.indexOfFirst { it?.title2 == data.productGroupId }//前面实例化过，所以这里firstLevel肯定不会空，但subItem可不一定
//            if (index == null || index == -1) {
//                firstLevel.apply {
//                    addSubItem(0, SecondLevel(data.productGroupId, 0))
//                }
//                index = 0//设为0是为了跟上面的secondList.add(保持一致，延用-1的话下面的[]数组越界会崩
//            }
//            // ③ 都创建好了，就把元素添加进去
//            firstLevel.getSubItem(index).apply {
//                addSubItem(data)
//                subItems.sortByDescending { it.qrCodeId }//根据这什么id进行逆序排序，据说id越高，说明越新，就越靠前
//            }
//            //④ 添加好了，就把二级目录再排序一下，要按id进行降序排列
//            firstLevel.subItems.sortByDescending {
//                when (it.title2) {
//                    0 -> Int.MAX_VALUE//如果是0（就是如果返回的没有该字段，那就把该项作为“无分类二级”，提到二级目录里的最前面）
//                    else -> it.title2
//                }
//            }
//        }
//        // ⑤ 最后一步，把1级目录做一个写死的排序
//        productList.sortBy {
//            when ((it as FirstLevel).title1) {
//                PURCHASE -> 2//代下单
//                NORMAL_SELL -> 1//常规销售
//                else -> 0//无分类
//            }
//        }
//
//        // ⑥  真正的最后一步，把各级包含的数量给算出来
//        productList.forEach {
//            (it as FirstLevel).subItems.forEach {
//                // 遍历二级目录
//                it.count = it.subItems.size//先把最底级的数量给算出来
//            }
//            //遍历一级目录
//            it.count = it.subItems.sumBy { it.count }
//        }
//
//        // ⑦ 绝对的最后一步！！
//        // 把标题名为空的分组的元素抽出来，一个个加到一级的位置去，避免标题为空还占个位置（需求里这个叫“无分类”
//
//        val unknown = productList.find { (it as FirstLevel?)?.title1 == "" }
//        if (unknown != null) {
//            productList.apply {
//                remove(unknown)
//                (unknown as FirstLevel).subItems.flatMap { (it as SecondLevel).subItems }.reversed().forEach {
//                    productList.add(0,it)
//                }
//            }
//        }

//         下面没写好，不要打开，说到底其实后台本身就不应该传这种数据过来嘛，不是说好了吗，肯定会有组名
//        //⑧ 哎呦我操真烦，还要加一步，把二级目录里没有标题的分为单独一组，放在对应的一级目录下
//        productList.forEach { firstLevel ->
//           (firstLevel as FirstLevel).subItems.forEach {
//               val dataLevel = it.subItems.firstOrNull { ! it.productGroupName.isNullOrEmpty() }
//               firstLevel.removeSubItem(it)
//               it.subItems.reverse()
//           }
//        }

        //在这里打断点获取整理后的列表的Json形式，便于观察
//        val jsonSee = Gson().toJson(productList)
//        val num = 1*/
        //======================以上为第一版时的数据处理方式======================
    }

    /**
     * 搜索
     */
    fun searchByKeyWord(key: String): List<MemGoodsExRes.Data.PurchaseData.QrCodeInfo>? {
        return searchPool?.filter { key in it.productTemplateName }
    }

}

