package com.urovo.activity.memorderact.repository

import com.urovo.activity.memorderact.net.MemOrderListRes
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.Base3Response
import okhttp3.Request
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException
import kotlin.coroutines.suspendCoroutine

/**
 * @author Sasuke on 2019-7-19 0019.
 */
class MemOrderListRepo {

    /**
     * 获取会员商品未处理订单详情
     */
    suspend fun queryMemOrderDetails(userNo: String) = suspendCoroutine<List<MemOrderListRes.ResultInfo>> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_MEM_ORDER_DETAILS,
                mapOf("userNo" to userNo), object : OkHttpDominator.ResultCallback<MemOrderListRes>() {
            override fun onError(request: Request?, e: Exception) {
                conti.resumeWithException(Exception("网络异常"))
            }

            override fun onResponse(res: MemOrderListRes?) {
                if (res == null || res.resultInfo == null || res.resultInfo.size == 0) {
                    conti.resumeWithException(Exception("暂无数据"))
                    return
                }
                if (res.msgCode != 0){
                    conti.resumeWithException(Exception(res.msgInfo))
                    return
                }
                conti.resume(res.resultInfo)
            }
        })
    }

    /**
     * 撤销会员商品订单
     */
    suspend fun cancelMemOrder(id: Int) = suspendCoroutine<Base3Response> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.CANCEL_MEM_ORDER ,
                mapOf("id" to id.toString()),object: OkHttpDominator.ResultCallback<Base3Response>(){
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(Exception("网络异常"))
            }

            override fun onResponse(res: Base3Response?) {
                if (res == null ){
                    conti.resumeWithException(Exception("返回为空"))
                    return
                }
                if (res.msgCode != 0){
                    conti.resumeWithException(Exception(res.msgInfo))
                    return
                }
                conti.resume(res)
            }
        })
    }


}