package com.urovo.activity.memorderact.repository

import com.urovo.activity.memorderact.net.ReceiversInfosRes
import com.urovo.wgmf.tool.netUtil.QiNiuDominator
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.Base3Response
import okhttp3.Request
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException
import kotlin.coroutines.suspendCoroutine

/**
 * @author Sasuke on 2019-07-21.
 */
class MemOrderSubmitRepo {

    /**
     * 获取收货人信息
     */
    suspend fun queryReceiversInfo(mobile: String) = suspendCoroutine<List<ReceiversInfosRes.ResultInfo>> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_RECEIVER_ADDRESS,
                mapOf("mobile" to mobile),object : OkHttpDominator.ResultCallback<ReceiversInfosRes>(){
            override fun onError(request: Request?, e: Exception) {
                conti.resumeWithException(e)
            }

            override fun onResponse(res: ReceiversInfosRes){
                if (res == null){
                    conti.resume(listOf())
                    return
                }
                if (res.msgCode != 0){
                    conti.resumeWithException(Exception(res.msgInfo))
                    return
                }
                conti.resume(res.resultInfo)
            }
        })
    }

    /**
     * 把图片传到七牛
     */
    suspend fun submitPhotos2Qiniu(photoPaths: MutableList<String>) = suspendCoroutine<String> { conti ->
        if (photoPaths.size == 0) {
            conti.resume("")
            return@suspendCoroutine
        }
        QiNiuDominator.uploadImages(photoPaths,
                { qiniuUrl ->
                    //CompleteCallback
                    //照片七牛url组装
                    var packUrls = ""
                    qiniuUrl.forEach { packUrls += "${it}," }
                    if (packUrls.isNotEmpty()) packUrls = packUrls.substring(0, packUrls.length - 1)
                    conti.resume(packUrls)
                },
                { str ->
                    //FailedCallback
                    conti.resumeWithException(Exception(str))
                }, false)
    }

    /**
     * 上传会员订单
     */
    suspend fun submitMemOrder(orderId: Int,addressId: Int,photoUrls: String,remarks: String) = suspendCoroutine<Base3Response> { conti ->
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.SUBMIT_MEM_ORDER,
                mapOf("id" to orderId.toString(),"addrId" to addressId.toString(),"invoicePics" to photoUrls,"remarks" to remarks),object: OkHttpDominator.ResultCallback<Base3Response>(){
            override fun onError(request: Request?, e: java.lang.Exception) {
                conti.resumeWithException(Exception("当前网络环境不佳，请点击确定按钮重新提交！"))
            }

            override fun onResponse(res: Base3Response?) {
                if (res == null){
                    conti.resumeWithException(Exception("返回为空，请点击确定按钮重新提交！"))
                    return
                }
                conti.resume(res)
            }
        })
    }

}