package com.urovo.activity.memorderact.view

import android.content.Intent
import android.os.Bundle
import androidx.appcompat.app.AlertDialog
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.memorderact.model.MemOrderListFactory
import com.urovo.activity.memorderact.model.MemOrderListModel
import com.urovo.activity.memorderact.repository.MemOrderListRepo
import com.urovo.activity.memorderact.ui.MemOrderAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.MemOrderActBinding
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.mem_order_act.*

/**
 * @author Sasuke on 2019-7-19 0019.
 */
class MemOrderAct : BaseActivity() {
    var loadWait: LoadWaitDominator? = null

    val model by lazy { ViewModelProviders.of(this, MemOrderListFactory(MemOrderListRepo(), application)).get(MemOrderListModel::class.java) }

    val binding by lazy { DataBindingUtil.setContentView<MemOrderActBinding>(this, R.layout.mem_order_act) }

    //是否第一次进入
    var isFirstTime = true

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        init()
        query()
    }

    fun bind() {
        binding.titleText = "会员商品未完成订单"
        binding.storeUserName = "${SystemHelper.LoginOrganName},${SystemHelper.LoginUserName}"
        initTitle()
        //容器转圈圈
        model.loadWaitText.observe(this, Observer {
            if (it.isNullOrBlank()) {
                loadWait?.dismissAll()
            } else {
                loadWait?.show(it)
            }
        })
        //转圈圈弹窗
        model.progressBarText.observe(this, Observer {
            if (it.isNullOrBlank()) {
                ViewUtil.dismissProgress()
            } else {
                ViewUtil.showProgress(this@MemOrderAct, "请稍等", it)
            }
        })
        //删除订单
        model.deleteOrderId.observe(this, Observer {
            AlertDialog.Builder(this@MemOrderAct).setTitle("提醒")
                    .setMessage("确认要删除该未完成订单？")
                    .setCancelable(false)
                    .setPositiveButton("确定", { dialog, which -> model.deleteOrder(it) })
                    .setNegativeButton("取消", null)
                    .show()
        })
        //触发刷新列表Rv
        model.refresh.observe(this, Observer {
            rV.adapter?.notifyDataSetChanged()
        })
    }

    fun init() {
        loadWait = LoadWaitDominator(this, container)
        rV.run {
            layoutManager = LinearLayoutManager(this@MemOrderAct)
            adapter = MemOrderAdapter(this@MemOrderAct, model.dataList,
                    { id -> model.onDelete(id) },//点击了删除订单按钮
                    { order ->
                        //点击了跳转提交界面的按钮【继续提交】
                        startActivity(Intent(this@MemOrderAct, MemOrderSubmitAct::class.java).apply {
                            putExtra(MemOrderSubmitAct.INTENT_ORDER, order)
                        })
                    })
        }

    }

    fun query() {
        loadWait?.show(LoadWaitDominator.LOADING)
        model.queryMemOrders(SystemHelper.LoginUserId)
    }

    override fun onResume() {
        super.onResume()
        if (!isFirstTime){
            //提交成功后返回，需要重新请求刷新一下列表。
            model.queryMemOrders(SystemHelper.LoginUserId)
        }
        isFirstTime = false
    }
}