package com.urovo.activity.messagecenter.presenter;

import android.content.Context;

import com.urovo.activity.messagecenter.model.ELearningData;
import com.urovo.activity.messagecenter.net.ELearningRes;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.response.Base3Response;
import com.urovo.wugumofang.response.BaseSimpleResponse;
import com.urovo.wugumofang.system.SystemHelper;
import com.zsy.libcore.event.EventManager;
import com.zsy.libcore.event.MsgCount;

import java.util.HashMap;
import java.util.Map;

import okhttp3.Request;

/**
 * Created by Sasuke on 2017/9/20.
 * 在线学习ELearning的Presenter
 */

public class ELearningPresenter implements MsgCenterContract.detailPresenter {
    MsgCenterContract.view mView;
    MsgCenterContract.view mActivity;
    Context context;
    ELearningData data;
    public ELearningPresenter(MsgCenterContract.view mActivity, MsgCenterContract.view mView) {
        if(mView != null) {//在外面单独使用这个Presenter时可以传空mView
            this.mView = mView;
            this.mView.setPresenter(this);
        }
        this.mActivity = mActivity;//(BaseActivity)
        this.data = new ELearningData();
    }


    @Override
    public void start() {

    }

    @Override
    public void refreshAll() {
        //请求内容的同时，请求未读数
        getMsgCount();
        getData().setPageNum(0);
        getELearningCard(0);
    }

    /**
     * ELearning卡片内容
     * @param pageNum （后端接口不支持分页，所以这里应该始终是传0 的）
     */
    public void getELearningCard(int pageNum){
        Map<String,String> params = new HashMap<>();
        params.put("userNo",SystemHelper.LoginUserId);
//        params.put("type", 50+"");
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.MSG_SERVICE.GET_ELEARNING_CARD,
                params,
                new OkHttpDominator.ResultCallback<ELearningRes>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(ELearningRes res) {
                        if(res.msgCode == 0){
                            getData().getModels().clear();
                            getData().getModels().addAll(res.getResultInfo());
                            if (mView != null){
                                mView.notifyAdapter();//不需要排序，毕竟这个是带上拉加载的
                            }
                        }
                    }
                });
    }

    /**
     * 获取未读数
     */
    public void getMsgCount(){
        HashMap<String,String> params = new HashMap<>();
        params.put("userNo", SystemHelper.LoginUserId);
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.MSG_SERVICE.GET_ELEARNING_COUNT,
                params,
                new OkHttpDominator.ResultCallback<BaseSimpleResponse>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(BaseSimpleResponse res) {
                        if(res.msgCode == 0 && res.resultInfo != null){
//                            getData().setMsgAmount(Integer.parseInt(res.resultInfo));
                            EventManager.getInstance().postMsgCenterCount(new MsgCount(
                                    ELearningPresenter.this.getClass().getSimpleName() , Integer.parseInt(res.resultInfo)));
                            if(mView != null){
                                mView.notifyAdapter();//更新Fra
                            }
                        }
                    }
                });
    }

    @Override
    public void loadMore() {
        getELearningCard(getData().getPageNum());
    }

    /**
     * 更新已读数
     * @param type
     * @param eId
     */
    public void uploadMsgCount(String type,String eId) {
        Map<String,String> params = new HashMap<>();
        params.put("userNo",SystemHelper.LoginUserId);
        params.put("type",type);
        params.put("eId",eId);
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.MSG_SERVICE.UPDATE_ELEARNING_COUNT,
                params,
                new OkHttpDominator.ResultCallback<Base3Response>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(Base3Response response) {

                    }
                });

    }

     @Override
    public void notifyAdapter() {
        if(mView != null){
            mView.notifyAdapter();
        }
    }

    @Override
    public ELearningData getData() {
        if(data == null){
            data = new ELearningData();
        }
        return data;
    }

}
