package com.urovo.activity.messagecenter.presenter;


import com.urovo.activity.messagecenter.model.SaleNewsData;
import com.urovo.activity.tabMenu.web.MsgCountResponse;
import com.urovo.activity.tabMenu.web.NewSaleDetailResponse;
import com.urovo.wugumofang.constants.ApiUrls;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.net.OkHttpDominator;
import com.urovo.wugumofang.system.SystemHelper;
import com.zsy.libcore.event.EventManager;
import com.zsy.libcore.event.MsgCount;

import java.util.HashMap;
import java.util.List;

import okhttp3.Request;

/**
 * 销售动态Presenter
 *
 * @author Sasuke
 * @date 2016/8/1
 */
public class SaleNewsPresenter implements MsgCenterContract.detailPresenter {
    private MsgCenterContract.view mView;
    private MsgCenterContract.view mActivity;
    private SaleNewsData data;

    @SuppressWarnings("unchecked")
    public SaleNewsPresenter(MsgCenterContract.view mActivity, MsgCenterContract.view mView) {
        //在外面单独使用这个Presenter时可以传空mView
        if (mView != null) {
            this.mView = mView;
            this.mView.setPresenter(this);
            this.data = new SaleNewsData();
        }
        this.mActivity = mActivity;
    }

    @Override
    public void start() {

    }

    @Override
    public SaleNewsData getData() {
        if (data == null) {
            data = new SaleNewsData();
        }
        return data;
    }

    /**
     * 一进来就加载全部（第1页）or下拉刷新
     */
    @Override
    public void refreshAll() {
        //重新进入的时候，把pageNum重新赋1，以后再刷新就会再从头刷起了
        //获取详情
        getSaleNewsDetail(1);
    }

    /**
     * 获取销售动态列表  详情
     *
     * @param pageNum ...
     */
    private void getSaleNewsDetail(int pageNum) {
        //做标记，上拉加载时会用
        getData().setPageNum(pageNum);
        HashMap<String, String> params = new HashMap<>(3);
        params.put("pageNum", pageNum + "");
        params.put("userId", SystemHelper.LoginUserId);
        params.put("storeId", SystemHelper.LoginOrganId);
//        if(BaseActivity.class.isAssignableFrom(context.getClass())){
//            new BaseAppSalerServerTask("getSaleDetail", NewSaleDetailResponse.class, (BaseActivity) context, "", false,
//                    "/message/getNewSaleDetail", params).execute("");
//        }else{
//            new BaseAppSalerServerTask("getSaleDetail", NewSaleDetailResponse.class, (TabMenuActivity) context, "", false,
//                    "/message/getNewSaleDetail", params).execute("");
//        }
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.NEW_SALE_GET_DETAIL, params,
                new OkHttpDominator.ResultCallback<NewSaleDetailResponse>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(NewSaleDetailResponse res) {
                        transData(res);
                    }
                });
    }

    private void transData(NewSaleDetailResponse res) {
        List<NewSaleDetailResponse.SaleMessage> newList = res.getReturnList();
        //新获取的列表添加到源Data里
        getData().addNewList(newList);
        notifyAdapter();
        //请求第1页时为空，则不用显示“没有更多了”
        if (newList == null) {
            return;
        }
        //上拉加载
        if (getData().getPageNum() != 1 && newList.size() == 0) {
            if (mActivity != null) {
                mActivity.showMsg("没有更多了");
            }
            return;
        }
        String lastId = res.getExtra();
        saveLastId(lastId);
        getData().pageNumPlus();//pageNum自增(一定要在最后做)
        getSaleNewsNum();//获取未读
    }

    /**
     * 上拉加载
     */
    @Override
    public void loadMore() {
        //获取某页详情（加载完成会在BaseActivity里PageNum自增1）
        getSaleNewsDetail(getData().getPageNum());
    }

    @Override
    public void notifyAdapter() {
        if (mView != null) {
            mView.notifyAdapter();
        }
    }

    /**
     * 获取销售动态未读  消息数
     */
    public void getSaleNewsNum() {
        HashMap<String, String> params = new HashMap<>(3);
        String lastId = PreferHelper.getInstance().getString(AppConstants.PreKeys.NEW_SALE_LAST_ID);
        //默认值为1
        lastId = "".equals(lastId) ? "1" : lastId;
        params.put("lastId", lastId);
        params.put("userId", SystemHelper.LoginUserId);
        params.put("storeId", SystemHelper.LoginOrganId);
//        if(BaseActivity.class.isAssignableFrom(context.getClass())){
//            new BaseAppSalerServerTask("getSaleNewsCount",MsgCountResponse.class,(BaseActivity) context,"",false,
//                    "/message/getNewSaleNum",params).execute("");
//        }else{
//            new BaseAppSalerServerTask("getSaleNewsCount",MsgCountResponse.class,(TabMenuActivity) context,"",false,
//                    "/message/getNewSaleNum",params).execute("");
//        }
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APPSALER.NEW_SALE_COUNT, params,
                new OkHttpDominator.ResultCallback<MsgCountResponse>() {
                    @Override
                    public void onError(Request request, Exception e) {

                    }

                    @Override
                    public void onResponse(MsgCountResponse res) {
//                        getData().setMsgAmount(res.getReturnBean());
                        //先映射数据库的数据，再顺便把未读数给存到Data里
                        EventManager.getInstance().postMsgCenterCount(new MsgCount(SaleNewsPresenter.this.getClass().getSimpleName(), res.getReturnBean()));
                    }
                });
    }

    /**
     * 存储已读文章的最大Id
     *
     * @param lastId ...
     */
    private void saveLastId(String lastId) {
        //当上拉加载更多时，再往上拉，会返回extra值为null，此时不应该写入已读,并且传来的lastId应该大于本地lastId才行
        String localLastId = PreferHelper.getInstance().getString(AppConstants.PreKeys.NEW_SALE_LAST_ID);
        if (lastId != null &&
                ("".equals(localLastId) || Integer.parseInt(localLastId) < Integer.parseInt(lastId))) {
            //写入已读
            PreferHelper.getInstance().setString(AppConstants.PreKeys.NEW_SALE_LAST_ID, lastId);
        }
    }

}
