package com.urovo.activity.messagecenter.ui;

import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.urovo.wugumofang.view.webview.BaseCookiesWebFra;
import com.urovo.activity.learning.LearningCenterDetailActivity;
import com.urovo.activity.messagecenter.model.ELearningData;
import com.urovo.activity.messagecenter.net.ArticleModel;
import com.urovo.activity.messagecenter.net.SysMsgModel;
import com.urovo.activity.messagecenter.presenter.MsgDetailPresenter;
import com.urovo.activity.messagecenter.util.Date2DayUtil;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * Created by Sasuke on 2016/10/7.
 */

public class MsgDetailAdapter extends BaseAdapter {
    Context context;
    MsgDetailPresenter mPresenter;
    List<SysMsgModel> dataList;

    public MsgDetailAdapter(Context context,
                            List<SysMsgModel> dataList, MsgDetailPresenter presenter) {
        this.context = context;
        this.dataList = dataList;
        this.mPresenter = presenter;
    }

    @Override
    public int getCount() {
        return dataList.size();
    }

    @Override
    public Object getItem(int i) {
        return null;
    }

    @Override
    public long getItemId(int i) {
        return i;
    }

    @Override
    public View getView(final int i, View view, ViewGroup viewGroup) {
        final ViewHolder holder;
        if (view == null) {
            view = LayoutInflater.from(context).inflate(R.layout.list_item_msg_detail, null);
            holder = new ViewHolder(view);
            view.setTag(holder);
        } else {
            holder = (ViewHolder) view.getTag();
        }

        //如果是文章更新
        if (mPresenter.getData().getActType() == 7 || mPresenter.getData().getActType() == 8) {//  7养生文库 8 学习中心
            ArticleModel item = (ArticleModel) dataList.get(i);
            holder.noticeTitle.setVisibility(View.VISIBLE);
            holder.noticeIsNew.setVisibility(item.getUsed() == 0 ? View.VISIBLE : View.INVISIBLE);//0未读 1已读
            holder.noticeTitle.setText("文章更新");//标题写死
            if (dataList.get(i).getType() == 7) {
                holder.noticeContent.setText("【养生文库】" + item.getTitle());
            } else if (dataList.get(i).getType() == 8) {
                holder.noticeContent.setText("【学习中心】" + item.getTitle());
            }
            holder.noticeTime.setText(Date2DayUtil.withLong(item.getCreate_time()));//已把long型转成带 昨天 前天 的日期格式
        } else if (mPresenter.getData().getLocalType().equals("ELearning")) {//如果是在线学习
            ELearningData.Model item = (ELearningData.Model) dataList.get(i);
            holder.noticeTitle.setVisibility(View.VISIBLE);
            holder.noticeTitle.setText(item.title);
            holder.noticeIsNew.setVisibility(item.getUsed() == 0 ? View.VISIBLE : View.INVISIBLE);//0未读 1已读
            holder.noticeContent.setText(item.content);
            holder.noticeTime.setText(Date2DayUtil.withDay(item.createdDate));//已把long型转成带 昨天 前天 的日期格式
        }else if (mPresenter.getData().getActType() == 10){//如果是线上订单
            SysMsgModel item = dataList.get(i);
            holder.noticeIsNew.setVisibility(View.GONE);//不显示未读标致
            holder.noticeTitle.setTextColor(context.getResources().getColor(R.color.text_color_gray2));
            holder.noticeTitle.setText(item.timestamp);//线上订单就是这样，需要在标题位显示精确时间。
            holder.noticeTime.setText(item.commissionType + " - " + item.commissionStatus);//线上订单就是这样，不要奇怪。
            holder.noticeContent.setText(item.con);
        }else{//如果是其它
            SysMsgModel item = dataList.get(i);
            holder.noticeIsNew.setVisibility(View.GONE);//不显示未读标致
            holder.noticeTitle.setVisibility(View.GONE);//不显示标题（标题位显示时间）
            holder.noticeTime.setText(item.getTime());//已转好格式的时间
            holder.noticeContent.setText(item.getCon());//卡片正文
        }


        view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int type = mPresenter.getData().getActType();
                if (type == 7 || type == 8) {
                    ArticleModel model = (ArticleModel) dataList.get(i);
                    if (model.getUsed() == 0) {//只有在它是未读状态的时候，才上传已读记录
                        holder.noticeIsNew.setVisibility(View.INVISIBLE);//去掉小红点
                        model.setUsed(1);//同步设置本地已读标记
                        mPresenter.uploadMsgCount(model.getType() + "", model.getPid());//同步上传已读标记
                        mPresenter.getData().msgAmountMinus();//同步把总未读数减1
                    }
                    //以下为跳转文章作传值
                    Intent intent = new Intent(context, LearningCenterDetailActivity.class);
                    intent.putExtra(AppConstants.ExtraKeys.ARTICLE_READ_TIMES_COUNT, -1);
                    intent.putExtra(AppConstants.ExtraKeys.ACTIVITY_TYPE, type == 7 ? "1" : "2");
                    intent.putExtra("title", model.getTitle());

                    if ("2".equals(model.getOriginType())) {
                        intent.putExtra("articleUrl", model.getArticleEditorExternalLink());
                    } else {
                        intent.putExtra("articleUrl", model.getArticleUrl());
                    }
                    intent.putExtra("isNeedTest", model.getIsNeedTest());
                    intent.putExtra("testLink", model.getTestLink());

                    intent.putExtra("articleId", model.getPid());
                    ((BaseActivity) context).startActivity(intent);
                } else if (mPresenter.getData().getLocalType().equals("ELearning")) {//如果是在线学习
                    ELearningData.Model model = (ELearningData.Model) dataList.get(i);
                    if (!TextUtils.isEmpty(model.url)) {//URL地址可能为空
                        BaseCookiesWebFra.launch(context, new Intent(), model.url, null, true);
                    }
                }
            }
        });

        return view;
    }

    class ViewHolder {
        public ViewHolder(View view) {
            ButterKnife.bind(this, view);
        }

        @BindView(R.id.content_layout)
        LinearLayout contentLayout;//整个布局
        @BindView(R.id.task_isNew)
        ImageView noticeIsNew;//是否未读
        @BindView(R.id.text_task_content)
        TextView noticeContent;//卡片内容
        @BindView(R.id.text_task_title)
        TextView noticeTitle;//卡片标题
        @BindView(R.id.text_task_time)
        TextView noticeTime;//更新时间
    }
}
