package com.urovo.activity.messagecenter.view;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.urovo.activity.messagecenter.presenter.MsgCenterContract;
import com.urovo.wgmf.uirefresh.PullToRefreshBase;
import com.urovo.wgmf.uirefresh.PullToRefreshListView;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.wgmf.trade.GlobalApp;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

/**
 * Created by Sasuke on 2016/8/11.
 */
public abstract class BaseRefreshFra extends Fragment {
    public PullToRefreshListView pullList;
    public TextView netErrorText;
    public TextView noDataText;
    public ProgressBar loading;
    Timer timer;
    public ListView mListView;
    public boolean isDone;//已刷新成功
    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        container  = (RelativeLayout)inflater.inflate(R.layout.fra_msg_center,null,false);
        initPullListView(container);
        return container;
    }

    public abstract MsgCenterContract.detailPresenter buildPre(MsgCenterContract.view mActivity);

    public abstract MsgCenterContract.detailPresenter getPre();

    public void showNoData(){
        noDataText.setVisibility(View.VISIBLE);
        netErrorText.setVisibility(View.GONE);
        loading.setVisibility(View.GONE);
    }

    public void showNetError(){
        if(pullList.getRefreshableView().getAdapter().getCount() <= 0) {//只有在
            noDataText.setVisibility(View.GONE);
            netErrorText.setVisibility(View.VISIBLE);
            loading.setVisibility(View.GONE);
        }
    }

    /**
     * 用来给外部修改提示内容
     * @param text 可以改成“没有找到”啦（适用搜索），或“暂无数据”啦（默认就是显示这个）
     */
    public void setShowText(String text){
        if(noDataText != null){
            noDataText.setText(text);
        }
    }

    public void hideErrorText(){
        noDataText.setVisibility(View.GONE);
        netErrorText.setVisibility(View.GONE);
        loading.setVisibility(View.GONE);
    }

    @Override
    public void onResume() {
        timer = new Timer();
        super.onResume();
    }

    @Override
    public void onPause() {
        timer.cancel();
        super.onPause();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    public void completeRefresh(final boolean isSuccessed){
        if(getActivity() == null || getActivity().isFinishing()) return;
        getActivity().runOnUiThread(new Runnable() {//没办法，Timer不肯跑
            @Override
            public void run() {
                pullList.onPullDownRefreshComplete();//停止下拉刷新
                pullList.onPullUpRefreshComplete();//停止上拉加载
                if(isSuccessed){
                    String now = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
                    pullList.setLastUpdatedLabel(now);//显示上次刷新时间（其实就是这次）
                }else{
                    ((BaseActivity) getActivity()).showMsg("请求超时，请重试");
                    showNetError();
                }
                isDone = true;//刷新完了就标记一下
            }
        });

    }

    public void initPullListView(ViewGroup container){
        pullList = container.findViewById(R.id.pull_refresh_list);
        netErrorText = container.findViewById(R.id.net_error_text);
        noDataText = container.findViewById(R.id.no_data_text);
        loading = container.findViewById(R.id.loading);

        pullList.setPullRefreshEnabled(true);//启用下拉刷新
        pullList.setPullLoadEnabled(true);//启用上拉加载
        mListView = pullList.getRefreshableView();
        mListView.setDividerHeight(0);
    }

    public abstract void refreshAll();

    /**
     * 上拉加载更多的开关
     * @param enable
     */
    public void setPullLoadEnabled(boolean enable){
        pullList.setPullLoadEnabled(enable);
    }

    /**
     * 下拉刷新的开关
     * @param enable
     */
    public void setPullRefreshEnabled(boolean enable){
        pullList.setPullRefreshEnabled(enable);
    }

    public abstract void loadMore();

    public void setRefreshListener(){
        pullList.setOnRefreshListener(new PullToRefreshBase.OnRefreshListener<ListView>() {
            @Override
            public void onPullDownToRefresh(PullToRefreshBase<ListView> refreshView) {
                isDone = false;//刷新中，未成功状态
                refreshAll();
                timer.schedule(new TimerTask() {
                    @Override
                    public void run() {
                        if(!isDone){
                            completeRefresh(false);
                        }
                    }
                }, GlobalApp.DEFAULT_TIME_OUT);
            }

            @Override
            public void onPullUpToRefresh(PullToRefreshBase<ListView> refreshView) {
                isDone = false;//刷新中，未成功状态
                loadMore();//获取某页详情
                timer.schedule(new TimerTask() {
                    @Override
                    public void run() {
                        if(!isDone){
                            completeRefresh(false);
                        }
                    }
                },GlobalApp.DEFAULT_TIME_OUT);
            }
        });
    }




}
