package com.urovo.activity.messagecenter.view;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewTreeObserver;
import android.widget.AdapterView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.urovo.activity.messagecenter.model.MsgActData;
import com.urovo.activity.messagecenter.presenter.MsgCenterContract;
import com.urovo.activity.messagecenter.presenter.MsgMainPresenter;
import com.urovo.activity.messagecenter.ui.LabelAdapter;
import com.urovo.wgmf.db.dao.dao.PrivilegeDbOperate;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.ui.FragDominator;
import com.zsy.libcore.event.MsgCount;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.HashMap;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;

/**
 * 消息中心Activity（8月5日前是“消息动态”） 前：550  後：174
 *
 * @author Sasuke on 2016/7/25.
 */
public class MsgCenterAct extends BaseActivity implements MsgCenterContract.view {
    //    @BindView(R.id.empty_action_bar)
//    RelativeLayout mEmptyActionBar;
    @BindView(R.id.label_list)
    ListView labelList;
    @BindView(R.id.label_ll)
    LinearLayout labelLayout;
    MsgMainPresenter mainPresenter;
    LabelAdapter labelAdapter;
    FragDominator fragDominator;
    boolean isFirstResume = true;
    Unbinder unbinder;
    @BindView(R.id.empty_view)
    TextView mEmptyViewTv;
    @BindView(R.id.msg_container)
    LinearLayout mMsgContainer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.act_msg_center);
        unbinder = ButterKnife.bind(this);
        EventBus.getDefault().register(this);
        mainPresenter = new MsgMainPresenter(this);
        mainPresenter.start();
        initPrivilege2VisibilityOrGone();
    }

    /**
     * 整个页面没有权限时显示的布局
     */
    private void initPrivilege2VisibilityOrGone() {
        if (!PrivilegeDbOperate.checkUserPrivilege(AppConstants.MsgCenter.MSG_CENTER)) {
            mMsgContainer.setVisibility(View.GONE);
            mEmptyViewTv.setText("暂无使用权限!");
        }
    }

    @Override
    public void initUi() {
//        StatusBarUtils.setViewStatus(mEmptyActionBar);
        //把Intent传进来的未读数给存进Data里
        fragDominator = new FragDominator(this, R.id.msg_frag_container);
        //这里先待监听到它绘制完成后，然后测定LinearLayout的整体高度，再进行ListView的各元素高度重设，使ListView元素平均铺满侧边栏
        labelLayout.getViewTreeObserver().addOnGlobalLayoutListener(new ViewTreeObserver.OnGlobalLayoutListener() {
            //默认调用两次，这里只让它执行一次回调
            boolean isFirst = true;

            @Override
            public void onGlobalLayout() {
                if (isFirst) {
                    isFirst = false;
                    //之所以要减40的个数等份，是因为新需求下方要空出40像素的空隙
                    int itemHeight = labelLayout.getMeasuredHeight() / mainPresenter.getData().countModels.length - (50 / mainPresenter.getData().countModels.length);
                    labelAdapter = new LabelAdapter(MsgCenterAct.this, itemHeight, mainPresenter.getData().countModels);
                    labelList.setAdapter(labelAdapter);

                    labelList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                        @Override
                        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                            switchToLabel(position);
                        }
                    });

                    //intent用于日后推送时跳转对应标签
                    int num = getIntent().getIntExtra(AppConstants.ExtraKeys.MSG_CENTER_DEFAULT_LABEL, 0);
                    //最开始以0为默认页,3是销售动态
                    switchToLabel(num);
                }
            }
        });

        mainPresenter.getData().setMsgCounts(
                (HashMap<String, Integer>) getIntent().getSerializableExtra(AppConstants.ExtraKeys.MSG_COUNT_MAP));
    }

    @Override
    public void setPresenter(Object presenter) {
        this.mainPresenter = (MsgMainPresenter) presenter;
    }

    public void switchToLabel(int position) {
        //刷新标签上的显示
        mainPresenter.getData().setChecked(position);
        labelAdapter.notifyDataSetChanged();//每次点击标签都要用的！不要删！！！
        //显示右边Fragment
        refreshFra(position);
    }

    /**
     * 在从二级列表返回到主页面的时候，重新请求数据以刷新
     */
    @Override
    protected void onResume() {
        //通知公告 和 销售活动 就算了，别刷了，这两个浪费资源
        if (!isFirstResume) {
            //刷！！！
            ((BaseRefreshFra) fragDominator.getFraWithTag(mainPresenter.getData().getCheckedNum() + "")).getPre().refreshAll();
        }
        super.onResume();
        isFirstResume = false;
    }

    /**
     * 通用的显示并刷新 Fragment 分页
     *
     * @param pos
     */
    public void refreshFra(int pos) {
        try {
            Class fraClass = mainPresenter.getData().countModels[pos].fraClass;
            //只有在第一次显示Fragment的时候才执行，后面就是直接show了
            if (!fragDominator.showFraByTag(pos + "")) {
                BaseRefreshFra fra = (BaseRefreshFra) fraClass.newInstance();
//                mainPresenter.getData().countModels[pos].preTag = fra.buildPre(this).getClass().getSimpleName();//构造pre+存Tag一步搞定
                //preTag必须要事先定好，因为在TabmenuActivity主页面请求未读数时，pre都还没建立起来。不可能等pre建立了再定tag。
                fra.buildPre(this);
                fragDominator.addWithTag(fra, pos + "");
                //如果是促销活动 或是 通知公告，那就不做后面的刷新。。。
            } else if (fraClass == SaleActivityFrag.class || fraClass == NoticeFra.class) {
                return;
            }
            //显！！！
            fragDominator.showFraByTag(pos + "");
            //刷！！！
            ((BaseRefreshFra) fragDominator.getFraWithTag(pos + "")).getPre().refreshAll();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    }

    /**
     * 处理刷新的消息数
     * (通过循环在FraDominator的Fralist 里遍历Fragment，再分别get到对应的Pre，就可以得到对应的tag,即Pre的getName()
     * 然后就可以通过这个tag来匹配是哪个标签，这样处理可以避免写死)
     */
    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMsgCount(MsgCount msg) {
        if (!TextUtils.isEmpty(msg.getTag())) {
            for (MsgActData.LabelModel model : mainPresenter.getData().countModels) {
                if (model.preTag.equals(msg.getTag())) {
                    model.msgCount = msg.getCount();
                    break;
                }
            }
            notifyAdapter();
        }
    }

    @Override
    public void notifyAdapter() {
        if (labelAdapter != null) {
            labelAdapter.notifyDataSetChanged();
        }
    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        EventBus.getDefault().unregister(this);
        if (unbinder != null) {
            unbinder.unbind();
        }
    }
}
