package com.urovo.activity.overduequeryactivity.view

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.LinearLayout
import android.widget.RadioButton
import android.widget.RadioGroup
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.addsellactivity.ui.AutoRadioGroup
import com.urovo.activity.overduequeryactivity.model.OverDueQueryFactory
import com.urovo.activity.overduequeryactivity.model.OverDueQueryModel
import com.urovo.activity.overduequeryactivity.model.OverDueQueryRepo
import com.urovo.activity.overduequeryactivity.ui.OverDueQueryAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.databinding.OverdueQueryActBinding
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.view.DividerItemDecoration
import com.urovo.wugumofang.view.LoadWaitDominator


/**
 * @author Sasuke on 2019/5/21.
 * 临/过期查询
 */
class OverDueQueryActivity : BaseActivity() {

    val model by lazy {
        ViewModelProviders.of(this, OverDueQueryFactory(OverDueQueryRepo())).get(OverDueQueryModel::class.java)
    }

    val binding by lazy {
        DataBindingUtil.setContentView<OverdueQueryActBinding>(this, R.layout.overdue_query_act)
    }

    var loadWait: LoadWaitDominator? = null
    var radioGroup: AutoRadioGroup? = null
    var rVadapter: OverDueQueryAdapter? = OverDueQueryAdapter(mutableListOf())

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bind()
        init()
        initTitle()
        model.queryData(SystemHelper.LoginOrganId)
    }

    fun bind() {
        //初始态
        binding.storeName = SystemHelper.LoginOrganName
        binding.titleLayout.titleText = "临/过期查询"
        //转圈圈状态变换
        loadWait = LoadWaitDominator(this, binding.containerLayout)
        model.loadWaitState.observe(this, Observer {
            loadWait?.show(it)
        })
        //请求返回数据变换
        model.dataCaches.observe(this, Observer {
            it?.let {
                //全非空情况
                if (!it.wgStockLqList.isNullOrEmpty() && !it.sbjStockLqList.isNullOrEmpty()) {
                    binding.titleLayout.relativeLayout1.visibility = View.GONE
                    binding.tapLayout.titleRoot.visibility = View.VISIBLE
                    initRadioTitle(binding.tapLayout.titleContainerLl)
                } else {//反情况，
                    binding.tapLayout.titleRoot.visibility = View.GONE
                    binding.titleLayout.relativeLayout1.visibility = View.VISIBLE
                    binding.titleLayout.titleText = "临期查询"
                    model.currCheckedPos.value = 0
                }
            }
        })

        model.currCheckedPos.observe(this, Observer {
            //有数据，就把它隐藏
            if (!model.getCurrList(it).isNullOrEmpty()){
                loadWait?.show(LoadWaitDominator.DISMISS)
            }
            //刷新数据
            rVadapter?.setDatas(model.getCurrList(it))
        })

    }

    /**
     * 初始化多标签顶栏
     */
    fun initRadioTitle(layout: LinearLayout) {
        radioGroup = AutoRadioGroup(layout)
        arrayOf("五谷磨房", "食补集").forEach {
            val radio: RadioButton = layoutInflater.inflate(R.layout.tap_title_radio_btn, layout, false) as RadioButton
            radioGroup?.addViewWithLayoutParams(view=radio, text=it, paramWith = 0, paramheight = RadioGroup.LayoutParams.MATCH_PARENT) {
                model.currCheckedPos.value = it
            }
        }
        //选中
        radioGroup?.setChildChecked(model.currCheckedPos.value!!)
    }

    fun init() {
        //因为是BaseActivity（），所以 initTitle()只顾及到了titleLayout的顶栏，而tapLayout顶栏要自己写返回响应了。
        binding.tapLayout.backLl.setOnClickListener { finish() }
        binding.dataRv.run {
            adapter = rVadapter
            addItemDecoration(DividerItemDecoration(context, DividerItemDecoration.HORIZONTAL_LIST, 1, Color.parseColor("#E3E7EE")))
            layoutManager = LinearLayoutManager(this@OverDueQueryActivity)
        }

    }

}