package com.urovo.activity.personalcenter.presenter

import com.urovo.activity.personalcenter.model.VoucherCardDataBean
import com.urovo.activity.personalcenter.model.VoucherCardQueryBean
import com.urovo.activity.personalcenter.model.VoucherCardUserInfoBean
import com.urovo.activity.personalcenter.model.VoucherCardUserInfoRequest
import com.urovo.activity.personalcenter.presenter.contract.VoucherCardContract
import com.urovo.activity.personalcenter.service.depository.VoucherCardDepository
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isEmptyParameter
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Response

/**
 * 权益卡查询 Presenter
 * @author LW
 */
class VoucherCardPresenter(
    private val view: VoucherCardContract.IView?,
    private var depository: VoucherCardDepository = VoucherCardDepository()
) : AbsPresenter(), VoucherCardContract.IPresenter {

    override fun requestVoucherCardData(ymd: String?) {
        if (isEmptyParameter(ymd)) {
            view?.showError(R.string.common_load_error)
            return
        }

        view?.showIsLoading()
        depository.requestVoucherCardData(ymd!!, SystemHelper.LoginUserId)
            .enqueue(object : RetrofitCallback<RequestResult<VoucherCardDataBean>>() {
                override fun onResponse(
                    call: Call<RequestResult<VoucherCardDataBean>>,
                    res: Response<RequestResult<VoucherCardDataBean>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            if (resultInfo != null) {
                                view?.voucherCardData(resultInfo)
                            } else {
                                view?.showState(LoadWaitDominator.NO_DATA)
                            }
                        } else {
                            view?.showError(msgInfo)
                        }
                    } ?: view?.showState(LoadWaitDominator.NET_ERROR)

                    view?.dismissLoading()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissLoading()
                    view?.showError(exception.message)
                }
            })
    }

    override fun requestVoucherCardInfo(mobile: String?) {
        if (isEmptyParameter(mobile)) {
            view?.showError(R.string.common_load_error)
            return
        }

        view?.showIsLoading()
        depository.requestVoucherCardInfo(mobile!!)
            .enqueue(object : RetrofitCallback<RequestResult<VoucherCardQueryBean>>() {
                override fun onResponse(
                    call: Call<RequestResult<VoucherCardQueryBean>>,
                    res: Response<RequestResult<VoucherCardQueryBean>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            if ((resultInfo.converRecords?.size
                                    ?: 0) + (resultInfo.purchaseCardRecords?.size
                                    ?: 0) + (resultInfo.megerRecords?.size ?: 0) > 0
                            ) {
                                view?.voucherCardQuery(resultInfo)
                            } else {
                                view?.voucherCardQuery(null)
                            }
                        } else {
                            view?.voucherCardQuery(null)
                        }
                    } ?: view?.showState(LoadWaitDominator.NET_ERROR)

                    view?.dismissLoading()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissLoading()
                    view?.showError(exception.message)
                }
            })
    }

    override fun requestVoucherCardFailurePostpone(memberId: String?) {
        if (isEmptyParameter(memberId)) {
            view?.showError(R.string.common_load_error)
            return
        }

        view?.showTipDialog(R.string.voucher_card_lose_efficacy_postpone)
        depository.requestVoucherCardFailurePostpone(memberId!!)
            .enqueue(object : RetrofitCallback<RequestResult<Any>>() {
                override fun onResponse(
                    call: Call<RequestResult<Any>>,
                    res: Response<RequestResult<Any>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.voucherCardFailureStatus(true)
                        } else {
                            view?.showToast(msgInfo)
                        }
                    } ?: view?.showToast(R.string.common_http_error)
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.showToast(exception.message)
                }
            })

    }

    override fun requestVoucherCardSendMessage(memberId: String?, mobile: String?) {
        if (isEmptyParameter(memberId, mobile)) {
            view?.showError(R.string.common_load_error)
            return
        }
        view?.showTipDialog(R.string.voucher_send_exchange_code_tip)
        depository.requestVoucherCardSendMessage(memberId!!, mobile!!)
            .enqueue(object : RetrofitCallback<RequestResult<Any>>() {
                override fun onResponse(
                    call: Call<RequestResult<Any>>,
                    res: Response<RequestResult<Any>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.voucherCardSendMessageStatus(true)
                        } else {
                            view?.showToast(msgInfo)
                        }
                    } ?: view?.showToast(R.string.common_http_error)
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.showToast(exception.message)
                }
            })

    }

    override fun requestDynamicParameterInfo() {
        depository.requestDynamicParameterInfo()
            .enqueue(object : RetrofitCallback<RequestResult<String>>() {
                override fun onResponse(
                    call: Call<RequestResult<String>>,
                    res: Response<RequestResult<String>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.dynamicParameterInfo(payload)
                        } else {
                            view?.showToast(msgInfo)
                        }
                    } ?: view?.showToast(R.string.common_http_error)
                }

                override fun onFailure(exception: ApiException) {
                    view?.showToast(exception.message)
                }
            })
    }

    override fun requestVoucherCardUserInfo(exchangeCode: String?, orderCode: String?) {
        view?.showTipDialog(R.string.phone_query_title)
        depository.requestVoucherCardUserInfo(VoucherCardUserInfoRequest(exchangeCode, orderCode))
            .enqueue(object : RetrofitCallback<RequestResult<VoucherCardUserInfoBean>>() {
                override fun onResponse(
                    call: Call<RequestResult<VoucherCardUserInfoBean>>,
                    res: Response<RequestResult<VoucherCardUserInfoBean>>
                ) {
                    val body = res.body()
                    body.takeIf { it != null }?.run {
                        if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                            view?.voucherCardUserInfo(payload)
                        } else {
                            view?.showToast(msgInfo)
                        }
                    } ?: view?.showToast(R.string.common_http_error)
                    view?.dismissDialog()
                }

                override fun onFailure(exception: ApiException) {
                    view?.dismissDialog()
                    view?.showToast(exception.message)
                }
            })
    }
}