package com.urovo.activity.personalcenter.service.depository

import com.urovo.activity.personalcenter.model.VoucherCardDataBean
import com.urovo.activity.personalcenter.model.VoucherCardQueryBean
import com.urovo.activity.personalcenter.model.VoucherCardUserInfoBean
import com.urovo.activity.personalcenter.model.VoucherCardUserInfoRequest
import com.urovo.activity.personalcenter.service.VoucherCardApiService
import com.urovo.wugumofang.net.RetrofitHelper
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call

/**
 * 权益卡查询数据仓库
 * @author LW
 */
class VoucherCardDepository {

    private val mApiService by lazy {
        RetrofitHelper.createApi(VoucherCardApiService::class.java)
    }

    private val mNewApiService by lazy {
        RetrofitHelper.createNewApi(VoucherCardApiService::class.java)
    }

    /**
     * 权益卡数据
     */
    fun requestVoucherCardData(
        ymd: String,
        staffId: String
    ): Call<RequestResult<VoucherCardDataBean>> {
        return mApiService.getVoucherCardData(ymd, staffId)
    }

    /**
     * 权益卡查询
     * @param mobile:String 手机号
     */
    fun requestVoucherCardInfo(mobile: String): Call<RequestResult<VoucherCardQueryBean>> {
        return mApiService.getVoucherCardInfo(mobile)
    }

    /**
     * 权益卡失效延期
     * @param memberId:String 券Id
     */
    fun requestVoucherCardFailurePostpone(memberId: String): Call<RequestResult<Any>> {
        return mNewApiService.getVoucherCardFailurePostpone(memberId)
    }

    /**
     * 发送权益卡二维码短信到手机
     * @param memberId:String 券Id
     * @param mobile: String 手机号
     */
    fun requestVoucherCardSendMessage(memberId: String,mobile:String): Call<RequestResult<Any>> {
        return mNewApiService.getVoucherCardSendMessage(memberId,mobile)
    }

    /**
     * 动态参数配置详情
     */
    fun requestDynamicParameterInfo(): Call<RequestResult<String>> {
        return mNewApiService.getDynamicParameterInfo("BENEFIT_CARD_H5_URL")
    }

    /**
     * 权益卡兑换码或商城订单编号查询购买者信息
     * @param exchangeCode String 	权益卡优惠券兑换码
     * @param orderCode String 权益卡商城订单编号
     */
    fun requestVoucherCardUserInfo(request: VoucherCardUserInfoRequest): Call<RequestResult<VoucherCardUserInfoBean>> {
        return mNewApiService.getVoucherCardUserInfo(request)
    }
}