package com.urovo.activity.personalcenter.ui.mergemember

import android.text.Spannable
import android.text.SpannableStringBuilder
import android.text.style.ForegroundColorSpan
import android.view.View
import androidx.core.content.ContextCompat
import com.alibaba.android.arouter.facade.annotation.Route
import com.urovo.activity.personalcenter.model.MergeMemberEntity
import com.urovo.activity.personalcenter.presenter.MergeMemberPresenter
import com.urovo.activity.personalcenter.presenter.contract.MergeMemberContract
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.constants.RouterParams
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.convertDefaultValue
import com.urovo.wugumofang.utils.notNullAndEmpty
import kotlinx.android.synthetic.main.activity_merge_member.*

/**
 * 合并会员页面
 * @author LW
 */
@Route(path = RouterParams.Member.MERGE_MEMBER)
class MergeMemberActivity : BaseActivity(), MergeMemberContract.IView {

    private var mMergeMemberPresenter: MergeMemberPresenter? = null

    override fun addRootLayoutId(): Int = R.layout.activity_merge_member

    private var mMergeNewMemberEntity: MergeMemberEntity? = null

    private var mMergeOldMemberEntity: MergeMemberEntity? = null

    override fun addPresenter(): BasePresenter {
        mMergeMemberPresenter = MergeMemberPresenter(this)
        return mMergeMemberPresenter!!
    }

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.merge_member_title)
    }

    override fun initViewNew() {
        initTitle()

        merge_member_page_desc_tv.setText(SpannableStringBuilder(getString(R.string.merge_member_page_desc)).apply {
            setSpan(
                ForegroundColorSpan(ContextCompat.getColor(mContext, R.color.common_two_txt_color)),
                0,
                56,
                Spannable.SPAN_EXCLUSIVE_INCLUSIVE
            )
            setSpan(
                ForegroundColorSpan(ContextCompat.getColor(mContext, R.color.color_common_red_two)),
                56,
                60,
                Spannable.SPAN_EXCLUSIVE_INCLUSIVE
            )
        })

        merge_member_new_title_tv.setText(SpannableStringBuilder(getString(R.string.merge_member_new_mobile)).apply {
            setSpan(
                ForegroundColorSpan(ContextCompat.getColor(mContext, R.color.color_common_red_two)),
                0,
                1,
                Spannable.SPAN_EXCLUSIVE_INCLUSIVE
            )
            setSpan(
                ForegroundColorSpan(ContextCompat.getColor(mContext, R.color.common_two_txt_color)),
                1,
                6,
                Spannable.SPAN_EXCLUSIVE_INCLUSIVE
            )
            setSpan(
                ForegroundColorSpan(ContextCompat.getColor(mContext, R.color.color_common_red_one)),
                6,
                23,
                Spannable.SPAN_EXCLUSIVE_INCLUSIVE
            )
        })

        merge_member_old_title_tv.setText(SpannableStringBuilder(getString(R.string.merge_member_old_mobile)).apply {
            setSpan(
                ForegroundColorSpan(ContextCompat.getColor(mContext, R.color.color_common_red_two)),
                0,
                1,
                Spannable.SPAN_EXCLUSIVE_INCLUSIVE
            )
            setSpan(
                ForegroundColorSpan(ContextCompat.getColor(mContext, R.color.common_two_txt_color)),
                1,
                6,
                Spannable.SPAN_EXCLUSIVE_INCLUSIVE
            )
        })
        merge_member_new_query_btn.setOnClickListener {
            val newMobile = merge_member_new_search_et.text.trim().toString()
            if (newMobile.notNullAndEmpty()) {
                merge_member_new_name_tv.text = "".convertDefaultValue()
                merge_member_new_id_tv.text = "".convertDefaultValue()
                merge_member_new_mobile_tv.text = "".convertDefaultValue()
                mMergeMemberPresenter?.requestMemberInfoNewMobile(newMobile)
            } else {
                merge_member_new_search_et.setShakeAnimation()
            }
        }
        merge_member_new_search_et.setOnClearClickListener {
            mergeMemberInfo(ConstantParams.CONSTANT_STARTS_2,null)
        }
        merge_member_old_query_btn.setOnClickListener {
            val oldMobile = merge_member_old_search_et.text.trim().toString()
            if (oldMobile.notNullAndEmpty()) {
                merge_member_old_name_tv.text = "".convertDefaultValue()
                merge_member_old_id_tv.text = "".convertDefaultValue()
                merge_member_old_mobile_tv.text = "".convertDefaultValue()
                mMergeMemberPresenter?.requestMemberInfoOldMobile(
                    oldMobile,
                    SystemHelper.LoginOrganId
                )
            } else {
                merge_member_old_search_et.setShakeAnimation()
            }
        }
        merge_member_old_search_et.setOnClearClickListener {
            mergeMemberInfo(ConstantParams.CONSTANT_STARTS_1,null)
        }
        merge_member_submit_btn.setOnClickListener {
            mMergeMemberPresenter?.requestMergeMemberInfoExecute(
                mMergeOldMemberEntity?.cusMobile,
                mMergeOldMemberEntity?.cusId.toString(), mMergeNewMemberEntity?.cusMobile,
                mMergeNewMemberEntity?.cusId.toString(), SystemHelper.LoginUserId
            )
        }
    }

    private fun clearViewAllData() {
        merge_member_new_search_et.setText("")
        merge_member_new_empty_iv.visibility = View.VISIBLE
        merge_member_new_info_ll.visibility = View.GONE
        merge_member_new_name_tv.text = "".convertDefaultValue()
        merge_member_new_id_tv.text = "".convertDefaultValue()
        merge_member_new_mobile_tv.text = "".convertDefaultValue()

        merge_member_old_search_et.setText("")
        merge_member_old_empty_iv.visibility = View.VISIBLE
        merge_member_old_info_ll.visibility = View.GONE
        merge_member_old_name_tv.text = "".convertDefaultValue()
        merge_member_old_id_tv.text = "".convertDefaultValue()
        merge_member_old_mobile_tv.text = "".convertDefaultValue()
    }

    override fun mergeMemberInfo(type: Int, mergeMemberBean: MergeMemberEntity?) {
        if (type == ConstantParams.CONSTANT_STARTS_1) {  //旧手机信息
            this.mMergeOldMemberEntity = mergeMemberBean
            if (mergeMemberBean != null) {
                merge_member_old_empty_iv.visibility = View.GONE
                merge_member_old_info_ll.visibility = View.VISIBLE
                mergeMemberBean.apply {
                    merge_member_old_name_tv.text = cusName.convertDefaultValue()
                    merge_member_old_id_tv.text = cusId.toString().convertDefaultValue()
                    merge_member_old_mobile_tv.text = cusMobile.convertDefaultValue()
                }
            } else {
                merge_member_old_empty_iv.visibility = View.VISIBLE
                merge_member_old_info_ll.visibility = View.GONE
                merge_member_old_name_tv.text = "".convertDefaultValue()
                merge_member_old_id_tv.text = "".convertDefaultValue()
                merge_member_old_mobile_tv.text = "".convertDefaultValue()
            }
        } else if (type == ConstantParams.CONSTANT_STARTS_2) {  //新手机信息
            this.mMergeNewMemberEntity = mergeMemberBean
            if (mergeMemberBean != null) {
                merge_member_new_empty_iv.visibility = View.GONE
                merge_member_new_info_ll.visibility = View.VISIBLE
                mergeMemberBean.apply {
                    merge_member_new_name_tv.text = cusName.convertDefaultValue()
                    merge_member_new_id_tv.text = cusId.toString().convertDefaultValue()
                    merge_member_new_mobile_tv.text = cusMobile.convertDefaultValue()
                }
            } else {
                merge_member_new_empty_iv.visibility = View.VISIBLE
                merge_member_new_info_ll.visibility = View.GONE
                merge_member_new_name_tv.text = "".convertDefaultValue()
                merge_member_new_id_tv.text = "".convertDefaultValue()
                merge_member_new_mobile_tv.text = "".convertDefaultValue()
            }
        }
    }

    override fun mergeMemberInfoExecuteStatus(status: Boolean) {
        if (status) {
            showToast(R.string.merge_member_success)
            clearViewAllData()
        }
    }
}