package com.urovo.activity.personalcenter.ui.phone

import android.view.View
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.personalcenter.model.VoucherCardUserInfoBean
import com.urovo.activity.personalcenter.presenter.VoucherCardPresenter
import com.urovo.activity.personalcenter.presenter.contract.VoucherCardContract
import com.urovo.activity.personalcenter.ui.phone.adapter.PhoneChooseModeListAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.utils.convertDefaultValue
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.urovo.wugumofang.view.DividerItemDecoration
import com.urovo.wugumofang.view.dialog.PublicSelectBottomDialog
import com.zsy.libext.widgets.clipboard.ClipboardUtil
import kotlinx.android.synthetic.main.activity_phone_query.*

/**
 * 手机号查询页面
 * @author LW
 */
class PhoneQueryActivity : BaseActivity(), VoucherCardContract.IView {

    private var mVoucherCardPresenter: VoucherCardPresenter? = null

    override fun addRootLayoutId(): Int = R.layout.activity_phone_query

    private var mChooseMode: Int = ConstantParams.CONSTANT_STARTS_0

    override fun addPresenter(): BasePresenter {
        mVoucherCardPresenter = VoucherCardPresenter(this)
        return mVoucherCardPresenter!!
    }

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.phone_query_title)
    }

    override fun initViewNew() {
        initTitle()

        phone_choose_mode_tv.setOnClickListener {
            showChooseModeBottomDialog()
        }

        phone_search_btn.setOnClickListener {
            val searchCode = phone_search_et.text.trim().toString()
            if (mChooseMode == ConstantParams.CONSTANT_STARTS_0) {
                mVoucherCardPresenter?.requestVoucherCardUserInfo(searchCode, "")
            } else if (mChooseMode == ConstantParams.CONSTANT_STARTS_1) {
                mVoucherCardPresenter?.requestVoucherCardUserInfo("", searchCode)
            }
        }

        phone_copy_btn.setOnClickListener {
            val phoneNumber = phone_member_phone_number_tv.tag.toString()
            if (phoneNumber.notNullAndEmpty()) {
                ClipboardUtil.copyText(mContext, phoneNumber)
                showToast(R.string.phone_copy_success)
            }
        }
    }

    private fun showChooseModeBottomDialog() {
        val chooseModeLists = mutableListOf(
            getString(R.string.phone_redemption_code_query),
            getString(R.string.phone_shop_order_query)
        )
        val builder = PublicSelectBottomDialog.Companion.Builder(this)
        builder.isDataEntry(chooseModeLists.isNullOrEmpty())
        builder.setTitle(R.string.phone_choose_search_type)
        builder.setDialogListLoadDataListener(object :
            PublicSelectBottomDialog.OnDialogListLoadDataListener {
            override fun onListLoadData(dialog: PublicSelectBottomDialog, list: RecyclerView) {
                val phoneChooseModeListAdapter =
                    PhoneChooseModeListAdapter(chooseModeLists)
                list.apply {
                    visibility = View.VISIBLE
                    layoutManager = LinearLayoutManager(mContext)
                    adapter = phoneChooseModeListAdapter
                    addItemDecoration(
                        DividerItemDecoration(
                            mContext,
                            DividerItemDecoration.VERTICAL_LIST,
                            1,
                            ContextCompat.getColor(mContext, R.color.pop_query_devider)
                        )
                    )
                }
                phoneChooseModeListAdapter.setOnItemClickListener { baseQuickAdapter, view, position ->
                    mChooseMode = position
                    if (position == ConstantParams.CONSTANT_STARTS_0) {
                        phone_search_et.setHint(R.string.phone_input_voucher_redemption_code)
                        phone_choose_mode_tv.setText(R.string.phone_redemption_code_query)
                    } else if (position == ConstantParams.CONSTANT_STARTS_1) {
                        phone_search_et.setHint(R.string.phone_input_shop_order)
                        phone_choose_mode_tv.setText(R.string.phone_shop_order_query)
                    }
                    dialog.dismiss()
                }
            }
        })
        builder.create().show()
    }

    override fun voucherCardUserInfo(voucherCardUserInfoBean: VoucherCardUserInfoBean?) {
        super.voucherCardUserInfo(voucherCardUserInfoBean)
        voucherCardUserInfoBean?.apply {
            phone_member_id_tv.text =
                getString(R.string.phone_member_id, cusId.toString().convertDefaultValue())
            phone_member_nickname_tv.text =
                getString(R.string.phone_member_nickname, cusNickname.convertDefaultValue())
            phone_member_phone_number_tv.text =
                getString(R.string.phone_member_phone_number, cusMobile.convertDefaultValue())
            if(cusMobile.notNullAndEmpty()) {
                phone_member_phone_number_tv.tag = cusMobile
                phone_copy_btn.visibility=View.VISIBLE
            }else{
                phone_copy_btn.visibility=View.GONE
            }
            phone_member_content_ll.visibility = View.VISIBLE
        } ?: apply {
            phone_member_content_ll.visibility = View.GONE
        }
    }
}