package com.urovo.activity.personalcenter.ui.voucher

import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alibaba.android.arouter.facade.annotation.Route
import com.urovo.activity.personalcenter.model.VoucherCardDataBean
import com.urovo.activity.personalcenter.presenter.VoucherCardPresenter
import com.urovo.activity.personalcenter.presenter.contract.VoucherCardContract
import com.urovo.activity.personalcenter.ui.voucher.adapter.VoucherCardDataAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.RouterParams
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.DialogHelper
import com.urovo.wugumofang.view.DividerUniteItemDecoration
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.act_voucher_header.*
import kotlinx.android.synthetic.main.activity_voucher_card_data.*
import java.util.*

/**
 * 权益卡数据页面
 * @author LW
 */
@Route(path = RouterParams.Member.VOUCHER_CARD_DATA)
class VoucherCardDataActivity : BaseActivity(), VoucherCardContract.IView {

    private var mVoucherCardPresenter: VoucherCardPresenter? = null

    private var mVoucherCardDataAdapter: VoucherCardDataAdapter? = null

    private var mVoucherCardDataBean: VoucherCardDataBean? = null

    private var mType: Int = 1

    private var mDate: String = DateUtil.getDate(Date())

    override fun rootViewGroup(): ViewGroup = mVoucherRootFl

    override fun addRootLayoutId(): Int = R.layout.activity_voucher_card_data

    override fun addPresenter(): BasePresenter {
        mVoucherCardPresenter = VoucherCardPresenter(this).apply {
            requestVoucherCardData(mDate)
        }
        return mVoucherCardPresenter!!
    }

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.voucher_card_data_title)
    }

    override fun initViewNew() {
        initTitle()

        date_text.text = mDate
        name_text.text = ",${SystemHelper.LoginUserName}"

        mVoucherCardDataAdapter = VoucherCardDataAdapter( mutableListOf(), mType)
        mVoucherListRv.apply {
            layoutManager =
                LinearLayoutManager(this@VoucherCardDataActivity, RecyclerView.VERTICAL, false)
            addItemDecoration(
                DividerUniteItemDecoration(
                    this@VoucherCardDataActivity,
                    DividerUniteItemDecoration.HORIZONTAL_LIST, 2, ContextCompat.getColor(
                        this@VoucherCardDataActivity, R.color.color_common_devider_line_one
                    )
                )
            )
            adapter = mVoucherCardDataAdapter
        }

        date_text_ll.setOnClickListener {
            DialogHelper.alertDatePicker(this, this, date_text)
        }

        mChooseRg.setOnCheckedChangeListener { _, checkedId ->
            when (checkedId) {
                R.id.mCardOpenCustomerDetailsRb -> {
                    this.mType = 1
                }
                R.id.mFirstWriteOffDetailsRb -> {
                    this.mType = 2
                }
            }
            refreshData()
        }
    }

    override fun onDialogResult(tag: Int) {
        mDate = date_text.text.toString()
        if (mDate.isNotEmpty()) {
            mVoucherCardPresenter?.requestVoucherCardData(mDate)
        }
    }

    private fun refreshData() {
        mVoucherCardDataAdapter?.setType(mType)
        if (mType == 1) {
            mFlagTv.text = getString(R.string.voucher_whether_card)
            mConsMoneyTv.visibility = View.GONE

            mVoucherCardDataBean?.cardsList?.let {
                if (it.isNotEmpty()) {
                    mVoucherCardDataAdapter?.setNewData(it)
                } else {
                    showState(LoadWaitDominator.NO_DATA)
                }
            }

        } else if (mType == 2) {
            mFlagTv.text = getString(R.string.voucher_write_off)
            mConsMoneyTv.visibility = View.VISIBLE
            mVoucherCardDataBean?.consList?.let {
                if (it.isNotEmpty()) {
                    mVoucherCardDataAdapter?.setNewData(it)
                } else {
                    showState(LoadWaitDominator.NO_DATA)
                }
            }
        }
    }

    override fun voucherCardData(voucherCardBean: VoucherCardDataBean?) {
        super.voucherCardData(voucherCardBean)
        mVoucherCardDataBean = voucherCardBean?.apply {
            mCanCardNumberTv.text = cardsAvailableNum
            cardNumberTv.text = cardsNum
            mOpenRateTv.text = cardsRate
        }
        refreshData()
    }
}