package com.urovo.activity.personalcenter.ui.voucher

import android.annotation.SuppressLint
import android.content.Intent
import android.os.CountDownTimer
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alibaba.android.arouter.facade.annotation.Route
import com.urovo.activity.personalcenter.model.ConverRecordsBean
import com.urovo.activity.personalcenter.model.VoucherCardQueryBean
import com.urovo.activity.personalcenter.model.VoucherMultipleItem
import com.urovo.activity.personalcenter.presenter.VoucherCardPresenter
import com.urovo.activity.personalcenter.presenter.contract.VoucherCardContract
import com.urovo.activity.personalcenter.ui.mergemember.MergeMemberActivity
import com.urovo.activity.personalcenter.ui.phone.PhoneQueryActivity
import com.urovo.activity.personalcenter.ui.voucher.adapter.VoucherCardQueryAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.base.BasePresenter
import com.urovo.wugumofang.constants.RouterParams
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.DensityUtil
import com.urovo.wugumofang.ui.SpacesItemDecoration
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.urovo.wugumofang.view.baseWebViewAct.WebViewAct
import kotlinx.android.synthetic.main.activity_voucher_card_query.*
import java.lang.ref.WeakReference

/**
 * 权益卡查询页面
 * @author LW
 */
@Route(path = RouterParams.Member.VOUCHER_CARD_QUERY)
class VoucherCardQueryActivity : BaseActivity(), VoucherCardContract.IView {
    private var mVoucherMultipleList = mutableListOf<VoucherMultipleItem<*>>()

    private var mVoucherCardPresenter: VoucherCardPresenter? = null

    private var mVoucherCardQueryAdapter: VoucherCardQueryAdapter? = null

    override fun rootViewGroup(): ViewGroup = mVoucherRootFl

    override fun addRootLayoutId(): Int = R.layout.activity_voucher_card_query

    private var mMemberPhone: String? = null

    private var mVoucherCardAddress: String? = null

    private var mVerifyCodeView: TextView? = null

    override fun addPresenter(): BasePresenter {
        mVoucherCardPresenter = VoucherCardPresenter(this).apply {
            val loginUserMobile = SystemHelper.LoginUserMobile
            requestVoucherCardInfo(loginUserMobile)
        }
        return mVoucherCardPresenter!!
    }

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.voucher_card_query_title)
    }

    override fun initViewNew() {
        initTitle()
        mVoucherCardQueryAdapter = VoucherCardQueryAdapter(mutableListOf(),
            object : VoucherCardQueryAdapter.OnItemChildClickListener {
                override fun onPostponeClick(position: Int, bean: ConverRecordsBean) {
                    val memberId = bean.memberId
                    mVoucherCardPresenter?.requestVoucherCardFailurePostpone(memberId)
                }

                override fun onQueryClick(position: Int, bena: ConverRecordsBean) {
                    if (mVoucherCardAddress.notNullAndEmpty()) {
                        WebViewAct.launch(
                            this@VoucherCardQueryActivity,
                            SystemHelper.LoginUserId,
                            getString(R.string.voucher_card_query_title),
                            mVoucherCardAddress
                        )
                    } else {
                        showToast(R.string.voucher_card_config_code_error)
                    }
                }

                override fun onSendConverCodeClick(
                    view: TextView,
                    position: Int,
                    bean: ConverRecordsBean
                ) {
                    mVerifyCodeView = view
                    if (bean.sendMessageNum > 0) {
                        val memberId = bean.memberId
                        mVoucherCardPresenter?.requestVoucherCardSendMessage(memberId, mMemberPhone)
                    } else {
                        showToast(R.string.voucher_exchange_code_quantity_exceeds_tip)
                    }
                }
            })
        mVoucherListRv.apply {
            layoutManager =
                LinearLayoutManager(this@VoucherCardQueryActivity, RecyclerView.VERTICAL, false)
            adapter = mVoucherCardQueryAdapter
            addItemDecoration(
                SpacesItemDecoration(
                    DensityUtil.dip2px(mContext, 15f),
                    adapter!!.itemCount,
                    true
                )
            )
        }
        mVoucherCardPresenter?.requestDynamicParameterInfo()
        mSearchBtn.setOnClickListener {
            mMemberPhone = mSearchCustomerEt.text.toString().replace(" ","")
            if (mMemberPhone.notNullAndEmpty()) {
                mVoucherCardPresenter?.requestVoucherCardInfo(mMemberPhone)
            } else {
                showMsg("输入会员手机号查询")
            }
        }
        mVoucherRetrievePhoneNumberBtn.setOnClickListener {
            jumpPhoneQueryActivity()
        }
    }

    /**
     * 跳转到PhoneQueryActivity
     */
    private fun jumpPhoneQueryActivity() {
        startActivity(Intent(mContext, PhoneQueryActivity::class.java))
    }

    override fun voucherCardQuery(voucherCardBean: VoucherCardQueryBean?) {
        super.voucherCardQuery(voucherCardBean)
        if (voucherCardBean != null) {
            mVoucherMultipleList = mutableListOf()
            mVoucherMultipleList.add(
                VoucherMultipleItem(
                    VoucherMultipleItem.PURCHASE_CARD_RECORD,
                    voucherCardBean,
                    getString(R.string.voucher_purchase_card_records)
                )
            )
            mVoucherMultipleList.add(
                VoucherMultipleItem(
                    VoucherMultipleItem.CONVER_RECORD,
                    voucherCardBean,
                    getString(R.string.voucher_conver_records)
                )
            )
            /*
        mVoucherMultipleList.add(
            VoucherMultipleItem(
                VoucherMultipleItem.MEGER_RECORD,
                voucherCardBean,
                getString(R.string.voucher_meger_records)
            )
        )
        */
            mVoucherCardQueryAdapter?.setNewData(mVoucherMultipleList)
        } else {
            showNoData(
                R.string.voucher_card_member_info_not,
                R.string.voucher_card_to_merge_member_info
            ) {
                startActivity(Intent(mContext, MergeMemberActivity::class.java))
            }
        }
    }

    override fun voucherCardFailureStatus(status: Boolean) {
        super.voucherCardFailureStatus(status)
        if (mMemberPhone.notNullAndEmpty()) {
            mVoucherCardPresenter?.requestVoucherCardInfo(mMemberPhone)
        }
    }

    override fun voucherCardSendMessageStatus(status: Boolean) {
        super.voucherCardSendMessageStatus(status)
        if (status) {
            if (mVerifyCodeView != null) {
                countDownWork(mVerifyCodeView!!)
            }
            showToast(R.string.voucher_exchange_code_send_tip)
        }
    }

    override fun dynamicParameterInfo(voucherCardAddress: String?) {
        super.dynamicParameterInfo(voucherCardAddress)
        this.mVoucherCardAddress = voucherCardAddress
    }

    /**
     * 验证码倒计时
     */
    private fun countDownWork(view: TextView) {
        val verifyCodeView = WeakReference(view)
        verifyCodeView.get()?.isEnabled = false
        object : CountDownTimer(60000, 1000) {
            @SuppressLint("SetTextI18n")
            override fun onTick(millisUntilFinished: Long) {
                verifyCodeView.get()?.text = "发送兑换码（${millisUntilFinished / 1000}）"
            }

            override fun onFinish() {
                verifyCodeView.get()?.isEnabled = true
                verifyCodeView.get()?.text = "重新获取"
            }
        }.start()
    }
}