package com.urovo.activity.photogallery.view

import android.app.Activity
import android.app.ActivityOptions
import android.content.Context
import android.content.Intent
import android.graphics.BitmapFactory
import android.os.Build
import android.os.Bundle
import android.view.Gravity
import android.view.KeyEvent
import android.view.View
import android.widget.Button
import android.widget.ImageButton
import android.widget.TextView
import android.widget.Toast
import androidx.core.text.HtmlCompat
import androidx.viewpager.widget.ViewPager
import com.bumptech.glide.Glide
import com.urovo.wgmf.adapter.MyPagerAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.Files
import com.urovo.wugumofang.system.DeviceHelper
import com.urovo.wugumofang.ui.PictureUtil
import com.urovo.wugumofang.view.PinchImageView
import kotlinx.android.synthetic.main.photo_gallery_act.*
import org.jetbrains.anko.dip
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.find
import org.jetbrains.anko.uiThread
import java.io.File
import java.net.URL


/**
 * @author Sasuke on 2020-4-21 0021.
 * 图片查看画廊
 */
class PhotoGalleryDominator : BaseActivity() {
    val pagerList = mutableListOf<View>()//用来填充viewPager的组件们
    var photoPaths: ArrayList<String>? = arrayListOf()//传进来的照片url或地址
    val startPhotoIndex by lazy { intent.getIntExtra("startPhotoIndex", 0) }//刚进来此页面时，指定显示的图片序号
    val diyToastView by lazy { layoutInflater.inflate(R.layout.diy_toast_layout, null) }//自定义Toast布局

    companion object {
        @JvmStatic
        fun show(context: Context, photoPaths: ArrayList<String>?, startPhotoIndex: Int = 0) {
            context.startActivity(Intent(context, PhotoGalleryDominator::class.java).apply {
                putStringArrayListExtra("photoPaths", photoPaths)
                putExtra("startPhotoIndex", startPhotoIndex)
            })
        }

        /**带转场动画的*/
        @JvmStatic
        fun showWithTrans(context: Context?, photoPaths: ArrayList<String>?, view: View, startPhotoIndex: Int = 0) {
            context?.let {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {//只有安卓5以上才支持
                    val viewPair: android.util.Pair<View, String> = android.util.Pair.create(view, "view")
                    context.startActivity(Intent(context, PhotoGalleryDominator::class.java).apply {
                        putStringArrayListExtra("photoPaths", photoPaths)
                        putExtra("startPhotoIndex", startPhotoIndex)
                    }, ActivityOptions.makeSceneTransitionAnimation(context as Activity, viewPair).toBundle())
                } else {
                    show(context, photoPaths, startPhotoIndex)
                }
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.photo_gallery_act)
        initTitle()
        init()
        setListener()
    }

    fun init() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {//只有安卓5以上才支持
            viewpager.transitionName = "view"
        }
        photoPaths = intent.getStringArrayListExtra("photoPaths")
//        photoPaths = arrayListOf(
//                "http://192.168.1.176/chfs/shared/temp/temp.jpg",
//                "http://bizhi.bcoderss.com/wp-content/uploads/2018/01/wallpaper_11.jpg",
//                "http://bbsfiles.vivo.com.cn/vivobbs/attachment/forum/201612/29/202450cmh7ipt8tbzit8pc.jpg",
//                "http://image.res.meizu.com/get/flymebbs/forum/201710/31/175115j19cz16ot1zzjm7b.jpg",
//                "http://attach.bbs.miui.com/forum/201708/14/215428ejy3plejiqlzeqq3.jpg",
//                "http://bbsfiles.vivo.com.cn/vivobbs/attachment/forum/201802/10/093147ubmikjhd341hqy31.jpg"
//        )
        val titleText = "${startPhotoIndex + 1}/${photoPaths?.size ?: 0}"
        titleTextView.text = titleText
        photoPaths?.forEach { path ->
            val layout = layoutInflater.inflate(R.layout.photo_gallery_item,null)
            val iv = layout.find<PinchImageView>(R.id.iv)/*.apply { setIsAdjustWidth(true) }*/
//            var iv = PinchImageView(this)
            Glide.with(this@PhotoGalleryDominator)
                    .load(path)
//                    .placeholder(R.drawable.progress_animation_small)
                    .into(iv)
            pagerList.add(layout)
        }
        viewpager.adapter = MyPagerAdapter(this, ArrayList(pagerList))
        viewpager.setCurrentItem(startPhotoIndex, false)
    }

    /**挂监听器*/
    fun setListener() {
        find<ImageButton>(R.id.back).setOnClickListener { onBackPressed() }//不能用 super 里的 back，否则关闭的时候没有动画
        //标题栏序号显示同步
        viewpager.addOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrollStateChanged(state: Int) {
            }

            override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {
            }

            override fun onPageSelected(position: Int) {
                val titleText = "${position + 1}/${photoPaths?.size ?: 0}"
                titleTextView.text = titleText
            }
        })

        //长按监听（保存图片
        pagerList.forEach {
            it.find<PinchImageView>(R.id.iv).setOnLongClickListener {
                doAsync {
                    val currPhotoPath = photoPaths?.get(viewpager.currentItem) ?: ""

                    //================如果已保存过================
                    if (File(Files.tempPicture)
                                    .list({ dir, name -> DeviceHelper.getMD5Str(currPhotoPath) in name }).size != 0) {
                        uiThread { showDIYToast("您已保存过此图片") }
                        return@doAsync
                    }
                    //================如果未保存过================
                    uiThread {
                        titleLayout.find<Button>(R.id.confirm).text = "保存中..."
                        titleLayout.find<Button>(R.id.confirm).visibility = View.VISIBLE
                    }
                    val bmp = BitmapFactory.decodeStream(URL(currPhotoPath).openStream())
                    var fileName = Files.transUrl2Filename(currPhotoPath)
                    PictureUtil.saveBitmapFile(bmp, Files.tempPicture, fileName)
                    uiThread {
                        titleLayout.find<Button>(R.id.confirm).visibility = View.GONE
                        showDIYToast("图片保存成功",
                                HtmlCompat.fromHtml("请前往<font color='#ffcc01'>微信朋友圈</font>选择图片",HtmlCompat.FROM_HTML_MODE_LEGACY))
                    }
                }
                true
            }
        }
    }

    /**弹出自定义Toast*/
    fun showDIYToast(msg: String, msg2 : CharSequence = "") {
        Toast(this).apply {
            //设置Toast要显示的位置，水平居中并在底部，X轴偏移0个单位，Y轴偏移70个单位，
            setGravity(Gravity.CENTER, 0, dip(-100))
            //设置显示时间
            duration = Toast.LENGTH_SHORT
            view = diyToastView
            diyToastView.find<TextView>(R.id.toastText1).text = msg
            diyToastView.find<TextView>(R.id.toastText2).text = msg2
            diyToastView.find<TextView>(R.id.toastText2).visibility = if (msg2.isNotBlank()) View.VISIBLE else View.GONE
        }.show()
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent?): Boolean {
        when (keyCode) {
            KeyEvent.KEYCODE_BACK -> {
                onBackPressed(); return false//要单独对返回键作处理，否则关闭动画有问题
            }
            else -> return super.onKeyDown(keyCode, event)
        }

    }
}