package com.urovo.activity.qualityBackAct.ui;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;

import com.urovo.wugumofang.R;

/**
 * @author Sasuke on 2018/4/27.
 * 上报反馈的到货误差那样的点按钮可以增删元素的布局抽出。
 *
 * 食用方法：
 * View child = new AddItemGroup.Builder(getActivity())
                    .attachTo(goodsContainer)
                    .addChildAt(pos,R.layout.goods_item)
                    .setOnAddListener(new AddItemGroup.OnAddListener() {
                            @Override
                            public void onAdd(int pos) {
                            addChildToLayout(pos);
            }
            })
            .build();
        //......holder里面各种findViewById
        final ChildHolder holder = new ChildHolder();
        //........setTag不要忘了，这样后面提取数据的时候，就可以通过getTag()得到holder再得到每个子组件了。
        child.setTag(holder);

 */
public class AddItemGroup {
    Builder builder;

    public AddItemGroup(Builder builder) {
        this.builder = builder;
    }

    public interface OnAddListener{
        void onAdd(int pos);
    }
    public interface OnRemoveListener{
        void onRemove(int pos);
    }

    public ViewGroup getChild(){
        return builder.child;
    }

    public View getContentView(){
        return builder.contentView;
    }

    //以下Builder===========================
    public static class Builder{
        Context context;
        ViewGroup parentLayout;
        View contentView;
        ViewGroup child;
        OnAddListener addListener;
        OnRemoveListener removeListener;

        public AddItemGroup.Builder setOnAddListener(OnAddListener addListener){
            this.addListener = addListener;
            return this;
        }

        public AddItemGroup.Builder setOnRemoveListener(OnRemoveListener removeListener){
            this.removeListener = removeListener;
            return this;
        }

        public Builder(Context context) {
            this.context = context;
        }

        public AddItemGroup.Builder attachTo(ViewGroup parentLayout){
            this.parentLayout = parentLayout;
            return this;
        }

        /**
         * 在pos位置插入一个新的child
         * @param pos
         */
        public AddItemGroup.Builder addChildAt(int pos, final int contentLayout){
            child = (ViewGroup) LayoutInflater.from(context).inflate(R.layout.goods_item_container,null,false);
            contentView = LayoutInflater.from(context).inflate(contentLayout,null,false);

            if(contentView != null && child != null){
                ((ViewGroup)child.findViewById(R.id.containerLayout)).addView(contentView);
                child.findViewById(R.id.add_goods_item).setTag(child);
                child.findViewById(R.id.remove_goods_item).setTag(child);//增删按钮保存其所在的ChildView，到时候点击时就可以直接吊出来remove了。
                ((ImageButton)child.findViewById(R.id.remove_goods_item)).setImageResource(//这个其实就是在初始时，container为空，默认添加一个时有作用。
                        parentLayout.getChildCount() == 0 ? R.drawable.feedback_minus2 : R.drawable.feedback_minus1);

                //添加按钮点击
                child.findViewById(R.id.add_goods_item).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        int pos = parentLayout.indexOfChild((View)v.getTag());//当前点击按钮所在元素的位置。
                        int childCount = parentLayout.getChildCount();
                        if (pos == 0 && childCount == 1) {//如果是第一个元素的按钮。点了之后要把自己的删除按钮也置彩色。
                            ImageButton removeBtn0 = (ImageButton)child.findViewById(R.id.remove_goods_item);//(ImageButton) ((View)builder.parentLayout.getChildAt(0).getTag()).findViewById(R.id.remove_goods_item);
                            removeBtn0.setImageResource(R.drawable.feedback_minus1);
                        }
                        if(addListener != null){
                            addListener.onAdd(pos + 1);
                        }
                    }
                });
                //删除按钮点击
                child.findViewById(R.id.remove_goods_item).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        int pos = parentLayout.indexOfChild((View)v.getTag());//当前点击按钮所在元素的位置。
                        if(contentView != null && child != null){
                            int childCount = parentLayout.getChildCount();
                            View child = (View) v.getTag();
                            if (childCount > 1) {//多于1个才删除当前ChildView。
                                parentLayout.removeView( child);
                                if(removeListener != null){
                                    removeListener.onRemove(pos);
                                }
                            }
                            if (childCount == 2) {//如果点击的时候已经只剩两个了，那就把仅剩的最后一个的删除按钮置灰。
                                ImageButton removeBtn0 =(ImageButton) parentLayout.getChildAt(0).findViewById(R.id.remove_goods_item);
                                removeBtn0.setImageResource(R.drawable.feedback_minus2);
                            }
                        }
                    }
                });
                parentLayout.addView(child,pos);
            }
            return this;
        }

        public AddItemGroup build(){
            AddItemGroup group = new AddItemGroup(this);
            return group;
        }
    }
}
