package com.urovo.activity.qualityBackAct.view

import android.content.Intent
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.EditText
import android.widget.Spinner
import android.widget.TextView
import androidx.fragment.app.Fragment
import com.github.yoojia.inputs.AndroidNextInputs
import com.github.yoojia.inputs.StaticScheme
import com.github.yoojia.inputs.ValueScheme
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.urovo.activity.qualityBackAct.model.DispatchNotProductEntity
import com.urovo.activity.qualityBackAct.model.DispatchProductEntity
import com.urovo.activity.qualityBackAct.model.QBackInput
import com.urovo.activity.qualityBackAct.net.ArriveErrorJsonEntity
import com.urovo.activity.qualityBackAct.net.BaseEntity
import com.urovo.activity.qualityBackAct.presenter.QBackContract
import com.urovo.activity.qualityBackAct.presenter.QBackInputPresenter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.data.Arith
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.EditTextDecimal
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.utils.notNullAndEmpty
import kotlinx.android.synthetic.main.ll_qback_report_error_show.*
import java.util.*

/**
 * 到货误差问题
 * @author LW
 */
class ArriveErrorFra : Fragment(), QBackContract.view {

    private val mAndroidNextInputs = AndroidNextInputs() //校验框架

    private var mQBackInputPresenter: QBackInputPresenter? = null

    private var mDispatchProductList: List<DispatchProductEntity.DispatchProductBean> = listOf()

    private var mDispatchNotProductList: MutableList<DispatchProductEntity.DispatchProductBean.DispatchDetailBean> =
        mutableListOf()

    private var mDispatchProductPos = -1

    private val mDispatchProductCodeList = arrayListOf<String?>()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        return inflater.inflate(R.layout.ll_qback_report_error_show, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initUI()
    }

    override fun initUI() {
        mQBackInputPresenter?.queryDispatchProduct(
            SystemHelper.LoginOrganId,
            SystemHelper.LoginUserId
        )
        mQBackInputPresenter?.queryDispatchNotProduct(
            SystemHelper.LoginOrganId,
            SystemHelper.LoginUserId
        )
        initReportErrorLayout()
    }

    /**
     * 初始化 到货误差 组件。
     */
    private fun initReportErrorLayout() {
        //提示信息
        head_text.text = getString(R.string.arrive_error_tip)
        //配送波次号
        boci_number.setOnClickListener { v: View ->
            mQBackInputPresenter?.selectorDispatchProduct(
                this@ArriveErrorFra,
                mDispatchProductList,
                mDispatchNotProductList,
                mDispatchProductPos,
                DispatchProductChooseActivity.DELIVERY_BILL
            )
        }
        //收发货日期。
        //shouhuo_date_show.text = DateUtil.format(Date(), "yyyy-MM-dd")
        shouhuo_date_show.setOnClickListener { v: View? ->
            DateUtil.pickDialog(
                activity,
                shouhuo_date_show,
                "yyyy-MM-dd"
            )
        }

        mQBackInputPresenter?.let {
            //外箱包装破损程度
            packing_damage_spinner.adapter = createArrayAdapter(it.arriveErrorData().damageArray)
            //外箱标识是否完整
            packing_id_compl_spinner.adapter = createArrayAdapter(it.arriveErrorData().complArray)
            //物流单签收情况
            ll_sign_state_spinner.adapter = createArrayAdapter(it.arriveErrorData().signArray)
            //误差原因
            ll_deviation_reason_spinner.adapter =
                createArrayAdapter(it.arriveErrorData().deviationArray)
        }
        //收货人、联系方式
        shouhuo_person_show.setText(SystemHelper.LoginUserName)
        telephone_show.setText(SystemHelper.LoginUserMobile)

        //选择波次号产品
        select_all_goods.setOnClickListener { v: View? ->
            mQBackInputPresenter?.selectorDispatchProduct(
                this@ArriveErrorFra,
                mDispatchProductList,
                mDispatchNotProductList,
                mDispatchProductPos,
                DispatchProductChooseActivity.DELIVERY_BILL_PRODUCT
            )
        }
    }

    /**
     * 配送波次号产品信息
     */
    fun refreshDispatchProduct(entity: DispatchProductEntity) {
        mDispatchProductList = entity.resultInfo ?: listOf()
    }

    fun refreshNotDispatchProduct(entity: DispatchNotProductEntity) {
        mDispatchNotProductList.clear()
        entity.resultInfo?.forEach {
            mDispatchNotProductList.add(
                DispatchProductEntity.DispatchProductBean.DispatchDetailBean(
                    false,
                    "",
                    "",
                    it.proCode,
                    it.proName,
                    it.proPrice,
                    "0",
                    "",
                    it.uomName
                )
            )
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            QBackInput.SELECT_MONTH_GOODS -> {  //到货误差 的 选择产品,配送单号
                if (resultCode == ConstantParams.CONSTANT_STARTS_1) {
                    val dispatchProductData =
                        data?.getStringExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_DATA)
                    if (dispatchProductData.notNullAndEmpty()) {
                        mDispatchProductList = Gson().fromJson(
                            dispatchProductData,
                            object :
                                TypeToken<List<DispatchProductEntity.DispatchProductBean>?>() {}.type
                        )
                    }
                    val dispatchNotProductData =
                        data?.getStringExtra(AppConstants.ExtraKeys.DISPATCH_NOT_PRODUCT_DATA)
                    if (dispatchNotProductData.notNullAndEmpty()) {
                        mDispatchNotProductList = Gson().fromJson(
                            dispatchNotProductData,
                            object :
                                TypeToken<List<DispatchProductEntity.DispatchProductBean.DispatchDetailBean>?>() {}.type
                        )
                    }
                    val dispatchProductType =
                        data?.getStringExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_TYPE)

                    if (dispatchProductType == DispatchProductChooseActivity.DELIVERY_BILL) {
                        val pos: Int =
                            data.getIntExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_POS, -1)
                        refreshDeliveryBill(pos)

                    } else if (dispatchProductType == DispatchProductChooseActivity.DELIVERY_BILL_PRODUCT) {
                        refreshDeliveryBillProduct()
                    }
                }
            }
        }
    }

    /**
     * 到货误差  添加子组件。
     * @param pickingCode 提货简码
     */
    private fun addChildToLayout(
        pickingCode: String?,
        dispatchDetailBean: DispatchProductEntity.DispatchProductBean.DispatchDetailBean
    ) {
        dispatchDetailBean.let {
            val child = LayoutInflater.from(context).inflate(R.layout.goods_item, null, false);
            val holder = ChildHolder().apply {
                goodsName = child.findViewById<TextView>(R.id.goods_name).apply {
                    text = it.proName
                    tag = it
                }
                selectDate = child.findViewById(R.id.select_date)
                kcuom = child.findViewById<TextView>(R.id.kcuom).apply { text = it.uomName }
                kcuom2 = child.findViewById<TextView>(R.id.kcuom2).apply { text = it.uomName }
                huodan =
                    child.findViewById<TextView>(R.id.huodan_num_input).apply { text = it.proQty }
                duanque = child.findViewById(R.id.duanque_input)
                shishou =
                    child.findViewById<EditTextDecimal>(R.id.shishou_num_item).apply {
                        addTextChangedListener(object : TextWatcher {
                            override fun beforeTextChanged(
                                s: CharSequence?,
                                start: Int,
                                count: Int,
                                after: Int
                            ) {
                            }

                            override fun onTextChanged(
                                s: CharSequence?,
                                start: Int,
                                before: Int,
                                count: Int
                            ) {
                            }

                            override fun afterTextChanged(s: Editable?) {
                                val huodanValue = if (huodan?.text.toString().isNotEmpty())
                                    huodan?.text.toString().toDouble()
                                else
                                    0.0
                                val shishouValue = if (s.toString().isNotEmpty())
                                    s?.toString()?.toDouble() ?: 0.0
                                else
                                    0.0
                                duanque?.text = StringUtils.formatDouble(
                                    Arith.sub(huodanValue, shishouValue),
                                    "0.00"
                                )
                            }
                        })
                    }

                is_chuanhao = child.findViewById(R.id.is_chuanhao_spinner)
                box_num = child.findViewById<TextView>(R.id.box_num_input).apply {
                    text = pickingCode
                }

                shuoming = child.findViewById(R.id.short_shuoming)
                price = child.findViewById<TextView>(R.id.product_price).apply { text = it.proPrice }

                selectDate?.setOnClickListener {
                    DateUtil.pickDialog(
                        activity,
                        selectDate,
                        "yyyy-MM-dd"
                    )
                }

                is_chuanhao?.adapter =
                    mQBackInputPresenter?.arriveErrorData()?.yesOrNoArray?.let {
                        createArrayAdapter(
                            it
                        )
                    }
            }
            mDispatchProductCodeList.add(it.proCode)

            child.tag = holder
            goods_items_container.addView(child)
        }
    }

    /**
     * 刷新配送波次号产品
     */
    private fun refreshDeliveryBillProduct() {
        if (mDispatchProductPos >= 0) {
            val dispatchProductBean = mDispatchProductList.get(mDispatchProductPos)

            val dispatchDetailList =
                mutableListOf<DispatchProductEntity.DispatchProductBean.DispatchDetailBean>()
            dispatchDetailList.addAll(dispatchProductBean.dispatchDetailList ?: listOf())
            dispatchDetailList.addAll(mDispatchNotProductList)
            val pickingCode = dispatchProductBean.pickingCode
            dispatchDetailList.forEachIndexed { _, bean ->
                if (bean.chooseStatus) {
                    //增加产品
                    if (!mDispatchProductCodeList.contains(bean.proCode)) {
                        addChildToLayout(pickingCode, bean)
                    }
                } else {
                    //删除产品
                    if (mDispatchProductCodeList.contains(bean.proCode)) {
                        removeChildToLayout(bean.proCode)
                    }
                }
            }
        }
    }

    /**
     * 到货误差  删除子组件。
     */
    private fun removeChildToLayout(code: String?) {
        if (code.notNullAndEmpty()) {
            val index = mDispatchProductCodeList.indexOf(code)
            mDispatchProductCodeList.removeAt(index)
            goods_items_container.removeViewAt(index)
        }
    }


    /**
     * 刷新配送波次号
     */
    private fun refreshDeliveryBill(pos: Int) {
        if (pos >= 0 && mDispatchProductPos != pos) {
            mDispatchProductPos = pos

            val dispatchProductBean =
                mDispatchProductList.get(mDispatchProductPos)
            //配送波次号
            boci_number.text = dispatchProductBean.waveNo
            //收货时间
            shouhuo_date_show.text = dispatchProductBean.takeGoodDate
            //应收箱数
            yingshou_num_input.setText(dispatchProductBean.boxUnreceivedNum)
            //实收箱数
            //shishou_num_input.setText(dispatchProductBean?.boxReceivedNum)
            removeAllGoodsData()
        }
    }

    /**
     * 移除GoodsItemContainer中控件,删除产品Id
     */
    private fun removeAllGoodsData() {
        if (goods_items_container.childCount > 0) {
            goods_items_container.removeAllViews()
        }
        for (bean in mDispatchProductList) {
            bean.dispatchDetailList?.forEach {
                it.chooseStatus = false
            }
        }
        mDispatchNotProductList.forEach {
            it.chooseStatus = false
        }
        mDispatchProductCodeList.clear()
    }

    /**
     * 把 到货误差 界面上的数据转成JSON
     */
    private fun transArriveError2Json(): ArriveErrorJsonEntity {
        val entity = ArriveErrorJsonEntity()
        entity.setTitle("到货误差流程") //标题
        //写死的表
        entity.tb1.setPeisbch(boci_number.text.toString())
        entity.tb1.setMendmc(SystemHelper.LoginOrganName)
        entity.tb1.setShouhsj(shouhuo_date_show.text.toString())
        entity.tb1.setShouhrjlxfs(shouhuo_person_show.text.toString())
        entity.tb1.setLianxfs(telephone_show.text.toString())
        entity.tb1.setYingsxs(yingshou_num_input.text.toString())
        entity.tb1.setShisxs(shishou_num_input.text.toString())
        entity.tb1.setWaixbzpscd(packing_damage_spinner.selectedItemPosition - 1)
        entity.tb1.setWaixbssfwz(packing_id_compl_spinner.selectedItemPosition - 1)
        entity.tb1.setWuldqsqk(ll_sign_state_spinner.selectedItemPosition - 1)
        entity.tb1.setWucyy(
            ll_deviation_reason_spinner.selectedItem as String
        )
        //可扩展的表
        val num: Int = goods_items_container.childCount
        for (i in 0 until num) {
            val item = entity.Table2Item()
            val view: View = goods_items_container.getChildAt(i)
            val holder: ChildHolder = view.tag as ChildHolder
            holder.let {
                val dispatchDetailBean =
                    it.goodsName?.tag as DispatchProductEntity.DispatchProductBean.DispatchDetailBean

                item.setChanpmc(it.goodsName?.text.toString()) //产品名
                item.produceDate = it.selectDate?.text.toString() //生产日期
                item.setHuodsl(
                    it.huodan?.text.toString() + it.kcuom?.text.toString()
                ) //货单数量（新需求：数量要拼接上单位）
                item.setShifsl(
                    it.shishou?.text.toString() + it.kcuom2?.text.toString()
                ) //实收数量
                item.setDuanqqk(it.duanque?.text.toString()) //短缺情况
                //item.setJingsr("无") //经手人
                item.setWaixbh(it.box_num?.text.toString()) //提货简码
                item.setJiandsm(it.shuoming?.text.toString()) //简短说明
                item.setWuldm(dispatchDetailBean.proCode) //物料代码（其实就是产品ID）
                item.setChanpjg(dispatchDetailBean.proPrice) //产品价格
                entity.tb2list.add(item)
            }
        }
        return entity
    }

    internal class ChildHolder {
        var goodsName //选择产品
                : TextView? = null
        var selectDate //选择日期
                : TextView? = null
        var huodan //货单数量
                : TextView? = null
        var shishou //实收数量
                : EditTextDecimal? = null
        var duanque //短缺情况
                : TextView? = null
        var is_chuanhao //是否串号
                : Spinner? = null
        var box_num //提货简码
                : TextView? = null
        var shuoming //简短说明
                : EditText? = null
        var kcuom //单位
                : TextView? = null
        var kcuom2 //单位2
                : TextView? = null
        var price //单位2
                : TextView? = null
    }

    /**
     * 检查输入合法性
     */
    private fun checkInput(): Boolean {
        if (packing_damage_spinner.selectedItemPosition == 0) {
            showMsg("请选择 外箱包装破损程度")
            return false
        } else if (packing_id_compl_spinner.selectedItemPosition == 0) {
            showMsg("请选择 外箱标识是否完整")
            return false
        } else if (ll_sign_state_spinner.selectedItemPosition == 0) {
            showMsg("请选择 物流单签收情况")
            return false
        } else if (ll_deviation_reason_spinner.selectedItemPosition == 0) {
            showMsg("请选择 误差原因")
            return false
        }

        mAndroidNextInputs.clear()
        //添加到校验Edittext的框架中。submit的时候一口气校验。
        //添加到Fireeye校验框架中。
        mAndroidNextInputs.add(
            boci_number,
            StaticScheme.Required(),
            ValueScheme.MinLength(12)
        ) //波次号是12位的纯数字
        mAndroidNextInputs.add(shouhuo_person_show, StaticScheme.Required())
        mAndroidNextInputs.add(
            telephone_show,
            StaticScheme.Required(),
            StaticScheme.ChineseMobile()
        )
        mAndroidNextInputs.add(yingshou_num_input, StaticScheme.Required())
        mAndroidNextInputs.add(shishou_num_input, StaticScheme.Required())
        val childNum: Int = goods_items_container.childCount
        if (childNum > 0) {
            for (i in 0 until childNum) {
                val view: View = goods_items_container.getChildAt(i)
                //if ((view.tag as ChildHolder).is_chuanhao?.getSelectedItemPosition() == 0) {
                //    showMsg("请选择 是否串号")
                //    return false
                //}
                //mAndroidNextInputs.add(
                //    (view.tag as ChildHolder).goodsName,
                //    StaticScheme.Required()
                //)
                mAndroidNextInputs.add(
                    (view.tag as ChildHolder).selectDate,
                    StaticScheme.Required().msgOnFail("未选择生产日期")
                )
                mAndroidNextInputs.add(
                    (view.tag as ChildHolder).huodan,
                    StaticScheme.Required()
                )
                mAndroidNextInputs.add(
                    (view.tag as ChildHolder).shishou,
                    StaticScheme.Required()
                )
                mAndroidNextInputs.add(
                    (view.tag as ChildHolder).duanque,
                    StaticScheme.Required()
                )
                mAndroidNextInputs.add(
                    (view.tag as ChildHolder).box_num,
                    StaticScheme.Required()
                )
                mAndroidNextInputs.add(
                    (view.tag as ChildHolder).shuoming,
                    StaticScheme.Required()
                )
            }
        } else {
            showMsg("请选择产品后再提交")
            return false
        }
        return if (mAndroidNextInputs.test()) {
            // 校验通过
            true
        } else {
            showMsg("填写有误，请检查")
            false
        }
    }

    /**
     * 生成给Spinner用的Adapter
     */
    private fun createArrayAdapter(array: Array<String>): ArrayAdapter<String> {
        return ArrayAdapter(
            activity,
            R.layout.gray_spinner_dropdown_item,
            array
        )
    }

    override fun setPresenter(presenter: QBackContract.presenter?) {
        mQBackInputPresenter = presenter as QBackInputPresenter
    }


    override fun showMsg(msg: String?) {
        MyToast.show(msg)
    }

    override fun packData(): BaseEntity? {
        return if (checkInput()) { //如果验证输入都是合法的，那才开始打包输入数据。
            transArriveError2Json()
        } else {
            null
        }
    }

    override fun showProgressBar(title: String?, msg: String?, cancelable: Boolean) {
        ViewUtil.showProgress(activity, title, msg, cancelable)
    }

    override fun dismissProgressBar() {
        ViewUtil.dismissProgress()
    }

}