package com.urovo.activity.qualityBackAct.view

import android.content.Intent
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.EditorInfo
import android.widget.RadioButton
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.urovo.activity.qualityBackAct.model.DispatchProductEntity
import com.urovo.activity.qualityBackAct.ui.DispatchBillChooseAdapter
import com.urovo.activity.qualityBackAct.ui.DispatchBillProductChooseAdapter
import com.urovo.activity.qualityBackAct.ui.DispatchNotBillProductChooseAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.ConstantParams
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.urovo.wugumofang.utils.setOnEditorActionListener
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.ui.ada.helper.DividerItemDecoration
import kotlinx.android.synthetic.main.activity_dispatch_product_choose.*

/**
 * 到货误差 选择波次号/产品
 * @author LW
 */
class DispatchProductChooseActivity : BaseActivity() {

    override fun addRootLayoutId(): Int = R.layout.activity_dispatch_product_choose

    override fun rootViewGroup(): ViewGroup = dispatch_product_choose_container_fl

    private var mDispatchBillChooseAdapter: DispatchBillChooseAdapter? = null

    private var mDispatchBillProductChooseAdapter: DispatchBillProductChooseAdapter? = null

    private var mDispatchNotBillProductChooseAdapter: DispatchNotBillProductChooseAdapter? = null

    private var mDispatchProductList: List<DispatchProductEntity.DispatchProductBean> = listOf()

    private var mDispatchNotProductList: List<DispatchProductEntity.DispatchProductBean.DispatchDetailBean> =
        listOf()

    private val mDispatchProductPos: Int by lazy {
        intent.getIntExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_POS, -1)
    }

    private val mDispathProductType: String by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_TYPE)
    }

    private var mCurrentPos: Int = -1

    private var mKeyWords: String = ""

    companion object {
        const val DELIVERY_BILL: String = "DELIVERY_BILL"

        const val DELIVERY_BILL_PRODUCT: String = "DELIVERY_BILL_PRODUCT"
    }

    override fun initTitle() {
        super.initTitle()
        if (mDispathProductType == DELIVERY_BILL) {
            titleTextView.text = getString(R.string.problem_escalation_chose_delivery_bill)
        } else if (mDispathProductType == DELIVERY_BILL_PRODUCT) {
            titleTextView.text = getString(R.string.choose_product)
        }
    }

    override fun initViewNew() {
        initTitle()
        val dispatchProductData =
            intent.getStringExtra(AppConstants.ExtraKeys.DISPATCH_PRODUCT_DATA)
        if (dispatchProductData.notNullAndEmpty()) {
            mDispatchProductList = Gson().fromJson(
                dispatchProductData,
                object : TypeToken<List<DispatchProductEntity.DispatchProductBean>?>() {}.type
            )
        }
        val dispatchNotProductData =
            intent.getStringExtra(AppConstants.ExtraKeys.DISPATCH_NOT_PRODUCT_DATA)
        if (dispatchNotProductData.notNullAndEmpty()) {
            mDispatchNotProductList = Gson().fromJson(
                dispatchNotProductData,
                object :
                    TypeToken<List<DispatchProductEntity.DispatchProductBean.DispatchDetailBean>?>() {}.type
            )
        }

        mCurrentPos = mDispatchProductPos
        dismissAllState()

        if (mDispathProductType == DELIVERY_BILL) {
            dispatch_product_choose_header_ll.visibility = View.GONE

            if (mDispatchProductList.isNotNullOrSize()) {
                mDispatchBillChooseAdapter =
                    DispatchBillChooseAdapter(
                        mDispatchProductPos,
                        mDispatchProductList
                    ).apply {
                        setOnItemClickListener { _, _, position ->
                            if (mDispatchProductPos >= 0) {
                                mDispatchProductList.get(mDispatchProductPos).dispatchDetailList?.forEach {
                                    it.chooseStatus = false
                                }
                            }

                            mCurrentPos = position
                            choseItemJumpLogic()
                        }
                    }
            } else {
                showState(LoadWaitDominator.NO_DATA)

            }
        } else if (mDispathProductType == DELIVERY_BILL_PRODUCT) {
            dispatch_product_choose_header_ll.visibility = View.VISIBLE

            dispatch_product_choose_wave_order_btn.setOnClickListener {
                dispatch_product_choose_wave_order_btn.isSelected = true
                dispatch_product_choose_not_wave_order_btn.isSelected = false
                waveOrderView()
            }

            dispatch_product_choose_not_wave_order_btn.setOnClickListener {
                dispatch_product_choose_wave_order_btn.isSelected = false
                dispatch_product_choose_not_wave_order_btn.isSelected = true
                notWaveOrderView()
            }

            dispatch_product_choose_total_ll.visibility = View.VISIBLE
            dispatch_product_choose_confirm_btn.setOnClickListener {
                choseItemJumpLogic()
            }

            dispatch_product_choose_num_tv.apply {
                val num = getCalculateSelectedNum()
                text = StringUtils.formatString(
                    mContext.getString(R.string.problem_escalation_selected_product),
                    num
                )
            }

            dispatch_product_choose_search_et.apply {
                setOnEditorActionListener { actionId, _ ->
                    if (actionId == EditorInfo.IME_ACTION_SEARCH) {
                        mKeyWords =
                            dispatch_product_choose_search_et.text.toString().trim { it <= ' ' }
                        notWaveOrderView()
                        return@setOnEditorActionListener true
                    }
                    return@setOnEditorActionListener false
                }

                setOnClearClickListener {
                    mKeyWords = ""
                    notWaveOrderView()
                }
            }
        }

        dispatch_product_choose_rv.apply {
            layoutManager = LinearLayoutManager(mContext, RecyclerView.VERTICAL, false)
            addItemDecoration(
                DividerItemDecoration(
                    mContext,
                    DividerItemDecoration.HORIZONTAL_LIST,
                    1,
                    ContextCompat.getColor(mContext, R.color.color_E8E8E8)
                )
            )
            if (mDispathProductType == DELIVERY_BILL) {
                adapter = mDispatchBillChooseAdapter
            }
        }

        if (mDispathProductType == DELIVERY_BILL_PRODUCT) {
            dispatch_product_choose_wave_order_btn.performClick()
        }
    }

    /**
     * 获取选中个数
     */
    private fun getCalculateSelectedNum(): Int {
        val dispatchDetailList = if (mDispatchProductPos >= 0) {
            mDispatchProductList[mDispatchProductPos].dispatchDetailList
                ?: listOf()
        } else {
            listOf()
        }
        val num = dispatchDetailList.filter { it.chooseStatus }.size +
                mDispatchNotProductList.filter { it.chooseStatus }.size
        return num
    }


    private fun notWaveOrderView() {
        dispatch_product_choose_search_et.visibility = View.VISIBLE

        dismissAllState()
        val list = if (mKeyWords.notNullAndEmpty()) {
            mDispatchNotProductList.filter {
                it.proName?.contains(mKeyWords) ?: false || it.proCode?.contains(
                    mKeyWords
                ) ?: false
            }.toList()
        } else {
            mDispatchNotProductList
        }
        if (mDispatchProductPos >= 0) {
            if (list.isNotNullOrSize()) {
                if (mDispatchNotBillProductChooseAdapter == null) {
                    mDispatchNotBillProductChooseAdapter =
                        DispatchNotBillProductChooseAdapter(list).apply {
                            setOnItemClickListener { baseQuickAdapter, view, position ->
                                val dispatchDetailBean =
                                    baseQuickAdapter.getItem(position) as DispatchProductEntity.DispatchProductBean.DispatchDetailBean
                                dispatchDetailBean.chooseStatus = !dispatchDetailBean.chooseStatus

                                val num = getCalculateSelectedNum()
                                dispatch_product_choose_num_tv.text = StringUtils.formatString(
                                    mContext.getString(R.string.problem_escalation_selected_product),
                                    num
                                )
                                baseQuickAdapter.notifyDataSetChanged()
                            }
                        }
                    dispatch_product_choose_rv.adapter = mDispatchNotBillProductChooseAdapter
                } else {
                    dispatch_product_choose_rv.adapter = mDispatchNotBillProductChooseAdapter
                    mDispatchNotBillProductChooseAdapter?.setNewData(list)
                }
            } else {
                showState(LoadWaitDominator.NO_DATA)
            }
        } else {
            showError(R.string.choose_product_emtry_tip)
        }
    }

    private fun waveOrderView() {
        dispatch_product_choose_search_et.visibility = View.GONE

        dismissAllState()
        var dispatchProductBean: DispatchProductEntity.DispatchProductBean? = null

        val dispatchDetailList = if (mDispatchProductPos >= 0) {
            dispatchProductBean = mDispatchProductList[mDispatchProductPos]
            mDispatchProductList[mDispatchProductPos].dispatchDetailList
                ?: listOf()
        } else {
            listOf()
        }

        if (dispatchDetailList.isNotNullOrSize()) {
            if (mDispatchBillProductChooseAdapter == null) {
                val deliveryBillChooseView =
                    layoutInflater.inflate(
                        R.layout.item_delivery_bill_choose,
                        dispatch_product_choose_search_et.parent as ViewGroup,
                        false
                    ).apply {
                        findViewById<RadioButton>(R.id.select_item_radio).visibility = View.GONE

                        findViewById<TextView>(R.id.delivery_bill).text =
                            mContext.getString(
                                R.string.problem_escalation_delivery_bill,
                                dispatchProductBean?.waveNo
                            )

                        findViewById<TextView>(R.id.delivery_date).text =
                            mContext.getString(
                                R.string.problem_escalation_delivery_date,
                                dispatchProductBean?.dispatchDate
                            )

                        findViewById<TextView>(R.id.delivery_total_amount).text =
                            mContext.getString(
                                R.string.problem_escalation_total_amount,
                                dispatchProductBean?.allProSumAmount
                            )

                        findViewById<TextView>(R.id.delivery_total_number).text =
                            mContext.getString(
                                R.string.problem_escalation_total_number,
                                dispatchProductBean?.allProSumQty
                            )
                    }

                mDispatchBillProductChooseAdapter =
                    DispatchBillProductChooseAdapter(dispatchDetailList).apply {
                        addHeaderView(deliveryBillChooseView)
                        setOnItemClickListener { baseQuickAdapter, view, position ->
                            val dispatchDetailBean =
                                baseQuickAdapter.getItem(position) as DispatchProductEntity.DispatchProductBean.DispatchDetailBean
                            dispatchDetailBean.chooseStatus = !dispatchDetailBean.chooseStatus

                            val num = getCalculateSelectedNum()
                            dispatch_product_choose_num_tv.text = StringUtils.formatString(
                                mContext.getString(R.string.problem_escalation_selected_product),
                                num
                            )
                            baseQuickAdapter.notifyDataSetChanged()
                        }
                    }
                dispatch_product_choose_rv.adapter = mDispatchBillProductChooseAdapter
            } else {
                dispatch_product_choose_rv.adapter = mDispatchBillProductChooseAdapter
                mDispatchBillProductChooseAdapter?.setNewData(dispatchDetailList)
            }
        } else {
            showError(R.string.choose_product_emtry_tip)
        }
    }

    /**
     * 选择列表逻辑处理
     */
    private fun choseItemJumpLogic() {
        setResult(ConstantParams.CONSTANT_STARTS_1, Intent().apply {
            putExtra(
                AppConstants.ExtraKeys.DISPATCH_PRODUCT_DATA,
                Gson().toJson(mDispatchProductList)
            )
            putExtra(
                AppConstants.ExtraKeys.DISPATCH_NOT_PRODUCT_DATA,
                Gson().toJson(mDispatchNotProductList)
            )
            putExtra(
                AppConstants.ExtraKeys.DISPATCH_PRODUCT_POS,
                mCurrentPos
            )
            putExtra(
                AppConstants.ExtraKeys.DISPATCH_PRODUCT_TYPE,
                mDispathProductType
            )
        })
        this.finish()
    }
}