package com.urovo.activity.qualityBackAct.view

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.fragment.app.Fragment
import com.github.yoojia.inputs.AndroidNextInputs
import com.github.yoojia.inputs.Scheme
import com.github.yoojia.inputs.StaticScheme
import com.urovo.activity.common.OrderGoodsChoseSingleActivity
import com.urovo.activity.qualityBackAct.model.QBackInput
import com.urovo.activity.qualityBackAct.net.BaseEntity
import com.urovo.activity.qualityBackAct.net.OverdueEntity
import com.urovo.activity.qualityBackAct.presenter.QBackContract
import com.urovo.activity.qualityBackAct.presenter.QBackInputPresenter
import com.urovo.activity.qualityBackAct.ui.AddItemGroup
import com.urovo.wgmf.db.dao.entitys.ProductEntity
import com.urovo.wgmf.ext.afterTextChanged
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.data.Arith
import com.urovo.wugumofang.data.MoneyUtils
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.DialogHelper
import com.urovo.wugumofang.ui.EditTextDecimal
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.MyToast
import com.wgmf.trade.utils.ArithUtil
import kotlinx.android.synthetic.main.goods_item_overdue.view.*
import kotlinx.android.synthetic.main.ll_qback_overdue.*
import org.jetbrains.anko.find
import org.jetbrains.anko.forEachChild
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author Sasuke on 2018/8/3.
 * 临/过期货报损申请
 */
class OverdueFra : Fragment(), QBackContract.view {
    var presenter: QBackInputPresenter? = null
    val inputs = AndroidNextInputs()//校验框架
    private var selectHolder: ProductHolder? = null//缓存跳转到选择产品页面的TextView

    override fun initUI() {
        if (goodsItemsContainer.getChildCount() == 0) {
            initOverdueLayout()
            addChildToLayout(0)
        }
    }

    override fun showMsg(msg: String?) {
        MyToast.show(msg)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        var view = inflater?.inflate(R.layout.ll_qback_overdue, container, false) as LinearLayout
        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initUI()
    }

    /**
     * 初始化基础布局
     */
    fun initOverdueLayout() {
        overdueAmountTv.text = "0.00元"
        loginOrgNumTv.text = SystemHelper.LoginOrganId
        loginOrgNameTv.text = SystemHelper.LoginOrganName
        zgNameEdit.setText(SystemHelper.LoginZgName)
        submiterEdit.setText(SystemHelper.LoginUserName)
        phoneNumTv.setText(SystemHelper.LoginUserMobile)

    }

    /**
     * 把孩子添加到哪个位置
     */
    fun addChildToLayout(pos: Int) {
        val group = AddItemGroup.Builder(activity)
                .attachTo(goodsItemsContainer)
                .addChildAt(pos, R.layout.goods_item_overdue)
                .setOnAddListener { pos ->
                    //自己调自己，因为此方法里要初始化holder
                    //还要setTag，还有设置选择产品的点击监听
                    //因为封装的里面不知道选择产品的组件是谁
                    addChildToLayout(pos)
                }.setOnRemoveListener {
                    refreshAmount()
                }
                .build()//build（）执行之后，其实就已经把child加载到容器布局里去了。
        val child = group.child//child用来在tag里存holder
        val content = group.contentView//content用来建holder

        val holder = ProductHolder()
        holder.apply {
            reportGoods = content.find(R.id.reportGoods)
            produceDate = content.find(R.id.produceDate)
            proNumTv = content.find(R.id.proNumTv)
            priceTv = content.find(R.id.priceTv)
            kcuom = content.find(R.id.kcuom)
        }


        /**跳转选择商品点击*/
        holder.reportGoods?.setOnClickListener {
            presenter?.selectorGoods(this)
            selectHolder = holder
        }
        /**选择日期点击*/
        holder.produceDate?.setOnClickListener {
            DialogHelper.alertDatePicker(activity as BaseActivity, it as TextView).datePicker.maxDate = Calendar.getInstance().timeInMillis
        }
        /**监听输入数量，即时改变总价显示*/
        holder.proNumTv?.afterTextChanged {
            refreshAmount()
        }
        child.tag = holder
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            QBackInput.SELECT_GOODS -> {
                if (resultCode == OrderGoodsChoseSingleActivity.activityResultCode) {
                    val entity = data?.getSerializableExtra(OrderGoodsChoseSingleActivity.activityResultData) as ProductEntity
                    if (entity != null && selectHolder != null) {
                        selectHolder?.apply {
                            //先在之前点击的组件上显示名字。
                            reportGoods?.text = "${entity.itemId} ${entity.itemName}"
                            priceTv?.text = "${entity.price}元/${entity.kcUom.run {
                                takeIf { "/" in it }?.split("/")?.get(1) ?: this
                            }}"
                            reportGoods?.tag = entity//把产品信息存到文本框的tag里
                            produceDate?.text = ""//置空生产日期
                            proNumTv?.setText("")//置空数量，让用户重新输入
                            proNumTv?.setDecimal(if (entity.isDecimal) 2 else 0)
                            kcuom?.text = entity.kcUom
                        }
                        presenter?.productData()?.product = entity//好像没什么卵用
                    }
                }
            }
        }
    }

    /**
     * 用遍历来重新初始化inputs规则，因为存在删减child的情况，必须要进行clear并重新初始化。
     */
    fun initInputsRules() {
        inputs.clear()
        inputs.add(zgNameEdit, StaticScheme.Required().msgOnFail("请输入主管姓名"))
        inputs.add(submiterEdit, StaticScheme.Required().msgOnFail("请输入姓名"))
        inputs.add(phoneNumTv, StaticScheme.Required(), StaticScheme.ChineseMobile())

        goodsItemsContainer.forEachChild {
            var holder = it.tag as ProductHolder
            inputs.add(holder.reportGoods, StaticScheme.Required().msgOnFail("请选择产品"))
            inputs.add(holder.produceDate, StaticScheme.Required().msgOnFail("请选择日期"))
            inputs.add(holder.proNumTv, StaticScheme.Required(), Scheme.create { s: String? ->
                (s?.toDouble() ?: 0.0) > 0.0
            }.msgOnFail("请输入大于0的数量"))
        }
    }

    /**
     * 刷新总价显示
     */
    fun refreshAmount() {
        var proAmount = 0.00
        goodsItemsContainer?.forEachChild {
            proAmount += Arith.mul((it.reportGoods.tag as ProductEntity?)?.price ?: 0.00,
                    (it.tag as ProductHolder).proNumTv?.text.toString().takeIf { !it.isNullOrEmpty() }?.run { this.toDouble() }
                            ?: 0.00)
        }
        overdueAmountTv.text = "${MoneyUtils.twolastDF(proAmount)}元"
    }

    override fun packData(): BaseEntity? {
        initInputsRules()
        //注：这里的itemId的获取是通过(reportGoods.tag as ProductEntity).ItemId来获取，而不是直接绑在reportGoods的tag上的
        //绑在该tag上的是entity
        if (inputs.test()) {
            goodsItemsContainer.forEachChild {
                if ((it.tag as ProductHolder).produceDate?.text.toString() == SimpleDateFormat("yyyy-MM-dd").format(Date())) {
                    showMsg("生产日期有误差，请核实并重新选择！")
                    return null
                }
            }
        } else {
            MyToast.show(activity, "填写有误，请检查", Toast.LENGTH_SHORT)
            return null
        }
        //打包数据
        val overdueEntity = OverdueEntity().apply {
            amount = overdueAmountTv.text.toString().replace("元", "")
            manager = zgNameEdit.text.toString()
            mobile = phoneNumTv.text.toString()
            shopId = SystemHelper.LoginOrganId
            shopName = SystemHelper.LoginOrganName
            submiter = submiterEdit.text.toString()
            title = presenter?.getData()?.parentType!!.showName

            goodsItemsContainer.forEachChild {
                this.details.add(OverdueEntity.Details().apply {
                    val holder = it.tag as ProductHolder
                    val entity = holder.reportGoods?.tag as ProductEntity
                    prodName = entity.itemName//2.40122 全球智选，需要把前面的id去掉
                    price = entity.price.toString()
                    prodNo = entity.itemId
                    date = holder.produceDate?.text.toString()
                    sum = holder.proNumTv?.text.toString()
                    numberUnit =  holder.kcuom?.text.toString()//新需求，传数量时需要拼接上单位
                    amount = MoneyUtils.twolastDF(ArithUtil.mul(holder.proNumTv?.text.toString().toDouble(), price.toDouble())).toString()
                })
            }
        }
        return overdueEntity
    }

    override fun showProgressBar(title: String?, msg: String?, cancelable: Boolean) {
    }

    override fun dismissProgressBar() {
        ViewUtil.dismissProgress()
    }

    override fun setPresenter(presenter: QBackContract.presenter?) {
        this.presenter = presenter as QBackInputPresenter
    }

    class ProductHolder {
        var reportGoods: TextView? = null//选择产品
        var produceDate: TextView? = null//生产日期
        var proNumTv: EditTextDecimal? = null//产品数量
        var priceTv: TextView? = null//产品单价
        var kcuom: TextView? = null//单位
    }
}