package com.urovo.activity.ra90listact.view

import android.os.Bundle
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.ra90listact.net.Ra90DetailRes
import com.urovo.activity.ra90listact.presenter.Ra90DetailPre
import com.urovo.activity.ra90listact.presenter.Ra90ListCon
import com.urovo.activity.ra90listact.ui.Ra90DetailAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.view.LoadWaitDominator
import com.wgmf.trade.widgets.RecycleViewDivider
import kotlinx.android.synthetic.main.ll_ra90_detail_act.*

/**
 * @author SASUKE on 2018/3/24.
 *  顾客90天未回头名单详情
 */
class Ra90ListDetailAct : BaseActivity(), Ra90ListCon.view {
    companion object {
        const val INTENT_CLIENT_TYPE = "INTENT_CLIENT_TYPE"//标题显示类型
        const val INTENT_MONTH = "INTENT_MONTH"//从上个页面传过来的月份

        const val NO_RETURN = 0//未回访
        const val RETURNED = 1//未回访
    }

    var loadWait: LoadWaitDominator? = null
    var mPre: Ra90DetailPre? = null
    var month: String? = null
    val dataList: MutableList<Ra90DetailRes.ResultInfo.DetailItem> = mutableListOf()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.ll_ra90_detail_act)
        initTitle()
        Ra90DetailPre(this).start()
    }

    /**
     * 根据intent传过来的type类型，来显示界面标题
     */
    fun setTitleText() {
        val title = "顾客90天未回头名单-" + when (intent.getIntExtra(INTENT_CLIENT_TYPE, -1)) {
            Ra90ListAct.ALL_RA -> "全部"
            Ra90ListAct.NEW_RA -> "新客"
            Ra90ListAct.OLD_RA -> "老客"
            else -> ""
        }
        titleTextView.text = title
        noReturnBtn.text = "未回访（0）"
        returnBtn.text = "已回访（0）"
    }

    override fun init() {
        setTitleText()
        radioGroup.setOnCheckedChangeListener { group, checkedId ->
            showData(when (checkedId) {
                noReturnBtn.id -> NO_RETURN
                returnBtn.id -> RETURNED
                else -> -1
            })
        }
        month = intent.getStringExtra(INTENT_MONTH)
        loadWait = LoadWaitDominator(this, dataContainer)
        detailRv.apply {
            layoutManager = LinearLayoutManager(this@Ra90ListDetailAct)
            adapter = Ra90DetailAdapter(this@Ra90ListDetailAct, dataList)
            addItemDecoration(RecycleViewDivider(this@Ra90ListDetailAct, RecyclerView.HORIZONTAL, 1, resources.getColor(R.color.divider_line)))
        }
    }

    override fun onResume() {
        super.onResume()
        month?.let {
            mPre?.queryData(when (true) {
                noReturnBtn.isChecked -> NO_RETURN
                returnBtn.isChecked -> RETURNED
                else -> -1
            }, intent.getIntExtra(INTENT_CLIENT_TYPE, -1), it)
        }//先显示未回访
    }

    override fun showData(labelType: Int) {
        if (this.isFinishing || labelType == -1) {
            return
        }
        mPre?.resSave?.let {
            //返回的不为空
            noReturnBtn.text = "未回访（${it.resultInfo.notCallList?.size ?: 0}）"
            returnBtn.text = "已回访（${it.resultInfo.callList?.size ?: 0}）"
            dataList.apply {
                clear()//清除原来的
                when (labelType) {
                    NO_RETURN -> it.resultInfo.notCallList
                    RETURNED -> it.resultInfo.callList
                    else -> null
                }?.let { addAll(it) }//添加新的
            }
            detailRv.adapter?.notifyDataSetChanged()//刷新
        }
    }

    override fun showLoading() {
        loadWait?.let { it.show(LoadWaitDominator.LOADING) }
    }

    override fun showNetError() {
        loadWait?.let { it.show(LoadWaitDominator.NET_ERROR) }
    }

    override fun showNoData() {
        loadWait?.let { it.show(LoadWaitDominator.NO_DATA) }
    }

    override fun dismissLoadWait() {
        loadWait?.let { it.dismissAll() }
    }

    override fun setPresenter(presenter: Ra90ListCon.pre?) {
        this.mPre = presenter as Ra90DetailPre
    }
}