package com.urovo.activity.report.presenter

import com.urovo.activity.report.model.LogisticsSpecialistEntity
import com.urovo.activity.report.service.LogisticsSpecialistDepository
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.mvp.AbsPresenter
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.net.ApiException
import com.urovo.wugumofang.net.callback.RetrofitCallback
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.utils.isNullOrEmpty
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.model.bean.RequestResult
import retrofit2.Call
import retrofit2.Response

/**
 * 物流专员信息 Presenter
 * @author LW
 */
class LogisticsSpecialistPresenter(
    private var view: LogisticsSpecialistContract.IView?,
    private var depository: LogisticsSpecialistDepository = LogisticsSpecialistDepository()
) : AbsPresenter(), LogisticsSpecialistContract.IPresenter {


    override fun requestLogisticsSpecialistList() {
        val shopId = SystemHelper.LoginOrganId
        if (shopId.isNullOrEmpty()) {
            view?.showError(R.string.common_load_error)
            return
        }
        view?.showIsLoading()
        depository.requestLogisticsSpecialistList(shopId).enqueue(object :
            RetrofitCallback<RequestResult<List<LogisticsSpecialistEntity>>>() {
            override fun onResponse(
                call: Call<RequestResult<List<LogisticsSpecialistEntity>>>,
                res: Response<RequestResult<List<LogisticsSpecialistEntity>>>
            ) {
                val body = res.body()
                body.takeIf { it != null }?.run {
                    if (msgCode == AppConstants.Http.RESPONSE_SUCCESS) {
                        if (payload.isNotNullOrSize()) {
                            view?.logisticsSpecialistList(payload)
                        } else {
                            view?.showState(LoadWaitDominator.NO_DATA)
                        }
                    } else {
                        view?.showError(msgInfo)
                    }
                } ?: view?.showState(LoadWaitDominator.NET_ERROR)

                view?.dismissLoading()
            }

            override fun onFailure(exception: ApiException) {
                view?.dismissLoading()
                view?.showError(exception.message)
            }
        })
    }
}