package com.urovo.activity.report.view.adapter

import android.widget.TextView
import androidx.core.content.ContextCompat
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.urovo.activity.report.model.LogisticsSpecialistEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.utils.TextColorDominator
import com.urovo.wugumofang.utils.convertDefaultValue
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.wgmf.trade.utils.IntentUtil

/**
 * 物流专员信息适配器
 * @author LW
 */
class LogisticsSpecialistAdapter(
    list: List<LogisticsSpecialistEntity>
) : BaseQuickAdapter<LogisticsSpecialistEntity, BaseViewHolder>(
    R.layout.item_logistics_telphone,
    list
) {
    override fun convert(helper: BaseViewHolder?, item: LogisticsSpecialistEntity?) {
        helper?.apply {
            item?.let {
                setText(R.id.logistics_telphone_item_position_tv, it.position)
                setText(
                    R.id.logistics_telphone_item_name_tv,
                    it.specialistName.convertDefaultValue()
                )
                getView<TextView>(R.id.logistics_telphone_item_phone_number_tv).apply {
                    val content = StringUtils.formatString(
                        mContext.getString(R.string.call_logistics_commissioner_phone_number),
                        it.phoneNumber
                    )
                    text = TextColorDominator.styleTextColor(
                        content, ContextCompat.getColor(
                            mContext,
                            R.color.blue_text_color
                        ), 5, content.length
                    )

                    tag = it.phoneNumber.toString()
                    setOnClickListener {
                        if (tag != null) {
                            val phone = tag as String
                            if (phone.notNullAndEmpty()) {
                                IntentUtil.callPhone(phone, context)
                            }
                        }
                    }

                }

                getView<TextView>(R.id.logistics_telphone_item_telephone_number_tv).apply {
                    val content = StringUtils.formatString(
                        mContext.getString(R.string.call_logistics_commissioner_telephone),
                        it.telephoneNumber
                    )

                    text = TextColorDominator.styleTextColor(
                        content, ContextCompat.getColor(
                            mContext,
                            R.color.blue_text_color
                        ), 3, content.length
                    )

                    tag = it.telephoneNumber.toString()
                    setOnClickListener {
                        if (tag != null) {
                            val phone = tag as String
                            if (phone.notNullAndEmpty()) {
                                IntentUtil.callPhone(phone, context)
                            }
                        }
                    }
                }
            }
        }
    }
}