package com.urovo.activity.resetpassword.presenter;

import android.text.TextUtils;

import com.urovo.activity.resetpassword.model.PassData;
import com.urovo.activity.resetpassword.web.CheckCodeResponse;
import com.urovo.wgmf.AsyncTask.BaseAppSalerServerTask;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.PreferHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Created by Sasuke on 2016/6/14.
 */
public class VerifyAccountPresenter implements ResetPassContract.Presenter {
    ResetPassContract.View mView;
    PassData data;
    Timer timer;//用于校验码的倒计时
    public VerifyAccountPresenter(ResetPassContract.View mView) {
        this.mView = mView;
        this.mView.setPresenter(this);
        if(data == null){
            data = new PassData();
        }
    }

    @Override
    public void start() {
        String pass = mView.getIntentPass();
        if(!TextUtils.isEmpty(pass)){
            data.setUserId(pass);
        }
        mView.setUserIdInput(data.getUserId());
    }

    public PassData getData(){
        return data;
    }

    /**
     * 请求验证码
     */
    @Override
    public void requestCode(String idInput,String phoneInput) {
        if(TextUtils.isEmpty(idInput) || TextUtils.isEmpty(phoneInput) || phoneInput.length() != 11){
            ((BaseActivity)mView).showMsg("请输入正确的工号、手机号！");
            mView.changeCountBtnState(true);
            return;
        }
        getData().setUserId(idInput);
        getData().setUserPhone(phoneInput);
        Map<String,String> params = new HashMap<>();
        params.put("userId",idInput);
        params.put("phoneNo",phoneInput);
        new BaseAppSalerServerTask("requestCode", CheckCodeResponse.class,
                (BaseActivity) mView, "正在获取验证码...", false, "/information/getResetPasswordCheckCode",
                        params).execute("");
    }

    /**
     * 通过网络校验验证码
     * @param code
     */
    @Override
    public void checkCode(String code,String userId,String phone) {//网络验证结果会在Activity中回调
        if(TextUtils.isEmpty(userId) || TextUtils.isEmpty(phone) || phone.length() != 11){
            ((BaseActivity)mView).showMsg("请输入正确的工号、手机号！");
            return;
        }
        //防止用户在获取验证码后偷梁换柱改id（输入ID与Pref的ID是否相同）
        String prefUserId = PreferHelper.getInstance().getString(AppConstants.PreKeys.CHECK_CODE_USER_ID);
        if(!TextUtils.isEmpty(prefUserId) && !prefUserId.equals(userId)){
            ((BaseActivity)mView).showMsg("请重新获取验证码");
            return;
        }
        //合法用户操作，信息存入data
        getData().setUserId(userId);
        getData().setUserPhone(phone);
        //如果是同一用户id时，就判断时间间隔
        long checkCodeStartTime = PreferHelper.getInstance().getLong(AppConstants.PreKeys.CHECK_CODE_START);
        if(System.currentTimeMillis() - checkCodeStartTime > 60 * 1000 * 10){
            if(checkCodeStartTime == -1) {
                ((BaseActivity) mView).showMsg("请先获取验证码");
            }else{
                ((BaseActivity) mView).showMsg("验证码已过期，请重新获取");
            }
            return;
        }
        Map<String,String> params = new HashMap<>();
        params.put("userId",getData().getUserId());
        params.put("phoneNo",getData().getUserPhone());
        params.put("checkCode",code);
        new BaseAppSalerServerTask("checkCode", CheckCodeResponse.class,
                (BaseActivity) mView, "正在核对验证码...", true, "/information/checkPasswordCode",
                params).execute("");
    }

    /**
     * 把该userId和验证码起始时间写入Pref
     * @param startTime
     */
    @Override
    public void set10minStart(long startTime) {//设置10分钟验证码失效的起始时间
       /* CheckCodePrefHelper.getInstance().setStartTime(startTime);
        CheckCodePrefHelper.getInstance().setUserId(getData().getUserId());*/
        PreferHelper.getInstance().setLong(AppConstants.PreKeys.CHECK_CODE_START,startTime);
        PreferHelper.getInstance().setString(AppConstants.PreKeys.CHECK_CODE_USER_ID,getData().getUserId());
    }

    @Override
    public void startCountDown() {
        if(timer != null){
            timer.cancel();
        }
        mView.changeCountBtnState(false);//倒计时期间，不允许点击
        timer = new Timer();
        getData().initContDown();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                mView.showCountDown(getData().getCountDown());//不断刷新倒计时数的UI显示
                if(getData().getCountDown() == 0) {
                    timer.cancel();
                }
                getData().countDown();//减减减！
            }
        },0,1000);
    }
    public Timer getTimer(){
        return this.timer;
    }

    /**
     * 停止倒计划，并且按钮复原
     */
    public void timerCancel(){
        if(timer != null) {
            getData().initContDown();//复位
            timer.cancel();
            mView.changeCountBtnState(true);//按钮设回可点击状态
        }
    }
}
