package com.urovo.activity.salesdetailnet.model

import androidx.databinding.ObservableField
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.urovo.activity.salesdetailnet.repo.SalesDetailRepo
import com.urovo.wgmf.db.dao.entitys.ProductsreturnList
import com.urovo.wgmf.db.dao.entitys.SaleProductList
import com.urovo.wugumofang.data.DateUtil
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.coroutines.launch
import java.util.*

/**
 * @author Sasuke on 2019-9-17 0017.
 */
class SalesDetailModel(val repo: SalesDetailRepo) : ViewModel() {
    companion object {
        const val TYPE_STORE_SALE = 1
        const val TYPE_MEMBER_SALE = 2
    }

    //起始查询日期
    var startDate = ObservableField<String>(DateUtil.getDate(Date()))
    //终止查询日期
    var endDate = ObservableField<String>(DateUtil.getDate(Date()))
    //总单数
    var billNum = ObservableField<String>("0")
    //总金额
    var billAmount = ObservableField<Double>(0.00)

    var loadWaitType = MutableLiveData<Int>().apply { value = LoadWaitDominator.LOADING }

    //选择弹窗使用的日期
    var popStartDate = MutableLiveData<String>().apply { value = startDate.get() }
    var popEndDate = MutableLiveData<String>().apply { value = endDate.get() }

    //查询类型
    var queryType = ObservableField<Int>(TYPE_STORE_SALE)

    //数据List
    var saleDataList = MutableLiveData<List<ProductsreturnList>>()

    //数据List
    var saleDataDetailList = MutableLiveData<List<SaleProductList>>()

    //开始查询
    fun query(userId: String, storeId: String, startTimeStr: String, endTimeStr: String) = viewModelScope.launch {
        loadWaitType.value = LoadWaitDominator.LOADING
        try {
            val res = repo.queryStoreSaleDetail(queryType.get()!!,userId, storeId, startTimeStr, endTimeStr)
            loadWaitType.value = LoadWaitDominator.DISMISS
            billNum.set(res.returnBean.totalConsNum)
            billAmount.set(res.returnBean.totalAmount.toDouble())
            saleDataList.value = res.returnBean.returnList
        } catch (e: Exception) {
            MyToast.show(e.message)
            loadWaitType.value = LoadWaitDominator.NET_ERROR
        }
    }
    //开始查询
    fun queryDetail(consId: String) = viewModelScope.launch {
        loadWaitType.value = LoadWaitDominator.LOADING
        try {
            val res = repo.querySaleDetailByConsId(consId)
            loadWaitType.value = LoadWaitDominator.DISMISS

            saleDataDetailList.value = res.resultInfo
        } catch (e: Exception) {
            MyToast.show(e.message)
            loadWaitType.value = LoadWaitDominator.NET_ERROR
        }
    }
}