package com.urovo.activity.shisanreportact.presenter

import android.content.Context
import com.google.gson.GsonBuilder
import com.urovo.activity.shisanreportact.model.ShisanReportEntity
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.Base3Response
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.LoadWaitDominator
import okhttp3.Request

/**
 * @author Sasuke on 2019-8-9 0009.
 */
class ShisanReportPresenter(val mView: ShisanReportContract.view) : ShisanReportContract.pre {
    init {
        mView.setPresenter(this)
    }

    //返回结果的缓存
    private var resultCache: List<ShisanReportEntity.ResultInfo.Wrapper.Details> = listOf()

    //从返回结果里提取的标签相关数据的缓存
    private var resultLabelCache: List<ShisanReportEntity.ResultInfo.Type> = listOf()

    override fun start() {
        mView.init()
    }

    /**通过当前列表第一个元素位置，来得到应该对应的Tab的select位置*/
    private fun getTabPosByRvHead(headPos: Int): Int {
        return resultLabelCache.indexOfFirst { it.itemType == resultCache.get(headPos).itemTypeId }
    }

    fun queryListData(storeNo: String) {
        mView.showLoadWait(LoadWaitDominator.LOADING)
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.QUERY_SHISAN_REPORT,
            mapOf("storeNo" to storeNo),
            object : OkHttpDominator.ResultCallback<ShisanReportEntity>() {
                override fun onError(request: Request?, e: Exception?) {
                    mView.showLoadWait(LoadWaitDominator.NET_ERROR)
                }

                override fun onResponse(res: ShisanReportEntity?) {
                    if (res == null || res.resultInfo.type.isEmpty()) {
                        mView.showLoadWait(LoadWaitDominator.NO_DATA)
                        return
                    }

                    res.resultInfo.run {
                        //浅拷贝缓存一下，用于后面提交
                        resultCache = wrapper.flatMap { it.details }
                        resultLabelCache = type
                        //【只存到第一个元素里】初始化一下本地数据模型的Cache字段，用于列表中的小标题标记
                        //                    wrapper.forEach { it.details[0].itemTypeId = it.itemType }
                        wrapper.forEach { group ->
                            group.details[0].itemTypeNameCache = group.itemTypeName//用于显示
                            group.details[0].itemTypeId = group.itemType//用于比较（后台不要传-1啊，这边-1是初始态）
                        }
                        mView.refreshRv(resultLabelCache, resultCache)
                        mView.dismissLoadWait()
                    }
                }
            })
    }

    fun submit(storeNo: String) {
        ViewUtil.showProgress(mView as Context, "请稍等", "正在上传数据")
        //构造一个排除protected字段的Gson，那些字段不需要传
        val gsonExclude = GsonBuilder().excludeFieldsWithoutExposeAnnotation().create()
        OkHttpDominator.postAsynByType(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.SUBMIT_SHISAN_REPORT + "?storeNo=${storeNo}",
            OkHttpDominator.JSON,
            mapOf("list" to gsonExclude.toJson(resultCache)),
            object : OkHttpDominator.ResultCallback<Base3Response>() {
                override fun onError(request: Request?, e: java.lang.Exception?) {
                    MyToast.show("网络异常：${e?.message}")
                    ViewUtil.dismissProgress()
                }

                override fun onResponse(res: Base3Response?) {
                    ViewUtil.dismissProgress()
                    res?.let {
                        MyToast.show(res.msgInfo)
                        if (res.msgCode == 0) {
                            mView.finish()
                        }
                    }
                }
            })
    }
}