package com.urovo.activity.shisanreportact.view.adapter

import android.app.AlertDialog
import android.content.Context
import android.graphics.drawable.BitmapDrawable
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.databinding.DataBindingUtil
import androidx.recyclerview.widget.RecyclerView
import com.squareup.picasso.Picasso
import com.urovo.activity.shisanreportact.model.ShisanReportEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.databinding.ShisanReportRvItemBinding
import com.urovo.wugumofang.view.CustomPopupWindow
import com.urovo.wugumofang.view.NumberKeyboard
import com.wgmf.trade.ui.ada.ViewHolderX

/**
 * @author Sasuke on 2019-08-15.
 * 门店柜型上报Adapter
 */
class ShisanReportAdapter(val context: Context, val dataList: List<ShisanReportEntity.ResultInfo.Wrapper.Details>) : RecyclerView.Adapter<ViewHolderX>() {
    private val keyboard: NumberKeyboard by lazy {
        NumberKeyboard.newInstance(null)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolderX {
        val binding = DataBindingUtil.inflate<ShisanReportRvItemBinding>(
                LayoutInflater.from(context),
                R.layout.shisan_report_rv_item,
                parent,
                false)
        return ViewHolderX(binding.root)

    }

    override fun getItemCount(): Int {
        return dataList.size
    }

    override fun onBindViewHolder(holder: ViewHolderX, pos: Int) {
        val binding = DataBindingUtil.getBinding<ShisanReportRvItemBinding>(holder.view)
        binding?.run {
            //类型小标题（仅该类的第一个元素要显示，后面跟着的都隐藏）
            itemTitleTv.text = dataList[pos].itemTypeNameCache
            itemTitleTv.visibility = if (dataList[pos].itemTypeNameCache.isBlank()) View.GONE else View.VISIBLE
            marginTopView.visibility = if (dataList[pos].itemTypeNameCache.isBlank() || pos == 0) View.GONE else View.VISIBLE//分隔空行

            shisanNameTv.text = dataList[pos].proName
            shisanCodeTv.text = dataList[pos].proCode
            countInput.text = dataList[pos].proQty.toString()
            countInput.tag = pos

            //查看图片 点击
            viewPhotoTv.setOnClickListener {
                imgPreviewPop.showAsLocation(viewPhotoTv, Gravity.CENTER, 0, 0)
                imgPreviewPop.getItemView(R.id.dismissBtn).setOnClickListener { imgPreviewPop.dismiss() }
                Picasso.with(context).load(dataList[pos].picUrl).placeholder(R.drawable.progress_animation_small).into(imgPreviewPop.getItemView(R.id.previewImg) as ImageView)

            }

            //减号逻辑
            jianBtn.setOnClickListener {
                var i = countInput.text.toString().toInt()
                i--
                countInput.text = i.toString()
                val ptn = countInput.tag as Int
                afterChangeWork(ptn, binding, i.toString())
            }

            //加号逻辑
            jiaBtn.setOnClickListener {
                var i = countInput.text.toString().toInt()
                i++
                countInput.text = i.toString()
                val ptn = countInput.tag as Int
                afterChangeWork(ptn, binding, i.toString())
            }

            //输入数字
            countInput.setOnClickListener { v ->
                keyboard.show(v as TextView, (context as AppCompatActivity).supportFragmentManager) {
                    val ptn = countInput.tag as Int
                    afterChangeWork(ptn, binding, it)
                }.selectAll()
            }
        }
    }

    fun afterChangeWork(ptn: Int, binding: ShisanReportRvItemBinding, str: String?) {
        binding.run {
            val str: String = str ?: "0"
            //先把库存拿出来，并排除空的情况
            var count = 1
            val minCount = 0

            if (str.isNotEmpty()) {
                try {
                    count = str.toInt()
                } catch (e: NumberFormatException) {
                    e.printStackTrace()
                    count = 0
                }

                if (count < minCount) {
                    count = minCount
                    countInput.setText(count.toString())
                    jianBtn.setEnabled(false)
                    jiaBtn.setEnabled(true)
                } else if (count == minCount) {
                    jianBtn.setEnabled(false)
                    jianBtn.setEnabled(true)
                } else if (count > AppConstants.Printer.maxCount) {
                    count = AppConstants.Printer.maxCount
                    countInput.setText(count.toString())
                    jiaBtn.setEnabled(false)
                    jianBtn.setEnabled(true)
                    //录入数量不允许超过最大值99999
                    AlertDialog.Builder(context)
                            .setMessage("录入数量不允许超过最大值999")
                            .setPositiveButton("我知道了", null)
                            .show()
                } else if (count == AppConstants.Printer.maxCount) {
                    jiaBtn.setEnabled(false)
                    jianBtn.setEnabled(true)
                } else {
                    jianBtn.setEnabled(true)
                    jiaBtn.setEnabled(true)
                }
            } else {
                count = minCount
                countInput.text = minCount.toString()
            }

            val dataItem = dataList[ptn]

            if (count != dataItem.proQty) {
                dataItem.proQty = count
            }
        }
    }

    //图片预览
    private val imgPreviewPop: CustomPopupWindow by lazy {
        CustomPopupWindow.Builder(context)
                .setContentView(R.layout.image_preview_dialog)
                .setFouse(true)
                .setOutSideCancel(false)
//                .setAnimationStyle(R.style.dropmenu_anim_style)
                .setBackgroundDrawable(BitmapDrawable())
                .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
                .setheight(ViewGroup.LayoutParams.MATCH_PARENT)
                .builder()
    }
}