package com.urovo.activity.slidemenu.presenter;

import com.urovo.activity.slidemenu.model.OrderEntity;
import com.urovo.wgmf.CommonController;
import com.urovo.wgmf.net.CommonNet;
import com.urovo.wugumofang.data.StringUtils;
import com.wgmf.trade.model.bean.CommonEntity;
import com.zsy.libcore.common.android.log.SYLoger;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

/**
 * presenter 层 处理view层的操作逻辑和数据回显，以及获取model 层 数据（本地和网络）
 *
 * @author chenji
 */
public class MemberStoreOrderPresenter implements CommonController.BasePre<OrderEntity> {

    /**
     * view 对象（View 层）
     */
    private CommonController.BaseView<OrderEntity> mView;
    /**
     * model 对象(网络层)
     */
    private CommonNet mCommonNet;

    public MemberStoreOrderPresenter(CommonController.BaseView<OrderEntity> view) {
        this.mView = view;
        this.mCommonNet = new CommonNet();
    }

    /**
     * model 层回调的数据 回传给p层,最后再回传给view 层显示
     *
     * @param resultInfo 请求成功的数据
     */
    @Override
    public void onResponseSuccess(CommonEntity<OrderEntity> resultInfo) {
        SYLoger.print("chenji-1->" + resultInfo.t.getData());
        //按照时间顺序降序
        sortList(resultInfo.t.getData());
        SYLoger.print("chenji-2->" + resultInfo.t.getData());
        mView.onResponseSuccess(resultInfo);
    }

    /**
     * 开始请求数据，入口，~少年，要开始拿数据了~
     *
     * @param params 。。。
     */
    @Override
    public void onStart2requestData(Map<String, String> params) {
        mCommonNet.start2requestMemberOrderData(this, params);
    }

    /**
     * @param failInfo 请求失败的数据
     */
    @Override
    public void onFailure(String failInfo) {
        mView.onFailure(failInfo);
    }


    /**
     * 订单列表根据购买时间降序排列
     *
     * @param list 请求后得到的列表数据
     */
    private void sortList(List<OrderEntity.DataBean> list) {
        //倒叙排列
        Collections.sort(list, new Comparator<OrderEntity.DataBean>() {//调用sort()方法，并实现Comparator接口中的compare()方法

            @Override
            public int compare(OrderEntity.DataBean lhs, OrderEntity.DataBean rhs) {
                if (StringUtils.toLong(lhs.getSubmitDate()) > StringUtils.toLong(rhs.getSubmitDate())) {
                    return -1;
                } else if (lhs.getSubmitDate().equals(rhs.getSubmitDate())) {
                    return 0;
                } else {
                    return 1;
                }
            }
        });
        //反转
//        Collections.reverse(list);
//        for (int i = 0; i < list.size(); i++) {
//            android.util.Log.i(TAG, "sortList: " + list.get(i).getSubmitDate() + "\t ");
//        }
    }
}