package com.urovo.activity.slidemenu.ui

import android.annotation.SuppressLint
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.gavin.com.library.PowerfulStickyDecoration
import com.gavin.com.library.listener.PowerGroupListener
import com.urovo.activity.slidemenu.adapter.FreeDeliveryQueryAdapter
import com.urovo.activity.slidemenu.model.FreeDeliveryQueryBean
import com.urovo.activity.slidemenu.presenter.FreeDeliveryQueryPresenter
import com.urovo.activity.slidemenu.presenter.contract.FreeDeliveryQueryContract
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.ui.DensityUtil
import kotlinx.android.synthetic.main.act_free_delivery_query.*

/**
 * 免费送直接兑换记录查询数据
 * @author LW
 */
class FreeDeliveryQueryActivity : BaseActivity(), FreeDeliveryQueryContract.View {
    override fun addRootLayoutId(): Int = R.layout.act_free_delivery_query

    override fun rootViewGroup(): ViewGroup = mFreeDeliveryContainer

    private var mFreeDeliveryQueryList = mutableListOf<FreeDeliveryQueryBean>()

    private var mFreeDeliveryQueryPresenter: FreeDeliveryQueryContract.Presenter? = null

    private var mFreeDeliveryQueryAdapter: FreeDeliveryQueryAdapter? = null

    private var mPowerfulStickyDecoration: PowerfulStickyDecoration? = null

    private val mPhoneNumber: String by lazy {
        intent.getStringExtra(AppConstants.ExtraKeys.PHONE_NUM)
    }

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.free_delivery_title)
    }

    override fun initViewNew() {
        initTitle()

        mFreeDeliveryQueryPresenter = FreeDeliveryQueryPresenter(this)

        mFreeDeliveryQueryAdapter = FreeDeliveryQueryAdapter(this, mutableListOf())
        mPowerfulStickyDecoration =
            PowerfulStickyDecoration.Builder.init(object : PowerGroupListener {

                override fun getGroupName(position: Int): String {
                    if (mFreeDeliveryQueryList.size > position) {
                        return mFreeDeliveryQueryList.get(position).exchageTime ?: ""
                    }
                    return ""
                }

                @SuppressLint("InflateParams")
                override fun getGroupView(position: Int): View? {
                    if (mFreeDeliveryQueryList.size > position) {
                        val view =
                            layoutInflater.inflate(
                                R.layout.item_free_delivery_query_title,
                                null,
                                false
                            )
                        view.findViewById<TextView>(R.id.tv_item_exchange_time).text =
                            mFreeDeliveryQueryList.get(position).exchageTime
                        view.findViewById<TextView>(R.id.tv_item_saler_name).text =
                            StringUtils.formatString(
                                getString(R.string.free_delivery_record_single),
                                mFreeDeliveryQueryList.get(position).salerName
                            )
                        view.findViewById<TextView>(R.id.tv_item_store_name).text =
                            mFreeDeliveryQueryList.get(position).storeName
                        return view
                    } else {
                        return null
                    }
                }
            }).setGroupHeight(DensityUtil.dip2px(this@FreeDeliveryQueryActivity, 60f))
                .setGroupBackground(
                    ContextCompat.getColor(
                        this@FreeDeliveryQueryActivity,
                        R.color.app_bg
                    )
                )
                .build()

        mFreeDeliveryListViewLv.apply {
            layoutManager =
                LinearLayoutManager(this@FreeDeliveryQueryActivity, RecyclerView.VERTICAL, false)
            addItemDecoration(mPowerfulStickyDecoration!!)
            adapter = mFreeDeliveryQueryAdapter
        }
    }

    override fun initDataNew() {
        mFreeDeliveryQueryPresenter?.requestActivityExchageDetail(mPhoneNumber)
    }

    override fun onResponseSuccess(freeDeliveryQueryList: List<FreeDeliveryQueryBean>) {
        super.onResponseSuccess(freeDeliveryQueryList)
        mFreeDeliveryQueryList = freeDeliveryQueryList as MutableList<FreeDeliveryQueryBean>
        mFreeDeliveryQueryAdapter?.setNewData(freeDeliveryQueryList)
    }

    override fun onFailure(errorMsg: String) {
        if (errorMsg.isNotEmpty()) {
            showMsg(errorMsg)
        }
    }

}