package com.urovo.activity.slidemenu.ui;

import android.os.Bundle;
import android.text.Html;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.TextView;

import com.chad.library.adapter.base.entity.MultiItemEntity;
import com.urovo.activity.slidemenu.adapter.ExpandableItemAdapter;
import com.urovo.activity.slidemenu.model.Level0Item;
import com.urovo.activity.slidemenu.model.Level1Item;
import com.urovo.activity.slidemenu.model.Level2Item;
import com.urovo.activity.slidemenu.model.OrderEntity;
import com.urovo.activity.slidemenu.presenter.MemberStoreOrderPresenter;
import com.urovo.wgmf.CommonController;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.ui.InputMethodManagerTool;
import com.urovo.wugumofang.ui.ViewUtil;
import com.urovo.wugumofang.utils.MyToast;
import com.wgmf.trade.model.bean.CommonEntity;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libext.utils.DateTimeUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import butterknife.Unbinder;

/**
 * @author chenji
 * @date 2017/12/19
 * 物流查询--会员商城订单列表 页面
 * 地址:http://test.wgmf.com/wechat-shop-back/swagger-ui.html#!/order-controller/getByPhoneUsingGET
 * 参数: phone
 */

public class LogisticsQueryAct extends BaseActivity implements CommonController.BaseView<OrderEntity> {
    public static String MEMBER_PHONE_NUM = "member_phone_num";
    @BindView(R.id.rv_member_store_order_list)
    RecyclerView mRecyclerView;
    @BindView(R.id.et_phone_num)
    EditText mEtPhoneNum;
    @BindView(R.id.tv_member_name)
    TextView mMemberName;
    private ExpandableItemAdapter mAdapter;
    Unbinder unbinder;
    private List<MultiItemEntity> mRes = new ArrayList<>();
    private View notDataView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.act_logistics_query);
        unbinder = ButterKnife.bind(this);
        initTitle();

        //保证不需要默认查询的页面进来后不请求
        final boolean isQueryLogistics = PreferHelper.getInstance().getBoolean(AppConstants.LOGISTICS_QUERY, true);
        if (isQueryLogistics) {
            //启动默认查询
            startQueryTask(getIntent().getStringExtra(MEMBER_PHONE_NUM));
        } else {
            //保证其它携带参数的页面进来可以默认查询
            PreferHelper.getInstance().setBoolean(AppConstants.LOGISTICS_QUERY, true);
        }
    }

    /**
     * 根据手机号码查询订单
     *
     * @param phoneNum 收货人手机号码
     */
    private void startQueryTask(String phoneNum) {
        SYLoger.print("phoneNum :" + phoneNum);
        mEtPhoneNum.setText(phoneNum);
        //将光标移至文字末尾
        mEtPhoneNum.setSelection(phoneNum == null ? 0 : phoneNum.length());
        //隐藏键盘
        InputMethodManagerTool.getInstance(LogisticsQueryAct.this).hideInputMethod(mEtPhoneNum);

        //开始请求数据
        final HashMap<String, String> map = new HashMap<>(1);
        map.put("phone", phoneNum);
        ViewUtil.showProgress(this, "正在玩命加载...", "请稍等...");
        new MemberStoreOrderPresenter(this).onStart2requestData(map);
    }

    /**
     * 重置收货人名字
     *
     * @param view ...
     */
    private void clearViewData(View view) {
        if (view != null) {
            if (view instanceof TextView) {
                ((TextView) view).setText(Html.fromHtml("<font color='#8E8E8E'>收货人姓名<big>&emsp&emsp -- </big></font>"));
            }
        }
    }

    /**
     * 控件初始化
     *
     * @param res 传入请求后的数据
     */
    private void initRecycleView(List<MultiItemEntity> res) {
        mAdapter = new ExpandableItemAdapter(this, res);
        final GridLayoutManager manager = new GridLayoutManager(this, 2);
        manager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                //因为布局是填充满的,所以都是2列铺满.
                return mAdapter.getItemViewType(position) == ExpandableItemAdapter.TYPE_LEVEL_2 ? 2 : manager.getSpanCount();
            }
        });
        mRecyclerView.setHasFixedSize(true);
        notDataView = getLayoutInflater().inflate(R.layout.layout_empty_view, (ViewGroup) mRecyclerView.getParent(), false);
        if (res.isEmpty()) {
            mAdapter.setEmptyView(notDataView);
        }
        mRecyclerView.setAdapter(mAdapter);
        // important! setLayoutManager should be called after setAdapter
        mRecyclerView.setLayoutManager(manager);
        for (int i = mAdapter.getData().size() - 1 + mAdapter.getHeaderLayoutCount(); i >= mAdapter.getHeaderLayoutCount(); i--) {
            mAdapter.expandAll(i, true, true);
        }
    }

    @Override
    public void initTitle() {
        super.initTitle();
        titleTextView.setText("会员商城订单列表");
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (unbinder != null) {
            unbinder.unbind();
        }
        //隐藏键盘
        if (mEtPhoneNum != null) {
            InputMethodManagerTool.getInstance(LogisticsQueryAct.this).hideInputMethod(mEtPhoneNum);
        }
    }

    /**
     * 列表数据转换(为了不改变通用适配器框架原本的实体类)
     *
     * @param dateList 网络请求后得到的数据
     * @return MultiItemEntity实体列表
     */
    @SuppressWarnings("all")
    private List<MultiItemEntity> convertListData(List<OrderEntity.DataBean> dateList) {
        //请求前清空上次的缓存数据
        if (!mRes.isEmpty()) {
            mRes.clear();
        }
        clearViewData(mMemberName);
        for (int i = 0; i < dateList.size(); i++) {

            //因为后台给的数据结构的原因,这里需要过滤一下,只取列表数据第一个的收货人姓名.
            if (i == 0 && mMemberName != null) {
                mMemberName.setText(
                        Html.fromHtml("<font color='#8E8E8E'>收货人姓名<big>&emsp&emsp " + dateList.get(i).getReceiverNameSnapshot() + "</big></font>"));
//                mMemberName.setText(Html.fromHtml("<font color=\"#808080\" size=\"26px\">收货人姓名     </font>" +
//                        "<big><font color=\"#333333\" size=\"36px\"> &emsp &emsp &emsp " + dateList.get(i).getReceiverNameSnapshot() + " &emsp &emsp &emsp </font></big>" +
//                        "<font color=\"#808080\" size=\"26px\">     句号</font>"));
            }

            // 购买时间&&订单归属
            final String submitDate = DateTimeUtils.getTimeStamp(dateList.get(i).getSubmitDate());
            String orderBelong = dateList.get(i).getStaffId();
            Level0Item lv0 = new Level0Item(submitDate, TextUtils.isEmpty(orderBelong) ? "无" : orderBelong);

            // 物流信息列表, 添加物流公司名和物流单号,为了后面查询物流信息提供字段信息
            for (int k = 0; k < dateList.get(i).getAppDeliveryResult().size(); k++) {
                lv0.deliveryPartyName = dateList.get(i).getAppDeliveryResult().get(k).getDeliveryPartyName();
                lv0.shipmentCode = dateList.get(i).getAppDeliveryResult().get(k).getShipmentCode();
            }

            //产品列表
            for (int j = 0; j < dateList.get(i).getAppProductResult().size(); j++) {
                Level1Item lv1 = new Level1Item(
                        dateList.get(i).getAppProductResult().get(j).getProductTemplateName(),
                        dateList.get(i).getAppProductResult().get(j).getQuantity()
                );
                lv0.addSubItem(lv1);
            }
            mRes.add(lv0);

            //订单状态和订单号
            mRes.add(new Level2Item(dateList.get(i).getOrderStatus(), dateList.get(i).getOrderCode()));
        }
        initRecycleView(mRes);
        return mRes;
    }

    @Override
    public void onResponseSuccess(CommonEntity<OrderEntity> resultInfo) {
        ViewUtil.dismissProgress();
        OrderEntity orderEntity = resultInfo.t;
        if (orderEntity != null && orderEntity.getData() != null) {
            convertListData(resultInfo.t.getData());
        }
    }

    @Override
    public void onFailure(String failInfo) {
        ViewUtil.dismissProgress();
        MyToast.show(failInfo);
    }

    /**
     * 查询按钮
     *
     * @param v ...
     */
    @OnClick({R.id.btn_query, R.id.et_phone_num})
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_query:
                //清除光标
                mEtPhoneNum.setCursorVisible(false);
                final String phoneNum = mEtPhoneNum.getText().toString().trim();
                startQueryTask(phoneNum);
                break;
            case R.id.et_phone_num:
                mEtPhoneNum.setCursorVisible(true);
                //当前点击其他控件时,必须清除焦点再获取焦点才会弹软键盘
                mEtPhoneNum.clearFocus();
                focus(mEtPhoneNum);
                //显示软键盘
                InputMethodManagerTool.getInstance(LogisticsQueryAct.this).getInputMethod(mEtPhoneNum);
                break;
            default:
                break;
        }
    }
}
