package com.urovo.activity.superdietitian.presenter

import com.urovo.activity.superdietitian.net.DietitianQueryRes
import com.urovo.activity.superdietitian.view.SuperDietitianAct
import com.urovo.wgmf.base.BaseContract
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.response.BaseSimpleResponse
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.MyToast
import okhttp3.Request

/**
 * @author Sasuke on 2018/6/25.
 */
class SuperDietitianPre(val mView: BaseContract.view) : BaseContract.pre {
    init {
        mView.setPresenter(this)
    }

    override fun start() {
        mView.init()
        mView.query()
    }

    /**
     * 查询证书信息
     */
    fun queryCertifyInfo(staffId: String) {
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.SUPER_DIETITIAN_QUERY,
                mapOf("staffId" to staffId),
                object : OkHttpDominator.ResultCallback<DietitianQueryRes>() {
                    override fun onError(request: Request?, e: Exception?) {
                        e?.printStackTrace()
                        MyToast.show("请求失败：${e?.printStackTrace().toString()}")
                    }

                    override fun onResponse(res: DietitianQueryRes?) {
                        if (res == null || res.resultInfo == null) {
                            MyToast.show("返回为null")
                            return
                        }
                        (mView as SuperDietitianAct).takeUnless { it.isFinishing }?.refreshShow(res.resultInfo)


                    }
                })
    }


    /**
     * 上传证书信息
     */
    fun uploadCertifyInfo(staffId: String?, areaOffice: String?, theoryScore: String?, operScore: String?, certPicUrl: String?, certNo: String?, peoplePicUrl: String?) {
        OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.SUPER_DIETITIAN_UPLOAD,
                mapOf("staffId" to staffId,
                        "areaOffice" to areaOffice,
                        "theoryScore" to theoryScore,
                        "operScore" to operScore,
                        "certPicUrl" to certPicUrl,
                        "certNo" to certNo,
                        "peoplePicUrl" to peoplePicUrl),
                object : OkHttpDominator.ResultCallback<BaseSimpleResponse>() {
                    override fun onError(request: Request?, e: Exception?) {
                        ViewUtil.dismissProgress()
                        e?.printStackTrace()
                        MyToast.show("网络连接超时，请检查您的网络状态，稍后重试！")
                    }

                    override fun onResponse(res: BaseSimpleResponse?) {
                        ViewUtil.dismissProgress()
                        (mView as BaseActivity).takeUnless { it.isFinishing }?.let {
                            it.apply {
                                showMsg(res?.msgInfo)
                                finish()
                            }
                        }
                    }
                })
    }

}