package com.urovo.activity.systemsetting.shareappqrcode2colleagues;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.tencent.mm.sdk.openapi.IWXAPI;
import com.tencent.mm.sdk.openapi.WXAPIFactory;
import com.umeng.socialize.Config;
import com.umeng.socialize.ShareAction;
import com.umeng.socialize.UMShareListener;
import com.umeng.socialize.bean.SHARE_MEDIA;
import com.umeng.socialize.media.UMImage;
import com.urovo.wgmf.zxing.ZxingCodeTool;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.constants.NetParam;
import com.urovo.wugumofang.ui.BitmapUtil;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;

import static com.urovo.wugumofang.constants.AppConstants.QrCodeShare.CONTENT;
import static com.urovo.wugumofang.constants.AppConstants.QrCodeShare.TITLE;
import static com.urovo.wugumofang.constants.AppConstants.QrCodeShare.URL;

/**
 * 分享App下载二维码给同事
 *
 * @author chenji
 * @date 2017/10/23
 */

public class ShareAppQrCode2ColleaguesAct extends BaseActivity implements View.OnClickListener {
    //    @BindView(R.id.loading_progress)
//    ProgressBar progressBar;
    @BindView(R.id.iv_erweima)
    ImageView mQrCodeShow;
    @BindView(R.id.share_to_friends)
    LinearLayout shareToFriends;
    @BindView(R.id.back)
    ImageButton back;
    @BindView(R.id.text_title)
    TextView mTextTitle;
    private Bitmap mBitmap;
    private IWXAPI mApi;
    private String mContent = "";
    private String mTitle = "";
    private String mUrl = "";
    private byte[] bitmapBytes;
    private Unbinder mBind;
    private UMShareListener mUmShareListener = new UMShareListener() {
        @Override
        public void onResult(SHARE_MEDIA platform) {
            showMsg("分享成功");
        }

        @Override
        public void onError(SHARE_MEDIA platform, Throwable t) {
            showMsg("分享失败");
        }

        @Override
        public void onCancel(SHARE_MEDIA platform) {
            showMsg("分享取消");
        }
    };

    public static void launch(Context context, String title, String content, String url) {
        Intent intent = new Intent(context, ShareAppQrCode2ColleaguesAct.class);
        intent.putExtra(TITLE, title);
        intent.putExtra(CONTENT, content);
        intent.putExtra(URL, url);
        context.startActivity(intent);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.share_code_colleagues_act);
        mBind = ButterKnife.bind(this);
        init();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mBind.unbind();
    }

    private void init() {
        mApi = WXAPIFactory.createWXAPI(this, NetParam.WXAPP_ID, false);
        mTextTitle.setText("分享给同事");
        shareToFriends.setOnClickListener(this);
        back.setOnClickListener(this);
        mTitle = getIntent().getStringExtra(TITLE);
        mUrl = getIntent().getStringExtra(URL);
        mContent = getIntent().getStringExtra(CONTENT);

//        如果要使用zxing包加载URL生成二维码图片，需要先调用以下方法
//        ThreadManager.getThreadPool().execute(new Runnable() {
//            @Override
//            public void run() {
//                //url 转化为二维码图片
//                setQrCode(transUrlTo2DCode(mUrl));//分享中的二维码图片是url解析出来的，会慢一点
//                refreshUI(getQrCode());//刷新当前界面的大的二维码 使用前去掉布局的二维码图片
//            }
//        });
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            // 返回关闭当前页面
            case R.id.back:
                finish();
                break;
            // 分享App下载地址给同事
            case R.id.share_to_friends:
                shareWeiXin(SHARE_MEDIA.WEIXIN);
                break;
            default:
                break;
        }
    }

    private void shareWeiXin(SHARE_MEDIA type) {
        if (!mApi.isWXAppInstalled() || !mApi.openWXApp()) {
            showMsg("您还未安装微信客户端");
            return;
        }
        showDialog();
        UMImage image = new UMImage(this, BitmapFactory.decodeResource(getResources(), R.drawable.share_qr_code_to_colleagues));
        new ShareAction(this).setPlatform(type).setCallback(mUmShareListener)
//                .withMedia(getUMImage()) 使用zxing包加载URL生成二维码图片需要打开此方法，生成分享用的缩略图
                .withMedia(image) // 本地图片作为分享缩略图
                .withText(mContent)
                .withTitle(mTitle)
                .withTargetUrl(mUrl)
                .share();
    }

    private void showDialog() {
        ProgressDialog dialog = new ProgressDialog(this);
        dialog.setTitle("启动分享中");
        dialog.setMessage("请稍等.....");
        Config.dialog = dialog;
    }

    private void setQrCode(Bitmap bitmap) {
        //展示的二维码
        this.mBitmap = bitmap;
        //分享的二维码链接的缩略图
        this.bitmapBytes = BitmapUtil.bitmap2bytes(bitmap);
    }

    /**
     * 展示的二维码和分享的二维码链接的缩略图
     *
     * @return ...
     */
    private Bitmap getQrCode() {
        return mBitmap == null ? BitmapFactory.decodeResource(this.getResources(), R.drawable.share_qr_code_to_colleagues) : mBitmap;
//        return mBitmap = BitmapFactory.decodeResource(this.getResources(), R.drawable.share_qr_code_to_colleagues);// 本地图片
    }

    /**
     * 友盟分享缩略图
     *
     * @return 分享需要的UMImage对象
     */
    private UMImage getUMImage() {
        if (bitmapBytes == null || bitmapBytes.length == 0) {
            return new UMImage(this, BitmapFactory.decodeResource(this.getResources(), R.drawable.share_qr_code_to_colleagues));
        }
        return new UMImage(this, BitmapUtil.bytes2Bitmap(bitmapBytes));
//        return new UMImage(this, BitmapFactory.decodeResource(this.getResources(), R.drawable.share_qr_code_to_colleagues));// 分享本地图片
    }

    /**
     * 把Url生成二维码
     */
    private Bitmap transUrlTo2DCode(String url) {
        if (url != null) {
//            return ZxingCodeTool.create2DCode(url, DensityUtil.dip2px(this, 350), DensityUtil.dip2px(this, 350));
            return ZxingCodeTool.create2DCodeWithLogo(url, BitmapFactory.decodeResource(getResources(), R.drawable.icon));
        }
        return null;
    }

    /**
     * 刷新分享二维码大图
     *
     * @param qrCode ...
     */
    private void refreshUI(final Bitmap qrCode) {
        if (this.isFinishing()) {
            return;
        }
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
//                if (progressBar != null) {
//                    progressBar.setVisibility(View.GONE);
//                    mQrCodeShow.setVisibility(View.VISIBLE);
                mQrCodeShow.setImageBitmap(qrCode);
//                }
            }
        });
    }
}
