package com.urovo.activity.systemsetting.view

import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.provider.Settings
import android.provider.Settings.SettingNotFoundException
import android.text.TextUtils
import android.view.KeyEvent
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alibaba.android.arouter.facade.annotation.Route
import com.urovo.activity.systemsetting.view.adapter.BluetoothInfoEntity
import com.urovo.activity.systemsetting.view.adapter.BluetoothSettingAdapter
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.constants.AppConstants
import com.urovo.wugumofang.constants.RouterParams
import com.urovo.wugumofang.data.PreferHelper
import com.urovo.wugumofang.system.BaseAppManager
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.notNullAndEmpty
import com.urovo.wugumofang.view.LoadWaitDominator
import com.zsy.libcore.ui.ada.helper.DividerItemDecoration
import kotlinx.android.synthetic.main.activity_bluetooth_setting.*
import kotlinx.android.synthetic.main.activity_bluetooth_setting.root_container_fl


/**
 * @author LW
 * 蓝牙设置
 */
@Route(path = RouterParams.ORIGIN.BLUETOOTHSETTINGACTIVITY)
class BluetoothSettingActivity : BaseActivity() {

    private var mFlag = false

    override fun rootViewGroup(): ViewGroup = root_container_fl

    private var mBluetoothAdapter: BluetoothAdapter? = null

    private var mBluetoothSettingAdapter: BluetoothSettingAdapter? = null

    private val mNewPrinterType = PreferHelper.getInstance().getBoolean(
        AppConstants.PreKeys.NEW_PRINTER_TYPE, false
    )

    override fun addRootLayoutId(): Int = R.layout.activity_bluetooth_setting

    override fun initTitle() {
        super.initTitle()
        titleTextView.text = getString(R.string.bluetooth_setting)
    }

    override fun initViewNew() {
        initTitle()

        if (mNewPrinterType) {
            bluetooth_setting_tip_tv.setText(R.string.bluetooth_setting_new_tip)
        } else {
            bluetooth_setting_tip_tv.setText(R.string.bluetooth_setting_old_tip)
        }

        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter()

        if (mBluetoothAdapter?.isEnabled == true) {
            bluetooth_setting_switch_btn.text = getString(R.string.bluetooth_setting_close)
            bluetooth_setting_search_btn.isEnabled = true
        } else {
            bluetooth_setting_switch_btn.text = getString(R.string.bluetooth_setting_open)
            bluetooth_setting_search_btn.isEnabled = false
        }
        if (mBluetoothAdapter == null) {
            showMsg(R.string.bluetooth_setting_device_not_use)
            bluetooth_setting_switch_btn.isEnabled = false
            bluetooth_setting_search_btn.isEnabled = false
        } else {
            val printAddress = PreferHelper.getInstance().getString(
                AppConstants.PreKeys.PRINT_ADDRESS
            )
            if (printAddress.isNullOrEmpty()) {
                bluetooth_setting_save_ll.visibility = View.GONE
            } else {
                bluetooth_setting_save_ll.visibility = View.VISIBLE
                val name = printAddress.substring(printAddress.indexOf("#") + 1)
                val address = printAddress.substring(
                    0,
                    printAddress.indexOf("#")
                )
                bluetooth_setting_save_info_tv.text =
                    if (name.notNullAndEmpty() && name != "null") {
                        "$name($address)"
                    } else {
                        address
                    }
            }
            bluetooth_setting_switch_btn.setOnClickListener {
                // 打开蓝牙
                if (mBluetoothAdapter?.isEnabled == false) {
                    mBluetoothAdapter?.enable()
                } else {
                    mBluetoothAdapter?.disable()
                    bluetooth_setting_search_btn.isEnabled = false
                    mBluetoothSettingAdapter?.setNewData(mutableListOf())
                    showState(LoadWaitDominator.NO_DATA)
                }
            }
            initIntentFilter()
            bluetooth_setting_search_btn.setOnClickListener {
                if (isLocationEnabled(mContext)) {
                    if (mBluetoothAdapter?.isDiscovering == true) mBluetoothAdapter?.cancelDiscovery()
                    mBluetoothSettingAdapter?.setNewData(mutableListOf())
                    dismissAllState()
                    mBluetoothAdapter?.startDiscovery()

                    mFlag = true
                } else {
                    startActivity(Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS))
                }
            }

            showState(LoadWaitDominator.NO_DATA)
            mBluetoothSettingAdapter = BluetoothSettingAdapter(mutableListOf())
            bluetooth_setting_list_rv.apply {
                layoutManager = LinearLayoutManager(mContext, RecyclerView.VERTICAL, false)
                addItemDecoration(
                    DividerItemDecoration(
                        mContext,
                        DividerItemDecoration.VERTICAL_LIST,
                        1,
                        ContextCompat.getColor(mContext, R.color.pop_query_devider)
                    )
                )
                adapter = mBluetoothSettingAdapter
            }
            mBluetoothSettingAdapter?.setOnItemClickListener { baseQuickAdapter, view, position ->
                try {
                    val item = baseQuickAdapter.getItem(position) as BluetoothInfoEntity
                    mBluetoothAdapter?.getRemoteDevice(item.address)?.let {
                        val createBondMethod = BluetoothDevice::class.java.getMethod("createBond")
                        createBondMethod.invoke(it)
                        PreferHelper.getInstance()
                            .setString(
                                AppConstants.PreKeys.PRINT_ADDRESS,
                                "${it.address}#${it.name}"
                            )
                        setResult(102, Intent())
                        finish()
                    }
                } catch (e: Exception) {
                    showMsg(R.string.bluetooth_setting_pairing_failed)
                    BaseAppManager.loger().info(e)
                }
            }
        }
    }

    /**
     * 判断定位服务
     */
    private fun isLocationEnabled(context: Context): Boolean {
        var locationMode = 0
        val locationProviders: String
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            locationMode = try {
                Settings.Secure.getInt(context.contentResolver, Settings.Secure.LOCATION_MODE)
            } catch (e: SettingNotFoundException) {
                e.printStackTrace()
                return false
            }
            locationMode != Settings.Secure.LOCATION_MODE_OFF
        } else {
            locationProviders = Settings.Secure.getString(
                context.contentResolver,
                Settings.Secure.LOCATION_PROVIDERS_ALLOWED
            )
            !TextUtils.isEmpty(locationProviders)
        }
    }


    private fun initIntentFilter() {
        // 设置广播信息过滤
        val intentFilter = IntentFilter()
        intentFilter.addAction(BluetoothDevice.ACTION_FOUND)
        intentFilter.addAction(BluetoothAdapter.ACTION_DISCOVERY_STARTED)
        intentFilter.addAction(BluetoothAdapter.ACTION_DISCOVERY_FINISHED)
        intentFilter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED)
        // 注册广播接收器，接收并处理搜索结果
        registerReceiver(mReceiver, intentFilter)
    }

    /**
     * 蓝牙广播接收器
     */
    private val mReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val action = intent.action
            if (BluetoothDevice.ACTION_FOUND == action) {
                val device =
                    intent.getParcelableExtra<BluetoothDevice>(BluetoothDevice.EXTRA_DEVICE)
                //if (device.bondState == BluetoothDevice.BOND_BONDED) {
                if (mNewPrinterType) {
                    if (device.name.notNullAndEmpty() && device.name.startsWith("BTP")) {
                        mBluetoothSettingAdapter?.addData(
                            BluetoothInfoEntity(
                                device.name,
                                device.address
                            )
                        )
                    }
                } else {
                    mBluetoothSettingAdapter?.addData(
                        BluetoothInfoEntity(
                            device.name,
                            device.address
                        )
                    )
                }
                //}
            } else if (BluetoothAdapter.ACTION_DISCOVERY_STARTED == action) {
                showTipDialog(R.string.bluetooth_setting_search_device_tip)
            } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED == action) {
                dismissDialog()
            }
            if (BluetoothAdapter.ACTION_STATE_CHANGED == action) {
                if (mBluetoothAdapter?.state == BluetoothAdapter.STATE_ON) {
                    bluetooth_setting_switch_btn.text = getString(R.string.bluetooth_setting_close)
                    bluetooth_setting_search_btn.isEnabled = true
                } else if (mBluetoothAdapter?.state == BluetoothAdapter.STATE_OFF) {
                    bluetooth_setting_switch_btn.text = getString(R.string.bluetooth_setting_open)
                    bluetooth_setting_search_btn.isEnabled = false
                }
            }
        }
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent?): Boolean {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if (mFlag) {
                ViewUtil.dismissProgress()
                mBluetoothAdapter?.cancelDiscovery()
                mFlag = false
            } else {
                return super.onKeyDown(keyCode, event)
            }
        }
        return super.onKeyDown(keyCode, event)
    }

    override fun onDestroy() {
        unregisterReceiver(mReceiver)
        super.onDestroy()
    }
}