package com.urovo.activity.tabMenu.home.act;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.melnykov.fab.FloatingActionButton;
import com.urovo.activity.home.SearchMemberActivity;
import com.urovo.activity.sell.SaveFailureActivity;
import com.urovo.activity.sell.ScanSaveSuccessAct;
import com.urovo.activity.tabMenu.home.ada.ScanResultListAda;
import com.urovo.activity.tabMenu.home.ent.ScanCodeListUtil;
import com.urovo.activity.tabMenu.home.pre.SubmitOrderPre;
import com.urovo.activity.tabMenu.home.task.OrderTask;
import com.urovo.activity.tabMenu.view.TabMenuActivity;
import com.urovo.wgmf.db.dao.dao.Djxs001DbOperate;
import com.urovo.wgmf.db.dao.dao.Djxs002DbOperate;
import com.urovo.wgmf.db.dao.entitys.BarcodeScanDb;
import com.urovo.wgmf.db.dao.entitys.OrderReqBean;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.Arith;
import com.urovo.wugumofang.data.DateUtil;
import com.urovo.wugumofang.data.MoneyUtils;
import com.urovo.wugumofang.system.PdaMsgConfig;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.system.ToActivity;
import com.urovo.wugumofang.ui.DialogHelper;
import com.urovo.wugumofang.utils.CommonTools;
import com.zsy.libcore.common.android.log.SYLoger;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;

/**
 * 条形码扫描结果页面
 *
 * @author chenji
 * @date 2018/6/12
 */
public class ScanResultAct extends BaseSubmitOrderAct implements View.OnClickListener, OrderTask.OrderResultListener, ScanResultListAda.TotalAmountListener {
    Unbinder unbinder;
    @BindView(R.id.proLv)
    RecyclerView mRecyclerView;
    @BindView(R.id.uploadOrderBtn)
    FloatingActionButton mFloatingActionButton;
    @BindView(R.id.sweepCode)
    Button mSweepCodeBtn;
    @BindView(R.id.actual_total_price)
    TextView mActualTotalPrice;
    OrderTask orderTask;

    ScanResultListAda scanResultListAda;

    /**
     * 选中的产品数量统计
     */
    private double checkCount;

    /**
     * 说明是扫码加粉页面传过来的
     */
    private boolean mScanCodeAddFlag;
    /**
     * 会员手机号
     */
    private static String mPhone = "";

    /**
     * 订单上传和回调
     */
    private SubmitOrderPre mSubmitOrderPre;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.pro_scan_result);
        unbinder = ButterKnife.bind(this);
        Intent intent = getIntent();
        mScanCodeAddFlag = intent.getBooleanExtra("scanCodeAddFlag", false);
        mPhone = intent.getStringExtra("phone");
        //test 服务器异常的情况
//        mPhone = "--";
        initTitle();
        // homeActivity --> CaptureActivity --> ScanResultAct
        initRv(ScanCodeListUtil.getInstance().getList());

        mSubmitOrderPre = new SubmitOrderPre(this, mPhone) {
            @Override
            public void skipToOrderResult() {
                ToActivity.toActivity(ScanResultAct.this, ScanSaveSuccessAct.class);
                finish();
            }
        };
    }

    @Override
    protected void onResume() {
        super.onResume();
        //刷新recycleView
        initRv(ScanCodeListUtil.getInstance().getList());
    }

    private void initRv(List<BarcodeScanDb> data) {
        if (data != null) {
            scanResultListAda = new ScanResultListAda(ScanResultAct.this, data);
            mRecyclerView.setHasFixedSize(true);
            LinearLayoutManager layoutManager = new LinearLayoutManager(ScanResultAct.this, RecyclerView.VERTICAL, false);
            mRecyclerView.setLayoutManager(layoutManager);
            //禁止滑动
            mRecyclerView.setNestedScrollingEnabled(false);
            mRecyclerView.setAdapter(scanResultListAda);
        }
    }

    @Override
    public void initTitle() {
        super.initTitle();
        if (mScanCodeAddFlag) {
            titleTextView.setText(TextUtils.isEmpty(mPhone) ? "--" : mPhone);
            mFloatingActionButton.setVisibility(View.INVISIBLE);
            mSweepCodeBtn.setCompoundDrawables(null, null, null, null);
            mSweepCodeBtn.setText("确认并保存");
            submit.setVisibility(View.GONE);
        } else {
            mFloatingActionButton.setVisibility(View.VISIBLE);
            mFloatingActionButton.setOnClickListener(this);
            titleTextView.setText("扫描结果");
            CommonTools.INSTANCE.setShowArrow(ScanResultAct.this, mSweepCodeBtn, CommonTools.ShowArrowDirection.right, R.drawable.go_white);
            mSweepCodeBtn.setText("去扫码加粉");
            submit.setVisibility(View.VISIBLE);
            submit.setOnClickListener(this);
            submit.setText("关闭");
        }
        //选中的产品总价
        mActualTotalPrice.setText(String.format(Locale.CHINA, "共%.2f元", getSelectProTotalAmount()));
        mSweepCodeBtn.setOnClickListener(this);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (unbinder != null) {
            unbinder.unbind();
        }
        //扫描界面CaptureActivity进来的，如果临时想不扫码回退就需要清空集合
        if (!mScanCodeAddFlag) {
            ScanCodeListUtil.getInstance().clearAll();
        }
        //取消任务
        if (null != orderTask) {
            orderTask.cancel(false);
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.confirm:
                DialogHelper.Alert3R(ScanResultAct.this, "提示", "确定要关闭当前吗？", null,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                gotoHomeTab(TabMenuActivity.class);
                            }
                        }, null);
                break;
            case R.id.uploadOrderBtn:
                //扫码后点击上传
                if (CommonTools.INSTANCE.isOneClick()) {
                    orderTask = new OrderTask(mFloatingActionButton, this);
                    orderTask.execute();
                }
                break;
            case R.id.sweepCode:
                if (!mScanCodeAddFlag) {
                    //不是扫码加粉页面传过来就点击去扫码加粉啊 scanCodeAddFlag：true 确定是去扫码加粉
                    ToActivity.toActivity(ScanResultAct.this, SearchMemberActivity.class, "scanCodeAddFlag", true);
                } else {
                    // 确认并保存
                    if (CommonTools.INSTANCE.isOneClick()) {
                        orderTask = new OrderTask(mSweepCodeBtn, this);
                        orderTask.execute();
                    }
                }
                break;
            default:
                break;
        }
    }

    /**
     * 获取选中产品总金额
     */
    private double getSelectProTotalAmount() {
        //选中的产品金额之和
        double selectTotalAmount = 0;
        checkCount = 0;

        // 选中要上传的产品计算出总金额
        List<BarcodeScanDb> list = ScanCodeListUtil.getInstance().getList();

        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).isCheck) {
                checkCount++;
//                selectTotalAmount += Double.parseDouble(list.get(i).getAmount());
                selectTotalAmount = Arith.add(selectTotalAmount,Double.parseDouble(list.get(i).getAmount()));
            }
        }
        return selectTotalAmount;
    }

    /**
     * 提交订单组装列表
     */
    @Override
    public boolean buildOrderForm() {
        boolean isSaveSuccess;

//=====================准备阶段===============================
//        double selectTotalAmount = 0;
//        double checkCount = 0;
//        // 选中要上传的产品计算出总金额
//
//        List<BarcodeScanDb> list = ScanCodeListUtil.getInstance().getList();
//
//        for (int i = 0; i < list.size(); i++) {
//            if (list.get(i).isCheck) {
//                checkCount++;
//                selectTotalAmount += Double.parseDouble(list.get(i).getAmount());
//            }
//        }

        double selectTotalAmount = formatPrice(getSelectProTotalAmount());//要保留两位小数

        //选中item的数目为0，说明没有选中
        if (checkCount == 0) {
            makeToastByHandlerPost(ScanResultAct.this, "请选择要上传的产品");
            return false;
        }
        if (selectTotalAmount == 0) {
            makeToastByHandlerPost(ScanResultAct.this, "请填写有效订单，总金额为0");
            return false;
        }

//=================================开始组装订单===============================================
        String storeId = SystemHelper.LoginOrganId;
        String userId = SystemHelper.LoginUserId;
        OrderReqBean orderRequestBean = new OrderReqBean();

        //销售日期
        orderRequestBean.setSaleDate(DateUtil.getDateTime(new Date()));

        //订单号
        orderRequestBean.setBill(storeId + userId + DateUtil.format(new Date(), "yyMMddHHmmss"));
        //会员手机号码
        orderRequestBean.setCUSTID(mPhone);
        // 老带新者号码
        orderRequestBean.setRecommendation("");
        orderRequestBean.setMDBC(SystemHelper.LoginBC);
        // 固定门店模式下posid 为4
        orderRequestBean.setPOSID(PdaMsgConfig.isPDA(this)? "2" : "4");
        //活动类型ID
        orderRequestBean.setPromid("");
        orderRequestBean.setSCDate(DateUtil.getDateTime(new Date()));
        orderRequestBean.setSaleTime(DateUtil.getTime(new Date()));
        orderRequestBean.setSkyId(userId);
        orderRequestBean.setStoreId(storeId);
        orderRequestBean.setExchangecode("");
        // （销售类型,1：销售；2：退货；3：团购）
        orderRequestBean.setXSTYPE("1");
        //原价总金额
        orderRequestBean.setTNET(selectTotalAmount);
        SYLoger.print("TNET-selectTotalAmount->" + selectTotalAmount);
        //实付总价
        orderRequestBean.setSKTNET(selectTotalAmount);
        orderRequestBean.setOrdertype(0);
        orderRequestBean.setDelivertype(0);
        orderRequestBean.setSaleBill("");
        List<OrderReqBean.ProductDetailsBean> productDetailsBeans = new ArrayList<>();
        for (BarcodeScanDb scanDb : ScanCodeListUtil.getInstance().getList()) {
            if (!scanDb.isCheck) {
                //过滤掉不选中的产品
                continue;
            }
            if (Double.parseDouble(scanDb.getAmount()) <= 0 || Double.parseDouble(scanDb.getNumOrWeight()) <= 0) {
                //过滤掉金额或者数量/重量小于等于0的产品
                continue;
            }
            OrderReqBean.ProductDetailsBean productDetailsBean = new OrderReqBean.ProductDetailsBean();
            productDetailsBean.setBarcode(scanDb.getItemId());
            productDetailsBean.setItemId(scanDb.getItemId());
            productDetailsBean.setItemName(scanDb.getItemName());
            productDetailsBean.setKCUom(scanDb.getUnit());
            productDetailsBean.setPrice(Double.parseDouble(scanDb.getPrice()));
            productDetailsBean.setUnit(1);
            productDetailsBean.setItemMode("P");
            productDetailsBean.setBrandType(AppConstants.BRAND.XDY + "");
            productDetailsBean.setAmount(Double.parseDouble(scanDb.getAmount()));
            productDetailsBean.setTQTY(Double.parseDouble(scanDb.getNumOrWeight()));
            productDetailsBeans.add(productDetailsBean);
        }
        orderRequestBean.setGiftMoney(0d);
        orderRequestBean.setProductDetails(productDetailsBeans);

        boolean isSaveOrder = Djxs001DbOperate.saveSaleOrder(orderRequestBean);
        boolean isSaveOrderDetail = Djxs002DbOperate.saveSaleOrderDetail(orderRequestBean.getBill(), productDetailsBeans);
        isSaveSuccess = isSaveOrder && isSaveOrderDetail;
        return isSaveSuccess;
    }

    private double formatPrice(double tempPrice) {
        String tempStr = MoneyUtils.twolastDF(tempPrice);
        return Double.valueOf(tempStr);
    }

    /**
     * 在线提交订单;
     */
    @Override
    public void onlineUploadOrders() {
        mSubmitOrderPre.submitOrder(ScanResultAct.this);
    }

    @Override
    public void onSubmitFailure() {
        mSubmitOrderPre.onSubmitFailure();
        ToActivity.toActivity(ScanResultAct.this, SaveFailureActivity.class, "isScanPage", true,"huiyuan", mPhone);
        finish();
    }

    @Override
    public void onSubmitSuccess() {
        mSubmitOrderPre.onSubmitSuccess();
    }

    @Override
    public void onSelectTotalAmountCallback() {
        mActualTotalPrice.setText(String.format(Locale.CHINA, "共%.2f元", getSelectProTotalAmount()));
    }
}
