package com.urovo.activity.tabMenu.home.ada;


import android.content.Context;
import androidx.annotation.Nullable;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.CheckBox;
import android.widget.TextView;

import com.chad.library.adapter.base.BaseQuickAdapter;
import com.chad.library.adapter.base.BaseViewHolder;
import com.urovo.wgmf.db.dao.entitys.BarcodeScanDb;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.data.MoneyUtils;
import com.urovo.wugumofang.ui.EditTextDecimal;

import java.util.List;
import java.util.Locale;

/**
 * 扫码结果列表适配器
 *
 * @author chenji
 * @date 2018/06/12
 */
public class ScanResultListAda extends BaseQuickAdapter<BarcodeScanDb, BaseViewHolder> {
    private TotalAmountListener totalAmountListener;

    public ScanResultListAda(TotalAmountListener totalAmountListener, @Nullable List<BarcodeScanDb> data) {
        super(R.layout.listview_pro_item_xdy0, data);
        this.totalAmountListener = totalAmountListener;
    }

    @Override
    protected void convert(final BaseViewHolder viewHolder, final BarcodeScanDb barcodeScanDb) {
        viewHolder.setText(R.id.tv_after_name, barcodeScanDb.getItemName() + "（" + barcodeScanDb.getUnit() + "）");
        viewHolder.setText(R.id.tv_after_bianma, barcodeScanDb.getPrice() + "元/" + barcodeScanDb.getUnit());
        viewHolder.setText(R.id.tv_item_id, barcodeScanDb.getItemId());
        viewHolder.setText(R.id.et_xcount, barcodeScanDb.getAmount());
        viewHolder.setChecked(R.id.cb_pro, barcodeScanDb.isCheck);
        viewHolder.setVisible(R.id.ll_btns, barcodeScanDb.isCheck);
        viewHolder.setVisible(R.id.tv_kg, barcodeScanDb.isCheck);
        viewHolder.setText(R.id.tv_kg, String.format("x %s", barcodeScanDb.getNumOrWeight()));
        viewHolder.setOnClickListener(R.id.item_pro_xdy, new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CheckBox checkBox = viewHolder.getView(R.id.cb_pro);
                barcodeScanDb.isCheck = !checkBox.isChecked();
                checkBox.setChecked(barcodeScanDb.isCheck);
                viewHolder.setVisible(R.id.ll_btns, barcodeScanDb.isCheck);
                viewHolder.setVisible(R.id.tv_kg, barcodeScanDb.isCheck);
                totalAmountListener.onSelectTotalAmountCallback();
            }
        });
        final EditTextDecimal editTextDecimal = viewHolder.getView(R.id.et_xcount);
        //设置金额输入框保留的小数点位数
        editTextDecimal.setDecimal(barcodeScanDb.totalAmountAccuracy);
        editTextDecimal.setSelection(barcodeScanDb.getAmount().trim().length());
        Object tag = editTextDecimal.getTag();
        if (tag != null && tag instanceof MyTextWatcher) {
            editTextDecimal.removeTextChangedListener((MyTextWatcher) tag);
        }
        MyTextWatcher myTextWatcher = new MyTextWatcher(barcodeScanDb, editTextDecimal, (TextView) viewHolder.getView(R.id.tv_kg));
        editTextDecimal.addTextChangedListener(myTextWatcher);
        editTextDecimal.setTag(myTextWatcher);

        //自动隐藏输入键盘
        editTextDecimal.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (v == editTextDecimal) {
                    try {
                        if (hasFocus) {
                            // Open keyboard
//                            ((InputMethodManager) requireNonNull(mContext.getSystemService(Context.INPUT_METHOD_SERVICE))).showSoftInput(editTextDecimal, InputMethodManager.SHOW_FORCED);
                        } else {
                            // Close keyboard
                            ((InputMethodManager) requireNonNull(mContext.getSystemService(Context.INPUT_METHOD_SERVICE))).hideSoftInputFromWindow(editTextDecimal.getWindowToken(), 0);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        });
    }

    /**
     * 判断非空
     *
     * @param o   ...
     * @param <T> ...
     * @return ...
     */
    private <T> T requireNonNull(T o) {
        if (o == null) {
            throw new NullPointerException();
        }
        return o;
    }

    private class MyTextWatcher implements TextWatcher {
        BarcodeScanDb barcodeScanDb;
        EditTextDecimal editTextDecimal;
        TextView tvKG;

        MyTextWatcher(BarcodeScanDb barcodeScanDb, EditTextDecimal editTextDecimal, TextView tvKG) {
            this.barcodeScanDb = barcodeScanDb;
            this.editTextDecimal = editTextDecimal;
            this.tvKG = tvKG;
        }

        @Override
        public void beforeTextChanged(CharSequence s, int start, int count, int after) {

        }

        @Override
        public void onTextChanged(CharSequence s, int start, int before, int count) {

        }

        @Override
        public void afterTextChanged(Editable s) {
            String totalAmount = editTextDecimal.getText().toString().trim();
            //根据金额除以单价，计算数量/重量
            if (TextUtils.isEmpty(totalAmount)) {
                totalAmount = String.format(Locale.CHINA, "%." + barcodeScanDb.totalNumOrWeightAccuracy + "f", 0d);
            }
            double numOrWeight = Double.parseDouble(totalAmount) / Double.parseDouble(barcodeScanDb.getPrice());
            tvKG.setText(String.format("x %s", MoneyUtils.anylastDF(numOrWeight, barcodeScanDb.totalNumOrWeightAccuracy)));
            //更改数据后需要重新赋值
            barcodeScanDb.setNumOrWeight(MoneyUtils.anylastDF(numOrWeight, barcodeScanDb.totalNumOrWeightAccuracy));
            barcodeScanDb.setAmount(totalAmount);

            totalAmountListener.onSelectTotalAmountCallback();
        }
    }

    public interface TotalAmountListener {
        /**
         * 选中产品总金额
         */
        void onSelectTotalAmountCallback();
    }
}