package com.urovo.activity.tabMenu.home.net;

import com.urovo.activity.tabMenu.home.ent.HomeAdEntity;
import com.urovo.activity.tabMenu.home.ent.HomeCarouselEntity;
import com.urovo.activity.tabMenu.home.ent.HomeMenuOnlineBean;
import com.urovo.wugumofang.net.CommonController;
import com.urovo.wugumofang.net.RetrofitHelper;
import com.urovo.wugumofang.response.CommonEntity;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libcore.model.bean.RequestResult;
import com.zsy.libcore.model.bean.ResultBaseCallback;
import com.zsy.libcore.model.bean.ResultCallback;

import java.net.SocketTimeoutException;
import java.util.List;
import java.util.Map;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * 首页接口
 *
 * @author chenji last on 2018/1/2 19:51
 */

public class HomeNet {
    private IHomeNet mIHomeNet;

    /**
     * retrofit的配置
     */
    public HomeNet() {
        mIHomeNet = RetrofitHelper.createApi(IHomeNet.class);
    }

    /**
     * 首页旋转木马
     *
     * @param presenter ...
     */
    public void start2request4carousel(final CommonController.BasePre<HomeCarouselEntity> presenter, Map<String, String> params) {
        Call<RequestResult<HomeCarouselEntity>> call = mIHomeNet.getHomeCarouselList(params);
        call.enqueue(new ResultCallback<HomeCarouselEntity>() {
            @Override
            protected void onResponseSuccess(
                    RequestResult<HomeCarouselEntity> responseBody) {
                super.onResponseSuccess(responseBody);
                if (responseBody.msgCode == 0) {
                    presenter.onResponseSuccess(new CommonEntity<>(responseBody.resultInfo));
                } else {
                    presenter.onFailure("首页轮播图接口数据异常！");
                }
            }

            @Override
            public void onFailure(Call<RequestResult<HomeCarouselEntity>> call, Throwable t) {
                String msgTips = "首页轮播图网络访问异常";
                if (t instanceof SocketTimeoutException) {
                    msgTips = "首页轮播图网络请求超时";
                }
                presenter.onFailure(msgTips + t);
            }
        });
    }

    /**
     * 首页菜单
     *
     * @param presenter ...
     */
    public void start2request4Menu(final CommonController.HomeMenuPre<List<List<HomeMenuOnlineBean.ResultInfoBean>>> presenter, Map<String, String> params) {
        final Call<HomeMenuOnlineBean> call = mIHomeNet.getHomeMenuList(params);
        call.enqueue(new ResultBaseCallback<HomeMenuOnlineBean>() {
            @Override
            public void onResponse(Call<HomeMenuOnlineBean> call, Response<HomeMenuOnlineBean> response) {
                if (response.isSuccessful()) {
                    final HomeMenuOnlineBean responseBody = response.body();
                    if (responseBody.msgCode != 0) {
//                        toastNetTips(responseBody.msgInfo);
                        presenter.onFailure4Menu("首页菜单页面配置异常！");
                    } else {//正常情况下
                        presenter.onResponseSuccess4Menu(new CommonEntity<>(response.body().getResultInfo()));
                    }
                } else {
                    this.onFailure(call, new Throwable(response.errorBody().toString()));
                }
            }

            @Override
            public void onFailure(Call<HomeMenuOnlineBean> call, Throwable t) {
                if (presenter != null) {
                    presenter.onFailure4Menu("首页菜单页面异常：" + t);
                }
            }
        });
    }

    /**
     * 快捷编辑菜单
     *
     * @param presenter ...
     * @param params    ...
     */
    public void start2request4Menu(final CommonController.BasePre<String> presenter, Map<String, String> params) {
        Call<RequestResult> call = mIHomeNet.getDragMenuList(params);
        call.enqueue(new Callback<RequestResult>() {
            @Override
            public void onResponse(Call<RequestResult> call, Response<RequestResult> response) {
                if (response.isSuccessful()) {
                    if (response.body().msgCode == 0) {
                        presenter.onResponseSuccess(new CommonEntity<>(response.body().msgInfo));
                    }
                } else {
                    if (presenter != null) {
                        presenter.onFailure("编辑菜单页面异常！");
                    }
                }
            }

            @Override
            public void onFailure(Call<RequestResult> call, Throwable t) {
                presenter.onFailure("编辑菜单页面异常:" + t);
            }
        });
    }

    /**
     * 广告位
     *
     * @param presenter ...
     */
    public void start2request4Ad(final CommonController.HomeAdPre<List<HomeAdEntity>> presenter) {
//        IHomeNet mIHomeNet = RetrofitHelper.createApi("http://192.168.2.104:8383/", IHomeNet.class);
        final Call<RequestResult<List<HomeAdEntity>>> call = mIHomeNet.getHomeAdList();
        call.enqueue(new Callback<RequestResult<List<HomeAdEntity>>>() {
            @Override
            public void onResponse(Call<RequestResult<List<HomeAdEntity>>> call, Response<RequestResult<List<HomeAdEntity>>> response) {
                if (response.isSuccessful()) {
                    if (response.body().msgCode == 0) {
                        final RequestResult<List<HomeAdEntity>> responseBody = response.body();
                        SYLoger.print("wgmf--responseBody.resultInfo: " + responseBody.resultInfo);
                        presenter.onResponseSuccess4Ad(new CommonEntity<>(responseBody.resultInfo));
                    }
                } else {
                    if (presenter != null) {
                        presenter.onFailure4Ad("广告位异常！");
                    }
                }
            }

            @Override
            public void onFailure(Call<RequestResult<List<HomeAdEntity>>> call, Throwable t) {
                presenter.onFailure4Ad("广告位异常:" + t);
            }
        });
    }
}
