package com.urovo.activity.user

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import com.urovo.activity.user.adapter.YearAwardAdapter
import com.urovo.activity.user.presenter.YearAwardPre
import com.urovo.wgmf.CommonController
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.response.BaseResponse
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.view.LoadWaitDominator
import kotlinx.android.synthetic.main.year_award_fra.*
import java.util.*

/**
 * @author Sasuke on 2018/3/1.
 * 年终奖查询
 */
class YearAwardFra : BackHandledFragment(), CommonController.YearAwardView {
    val currentYear: Int by lazy {
        Calendar.getInstance().get(Calendar.YEAR)
    }
    var mPre: YearAwardPre? = null
    var mYear: Int = 0//查询请求的时候所用的年份值
    var loadWaitDominator: LoadWaitDominator? = null// 加载中、暂无数据、网络错误等等状态显示的全家桶。

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val view: View = inflater.inflate(R.layout.year_award_fra, container, false)
        view.isClickable = true
        mPre = YearAwardPre(this)
        return view
    }

    /**
     * 给布局里的组件挂监听器的操作一定要在onVIewCreated里做而不是onCreateView()里
     * 因为获取子组件是通过getView().findViewById()来实现的，在return之前，getView都是空的。
     */
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        init()
        query()
    }

    fun query(){
        mPre?.query(mYear.toString(), SystemHelper.LoginUserId)
    }

    /**
     * 获取年终奖说明
     */
    fun getCopywriter(): String{
        return mPre?.copywriter ?: ""
    }

    override fun init() {
        mYear = currentYear
        storeAndStaff.text = "${SystemHelper.LoginOrganName}, ${SystemHelper.LoginUserName}"
        leftBtn.setOnClickListener {
            yearTv.text = (--mYear).toString() + "年"
            setBtnEnability(!(mYear == currentYear - 1), //等于上一年，禁用左按钮
                    !(mYear == currentYear))//等于今年，禁用右按钮
            query()
        }
        rightBtn.setOnClickListener {
            yearTv.text = (++mYear).toString() + "年"
            setBtnEnability(!(mYear == currentYear - 1), //等于上一年，禁用左按钮
                    !(mYear == currentYear))//等于今年，禁用右按钮
            query()
        }
        yearAwardRv.apply { layoutManager = LinearLayoutManager(activity)
        adapter = YearAwardAdapter(mPre?.dataList)}
        yearTv.text = mYear.toString() + "年"
        setBtnEnability(true, false)
        loadWaitDominator = LoadWaitDominator(activity as AppCompatActivity,loadContainer)
    }

    override fun refreshData() {
        loadWaitDominator?.dismissAll()//隐藏所有加载标识
        //为空的情况，从头初始化
        yearAwardRv.adapter?.notifyDataSetChanged()
    }

    /**
     * 设置两个按钮的状态
     */
    fun setBtnEnability(isLeft: Boolean, isRight: Boolean) {
        leftBtn.isEnabled = isLeft
        rightBtn.isEnabled = isRight
    }

    override fun showNoData() {
        loadWaitDominator?.show(LoadWaitDominator.NO_DATA)
    }

    override fun showNetError() {
        loadWaitDominator?.show(LoadWaitDominator.NET_ERROR)
    }

    override fun showLoading() {
        loadWaitDominator?.show(LoadWaitDominator.LOADING)
    }


    //===================下面的不晓得是么鬼，反正是继承那个什么Fragment重载的========================


    override fun refresh(tag: String?, res: BaseResponse?) {

    }

    override fun onBackPressed(): Boolean {
        return false
    }


    override fun showMsg(msg: String?) {
        (activity as BaseActivity).showMsg(msg)
    }
}