package com.urovo.activity.wechatqrcode.view

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.view.View
import com.bumptech.glide.Glide
import com.urovo.activity.tabMenu.home.ent.UserQrCodeRes
import com.urovo.activity.wechatqrcode.pre.WeChatQrcodeContract
import com.urovo.activity.wechatqrcode.pre.WeChatQrcodePresenter
import com.urovo.wgmf.tool.netUtil.QiNiuDominator
import com.urovo.wugumofang.R
import com.urovo.wugumofang.base.BaseActivity
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.PhotoPickDominator
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.view.webview.BaseCookiesWebFra
import kotlinx.android.synthetic.main.wechat_qrcode_act.*

/**
 * @author Sasuke on 2020-4-17 0017.
 * 微信二维码
 */
class WeChatQrcodeActivity : BaseActivity(), WeChatQrcodeContract.view {
    var mPre: WeChatQrcodePresenter? = null
    var picker: PhotoPickDominator? = null
    var uploadTipUrl: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.wechat_qrcode_act)
        WeChatQrcodePresenter(this).start()
        query()
    }

    override fun init() {
        initTitle()
        titleTextView.text = "微信二维码"
        picker = PhotoPickDominator(this)

        //上传指引
        uploadGuideTv.setOnClickListener {
            BaseCookiesWebFra.launchWithShare(this, Intent(), uploadTipUrl, null, false)
        }

        //提交按钮
        submitBtn.setOnClickListener {
            picker?.selectByLocal(this@WeChatQrcodeActivity)
        }
    }

    /**初始请求*/
    fun query() {
        mPre?.queryIfHasUpload(SystemHelper.LoginUserId, SystemHelper.LoginOrganId)
    }

    /**弹提示*/
    override fun showAlertMsg(msg: String) {
        MyToast.show(msg)
    }

    /**上传二维码url成功*/
    override fun updateSuccessed(url: String) {
        setResult(Activity.RESULT_OK, Intent().putExtra("url", url))
        MyToast.show("上传成功")
        finish()
    }

    /**把二维码显示出来*/
    override fun showQrcode(data: UserQrCodeRes.Data?) {
        uploadGuideTv.visibility = View.VISIBLE
        uploadGuideTv.text = data?.uploadTiltle
        uploadTipUrl = data?.uploadTipUrl

        if (data?.personalWechatUrl.isNullOrBlank()) return
        topAlertTv.visibility = View.GONE
        qrCodeImage.visibility = View.VISIBLE
        Glide.with(this)
            .load(data?.personalWechatUrl)
            .placeholder(R.drawable.progress_animation_small)
            .into(qrCodeImage)
        submitBtn.text = "重新上传"
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        picker?.let {
            val photoPath = picker?.onPhotoPathResult(this, requestCode, resultCode, data)
            if (photoPath.isNullOrBlank()) {
//                MyToast.show("图片路径错误")
                return
            }

            //开始上传（只用传一张就可以了）
            ViewUtil.showProgress(this@WeChatQrcodeActivity, "请稍等", "正在上传二维码...")
            QiNiuDominator.uploadImages(listOf(photoPath), { urls ->
                //成功
                mPre?.updatePersonalWechatUrl(SystemHelper.LoginUserId, urls[0])
            }, {
                //失败 it ->
                runOnUiThread {
                    MyToast.show(it)
                }
            }, true)

//            doAsync {
//                        //先解析一下看看该图是不是二维码
//                        if (ZxingCodeTool.decodeLocalQrCode(photoPath) == null) {
//                            uiThread { MyToast.show("该图片不是二维码，请重新上传") }
//                            return@doAsync
//                        }
//                        uiThread {
//                            //开始上传（只用传一张就可以了）
//                            ViewUtil.showProgress(this@WeChatQrcodeActivity,"请稍等","正在上传二维码...")
//                            QiNiuDominator.uploadImages(listOf(photoPath), { urls ->
//                                //成功
//                                mPre?.updatePersonalWechatUrl(SystemHelper.LoginUserId, urls[0])
//                            }, {
//                                //失败 it ->
//                                MyToast.show(it)
//                            }, true)
//                }
//            }
        }

    }

    override fun setPresenter(presenter: WeChatQrcodeContract.presenter?) {
        mPre = presenter as WeChatQrcodePresenter
    }
}