package com.urovo.activity.wxfriendsres.ui

import android.content.Context
import android.media.MediaPlayer
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.urovo.activity.wxfriendsres.net.WxFriendsMaterialRes
import com.urovo.wugumofang.R
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.utils.MyToast
import com.urovo.wugumofang.utils.PhotoPackDLDominator
import com.urovo.wugumofang.view.CustomPopupWindow
import com.zsy.libext.widgets.clipboard.ClipboardUtil
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.find
import org.jetbrains.anko.uiThread
import java.text.SimpleDateFormat
import java.util.*


/**
 * @author Sasuke on 2020-4-23 0023.
 */
class WxFriendsResAdapter(val context: Context?, val dataList: List<WxFriendsMaterialRes.ResultInfo>) : RecyclerView.Adapter<WxFriendsResAdapter.WxFriendsResHolder>() {
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): WxFriendsResHolder {
        val view = LayoutInflater.from(context).inflate(R.layout.wx_friends_res_item, parent, false)
        return WxFriendsResHolder(view)
    }

    override fun getItemCount(): Int {
        return dataList.size
    }

    override fun onBindViewHolder(holder: WxFriendsResHolder, position: Int) {
    }

    override fun onBindViewHolder(holder: WxFriendsResHolder, pos: Int, payloads: List<Any?>) {
        val item = dataList[pos]
        holder.apply {
            //只刷新这一个组件
            payloads.takeIf { it.isNotEmpty() && it[0] == "videoPlayBtnTv" }?.let {
                videoPlayBtnTv.text = item.videoDuration
                return
            }
            //Group区
            foldArrow.isChecked = item.isExpanded
            publishMark.visibility = if (item.released == 0) View.GONE else View.VISIBLE
            val releaseTime = if (item.releaseTime.length >= 19) "发布时间：今天${item.releaseTime.substring(11, 16)}" else item.releaseTime
            publishTimeTv.text = releaseTime
            groupLayout.setOnClickListener { if (item.isExpanded) collapseGroup(pos) else expandGroup(pos) }

            //Child区
            childLayout.visibility = if (item.isExpanded) View.VISIBLE else View.GONE
            mtrRequireTitleTv.text = item.mtrRequireTitle
            mtrRequireTv.text = item.mtrRequire
            contentTitleTv.text = item.contentTitle
            contentTv.text = item.content
            replyTitleTv.text = item.replyTitle
            replyTv.text = item.reply

            mtrRequireTitleTv.visibility = if (!item.mtrRequire.isNullOrEmpty()) View.VISIBLE else View.GONE
            mtrRequireTv.visibility = if (!item.mtrRequire.isNullOrEmpty()) View.VISIBLE else View.GONE
            contentLl.visibility = if (!item.content.isNullOrEmpty()) View.VISIBLE else View.GONE
            contentTitleTv.visibility = if (!item.content.isNullOrEmpty()) View.VISIBLE else View.GONE
            contentTv.visibility = if (!item.content.isNullOrEmpty()) View.VISIBLE else View.GONE
            replyLl.visibility = if (!item.reply.isNullOrEmpty()) View.VISIBLE else View.GONE
            replyTitleTv.visibility = if (!item.reply.isNullOrEmpty()) View.VISIBLE else View.GONE
            replyTv.visibility = if (!item.reply.isNullOrEmpty()) View.VISIBLE else View.GONE
            sampleImagesTitleTv.visibility = if (!item.sampleImages.isNullOrEmpty()) View.VISIBLE else View.GONE
            sampleImage.visibility = if (!item.sampleImages.isNullOrEmpty()) View.VISIBLE else View.GONE

            contentCopyBtn.setOnClickListener { ClipboardUtil.copyText(context, contentTv.text.toString()); MyToast.show("复制成功"); sendPublishState(item.id, SystemHelper.LoginUserId) }
            replyCopyBtn.setOnClickListener { ClipboardUtil.copyText(context, replyTv.text.toString()); MyToast.show("复制成功"); sendPublishState(item.id, SystemHelper.LoginUserId) }

            //配图
            val hasImages = !item.images.isNullOrEmpty()
            val hasVideo = !item.video.isNullOrBlank()
            imagesTitle.text = if (hasImages) item.imagesTitle else if (hasVideo) item.videoTitle else ""
            //配图 之 列表
            imagesGridRv.visibility = if (hasImages) View.VISIBLE else View.GONE
            imagesGridRv.adapter = WxFriendsResImgGridAdapter(context, item.images)
            imagesGridRv.layoutManager = object : GridLayoutManager(context, 3) {
                override fun canScrollVertically(): Boolean {//不允许上下滑动
                    return false
                }
            }
            //配图 之 视频
            videoLayout.visibility = if (hasVideo) View.VISIBLE else View.GONE
            if (hasVideo) {
                Glide.with(context)
                        .load(item.videoCover)
                        .into(videoCover)
                videoPlayBtnTv.text = item.videoDuration

                if (item.videoDuration.isNullOrBlank()) {//如果没有获取到视频时长，就去获取
                    doAsync {
                        item.videoDuration = SimpleDateFormat("mm:ss", Locale.CHINA).format(Date(MediaPlayer().apply {
                            setDataSource(item.video)
                            prepare()
                        }.duration.toLong()))
                        uiThread { notifyItemChanged(pos, "videoPlayBtnTv") }
                    }
                }
            }
            videoLayout.setOnClickListener {//点击弹出弹窗开始播放视频
                videoPop.apply {
                    showAtLocation(it, Gravity.CENTER, 0, 0)
                    (getItemView(R.id.vv) as VideoView).setVideoPath(item.video)
                    (getItemView(R.id.vv) as VideoView).start()
                }
            }


            //配图+视频 之 一键下载
            imagesDLBtn.setOnClickListener {
                sendPublishState(item.id, SystemHelper.LoginUserId)
                //下载图片
                if (!item.images.isNullOrEmpty()) {
                    PhotoPackDLDominator(context, item.images!!, true).download()
                }
                //下载视频
                if (!item.video.isNullOrEmpty()) {
//                    val filePath = Files.tempPicture + Files.transUrl2Filename(item.video)
                    PhotoPackDLDominator(context, listOf(item.video!!), false).download()

//                    ViewUtil.showProgress(context,"请稍等","正在下载视频...")
//                    OkHttpDominator.downloadAsyn(item.video, filePath,object: OkHttpDominator.ResultCallback<String>(){
//                        override fun onError(request: Request?, e: Exception?) {
//                            ViewUtil.dismissProgress()
//                            serverErrorPop.showAtLocation(it, Gravity.CENTER, 0, 0)
//                        }
//
//                        override fun onResponse(res: String?) {
//                            ViewUtil.dismissProgress()
//                            showDIYToast(context, "保存视频成功",
//                                    HtmlCompat.fromHtml("请前往<font color='#ffcc01'>微信朋友圈</font>选择视频", HtmlCompat.FROM_HTML_MODE_LEGACY))
//                            context?.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.fromFile(File(filePath))))
//                        }
//                    })
                }
            }
            //示例图
            sampleImagesTitleTv.text = item.sampleImagesTitle
            Glide.with(context)
                    .load(if (item.sampleImages.isNullOrEmpty()) "" else item.sampleImages?.get(0))
                    .placeholder(R.drawable.pop_hone_qgwsr_default)
                    .into(sampleImage)
        }
    }

    /**展开某个Group*/
    private fun expandGroup(pos: Int) {
        dataList[pos].isExpanded = true
        notifyItemChanged(pos)
    }

    /**收起某个Group*/
    private fun collapseGroup(pos: Int) {
        dataList[pos].isExpanded = false
        notifyItemChanged(pos)
    }

    class WxFriendsResHolder(view: View) : RecyclerView.ViewHolder(view) {
        //Group区
        var groupLayout = view.find<RelativeLayout>(R.id.groupLayout)//group布局
        var publishTimeTv = view.find<TextView>(R.id.publishTimeTv)//发布时间
        var publishMark = view.find<ImageView>(R.id.publishMark)//已发布标志
        var foldArrow = view.find<RadioButton>(R.id.foldArrow)//折叠箭头

        //Child区
        var childLayout = view.find<LinearLayout>(R.id.childLayout)//child布局
        var mtrRequireTitleTv = view.find<TextView>(R.id.mtrRequireTitle)//素材要求标题
        var mtrRequireTv = view.find<TextView>(R.id.mtrRequire)//素材要求内容
        var contentLl = view.find<LinearLayout>(R.id.contentLl)//内容文案布局
        var replyLl = view.find<LinearLayout>(R.id.replyLl)//评论回复布局
        var contentTitleTv = view.find<TextView>(R.id.contentTitle)//内容文案标题
        var contentTv = view.find<TextView>(R.id.content)//内容文案内容
        var contentCopyBtn = view.find<Button>(R.id.contentCopyBtn)//内容文案一键复制
        var replyTitleTv = view.find<TextView>(R.id.replyTitle)//评论回复标题
        var replyTv = view.find<TextView>(R.id.reply)//评论回复内容
        var replyCopyBtn = view.find<Button>(R.id.replyCopyBtn)//评论回复一键复制

        var imagesTitle = view.find<TextView>(R.id.imagesTitle)//配图标题 or 视频标题
        var imagesGridRv = view.find<RecyclerView>(R.id.imagesGridRv)//配图
        var imagesDLBtn = view.find<Button>(R.id.imagesDLBtn)//一键下载图片 or 视频

        var sampleImagesTitleTv = view.find<TextView>(R.id.sampleImagesTitleTv)//示例图
        var sampleImage = view.find<ImageView>(R.id.sampleImage)//示例图

        var videoLayout = view.find<RelativeLayout>(R.id.videoLayout)//视频容器布局
        var videoPlayBtnTv = view.find<TextView>(R.id.playBtnTv)//播放按钮
        var videoCover = view.find<ImageView>(R.id.videoCover)//视频封面

    }

    //视频播放弹窗
    val videoPop by lazy {
        CustomPopupWindow.Builder(context)
                .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
                .setheight(ViewGroup.LayoutParams.MATCH_PARENT)
                .setContentView(R.layout.video_play_pop) //设置布局文件
                .builder().apply {
                    val videoView = getItemView(R.id.vv) as VideoView
                    videoView.setMediaController(MediaController(context))
                    getItemView(R.id.back).setOnClickListener { dismiss() }
//                    getItemView(R.id.root).setOnClickListener { dismiss() }
                }
    }

    /**状态改为已发布*/
    fun sendPublishState(fmid: Int, userNo: String) = OkHttpDominator.postAsyn(NetParam.YINXIAOBU_SERVER + ApiUrls.APP.FRIENDS_CIRCLE_MATERIAL_PUBLISH_STATE,
            mapOf("fmid" to fmid.toString(), "userNo" to userNo), null)
//
//    //服务器错误弹窗
//    val serverErrorPop: CustomPopupWindow by lazy {
//        CustomPopupWindow.Builder(context)
//                .setContentView(R.layout.server_error_pop)
//                .setFouse(true)
//                .setOutSideCancel(false)
////                .setAnimationStyle(R.style.dropmenu_anim_style)
//                .setBackgroundDrawable(BitmapDrawable())
//                .setwidth(ViewGroup.LayoutParams.MATCH_PARENT)
//                .setheight(ViewGroup.LayoutParams.MATCH_PARENT)
//                .builder().apply {
//                    getItemView(R.id.dismissBtn).setOnClickListener { dismiss() }
//                }
//    }

//    /**弹出自定义Toast*/
//    fun showDIYToast(context: Context?, msg: String, msg2: CharSequence = "") {
//        context?.let {
//            Toast(context).apply {
//                //设置Toast要显示的位置，水平居中并在底部，X轴偏移0个单位，Y轴偏移70个单位，
//                setGravity(Gravity.CENTER, 0, TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, -100f, Resources.getSystem().displayMetrics).toInt())
//                //设置显示时间
//                duration = Toast.LENGTH_SHORT
//                val diyToastView = LayoutInflater.from(context).inflate(com.urovo.wugumofang.libutils.R.layout.diy_toast_layout, null)
//                view = diyToastView
//                diyToastView.find<TextView>(com.urovo.wugumofang.libutils.R.id.toastText1).text = msg
//                diyToastView.find<TextView>(com.urovo.wugumofang.libutils.R.id.toastText2).text = msg2
//                diyToastView.find<TextView>(com.urovo.wugumofang.libutils.R.id.toastText2).visibility = if (msg2.isNotBlank()) View.VISIBLE else View.GONE
//            }.show()
//        }
//    }


}