package com.urovo.wgmf;


import com.urovo.wgmf.base.BasePresenter;
import com.urovo.wgmf.base.BaseView1;
import com.wgmf.trade.model.bean.CommonEntity;

import java.util.Map;

/**
 * 接口代理（公用）
 *
 * @author chenji
 * @date 2017/5/18
 */

public interface CommonController {
    /**
     * view 层 接口，只做数据展示
     */
    interface View extends BaseView1 {
        /**
         * 初始化标题tab
         */
        void initTitleTab();

        /**
         * 初始化指示器
         */
        void initTabLayout();

        /**
         * 初始化viewpager
         */
        void initViewPager();

        /**
         * 默认初始化门店/日期信息
         *
         * @param day ...
         */
        void initStoresDateInfo(int day);

        /**
         * 返回view层的请求成功的回调
         *
         * @param resultInfo ...
         */
        void onResponseSuccess(CommonEntity resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);
    }

    /**
     * 首页 view 层
     */
    interface HomeView {

        /**
         * 返回view层的请求成功的回调
         *
         * @param resultInfo ...
         */
        void onResponseSuccess(CommonEntity resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);
    }

    interface UpLoadOrderView {
        /**
         * 返回view层的请求成功的回调
         */
        void onResponseSuccess(String bill);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);
    }

    interface FlowSalesPersonView {
        /**
         * 返回view层的请求成功的回调
         *
         * @param tag        ...
         * @param resultInfo ...
         */
        void onResponseSuccess(String tag, CommonEntity resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);

        /**
         * 点击按钮选择的月份是当月的操作
         *
         * @param currentMonth 当月
         */
        void currentMonthSelected(String currentMonth);

        /**
         * 点击按钮选择的月份是上一个月的操作
         *
         * @param lastMonth 上一月
         */
        void lastMonthSelected(String lastMonth);

        /**
         * 展示吐司
         *
         * @param msg ...
         */
        void showToast(String msg);

    }

    /**
     * 带有tag的base的View接口
     */
    interface BaseTagView<T> {
        /**
         * 返回view层的请求成功的回调
         *
         * @param tag        ...
         * @param resultInfo ...
         */
        void onResponseSuccess(String tag, CommonEntity<T> resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);
    }

    /**
     * base的View接口
     */
    interface BaseView<T> {
        /**
         * 返回view层的请求成功的回调
         *
         * @param resultInfo ...
         */
        void onResponseSuccess(CommonEntity<T> resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);
    }

    /**
     * 库存查找相关的view接口
     *
     * @param <T> ...
     */
    interface StockSearchView<T> extends BaseView<T> {

    }

    /**
     * 产品月销量查询view 请求的接口
     */
    interface MonthSaleProductView {

        void requestData(int index,String brand);

        /**
         * 返回view层的请求成功的回调
         *
         * @param resultInfo ...
         * @param index      ...
         * @param brand      区别门店--》全部 五谷磨房 鲜稻研 会员商城 这四个类别
         */
        void onResponseSuccess(CommonEntity resultInfo, int index, String brand);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String failInfo);
    }

    /**
     * 月销量查询 view 的请求接口
     */
    interface MonthSaleView extends HomeView {
        void requestData();
    }

    /**
     * 新客三响炮view的请求接口
     */
    interface SanXiangPaoInfoView {
        /**
         * 返回view层的请求成功的回调
         *
         * @param resultInfo ...
         */
        void onResponseSuccess(String tag, CommonEntity resultInfo);

        /**
         * 返回view层的请求失败的回调
         *
         * @param failInfo ...
         */
        void onFailure(String tag, String failInfo);
    }

    /**
     * persenter 层 接口，view层的逻辑操作和数据回显，model层的数据请求逻辑操作
     */
    interface Presenter extends BasePresenter {
        /**
         * 返回Persenter层的请求成功的回调
         *
         * @param resultInfo 请求成功的数据
         */
        void onResponseSuccess(CommonEntity resultInfo);

        /**
         * 返回Persenter层的请求失败的回调
         *
         * @param failInfo 请求失败的数据
         */
        void onFailure(String failInfo);

        /**
         * view 层调用
         *
         * @param mParams ...
         */
        void start2requestData(Map<String, String> mParams);
    }

    interface UpLoadOrderPre {
        /**
         * 考勤上传
         *
         * @param params 请求参数集合
         * @param bill   考勤状态
         */
        void onUpLoad(Map<String, String> params, String bill);

        void onResponseSuccess(String bill);

        void onFailure(String failInfo);
    }

    /**
     * 流动顾问销售完成情况
     */
    interface FlowSalesPersonPre {
        void onStart2requestData(Map<String, String> params);

        void onResponseSuccess(String tag, CommonEntity resultInfo);

        void onFailure(String failInfo);

        void postDelayed(long delayTime);

        int timeLimit(String futureMonth, String referMonth);
    }

    /**
     * 年终奖查询
     */
    interface YearAwardPre {
        void query(String year, String userNo);
    }

    interface YearAwardView {
        void showMsg(String msg);

        void refreshData();

        void showNoData();

        void showLoading();

        void showNetError();

    }

    /**
     * 库存查询
     */
    interface StockSearchPre<T> extends BasePre<T> {
    }

    /**
     * 带有标志的base的Pre接口
     */
    @SuppressWarnings("unused")
    interface BaseTagPre<T> {
        void onStart2requestData(String tag, Map<String, String> params);

        void onResponseSuccess(String tag, CommonEntity<T> resultInfo);

        void onFailure(String failInfo);
    }

    /**
     * base的Pre接口
     */
    interface BasePre<T> {
        void onStart2requestData(Map<String, String> params);

        void onResponseSuccess(CommonEntity<T> resultInfo);

        void onFailure(String failInfo);
    }

    /**
     * 新客三响炮Pre的请求接口
     */
    interface SanXiangPaoInfoPre {
        /**
         * 返回Persenter层的请求成功的回调
         *
         * @param tag        是会员关怀数量还是会员关怀列表：AppConstants.SanXiangPao.QUERY_COUNT/AppConstants.SanXiangPao.QUERY_LIST
         * @param resultInfo 请求成功的数据
         */
        void onResponseSuccess(String tag, CommonEntity resultInfo);

        /**
         * 返回Persenter层的请求失败的回调
         *
         * @param tag      同上
         * @param failInfo 请求失败的数据
         */
        void onFailure(String tag, String failInfo);

        /**
         * view 层调用
         *
         * @param tag     ...
         * @param mParams ...
         */
        void start2requestData(String tag, Map<String, String> mParams);
    }

    /**
     * retrofit 请求的接口，业务的接口请求都放这里了。
     * p 层调用
     */
    interface Net {
        void start2requestData(Presenter presenter, Map<String, String> mParams);
// 这里可以定义网络请求的所需要的方法
    }

    /**
     * 插入一个数据
     */
    interface Pre extends BasePresenter {
        void onResponseSuccess(CommonEntity resultInfo);

        void onFailure(String failInfo);

        void start2requestData(Map<String, String> mParams, int index, String brand);

    }

    interface NotifyData {
//        void notifyDataSetChanged(int index);

        /**
         * 刷新列表
         *
         * @param index 门店和会员商城的区分的标志
         * @param brand 品牌名
         */
        void notifyDataSetChanged(int index, String brand);
    }
}
