package com.urovo.wgmf.adapter;

import java.util.ArrayList;
import java.util.List;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wgmf.db.dao.dao.Czyda001DbOperate;
import com.urovo.wgmf.db.dao.entitys.CXYKQEntity;
import com.urovo.wugumofang.R;

public class KaoqinAdapter extends BaseAdapter {
	private List<CXYKQEntity> list;
	private BaseActivity context;
	private List<String> group;
	private List<List<CXYKQEntity>> child;
	private static final int TYPE_GROUP = 0;
	private static final int TYPE_CHILD = 1;

	public KaoqinAdapter(List<CXYKQEntity> list, BaseActivity context) {
		this.list = list;
		this.context = context;
		group = new ArrayList<String>();
		child = new ArrayList<List<CXYKQEntity>>();
		if(list != null && list.size() > 0) {
			group.add(list.get(0).KQDATE);
			int count = 1;
			for (CXYKQEntity a : list) {
				if(!group.contains(a.KQDATE)) {
					count += 1;
					group.add(a.KQDATE);
				}
			}
			for (String date : group) {
				ArrayList<CXYKQEntity> childItem = new ArrayList<CXYKQEntity>();
				for (CXYKQEntity a : list) {
					if(a.KQDATE.equals(date)) {
						childItem.add(a);
					}
				}
				child.add(childItem);
			}
		}

	}

	@Override
	public int getCount() {

		return list == null ? 0 : (list.size() + group.size());
	}

	@Override
	public Object getItem(int position) {

		if(list == null) {
			return null;
		}
		// 同一分类内，第一个元素的索引值
		int categroyFirstIndex = 0;
		int groupIndex = 0;
		for (List<CXYKQEntity> childItem : child) {
			int size = childItem.size() + 1;
			// 在当前分类中的索引值
			int categoryIndex = position - categroyFirstIndex;
			// item在当前分类内
			if(categoryIndex < size) {
				if(categoryIndex == 0) {
					return group.get(groupIndex);
				}
				else {
					return childItem.get(categoryIndex - 1);
				}

			}
			// 索引移动到当前分类结尾，即下一个分类第一个元素索引
			categroyFirstIndex += size;
			groupIndex += 1;
		}
		return null;
	}

	@Override
	public long getItemId(int position) {

		return position;
	}

	@Override
	public int getViewTypeCount() {

		return 2;
	}

	@Override
	public int getItemViewType(int position) {

		if(list == null) {
			return TYPE_CHILD;
		}
		int categroyFirstIndex = 0;

		for (List<CXYKQEntity> childItem : child) {
			int size = childItem.size() + 1;
			// 在当前分类中的索引值
			int categoryIndex = position - categroyFirstIndex;
			if(categoryIndex == 0) {
				return TYPE_GROUP;
			}
			categroyFirstIndex += size;
		}

		return TYPE_CHILD;
	}

	@Override
	public View getView(final int position, View convertView, ViewGroup parent) {
		ViewHolder viewHolder;
		int itemViewType = getItemViewType(position);
		switch (itemViewType) {
		case TYPE_GROUP:
			if(convertView == null) {
				convertView = LayoutInflater.from(context).inflate(R.layout.listview_date_item, null);
			}
			TextView dateTextView = (TextView) convertView.findViewById(R.id.date);
			String dateInfo = (String) getItem(position);
			dateTextView.setText(dateInfo);
			break;

		case TYPE_CHILD:
			if(convertView == null) {
				convertView = LayoutInflater.from(context).inflate(R.layout.second_kaoqin_query_item, null);
				viewHolder = new ViewHolder();
				viewHolder.name = (TextView) convertView.findViewById(R.id.name);
				viewHolder.type = (TextView) convertView.findViewById(R.id.type);
				viewHolder.day = (TextView) convertView.findViewById(R.id.day);
				viewHolder.time = (TextView) convertView.findViewById(R.id.time);
				viewHolder.valid = (TextView) convertView.findViewById(R.id.valid);
				convertView.setTag(viewHolder);
			}
			else {
				viewHolder = (ViewHolder) convertView.getTag();
			}
			CXYKQEntity ent = (CXYKQEntity) getItem(position);
			String nameString = Czyda001DbOperate.GetOperateName(ent.SKYID);
			viewHolder.name.setText(nameString);
			viewHolder.day.setText(ent.KQDATE);
			viewHolder.time.setText(ent.KQTIME);
			String typeString = ("0".equals(ent.KQTYPE)) ? "上班" : "下班";
			viewHolder.type.setText(typeString);
			
			if("1".equals(ent.Position_Valid))
			{
				viewHolder.valid.setText("正常");
			}
			else if("2".equals(ent.Position_Valid))
			{
				viewHolder.valid.setText("范围外");
			}
			else
			{
				viewHolder.valid.setText("无地理位置");
			}
			break;
		}
		return convertView;
	}

	class ViewHolder {
		TextView name;
		TextView type;
		TextView day;
		TextView time;
		TextView valid;
	}

}
