package com.urovo.wgmf.adapter;

import java.util.ArrayList;
import java.util.List;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wgmf.db.dao.entitys.QueryEntity;
import com.urovo.wugumofang.R;

public class PeiSongQueryAdapter extends BaseAdapter {
	private List<QueryEntity> list;
	private BaseActivity context;
	private List<String> group;
	private List<List<QueryEntity>> child;
	private static final int TYPE_GROUP = 0;
	private static final int TYPE_CHILD = 1;

	public PeiSongQueryAdapter(List<QueryEntity> list, BaseActivity context) {
		this.list = list;
		this.context = context;
		group = new ArrayList<String>();
		child = new ArrayList<List<QueryEntity>>();
		if(list != null && list.size() > 0) {
			group.add(list.get(0).WDate);
			int count = 1;
			for (QueryEntity a : list) {
				if(!group.contains(a.WDate)) {
					count += 1;
					group.add(a.WDate);
				}
			}
			for (String date : group) {
				ArrayList<QueryEntity> childItem = new ArrayList<QueryEntity>();
				for (QueryEntity a : list) {
					if(a.WDate.equals(date)) {
						childItem.add(a);
					}
				}
				child.add(childItem);
			}
		}

	}

	@Override
	public int getCount() {

		return list == null ? 0 : (list.size() + group.size());
	}

	@Override
	public Object getItem(int position) {

		if(list == null) {
			return null;
		}
		// 同一分类内，第一个元素的索引值
		int categroyFirstIndex = 0;
		int groupIndex = 0;
		for (List<QueryEntity> childItem : child) {
			int size = childItem.size() + 1;
			// 在当前分类中的索引值
			int categoryIndex = position - categroyFirstIndex;
			// item在当前分类内
			if(categoryIndex < size) {
				if(categoryIndex == 0) {
					return group.get(groupIndex);
				}
				else {
					return childItem.get(categoryIndex - 1);
				}

			}

			// 索引移动到当前分类结尾，即下一个分类第一个元素索引
			categroyFirstIndex += size;
			groupIndex += 1;
		}

		return null;
	}

	@Override
	public long getItemId(int position) {

		return position;
	}

	@Override
	public int getViewTypeCount() {

		return 2;
	}

	@Override
	public int getItemViewType(int position) {

		if(list == null) {
			return TYPE_CHILD;
		}
		int categroyFirstIndex = 0;

		for (List<QueryEntity> childItem : child) {
			int size = childItem.size() + 1;
			// 在当前分类中的索引值
			int categoryIndex = position - categroyFirstIndex;
			if(categoryIndex == 0) {
				return TYPE_GROUP;
			}
			categroyFirstIndex += size;
		}

		return TYPE_CHILD;
	}

	@Override
	public View getView(final int position, View convertView, ViewGroup parent) {
		ViewHolder viewHolder;
		int itemViewType = getItemViewType(position);
		switch (itemViewType) {
		case TYPE_GROUP:
			if(convertView == null) {
				convertView = LayoutInflater.from(context).inflate(R.layout.listview_date_item, null);
			}
			TextView dateTextView = (TextView) convertView.findViewById(R.id.date);
			String dateInfo = (String) getItem(position);
			dateTextView.setText(dateInfo);
			break;

		case TYPE_CHILD:
			if(convertView == null) {
				convertView = LayoutInflater.from(context).inflate(R.layout.second_goods_query_item, null);
				viewHolder = new ViewHolder();
				viewHolder.name = (TextView) convertView.findViewById(R.id.goods_name);
				viewHolder.num = (TextView) convertView.findViewById(R.id.goods_num);
				viewHolder.price = (TextView) convertView.findViewById(R.id.goods_price);
				viewHolder.count = (TextView) convertView.findViewById(R.id.goods_count);
				viewHolder.amount = (TextView) convertView.findViewById(R.id.goods_amount);
				viewHolder.people = (TextView) convertView.findViewById(R.id.goods_query_people);
				viewHolder.bill = (TextView) convertView.findViewById(R.id.goods_query_bill);
				viewHolder.date = (TextView) convertView.findViewById(R.id.goods_query_date);
				viewHolder.isShow = (LinearLayout) convertView.findViewById(R.id.is_show);
				viewHolder.isShow.setVisibility(View.GONE);
				convertView.setTag(viewHolder);
			}
			else {
				viewHolder = (ViewHolder) convertView.getTag();
			}
			QueryEntity ent = (QueryEntity) getItem(position);
			viewHolder.name.setText(ent.ItemName);
			viewHolder.num.setText(ent.ItemId);
			viewHolder.price.setText(ent.Price);
			viewHolder.count.setText(ent.TQTY);
			viewHolder.amount.setText(ent.Amount);
			viewHolder.people.setText(ent.Lr_Name);
			viewHolder.bill.setText(ent.Bill);
			viewHolder.date.setText(ent.WDate);
			break;
		}
		return convertView;
	}

	class ViewHolder {
		TextView name;
		TextView num;
		TextView price;
		TextView count;
		TextView amount;
		TextView people;
		TextView bill;
		TextView date;
		LinearLayout isShow;

	}

}
