package com.urovo.wgmf.adapter;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.DialogInterface;
import android.text.TextUtils;
import android.text.method.DigitsKeyListener;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.EditText;
import android.widget.TextView;

import com.urovo.activity.salesmanager.CountEditListener;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wgmf.db.dao.bill.DalCommon;
import com.urovo.wgmf.db.dao.entitys.ProductEntity;
import com.urovo.wugumofang.ui.DialogHelper;
import com.urovo.wugumofang.R;

import java.util.List;

public class SellAdapter extends BaseAdapter {
    private List<ProductEntity> list;
    private BaseActivity context;
    private String tag = null;
    public static final int DELETE_TAG = 0;
    public static final int EDIT_TAG = 1;

    public SellAdapter(List<ProductEntity> list, BaseActivity context) {
        this.list = list;
        this.context = context;

    }

    public SellAdapter(List<ProductEntity> list, BaseActivity context, String tag) {
        this.list = list;
        this.context = context;
        this.tag = tag;

    }

    @Override
    public int getCount() {

        return list == null ? 0 : list.size();
    }

    @Override
    public Object getItem(int position) {

        return list == null ? null : list.get(position);
    }

    @Override
    public long getItemId(int position) {

        return position;
    }

    @Override
    public View getView(final int position, View convertView, ViewGroup parent) {
        ViewHolder viewHolder;
        if (convertView == null) {
            convertView = LayoutInflater.from(context).inflate(R.layout.sell_list_item, null);
            viewHolder = new ViewHolder();
            viewHolder.text1 = (TextView) convertView.findViewById(R.id.text1);
            viewHolder.text2 = (TextView) convertView.findViewById(R.id.text2);
            viewHolder.text3 = (TextView) convertView.findViewById(R.id.text3);
            viewHolder.text4 = (TextView) convertView.findViewById(R.id.text4);
            viewHolder.text5 = (TextView) convertView.findViewById(R.id.text5);
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        viewHolder.text1.setText(list.get(position).getItemName());
        viewHolder.text2.setText(list.get(position).getKCUom());
        viewHolder.text3.setText(list.get(position).getPrice() + "");
        viewHolder.text4.setText(list.get(position).getTQTY() + "");
        viewHolder.text5.setText(list.get(position).getAmount() + "");
        if (tag != null) {
            viewHolder.text1.setOnClickListener(new OnClickListener() {

                @Override
                public void onClick(View v) {

                    DialogHelper.Alert2(context, "提示", "是否删除该产品", null, new DialogInterface.OnClickListener() {

                        @Override
                        public void onClick(DialogInterface dialog, int which) {

                            list.remove(position);
                            context.onDialogResult(DELETE_TAG);

                        }
                    });
                }
            });
            viewHolder.text4.setOnClickListener(new CountEditListener(context, list, position));
        }
        viewHolder.text5.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View v) {

                boolean isCanEdit = DalCommon.CanEditAmount(list.get(position).getItemId());
                if (isCanEdit) {
                    Builder builder = new Builder(context);
                    builder.setTitle("请输入修改的金额");
                    final EditText editText = new EditText(context);
                    editText.setKeyListener(new DigitsKeyListener(false, true));
                    editText.setText(list.get(position).getAmount() + "");
                    builder.setView(editText);
                    builder.setNegativeButton("取消", null);
                    builder.setPositiveButton("确定", new DialogInterface.OnClickListener() {

                        @Override
                        public void onClick(DialogInterface dialog, int which) {

                            dialog.dismiss();
                            String num = editText.getText().toString().trim();
                            if (!TextUtils.isEmpty(num)) {
                                list.get(position).setAmount(Double.parseDouble(num));
                                context.onDialogResult(SellAdapter.EDIT_TAG);
                            }
                        }

                    });
                    builder.create().show();

                } else {
                    context.showMsg("该产品不能修改金额！");
                }

            }
        });
        return convertView;
    }

    class ViewHolder {
        TextView text1;
        TextView text2;
        TextView text3;
        TextView text4;
        TextView text5;

    }

}
