package com.urovo.wgmf.net;


import com.urovo.activity.data.querymyweivip.model.SweepCodeDetailListEntity;
import com.urovo.activity.data.queryproductsales.model.MonthSaleProductEntity;
import com.urovo.activity.data.querysales.model.MonthSaleEntity;
import com.urovo.activity.user.bean.FlowSalesPersonCompletionHeadEntity;
import com.urovo.activity.sanxiangpao.model.SanXiangPaoCountEntity;
import com.urovo.activity.sanxiangpao.model.SanXiangPaoListEntity;
import com.urovo.activity.user.bean.FlowSalesPersonCompletionListEntity;
import com.urovo.wgmf.db.dao.entitys.StockGoodsDetailEntity;
import com.urovo.wgmf.entity.response.FirstPageQueryResponse;
import com.urovo.wgmf.db.dao.entitys.StockSearchEntity;
import com.urovo.wugumofang.constants.ApiUrls;
import com.zsy.libcore.model.bean.Request2Result;
import com.zsy.libcore.model.bean.RequestResult;

import java.util.List;
import java.util.Map;

import retrofit2.Call;
import retrofit2.http.POST;
import retrofit2.http.Query;
import retrofit2.http.QueryMap;

/**
 * retrofit 请求需要用到接口
 *
 * @author chenji
 */
public interface ICommonNet {
    /**
     * 符合/不符合新客微会员 共用
     *
     * @param shopId 门店编号
     * @param ymd    年月日 ,格式：yyyy-MM-dd
     * @return InConformityEntity对象
     */
    @POST(ApiUrls.APP.SWEEP_CODE_DETAIL)
    Call<RequestResult<SweepCodeDetailListEntity>> getSweepCodeDetailListData(
            @Query("staffId") String staffId,
            @Query("shopId") String shopId,
            @Query("ymd") String ymd
    );

    /**
     * 销售业绩----月销量查询
     * 门店和会员商城共用
     *
     * @param shopId 门店编号
     * @param ym     年月 ,格式：yyyy-MM
     * @return MonthSaleEntity对象
     */
    @POST(ApiUrls.APP.MONTH_CONSUMPTION)
    Call<RequestResult<MonthSaleEntity>> getMonthSaleData(
            @Query("shopId") String shopId,
            @Query("ym") String ym,
            @Query("staffId") String staffId
    );

    /**
     * 销售业绩----产品月销量查询
     * 门店和会员商城共用
     *
     * @param shopId 门店编号
     * @param ym     年月 ,格式：yyyy-MM
     * @return MonthSaleProductEntity对象
     */
    @POST(ApiUrls.APP.MONTH_SALE_PRODUCT_DETAIL)
    Call<RequestResult<MonthSaleProductEntity>> getMonthSaleProductDetailData(
            @Query("shopId") String shopId,
            @Query("ym") String ym,
            @Query("staffId") String staffId
    );

    /**
     * 流动顾问销售完成情况(头部信息)
     *
     * @param map ...
     * @return ...
     */
    @POST(ApiUrls.APPSALER.FLOW_SALES_PERSON_COMPLETION)
    Call<RequestResult<FlowSalesPersonCompletionHeadEntity>> getFlowSalesPersonCompletionHeadData(@QueryMap Map<String, String> map);

    /**
     * 流动顾问销售完成情况(列表)
     *
     * @param map ...
     * @return ...
     */
    @POST(ApiUrls.APPSALER.FLOW_SALES_PERSON_COMPLETION)
    Call<RequestResult<List<FlowSalesPersonCompletionListEntity>>> getFlowSalesPersonCompletionListData(@QueryMap Map<String, String> map);

    /**
     * 库存查询主档
     *
     * @param storeNo 参数 门店号
     * @return 对象
     */
    @POST(ApiUrls.APP.STOCK_SEARCH)
    Call<RequestResult<StockSearchEntity>> getStockSearchData(@Query("storeNo") String storeNo);

    /**
     * 库存查询明细列表
     *
     * @param storeNo 参数 门店号
     * @return 明细列表
     */
    @POST(ApiUrls.APP.STOCK_GOODS_DETAIL)
    Call<RequestResult<StockGoodsDetailEntity>> getStockGoodsDetailData(@Query("storeNo") String storeNo);

    /**
     * 获取会员关怀人数
     *
     * @param map 三响炮会员关怀人数需要的参数集合
     * @return ...
     */
    @POST(ApiUrls.APP.GET_CUSTOMER_CARE_COUNT)
    Call<RequestResult<SanXiangPaoCountEntity>> getSanXiangPaoQueryCount(
            @QueryMap Map<String, String> map
    );

    /**
     * 获取会员关怀列表
     *
     * @param map 三响炮数据列表需要的参数集合
     * @return ...
     */
    @POST(ApiUrls.APP.GET_CARE_CUSTOMER_LIST)
    Call<RequestResult<List<SanXiangPaoListEntity>>> getSanXiangPaoQueryList(
            @QueryMap Map<String, String> map
    );

    /**
     * 首页除了本月薪资预估数据外的门店数据（包括首页轮播图）的接口
     *
     * @param userId  员工的工号
     * @param storeId 门店的ID
     * @return ...
     */
    //http://test.wgmf.com/app/information/getStoreTaskInfo
    @POST(ApiUrls.APP.STORE_TASK_INFO)
    Call<Request2Result<FirstPageQueryResponse.InnerClass>> getStoreTaskInfo(
            @Query("userId") String userId,
            @Query("storeId") String storeId
    );

    /**
     * 考勤单
     *
     * @param map 考勤打卡上传需要的参数集合
     * @return ...
     */
    @POST(ApiUrls.APPSALER.SIGN_IN_OR_OUT_MSG)
    Call<Request2Result<Object>> getUpLoadSignInOrOutData(
            @QueryMap Map<String, String> map
    );
}
