package com.urovo.wgmf.tool;

import android.app.Activity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.view.Gravity;
import android.view.View;

import com.alibaba.android.arouter.launcher.ARouter;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.constants.RouterParams;
import com.urovo.wugumofang.notifylibrary.lib.Configuration;
import com.urovo.wugumofang.notifylibrary.lib.Effects;
import com.urovo.wugumofang.notifylibrary.lib.NiftyNotificationView;

import androidx.core.app.NotificationCompat;
import androidx.core.app.TaskStackBuilder;

/**
 * 自定义通知条幅显示
 *
 * @author chenji
 * @date 2017/10/11
 */

public class NotificationCenterManager {
    private static Configuration cfg = null;
    private static NotificationManager mNotificationManager;
    private static NiftyNotificationView mInstance;

    /**
     * 页面横幅通知
     *
     * @param activity       ...
     * @param tickerText     ...
     * @param contentTitle   ...
     * @param msg            ...
     * @param cls            ...
     * @param notifyId       ...
     * @param effect         ...
     * @param viewGroupResId ...
     * @param iconRes        ... 图片资源地址
     * @param count          ... 考勤异常数量
     */
    public static void showNotification(final Activity activity, CharSequence tickerText,
                                        CharSequence contentTitle, CharSequence msg, Class<?> cls, int notifyId,
                                        Effects effect, int viewGroupResId, int iconRes, final int count) {
        if (cfg == null) {
            mInstance = NiftyNotificationView.build(activity, contentTitle + ", " + msg, effect, viewGroupResId)
                    .setIcon(iconRes)         //You must call this method if you use ThumbSlider effect
                    .setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            if (count > 0) {
                                ARouter.getInstance().build(RouterParams.MoreIndex.UntreatedOrder).navigation();
                            }
                        }
                    });
        } else {
            mInstance = NiftyNotificationView.build(activity, contentTitle + ", " + msg, effect, viewGroupResId, cfg)
                    .setIcon(iconRes)               //remove this line ,only text
                    .setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            if (count > 0) {
                                ARouter.getInstance().build(RouterParams.MoreIndex.UntreatedOrder).navigation();
                            }
                        }
                    });
        }
        mInstance.show();
//        invokeSystemNotificationBar(activity, iconRes, contentTitle, msg, cls, notifyId);
        invokeSimpleNB(activity, iconRes, tickerText, contentTitle, msg, cls, notifyId, count);
    }

    public static void hideNotification() {
        if (mInstance != null) {
            mInstance.hide();
        }
    }

    @SuppressWarnings("all")
    public static void setDetailedConfiguration(long animDuration, long dispalyDuration,
                                                String backgroundColor, String textColor,
                                                String iconBackgroundColor, int textPadding,
                                                int viewHeight, int textLines) {
        cfg = new Configuration.Builder()
                .setAnimDuration(animDuration)
                .setDispalyDuration(dispalyDuration)//通知显示消失的时间间隔
                .setBackgroundColor(backgroundColor)
                .setTextColor(textColor)
                .setIconBackgroundColor(iconBackgroundColor)
                .setTextPadding(textPadding)                      //dp
                .setViewHeight(viewHeight)                      //dp
                .setTextLines(textLines)                        //You had better use setViewHeight and setTextLines together
                .setTextGravity(Gravity.CENTER)         //only text def  Gravity.CENTER,contain icon Gravity.CENTER_VERTICAL
                .build();
    }

    private static void invokeSimpleNB(Context context, int smallIcon,
                                       CharSequence tickerText, CharSequence contentTitle,
                                       CharSequence contentText, Class<?> cls, int notifyId, int count) {
//        initNotificationManager(context);
        //为了版本兼容  选择V7包下的NotificationCompat进行构造
        NotificationCompat.Builder builder = new NotificationCompat.Builder(context, AppConstants.NOTI_CHANNEL_ID.HIGH);
        //Ticker是状态栏显示的提示
        builder.setTicker(tickerText);
        //第一行内容  通常作为通知栏标题
        builder.setContentTitle(contentTitle);
        //第二行内容 通常是通知正文
        builder.setContentText(contentText);
        //第三行内容 通常是内容摘要什么的 在低版本机器上不一定显示
//        builder.setSubText("这里显示的是通知第三行内容！");
        //ContentInfo 在通知的右侧 时间的下面 用来展示一些其他信息
        //builder.setContentInfo("3");
        //number设计用来显示同种通知的数量和ContentInfo的位置一样，如果设置了ContentInfo则number会被隐藏
//        builder.setNumber(2);
        //可以点击通知栏的删除按钮删除
        builder.setAutoCancel(true);
        //系统状态栏显示的小图标
        builder.setSmallIcon(smallIcon);
        //下拉显示的大图标
//        builder.setLargeIcon(BitmapFactory.decodeResource(context.getResources(), R.mipmap.ic_launcher_round));
        if (count > 0) {
            Intent intent = new Intent(context, cls);
            PendingIntent pIntent = PendingIntent.getActivity(context, 1, intent, PendingIntent.FLAG_UPDATE_CURRENT);
            //点击跳转的intent
            builder.setContentIntent(pIntent);

//            ARouter.getInstance().build(RouterParams.MoreIndex.UntreatedOrder).navigation();

//            Intent resultIntent = new Intent(context, cls);
//            // The stack builder object will contain an artificial back stack for the started Activity.
//            // This ensures that navigating backward from the Activity leads out of your application to the Home screen.
//            TaskStackBuilder stackBuilder = TaskStackBuilder.create(context);
//            // Adds the back stack for the Intent (but not the Intent itself)
//            stackBuilder.addNextIntent(resultIntent);
//            PendingIntent resultPendingIntent = stackBuilder.getPendingIntent(0, PendingIntent.FLAG_UPDATE_CURRENT);
//            builder.setContentIntent(resultPendingIntent);
        }
        //通知默认的声音 震动 呼吸灯
        builder.setDefaults(NotificationCompat.DEFAULT_ALL);
        Notification notification = builder.build();
        mNotificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
        mNotificationManager.notify(notifyId, notification);
    }

    /**
     * 删除系统所有通知栏消息
     */
    public static void notificationCancle() {
        if (mNotificationManager != null) {
            mNotificationManager.cancelAll();
        }
    }

    /**
     * 删除系统指定的通知栏消息
     */
    public static void notificationCancle(int notifyId) {
        if (mNotificationManager != null) {
            mNotificationManager.cancel(notifyId);
        }
    }

    public static void notificationCancle(String tag, int notifyId) {
        if (mNotificationManager != null) {
            mNotificationManager.cancel(tag, notifyId);
        }
    }

    private static void invokeSystemNotificationBar(Context context, int smallIcon, CharSequence contentTitle, CharSequence contentText, Class<?> cls, int notifyId) {
        NotificationCompat.Builder mBuilder = new NotificationCompat.Builder(context,AppConstants.NOTI_CHANNEL_ID.HIGH)
                .setSmallIcon(smallIcon)
//                .setColor(Color.parseColor("#EAA935"))
                .setContentTitle(contentTitle)
                .setContentText(contentText)
                .setDefaults(Notification.DEFAULT_ALL)
//                .setStyle(new NotificationCompat.BigTextStyle().bigText("点击打开本机管理处理"))
                ;
        // Creates an explicit intent for an Activity in your app
        Intent resultIntent = new Intent(context, cls);
        // The stack builder object will contain an artificial back stack for the started Activity.
        // This ensures that navigating backward from the Activity leads out of your application to the Home screen.
        TaskStackBuilder stackBuilder = TaskStackBuilder.create(context);
        // Adds the back stack for the Intent (but not the Intent itself)
        stackBuilder.addNextIntent(resultIntent);
        PendingIntent resultPendingIntent = stackBuilder.getPendingIntent(0, PendingIntent.FLAG_UPDATE_CURRENT);
        mBuilder.setContentIntent(resultPendingIntent);
//        mBuilder.setFullScreenIntent(resultPendingIntent, true);
//        mBuilder.addAction(R.mipmap.ic_launcher_round,
//                getString(R.string.app_name), resultPendingIntent);
        mNotificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
        // mId allows you to update the notification later on.
        mNotificationManager.notify(notifyId, mBuilder.build());
    }
}
