package com.urovo.wgmf.tool;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.Uri;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.os.SystemClock;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.hjq.permissions.OnPermission;
import com.hjq.permissions.XXPermissions;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.constants.Files;
import com.urovo.wugumofang.net.HttpUtil;
import com.urovo.wugumofang.net.HttpUtil.downFileProgress;
import com.urovo.wugumofang.utils.FuckGoogleAdaptUtil;
import com.zsy.libcore.ui.act.SYActManager;

import java.io.File;
import java.text.DecimalFormat;
import java.util.List;

/**
 * 应用程序更新工具包
 */
public class UpdateManager {

    private static final int DOWN_NOSDCARD = 0;
    private static final int DOWN_UPDATE = 1;
    private static final int DOWN_OVER = 2;
    private static final int DIALOG_TYPE_LATEST = 0;
    private static final int DIALOG_TYPE_FAIL = 1;
    private static UpdateManager updateManager;
    private Context context;
    // 通知对话框
    private Dialog noticeDialog;
    // 下载对话框
    private Dialog downloadDialog;
    // '已经是最新' 或者 '无法获取最新版本' 的对话框
    private Dialog latestOrFailDialog;
    // 进度条
    private ProgressBar mProgress;
    // 显示下载数值
    private TextView mProgressText;
    // 查询动画
    private ProgressDialog mProDialog;
    // 进度值
    private int progress;
    // 下载线程
    private Thread downLoadThread;
    // 终止标记
    private boolean interceptFlag;
    // 提示语
    private String updateMsg = "";
    // 返回的安装包url
    private String apkUrl;
    // 下载包保存路径
    private String savePath = "";
    // 解压保存路劲
    private String zipPath = "";
    // apk保存完整路径
    private String apkFilePath = "";
    // 临时下载文件路径
    private String tmpFilePath = "";
    // 下载文件大小
    private String apkFileSize;
    // 已下载文件大小
    private String tmpFileSize;
    private String apkfiles;
    public String curVersionName = "";
    private int curVersionCode;
    private String upLoadVersionCode;

    public UpdateManager(String upLoadVersionCode) {
        this.upLoadVersionCode = upLoadVersionCode;
    }

    public UpdateManager(String versionString, String upLoadString) {


        this.upLoadVersionCode = versionString;
        this.apkUrl = upLoadString;
    }

    private Handler mHandler = new Handler() {
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case DOWN_UPDATE:
                    mProgress.setProgress(progress);
                    mProgressText.setText(tmpFileSize + "/" + apkFileSize);
                    break;
                case DOWN_OVER:
                    downloadDialog.dismiss();
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                        XXPermissions.with((Activity) context)
                                .constantRequest() //可设置被拒绝后继续申请，直到用户授权或者永久拒绝
                                //.permission(Permission.SYSTEM_ALERT_WINDOW, Permission.REQUEST_INSTALL_PACKAGES) //支持请求6.0悬浮窗权限8.0请求安装权限
//                    .permission(Permission.Group.STORAGE, Permission.Group.CALENDAR) //不指定权限则自动获取清单中的危险权限
                                .permission(//存储
                                        //安装APK
                                        Manifest.permission.REQUEST_INSTALL_PACKAGES)
                                .request(new OnPermission() {
                                    @Override
                                    public void hasPermission(List<String> granted, boolean isAll) {
                                        installApk();
                                    }

                                    @Override
                                    public void noPermission(List<String> denied, boolean quick) {
                                        if (quick){
                                            new AlertDialog.Builder(context).setTitle("提示")
                                                    .setMessage("为保证生意管家的正常运行，请打开相应权限")
                                                    .setPositiveButton("去打开", new DialogInterface.OnClickListener() {
                                                        @Override
                                                        public void onClick(DialogInterface dialog, int which) {
                                                            XXPermissions.gotoPermissionSettings((context));
                                                        }
                                                    })
                                                    .show();
                                        }
                                    }
                                });
                    }else{//如果是6.0以下的，就直接安装
                        installApk();
                    }
                    break;
                case DOWN_NOSDCARD:
                    downloadDialog.dismiss();
//                    MyToast.show(context, "无法下载安装文件，请检查SD卡是否挂载", 3000);
                    Toast.makeText(context, "无法下载安装文件，请检查SD卡是否挂载", Toast.LENGTH_LONG).show();
                    break;
            }
        }

        ;
    };

    /**
     * 检查App更新
     * @param context
     * @return
     */
    public boolean checkAppUpdate(Context context) {
        this.context = context;
        getCurrentVersion();
        if (upLoadVersionCode == null || curVersionName == null)
            return false;
        //可弹窗下载的情况：1、当前版本号比传过来的版本号小  2、当前版本号是4位，即beta版，提供回滚到3位的正式版。3、强行切换的Beta环境
//        if (curVersionName.compareTo(upLoadVersionCode) < 0 || NetParam.YINXIAOBU_SERVER.equals(ApiConstants.productionEnvServer)) {
//            showDownloadDialog(context);
//            return true;
//        } else {
//            return false;
//        }
        showDownloadDialog(context);
        return true;

    }

    /**
     * 显示'已经是最新'或者'无法获取版本信息'对话框
     */
    private void showLatestOrFailDialog(int dialogType) {
        if (latestOrFailDialog != null) {
            // 关闭并释放之前的对话框
            latestOrFailDialog.dismiss();
            latestOrFailDialog = null;
        }
        AlertDialog.Builder builder = new Builder(context);
        builder.setTitle("系统提示");
        if (dialogType == DIALOG_TYPE_LATEST) {
            builder.setMessage("您当前已经是最新版本");
        } else if (dialogType == DIALOG_TYPE_FAIL) {
            builder.setMessage("无法获取版本更新信息");
        }
        builder.setPositiveButton("更新", null);
        latestOrFailDialog = builder.create();
        latestOrFailDialog.show();
    }

    /**
     * 获取当前客户端版本信息
     */
    public void getCurrentVersion() {
        try {
            PackageInfo info = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
            curVersionName = info.versionName;
            curVersionCode = info.versionCode;
        } catch (NameNotFoundException e) {
            e.printStackTrace(System.err);
        }
    }

    /**
     * 显示版本更新通知对话框
     */
    private void showNoticeDialog() {
        AlertDialog.Builder builder = new Builder(context);
        builder.setTitle("软件版本更新");
        builder.setMessage(updateMsg);
        builder.setPositiveButton("立即更新", new OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();

            }
        });
        builder.setNegativeButton("以后再说", new OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
            }
        });
        noticeDialog = builder.create();
        noticeDialog.show();
    }

    /**
     * 显示下载对话框
     */
    public void showDownloadDialog(Context context) {
        if(this.context == null){
            this.context = context;
        }
        AlertDialog.Builder builder = new Builder(context);
        builder.setTitle("正在下载新版本" + upLoadVersionCode);
        final LayoutInflater inflater = LayoutInflater.from(context);
        View v = inflater.inflate(R.layout.update_progress, null);
        mProgress = (ProgressBar) v.findViewById(R.id.update_progress);
        mProgressText = (TextView) v.findViewById(R.id.update_progress_text);
        builder.setView(v);
        builder.setCancelable(false);
        builder.setNegativeButton("取消下载，退出应用", new OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
                interceptFlag = true;
                SYActManager.getInstance().AppExit();
            }
        });
        builder.setOnCancelListener(new OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                dialog.dismiss();
                interceptFlag = true;
                SYActManager.getInstance().AppExit();
            }
        });
        downloadDialog = builder.create();
        downloadDialog.setCanceledOnTouchOutside(false);
        downloadDialog.show();
        downloadApk();
    }

    private Runnable mdownApkRunnable = new Runnable() {
        @Override
        public void run() {
            try {
                String apkName = "update" + upLoadVersionCode + ".apk";
                apkfiles = Files.downfile;//唉写死算了。。。反正又没几个地方用// "mnt/sdcard/ZT/UPAPK/";
                File file = new File(apkfiles);
                String cmd = "chmod 777" + file.getAbsolutePath();
                try {
                    Runtime.getRuntime().exec(cmd);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                if (!file.exists())
                    file.mkdirs();
                //apkFilePath = apkfiles + Files.upgname;// "/ZTO.zip";
                apkFilePath = apkfiles + apkName;// "/ZTO.zip";
                HttpUtil.downloadFile(apkUrl, apkFilePath, new downFileProgress() {
                    @Override
                    public void onStart(int total) {
                        // 显示文件大小格式：2个小数点显示
                        DecimalFormat df = new DecimalFormat("0.00");
                        // 进度条下面显示的总文件大小
                        apkFileSize = df.format((float) total / 1024 / 1024) + "MB";
                        interceptFlag = false;
                    }

                    @Override
                    public boolean onProgress(int total, int pro) {

                        DecimalFormat df = new DecimalFormat("0.00");
                        // 进度条下面显示的当前下载文件大小
                        tmpFileSize = df.format((float) pro / 1024 / 1024) + "MB";
                        // 当前进度值
                        progress = (int) (((float) pro / total) * 100);
                        // 更新进度
                        mHandler.sendEmptyMessage(DOWN_UPDATE);
                        return interceptFlag;
                    }

                    @Override
                    public void onFinish(int total, int pro) {

                        // 通知安装
                        mHandler.sendEmptyMessage(DOWN_OVER);
                    }
                });

                // FileOutputStream fos = new FileOutputStream(ApkFile);
                // URL url = new URL(apkUrl);
                // HttpURLConnection conn = (HttpURLConnection)
                // url.openConnection();
                // conn.connect();
                // int length = conn.getContentLength();
                // InputStream is = conn.getInputStream();
                // // 显示文件大小格式：2个小数点显示
                // DecimalFormat df = new DecimalFormat("0.00");
                // // 进度条下面显示的总文件大小
                // apkFileSize = df.format((float) length / 1024 / 1024) + "MB";
                // int count = 0;
                // byte buf[] = new byte[1024];
                // do
                // {
                // int numread = is.read(buf);
                // count += numread;
                // // 进度条下面显示的当前下载文件大小
                // tmpFileSize = df.format((float) count / 1024 / 1024) + "MB";
                // // 当前进度值
                // progress = (int) (((float) count / length) * 100);
                // // 更新进度
                // mHandler.sendEmptyMessage(DOWN_UPDATE);
                // if (numread <= 0)
                // {
                // // 下载完成 - 将临时下载文件转成APK文件
                // // if (tmpFile.renameTo(ApkFile)) {
                // // 通知安装
                // mHandler.sendEmptyMessage(DOWN_OVER);
                // // }
                // break;
                // }
                // fos.write(buf, 0, numread);
                // }
                // while (!interceptFlag);// 点击取消就停止下载
                // fos.close();
                // is.close();
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
    };

    /**
     * 下载apk
     */
    public void downloadApk() {
        downLoadThread = new Thread(mdownApkRunnable);
        downLoadThread.start();

    }

    /**
     * 安装apk
     */
    private void installApk() {
        zipPath = apkfiles + "update" + upLoadVersionCode + ".apk";
        //File apkfile = new File(apkFilePath);
        try {
/*			// File file=new File("/data/data/com.urovo.YunDa/files");
            ZipToFile.upZipFile(apkFilePath, zipPath);

			if(!apkfile.exists()) {
				return;
			}
			// String cmd = "chmod 777 " +file.getAbsolutePath();
			// String cmd2 = "chmod 777 " +apkfile.getAbsolutePath();
			// Runtime.getRuntime().exec(cmd);
			// Runtime.getRuntime().exec(cmd2);*/


//            Uri uri = Uri.fromFile(new File(zipPath));
            Uri uri = FuckGoogleAdaptUtil.android7AdaptUri(context,Files.fileProviderAuth,new File(zipPath));//FileProvider.getUriForFile(context,Files.fileProviderAuth,new File(zipPath));
            Intent intent = new Intent();
            intent.setAction(android.content.Intent.ACTION_VIEW);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            intent.setDataAndType(uri, "application/vnd.android.package-archive");
            context.startActivity(intent);
            SystemClock.sleep(2000);
            SYActManager.getInstance().AppExit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 安装是否成功
     */

    /**
     * 比较两个字符串,前者大返回true
     *
     * @param s1
     * @param s2
     * @return result boolean
     */
    private int compare(String s1, String s2) {
        return s1.compareTo(s2);
    }

}
