package com.urovo.wgmf.view;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import android.content.Context;
import android.graphics.Color;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.GridView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.urovo.wgmf.db.dao.entitys.CalendarBaseModelResponse;
import com.urovo.wugumofang.R;

public class DayTaskCalendar extends LinearLayout {

	private static Context context;
	private static String dayGrid;
	private static String choiceMonth;// 选中的年月
	private static Date nowDate;// 当前日期
	private static String viewDate;// 控件显示日期
	private static List<String> gvList = new ArrayList<String>();// 存放天
	private static List<CalendarBaseModelResponse> daysHasThingList = new ArrayList<CalendarBaseModelResponse>();
	private GridView gv;
	private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM");// 日期格式化

	private OnDaySelectListener callBack;// 回调函数

	public calendarGridViewAdapter gridViewAdapter;

	public static String getChoiceMonth() {
		return choiceMonth;
	}

	public static void setChoiceMonth(String choiceMonth) {
		DayTaskCalendar.choiceMonth = choiceMonth;
	}

	/**
	 * 构造函数
	 * 
	 * @param context
	 */
	public DayTaskCalendar(Context context) {
		super(context);
		DayTaskCalendar.context = context;
		init();
	}

	/**
	 * 构造函数
	 * 
	 * @param context
	 */
	public DayTaskCalendar(Context context, AttributeSet attrs) {
		super(context, attrs);
		DayTaskCalendar.context = context;
		init();
	}

	/**
	 * 初始化日期以及view等控件
	 */
	private void init() {
		final Calendar ca = Calendar.getInstance();// 获取日历实例
		nowDate = new Date();// 获取当前date
		ca.setTime(nowDate);// cal设置为当天的
		Date resultDate = ca.getTime(); // 结果
		String num = dateFormat.format(resultDate);
		choiceMonth = num;
		ca.set(Calendar.DATE, 1);// cal设置当前day为当前月第一天
		int tempSum = countNeedHowMuchEmpety(ca);// 获取当前月第一天为星期几
		int dayNumInMonth = getDayNumInMonth(ca);// 获取当前月有多少天
		setGvListData(tempSum, dayNumInMonth);

		View view = LayoutInflater.from(context).inflate(R.layout.day_task_calendar, this, true);// 获取布局，开始初始化

		gv = (GridView) view.findViewById(R.id.gv_calendar);
		gridViewAdapter = new calendarGridViewAdapter();
		gv.setAdapter(gridViewAdapter);
		setGvListData(countNeedHowMuchEmpety(ca), getDayNumInMonth(ca));
		gridViewAdapter.notifyDataSetChanged();
		gv.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> adapterView, View arg1, int position, long arg3) {
				String choiceDay = (String) adapterView.getAdapter().getItem(position);
				if(!"".equals(choiceDay)) {
					if(Integer.parseInt(choiceDay) < 10) {
						choiceDay = "0" + choiceDay;
					}
					String date = choiceMonth + "-" + choiceDay;
					if(callBack != null) {// 调用回调函数回调数据
						callBack.onDaySelectListener(date);
					}
				}
			}
		});
	}

	/**
	 * 为gridview中添加需要展示的数据
	 * 
	 * @param tempSum
	 * @param dayNumInMonth
	 */
	public static void setGvListData(int tempSum, int dayNumInMonth) {
		gvList.clear();
		for (int i = 0; i < tempSum; i++) {
			gvList.add("");
		}

		for (int j = 1; j <= dayNumInMonth; j++) {
			gvList.add(String.valueOf(j));
		}
	}

	/**
	 * 获取当前月的总共天数
	 * 
	 * @param cal
	 * @return
	 */
	public static int getDayNumInMonth(Calendar cal) {
		return cal.getActualMaximum(Calendar.DATE);
	}

	/**
	 * 获取当前月第一天在第一个礼拜的第几天，得出第一天是星期几
	 * 
	 * @param cal
	 * @return
	 */
	public static int countNeedHowMuchEmpety(Calendar cal) {
		int firstDayInWeek = cal.get(Calendar.DAY_OF_WEEK) - 1;
		return firstDayInWeek;
	}

	static class GrideViewHolder {

		TextView tvDay;
	}

	public static class calendarGridViewAdapter extends BaseAdapter {

		@Override
		public int getCount() {
			return gvList.size();
		}

		@Override
		public String getItem(int position) {
			return gvList.get(position);
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			GrideViewHolder holder;
			if(convertView == null) {
				holder = new GrideViewHolder();
				convertView = inflate(context, R.layout.day_task_calendar_gridview_item, null);
				holder.tvDay = (TextView) convertView.findViewById(R.id.tv_calendar_day);
				convertView.setTag(holder);
			}
			else {
				holder = (GrideViewHolder) convertView.getTag();
			}
			final CalendarBaseModelResponse cell = new CalendarBaseModelResponse();
			dayGrid = getItem(position);
			holder.tvDay.setText(dayGrid);

			if("" != dayGrid) {
				holder.tvDay.setTextColor(Color.parseColor("#FFFFFF"));
				holder.tvDay.setBackgroundColor(Color.parseColor("#EE6363"));
				viewDate = ProcessDate();
				if(viewDate.equals(getToday())) {// 设置当天字体颜色为黑色
					holder.tvDay.setTextColor(Color.parseColor("#000000"));
				}
				for (int i = 0; i < daysHasThingList.size(); i++) {
					cell.dayShow = daysHasThingList.get(i).dayShow;
					cell.total = daysHasThingList.get(i).total;
					cell.count = daysHasThingList.get(i).count;
					if(viewDate.equals(cell.dayShow)) {// 找到对应日期，设置背景颜色
						if(cell.total.equals(cell.count)) {// 已设置
							holder.tvDay.setBackgroundColor(Color.parseColor("#00FF00"));
						}
						else
							if("0".equals(cell.count)) {// 未设置
								holder.tvDay.setBackgroundColor(Color.parseColor("#EE6363"));
							}
							else {// 部分设置
								holder.tvDay.setBackgroundColor(Color.parseColor("#EEEE00"));
							}
					}
				}
			}
			else {
				holder.tvDay.setBackgroundColor(Color.parseColor("#FFFFFF"));
			}
			return convertView;
		}

		/**
		 * 
		 * @Title: ProcessDate
		 * @Description: 对日历控件日期进行格式加工
		 * @param @return 参数说明
		 * @return String 返回类型
		 * @throws
		 */
		private String ProcessDate() {
			String data;
			if(Integer.valueOf(dayGrid).intValue() < 10) {
				data = choiceMonth + "-0" + dayGrid;
			}
			else {
				data = choiceMonth + "-" + dayGrid;
			}
			return data;
		}
	}

	/**
	 * 
	 * @Title: getToday
	 * @Description: 获取当天日期
	 * @param @return 参数说明
	 * @return String 返回类型
	 * @throws
	 */
	private static String getToday() {
		Calendar calendar = Calendar.getInstance();
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
		String time = format.format(calendar.getTime());
		return time;
	}

	/**
	 * @ClassName: OnDaySelectListener
	 * @Description: 自定义监听接口
	 * @author XP
	 * @date 2015-12-29 上午11:04:48
	 */
	public interface OnDaySelectListener {
		void onDaySelectListener(String date);
	}

	/**
	 * 自定义监听接口设置对象
	 * 
	 * @param o
	 */
	public void setOnDaySelectListener(OnDaySelectListener o) {
		callBack = o;
	}

	/**
	 * @Title: setDaysHasThingList
	 * @Description: 设置事务
	 * @param @param daysHasThingList 参数说明
	 * @return void 返回类型
	 * @throws
	 */
	public void setDaysHasThingList(List<CalendarBaseModelResponse> daysHasThingList) {
		this.daysHasThingList = daysHasThingList;
		gridViewAdapter.notifyDataSetChanged();
	}
}
