package com.urovo.wgmf.view;

import android.content.Context;
import android.util.AttributeSet;
import android.view.MotionEvent;

import com.lsjwzh.widget.recyclerviewpager.RecyclerViewPager;

import static android.view.MotionEvent.ACTION_DOWN;
import static android.view.MotionEvent.ACTION_MOVE;
import static android.view.MotionEvent.ACTION_UP;

/**
 * User: lemonade(823307073@qq.com)
 * Date: 2017/05/14
 * Time: 00:48
 * 自定义ViewPager，解决不同向的滑动冲突。
 */
public class ViewPager extends RecyclerViewPager {
    private boolean inHorizonMoving;
    private boolean inVerticalMoving;
    private MotionEvent startEvent;

    public ViewPager(Context context) {
        super(context);
    }

    public ViewPager(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    public ViewPager(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
    }

    /**
     * 暂时处理 横向和竖向的滑动冲突（有更好的处理方式，欢迎修正）
     *
     * @param ev ...
     * @return ...
     */
    @Override
    public boolean dispatchTouchEvent(MotionEvent ev) {
        switch (ev.getAction()) {
            case ACTION_DOWN:
                inHorizonMoving = inVerticalMoving = false;
                startEvent = MotionEvent.obtain(ev);
//                Log.d("按下", "按下了" + ev.getX() + "  " + ev.getY());
                return super.dispatchTouchEvent(ev);
            case ACTION_MOVE:
//                Log.d("MOVE", "Move中" + ev.getX() + "  " + ev.getY());

                float deltaX = Math.abs(startEvent.getX() - ev.getX());
                float deltaY = Math.abs(startEvent.getY() - ev.getY());
                float tan = deltaY / deltaX;
//                Log.d("tan", "tan=" + tan);

                if (tan < 0.46) {
                    // 水平滚动
//                    Log.d("水平滚动", "水平滚动");
                    inHorizonMoving = true;
                    MotionEvent cloneMotionEvent = MotionEvent.obtain(ev);
                    if (inVerticalMoving)
                        cloneMotionEvent.setLocation(startEvent.getX(), cloneMotionEvent.getY());
                    else
                        cloneMotionEvent.setLocation(cloneMotionEvent.getX(), startEvent.getY());
                    return super.dispatchTouchEvent(cloneMotionEvent);
                } else if (Float.isInfinite(tan) || Float.isNaN(tan)) {
//                    Log.d("无滚动", "无滚动");
                    return true;
                } else {
                    // 垂直滚动
//                    Log.d("垂直滚动", "垂直滚动");
                    inVerticalMoving = true;
                    MotionEvent cloneMotionEvent = MotionEvent.obtain(ev);
                    if (inHorizonMoving)
                        cloneMotionEvent.setLocation(cloneMotionEvent.getX(), startEvent.getY());
                    else
                        cloneMotionEvent.setLocation(startEvent.getX(), cloneMotionEvent.getY());

                    return super.dispatchTouchEvent(cloneMotionEvent);
                }
            case ACTION_UP:
                inHorizonMoving = inVerticalMoving = false;
//                Log.d("放开", "放开了" + ev.getX() + "  " + ev.getY());
                startEvent = null;
                return super.dispatchTouchEvent(ev);

        }
        return super.dispatchTouchEvent(ev);
    }

//    int mLastMotionY;
//    int mLastMotionX;
//
//    @Override
//    public boolean onInterceptTouchEvent(MotionEvent e) {
//        int y = (int) e.getRawY();
//        int x = (int) e.getRawX();
//        boolean resume = false;
//        switch (e.getAction()) {
//            case MotionEvent.ACTION_DOWN:
//                // 发生down事件时,记录y坐标
//                mLastMotionY = y;
//                mLastMotionX = x;
//                resume = false;
//                break;
//            case MotionEvent.ACTION_MOVE:
//                // deltaY > 0 是向下运动,< 0是向上运动
//                int deltaY = y - mLastMotionY;
//                int deleaX = x - mLastMotionX;
//                //当前正处于纵向滑动，父类需要拦截，让子类滑动
//                if (Math.abs(deleaX) < Math.abs(deltaY)) {
//                    resume = false;
//                } else {
//                    //当前正处于横向滑动，不需要拦截，父类自己消费事件
//                    if (isScrollContainer()) {
//                        Log.d(TAG, "onInterceptTouchEvent: "
//                        +isScrollContainer()
//                        +isVerticalScrollBarEnabled()
//                        +isHorizontalScrollBarEnabled()
//                        +isScrollbarFadingEnabled()
//                        +isNestedScrollingEnabled());
//                        resume = true;
//                    }
//                }
//                break;
//            case MotionEvent.ACTION_UP:
//            case MotionEvent.ACTION_CANCEL:
//                break;
//        }
//        return resume;
//    }
}
