/*
 * Copyright (C) 2008 ZXing authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.urovo.wgmf.zxing;

import java.util.Hashtable;
import java.util.Vector;
import java.util.concurrent.CountDownLatch;

import android.content.SharedPreferences;
import android.os.Handler;
import android.os.Looper;
import android.preference.PreferenceManager;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.DecodeHintType;
import com.google.zxing.ResultPointCallback;
import com.urovo.wugumofang.constants.AppConstants;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.utils.MyToast;
import com.zsy.libext.widgets.tips.ToastUtil;

/**
 * This thread does all the heavy lifting of decoding the images.
 *
 * @author dswitkin@google.com (Daniel Switkin)
 */
final class DecodeThread extends Thread {

    public static final String BARCODE_BITMAP = "barcode_bitmap";

    private final CaptureActivity activity;
    private final Hashtable<DecodeHintType, Object> hints;
    private Handler handler;
    private final CountDownLatch handlerInitLatch;

    DecodeThread(CaptureActivity activity, Vector<BarcodeFormat> decodeFormats, String characterSet, ResultPointCallback resultPointCallback) {

        this.activity = activity;
        handlerInitLatch = new CountDownLatch(1);

        hints = new Hashtable<>(3);

        // // The prefs can't change while the thread is running, so pick them
        // up once here.
//        if (decodeFormats == null || decodeFormats.isEmpty()) {
//            SharedPreferences prefs =
//                    PreferenceManager.getDefaultSharedPreferences(activity);
//            decodeFormats = new Vector<>();
//            if (prefs.getBoolean(Preferences.KEY_DECODE_1D, false)) {
//                decodeFormats.addAll(DecodeFormatManager.ONE_D_FORMATS);
//                ToastUtil.show(activity,"一维码");
//            }
//            if (prefs.getBoolean(Preferences.KEY_DECODE_QR, false)) {
//                decodeFormats.addAll(DecodeFormatManager.QR_CODE_FORMATS);
//                ToastUtil.show(activity,"二维码");
//            }
//            if (prefs.getBoolean(Preferences.KEY_DECODE_DATA_MATRIX, false)) {
//                decodeFormats.addAll(DecodeFormatManager.DATA_MATRIX_FORMATS);
//                ToastUtil.show(activity,"矩阵码");
//            }
//        }

//        if (decodeFormats == null || decodeFormats.isEmpty()) {
//            decodeFormats = new Vector<>();
//            decodeFormats.addAll(DecodeFormatManager.ONE_D_FORMATS);
//            decodeFormats.addAll(DecodeFormatManager.QR_CODE_FORMATS);
//            decodeFormats.addAll(DecodeFormatManager.DATA_MATRIX_FORMATS);
//        }

        if (decodeFormats == null || decodeFormats.isEmpty()) {
            decodeFormats = new Vector<>();
            if (PreferHelper.getInstance().getBoolean(Preferences.KEY_DECODE_1D, false)) {
                decodeFormats.addAll(DecodeFormatManager.ONE_D_FORMATS);
//                ToastUtil.show(activity, "一维码");
            } else if (PreferHelper.getInstance().getBoolean(Preferences.KEY_DECODE_QR, false)) {
                decodeFormats.addAll(DecodeFormatManager.QR_CODE_FORMATS);
//                ToastUtil.show(activity, "二维码");
            } else if (PreferHelper.getInstance().getBoolean(Preferences.KEY_DECODE_DATA_MATRIX, false)) {
                decodeFormats.addAll(DecodeFormatManager.DATA_MATRIX_FORMATS);
//                ToastUtil.show(activity, "矩阵码");
            } else {
                //默认是兼容各种码
//                ToastUtil.show(activity, "默认是兼容各种码");
                decodeFormats = new Vector<>();
                decodeFormats.addAll(DecodeFormatManager.ONE_D_FORMATS);
                decodeFormats.addAll(DecodeFormatManager.QR_CODE_FORMATS);
                decodeFormats.addAll(DecodeFormatManager.DATA_MATRIX_FORMATS);
            }
        }

        hints.put(DecodeHintType.POSSIBLE_FORMATS, decodeFormats);

        if (characterSet != null) {
            hints.put(DecodeHintType.CHARACTER_SET, characterSet);
        }

        hints.put(DecodeHintType.NEED_RESULT_POINT_CALLBACK, resultPointCallback);
    }

    Handler getHandler() {
        try {
            handlerInitLatch.await();
        } catch (InterruptedException ie) {
            // continue?
        }
        return handler;
    }

    @Override
    public void run() {
        Looper.prepare();
        handler = new DecodeHandler(activity, hints);
        handlerInitLatch.countDown();
        Looper.loop();
    }

}
