@file:JvmName("BasicDataDownloader")

package com.wgmf.trade.model.data.db

import android.app.Activity
import android.app.AlertDialog
import android.text.TextUtils
import android.widget.Toast
import com.google.gson.Gson
import com.urovo.wgmf.db.dao.dao.Table_toposDbOperate
import com.urovo.wgmf.db.dao.entitys.ActionFreshBtnBean
import com.urovo.wgmf.db.dao.entitys.BasicDataRes
import com.urovo.wugumofang.constants.ApiUrls
import com.urovo.wugumofang.constants.Files
import com.urovo.wugumofang.constants.NetParam
import com.urovo.wugumofang.data.JsonDominator
import com.urovo.wugumofang.net.OkHttpDominator
import com.urovo.wugumofang.system.BaseAppManager
import com.urovo.wugumofang.system.SystemHelper
import com.urovo.wugumofang.ui.ViewUtil
import com.urovo.wugumofang.utils.MyToast
import okhttp3.Request
import org.greenrobot.eventbus.EventBus
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.uiThread
import java.lang.ref.WeakReference

/**
 * Created by Sasuke on 2017/9/7.
 * 专注于基础资料下载（数据库操作单独放在Table_toposDbOperate里）
 */
class BasicDataDownloader(activity: Activity?, loginOrganId: String, dealResultCallBack: DatabaseOperate) {
    private var ref: WeakReference<Activity?>? = null
    private var loginOrganId: String = ""//SystemHelper.LoginOrganId
    private var dealResultCallBack: DatabaseOperate//对下载下来的数据进行的处理
    //    var isDownAll = false//是否全量下载（现在用JSON来判断 ）
    //以下可以用set方法逐个添加进来
    private var isShowProgress = false //是否显示下载转圈圈
    private var isShowDialog = false//如果下载失败，是否“强制”弹窗
    private var timeJson = ""//时间戳，传空就是默认下载全部（val timeJson = Table_toposDbOperate.transToJson(Table_toposDbOperate.selectBy(null))）

    init {
        activity?.let { this.ref = WeakReference(activity) }
        this.loginOrganId = loginOrganId
        this.dealResultCallBack = dealResultCallBack
    }

    fun setIsShowProgress(isShowProgress: Boolean): BasicDataDownloader {
        this.isShowProgress = isShowProgress
        return this
    }

    fun setIsShowErrorDialog(isShowDialog: Boolean): BasicDataDownloader {
        this.isShowDialog = isShowDialog
        return this
    }

    fun setTimeJson(timeJson: String): BasicDataDownloader {
        this.timeJson = timeJson
        return this
    }

    /**
     * 解耦数据库操作
     */
    interface DatabaseOperate {
        fun dealBasicDataResult(res: BasicDataRes): Boolean
    }


    /**
     * 下载基础资料请求
     */
    fun download() {
        val params = hashMapOf<String, String>()
        if (!TextUtils.isEmpty(timeJson)) {//如果是按时间戳来下载
            BaseAppManager.loger().info("报文:开始读取时间戳")
            params.put("", timeJson)//json为时间戳
        } else {//如果是下载全部
            params.put("", Gson().toJson(mutableListOf<String>()))
        }
        BaseAppManager.loger().info("报文:开始请求基础资料")
        getActivity(ref)?.let { if (isShowProgress) ViewUtil.showProgress(getActivity(ref), "请稍等", "正在下载基础资料...") }
//        OkHttpDominator.postAsynByType("http://192.168.2.152:7000/" + ApiUrls.APPMANAGE.BASE_DATA_DOWN + loginOrganId,
//        测试用下面这个
//        OkHttpDominator.postAsynByType("http://192.168.5.52:7000/" + ApiUrls.APPMANAGE.BASE_DATA_DOWN + loginOrganId,
        OkHttpDominator.postAsynByType(NetParam.YINXIAOBU_SERVER + ApiUrls.APPMANAGE.BASE_DATA_DOWN + loginOrganId,
                OkHttpDominator.JSON,
                params,
                object : OkHttpDominator.ResultCallback<BasicDataRes>() {
                    override fun onError(request: Request?, e: Exception?) {
                        ViewUtil.dismissProgress()
                        //就算失败，也初始化一下品牌标记
                        Table_toposDbOperate.markBrand()
                        //下载失败 && 不存在缓存文件
                        if (!Files.getBasicDataCache(SystemHelper.LoginOrganId).exists()) {
                            showDownErrorDialog("DOWN_FAILED_CACHE_EMPTY_")
                            return
                        }
                        //网络异常，读缓存文件
                        val cache = JsonDominator.loadFromDir(Files.getBasicDataCache(SystemHelper.LoginOrganId), BasicDataRes::class.java, true)
                        cache?.run {
                            writeRes2DbAsync(this, false, isShowDialog)
                        }
                    }

                    override fun onResponse(res: BasicDataRes) {
                        ViewUtil.dismissProgress()
                        var resNew = res
                         JsonDominator.loadFromSD(BasicDataRes::class.java)?.let {
                             resNew = it
                         }

                        if (resNew.msgCode == "0") {//下载成功
                            //写入缓存文件
                            doAsync { JsonDominator.fire2Dir(Gson().toJson(resNew), Files.getBasicDataCache(SystemHelper.LoginOrganId), true) }
                            //建个新线程来写入基础资料到本地数据库
                            resNew?.run {
                                writeRes2DbAsync(this, true, isShowDialog)
                            }
                        } else {//下载失败
                            MyToast.show(BaseAppManager.appContext(), resNew.msgInfo, Toast.LENGTH_SHORT)
                            //读缓存文件
                            val cache = JsonDominator.loadFromDir(Files.getBasicDataCache(SystemHelper.LoginOrganId), BasicDataRes::class.java, true)
                            cache?.run {
                                writeRes2DbAsync(this, false, isShowDialog)
                            }
                        }
                    }
                })
    }


    /**
     * 建个新线程 写入到数据库
     * isShowDialog 是否强制显示错误弹窗
     * writeByCache 传进来的res是否是缓存
     */
    fun writeRes2DbAsync(res: BasicDataRes, downSuccess: Boolean, isShowDialog: Boolean) {
        doAsync {
            val dbResult: Boolean = dealResultCallBack.dealBasicDataResult(res)
            var error = ""
            if (!dbResult) error += "WRITE_FAILED_\n"
//        if (!Files.getBasicDataCache(SystemHelper.LoginOrganId).exists()) error += "CACHE_EMPTY_"
            if (!downSuccess) error += "DOWN_FAILED_"
            if (!error.isNullOrEmpty() && isShowDialog) showDownErrorDialog(error)

            //如果写入成功&& 下载成功 &&显示过等待圆圈圈，那就给个成功提示呗
            if (dbResult && downSuccess && isShowProgress && getActivity(ref) != null) {
                uiThread { MyToast.show(BaseAppManager.appContext(), "基础资料下载成功！", Toast.LENGTH_SHORT) }
            }
            BaseAppManager.loger().info("报文:基础资料处理完成")
            Table_toposDbOperate.markBrand()
            EventBus.getDefault().post(ActionFreshBtnBean())
        }
    }

    /**
     * 基础资料下载失败
     */
    fun showDownErrorDialog(errorMsg: String?) {
        AlertDialog.Builder(getActivity(ref)).setTitle("基础资料下载失败！")
                .setMessage(errorMsg)
                .setCancelable(false)
                .setPositiveButton("退出", { dialog, which -> System.exit(0) })
                .setNegativeButton("重新下载", { dialog, which ->
                    getActivity(ref)?.let {
                        setTimeJson("")//如果失败，就全量下载
                        download()
                    }
                })//重新下载
                .create().show()
    }

//    /**
//     * 处理下载基础资料的返回结果
//     */
//    fun dealBasicDataResult(res: BasicDataRes){
//
//
//    }


    fun getActivity(ref: WeakReference<Activity?>?): Activity? {
        if (ref != null && ref.get() != null) {
            return ref.get()
        } else {
            return null
        }
    }
}
