package com.wgmf.trade.presenter

import com.urovo.wgmf.entity.response.ExchangeReasonRes
import com.urovo.wugumofang.net.RetrofitHelper
import com.urovo.wugumofang.response.BaseSimpleResponse
import com.urovo.wugumofang.system.SystemHelper
import com.wgmf.trade.model.data.net.ExchangeReasonNet
import com.wgmf.trade.view.ExchangeReasonCon
import retrofit2.Call
import retrofit2.Response
import java.text.SimpleDateFormat
import java.util.*

/**
 * Created by Sasuke on 2017/7/4.
 */
class ExchangeReasonPre(mView: ExchangeReasonCon.View) : ExchangeReasonCon.Pre {
    var mView: ExchangeReasonCon.View
    var net: ExchangeReasonNet

    init {
        this.mView = mView
        this.mView.setPresenter(this)
        net = RetrofitHelper.createApi(ExchangeReasonNet::class.java)
    }

    /**
     * 联网校验该日期是否可撤销兑换
     */
    fun queryIsChecked(date: String?) {
        if (date != null && date != "") {
            net.checkIsTransfered(SystemHelper.LoginOrganId, date).enqueue(
                    object : retrofit2.Callback<BaseSimpleResponse> {
                        override fun onFailure(call: Call<BaseSimpleResponse>?, t: Throwable?) {
                            mView?.showMsg("网络异常，校验失败")
                            mView?.showDate(SimpleDateFormat("yyyy-MM-dd").format(Date()))
                        }

                        override fun onResponse(call: Call<BaseSimpleResponse>?, res: Response<BaseSimpleResponse>?) {
                            if (res == null || res.body()?.resultInfo != "0") {//0表示未盘点  1表示已盘点
                                mView?.showAlert("${date.substring(0, 7)} 月的盘点单已审核，请联系销管部订货对接人或撤销到当前月份！")
                                mView?.showDate(SimpleDateFormat("yyyy-MM-dd").format(Date()))//恢复到今天
                            } else {//校验成功
                                mView?.showDate(date)
                            }
                        }
                    }
            )

        }
    }

    /**
     * 提交撤销兑换
     */
    fun pointExchangeCancel(exchid: String, date: String, billNo: String, opno: String, remark: String) {
        mView?.showProgress("正在撤销兑换...")
        net.pointExchangeCancel(exchid, date, billNo, opno, remark).enqueue(
                object : retrofit2.Callback<ExchangeReasonRes> {
                    override fun onFailure(call: Call<ExchangeReasonRes>?, t: Throwable?) {
                        mView?.dismissProgress()
                        mView?.showMsg("网络异常，兑换失败")
                    }

                    override fun onResponse(call: Call<ExchangeReasonRes>?, res: Response<ExchangeReasonRes>?) {
                        mView?.dismissProgress()
                        if (res != null && res.isSuccessful) {
                            res.body()?.let { mView?.showMsg(it.retInfo) }
                            if (res.body()?.ret == "0") {
                                mView?.successAndFinish()//成功并退出
                            }
                        }
                    }
                }
        )
    }


    override fun start() {
        mView.init()
    }
}