package com.wgmf.trade.presenter;

import android.widget.Toast;

import com.urovo.activity.goodsmanager.model.OrderCommonEntity;
import com.urovo.wgmf.db.dao.entitys.DingHuoOplQtyEntity;
import com.urovo.activity.goodsmanager.model.OrderDeliveryDifferenceQueryEntity;
import com.urovo.activity.goodsmanager.model.OrderGoodsQueryEntity;
import com.urovo.wugumofang.base.BaseActivity;
import com.urovo.wugumofang.data.JsonDominator;
import com.urovo.wugumofang.net.RetrofitHelper;
import com.urovo.wugumofang.system.SystemHelper;
import com.urovo.wugumofang.ui.ViewUtil;
import com.urovo.wugumofang.utils.MyToast;
import com.wgmf.trade.model.bean.OrderSubmitReqBean;
import com.wgmf.trade.model.data.net.OrderGoodDataNet;
import com.wgmf.trade.view.OrderGoodCon;
import com.zsy.libcore.LibCoreApp;
import com.zsy.libcore.model.bean.RequestResult;
import com.zsy.libcore.model.bean.ResultCallback;

import org.jetbrains.annotations.NotNull;

import java.net.SocketTimeoutException;
import java.util.List;

import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Created by samy on 16/12/22.
 */
public class OrderGoodPre implements OrderGoodCon.Pre {
    OrderGoodDataNet orderGoodDataNet;
    OrderGoodCon.View view;

    public OrderGoodPre(OrderGoodCon.View view) {
        this.orderGoodDataNet = RetrofitHelper.createApi(OrderGoodDataNet.class);
        this.view = view;
    }

    @Override
    public void getOrderGoodsDiff(String storeNo, String beginDate, String endDate) {
        ViewUtil.showProgress((BaseActivity) view, "请稍等", "正在查询订货与发货差异数据...");
        Call<RequestResult<List<OrderDeliveryDifferenceQueryEntity>>> call =
                orderGoodDataNet.getOrderGoodsDiff(storeNo, beginDate, endDate);
        call.enqueue(new Callback<RequestResult<List<OrderDeliveryDifferenceQueryEntity>>>() {
            @Override
            public void onResponse(@NotNull Call<RequestResult<List<OrderDeliveryDifferenceQueryEntity>>> call,
                                   @NotNull Response<RequestResult<List<OrderDeliveryDifferenceQueryEntity>>> response) {
                ViewUtil.dismissProgress();
                if (response.isSuccessful()) {
                    RequestResult<List<OrderDeliveryDifferenceQueryEntity>> res = response.body();
                    if (res != null) {
                        if (res.msgCode == 0) {
                            if (view != null) {
                                view.showOrderGoodsDiff(res.resultInfo);
                            }
                        } else {
                            if (view != null) {
                                view.showOrderGoodsDetail(null);
                            }
                            MyToast.show(res.msgInfo);
                        }
                    }
                } else {
                    ResponseBody body = response.errorBody();
                    if (body != null) {
                        this.onFailure(call, new Throwable(body.toString()));
                        view.showError(body.toString());
                    }
                }
            }

            @Override
            public void onFailure(@NotNull Call<RequestResult<List<OrderDeliveryDifferenceQueryEntity>>> call, @NotNull Throwable t) {
                ViewUtil.dismissProgress();
                if (view != null) {
                    MyToast.show(t.getMessage());
                }
            }
        });
    }


    @Override
    public void getOrderGoodsDetail(String storeNo, String beginDate, String endDate) {
        /*
        DingHuoQueryNewRes entity = JsonDominator.loadFromSD("DingHuoQueryNewRes", DingHuoQueryNewRes.class);
        if (entity != null && view != null) {
            view.showOrderGoodsDetail(entity);
            return;
        }
        */
        ViewUtil.showProgress((BaseActivity) view, "请稍等", "正在查询订货数据...");
        Call<RequestResult<List<OrderGoodsQueryEntity>>> call =
                orderGoodDataNet.getOrderGoodsDetail(storeNo, beginDate, endDate);
        call.enqueue(new Callback<RequestResult<List<OrderGoodsQueryEntity>>>() {
            @Override
            public void onResponse(@NotNull Call<RequestResult<List<OrderGoodsQueryEntity>>> call,
                                   @NotNull Response<RequestResult<List<OrderGoodsQueryEntity>>> response) {
                ViewUtil.dismissProgress();
                if (response.isSuccessful()) {
                    RequestResult<List<OrderGoodsQueryEntity>> res = response.body();
                    if (res != null) {
                        if (res.msgCode == 0) {
                            if (view != null) {
                                view.showOrderGoodsDetail(res.resultInfo);
                            }
                        } else {
                            if (view != null) {
                                view.showOrderGoodsDetail(null);
                            }
                            MyToast.show(res.msgInfo);
                        }
                    }
                } else {
                    ResponseBody body = response.errorBody();
                    if (body != null) {
                        this.onFailure(call, new Throwable(body.toString()));
                        view.showError(body.toString());
                    }
                }
            }

            @Override
            public void onFailure(@NotNull Call<RequestResult<List<OrderGoodsQueryEntity>>> call, @NotNull Throwable t) {
                ViewUtil.dismissProgress();
                if (view != null) {
                    MyToast.show(t.getMessage());
                }
            }
        });
    }

    @Override
    public void getOrderCommonList(String storeId) {
        Call<RequestResult<List<OrderCommonEntity>>> call =
                orderGoodDataNet.getOrderCommonList(storeId);
        call.enqueue(new Callback<RequestResult<List<OrderCommonEntity>>>() {
            @Override
            public void onResponse(@NotNull Call<RequestResult<List<OrderCommonEntity>>> call,
                                   @NotNull Response<RequestResult<List<OrderCommonEntity>>> response) {
                if (response.isSuccessful()) {
                    RequestResult<List<OrderCommonEntity>> res = response.body();
                    if (res != null) {
                        if (res.msgCode == 0) {
                            if (view != null) {
                                view.showOrderCommon(res.resultInfo);
                            }
                        } else {
                            if (view != null) {
                                view.showOrderCommon(null);
                            }
                            MyToast.show(res.msgInfo);
                        }
                    }
                } else {
                    ResponseBody body = response.errorBody();
                    if (body != null) {
                        this.onFailure(call, new Throwable(body.toString()));
                    }
                }
            }

            @Override
            public void onFailure(@NotNull Call<RequestResult<List<OrderCommonEntity>>> call, @NotNull Throwable t) {
                if (view != null) {
                    MyToast.show(t.getMessage());
                }
            }
        });
    }


    @Override
    public void isCanOrdering(final String storeId, final String userId, final boolean isSubmitOrder) {
        Call<RequestResult<String>> call = orderGoodDataNet.isCanOrdering(storeId, userId);
        call.enqueue(new Callback<RequestResult<String>>() {
            @Override
            public void onResponse(@NotNull Call<RequestResult<String>> call, @NotNull Response<RequestResult<String>> response) {
                if (response.isSuccessful()) {
                    view.showDismissAll();
                    RequestResult<String> res = response.body();

                    RequestResult resMock = JsonDominator.loadFromSD("isCanOrdering", RequestResult.class);
                    if (resMock != null) {
                        res = resMock;
                    }

                    int msgCode = res.msgCode;
                    if (msgCode >= 0) {//进入下一步操作
                        if (isSubmitOrder) {//提交订单情况下；
                            if (view != null) {
                                view.showCanOrdering(msgCode, res.msgInfo, isSubmitOrder);
                            }
                        } else {
                            if (msgCode == 0) {
                                getOrderList(storeId, userId, SystemHelper.LoginUserBrandList);
                            } else {
                                if (view != null) {
                                    view.showCanOrdering(msgCode, res.msgInfo, isSubmitOrder);
                                }
                            }
                        }
                    } else {//弹框提示；
                        if (view != null) {
                            view.showCanOrdering(msgCode, res.msgInfo, isSubmitOrder);
                        }
                    }
                } else {
                    this.onFailure(call, new Throwable(response.errorBody().toString()));
                    view.showError(response.errorBody().toString());
                }
            }

            @Override
            public void onFailure(Call<RequestResult<String>> call, Throwable t) {
                String msgTips = "网络访问异常";
                if (t instanceof SocketTimeoutException) {
                    msgTips = "网络请求超时";
                }
                Toast.makeText(LibCoreApp.getInstance(), msgTips, Toast.LENGTH_SHORT).show();
                view.showError(msgTips);
            }
        });
    }

    @Override
    public void getOrderList(String storeId, String staffId, String brands) {
        view.showIsLoading();
        DingHuoOplQtyEntity mock = JsonDominator.loadFromSD(DingHuoOplQtyEntity.class);
        if (mock != null && null != view) {
            view.showOrderList(mock.resultInfo);
            return;
        }

        Call<RequestResult<DingHuoOplQtyEntity.ResultInfo>> call = orderGoodDataNet.getOrderList(storeId, staffId, brands);
        call.enqueue(new ResultCallback<DingHuoOplQtyEntity.ResultInfo>() {
            @Override
            protected void onResponseSuccess(RequestResult<DingHuoOplQtyEntity.ResultInfo> responseBody) {
                super.onResponseSuccess(responseBody);
                view.showDismissAll();
                if (null != view) {
                    DingHuoOplQtyEntity.ResultInfo resultInfo = responseBody.resultInfo;
                    view.showOrderList(resultInfo);
                }
            }

            @Override
            public void onFailure(Call<RequestResult<DingHuoOplQtyEntity.ResultInfo>> call, Throwable t) {
                view.showDismissAll();
                super.onFailure(call, t);
            }
        });
    }

    @Override
    public void orderSubmit(OrderSubmitReqBean orderSubmitReqBean) {
        Call<RequestResult<String>> call = orderGoodDataNet.orderSubmit(orderSubmitReqBean);
        call.enqueue(new ResultCallback<String>() {
//      @Override protected void onResponseSuccess(RequestResult<String> responseBody) {
//        super.onResponseSuccess(responseBody);
//        Toast.makeText(LibCoreApp.getInstance(), responseBody.msgInfo, Toast.LENGTH_SHORT).show();
//        if (null!=view){
//          view.showOrderSubmit();
//        }
//      }

            @Override
            public void onResponse(Call<RequestResult<String>> call, Response<RequestResult<String>> response) {
                super.onResponse(call, response);
                if (response.isSuccessful()) {
                    ViewUtil.dismissProgress();
                    RequestResult<String> res = response.body();
                    if (res != null) {
                        int msgCode = res.msgCode;
                        if (msgCode == 0) {
                            if (view != null) {
                                view.submitSuccess(res.msgInfo);
                            }
                        } else {//弹框提示；
                            if (view != null) {
                                view.showCanOrdering(msgCode, res.msgInfo, true);
                            }
                        }
                    }
                }
            }

            @Override
            public void onFailure(Call<RequestResult<String>> call, Throwable t) {
                super.onFailure(call, t);
                ViewUtil.dismissProgress();
                if (null != view) {
                    view.submitFailed("网络异常，提交失败");
                }
//        if (null!=view){
//          view.showOrderSubmit();
//        }
            }
        });
    }
}
