package com.wgmf.trade.presenter;

import android.os.Handler;
import android.os.Looper;

import com.google.gson.Gson;
import com.urovo.activity.sell.SaveFailureActivity;
import com.urovo.wgmf.db.dao.dao.Djxs001DbOperate;
import com.urovo.wgmf.db.dao.entitys.OrderReqBean;
import com.urovo.wugumofang.net.RetrofitHelper;
import com.urovo.wugumofang.response.BaseSimpleResponse;
import com.urovo.wugumofang.system.BaseAppManager;
import com.wgmf.trade.GlobalApp;
import com.wgmf.trade.model.bean.FirstShopGiftBean;
import com.wgmf.trade.model.bean.OrderResBean;
import com.wgmf.trade.model.data.net.SaveSuccessDataNet;
import com.wgmf.trade.ui.fra.index.MemberSalesFra;
import com.wgmf.trade.view.SaveSuccessCon;
import com.zsy.libcore.common.android.log.SYLoger;
import com.zsy.libcore.model.bean.RequestResult;
import com.zsy.libcore.model.bean.ResultBaseCallback;
import com.zsy.libext.widgets.tips.ToastMsg;

import org.jetbrains.anko.ToastsKt;

import java.io.IOException;
import java.net.SocketTimeoutException;
import java.util.Map;

import retrofit2.Call;
import retrofit2.Response;

/**
 * @author samy
 * @date 16/11/4
 */

public class SaveSuccessPre implements SaveSuccessCon.Pre {

    private SaveSuccessDataNet saveSuccessDataNet;

    private SaveSuccessCon.View saveSuccessConView;

    public SaveSuccessPre(SaveSuccessCon.View saveSuccessConView) {
        this.saveSuccessConView = saveSuccessConView;
        saveSuccessDataNet = RetrofitHelper.createApi(SaveSuccessDataNet.class);
    }

    /**
     * 一键上传情况；
     */
    public SaveSuccessPre() {
        saveSuccessDataNet = RetrofitHelper.createApi(SaveSuccessDataNet.class);
    }

    /**
     * 获取适合Ta的会员商品推荐
     *
     * @param mobile ...
     */
    public void queryRecomProduct(String mobile) {
        Call<BaseSimpleResponse> call = saveSuccessDataNet.queryRecomProduct(mobile);
        call.enqueue(new ResultBaseCallback<BaseSimpleResponse>() {
            @Override
            public void onResponse(Call<BaseSimpleResponse> call, Response<BaseSimpleResponse> response) {
                super.onResponse(call, response);
                BaseSimpleResponse responseBody = response.body();
                if (responseBody != null && response.isSuccessful()) {
                    if (response.body() != null && saveSuccessConView != null) {
                        saveSuccessConView.showRecomProduct(responseBody.resultInfo);
                    }
                } else {
                    this.onFailure(call, new Throwable(requireNonNull(response.errorBody()).toString()));
                }
            }

            @Override
            public void onFailure(Call<BaseSimpleResponse> call, Throwable t) {
                super.onFailure(call, t);

            }
        });

    }

    @Override
    public void sendTicket(Map<String, String> params) {
        Call<RequestResult> call = saveSuccessDataNet.sendTicket(params);
        call.enqueue(new ResultBaseCallback<RequestResult>() {
            @Override
            public void onResponse(Call<RequestResult> call, Response<RequestResult> response) {
                if (response.isSuccessful()) {
                    final RequestResult responseBody = response.body();
                    if (responseBody != null && responseBody.msgCode == 0) {
                        SYLoger.print("SaveSuccessPre=1=> " + responseBody.msgInfo);
                        ToastsKt.toast(BaseAppManager.appContext(), responseBody.msgInfo);
                    } else {
                        SYLoger.print("SaveSuccessPre=0=> 服务器异常:优惠券获取失败");
                        ToastsKt.toast(BaseAppManager.appContext(), "服务器异常:优惠券获取失败！");
                    }
                } else {
                    SYLoger.print("SaveSuccessPre=1=> " + response.errorBody());
                    this.onFailure(call, new Throwable(requireNonNull(response.errorBody()).toString()));
                }
            }

            @Override
            public void onFailure(Call<RequestResult> call, Throwable t) {
                String msgTips = "网络访问异常";
                if (t instanceof SocketTimeoutException) {
                    msgTips = "网络请求超时";
                }
                ToastsKt.toast(BaseAppManager.appContext(), msgTips);
            }
        });
    }


    @Override
    public void queryFirstShopGift(Map<String, String> params) {
        Call<RequestResult<FirstShopGiftBean>> call = saveSuccessDataNet.queryFirstShopGift(params);
        call.enqueue(new ResultBaseCallback<RequestResult<FirstShopGiftBean>>() {
            @Override
            public void onResponse(Call<RequestResult<FirstShopGiftBean>> call, Response<RequestResult<FirstShopGiftBean>> response) {
                if (response.isSuccessful()) {
                    final RequestResult<FirstShopGiftBean> responseBody = response.body();
                    if (responseBody != null && responseBody.msgCode == 0) {
                        saveSuccessConView.firstShopGift(responseBody.resultInfo);
                    }
                } else {
                    this.onFailure(call, new Throwable(requireNonNull(response.errorBody()).toString()));
                }
            }

            @Override
            public void onFailure(Call<RequestResult<FirstShopGiftBean>> call, Throwable t) {
                String msgTips = "网络访问异常";
                if (t instanceof SocketTimeoutException) {
                    msgTips = "网络请求超时";
                }
                ToastsKt.toast(BaseAppManager.appContext(), msgTips);
            }
        });
    }

    /**
     * UploadXiaoShouTask对应上；
     *
     * @param content 下单的json信息
     */
    @Override
    public void submitOrder(String content) {
    }

    /**
     * 用于一键上传异常单时的 非异步上传销售
     *
     * @param orderRequestBean ...
     */
    public String submitOrderSyn(final OrderReqBean orderRequestBean) {
        Call<OrderResBean> call = saveSuccessDataNet.submitOrder(orderRequestBean);
        Response<OrderResBean> response;
        try {
            response = call.execute();
            if (response.isSuccessful()) {
                return dealWithResponse(orderRequestBean, response, true);
            } else {
                new Handler(Looper.getMainLooper()).post(new Runnable() {
                    @Override
                    public void run() {
                        ToastMsg.showToastMsg(GlobalApp.getInstance(), "网络数据异常，请检查网络重试!");
                    }
                });
                new Gson().toJson(response);
            }

        } catch (IOException e) {
            e.printStackTrace();
        }
        //随便写的返回值，应该是走不到这里来的，如果真走这里来然后爆炸了，记得改这里。
        return "-1";
    }

    /**
     * 打印页面保存按钮点击后下单上传到后台 跳转到保存成功页面。
     *
     * @param orderRequestBean ...
     */
    @Override
    public void submitOrder(final OrderReqBean orderRequestBean) {
        if (saveSuccessConView != null) {
            saveSuccessConView.onShowProgress();
        }
        Call<OrderResBean> call = saveSuccessDataNet.submitOrder(orderRequestBean);
        call.enqueue(new ResultBaseCallback<OrderResBean>() {
            @Override
            public void onResponse(Call<OrderResBean> call, Response<OrderResBean> response) {
                super.onResponse(call, response);
                if (response.isSuccessful()) {
                    final OrderResBean responseBody = response.body();
                    if (responseBody != null) {
                        //如果不在首页，就显示后台返回的文字；如果是在首页，就显示写死的“上传销售单成功”
//                        String tempInfo = saveSuccessConView != null && (saveSuccessConView.getClass() != TabMenuActivity.class)
//                                ? responseBody.retInfo : "上传销售单成功";
                        String tempInfo = responseBody.retInfo;
                        //正常请求成功时的显示
                        dealWithResponse(orderRequestBean, response, false);
                        if ("0".equals(responseBody.ret)) {
                            if (saveSuccessConView != null) {
                                saveSuccessConView.onResponseSuccess();
                            }
                        } else {
                            //服务器异常的情况的显示
                            if (saveSuccessConView != null) {
                                saveSuccessConView.onFailure();
                            }
                        }
                        //toast 必须放到逻辑处理完后弹出，否则有可能没有执行到就跳过了。
                        ToastsKt.toast(BaseAppManager.appContext(), tempInfo);
                        SYLoger.print("tempInfo--> " + tempInfo);
                    }
                } else {
                    this.onFailure(call, new Throwable(requireNonNull(response.errorBody()).toString()));
                }
            }

            @Override
            public void onFailure(Call<OrderResBean> call, Throwable t) {
                super.onFailure(call, t);
                if (saveSuccessConView != null) {
                    saveSuccessConView.onFailure();
                }
                ToastsKt.toast(BaseAppManager.appContext(), "网络数据异常，请检查网络重试!");
            }
        });
    }

    private <T> T requireNonNull(T o) {
        if (o == null) {
            throw new NullPointerException();
        }
        return o;
    }

    /**
     * 把上面同步和异步两个半斤八两的处理提取共同部分
     * （isFromUntreatedOrder如果是从一键上传过来的，那就始终让tempInfo从response获取，因为要把异常信息写入本地，写个“上传成功”算哪回事？）
     *
     * @param response ...
     */
    private String dealWithResponse(OrderReqBean orderRequestBean, Response<OrderResBean> response, boolean isFromUntreatedOrder) {
        SYLoger.print("条码打印 保存页面： " + orderRequestBean.getSKTNET());
        OrderResBean orderResBody = response.body();
        SYLoger.print("wgmf---OrderResBean: " + orderResBody);
        if (orderResBody == null) {
            return "";
        }
        String tempInfo = ((saveSuccessConView != null) || isFromUntreatedOrder) ? orderResBody.retInfo : "上传销售单成功";
        String billId = orderRequestBean.getBill();
        if ("0".equals(orderResBody.ret)) {
            if (saveSuccessConView != null) {
                saveSuccessConView.commitOrderState(orderResBody.resultStr);
            }
            Djxs001DbOperate.UpdateBillState(billId, 1, "服务器反馈：" + tempInfo);
            return "";
            //ret不为0时
        } else if ("1036".equals(orderResBody.ret)) {
            Djxs001DbOperate.UpdateBillState(billId, 1, "服务器反馈：" + tempInfo);
            return new Gson().toJson(orderResBody);
            //下面的都是异常情况了
            //标记为坏单（正常下单时，如果上传失败，是不标记为坏单的（坏单即为2）；点击“重新上传”，再失败，就要标记坏单。一键上传那里同样要标记坏单。
        } else if (saveSuccessConView == null ||
                (saveSuccessConView.getClass() != MemberSalesFra.class && saveSuccessConView.getClass() != SaveFailureActivity.class)) {//不是来自常规下单的请求，那才记录为坏单。否则仍然保持0的标记。
            Djxs001DbOperate.UpdateBillState(billId, 2, "服务器反馈：" + tempInfo);
            return new Gson().toJson(orderResBody);
        }
        return new Gson().toJson(orderResBody);
    }
}
