package com.wgmf.trade.ui.ada;

import android.app.AlertDialog;
import android.content.Context;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.urovo.wgmf.db.dao.dao.Item001DbOperate;
import com.urovo.wgmf.db.dao.entitys.ProductEntity;
import com.urovo.wugumofang.R;
import com.urovo.wugumofang.constants.ConstantParams;
import com.urovo.wugumofang.data.Arith;
import com.urovo.wugumofang.data.MoneyUtils;
import com.urovo.wugumofang.data.PreferHelper;
import com.urovo.wugumofang.ui.EditTextDecimal;
import com.wgmf.trade.model.bean.ProGroupInfoBean;
import com.wgmf.trade.model.bean.ProductInfoBean;
import com.wgmf.trade.utils.VerifyUtil;
import com.zsy.libcore.common.android.log.SYLoger;

import java.util.ArrayList;
import java.util.regex.Pattern;

import androidx.core.content.ContextCompat;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * 扩展Adapter
 *
 * @author samy
 * @date 16/9/23
 */
public class ProSelectedAda extends BaseExpandableListAdapter {
    private final Context mContext;
    private final ArrayList<ProGroupInfoBean> mProGroupInfoBeans;
    private final boolean mIsGift;//是否是”添加赠品“点击进来的。下面的添加列表可能会根据这个字段的不同而不同。
    private final int mActivityType;
    private final String mProductTitle;

    public ProSelectedAda(Context context, ArrayList<ProGroupInfoBean> proGroupInfoBeans, boolean isGift, int activityType, String productTitle) {
        this.mContext = context;
        this.mProGroupInfoBeans = proGroupInfoBeans;
        this.mIsGift = isGift;
        this.mActivityType = activityType;
        this.mProductTitle = productTitle;
        String notFoundPros = PreferHelper.getInstance().getString("notFoundPros");
        if (!isGift && !TextUtils.isEmpty(notFoundPros)) {//不是从选择赠品进入，并且组合产品中有产品不存在
            new AlertDialog.Builder(context)
                    .setTitle("以下产品不存在！")
                    .setMessage(notFoundPros + "\n先下载基础资料，还是报错的话请联系主管/文员找销管部对应的货品专员维护产品目录。")
                    .setPositiveButton("我知道了", null)
                    .create().show();
        }
    }


    @Override
    public ProGroupInfoBean getGroup(int groupPosition) {
        return mProGroupInfoBeans.get(groupPosition);
    }

    @Override
    public ProductInfoBean getChild(int groupPosition, int childPosition) {
        return mProGroupInfoBeans.get(groupPosition).product.get(childPosition);
    }

    @Override
    public int getGroupCount() {
        if (null == mProGroupInfoBeans || mProGroupInfoBeans.size() == 0) {
            return 0;
        }
        return mProGroupInfoBeans.size();
    }

    @Override
    public int getChildrenCount(int groupPosition) {
        if (mProGroupInfoBeans.get(groupPosition).product == null || mProGroupInfoBeans.get(groupPosition).product.isEmpty()) {
            return 0;
        }
        return mProGroupInfoBeans.get(groupPosition).product.size();
    }


    @Override
    public long getGroupId(int groupPosition) {
        return groupPosition;
    }

    @Override
    public long getChildId(int groupPosition, int childPosition) {
        return childPosition;
    }

    // 是否指定分组视图及其子视图的ID对应的后台数据改变也会保持该ID。
    @Override
    public boolean hasStableIds() {
        return true;
    }

    // 指定位置的子视图是否可选择
    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return true;
    }

    @Override
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView, ViewGroup parent) {
        convertView = LayoutInflater.from(mContext).inflate(R.layout.item_pro_selected_title, parent, false);
        TextView titleTv = convertView.findViewById(R.id.gift_name_tv);
        titleTv.setVisibility(View.VISIBLE);

        if ((ConstantParams.CONSTANT_STARTS_14 == mActivityType || ConstantParams.CONSTANT_STARTS_26 == mActivityType) && !mIsGift) {
            titleTv.setText(mProductTitle);
        } else {
            ProductInfoBean proBean = getGroup(groupPosition).product.get(0);
            //注意这里的isGift，添加赠品时所有的限制都是数量；添加产品【即 非赠品时】才根据isDecimal才区分是金额还是数量
            String titleText = "以下需满足总" + ((proBean.isDecimal() && !mIsGift) ? "金额：" : "数量：");
            String titleText2 = (proBean.isDecimal() && !mIsGift) ? getGroup(groupPosition).money : getGroup(groupPosition).num;
            titleText2 = titleText2.replace("+", "或以上");
            titleText2 = titleText2.replace(",", "或");
            if (proBean.isDecimal() && !mIsGift) titleText2 = titleText2 + "或以上";
            titleTv.setText(titleText + titleText2);
        }
        return convertView;
    }

    @Override
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild,
                             View convertView, ViewGroup parent) {
        ProductInfoBean productInfoBeanOut = getChild(groupPosition, childPosition);

        if (productInfoBeanOut.isDecimal() && mIsGift) {//允许有小数位的，并且是添加赠品的情况。那就是数量可编辑。
            convertView = LayoutInflater.from(mContext).inflate(R.layout.item_pro_selected_kg_decimal, null);
            ViewHolderKgDecimal holder = new ViewHolderKgDecimal(convertView);
            setItemDataKgDecimal(holder, productInfoBeanOut);
        } else if (productInfoBeanOut.isDecimal() && !mIsGift) {//允许有小数位，但不是添加赠品的情况，那就是金额可编辑。
            convertView = LayoutInflater.from(mContext).inflate(R.layout.item_pro_selected_amount_decimal, null);
            ViewHolderAmountDecimal holder = new ViewHolderAmountDecimal(convertView);
            setItemDataAmountDecimal(holder, productInfoBeanOut);
        } else {//数量是整型的(基础资料中没有该产品时，拿不到isDecimal值，也会默认走这个分支)。
            convertView = LayoutInflater.from(mContext).inflate(R.layout.item_pro_selected_not_decimal, parent, false);
            ViewHolder holder = new ViewHolder(convertView);
            //因数据问题,不做复用处理;
            setItemData(holder, productInfoBeanOut);
            checkProduct(holder, productInfoBeanOut);
        }
        return convertView;
    }

    /*
     * 判断是否为浮点数，包括double和float
     * @param str 传入的字符串
     * @return 是浮点数返回true,否则返回false
     */
    public boolean isDouble(String str) {
        Pattern pattern = Pattern.compile("^[-\\+]?[.\\d]*$");
        return pattern.matcher(str).matches();
    }

    private void setItemDataAmountDecimal(final ViewHolderAmountDecimal holder, final ProductInfoBean productInfoBean) {

        holder.proNameXdyTv.setText(productInfoBean.name);
        holder.tvPricePerKgTv.setText(String.format("%s元/" + productInfoBean.getKCUom(), MoneyUtils.twolastDF(productInfoBean.getPrice())));
        //这个地方kg数需要重新计算，要保证Amount是用户输入的精确值，而不是计算得到的，计算后会产生误差
        holder.proAmountEdit.setText(String.format("%s", MoneyUtils.twolastDF(productInfoBean.getAmount())));
        holder.proAmountEdit.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                //刷新item显示
                Double price = productInfoBean.getPrice();
                Double kg = 0.00;
                Double amount = 0.0;
                if (!TextUtils.isEmpty(s.toString()) && !s.toString().equals(".")) {
                    //获得总价
                    amount = Double.parseDouble(s.toString());
                } else {
                    amount = 0.00;
                }
                //通过以上的限制，把amount回复到规定范围，变化完成，接下来改变金额
                //保留啥小数保留，不保留不保留，就显示的时候保留一下，就把一大串的kg数丢给后台
                kg = Arith.div(amount, price);
                productInfoBean.setProNum(kg);
                productInfoBean.Amount = amount;
            }
        });
    }

    private void setItemDataKgDecimal(final ViewHolderKgDecimal holder, final ProductInfoBean productInfoBean) {
        SYLoger.print("条码打印 活动 选择产品：productInfoBean.num: " + productInfoBean.num);
        final double[] ruleArr = productInfoBean.getProNumMinAndMax(productInfoBean.num);

        holder.proNameXdyTv.setText(productInfoBean.name);
        holder.tvPricePerKgTv.setText(String.format("%s元/" + productInfoBean.getKCUom(), MoneyUtils.twolastDF(productInfoBean.getPrice())));
        //这个地方kg数需要重新计算，要保证Amount是用户输入的精确值，而不是计算得到的，计算后会产生误差
        holder.kgTv.setText(String.format("%s", MoneyUtils.fourlastDF(Arith.div(productInfoBean.getAmount(), productInfoBean.getPrice()))));
        holder.kgTv.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                //刷新item显示
                Double price = productInfoBean.getPrice();
                Double kg = 0.0000;
                Double amount = 0.0;
                if (!TextUtils.isEmpty(s.toString()) && !s.toString().equals(".")) {
                    //获得重量
                    kg = Double.parseDouble(s.toString());
                } else {
                    kg = 0.00;
                }
                //通过以上的限制，把amount回复到规定范围，变化完成，接下来改变金额
                //保留啥小数保留，不保留不保留，就显示的时候保留一下，就把一大串的kg数丢给后台
                amount = Arith.mul(kg, price);
                productInfoBean.setProNum(kg);
                productInfoBean.Amount = amount;
            }
        });
    }

    private void setItemData(final ViewHolder holder, final ProductInfoBean productInfoBean) {
        SYLoger.print("条码打印 活动 选择产品：productInfoBean.num: " + productInfoBean.num);
        final double[] ruleArr = productInfoBean.getProNumMinAndMax(productInfoBean.num);
        holder.proNameTv.setText(productInfoBean.name);
        holder.proPriceTv.setText(String.format("%s元/" + productInfoBean.getKCUom(), MoneyUtils.twolastDF(productInfoBean.getPrice())));
        //初始化显示数据 因为跟鲜稻研重量共用字段，需要强转int
        holder.proOptEt.setText(String.format("%s", (int) productInfoBean.getProNum()));
        holder.proOptEt.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                itemOperationShow(s.toString(), holder, productInfoBean, ruleArr);

            }
        });
        holder.proOptSubIb.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                double tempEditBuyNum = Double.parseDouble(holder.proOptEt.getText().toString());
                tempEditBuyNum--;
                holder.proOptEt.setText(MoneyUtils.nolastDF(tempEditBuyNum));
            }
        });
        holder.proOptAddIb.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                double tempEditBuyNum = Double.parseDouble(holder.proOptEt.getText().toString());
                tempEditBuyNum++;
                holder.proOptEt.setText(MoneyUtils.nolastDF(tempEditBuyNum));
            }
        });
        checkInputOperation(holder, productInfoBean, ruleArr);
    }

    private void itemOperationShow(String temp, ProSelectedAda.ViewHolder holder, ProductInfoBean productInfoBean, double[] ruleArr) {
        double minCount = ruleArr[0];
        double maxCount = ruleArr[1];
        SYLoger.print("条码打印 活动 选择产品：maxCount: " + maxCount);
        double tempEditBuyNum = 0;
        holder.proOptEt.setEnabled(true);
        if (temp.equals(".")) {
            temp = "0.0";
        }

        if (!TextUtils.isEmpty(temp)) {
            tempEditBuyNum = Double.parseDouble(temp);
            if (tempEditBuyNum < minCount) {
                tempEditBuyNum = minCount;
                holder.proOptEt.setText(MoneyUtils.nolastDF(tempEditBuyNum));
            } else if (tempEditBuyNum == minCount) {
                holder.proOptSubIb.setEnabled(false);
                holder.proOptAddIb.setEnabled(true);
            } else if (tempEditBuyNum > maxCount) {
                tempEditBuyNum = maxCount;
                holder.proOptEt.setText(String.format("%s", tempEditBuyNum));
            } else if (tempEditBuyNum == maxCount) {
                holder.proOptSubIb.setEnabled(true);
                holder.proOptAddIb.setEnabled(false);
            } else {
                holder.proOptSubIb.setEnabled(true);
                holder.proOptAddIb.setEnabled(true);
            }
        } else {
            //表示有，和数字情况下；再去校验；
            if (ruleArr[2] == 1) {
                //不符合规则直接最大值；
                if (!VerifyUtil.compareNum(productInfoBean.num, tempEditBuyNum)) {
                    SYLoger.print("条码打印 活动 选择产品：maxCount: " + maxCount + ", tempEditBuyNum:" + tempEditBuyNum);
                    tempEditBuyNum = maxCount;
                    holder.proOptEt.setText(String.format("%s", tempEditBuyNum));
                }
            }
            holder.proOptEt.setText(String.format("%s", minCount));
        }

        //更新内存数据;方便提交数据判断用; 这里千万不能用final修饰的处理；
        productInfoBean.setProNum(Double.parseDouble(holder.proOptEt.getText().toString()));
        //改变的同时，给amount赋值，否则在某些特定BUG情况，amount为空不太方便。
        productInfoBean.Amount = productInfoBean.getAmountActual();
    }

    @SuppressWarnings("all")
    private void checkInputOperation(ViewHolder holder, ProductInfoBean productInfoBean, double[] ruleArr) {
        //组合商品+规则
        if (ruleArr[2] == 1) {
            holder.proOptEt.setEnabled(true);
            holder.proOptSubIb.setEnabled(false);
            holder.proOptAddIb.setEnabled(false);
        }
    }

    /**
     * 检查组合成品产品是否存在
     *
     * @param holder
     * @param productInfoBean
     */
    private void checkProduct(ViewHolder holder, ProductInfoBean productInfoBean) {
        ProductEntity productEntityTemp = Item001DbOperate.GetProductByBarcode(productInfoBean.code);
        if (productEntityTemp == null) {
            holder.proOptLl.setVisibility(View.GONE);
            holder.proPriceTv.setTextColor(ContextCompat.getColor(mContext, R.color.red_text_color));
            holder.proPriceTv.setText("(" + productInfoBean.code + ")产品不存在");
        }
    }

    /**
     * 只能用加减号修改数量的
     */
    static class ViewHolder {
        @BindView(R.id.pro_name_tv)
        TextView proNameTv;
        @BindView(R.id.pro_opt_sub_ib)
        ImageButton proOptSubIb;
        @BindView(R.id.pro_opt_et)
        EditText proOptEt;
        @BindView(R.id.pro_opt_add_ib)
        ImageButton proOptAddIb;
        @BindView(R.id.pro_opt_ll)
        LinearLayout proOptLl;
        @BindView(R.id.pro_price_tv)
        TextView proPriceTv;

        ViewHolder(View view) {
            ButterKnife.bind(this, view);
        }
    }

    /**
     * 手动输入可小数的数量
     */
    static class ViewHolderKgDecimal {
        /**
         * 鲜稻研用的 item父布局
         */
        @BindView(R.id.pro_selected_xdy_ll)
        LinearLayout proSelectedXdyLl;
        /**
         * 鲜稻研用的 产品名称
         */
        @BindView(R.id.pro_name_xdy_tv)
        TextView proNameXdyTv;
        /**
         * 鲜稻研用的 单价
         */
        @BindView(R.id.tv_price_per_kg)
        TextView tvPricePerKgTv;
        /**
         * 鲜稻研用的 产品重量
         */
        @BindView(R.id.tv_kg)
        EditTextDecimal kgTv;

        ViewHolderKgDecimal(View view) {
            ButterKnife.bind(this, view);
        }
    }

    /**
     * 手动输入可小数的价格
     */
    static class ViewHolderAmountDecimal {
        /**
         * 鲜稻研用的 item父布局
         */
        @BindView(R.id.pro_selected_xdy_ll)
        LinearLayout proSelectedXdyLl;
        /**
         * 鲜稻研用的 产品名称
         */
        @BindView(R.id.pro_name_xdy_tv)
        TextView proNameXdyTv;
        /**
         * 鲜稻研用的 单价
         */
        @BindView(R.id.tv_price_per_kg)
        TextView tvPricePerKgTv;
        /**
         * 鲜稻研用的可输入的总价格
         */
        @BindView(R.id.pro_opt_xdy_et)
        EditTextDecimal proAmountEdit;

        ViewHolderAmountDecimal(View view) {
            ButterKnife.bind(this, view);
        }
    }
}