package com.wgmf.trade.utils;

import android.text.TextUtils;

import com.wgmf.trade.model.bean.ProductInfoBean;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * 校验验证类
 * <p>
 * 判断金额的那个compareMoney（）是后来新加的，
 * 跟判断数量的那个是且的关系，并且加入了空判断，即其中一个规则可以传入空
 * Created by samy on 16/9/26.
 */
public class VerifyUtil {

    /**
     * 判断传入产品数量是否符合规则（比较数量）
     * eg:
     * 2+  表示数量>=2
     * 2,3,4  表示数量为2,3,4个
     * 2-4  表示数量>=2 && <=4
     * 2  表示数量只能为2个
     *
     * @param proNum 产品数量
     * @param rule   规则数量
     * @return true符合     false不符合
     */
    public static boolean compareNum(String rule, double proNum) {
        if (TextUtils.isEmpty(rule)) {//现在可以传入空了，因为和校验金额一起判断，所以其中一个可以为空，为空就是没有限制
            return true;
        }
        if (rule.contains("+")) {
            return Double.parseDouble(rule.replace("+", "")) <= proNum;
        } else if (rule.contains("-")) {
            String[] array = rule.split("-");
            return proNum >= Double.parseDouble(array[0]) && proNum <= Double.parseDouble(array[1]);
        } else if (rule.contains(",")) {
            String[] array = rule.split(",");
            for (String s : array) {
                if (proNum == Double.parseDouble(s)) return true;
            }
        } else {
            return proNum == Double.parseDouble(rule);
        }
        return false;

    }

    /**
     * 从规则里面获取数量
     *
     * @return 获取最接近值
     */
    public static double getNearestNum(String rule, Double valueNum) {
        double proNum = 0;
        //字符串规则不为空是，修改；为空时，显示组装默认的数据；
        if (!TextUtils.isEmpty(rule)) {
            if (rule.contains("+")) {
                proNum = Double.parseDouble(rule.replace("+", ""));
            } else if (rule.contains("-")) {
                String[] array = rule.split("-");
                proNum = Double.parseDouble(array[0]);
            } else if (rule.contains(",")) {
                String[] array = rule.split(",");
                for (String s : array) {
                    if (valueNum < Double.parseDouble(s)) {
                        proNum = Double.parseDouble(s);
                        break;
                    }
                }

            } else {
                proNum = Double.parseDouble(rule);
            }
        }
        return proNum;
    }

    /**
     * 判断传入产品数量是否符合规则（比较数量）
     * 满足倍数关系
     *
     * @param proNum 产品数量
     * @param rule   规则数量
     * @return true符合     false不符合
     */
    public static boolean compareProportRelation(String rule, double proNum) {
        if (TextUtils.isEmpty(rule)) {//现在可以传入空了，因为和校验金额一起判断，所以其中一个可以为空，为空就是没有限制
            return true;
        }

        if (proNum <= 0.0) {
            return false;
        } else {
            return proNum % Double.parseDouble(rule) == 0;
        }
    }

    /**
     * 判断特殊活动是否符合或逻辑规则（比较数量）
     *
     * @param proNum 产品数量
     * @param rule   规则数量
     * @return true符合     false不符合
     */
    public static boolean compareLogicNum(String rule, double proNum) {
        if (TextUtils.isEmpty(rule)) {//现在可以传入空了，因为和校验金额一起判断，所以其中一个可以为空，为空就是没有限制
            return true;
        }
        if (rule.contains("+")) {
            if (Double.parseDouble(rule.replace("+", "")) <= proNum) return true;
        } else if (rule.contains("-")) {
            String[] array = rule.split("-");
            if (proNum >= Double.parseDouble(array[0]) && proNum <= Double.parseDouble(array[1]))
                return true;
        } else if (rule.contains(",")) {
            String[] array = rule.split(",");
            for (int i = 0; i < array.length; i++) {
                if (proNum == Double.parseDouble(array[i])) return true;
            }
        } else {
            if (proNum == Double.parseDouble(rule)) return true;
        }
        return false;
    }

    /**
     * 判断传入产品数量是否符合规则（比较金额）
     *
     * @param rule     规则金额（是否大于等于该金额）
     * @param proMoney 待验证的产品金额
     * @return true符合     false不符合
     */
    public static boolean compareMoney(String rule, Double proMoney) {
        if (TextUtils.isEmpty(rule)) {//注释同上
            return true;
        }
        if (proMoney >= Double.parseDouble(rule)) return true;

        return false;
    }

    /**
     * 更新重复的数据并且有序 为有相同元素的集合
     *
     * @param
     * @return
     */
    public static List updateDuplicateWithOrder(List list) {
        Set set = new HashSet();
        List newList = new ArrayList();
        for (Iterator iter = list.iterator(); iter.hasNext(); ) {
            Object element = iter.next();
            if (set.contains(element)) {//如果存在这个数
                newList.add(element);
            } else {//如果是相同元素,把之前的踢出,用第二个新的的元素;
                set.add(element);
            }
        }
        return newList;
    }

    /**
     * 移除重复的数据并且有序
     *
     * @return
     */
    public static ArrayList<ProductInfoBean> removeDuplicateWithOrder(ArrayList<ProductInfoBean> listTemp) {
        Set set = new HashSet();
        ArrayList<ProductInfoBean> newList = new ArrayList();
        Collections.reverse(listTemp);//如果是相同元素,把之前的踢出,用第二个新的的元素;
        for (Iterator iter = listTemp.iterator(); iter.hasNext(); ) {
            ProductInfoBean element = (ProductInfoBean) iter.next();
            if (set.add(element)) {//没有加过的;
                newList.add(element);
            }
        }
        Collections.reverse(newList);//然后再倒序回来;
        return newList;
    }

    private void removeNotRepeatList(ArrayList<ProductInfoBean> productInfoBeansTemp, ArrayList<ProductInfoBean> productInfoBeansResultTemp) {
        Iterator<ProductInfoBean> iterator = productInfoBeansResultTemp.iterator();
        while (iterator.hasNext()) {
            ProductInfoBean next = iterator.next();
            if (next.getProNum() == 0) {
                iterator.remove();
                continue;
            }
            if (productInfoBeansTemp.contains(next)) { //有相同的话更新
                int index = productInfoBeansTemp.indexOf(next);
                productInfoBeansTemp.set(index, next);
            } else {//没的话直接添加进入;
                productInfoBeansTemp.add(next);
            }
        }
    }
}
