package com.wgmf.trade.wx.act;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Rect;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import com.taobao.weex.IWXRenderListener;
import com.taobao.weex.WXSDKEngine;
import com.taobao.weex.WXSDKInstance;
import com.taobao.weex.common.WXRenderStrategy;
import com.taobao.weex.utils.WXFileUtils;
import com.taobao.weex.utils.WXSoInstallMgrSdk;
import com.urovo.wugumofang.BuildConfig;
import com.urovo.wugumofang.base.BaseActivity;
import com.wgmf.trade.utils.AssertUtil;
import com.wgmf.trade.wx.WXApp;
import com.zsy.libcore.common.android.log.SYLoger;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import butterknife.Unbinder;

/**
 * Weex界面抽象层；
 */
public abstract class BaseOldWXAct extends BaseActivity implements IWXRenderListener {
  protected ViewGroup mContainer;
  protected WXSDKInstance mInstance;
  protected BroadcastReceiver mReloadReceiver;
  protected FrameLayout jsContainerFl;

  //@Override protected void onCreate(@Nullable Bundle savedInstanceState) {
  //  super.onCreate(savedInstanceState);
  //  createWeexInstance();
  //  mInstance.onActivityCreate();
  //}
  Unbinder unbinder;
  /**是否加载离线资源,用于调试或者热更新配置*/
  protected abstract boolean isOnLineMode();
  //protected abstract String setSelfTitle();
  protected abstract String setJSUrl();

  protected void initWxWidgetsAndEvents(Bundle bundle) {
    createWeexInstance();
    mInstance.onActivityCreate();
    loadRenderView();
  }

  protected final void setContainer(ViewGroup container) {
    mContainer = container;
  }

  protected final ViewGroup getContainer() {
    return mContainer;
  }


  protected void loadRenderView() {
    //setContainer(jsContainerFl);
    if (!WXSoInstallMgrSdk.isCPUSupport()) {
      showMsg("您当前手机不支持我们App!");
      return;
    }

    if (isOnLineMode()) {
      renderPageByURL(setJSUrl());
    } else {
      Uri url = Uri.parse(setJSUrl());
      String path = url.getScheme().equals("file") ? assembleFilePath(url) : url.toString();
      renderPage(WXFileUtils.loadAsset(path, this), url.toString());
    }

    mReloadReceiver = new BroadcastReceiver() {
      @Override public void onReceive(Context context, Intent intent) {
        createWeexInstance();
        if (isOnLineMode()) {
          //renderPageByURL(WEEX_INDEX_URL);
        } else {
          //renderPage(WXFileUtils.loadAsset(setJSUrl(), WXContainerOldAct.this),WEEX_INDEX_URL);
        }
        //renderLoadingPb.setVisibility(View.VISIBLE);
      }
    };

    LocalBroadcastManager.getInstance(this)
        .registerReceiver(mReloadReceiver, new IntentFilter(WXSDKEngine.JS_FRAMEWORK_RELOAD));
  }



  protected void destoryWeexInstance() {
    if (mInstance != null) {
      mInstance.registerRenderListener(null);
      mInstance.destroy();
      mInstance = null;
    }
  }

  protected void createWeexInstance() {
    destoryWeexInstance();

    Rect outRect = new Rect();
    getWindow().getDecorView().getWindowVisibleDisplayFrame(outRect);

    mInstance = new WXSDKInstance(this);
    mInstance.registerRenderListener(this);
  }

  protected void renderPage(String template, String source) {
    renderPage(template, source, null);
  }

  protected void renderPage(String template, String source, String jsonInitData) {
    AssertUtil.throwIfNull(mContainer,
        new RuntimeException("Can't render page, container is null"));
    Map<String, Object> options = new HashMap<>();
    options.put(WXSDKInstance.BUNDLE_URL, source);
    mInstance.render(getPageName(), template, options, jsonInitData,
        //ScreenUtil.getDisplayWidth(this),
        //ScreenUtil.getDisplayHeight(this),
        WXApp.SCREEN_WIDTH, WXApp.SCREEN_HEIGHT, WXRenderStrategy.APPEND_ASYNC);
  }

  protected void renderPageByURL(String url) {
    renderPageByURL(url, null);
  }


  protected String assembleFilePath(Uri uri) {
    if (uri != null && uri.getPath() != null) {
      return uri.getPath().replaceFirst("/", "");
    }
    return "";
  }

  protected void renderPageByURL(String url, String jsonInitData) {
    AssertUtil.throwIfNull(mContainer,
        new RuntimeException("Can't render page, container is null"));
    Map<String, Object> options = new HashMap<>();
    options.put(WXSDKInstance.BUNDLE_URL, url);
    mInstance.renderByUrl(getPageName(), url, options, jsonInitData,
        //ScreenUtil.getDisplayWidth(this),
        //ScreenUtil.getDisplayHeight(this),
        WXApp.SCREEN_WIDTH, WXApp.SCREEN_HEIGHT, WXRenderStrategy.APPEND_ASYNC);
  }

  protected String getPageName() {
    return  this.getClass().getSimpleName();
  }

  @Override protected void onStart() {
    super.onStart();
    if (mInstance != null) {
      mInstance.onActivityStart();
    }
  }

  @Override protected void onResume() {
    super.onResume();
    if (mInstance != null) {
      mInstance.onActivityResume();
    }
  }

  @Override protected void onPause() {
    super.onPause();
    if (mInstance != null) {
      mInstance.onActivityPause();
    }
  }

  @Override protected void onStop() {
    super.onStop();
    if (mInstance != null) {
      mInstance.onActivityStop();
    }
  }

  @Override public void onDestroy() {
//    ButterKnife.unbind(this);
    if(unbinder != null){
      unbinder.unbind();
    }
    LocalBroadcastManager.getInstance(this).unregisterReceiver(mReloadReceiver);
    if (mInstance != null) {
      mInstance.onActivityDestroy();
    }
    super.onDestroy();
  }

  @Override public void onViewCreated(WXSDKInstance wxsdkInstance, View view) {
    if (mContainer != null) {
      mContainer.addView(view);
    }
  }

  @Override public void onRefreshSuccess(WXSDKInstance instance, int width, int height) {

  }

  /**
   * 跳转weex界面和native界面
   * 请求样式:
   * weexPage:{
   *   'url':'http:192.168.0.2:12580/examples/build/component/list/list-demo.js';
   * },
   * nativePage:{
   *   'url':'android:class://com.zsy.frame.weex.sample/view/fragment/TestFragment?title=Native测试'
   * }
   * @param intent
   */
  private void initFromIntent(Intent intent){
    //
    //有个instanceID, wx示例标识，目前还不清楚作用
    Uri parse = getIntent().getData();

    Bundle bundle = intent.getBundleExtra("BUNDLE_KEY_ARGS");
    if (bundle == null){
      bundle = new Bundle();
    }
    bundle.putString("BUNDLE_KEY_ARGS_URI",parse.toString());

    String scheme = parse.getScheme();
    //  获取标题栏参数；

    String title = parse.getQueryParameter("title");

    //SupportFragment instantiate;

    if (scheme.startsWith("android.class")){
      //  类文件
      String packageName = parse.getEncodedAuthority();
      List<String> pathSegments = parse.getPathSegments();

      StringBuilder path  = new StringBuilder(packageName);

      //  本地文件
      for (String segment: pathSegments){
        path.append(",");
        //去除class文件名后缀
        if (segment.contains(".")){
          segment = segment.substring(0, segment.indexOf("."));
        }
        path.append(segment);
      }
      String classPath = path.toString();
      if (BuildConfig.DEBUG){
        SYLoger.print("获取class文件信息："+title + classPath);
      }

      //instantiate = (SupportFragment).instantiate(this, classPath, bundle);
    }
  }

}
